;; Machine description for ST Microelectronics Loongson-2E/2F.
;; Copyright (C) 2007 Free Software Foundation, Inc.
;; Contributed by CodeSourcery.
;;
;; This file is part of GCC.
;;
;; GCC is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3, or (at your option)
;; any later version.

;; GCC is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GCC; see the file COPYING3.  If not see
;; <http://www.gnu.org/licenses/>.

;; Mode macros and attributes.

;; 64-bit vectors of bytes.
(define_mode_macro VB [V8QI])

;; 64-bit vectors of halfwords.
(define_mode_macro VH [V4HI])

;; 64-bit vectors of words.
(define_mode_macro VW [V2SI])

;; 64-bit vectors of halfwords and bytes.
(define_mode_macro VHB [V4HI V8QI])

;; 64-bit vectors of words and halfwords.
(define_mode_macro VWH [V2SI V4HI])

;; 64-bit vectors of words, halfwords and bytes.
(define_mode_macro VWHB [V2SI V4HI V8QI])

;; 64-bit vectors of words, halfwords and bytes; and DImode.
(define_mode_macro VWHBDI [V2SI V4HI V8QI DI])

;; The Loongson instruction suffixes corresponding to the modes in the
;; VWHB macro.
(define_mode_attr V_suffix [(V2SI "w") (V4HI "h") (V8QI "b")])

;; Given a vector type T, the mode of a vector half the size of T
;; and with the same number of elements.
(define_mode_attr V_squash [(V2SI "V2HI") (V4HI "V4QI")])

;; Given a vector type T, the mode of a vector the same size as T
;; but with half as many elements.
(define_mode_attr V_stretch_half [(V2SI "DI") (V4HI "V2SI") (V8QI "V4HI")])

;; The Loongson instruction suffixes corresponding to the transformation
;; expressed by V_stretch_half.
(define_mode_attr V_stretch_half_suffix [(V2SI "wd") (V4HI "hw") (V8QI "bh")])

;; Given a vector type T, the mode of a vector the same size as T
;; but with twice as many elements.
(define_mode_attr V_squash_double [(V2SI "V4HI") (V4HI "V8QI")])

;; The Loongson instruction suffixes corresponding to the conversions
;; specified by V_half_width.
(define_mode_attr V_squash_double_suffix [(V2SI "wh") (V4HI "hb")])

;; Move patterns.

;; Expander to legitimize moves involving values of vector modes.
(define_expand "mov<mode>"
  [(set (match_operand:VWHB 0 "nonimmediate_operand")
	(match_operand:VWHB 1 "move_operand"))]
  ""
{
  if (mips_legitimize_move (<MODE>mode, operands[0], operands[1]))
    DONE;
})

(define_split
  [(set (match_operand:VWHB 0 "nonimmediate_operand")
	(match_operand:VWHB 1 "move_operand"))]
  "reload_completed && HAVE_LOONGSON_VECTOR_MODES
   && mips_split_64bit_move_p (operands[0], operands[1])"
  [(const_int 0)]
{
  mips_split_64bit_move (operands[0], operands[1]);
  DONE;
})

;; Handle legitimized moves between values of vector modes.
(define_insn "mov<mode>_internal"
  [(set (match_operand:VWHB 0 "nonimmediate_operand" "=m,f,f,r,f,r,m,f")
	(match_operand:VWHB 1 "move_operand" "f,m,f,f,r,r,YG,YG"))]
  "HAVE_LOONGSON_VECTOR_MODES"
{
  return mips_output_move (operands[0], operands[1]);
}
  [(set_attr "type" "fpstore,fpload,fmove,mfc,mtc,move,fpstore,mtc")
   (set_attr "mode" "<MODE>")])

;; Initialization of a vector.

(define_expand "vec_init<mode>"
  [(set (match_operand:VWHB 0 "register_operand" "=f")
	(match_operand 1 "" ""))]
  "HAVE_LOONGSON_VECTOR_MODES"
  {
    mips_expand_vector_init (operands[0], operands[1]);
    DONE;
  }
)

;; The following 3 patterns are used in o32 mode, which is where a
;; pair of fp regs hold a vector operand.  They are equivalent to the
;; scalar md file's patterns for moving DF operands between the float and
;; integer registers or memory.
;; Note that these patterns include DImode, which the scalar md file
;; does not support.

;; Load the low word of operand 0 with operand 1.  This is the vector
;; (and DImode) equivalent of *load_df_low
(define_insn "*load_<mode>_low"
  [(set (match_operand:VWHBDI 0 "register_operand" "=f,f")
	(unspec:VWHBDI [(match_operand:SI 1 "general_operand" "dJ,m")]
		       UNSPEC_LOAD_DF_LOW))]
  "HAVE_LOONGSON_VECTOR_MODES && !TARGET_FLOAT64"
{
  operands[0] = mips_subword (operands[0], 0);
  return mips_output_move (operands[0], operands[1]);
}
  [(set_attr "type"	"mtc,fpload")
   (set_attr "mode"	"SI")])

;; Load the high word of operand 0 from operand 1, preserving the value
;; in the low word.  This is the vector (and DImode) equivalent of
;; *load_df_high
(define_insn "*load_<mode>_high"
  [(set (match_operand:VWHBDI 0 "register_operand" "=f,f")
	(unspec:VWHBDI [(match_operand:SI 1 "general_operand" "dJ,m")
		    (match_operand:VWHBDI 2 "register_operand" "0,0")]
		   UNSPEC_LOAD_DF_HIGH))]
  "HAVE_LOONGSON_VECTOR_MODES && !TARGET_FLOAT64"
{
  operands[0] = mips_subword (operands[0], 1);
  return mips_output_move (operands[0], operands[1]);
}
  [(set_attr "type"	"mtc,fpload")
   (set_attr "mode"	"SI")])

;; Store the high word of operand 1 in operand 0.  The corresponding
;; low-word move is done in the normal way.  This is the vector (and
;; DImode) equivalent of *store_df_high
(define_insn "*store_<mode>_high"
  [(set (match_operand:SI 0 "nonimmediate_operand" "=d,m")
	(unspec:SI [(match_operand:VWHBDI 1 "register_operand" "f,f")]
		   UNSPEC_STORE_DF_HIGH))]
  "HAVE_LOONGSON_VECTOR_MODES && !TARGET_FLOAT64"
{
  operands[1] = mips_subword (operands[1], 1);
  return mips_output_move (operands[0], operands[1]);
}
  [(set_attr "type"	"mfc,fpstore")
   (set_attr "mode"	"SI")])

;; Instruction patterns for SIMD instructions.

;; Pack with signed saturation.
(define_insn "vec_pack_ssat_<mode>"
  [(set (match_operand:<V_squash_double> 0 "register_operand" "=f")
        (vec_concat:<V_squash_double>
	  (ss_truncate:<V_squash> (match_operand:VWH 1 "register_operand" "f"))
          (ss_truncate:<V_squash> (match_operand:VWH 2 "register_operand" "f")))
	
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "packss<V_squash_double_suffix>\t%0,%1,%2"
  [(set_attr "type" "fmul")])

;; Pack with unsigned saturation.
(define_insn "vec_pack_usat_<mode>"
  [(set (match_operand:<V_squash_double> 0 "register_operand" "=f")
        (vec_concat:<V_squash_double>
	  (us_truncate:<V_squash> (match_operand:VH 1 "register_operand" "f"))
          (us_truncate:<V_squash> (match_operand:VH 2 "register_operand" "f"))
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "packus<V_squash_double_suffix>\t%0,%1,%2"
  [(set_attr "type" "fmul")]
)

;; Addition, treating overflow by wraparound.
(define_insn "add<mode>3"
  [(set (match_operand:VWHB 0 "register_operand" "=f")
        (plus:VWHB
          (match_operand:VWHB 1 "register_operand" "f")
          (match_operand:VWHB 2 "register_operand" "f")
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "padd<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fadd")])

;; Addition of doubleword integers stored in FP registers.
;; Overflow is treated by wraparound.
(define_insn "paddd"
  [(set (match_operand:DI 0 "register_operand" "=f")
        (plus:DI
          (match_operand:DI 1 "register_operand" "f")
          (match_operand:DI 2 "register_operand" "f")
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "paddd\t%0,%1,%2"
  [(set_attr "type" "fadd")])

;; Addition, treating overflow by signed saturation.
(define_insn "ssadd<mode>3"
  [(set (match_operand:VHB 0 "register_operand" "=f")
        (ss_plus:VHB
          (match_operand:VHB 1 "register_operand" "f")
          (match_operand:VHB 2 "register_operand" "f")
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "padds<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fadd")])

;; Addition, treating overflow by unsigned saturation.
(define_insn "usadd<mode>3"
  [(set (match_operand:VHB 0 "register_operand" "=f")
        (us_plus:VHB
          (match_operand:VHB 1 "register_operand" "f")
          (match_operand:VHB 2 "register_operand" "f")
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "paddus<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fadd")])

;; Logical AND NOT.
(define_insn "loongson_and_not_<mode>"
  [(set (match_operand:VWHBDI 0 "register_operand" "=f")
        (and:VWHBDI
          (not:VWHBDI (match_operand:VWHBDI 1 "register_operand" "f"))
          (match_operand:VWHBDI 2 "register_operand" "f")
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pandn\t%0,%1,%2"
  [(set_attr "type" "fmul")])

;; Average.
(define_insn "loongson_average_<mode>"
  [(set (match_operand:VHB 0 "register_operand" "=f")
        (unspec:VHB
          [(match_operand:VHB 1 "register_operand" "f")
           (match_operand:VHB 2 "register_operand" "f")]
	  UNSPEC_LOONGSON_AVERAGE
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pavg<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fadd")])

;; Equality test.
(define_insn "loongson_eq_<mode>"
  [(set (match_operand:VWHB 0 "register_operand" "=f")
        (unspec:VWHB
          [(match_operand:VWHB 1 "register_operand" "f")
           (match_operand:VWHB 2 "register_operand" "f")]
	  UNSPEC_LOONGSON_EQ
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pcmpeq<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fadd")])

;; Greater-than test.
(define_insn "loongson_gt_<mode>"
  [(set (match_operand:VWHB 0 "register_operand" "=f")
        (unspec:VWHB
          [(match_operand:VWHB 1 "register_operand" "f")
           (match_operand:VWHB 2 "register_operand" "f")]
	  UNSPEC_LOONGSON_GT
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pcmpgt<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fadd")])

;; Extract halfword.
(define_insn "loongson_extract_halfword"
  [(set (match_operand:VH 0 "register_operand" "=f")
        (unspec:VH
          [(match_operand:VH 1 "register_operand" "f")
           (match_operand:SI 2 "register_operand" "f")]
	  UNSPEC_LOONGSON_EXTRACT_HALFWORD
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pextr<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fmul")])

;; Insert halfword.
(define_insn "loongson_insert_halfword_0"
  [(set (match_operand:VH 0 "register_operand" "=f")
        (unspec:VH
          [(match_operand:VH 1 "register_operand" "f")
           (match_operand:VH 2 "register_operand" "f")]
	  UNSPEC_LOONGSON_INSERT_HALFWORD_0
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pinsr<V_suffix>_0\t%0,%1,%2"
  [(set_attr "type" "fdiv")])

(define_insn "loongson_insert_halfword_1"
  [(set (match_operand:VH 0 "register_operand" "=f")
        (unspec:VH
          [(match_operand:VH 1 "register_operand" "f")
           (match_operand:VH 2 "register_operand" "f")]
	  UNSPEC_LOONGSON_INSERT_HALFWORD_1))
  ]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pinsr<V_suffix>_1\t%0,%1,%2"
  [(set_attr "type" "fdiv")])

(define_insn "loongson_insert_halfword_2"
  [(set (match_operand:VH 0 "register_operand" "=f")
        (unspec:VH
          [(match_operand:VH 1 "register_operand" "f")
           (match_operand:VH 2 "register_operand" "f")]
	  UNSPEC_LOONGSON_INSERT_HALFWORD_2
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pinsr<V_suffix>_2\t%0,%1,%2"
  [(set_attr "type" "fdiv")])

(define_insn "loongson_insert_halfword_3"
  [(set (match_operand:VH 0 "register_operand" "=f")
        (unspec:VH
          [(match_operand:VH 1 "register_operand" "f")
           (match_operand:VH 2 "register_operand" "f")]
	  UNSPEC_LOONGSON_INSERT_HALFWORD_3
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pinsr<V_suffix>_3\t%0,%1,%2"
  [(set_attr "type" "fdiv")])

;; Multiply and add packed integers.
(define_insn "loongson_mult_add"
  [(set (match_operand:<V_stretch_half> 0 "register_operand" "=f")
        (unspec:<V_stretch_half>
          [(match_operand:VH 1 "register_operand" "f")
           (match_operand:VH 2 "register_operand" "f")]
	  UNSPEC_LOONGSON_MULT_ADD
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pmadd<V_stretch_half_suffix>\t%0,%1,%2"
  [(set_attr "type" "fmul")])

;; Maximum of signed halfwords.
(define_insn "smax<mode>3"
  [(set (match_operand:VH 0 "register_operand" "=f")
        (smax:VH
          (match_operand:VH 1 "register_operand" "f")
          (match_operand:VH 2 "register_operand" "f")
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pmaxs<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fadd")])

;; Maximum of unsigned bytes.
(define_insn "umax<mode>3"
  [(set (match_operand:VB 0 "register_operand" "=f")
        (umax:VB
          (match_operand:VB 1 "register_operand" "f")
          (match_operand:VB 2 "register_operand" "f")
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pmaxu<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fadd")])

;; Minimum of signed halfwords.
(define_insn "smin<mode>3"
  [(set (match_operand:VH 0 "register_operand" "=f")
        (smin:VH
          (match_operand:VH 1 "register_operand" "f")
          (match_operand:VH 2 "register_operand" "f")
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pmins<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fadd")])

;; Minimum of unsigned bytes.
(define_insn "umin<mode>3"
  [(set (match_operand:VB 0 "register_operand" "=f")
        (umin:VB
          (match_operand:VB 1 "register_operand" "f")
          (match_operand:VB 2 "register_operand" "f")
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pminu<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fadd")])

;; Move byte mask.
(define_insn "loongson_move_byte_mask"
  [(set (match_operand:VB 0 "register_operand" "=f")
        (unspec:VB
          [(match_operand:VB 1 "register_operand" "f")]
	  UNSPEC_LOONGSON_MOVE_BYTE_MASK
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pmovmsk<V_suffix>\t%0,%1"
  [(set_attr "type" "fabs")])

;; Multiply unsigned integers and store high result.
(define_insn "umul<mode>3_highpart"
  [(set (match_operand:VH 0 "register_operand" "=f")
        (unspec:VH
          [(match_operand:VH 1 "register_operand" "f")
           (match_operand:VH 2 "register_operand" "f")]
	  UNSPEC_LOONGSON_UMUL_HIGHPART
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pmulhu<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fmul")])

;; Multiply signed integers and store high result.
(define_insn "smul<mode>3_highpart"
  [(set (match_operand:VH 0 "register_operand" "=f")
        (unspec:VH
          [(match_operand:VH 1 "register_operand" "f")
           (match_operand:VH 2 "register_operand" "f")]
	  UNSPEC_LOONGSON_SMUL_HIGHPART
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pmulh<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fmul")])

;; Multiply signed integers and store low result.
(define_insn "loongson_smul_lowpart"
  [(set (match_operand:VH 0 "register_operand" "=f")
        (unspec:VH
          [(match_operand:VH 1 "register_operand" "f")
           (match_operand:VH 2 "register_operand" "f")]
	  UNSPEC_LOONGSON_SMUL_LOWPART
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pmull<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fmul")])

;; Multiply unsigned word integers.
(define_insn "loongson_umul_word"
  [(set (match_operand:DI 0 "register_operand" "=f")
        (unspec:DI
          [(match_operand:VW 1 "register_operand" "f")
           (match_operand:VW 2 "register_operand" "f")]
	  UNSPEC_LOONGSON_UMUL_WORD
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pmulu<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fmul")])

;; Absolute difference.
(define_insn "loongson_pasubub"
  [(set (match_operand:VB 0 "register_operand" "=f")
        (unspec:VB
          [(match_operand:VB 1 "register_operand" "f")
	   (match_operand:VB 2 "register_operand" "f")]
	  UNSPEC_LOONGSON_PASUBUB
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pasubub\t%0,%1,%2"
  [(set_attr "type" "fadd")])

;; Sum of unsigned byte integers.
(define_insn "reduc_uplus_<mode>"
  [(set (match_operand:<V_stretch_half> 0 "register_operand" "=f")
        (unspec:<V_stretch_half>
          [(match_operand:VB 1 "register_operand" "f")]
	  UNSPEC_LOONGSON_BIADD
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "biadd\t%0,%1"
  [(set_attr "type" "fabs")])

;; Sum of absolute differences.
(define_insn "loongson_psadbh"
  [(set (match_operand:<V_stretch_half> 0 "register_operand" "=f")
        (unspec:<V_stretch_half>
          [(match_operand:VB 1 "register_operand" "f")
           (match_operand:VB 2 "register_operand" "f")]
	  UNSPEC_LOONGSON_PSADBH
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pasubub\t%0,%1,%2;biadd\t%0,%0"
  [(set_attr "type" "fadd")])

;; Shuffle halfwords.
(define_insn "loongson_pshufh"
  [(set (match_operand:VH 0 "register_operand" "=f")
        (unspec:VH
          [(match_operand:VH 1 "register_operand" "0")
	   (match_operand:VH 2 "register_operand" "f")
           (match_operand:SI 3 "register_operand" "f")]
	  UNSPEC_LOONGSON_PSHUFH
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "pshufh\t%0,%2,%3"
  [(set_attr "type" "fmul")])

;; Shift left logical.
(define_insn "loongson_psll<mode>"
  [(set (match_operand:VWH 0 "register_operand" "=f")
        (ashift:VWH
          (match_operand:VWH 1 "register_operand" "f")
	  (match_operand:SI 2 "register_operand" "f")
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "psll<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fmul")])

;; Shift right arithmetic.
(define_insn "loongson_psra<mode>"
  [(set (match_operand:VWH 0 "register_operand" "=f")
        (ashiftrt:VWH
          (match_operand:VWH 1 "register_operand" "f")
	  (match_operand:SI 2 "register_operand" "f")
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "psra<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fdiv")])

;; Shift right logical.
(define_insn "loongson_psrl<mode>"
  [(set (match_operand:VWH 0 "register_operand" "=f")
        (lshiftrt:VWH
          (match_operand:VWH 1 "register_operand" "f")
	  (match_operand:SI 2 "register_operand" "f")
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "psrl<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fdiv")])

;; Subtraction, treating overflow by wraparound.
(define_insn "sub<mode>3"
  [(set (match_operand:VWHB 0 "register_operand" "=f")
        (minus:VWHB
          (match_operand:VWHB 1 "register_operand" "f")
          (match_operand:VWHB 2 "register_operand" "f")
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "psub<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fadd")])

;; Subtraction of doubleword integers stored in FP registers.
;; Overflow is treated by wraparound.
(define_insn "psubd"
  [(set (match_operand:DI 0 "register_operand" "=f")
        (minus:DI
          (match_operand:DI 1 "register_operand" "f")
          (match_operand:DI 2 "register_operand" "f")
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "psubd\t%0,%1,%2"
  [(set_attr "type" "fadd")])

;; Subtraction, treating overflow by signed saturation.
(define_insn "sssub<mode>3"
  [(set (match_operand:VHB 0 "register_operand" "=f")
        (ss_minus:VHB
          (match_operand:VHB 1 "register_operand" "f")
          (match_operand:VHB 2 "register_operand" "f")
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "psubs<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fadd")])

;; Subtraction, treating overflow by unsigned saturation.
(define_insn "ussub<mode>3"
  [(set (match_operand:VHB 0 "register_operand" "=f")
        (us_minus:VHB
          (match_operand:VHB 1 "register_operand" "f")
          (match_operand:VHB 2 "register_operand" "f")
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "psubus<V_suffix>\t%0,%1,%2"
  [(set_attr "type" "fadd")])

;; Unpack high data.
(define_insn "vec_interleave_high<mode>"
  [(set (match_operand:VWHB 0 "register_operand" "=f")
        (unspec:VWHB
          [(match_operand:VWHB 1 "register_operand" "f")
           (match_operand:VWHB 2 "register_operand" "f")]
	  UNSPEC_LOONGSON_UNPACK_HIGH
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "punpckh<V_stretch_half_suffix>\t%0,%1,%2"
  [(set_attr "type" "fdiv")])

;; Unpack low data.
(define_insn "vec_interleave_low<mode>"
  [(set (match_operand:VWHB 0 "register_operand" "=f")
        (unspec:VWHB
          [(match_operand:VWHB 1 "register_operand" "f")
           (match_operand:VWHB 2 "register_operand" "f")]
	  UNSPEC_LOONGSON_UNPACK_LOW
	)
   )]
  "HAVE_LOONGSON_VECTOR_MODES"
  "punpckl<V_stretch_half_suffix>\t%0,%1,%2"
  [(set_attr "type" "fdiv")])
