# Copyright (c) 1996-2007 MIPS Technologies, Inc.
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above
#      copyright
#        notice, this list of conditions and the following disclaimer
#        in the documentation and/or other materials provided with
#        the distribution.
#      * Neither the name of MIPS Technologies Inc. nor the names of its
#        contributors may be used to endorse or promote products derived
#        from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#
# examples/make.mk
#
# Shared make include file for MIPS SDE example programs

ifdef DEPTH
# Backwards compatibility, use $(DEPTH) if defined
SDETOP ?= $(DEPTH)/..
else
SDETOP ?= ../..
endif

# Location of SDE source code kit
KITDIR		?=$(SDETOP)/kit

# System/eval board: 
ifeq (,$(SBD))
override SBD := $(patsubst %.sbd,%,$(wildcard *.sbd))
ifeq (,$(SBD))
override SBD := NOSBD
endif
MONITOR = $(SBD)
endif

# defaults, which may be overriden by sbd.mk
CPU	=r3k
TLB	=r3k
CACHES	=yes
FPU	=no
PCI	=no
ENDIAN	=-EB
ROM_U	=bfc00000
ROM_C	=9fc00000
ROM_SZ	=400000
RAM_U	=a0000280
RAM_C	=80000280

SBDDIR	=$(KITDIR)/$(SBD)
include $(SBDDIR)/sbd.mk
include $(KITDIR)/bare/monitor.mk
ifdef MONITOR
include $(KITDIR)/$(MONITOR)/monitor.mk
endif

-include $(KITDIR)/archs.mk

-include $(KITDIR)/sdeversion

# include default rules
include $(KITDIR)/rules.mk

ifndef SDE_MAJOR
# Guess major SDE version based on compiler version
SDE_MAJOR = $(shell \
	if $(CC) -v 2>&1 | fgrep 'gcc version 4.' >/dev/null; then \
	  echo "7"; \
	elif $(CC) -v 2>&1 | fgrep 'gcc version 3.' >/dev/null; then \
	  echo "6"; \
	else \
	  echo "5"; \
	fi)
endif

ifndef SBDDIRS
SBDDIRS	=$(SBDDIR)
endif

# Allow application-specific override of ISA and CPU type

ifdef APPISA
ISA	=$(APPISA)
endif

ifdef APPCPU
CPU	=$(APPCPU)
endif

# 
# For rommable programs link code and read-only data to ROM,
# and writeable data to RAM (just above the exception vectors).
#
# For interactive debugging link everything into RAM.
# The "kit/romlow" code will then copy both the code and data 
# to RAM, so breakpoints can be set.
#

ifndef ROMLDFLAGS
 ifeq ($(UNCACHED),yes)
   ROM_BASE = $(ROM_U)
   RAM_BASE = $(RAM_U)
 else
   ROM_BASE = $(ROM_C)
   RAM_BASE = $(RAM_C)
 endif
 LAYOUT ?= $(SBDLAYOUT)
 ifneq (,$(strip $(filter predev imm% copy romcopy ram,$(RDEBUG) $(LAYOUT))))
   # code and data in RAM
   ROMLDFLAGS	=-Ttext $(RAM_BASE)
 else
   # code in ROM, data in RAM
   ROMLDFLAGS	=-Ttext $(ROM_BASE) -Tdata $(RAM_BASE)
 endif
endif

# 
# For downloadable programs link all code and data to RAM.
#
ifndef RAMLDFLAGS
 ifeq ($(UNCACHED),yes)
   RAMLDFLAGS	=-Ttext $(DLBASE_U)
 else
   RAMLDFLAGS	=-Ttext $(DLBASE_C)
 endif
endif

# Lowest safe address for downloadable, but standalone programs
ifndef SABASE_C
 SABASE_U	=$(DLBASE_U)
 SABASE_C	=$(DLBASE_C)
endif

ifndef SALDFLAGS
 ifeq ($(UNCACHED),yes)
   SALDFLAGS	=-Ttext $(SABASE_U)
 else
   SALDFLAGS	=-Ttext $(SABASE_C)
 endif
endif

#
# Set the proper remote debug symbol for the linker.
#
ifneq (,$(strip $(filter predev imm% ram,$(RDEBUG))))
  KITFEATURES += rdebug
  FEATURES += rdebug
  ifneq (,$(findstring predev,$(RDEBUG)))
    LDSYMS += -Wl,--defsym,__sde_feature_rdbg=3
  else
    ifneq (,$(findstring imm,$(RDEBUG)))
      LDSYMS += -Wl,--defsym,__sde_feature_rdbg=2
    else
      LDSYMS += -Wl,--defsym,__sde_feature_rdbg=1
    endif
  endif
endif

ifneq ($(PCI),no)
  # Enable optional PCI features
  KITFEATURES += pci pcilookup pciconf
endif

# Work out the flags which tell the compiler what type of
# floating-point code to generate, and tell the run-time 
# startup code how to initialise the hardware and/or emulator.
#
# Permissible values for FLOAT are:
#   no		doesn't do any f.p arithmetic (default)
#   yes		uses f.p, but won't underflow or get denormalised results
#   ieee	full support for IEEE denormalised numbers and u/flow
#   trap	(for testing only) force use of CP1 trap emulation

FLOAT	?=no
ifeq ($(FLOAT),no)
 # No floating-point used in code
 FP		=-mno-float
 FPFLAGS	='-Afloat(no)'
else
 # Floating-point used, select what type
 ifeq ($(FPU),no)
  # No hardware fpu: use compiler s/w emulation
  FP		=-msoft-float
  FPFLAGS	='-Afloat(soft)'
  # soft-float library, no instruction emulation
  FPLIBS	=-le
 else
  # Possible hardware fpu: generate h/w fp insns
  ifeq ($(FPU),single)
   # R4640/50 single-precision fpu only
   FP		=-msingle-float
   FPFLAGS	='-Afloat($(strip $(FLOAT)))'
   # fp double instruction emulation required
   FPLIBS	=-lcp1 -le
  else
   ifneq (,$(strip $(filter test maybe switch,$(FPU))))
     # Possible hardware floating-point - check at run-time
     FP		=-mhard-float
     FPFLAGS	='-Afloat(switch)'
     # fp instruction emulation may be required
     FPLIBS	=-lcp1 -le
   else
     # Definite hardware floating-point
     ifeq ($(FLOAT),ieee)
       # exceptional fp instruction emulation required
       FPLIBS	=-lcp1 -le
     endif
     ifeq ($(FPU),yes)
       FP	=-mhard-float
     else
       # Use $(FPU) to select
       FP	=-mhard-float $(FPU)
     endif
     FPFLAGS	='-Afloat($(strip $(FLOAT)))'
   endif
  endif
 endif
#
# Set the proper FP init symbol for the linker.
#
 ifeq ($(FLOAT),yes)
   LDSYMS += -Wl,--defsym,__sde_feature_float=1
 else
  ifeq ($(FLOAT),ieee)
   ifeq ($(MONITOR),gnusim)
     LDSYMS += -Wl,--defsym,__sde_feature_float=3
   else
     LDSYMS += -Wl,--defsym,__sde_feature_float=2
   endif
  else
   ifeq ($(FLOAT),trap)
     LDSYMS += -Wl,--defsym,__sde_feature_float=4
   endif
  endif
 endif
 LIBM = -lm
endif

#
# Default PROM programmer download format is 32-bit S-records
#
ROMDLFMT ?=s3

ifndef ALL
 ifdef DLFMT
  ALL	=ram
  ifndef SBDNOSER
    ALL += sa
  endif
  ifneq (,$(SBDLOW))
    ALL +=rom
  endif
 else
  ALL	=rom
 endif
endif

ALL += $(EXTRATARGETS)

DLFMT 	?=s3

#
# Default to a high, but not ultimate, speed optimisation
#
CFLAGS		?=-O2
CXXFLAGS	?=-O2

ifndef ASFLAGS
ASFLAGS := $(CFLAGS)
endif

ifndef CPUASSERT
 ifndef CPUNAMES
  CPUALIAS	=$(patsubst r%k,r%000,$(filter r%k,$(CPU)))
  CPUNAMES	=$(CPU) $(CPUALIAS) $(CPUVARIANT)
 endif
 CPUASSERT	=$(patsubst %,'-Acpu(%)',$(CPUNAMES))
endif

ifndef CACHEDEF
  ifeq (,$(CACHES))
    CACHEDEF = '-Acache(no)'
  else
    CACHEDEF = '-Acache(yes)'
  endif
endif

ifndef MMUDEF
  ifeq (m4k,$(CPU))
    MMUDEF = '-Ammu(no)'
  else
    ifeq (cw10,$(CPU))
      MMUDEF = '-Ammu(cw10)'
    else
      ifeq (r3k,$(CPU))
        MMUDEF = '-Ammu(type1)'
      else
        MMUDEF = '-Ammu(type2)'
      endif
    endif
  endif
endif

ifneq (,$(filter $(filter -mips%,$(ISA)),-mips3 -mips4 -mips64 -mips64r2))
ifeq (,$(filter $(filter -mgp%,$(ISA)),-mgp32))
ISA_IS_64BIT = 1
endif
endif

ifneq (,$(strip $(filter 7,$(SDE_MAJOR))))
 CPUARCH = -march=$(CPU)
 # ISA was traditionally used to indicate 32-bit/64-bit support.
 # FIXME temporary workaround for SDE 7 ABI selection
 ifeq (,$(ISA_IS_64BIT))
  ISA =
  ABI = -mabi=32
 else
  ISA =
  ABI = -mabi=n32
 endif
endif

# Pick CPU architecture for compiler to tune for
ifndef CPUTUNE
 ifneq (,$(strip $(filter 5,$(SDE_MAJOR))))
  CPUTUNE	=-mcpu=$(CPU)
  CPUTUNE	+=$(patsubst %,-mcpu=%,$(CPUNAMES))
 else
  ifneq (,$(strip $(filter 6,$(SDE_MAJOR))))
   CPUTUNE	=-mtune=$(CPU)
  endif
 endif
endif

#
# Finally the standard make rules
#
ifneq (,$(strip $(filter 7,$(SDE_MAJOR))))
# FIXME temporary workaround for SDE 7
ifeq (,$(ISA_IS_64BIT))
ifeq (-EL,$(strip $(ENDIAN)))
override LD_OFORMAT='-Wl,--oformat=elf32-tradlittlemips'
else
override LD_OFORMAT='-Wl,--oformat=elf32-tradbigmips'
endif
else
ifeq (-EL,$(strip $(ENDIAN)))
override LD_OFORMAT='-Wl,--oformat=elf32-ntradlittlemips'
else
override LD_OFORMAT='-Wl,--oformat=elf32-ntradbigmips'
endif
endif
endif

CPUFLAGS	=$(ABI) $(ISA) $(CPUARCH) $(CPUTUNE) $(ASE) $(ENDIAN) $(CPUASSERT) $(CACHEDEF) $(MMUDEF) $(FP)
INCDIRS		+=$(patsubst %,-I$(SBDDIR)/%,$(subst :, ,$(SBDDIRS)))
INCDIRS		+=$(STDINCDIR)
CPPFLAGS	+=$(SBDFLAGS) $(SBDOPTIONS) $(FPFLAGS) \
		  $(ARCHFLAGS) $(INCDIRS)

ifneq (,$(strip $(filter r3k,$(CPU))))
 CPUFLAGS += -Wa,-break
endif

override ASFLAGS  +=$(CPUFLAGS)
override CFLAGS   +=$(CPUFLAGS)
override CXXFLAGS +=$(CPUFLAGS)
override LDFLAGS  +=$(CPUFLAGS) $(LD_OFORMAT)

# Add profiling options based on PROFILE variable
ifneq (,$(strip $(filter-out no none off,$(PROFILE))))
  # Profiling build requested 
  override CFLAGS	+=$(filter -%,$(PROFILE))
  ifneq (,$(strip $(filter-out gcov feedback% -%,$(PROFILE))))
    # All but gcov, feedback enables normal gprof profiling too
    override ASFLAGS	+=-pg
    override CFLAGS	+=-pg
    override CXXFLAGS	+=-pg
  endif
  ifneq (,$(strip $(filter line lines,$(PROFILE))))
    # Line-based gprof profiling requires debug info
    override ASFLAGS	+=-g
    override CFLAGS	+=-g
    override CXXFLAGS	+=-g
  endif
  ifneq (,$(strip $(filter gcov,$(PROFILE))))
    # Generate gcov code coverage data
    override ASFLAGS	+=-ftest-coverage
    override CFLAGS	+=-ftest-coverage
    override CXXFLAGS	+=-ftest-coverage
    LIBGCOV = -lgcov
  endif
  ifneq (,$(strip $(filter arc arcs branch gcov,$(PROFILE))))
    # Arc profiling or gcov
    override ASFLAGS	+=-fprofile-arcs
    override CFLAGS	+=-fprofile-arcs
    override CXXFLAGS	+=-fprofile-arcs 
    LIBGCOV = -lgcov
  endif
  ifneq (,$(strip $(filter feedback-generate,$(PROFILE))))
    ifeq (,$(strip $(filter 5,$(SDE_MAJOR))))
      override ASFLAGS	+=-fprofile-generate
      override CFLAGS	+=-fprofile-generate
      override CXXFLAGS	+=-fprofile-generate
    else
      override ASFLAGS	+=-fprofile-arcs
      override CFLAGS	+=-fprofile-arcs
      override CXXFLAGS	+=-fprofile-arcs 
    endif
    LIBGCOV = -lgcov
  endif
  ifneq (,$(strip $(filter feedback-use,$(PROFILE))))
    # Using gprof data to feedback into compiler
    ifeq (,$(strip $(filter 5,$(SDE_MAJOR))))
      override ASFLAGS    +=-fprofile-use
      override CFLAGS     +=-fprofile-use
      override CXXFLAGS   +=-fprofile-use
    else
      override ASFLAGS    +=-fbranch-probabilities
      override CFLAGS     +=-fbranch-probabilities
      override CXXFLAGS   +=-fbranch-probabilities
    endif
  endif
endif

# Default to generating debug data, for better Insight debugging.
# If you don't want this then add -g0 to CFLAGS, or set NODEBUG=on
ifeq (,$(strip $(filter on yes true,$(NODEBUG))))
  ifeq (,$(strip $(filter -g%,$(CFLAGS))))
    override ASFLAGS	+=-g
    override CFLAGS	+=-g
    override CXXFLAGS	+=-g
  endif
endif

# profiling conflicts with function sections
ifneq (,$(strip $(filter-out no none off,$(PROFILE))))
  override ASFLAGS  := $(filter-out -ffunction-sections,$(ASFLAGS))
  override CFLAGS   := $(filter-out -ffunction-sections,$(CFLAGS))
  override CXXFLAGS := $(filter-out -ffunction-sections,$(CXXFLAGS))
endif

LDMAPFILE = -Wl,-Map,$@.map

# Don't export the application build options when building
# the run-time kit.
unexport ASFLAGS
unexport CPPFLAGS
unexport CFLAGS
unexport CXXFLAGS
unexport LDFLAGS

#
# Select the correct startup file
#
CRT0		=crt0.o
CRTN		=$(SBDDIR)/crtn.o

#
# Build the intersection of requested and supported features for
# this application and board kit, and pass to ld.
# 
stdfeatures	:=$(SBDFEATURES) $(KITFEATURES)
extrafeatures	:=unaligned $(MONFEATURES) $(THREADFEATURES) $(PKGFEATURES)
preffeatures	:=$(patsubst /%,%,$(filter /%,$(FEATURES)))
exclfeatures	:=$(patsubst -%,%,$(filter -%,$(FEATURES)))
allfeatures	:=$(patsubst all,$(stdfeatures),$(filter-out -% /%,$(FEATURES)))
wantfeatures	:=$(sort $(allfeatures))
selectfeatures	:=$(filter $(wantfeatures) $(preffeatures),$(stdfeatures) $(extrafeatures))
loadfeatures	:=$(strip $(filter-out $(exclfeatures),$(selectfeatures)))

missingfeatures :=$(filter-out $(loadfeatures) $(exclfeatures),$(wantfeatures))
ifneq (,$(missingfeatures))
 ifeq (,$(filter $(MAKECMDGOALS),clean clobber reallyclean))
   $(warning FEATURES not available: $(missingfeatures))
 endif
endif

LDSYMS += $(patsubst %,-u _%_load,$(loadfeatures))

# Select the correct arguments for sde-conv when making a ROM
ifeq ($(ROM_U)/$(ROM_SZ),bfc00000/400000)
  ROMFLG=-p
else
  ROMFLG=-a $(ROM_U)/$(ROM_SZ)
endif

ifdef RDEBUG
RDEBUGROM = $(SBDDIR)/librdebugrom.a
LIBRDEBUGROM = -lrdebugrom
  # Only use librdebugram if the monitor has no native remote debugging.
  ifndef MON_NATIVE_RDEBUG
  RDEBUGRAM = $(SBDDIR)/librdebugram.a
  LIBRDEBUGRAM = -lrdebugram
  endif
endif

SBDROM =$(SBDDIR)/romstart.o $(RDEBUGROM) $(SBDDIR)/librom.a

SBDRAM =$(RDEBUGRAM) $(SBDDIR)/libram.a

SBDSA =$(SBDDIR)/sastart.o $(RDEBUGROM) $(SBDDIR)/librom.a

SBDLIBS=

ifneq (,$(filter pci,$(loadfeatures)))
  SBDFLAGS += '-Apci($(strip $(PCI)))'
endif

ifneq (,$(filter network,$(loadfeatures)))
ifeq (,$(SBDNET))
ifeq (,$(filter $(MAKECMDGOALS),clean clobber reallyclean))
$(error No network support for board $(SBD))
endif
endif
# Add network library
INCDIRS   +=-idirafter $(SDETOP)/net/sys
SBDLIBS   +=$(SBDDIR)/libnet.a
LOADLIBES +=-lnet
endif

ifneq (,$(filter pthreadlite,$(loadfeatures)))
ifeq (,$(filter 34k%,$(CPU)))
ifeq (,$(filter -mt,$(SBDOPTIONS)))
ifeq (,$(filter $(MAKECMDGOALS),clean clobber reallyclean))
$(error No pthreadlite support for board $(SBD))
endif
endif
endif
# Add Pthreadlite library
SBDLIBS   +=$(SBDDIR)/libthlite.a
LOADLIBES +=-lthlite
else
  ifneq (,$(filter network pthread,$(loadfeatures)))
    ifeq (,$(RTPX))
      ifeq (,$(filter $(MAKECMDGOALS),clean clobber reallyclean))
        $(error No pthread support for board $(SBD))
      endif
    endif
    # Add Pthread library
    SBDLIBS   +=$(SBDDIR)/libth.a
    LOADLIBES +=-lth
  endif
endif

ifneq (,$(filter threadx tx_posix,$(loadfeatures)))
  -include $(TXROOTDIR)/txarch.mk
  ifndef TX_ARCH_TYPE
    ifeq (,$(filter $(MAKECMDGOALS),clean clobber reallyclean))
      $(error No ThreadX support for board $(SBD))
    endif # MAKECMDGOALS != clean | clobber | reallyclean
  else # defined TX_ARCH_TYPE
    # Add Threadx library
    SBDLIBS   +=$(SBDDIR)/libthreadx.a
    LOADLIBES +=-lthreadx
    # Append CPPFLAGS
    ifeq (,$(TXROOTDIR))
      $(warning TXROOTDIR not explicitly specified - assuming TXROOTDIR=$(KITDIR)/../threadx)
      TXROOTDIR=$(KITDIR)/../threadx
    endif
    TXSRCDIR ?= $(TXROOTDIR)
    INCDIRS  += -I- -I$(TXROOTDIR)/glue -I. \
                -I$(TXSRCDIR)/posix/TX_POSIX \
	        -I$(TXSRCDIR)/$(TX_ARCH_TYPE)/threadx -D_SDE_TSP
  endif # TX_ARCH_TYPE
endif

ifeq (,$(strip $(filter 5,$(SDE_MAJOR))))
# SDE 6 needs separate gcov library when collecting code coverage or
# compiler feedback data
LOADLIBES  +=$(LIBGCOV)
endif

OBJS ?= $(filter %.o,$(SRCS:.c=.o) $(SRCS:.cc=.o) $(SRCS:.sx=.o) $(SRCS:.s=.o) $(SRCS:.S=.o))

all: $(ALL)

config: $(SBD).sbd

rom: $(PROG)rom.$(ROMDLFMT)$(ROMDLSPLIT)

ifneq (none,$(strip $(DLFMT)))
ram: $(PROG)ram.$(DLFMT)

sa: $(PROG)sa.$(DLFMT)
else
ram: $(PROG)ram

sa: $(PROG)sa
endif

rel: $(PROG)rel

ifeq (,$(findstring s,$(MAKEFLAGS)))
CONV_V := -v
else
CONV_V :=
endif

$(PROG)ram.$(DLFMT): $(PROG)ram
	$(CONV) $(CONV_V) -f $(DLFMT) $(DLSYMS) -o $@ $(PROG)ram

$(PROG)sa.$(DLFMT): $(PROG)sa
	$(CONV) $(CONV_V) -f $(DLFMT) $(DLSYMS) -o $@ $(PROG)sa

$(PROG)rom.$(ROMDLFMT): $(PROG)rom
	$(CONV) $(CONV_V) -f $(ROMDLFMT) $(ROMFLG) -o $(PROG)rom.$(ROMDLFMT) $(PROG)rom

$(PROG)rom.$(ROMDLFMT).byte: $(PROG)rom
	$(CONV) $(CONV_V) -b0 -f$(ROMDLFMT) $(ROMFLG) -o $(PROG)rom.$(ROMDLFMT).b0 $(PROG)rom
	$(CONV) $(CONV_V) -b1 -f$(ROMDLFMT) $(ROMFLG) -o $(PROG)rom.$(ROMDLFMT).b1 $(PROG)rom
	$(CONV) $(CONV_V) -b2 -f$(ROMDLFMT) $(ROMFLG) -o $(PROG)rom.$(ROMDLFMT).b2 $(PROG)rom
	$(CONV) $(CONV_V) -b3 -f$(ROMDLFMT) $(ROMFLG) -o $(PROG)rom.$(ROMDLFMT).b3 $(PROG)rom
	@touch $@

$(PROG)rom.$(ROMDLFMT).half: $(PROG)rom
	$(CONV) $(CONV_V) -b01 -f$(ROMDLFMT) $(ROMFLG) -o $(PROG)rom.$(ROMDLFMT).h0 $(PROG)rom
	$(CONV) $(CONV_V) -b23 -f$(ROMDLFMT) $(ROMFLG) -o $(PROG)rom.$(ROMDLFMT).h3 $(PROG)rom
	@touch $@

$(PROG)rom.$(ROMDLFMT).hswap: $(PROG)rom
	$(CONV) $(CONV_V) -b10 -f$(ROMDLFMT) $(ROMFLG) -o $(PROG)rom.$(ROMDLFMT).h0 $(PROG)rom
	$(CONV) $(CONV_V) -b32 -f$(ROMDLFMT) $(ROMFLG) -o $(PROG)rom.$(ROMDLFMT).h3 $(PROG)rom
	@touch $@

$(PROG)rom.$(ROMDLFMT).wswap: $(PROG)rom
	$(CONV) $(CONV_V) -b3210 -f$(ROMDLFMT) $(ROMFLG) -o $@ $(PROG)rom

$(PROG)rom.$(ROMDLFMT).dswap: $(PROG)rom
	$(CONV) $(CONV_V) -w64 -b76543210 -f$(ROMDLFMT) $(ROMFLG) -o $@ $(PROG)rom

# --gc-sections works only for a final link
comma := ,
$(PROG)rel: LDFLAGS := $(filter-out -Wl$(comma)--gc-sections, $(LDFLAGS))
$(PROG)rel: LDSCRIPT ?= -Wl,--script=$(KITDIR)/share/sde-rsp.ld
$(PROG)rel: $(CRT0) $(OBJS) $(LDLIBS) $(SBDRAM) $(SBDLIBS) $(CRTN)
	$(LINK.o) -r -d -L$(SBDDIR) $(STDLIBDIR) $(LDMAPFILE) \
	$(LDSYMS) $(CRT0) $(OBJS) $(LOADLIBES) $(LDLIBS) -Wl,--no-undefined \
	-Wl,--start-group -lc $(LIBM) $(LIBRDEBUGRAM) -lram -lgcc $(FPLIBS) -Wl,--end-group \
	$(CRTN) $(LDSCRIPT) -o $@ 

$(PROG)rom: LDSCRIPT ?= -Wl,--script=$(KITDIR)/share/sde.ld
$(PROG)rom: $(CRT0) $(OBJS) $(LDLIBS) $(SBDROM) $(SBDLIBS) $(CRTN)
	$(LINK.o) $(ROMLDFLAGS) -L$(SBDDIR) $(STDLIBDIR) $(LDMAPFILE) \
	$(SBDDIR)/romstart.o -Wl,--entry=___reset $(LDSYMS) $(CRT0) $(OBJS) $(LOADLIBES) $(LDLIBS) \
	-Wl,--start-group -lc $(LIBM) $(LIBRDEBUGROM) -lrom -lgcc $(FPLIBS) -Wl,--end-group \
	$(CRTN) $(LDSCRIPT) -Wl,-n -o $@

$(PROG)ram: LDSCRIPT ?= -Wl,--script=$(KITDIR)/share/sde.ld
$(PROG)ram: $(CRT0) $(OBJS) $(LDLIBS) $(SBDRAM) $(SBDLIBS) $(CRTN)
	$(LINK.o) $(RAMLDFLAGS) -L$(SBDDIR) $(STDLIBDIR) $(LDMAPFILE) \
	$(LDSYMS) $(CRT0) $(OBJS) $(LOADLIBES) $(LDLIBS) \
	-Wl,--start-group -lc $(LIBM) $(LIBRDEBUGRAM) -lram -lgcc $(FPLIBS) -Wl,--end-group \
	$(CRTN) $(LDSCRIPT) -Wl,-n -o $@

$(PROG)sa: LDSCRIPT ?= -Wl,--script=$(KITDIR)/share/sde.ld
$(PROG)sa: $(CRT0) $(OBJS) $(LDLIBS) $(SBDSA) $(SBDLIBS) $(CRTN)
	$(LINK.o) $(SALDFLAGS) -L$(SBDDIR) $(STDLIBDIR) $(LDMAPFILE) \
	$(SBDDIR)/sastart.o $(LDSYMS) $(CRT0) $(OBJS) $(LOADLIBES) $(LDLIBS) \
	-Wl,--start-group -lc $(LIBM) $(LIBRDEBUGROM) -lrom -lgcc $(FPLIBS) -Wl,--end-group \
	$(CRTN) $(LDSCRIPT) -Wl,-n -o $@

# Override default build rules for files which need particular #defines
$(CRT0): $(KITDIR)/share/crt0.sx
	$(AS) $(ASFLAGS) $(CPPFLAGS) $(CRT0FLAGS) -D__ASSEMBLER__ -x assembler-with-cpp $< -c -o $@

# These next two targets make sure that everything gets rebuilt if 
# the SBD variable, it's associated sbd.mk file, or this program's
# own Makefile are changed.

$(OBJS) $(CRT0): Makefile $(SBD).sbd $(SBDDIR)/sbd.mk

ifeq (,$(findstring s,$(MAKEFLAGS)))
PRINTINFO = echo
else
PRINTINFO = true
endif

$(SBD).sbd:
	@[ $@ != NOSBD.sbd ] || {                              \
		echo '**************************************'; \
		echo 'Please specify a valid SBD target name'; \
		echo '**************************************'; \
		exit 1; \
	}
	@if [ -f *.sbd ]  ; then                                            \
		$(PRINTINFO) "++ Rebuilding because SBD variable changed" ; \
		$(MAKE) clean $(SUBCLEAN_INHERIT_F)                       ; \
		rm -f *.sbd                                               ; \
	else                                                                \
		$(PRINTINFO)  "Creating $(SBD).sbd  "                     ; \
	fi
	@echo 'SBDFILE = $$(wildcard *.sbd)' >$@
	@echo 'ifneq (,$$(SBDFILE))' >>$@
	@echo 'export SDETOP ?= $(SDETOP)' >>$@
	@echo 'SBD ?= $$(shell basename $$(SBDFILE) .sbd)' >>$@
	@echo 'endif' >>$@

#Only for when there is no Makefile in cwd.
Makefile:
	@/bin/true

$(SBDROM) $(SBDRAM) $(SBDSA) $(CRTN): kit

.PHONY: kit

ifndef KITBUILT
kit:
	@$(MAKE) -C $(SBDDIR) SBD=$(SBD) kit

$(SBDDIR)/%.a: 
	@$(MAKE) -C $(SBDDIR) SBD=$(SBD) $(notdir $@)
else
kit:

$(SBDDIR)/%.a: 
endif

ifndef MAKEMAKEFILE
# Run makefile recursively with MAKEMAKEFILE defined
.PHONY: SDEmakefile
SDEmakefile: 
	@$(MAKE) MAKEMAKEFILE=yes $@
else
# Export all make variables to environment and run shell script
.EXPORT_ALL_VARIABLES:
export ASFLAGS
export CPPFLAGS
export CFLAGS
export CXXFLAGS
export LDFLAGS

# A tricky one, we need the unexpanded $@.
LDMAPFILE := -Wl,-Map,$$@.map

.PHONY: SDEmakefile
SDEmakefile:	
	sh $(SDETOP)/examples/makemake.sh $@ >$@
endif

ifneq (,$(filter $(MAKECMDGOALS),clean clobber reallyclean))
.NOTPARALLEL:
endif

.PHONY: clean
clean:
	rm -f *.[ado]
	rm -f $(PROG)rom $(PROG)ram $(PROG)sa $(PROG)rel $(EXTRACLEAN)
	rm -f $(PROG)rom.* $(PROG)ram.* $(PROG)sa.*
	rm -f $(SBDCLEAN)

.PHONY: clobber reallyclean
clobber reallyclean: clean
	rm -f *.[dis] *.sbd *[~#] core *.core ,* *.map *.gcno *.gcda
	rm -f SDEmakefile

ifdef SBDMKF
-include $(SBDDIR)/$(SBDMKF)
endif

# Obsolete dummy depend target, this is now handled automagically.
.PHONY: depend
depend:

%.d: %.c
	@$(CC) -MM $(CPPFLAGS) $(CFLAGS) $< > $@.$$$$; \
	sed 's,\($*\)\.o[ :]*,\1.o $@ : ,g' <$@.$$$$ > $@; \
	rm -f $@.$$$$

%.d: %.cc
	@$(CC) -MM $(CPPFLAGS) $(CXXFLAGS) $< > $@.$$$$; \
	sed 's,\($*\)\.o[ :]*,\1.o $@ : ,g' <$@.$$$$ > $@; \
	rm -f $@.$$$$

%.d: %.sx
	@$(CC) -MM $(CPPFLAGS) $(ASFLAGS) $< > $@.$$$$; \
	sed 's,\($*\)\.o[ :]*,\1.o $@ : ,g' <$@.$$$$ > $@; \
	rm -f $@.$$$$

%.d: %.S
	@$(CC) -MM $(CPPFLAGS) $(ASFLAGS) $< > $@.$$$$; \
	sed 's,\($*\)\.o[ :]*,\1.o $@ : ,g' <$@.$$$$ > $@; \
	rm -f $@.$$$$

ifeq (,$(filter $(MAKECMDGOALS),clean clobber reallyclean))
-include $(filter %.d,$(SRCS:.c=.d) $(SRCS:.cc=.d) $(SRCS:.sx=.d) $(SRCS:.S=.d))
endif
