#!/bin/sh

cat <<EOF
# This file is auto-generated

# SDE tools
AS	=$AS
CC	=$CC
C++	=$CXX
CXX	=$CXX
LINK	=$LINK
AR	=$AR
RANLIB	=$RANLIB
SIZE	=$SIZE
STRIP	=$STRIP

# SDE Kit directory
SDETOP  =$SDETOP
SBD	=$SBD
EOF

## escape regular expression magic characters in $SDETOP
root=`echo "${SDETOP}" | sed 's/[./*]/\\\\&/g'`

## replace paths starting with $SDETOP by "$(SDETOP)
sed -e "s/\\([^/]\\)${root}/\\1\$(SDETOP)/g" <<EOF
# Location of SDE source code kit
KITDIR	?=\$(SDETOP)/kit
SBDDIR	=\${KITDIR}/\$(SBD)

# Auto-generated compiler flags
ASFLAGS	=$ASFLAGS
CFLAGS	=$CFLAGS
CXXFLAGS=$CXXFLAGS
CPPFLAGS=$CPPFLAGS

# Auto-generated linker libraries and flags
LOADLIBES =${LOADLIBES:-}
LDFLAGS	=$LDFLAGS
EOF

for target in $ALL; do
case $target in
rom)
    echo "ROMLDFLAGS=$ROMLDFLAGS"
    ;;
ram)
    echo "RAMLDFLAGS=$RAMLDFLAGS"
    ;;
sa)
    echo "SALDFLAGS=$SALDFLAGS"
    ;;
esac
done

sed -e "s/\\([^/]\\)${root}/\\1\$(SDETOP)/g" <<EOF

${VPATH:+VPATH	=${VPATH}}

# User specified source files
SRCS	=$SRCS

OBJS	?= \$(filter %.o,\$(SRCS:.c=.o) \$(SRCS:.cc=.o) \$(SRCS:.sx=.o) \$(SRCS:.s=.o) \$(SRCS:.S=.o))


# User specified libraries
LDLIBS	=${LDLIBS:-}

EOF

echo "all: $ALL"
echo ""

for target in $ALL; do

case $target in
rom)
    sed -e "s/\\([^/]\\)${root}/\\1\$(SDETOP)/g" <<EOF
rom: ${PROG}rom.${ROMDLFMT}

${PROG}rom.${ROMDLFMT}: ${PROG}rom
	sde-conv -vf ${ROMDLFMT} ${ROMFLG} -o \$@ \$<

${PROG}rom: LDSCRIPT ?= -Wl,--script=\$(KITDIR)/share/sde.ld
${PROG}rom: ${CRT0} \${OBJS} \${LDLIBS} ${SBDROM} ${SBDLIBS} ${CRTN}
	\$(LINK.o) \${ROMLDFLAGS} -L\${SBDDIR} ${STDLIBDIR} ${LDMAPFILE} \\
	\${SBDDIR}/romstart.o -Wl,--entry=___reset ${LDSYMS} ${CRT0} \${OBJS} \${LOADLIBES} \${LDLIBS} \\
	-Wl,--start-group -lc ${LIBM} ${LIBRDEBUGROM} -lrom -lgcc ${FPLIBS} -Wl,--end-group \\
	 ${CRTN} \$(LDSCRIPT) -Wl,-n -o \$@

EOF
    ;;

ram)
    if [ "${DLFMT}" != "none" ]; then
      cat <<EOF
ram: ${PROG}ram.${DLFMT}

${PROG}ram.${DLFMT}: ${PROG}ram
	sde-conv -vf ${DLFMT} ${DLSYMS} -o \$@  \$<
EOF
    else
      cat <<EOF
ram: ${PROG}ram
EOF
    fi

    sed -e "s/\\([^/]\\)${root}/\\1\$(SDETOP)/g" <<EOF
${PROG}ram: LDSCRIPT ?= -Wl,--script=\$(KITDIR)/share/sde.ld
${PROG}ram: ${CRT0} \${OBJS} \${LDLIBS} ${SBDRAM} ${SBDLIBS} ${CRTN}
	\$(LINK.o) \${RAMLDFLAGS} -L\${SBDDIR} ${STDLIBDIR} ${LDMAPFILE} ${LDSYMS} \\
	${CRT0} \${OBJS} \${LOADLIBES} \${LDLIBS} \\
	-Wl,--start-group -lc ${LIBM} ${LIBRDEBUGRAM} -lram -lgcc ${FPLIBS} -Wl,--end-group \\
	 ${CRTN} \$(LDSCRIPT) -Wl,-n -o \$@

EOF
    ;;

sa)
    if [ "${DLFMT}" != "none" ]; then
      cat <<EOF
sa: ${PROG}sa.${DLFMT}

${PROG}sa.${DLFMT}: ${PROG}sa
	sde-conv -vf ${DLFMT} ${DLSYMS} -o \$@  \$<
EOF
    else
      cat <<EOF
sa: ${PROG}sa
EOF
    fi

    sed -e "s/\\([^/]\\)${root}/\\1\$(SDETOP)/g" <<EOF
${PROG}sa: LDSCRIPT ?= -Wl,--script=\$(KITDIR)/share/sde.ld
${PROG}sa: ${CRT0} \${OBJS} \${LDLIBS} ${SBDSA} ${SBDLIBS} ${CRTN}
	\$(LINK.o) \${SALDFLAGS} -L\${SBDDIR} ${STDLIBDIR} ${LDMAPFILE} \\
	\${SBDDIR}/sastart.o ${LDSYMS} ${CRT0} \${OBJS} \${LOADLIBES} \${LDLIBS} \\
	-Wl,--start-group -lc ${LIBM} ${LIBRDEBUGROM} -lrom -lgcc ${FPLIBS} -Wl,--end-group \\
	 ${CRTN} \$(LDSCRIPT) -Wl,-n -o \$@

EOF
    ;;

rel)
    sed -e "s/\\([^/]\\)${root}/\\1\$(SDETOP)/g" <<EOF
rel: ${PROG}rel

# --gc-sections works only for a final link
comma := ,
${PROG}rel: LDFLAGS := \$(filter-out -Wl\$(comma)--gc-sections, \$(LDFLAGS))
${PROG}rel: LDSCRIPT ?= -Wl,--script=\$(KITDIR)/share/sde-rsp.ld
${PROG}rel: ${CRT0} \${OBJS} \${LDLIBS} ${SBDRAM} ${SBDLIBS} ${CRTN}
	\$(LINK.o) -r -d -L\${SBDDIR} ${STDLIBDIR} ${LDMAPFILE} \\
	${LDSYMS} ${CRT0} \${OBJS} \${LOADLIBES} \${LDLIBS} -Wl,--no-undefined \\
	-Wl,--start-group -lc ${LIBM} ${LIBRDEBUGRAM} -lrom -lgcc ${FPLIBS} -Wl,--end-group \\
	 ${CRTN} \$(LDSCRIPT) -o \$@

EOF
    ;;

esac
done

cat <<EOF
# Override default build rules for files which need particular #defines
${CRT0}: \$(KITDIR)/share/crt0.sx
	\$(AS) \$(ASFLAGS) \$(CPPFLAGS) \$(CRT0FLAGS) -D__ASSEMBLER__ -x assembler-with-cpp \$< -c -o \$@

\$(OBJS) ${CRT0}: Makefile \$(SBD).sbd

\$(SBD).sbd:
	@[ \$@ != NOSBD.sbd ] || {                             \\
		echo '**************************************'; \\
		echo 'Please specify a valid SBD target name'; \\
		echo '**************************************'; \\
		exit 1; \
	}
	@if [ -f *.sbd ] ; then                                     \\
		echo "++ Rebuilding because SBD variable changed" ; \\
		\$(MAKE) clean \$(SUBCLEAN_INHERIT_F)             ; \\
		rm -f *.d *.sbd                                   ; \\
	else                                                        \\
		echo "Creating \$(SBD).sbd "                      ; \\
	fi
	@echo 'SBDFILE = \$\$(wildcard *.sbd)' >\$@
	@echo 'ifneq (,\$\$(SBDFILE))' >>\$@
	@echo 'export SDETOP ?= \$(SDETOP)' >>\$@
	@echo 'SBD ?= \$\$(shell basename \$\$(SBDFILE) .sbd)' >>\$@
	@echo 'endif' >>\$@

.PHONY: clean
clean:
	rm -f *.[ao]
	rm -f ${PROG}rom ${PROG}ram ${PROG}sa ${PROG}rel
	rm -f ${PROG}rom.* ${PROG}ram.* ${PROG}sa.*
	rm -f ${SBDCLEAN}

.PHONY: clobber reallyclean
clobber reallyclean: clean
	rm -f *.[dis] *.sbd *[~#] core *.core ,* *.map *.gcno *.gcda

# Support .sx extension like .S
SUFFIXES += .sx

%.sx:

%.o: %.sx
	\$(COMPILE.S) \$< \$(OUTPUT_OPTION)

# Obsolete dummy depend target, this is now handled automagically.
.PHONY: depend
depend:

%.d: %.c
	@\$(CC) -MM -MG \$(CPPFLAGS) \$(CFLAGS) \$< > \$@.\$\$\$\$; \\
	sed 's,\(\$*\)\.o[ :]*,\1.o \$@ : ,g' <\$@.\$\$\$\$ > \$@; \\
	rm -f \$@.\$\$\$\$

%.d: %.cc
	@\$(CC) -MM -MG \$(CPPFLAGS) \$(CXXFLAGS) \$< > \$@.\$\$\$\$; \\
	sed 's,\(\$*\)\.o[ :]*,\1.o \$@ : ,g' <\$@.\$\$\$\$ > \$@; \\
	rm -f \$@.\$\$\$\$

%.d: %.sx
	@\$(CC) -MM -MG \$(CPPFLAGS) \$(ASFLAGS) \$< > \$@.\$\$\$\$; \\
	sed 's,\(\$*\)\.o[ :]*,\1.o \$@ : ,g' <\$@.\$\$\$\$ > \$@; \\
	rm -f \$@.\$\$\$\$

%.d: %.S
	@\$(CC) -MM -MG \$(CPPFLAGS) \$(ASFLAGS) \$< > \$@.\$\$\$\$; \\
	sed 's,\(\$*\)\.o[ :]*,\1.o \$@ : ,g' <\$@.\$\$\$\$ > \$@; \\
	rm -f \$@.\$\$\$\$

ifeq (,\$(filter \$(MAKECMDGOALS),clean clobber reallyclean))
-include \$(filter %.d,\$(SRCS:.c=.d) \$(SRCS:.cc=.d) \$(SRCS:.sx=.d) \$(SRCS:.S=.d))
endif
EOF
