/*
 * Copyright (c) 2005-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * spxcpt/spxcpt.c: simple MIPS SDE example showing cpu timer
 * exceptions on SP side of a 34K equipped Malta board.
 */

#include <stdio.h>
#include <mips/xcpt.h>
#include <sbd.h>

#ifdef LEDBAR_ADDR
volatile unsigned int *	const leds = PA_TO_KVA1 (LEDBAR_ADDR);
int 			cv = 0;
#endif

volatile int		ticks;
int			delta;

int
cpu_tick (int code, struct xcptcontext *xcp)
{
    /* Clear interrupt and schedule next tick. */
    mips_setcompare (mips_getcount () + delta);

    /* Update LED display and bump counter. */
#ifdef LEDBAR_ADDR
    *leds = (1 << cv);
    cv = (cv + 1) % 8;
#endif
    ++ticks;

    return (0);
}


void
cpu_timer_setup (void)
{
    /* Clear counter interrupt and program initial tick. */
    mips_setcompare (mips_getcount () + delta);

    /* Install handler for interrupt 7 (IP5). */
    intrupt (7, cpu_tick, 0);

    /* Enable interrupt 7. */
    mips_bissr (SR_IM7);
}


int
main (int argc, char **argv)
{
    /* Interrupt every 0.1 seconds. */
    delta = _sbd_pipefreq () / 10;

#ifdef LEDBAR_ADDR
    /* Initialise LED display. */
    *leds = 0x0;
#endif

    /* Start timer running. */
    cpu_timer_setup ();

    /* Say hello. */
    printf ("hello world\n");

    /* Never exit, or the program stops. */
    while (1 /*ticks < 60*/) ;
	
    return 0;
}
