/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * ALGVME/sbd.h: Algorithmics VME board definitions
 */


#ifndef _ALGVME_SBD_H_
#define _ALGVME_SBD_H_

/* 
 * ROM/RAM and fixed memory address spaces etc.
 */

#define LOCAL_MEM	0x00000000 	/* Local on-board/private memory */
#define LOCAL_MEM_SIZE	0x10000000 	/* Local memory size (256Mb max) */

#define VME_A24D16_BASE	0x10000000 	/* VME A24D16 accesses */
#define VME_A24D32_BASE	0x11000000 	/* VME A24D32 accesses */
#define VME_A24_SIZE	0x02000000	/* 32Mb dedicated A24 address space */

#define LOCAL_PROM	0x1f000000	/* PROM decode address */
#define LOCAL_PROM_SIZE	0x00f00000	/* PROM decode size (15Mb max)*/

#define BOOTPROM	0x1fc00000	/* Boot Prom address */
#define BOOTPROM_SIZE	0x00080000	/* Boot Prom size (512k max)*/

#define VME_A16AM2D	0x1ffe0000 	/* VMEbus A16 AM=0x2d (supervisor) */
#define VME_A16_SIZE	0x00010000	/* VMEbus A16 64Kb */

/*
 * Device Address Spaces
 */
#define SCSI_BASE	0x1ff00000	/* SCSI controller */
#define SONIC_BASE	0x1ff20000	/* Ethernet controller */
#define BCRR		0x1ff40000 	/* Board Configuration/Reset Reg */
#define IACK_BASE	BCRR		/* Interrupt acknowledge */
#define MPSC_BASE	0x1ff44000	/* uPD72001 serial controller */
#define ALPHA_BASE	0x1ff48000 	/* Alphanumeric Display */
#define LED_BASE	0x1ff48000 	/* SL3000 LED Display */
#define ERRREG		0x1ff4c000	/* Error Register (VME3000 only) */
#define RTCLOCK_BASE	0x1ff60000	/* Mk48T02 NVRAM/RTC */
#define IACK_BASE2	0x1ffa0000	/* alternative interrupt ack */
#define VIC_BASE	0x1ffc0000	/* VIC device */
#define VAC_BASE	0x1ffd0000	/* VAC device */

/* Wiring of local interrupts to VIC interrupt controller */
#define LIRQ_SONIC	1		/* Sonic ethernet */
#define LIRQ_TIMER	2		/* VIC timer */
#define LIRQ_SCSI	3		/* NCR SCSI */
#define LIRQ_VACSER	4		/* VAC serial */
#define LIRQ_MPSC	5		/* MPSC serial */
#define LIRQ_DEBUG	6		/* VAC debug/perr */
#define LIRQ_MAILBOX	7		/* VAC mailbox */

/*
 * Configuration/reset register (write-only)
 */
#ifdef SL3000
#define BCRR_LBLK	0x01000000	/* SL3000 LED blank (active high) */
#else
#define BCRR_AUBLK	0x01000000	/* display blank (active low)*/
#define BCRR_AUCLR	0x02000000	/* display clear (active low) */
#define BCRR_ACSR	0x04000000	/* cursor enable */
#endif
#define BCRR_USART	0x08000000	/* enable uPD72001 usart */
#define BCRR_ETH	0x10000000	/* enable SONIC ethernet */
#define BCRR_SCSI	0x20000000	/* enable NCR SCSI */

/* Alphanumeric display */
#ifndef SL3000
/* offsets to display entries */
#define ALPHA_ENTRY(n)	(16 + ((3 - (n)) * 4))
#define ALPHA_DISP(n)	(ALPHA_BASE + ALPHA_ENTRY(n))
#endif

/* Error register (VME3000 only) */
#define ERR_CYC_MASK	0x38000000
#define ERR_CYC_SHIFT	27
#define ERR_CYC_NONE	0
#define ERR_CYC_CPU	1
#define ERR_CYC_IO	2
#define ERR_CYC_CPE	3
#define ERR_CYC_BRST4	4
#define ERR_CYC_BRST8	5
#define ERR_CYC_BRST16	6
#define ERR_CYC_BRST32	7
#define ERR_CYC_BMASK	0x20000000
#define ERR_ADDR_SHIFT	2
#define ERR_PAIR_SHIFT	5
#define ERR_B0A_MASK	0x07ffffe0
#define ERR_B4A_MASK	0x07ffffc0
#define ERR_B4P_MASK	0x00000020
#define ERR_B8A_MASK	0x07ffff80
#define ERR_B8P_MASK	0x00000060
#define ERR_B16A_MASK	0x07ffff00
#define ERR_B16P_MASK	0x000000e0
#define ERR_B32A_MASK	0x07fffe00
#define ERR_B32P_MASK	0x000001e0
#define ERR_BANK_MASK	0x00000010
#define ERR_BANKA	0x00000010
#define ERR_BANKB	0x00000000
#define ERR_BYTE3	0x00000008
#define ERR_BYTE2	0x00000004
#define ERR_BYTE1	0x00000002
#define ERR_BYTE0	0x00000001

#ifndef __ASSEMBLER__
void _vic_vmeinterrupt_enable (int);
void _vic_vmeinterrupt_disable (int);
void _vic_dmainterrupt_enable (void);
void _vic_dmainterrupt_disable (void);
void _vic_icgsinterrupt_enable (int, int, int, int);
void _vic_icgsinterrupt_disable (int, int, int, int);
void _vic_icmsinterrupt_enable (int, int, int, int);
void _vic_icmsinterrupt_disable (int, int, int, int);
void _vic_errorinterrupt_enable (int, int, int, int);
void _vic_errorinterrupt_disable (int, int, int, int);
void _vic_locinterrupt_enable (int);
void _vic_locinterrupt_disable (int);
#ifdef _XCPT_H_
xcpt_t _vic_locintrupt (unsigned int, xcpt_t, int);
#endif
void _vac_acktimer (void);
void _vic_initialise (void);
#endif

#endif /* _ALGVME_SBD_H_ */

