/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * VME4000/sbdclock.c: timer support functions for Algorithmics VME board
 */


#if #cpu(r4000)

/*
 * On an R4000 board, use the on-chip timer.
 */

#include "../share/r4kclock.c"

#else

#ident "kit/ALGVME/sbdclock.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * We have to use the VIC timer.  This runs either 50Hz, 100Hz or 1000Hz.
 */

#include <stddef.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/clock.h>
#include <mips/cpu.h>
#include <mips/xcpt.h>
#include <assert.h>

#include "kit_impl.h"
#include "sbd.h"
#include "vic068.h"

static clock_t		(*tickfunc)();
static clock_t		tick;
static clock_t		maxtick;
static int		ticks;

clock_t			_clk_tck;

static void
vic_startrtclock (int hz)
{
    unsigned int sel0;

    sel0 = *VIC_SEL0CR0 & 0x3f;
    switch (hz) {
    default:
	abort();
    case 0:
	sel0 |= VIC_SL0TNONE;
	break;
    case 50:
	sel0 |= VIC_SL0T50;
	break;
    case 100:
	sel0 |= VIC_SL0T100;
	break;
    case 1000:
	sel0 |= VIC_SL0T1000;
	break;
    }
    *VIC_SEL0CR0 = sel0;
    mips_wbflush ();
    _clk_tck = hz;
}


#define HZ50	(1000000 / 50)
#define HZ100	(1000000 / 100)
#define HZ1000	(1000000 / 1000)

void
_sbd_settimer (clock_t newtick)
{
    clock_t res;

    if (newtick != tick) {
	if (newtick >= HZ50 && (newtick % HZ50) == 0) {
	    vic_startrtclock (50);
	    res = HZ50;
	} 
	else if (newtick >= HZ100 && (newtick % HZ100) == 0) {
	    vic_startrtclock (100);
	    res = HZ100;
	}
	else {
	    vic_startrtclock (1000);
	    res = HZ1000;
	}
	ticks = (newtick + res / 2) / res;
	tick = newtick;
    }
}


static int
clkintr (int arg, struct xcptcontext *xcp)
{
    clock_t newtick;

    /* call timer tick function */
    newtick = (*tickfunc) (ticks, xcp);
    
    /* reprogram timer if a new tick interval is required */
    if (newtick != tick)
	_sbd_settimer (newtick);

    return 0;
}


clock_t
_sbd_gettimer ()
{
    /* can't read timer, sorry */
    return 0;
}


clock_t
_sbd_inittimer (clock_t (*func)())
{
    /* worst resolution (and thus longest tick) is 50Hz */
    maxtick = HZ50;
    
    /* make sure that interrupt controller is programmerd */
    _vic_initialise ();

    /* stop real-time clock */
    vic_startrtclock (0);

    /* install our interrupt handlers */
    tickfunc = func;
    _vic_locintrupt (LIRQ_TIMER, clkintr, 0);
    _vic_locinterrupt_enable (LIRQ_TIMER);

    return maxtick;
}

#endif

