#ident "kit/BDMR4102/sbdclock.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1999-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * BDMR4102/sbdclock.c: timing support functions for LSI Logic BDMR4102
 */


#include "sbd.h"

#include <stddef.h>
#include <sys/time.h>
#include <sys/clock.h>
#include <mips/cpu.h>
#include <mips/xcpt.h>
#include <assert.h>

#include "kit_impl.h"
#include "sbd.h"

static clock_t		(*tickfunc)();

static clock_t		curtime;
static clock_t		nexttime;
static unsigned long	cycpertick;
static unsigned long	cycperusec;
static unsigned long	usecpertick;
static unsigned int	tickspermsec;

clock_t			_clk_tck;

static volatile struct _timer_module * const tmr
	= (struct _timer_module *) TMR_BASE;

void
_sbd_settimer (clock_t newtick)
{
    /* next interrupt will catch it */
    nexttime = newtick;
}


static int
clkintr (int arg, struct xcptcontext *xcp)
{
    /* clear timer interrupt */
    tmr->tstat = TSTAT_IEN0;
    
    /* bump microsecond counter */
    curtime += usecpertick;
    if (curtime >= nexttime) {
	/* call timer tick function on hitting limit */
	nexttime = (*tickfunc) (1, xcp);
	curtime = 0;
    }

    return 0;
}


clock_t
_sbd_gettimer ()
{
    /* return how many microseconds have passed since last interrupt */
    unsigned long cycles;

    /* get timer 0 */
    cycles = tmr->tcc0;

    /* convert downcount to upcount */
    cycles = (cycpertick - cycles);

    /* check for possible missed interrupt */
    if (cycles < cycpertick / 2 && (tmr->tstat & TSTAT_INT0))
      cycles += cycpertick;

    return curtime + (cycles / cycperusec);
}


clock_t
_sbd_inittimer (clock_t (*func)())
{
    if (_clk_tck == 0)
	/* get CPU frequency */
	_clk_tck = _sbd_pipefreq ();

    /* work out timing parameters based on CPU frequency */
    
    /* get close to 1 interrupt (tick) per msec */
    cycperusec = _clk_tck / 1000000;
    cycpertick = _clk_tck / 1000;
    tickspermsec = 1;
    usecpertick = 1000;
    while (cycpertick >= 65536) {
	tickspermsec *= 2;
	cycpertick /= 2;
	usecpertick /= 2;
    }

    /* program timer with chosen frequency */
    tmr->tmode &= ~TMODE_MASK0;	/* disable timer 0 */
    tmr->tic0 = cycpertick;	/* program counter value */
    tmr->tstat = TSTAT_IEN0;	/* enable timer interrupts */
    tmr->tmode |= TMODE_EN0;	/* enable timer 0 */

#if 0
    /* enable timer 0 interrupt 0 in SCR2 register XXX this is a guess */
    *(volatile unsigned int *)BSYS_SCR2 |= SCR2_TMRIE0;
#endif

    /* disable calls to tickfunc until settimer called */
    curtime = 0;
    nexttime = ~0;

    /* install interrupt handler */
    tickfunc = func;
    intrupt (INTR_TMR0, clkintr, 0);
    mips_bissr (SR_IM0 << INTR_TMR0);

    /* Return the worst resolution of the timer in microseconds
       (i.e. the longest it can count before interrupting). 
       We implement a pseudo 32-bit timer. */
    return 0x7fffffff;
}
