#ident "kit/SKEL/sbdclock.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * SKEL/sbdclock.c: timing support functions for SKEL board
 */


#include <stddef.h>
#include <sys/time.h>
#include <sys/clock.h>
#include <mips/cpu.h>
#include <mips/xcpt.h>
#include <assert.h>

#include "kit_impl.h"
#include "sbd.h"

/*
 * This module should contain the low-level timing support
 * code for this board.  If this is an R4000 based board, then
 * you can probably just #include "../share/r4kclock.c".
 * 
 * Otherwise see other examples as follows:
 * ../share/s2681clk.c		Signetics S2681/Motorola 68681 UART timer
 * ../ALGVME/sbdclock.c		VIC068 interval timer
 * ../IDT385/sbdclock.c		Intel i8254 programmable timer
 * ../LSIPR/sbdclock.c		LSI LR330x0 on-chip timer
 */


/* Define the best resolution of the timer in Hz */
clock_t		_clk_tck = MHZ * 1000000;

/* The interrupt function to call, and the interval (in microseconds) */
static clock_t	(*tickfunc)();
static clock_t	tick;


void
_sbd_settimer (clock_t newtick)
{
    if (tick != newtick) {
	/* Program the timer to interrupt every NEWTICK microseconds, */
    }
}


clock_t
_sbd_gettimer ()
{
    /* Return how many microseconds have passed since last interrupt,
       as accurately as possible. */
    return (0);
}


static int 
clkintr (int arg, struct xcptcontext *xcp)
{
    clock_t newtick;
    int ticks;

    /* Call the generic timer code's tick function, passing
       the number of interrupt times that have elapsed since
       the last call (usually 1). */
    ticks = 1;
    newtick = (*tickfunc) (ticks, xcp);
    
    /* You may be asked to vary the interrupt rate now. */
    if (newtick != tick)
	_sbd_settimer (newtick);

    return 0;
}


clock_t
_sbd_inittimer (clock_t (*func)())
{
    int maxtick;

    /* Initialise the timer, and arrange for it to call FUNC on
       every interrupt.  Install the interrupt handler and
       enable the interrupt. */
    
    tickfunc = func;
    intrupt (6, clkintr, 0);
    mips_bissr (SR_IM6);	/* enable interrupt now */

    /* Return the worst resolution of the timer in microseconds
       (i.e. the longest it can count before interrupting). */
    maxtick = 1000000;

    return maxtick;
}


