#ident "kit/SKEL/sbdfreq.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * SKEL/sbdfreq.c: CPU frequency determination for SKEL board
 */


#include <stddef.h>
#include <time.h>
#include <errno.h>
#include <mips/cpu.h>
#include <mips/prid.h>
#include "kit_impl.h"

/*
 * Dynamically determine CPU pipeline frequency by running the CPU
 * on-chip timer against a known frequency timer.
 */
long
_sbd_pipefreq (void)
{
    /* If you have both an on-chip CPU timer, and an external timer
       with a known frequency, then you can dynamically determine
       the CPU pipeline frequency (instruction execution rate).
       
       This is really only necessary for boards using an on-chip
       timer for sbdclock.c (e.g. R4000-based boards), see
       ../p4032/sbdfreq.c for an example.

       On many boards you can just return a fixed frequency, e.g.
       33 MHz or whatever you are designing. */

    return 33000000;
}


/*
 * Return the CPU bus clock frequency (usually same input clock).
 */

#ifdef CFG_ECMASK
/* Bus clock to pipeline clock ratio tables */
struct ratio {char mult; char div;};
static const struct ratio ratios_norml[8] = {{2,1}, {3,1}, {4,1}, {5,1},
					     {6,1}, {7,1}, {8,1}, {1,1}};
static const struct ratio ratios_r4300[8] = {{2,1}, {3,1}, {4,1}, {5,1},
					     {6,1}, {7,1}, {1,1}, {3,2}};
static const struct ratio ratios_r54xx[8] = {{2,1}, {5,2}, {3,1}, {4,1},
					     {5,1}, {6,1}, {7,1}, {1,1}};
#endif

long
_sbd_cpufreq (void)
{
    static long cpufreq;
    if (cpufreq == 0) {
	long freq = _sbd_pipefreq ();
#ifdef CFG_ECMASK
	/* pipeline runs at multiple of input clock, so divide down */
	const struct ratio *ratios;
	switch ((mips_getprid() >> 8) & 0xff) {
	case PRID_R4300:
	    ratios = ratios_r4300;
	    break;
	case PRID_R5400:
	    ratios = ratios_r54xx;
	    break;
	default:
	    ratios = ratios_norml;
	    break;
	}
	ratios += (mips_getconfig() & CFG_ECMASK) >> CFG_ECSHIFT;
	freq = freq * ratios->div / ratios->mult;
#endif
	cpufreq = freq;
    }
    return cpufreq;
}
