#ident "kit/SKEL/sbdmem.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2000-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * SKEL/sbdmem.c: determine memory layout on SKEL board
 */


#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <mips/cpu.h>
#include "kit_impl.h"

#include "sbd.h"

/*
 * _sbd_memlayout returns a pointer to a table of memory regions.
 *
 * This function is only needed if the board contains non-contigusous
 * RAM regions.  At the moment sbrk() only uses the first region in
 * the table, so usually you will want to make sure that the largest
 * region is first.
 */

const struct sbd_region *
_sbd_memlayout (void)
{
    static struct sbd_region mem[3];
    struct sbd_region *m = mem;
    extern paddr_t _mem_top;

    if (m->type == SBD_MEM_END) {
	/* DRAM first, if it's enabled */
	if (SYSCTLR_DRAM & SYSCTLR_DRAM_ENB) { 
	    m->type = SBD_MEM_RAM;
	    m->base = SYSCTLR_DRAM_BASE;	/* physical addr */
	    m->size = _mem_top - m->base;	/* from memory size test */
	    ++m;
	}

	/* SRAM second, if accessible and disjoint from DRAM */
	m->base = SYSCTLR_SRAM_BASE;
	if (m->base == 0 || (m->base < KSEG1_SIZE && m->base > _mem_top)) {
	    /* disjoint SRAM */
	    m->type = SBD_MEM_RAM;
	    m->size = ~SYSCTLR_SRAM_MASK + 1;	/* from h/w address mask */
	}
    }

    return mem;
}
