#ident "kit/SKEL/sbdnvram.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1997-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * SKEL/sbdnvram.c: manage board-specific non-volatile memory
 */


#include <string.h>
#include <stdlib.h>

#include "kit_impl.h"
#include "sbd.h"

#define EESIZE		0	/* total size of nvram (bytes) */
#define NVSIZE		0	/* size of environment area (bytes) */
#define NVOFFSET	0	/* byte offset of environment area */

#ifdef GENERIC_NVENV
#include "nvenv.h"		/* generic environment definitions */
#endif

/* Return total size of non-volatile memory, and optionally the size
   and position of the area within it to hold the NAME=VALUE
   environment store. */
unsigned int 
_sbd_nvram_size (unsigned int *nvoffs, unsigned int *nvsize)
{
    if (nvoffs) *nvoffs = NVOFFSET;
    if (nvsize) *nvsize = NVSIZE;
    return EESIZE;
}


/* Called to indicate that any pending writes to the storage element
   containing byte offset OFFS should be saved.  If OFFS == -1 then
   flush all pending writes.  */
void
_sbd_nvram_flush (int offs)
{
}


/* Return the byte at offset OFFS */
unsigned char
_sbd_nvram_getbyte (unsigned int offs)
{
    return 0;
}


/* Set the byte at offset OFFS to VAL */
void
_sbd_nvram_setbyte (unsigned int offs, unsigned char val)
{
}


/* Return the 16-bit word at byte offset OFFS */
unsigned short
_sbd_nvram_getshort (unsigned int offs)
{
    return 0;
}


/* Set the 16-bit word at byte offset OFFS to VAL */
void
_sbd_nvram_setshort (unsigned int offs, unsigned short val)
{
}


/* Run a quick r/w hardware test on the NVRAM, and return 1 for
   success, 0 for failure. */
int
_sbd_nvram_hwtest (void)
{
    return 1;
}


/* Return non-zero if user has somehow requested a forcible reset
   of the environment store (i.e. by inserting a link or linking
   some UART flow-control pins together). */
int
_sbd_env_force_reset (void)
{
    return 0;
}


#ifdef PRIVATE_NVENV

/* 
 * Manage the non-volatile environment store yourself.
 * Normally add ../share/nvenv.c or ../share/flashenv.c to sbd.mk
 */


/* Return the value of NAME */
char *	
_sbd_getenv (const char *name)
{
    return NULL;
}


/* Set NAME to be the string VAL; don't overwrite an existing value
   unless OVERWRITE is non-zero.  Return zero for success, non-zero
   otherwise. */
int 
_sbd_setenv (const char *name, const char *val, int overwrite)
{
    return 0;
}


/* Remove the NAME variable. */
void
_sbd_unsetenv (const char *name)
{
}


/* For each NAME=VALUE pair, call the supplied function. */
void 
_sbd_mapenv (int (*func)(const char *name, const char *val))
{
}

/* Do any initialisation or preloading of the NVRAM, and return an
   error string if there is any problem, or NULL if all is well. */
const char *
_sbd_envinit (void)
{
    return NULL;
}


#endif /* !GENERIC_NVRAM */
