#ident "kit/SKEL/sbdpanel.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2000-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * SKEL/sbdpanel.c: interface to the LED panel display
 */


#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <sys/panelio.h>
#include <mips/cpu.h>

#include "kit_impl.h"
#include "sbd.h"


/* local display mode */
static struct panelmode	led_mode;


/* write NB characters from ascii string STR to <ROW,COL> on LED display, 
   using options OPT */
void
_sbdpanelmessage (int row, int col, const char *str, size_t nb, int opt)
{
    volatile unsigned int *flash = PA_TO_KVA1 (LED_BASE + HD2532_FLASH_OFFSET);
    volatile unsigned int *led = PA_TO_KVA1 (LED_BASE + HD2532_CRAM);
    int i;

    /* ignore row */
    for (i = col; i < 8; i++) {
	if ((*str || (opt & PANELOPT_IGNNUL)) && nb > 0) {
	    led[i] = *str++;
	    flash[i] = (opt & PANELOPT_FLASH) ? 1 : 0;
	    nb--;
	}
	else if (opt & (PANELOPT_PAD | PANELOPT_CENTRE)) {
	    led[i] = ' ';
	    flash[i] = 0;
	}
	else 
	    break;
    }
}


/* update progress bar graph */
void
_sbdpanelprogress (int percent)
{
    /* (not supported) */
}


/* new control interface */
void 
_sbdpanelmode (const struct panelmode *new, struct panelmode *old)
{
    if (old)
	*old = led_mode;

    if (new) {
	volatile unsigned int *ledcw = PA_TO_KVA1 (LED_BASE) + HD2532_CW;

	if (new->blinkoff != 0 && led_mode.blinkoff == 0) {
	    /* switch blinking on (can't control speed) */
	    *ledcw |= HD2532_CW_BL;
	}
	else if (new->blinkoff == 0 && led_mode.blinkoff != 0) {
	    /* switch blinking off */
	    *ledcw &= ~HD2532_CW_BL;
	}

	if (led_mode.brighton != new->brighton) {
	    /* change brightness */
	    unsigned char b;
	    switch (new->brighton) {
	    case  0 ...  6:
		b = HD2532_CW_B0;	/* 0% */
		break;
	    case  7 ... 16:
		b = HD2532_CW_B13;	/* 13% */
		break;
	    case 17 ... 23:
		b = HD2532_CW_B20;	/* 20% */
		break;
	    case 24 ... 33:
		b = HD2532_CW_B27;	/* 27% */
		break;
	    case 34 ... 46:
		b = HD2532_CW_B40;	/* 40% */
		break;
	    case 47 ... 66:
		b = HD2532_CW_B53;	/* 50% */
		break;
	    case 67 ... 90:
		b = HD2532_CW_B80;	/* 80% */
		break;
	    default:
		b = HD2532_CW_B100;	/* 100% */
		break;
	    }
	    *ledcw = (*ledcw & ~HD2532_CW_BMASK) | b;
	}

	led_mode = *new;
    }
}


/* clear whole display */
void
_sbdpanelclear (void)
{
    *ledcw |= HD2532_CW_C;
}


/* query display */
void 
_sbdpanelinfo (struct panelinfo *info)
{
    /* 1 x 8 LED display with controllable brightness and blink/flash support */
    info->type = PANELTYPE_ALPHA;
    info->flags = PANELFLGS_BRIGHTNESS | PANELFLGS_BLINK | PANELFLGS_FLASH;
    info->rows = 1;
    info->cols = 8;
}


int 
_sbdpanelinit (void)
{
    volatile unsigned int *ledcw = PA_TO_KVA1 (LED_BASE) + HD2532_CW;

    /* initialise hardware */
    *ledcw = HD2532_CW_C | HD2532_CW_B100;

    /* set default mode */
    memset (&led_mode, 0, sizeof (led_mode));
    led_mode.brighton = led_mode.contrast = 100;

    return 1;
}
