#ident "kit/atlas/sbdenv.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2000-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * sbdenv.c: dummy non-volatile environment store for YAMON
 */


#include <string.h>
#include "kit_impl.h"

extern char **_yamon_env;	/* set in ../yamon/yamonlow.sx */

char *	
_sbd_getenv (const char *name)
{
    if (_yamon_env) {
	/* scan non-standard YAMON environment */
	char **envp;
	for (envp = _yamon_env; *envp; envp += 2)
	    if (strcmp (name, envp[0]) == 0)
		return envp[1];
    }
    return NULL;
}

int 
_sbd_setenv (const char *name, const char *val, int overwrite)
{
    /* we don't allow modification of YAMON environment */
    return 0;
}

void
_sbd_unsetenv (const char *name)
{
    /* we don't allow modification of YAMON environment */
}

void 
_sbd_mapenv (int (*func)(const char *name, const char *val))
{
    if (_yamon_env) {
	/* scan non-standard YAMON environment */
	char **envp;
	for (envp = _yamon_env; *envp; envp += 2)
	    (*func) (envp[0], envp[1]);
    }
}

const char *
_sbd_envinit (void)
{
    return NULL;
}
