/*
 * Copyright (c) 2002-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * atlas/sbdicu.h: interrupt numbering for MIPS Atlas board
 */


/* MIPS RTPX specials */
#define INTR_CLOCK	-1
#define INTR_NONE	0

#define INTR_IS_SAA9730(n)	(((n) & 0x78) == 0x00)
#define INTR_IS_CPU(n)		(((n) & 0x78) == 0x08)
#define INTR_IS_ATLAS(n)	((n) & 0x20)
#define INTR_IS_GT64120(n)	((n) & 0x40)

/* CPU interrupts */
#define INTR_SINT0		8
#define INTR_SINT1		9
#define INTR_ICUA		10	/* Atlas int ctrlr */
#define INTR_HINT1		11
#define INTR_HINT2		12
#define INTR_HINT3		13
#define INTR_HINT4		14
#define INTR_TIMER		15	/* CPU timer int */ 

/* External ATLAS interrupt controller */
#define INTR_UART0		32	/* Serial port */
#define INTR_TIM0		33	/* Timer 0 in CBUS FPGA */
#define INTR_RTC		36	/* Real Time Clock */
#define INTR_COREHI		37	/* Core Card Hi (GT64120) */
#define INTR_CORELO		38	/* Core Card Lo (n/a) */
#define INTR_CPCI_A		40	/* Compact PCI IRQA */
#define INTR_CPCI_B		41	/* Compact PCI IRQB */
#define INTR_CPCI_C		42	/* Compact PCI IRQC */
#define INTR_CPCI_D		43	/* Compact PCI IRQD */
#define INTR_ENUM		44	/* PCI ENUM# signal */
#define INTR_DEG		45	/* PCI DEG# signal */
#define INTR_ATXFAIL		46	/* ATX PSU */
#define INTR_PCI_A		47	/* Primary PCI IRQA*/
#define INTR_PCI_B		48	/* Primary PCI IRQB (inc SAA9730) */
#define INTR_PCI_C		49	/* Primary PCI IRQC (inc SCSI) */
#define INTR_PCI_D		50	/* Primary PCI IRQD */
#define INTR_SERR		51	/* PCI SERR# signal */

/* Core Galileo GT64120 interrupts */
#define  INTR_INTSUM		64
#define  INTR_MEMOUT		65
#define  INTR_DMAOUT		66
#define  INTR_CPUOUT		67
#define  INTR_DMA0COMP		68
#define  INTR_DMA1COMP		69
#define  INTR_DMA2COMP		70
#define  INTR_DMA3COMP		71
#define  INTR_T0EXP		72
#define  INTR_T1EXP		73
#define  INTR_T2EXP		74
#define  INTR_T3EXP		75
#define  INTR_MASRDERR		76
#define  INTR_SLVWRERR		77
#define  INTR_MASWRERR		78
#define  INTR_SLVRDERR		79
#define  INTR_ADDRERR		80
#define  INTR_MEMERR		81
#define  INTR_MASABORT		82
#define  INTR_TARABORT		83
#define  INTR_RETRYCTR0		84
#define  INTR_PMCINT0		85
#define  INTR_CPU2PCIA		86
#define  INTR_CPU2PCIB		87
#define  INTR_CPU2PCIC		88
#define  INTR_CPU2PCID		89
#define  INTR_PCI2CPUA		90
#define  INTR_PCI2CPUB		91
#define  INTR_PCI2CPUC		92
#define  INTR_PCI2CPUD		93
#define  INTR_CPUINTSUM		94
#define  INTR_PCIINTSUM		95

#endif
