#ident "kit/atlas/sbdrtc.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2000-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * atlas/sbdrtc.c: low-level interface to Dallas 1687 real-time clock chip
 */


#include <stddef.h>
#include <mips/cpu.h>

#include "sbd.h"
#include "ds1687.h"

static unsigned int
rtc_select (int reg)
{
    volatile unsigned int * rtcadr = PA_TO_KVA1 (RTC_ADDR);
    volatile unsigned int * rtcdat = PA_TO_KVA1 (RTC_DATA);
    unsigned char rtca;

    /* PC97307 RTC */
    if (reg < 0x40)
	/* accessible from any bank */
	*rtcadr = reg;
    else {
	*rtcadr = RTC_STATUSA;
	rtca = *rtcdat;
	if (reg < 0x80) {
	    /* accessible from bank 0 */
	    if ((rtca & RTCSA_DVMASK) != RTC_DV0_OSC_ON)
		*rtcdat = (rtca & ~RTCSA_DVMASK) | RTC_DV0_OSC_ON;
	    *rtcadr = reg;
	}
	else {
	    /* accessible indirectly via bank 1 */
	    if ((rtca & RTCSA_DVMASK) != RTC_DV1_OSC_ON)
		*rtcdat = (rtca & ~RTCSA_DVMASK) | RTC_DV1_OSC_ON;
	    *rtcadr = RTC_BANK1_URADDR;
	    *rtcdat = reg - 0x80;
	    *rtcadr = RTC_BANK1_URDATA;
	}
    }
    return *rtcdat & 0xff;
}


unsigned int
_rtc_get (int reg)
{
    return rtc_select (reg);
}


unsigned int
_rtc_set (int reg, unsigned int val)
{
    unsigned int o;

    o = rtc_select (reg);
    if (val != o)
	*(volatile unsigned int *) PA_TO_KVA1 (RTC_DATA) = val;
    return (o);
}

unsigned int
_rtc_bis (int reg, unsigned int val)
{
    unsigned int o, n;

    o = rtc_select (reg);
    n = o | val;
    if (n != o)
	*(volatile unsigned int *) PA_TO_KVA1 (RTC_DATA) = n;
    return (o);
}


unsigned int
_rtc_bic (int reg, unsigned int val)
{
    unsigned int o, n;

    o = rtc_select (reg);
    n = o & ~val;
    if (n != o)
	*(volatile unsigned int *) PA_TO_KVA1 (RTC_DATA) = n;
    return (o);
}
