/*
 * Copyright (c) 2000-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * idt332x/sbd.h: board-specific definitions for IDT 79S33x boards
 */


#ifndef _IDT33X_SBD_H_
#define _IDT33X_SBD_H_

/* Board parameters which must be altered to match the hardware: */

/* System clock frequency */
#ifndef SYSCLK_HZ
# if defined(IDT334A)
#  define SYSCLK_HZ		75000000	/* 75 MHz */
# elif defined(IDT332)
#  define SYSCLK_HZ		66500000	/* 66.5 MHz */
# else
#  error Unknown board - please set SYSCLK_HZ
# endif
#endif

/* Memory type and layout (default: 32MB SODIMM + SRAM) */
#ifndef IDT33X_SRAM_FIRST
#define IDT33X_SRAM_FIRST	0	/* 1 if SRAM @ 0x00000000 */
#define IDT33X_SRAM_LAST	1	/* 1 if DRAM @ 0x00000000 */
#define IDT33X_DRAM_EDO		0	/* 1 if EDO DRAM fitted */
#define IDT33X_DRAM_SDRAM	1	/* 1 if SDRAM fitted */
#define IDT33X_SDRAM_32MB	0	/* 1 if SDRAM is 32Mb */
#define IDT33X_SDRAM_32MBSO	1	/* 1 if SDRAM is 32Mb SODIMM */
#define IDT33X_SDRAM_128MB	0	/* 1 if SDRAM is 128Mb */
#endif

/* Console device (default: internal) */
#if !defined(IDT33X_CONSOLE_INTERN)
#define IDT33X_CONSOLE_INTERN 	1	/* 0 for external */
#endif

/* set CPU Intr5 wiring (default: CPU timer) */
#ifndef IDT33X_HINT5_TIMER
#define IDT33X_HINT5_TIMER	1	/* 0 for PCI INTD */
#endif

/* Bit-wise OR of possible Flash memory width in bits */
#ifndef _SBD_FLASHWIDTH
#define _SBD_FLASHWIDTH		(8 | 32)
#endif

/* Maximum CPU pipeline clock in MHz (fixed) */
#ifndef MHZ
#define MHZ			((SYSCLK_HZ * 2 + 500000) / 1000000)
#endif

#if 0
/* 
 * Optionally define a non-zero physical memory base offset
 * XXX (strongly discouraged!).
 */
# define MEMSTART		0x0
#endif

/* Onboard device addresses */
#define DRAM_SIZE		0x10000000	/* 256MB DRAM max */
#define SRAM_SIZE		0x00100000	/*   1MB SRAM fixed */
#define INTSTAT_BASE		0x10000000	/* CS2: i/u status 	 8b */
#define NVRAM_BASE		0x12000000	/* CS3: xicor x20c04	 8b */
#define LED_BASE		0x14000000	/* CS4: 4-char display 	 8b */
#define  LED_CHAR(n)		 (((3-(n))*4)+3) /* offs of char N */
#define  LED_CLR		 0x400		 /* clear display */
#define DUART_BASE		0x16000000	/* CS5: ext Z8530 UART 	 8b */
#define RC3233X_BASE		0x18000000	/* Internal regs 	32b */
#define PCI_IO_SPACE		0x18800000	/* CPU->PCI i/o space	32b */
#define PCI_IO_SPACE_SIZE	0x00100000	/* 1Mb i/o window  */
#define BOOTPROM_BASE		0x1fc00000	/* EPROM/Flash 		??b */
#define FLASH_BASE		0x1fc00000	/*      /Flash		??b */
#define EPROM_BASE		0x1fc00000	/* EPROM/		??b */
#define FLASH_SIZE		0x00200000	/* 2MB = 4 * 29F040 (512KB) */
#define EPROM_SIZE		0x00400000	/* 4MB = 4 * 27C801 (  1MB) */
#define PCI_MEM_SPACE		0x40000000	/* CPU->PCI mem #1 	32b */
#define PCI_MEM2_SPACE		0x60000000	/* CPU->PCI mem #2 	32b */
#define PCI_MEM_SPACE_SIZE	0x20000000	/* 512Mb mem window #1 */
#define PCI_MEM2_SPACE_SIZE	0x20000000	/* 512Mb mem window #2 */

/* PCI bus address spaces */
#define PCI_IO_SPACE_PCI_BASE	0x00000000
#define PCI_MEM_SPACE_PCI_BASE	0x00000000

/* PCI mem 40000000 -> CPU 00000000 */
#define PCI_LOCAL_MEM_PCI_BASE	0x40000000

/* PCI i/o 00100000 -> CPU 18000500 */
#define PCI_LOCAL_IO_PCI_BASE	0x00100000

/* External uart input clock frequency */
#define Z8530CLOCK	3686400

/* External uart delay between accesses */
#define Z8530DELAY	0

/* External uart register layout offsets; 8-bit bus width */
/* note: unusually byte offsets are endian independent */
#define Z8530_CHB	0
#define Z8530_CHA	8
#ifdef __ASSEMBLER__
# define Z8530_CMD	3
# define Z8530_DATA	7
# define Z8530RESET(reg) lbu zero,0x400(reg)
#else
# define z8530reg0	unsigned :24; unsigned cmd:8
# define z8530reg1	unsigned :24; unsigned data:8
# define Z8530RESET(dp)	(void)*((volatile char *)dp + 0x400)
#endif

/* Internal UART #0 */
#define UART0_BASE	(RC3233X_BASE + 0x800)

#if defined(IDT334A) || defined(IDT334)
/* Internal UART #1 - only pinned out on RC32334 */
#define UART1_BASE	(RC3233X_BASE + 0x820)
#endif

/* Internal UART runs directly off bus clock! */
#define NS16550_HZ	SYSCLK_HZ

/* Internal UART register reads are fast, so increase wait timer */
#define NS16550_TXWAIT	100000

/* Internal UART registers mapped every 4 bytes */
#if #endian(big)
#define NSREG(x)	((x)*4+3)
#else
#define NSREG(x)	((x)*4+0)
#endif

/* IntStat bits */
#define INTSTAT_PCIA_N	0x01	/* PCI INTA~ */
#define INTSTAT_PCIB_N	0x02	/* PCI INTB~ */
#define INTSTAT_PCIC_N	0x04	/* PCI INTC~ */
#define INTSTAT_PCID_N	0x08	/* PCI INTD~ */
#define INTSTAT_PCIM66	0x10	/* 66MHz PCI link in */
#define INTSTAT_SCC_N	0x20	/* Ext UART i/u~ */
#define INTSTAT_PCISAT	0x40	/* PCI in satellite mode */
#define INTSTAT_DCD_N	0x80	/* UART DCD~ */

/* board features */
#define _SBD_PANEL

/*
 * rough scaling factors for 2 instruction DELAY loop to get 1ms and 1us delays
 */
#define ASMDELAY(ns,icycle)	\
	(((ns) + (icycle)) / ((icycle) * 2))

#define CACHENS(ns)	ASMDELAY((ns), CACHECYCLE)
#define RAMNS(ns)	ASMDELAY((ns), CACHEMISS+RAMCYCLE)
#define ROMNS(ns)	ASMDELAY((ns), CACHEMISS+ROMCYCLE)
#define CACHEUS(us)	ASMDELAY((us)*1000, CACHECYCLE)
#define RAMUS(us)	ASMDELAY((us)*1000, CACHEMISS+RAMCYCLE)
#define ROMUS(us)	ASMDELAY((us)*1000, CACHEMISS+ROMCYCLE)
#define CACHEMS(ms)	((ms) * ASMDELAY(1000000, CACHECYCLE))
#define RAMMS(ms)	((ms) * ASMDELAY(1000000, CACHEMISS+RAMCYCLE))
#define ROMMS(ms)	((ms) * ASMDELAY(1000000, CACHEMISS+ROMCYCLE))

/* fastest possible cycle times */
#define RAMCYCLE	60			/* 60ns single sram cycle */
#define ROMCYCLE	120			/* 120ns single rom cycle */
#define CACHECYCLE	(1000/MHZ) 		/* ??ns cpu pipeline clock */
#define CYCLETIME	CACHECYCLE
#define CACHEMISS	(CYCLETIME * 6)

#ifndef __ASSEMBLER__
/* C level delay functions can compute accurate delay */
extern void _sbd_nsdelay (unsigned long);
#define _sbd_usdelay(us) _sbd_nsdelay((us)*1000)
#define _sbd_msdelay(ms) _sbd_nsdelay((ms)*1000000)
#endif

#endif /* _IDT33X_SBD_H_ */
