/*
 * Copyright (c) 2000-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * idt33x/sbdicu.h: interrupt numbering for IDT 79S33x boards
 */


#ifndef _IDT33X_SBDICU_H_
#define _IDT33X_SBDICU_H_

/* Interrupt numbering: 0xAB: A=group B=sub-interrupt */
#define INTR_GROUP(x)		((x) >> 4)
#define INTR_SUBINT(x)		((x) & 0x0f)

/* MIPS RTPX clock interrupt */
#define INTR_CLOCK		-1
#define INTR_NONE		0

/* CPU interrupts */
#define	INTR_SINT0		0x08	/* s/w int 0 */
#define	INTR_SINT1		0x09	/* s/w int 0 */
#define	INTR_DUART		0x0a	/* external UART */
#define	INTR_PCI_A		0x0b	/* PCI INTA | INTB | INTC | INTD */
#define INTR_PCI_B		0x0c	/* PCI INTB (not on S332) */
#define INTR_RC3233X_MUX 	0x0d	/* expansion interrupt multiplex */
#define INTR_PCI_C		0x0e	/* PCI INTC (not on S332) */
#define INTR_PCI_D		0x0f	/* PCI INTD (not on S332) */
#define INTR_TIMER		0x0f	/* ... or CPU timer */

/* Expansion interrupts */
#define INTR_BUSERR		0x10
/* ----- */
#define INTR_PIOLOW_PIO0	0x20
#define INTR_PIOLOW_PIO1	0x21
#define INTR_PIOLOW_PIO2	0x23	/* note skip */
#define INTR_PIOLOW_PIO3	0x24
#define INTR_PIOLOW_PIO4	0x25
#define INTR_PIOLOW_PIO5	0x26
#define INTR_PIOLOW_PIO6	0x27
#define INTR_PIOLOW_PIO7	0x28
#define INTR_PIOLOW_PIO8	0x29
#define INTR_PIOLOW_PIO9	0x2a
#define INTR_PIOLOW_PIO10	0x2b
/* ----- */
#define INTR_PIOHIGH_PIO0	0x30
#define INTR_PIOHIGH_PIO1	0x31
#define INTR_PIOHIGH_PIO2	0x33	/* note skip */
#define INTR_PIOHIGH_PIO3	0x34
#define INTR_PIOHIGH_PIO4	0x35
#define INTR_PIOHIGH_PIO5	0x36
#define INTR_PIOHIGH_PIO6	0x37
#define INTR_PIOHIGH_PIO7	0x38
#define INTR_PIOHIGH_PIO8	0x39
#define INTR_PIOHIGH_PIO9	0x3a
#define INTR_PIOHIGH_PIO10	0x3b
/* ----- */
#define INTR_TIMER_GP0		0x40
#define INTR_TIMER_GP1		0x41
#define INTR_TIMER_GP2		0x42
#define INTR_TIMER_WDOG		0x43
#define INTR_TIMER_CPUBUS	0x44
#define INTR_TIMER_IPBUS	0x45
#define INTR_TIMER_RFRSH	0x46
#define INTR_TIMER_RESET	0x47
/* ----- */
#define INTR_UART0		0x50
#define INTR_UART0_TXRDY	0x51
#define INTR_UART0_RXRDY	0x52
/* ----- */
#define INTR_UART1		0x60
#define INTR_UART1_TXRDY	0x61
#define INTR_UART1_RXRDY	0x62
/* ----- */
#define INTR_DMA0_DONE		0x70
#define INTR_DMA0_EARLY_END	0x71
#define INTR_DMA0_NOT_OWNED	0x72
#define INTR_DMA0_COMPLETE	0x73
#define INTR_DMA0_CLEAR		0x74
/* ----- */
#define INTR_DMA1_DONE		0x80
#define INTR_DMA1_EARLY_END	0x81
#define INTR_DMA1_NOT_OWNED	0x82
#define INTR_DMA1_COMPLETE	0x83
#define INTR_DMA1_CLEAR		0x84
/* ----- */
#define INTR_DMA2_DONE		0x90
#define INTR_DMA2_EARLY_END	0x91
#define INTR_DMA2_NOT_OWNED	0x92
#define INTR_DMA2_COMPLETE	0x93
#define INTR_DMA2_CLEAR		0x94
/* ----- */
#define INTR_DMA3_DONE		0xa0
#define INTR_DMA3_EARLY_END	0xa1
#define INTR_DMA3_NOT_OWNED	0xa2
#define INTR_DMA3_COMPLETE	0xa3
#define INTR_DMA3_CLEAR		0xa4
/* ----- */
#define INTR_PCIERR_MASTER_WR	0xb0
#define INTR_PCIERR_MASTER_RD	0xb1
#define INTR_PCIERR_MASTER_PAR	0xb2
#define INTR_PCIERR_TARGET_PAR	0xb3
/* ----- */
/*#define INTR_CPU2PCI		0xc0*/
/* ----- */
#define INTR_PCIMBOX0		0xd0
#define INTR_PCIMBOX1		0xd1
#define INTR_PCIMBOX2		0xd2
#define INTR_PCIMBOX3		0xd3
/* ----- */
#define INTR_SPI		0xe0
/* ----- */
#define INTR_MAX		(INTR_SPI + 1)

#endif /* _IDT33X_SBDICU_H_ */
