#ident "kit/idt33x/sbdmem.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2000-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * sbdmem.c: determine memory layout on IDT S332/334 board
 */


#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <mips/cpu.h>
#include "kit_impl.h"

#include <idt/rc3233x.h>
#include "sbd.h"

/*
 * _sbd_memlayout returns a pointer to a table of memory regions.
 *
 * This is only needed if SRAM has been selected to be the lowest
 * addressed memory, in which case we will have two discontiguous
 * regions of memory. We always put the DEAM first in the table 
 * though, because currently sbrk() only allocates memory from
 * the first region, and DRAM will be the largest.
 */

const struct sbd_region *
_sbd_memlayout (void)
{
    static struct sbd_region mem[3];
    struct sbd_region *m = mem;
    extern paddr_t _mem_top;

    if (m->type == SBD_MEM_END) {
	/* DRAM first, if it's enabled */
	if ((RC33X_SDRAM & RC33X_SDRAM_ENB)
	    || (RC33X_EDO & RC33X_SDRAM_ENB)) {
	    m->type = SBD_MEM_RAM;
	    m->base = RC33X_DRAM_ADR_0;
	    m->size = _mem_top - m->base;
	    ++m;
	}

	/* SRAM second, if accessible and disjoint from DRAM */
	m->base = RC33X_MEM_ADR_1;
	if (m->base == 0 || (m->base < KSEG1_SIZE && m->base > _mem_top)) {
	    /* disjoint SRAM */
	    m->type = SBD_MEM_RAM;
	    m->size = (~RC33X_MEM_MSK_1) + 1;
	}
    }

    return mem;
}
