/*
 * Copyright (c) 2000-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * idt33x/sbdrtpx.h: RTPX board-specific definitions for IDT 79S33x
 */


#ifndef _SBDRTPX_H__
#define _SBDRTPX_H__

#include "sbd.h"
#include "sbdicu.h"

/* number of interrupts and interrupt enable "mask" */
#define _SBD_INTR_MAX	INTR_MAX

/* internal interrupt mask type */
typedef struct {
    unsigned int	sr;		/* CPU status register mask */
    unsigned int	grp[16];	/* expansion interrupt group mask */
} _sbd_intrmask_t;

/* the ipl0 (user) interrupt mask type, plus a macro to extract
   the CPU status register mask bits from it */
extern _sbd_intrmask_t	_sbd_ipl0_mask;
#define _sbd_ipl0_sr	_sbd_ipl0_mask.sr

/* ipl1 is the "scheduler mode" value of the status register: only
   debug/panic and other non-RTPX interrupts are enabled.  */
extern unsigned int 	_sbd_ipl1_sr;

/* iplhi is the all interrupts disabled value of the status register. */
extern unsigned int 	_sbd_iplhi_sr;


/* number of CPUs - must be 1 for now */
#define _SBD_CPU_MAX	1

/* override default round-robin every 10ms = 100Hx */
/* #define _RTPX_RRFREQ	100 */

/* override default event log size */
/*#define _RTPX_EVLOGSIZE 256 */

/* enter/leave kernel mode hooks */
#if IDT33X_HINT5_TIMER
#define _sbd_intrenter()
#define _sbd_intrleave()
#else
extern int _sbd_kernelmode;
#define _sbd_intrenter()	_sbd_kernelmode = 1
#define _sbd_intrleave()	_sbd_kernelmode = 0
#endif

extern struct sbdserint ns16550int;
extern struct sbdserint z8530int;

/* list of interrupt driven "tty" drivers */
#if IDT33X_CONSOLE_INTERN
#define _RTPX_SERINT_LIST	\
	&ns16550int
#else
/* XXX warning no interrupt driver for 85C30 uart */
#endif

#endif /* _SBDRTPX_H__ */
