/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * MIPS SDE kit idt/sim interface definitions
 */


#include <stdarg.h>

struct sim_command_tab {
    const char	*cmdt_name;
    const char	*cmdt_abrv_str;
    int		(*cmdt_routine)(int, char **, struct sim_command_tab *);
    int		(*cmdt_init_routine)(void);
    const char	*cmdt_usage;
};


#define SIM_MAXARGS	16
#define SIM_MAXSTRLNGTH	256
struct sim_argv_array {
    char	*argv_ptrs[SIM_MAXARGS];
    char	der_strings[SIM_MAXSTRLNGTH];
};


typedef struct {
    unsigned int	mem_size;
    unsigned int	icache_size;
    unsigned int	dcache_size;
#if !#cpu(r3000)
    unsigned int	scache_size;
#endif
} sim_mem_config;


/* device switch table entry */
struct sim_dev_sw_tab {
    int		(*d_open)();		/* open routine */
    int		(*d_close)();		/* close routine */
    int		(*d_read)();		/* read doutine */
    int		(*d_write)();		/* write routinre */
    int		(*d_init)();		/* initialization routine */
    int		(*d_strategy)();	/* io startegy rouitne */
    int		(*d_ioctl)();		/* io strategy routine */
    const char *d_driver_name;		/* pointer to driver name */
};


/* device init table entry */
struct sim_dev_init_tab {
    const char *dev_name;
    const char *dev_descrip;
    const char *dev_drv_name;
    int		dev_cntl;
    int		dev_unit;
    int		dev_part;
    int		dev_io_addr;
};


/* setjmp/longjmp buffer */
#define SIM_JBSIZE	11
#if __mips64
typedef long long	sim_jmp_buf[SIM_JBSIZE];
#else
typedef int		sim_jmp_buf[SIM_JBSIZE];
#endif

extern int	sim_read (int, void *, int);
extern int	sim_write (int, const void *, int);
extern int	sim_open (const char *, int);
extern int	sim_close (int);
extern int	sim_ioctl (int, int, int);

extern int	sim_printf (const char *, ...) __attribute__ ((format (printf, 1, 2)));
extern int	sim_sprintf (char *, const char *, ...) __attribute__ ((format (printf, 2, 3)));
extern void	sim_putchar (int);
extern int	sim_getchar (void);
extern int	sim_showchar (int);
extern void	sim_puts (const char *);
extern char *	sim_gets (char *);

extern char *	sim_atob (const char *, unsigned, unsigned, int);
extern int	sim_strcmp (const char *, const char *);
extern int	sim_strlen (const char *);
extern char *	sim_strcpy (char *, const char *);
extern char *	sim_strcat (char *, const char *);

extern void	sim_cli (struct	sim_command_tab *, const char *);
extern int	sim_get_range (const char *, unsigned *, int *);
extern int	sim_tokenize (const char *, struct sim_argv_array *);
extern int	sim_help (int, char **, struct sim_command_tab *);

extern int	sim_get_mem_conf (sim_mem_config *);
extern int	sim_set_mem_conf (const	sim_mem_config *);

extern void	sim_reinit (void) __attribute__ ((noreturn));
extern void	sim_restart (void) __attribute__ ((noreturn));
extern void	sim_reset (void) __attribute__ ((noreturn));

extern int	sim_install_immediate_int (void *);
extern int	sim_install_normal_int (void *);

extern int	sim_install_commands (const struct sim_command_tab *);
extern void	sim_install_new_dev (const struct sim_dev_sw_tab *,
				     const struct sim_dev_init_tab *);

extern void	sim_flush_cache (void);
extern void	sim_clear_cache (unsigned int, int);

extern int	sim_setjmp (sim_jmp_buf);
extern void	sim_longjmp (sim_jmp_buf, int);

extern void	sim_timer_start (void);
extern unsigned int sim_timer_stop (void);

#ifdef RFILE
extern int	sim_ropen (const char *, int);
extern int	sim_rclose (int);
extern int	sim_rread (int, void *, int);
extern int	sim_rwrite (int, const void *, int);
extern int	sim_rprintf (const char *, ...) __attribute__ ((format (printf, 1, 2)));
extern char *	sim_rgets (char *);
extern int	sim_rfileinit (void);
extern long	sim_rlseek (int, long, int);
#endif

#ifdef INET
extern int	sim_tftpopen (const char *, int);
extern int	sim_tftpclose (int);
extern int	sim_tftpread (int, void *, int);
extern int	sim_tftpwrite (int, const void *, int);
extern long	sim_tftplseek (int, long, int);
#endif

/* These are the useful ioctls in IDT/sim */
#define	SIM_TIOCRAW	(('t'<<8) | 1)	/* enable/disable raw mode */
#define	SIM_TIOCFLUSH	(('t'<<8) | 2)	/* flush input */
#define	SIM_TIOCREOPEN	(('t'<<8) | 4)	/* reopen */
#define	SIM_TIOCBAUD	(('t'<<8) | 5)	/* set baud rate */

#define	SIM_FIOCNBLOCK	(('f'<<8) | 1)	/* set non-blocking io */
#define	SIM_FIOCSCAN	(('f'<<8) | 2)	/* scan device for pending input */
#define	SIM_FIOCINTBRK	(('f'<<8) | 3)	/* enable break interrupt */
#define	SIM_FIOCINTBRKNOT (('f'<<8) | 4) /* enable break interrupt */
#define	SIM_FIOCCLRINT	(('f'<<8) | 5) /* clear external interrupt */

/* open file flags */
#define SIM_F_READ	0x0001
#define SIM_F_WRITE	0x0002
#define SIM_F_NBLOCK	0x0004
#define SIM_F_SCAN	0x0008
#define SIM_F_STRAT	0x0010
#define SIM_F_REMOTE	0x0020


