#ident "kit/idtsim/simgmon.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * simgmon.c: write profiling data somewhere when program exits.
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "kit_impl.h"

extern void *sbrk (int);
static void *gmonbase;
static int gmonerr;

#if defined(INET) || defined(RFILE)
#define gmoncreat	static int gmemcreat
#define gmonwrite	static ssize_t gmemwrite
#define gmonclose	static int gmemclose
#else
#define gmoncreat	int _gmoncreat
#define gmonwrite	ssize_t _gmonwrite
#define gmonclose	int _gmonclose
#endif

/* 
 * When no remote file access, just collect it in memory and leave it 
 * to the user to do something with it!
 */

gmoncreat (const char *name)
{
    gmonbase = sbrk (0);
    gmonerr = 0;
    return 0;
}

gmonwrite (int fd, const void *data, size_t n)
{
    void *mem;
    
    mem = sbrk (n);
    if (mem == (void*)-1) {
	gmonerr++;
	return -1;
    }
    memcpy (mem, data, n);
    return n;
}


gmonclose (int fd)
{
    void *lim;
    
    if (gmonerr) {
	_mon_printf ("Not enough memory to collect profiling data\n");
    } else {
	lim = sbrk(0);
	_mon_printf ("Profiling data at %p-%p (size 0x%lx)\n",
		     gmonbase, lim, lim - gmonbase);
    }
    return 0;
}


#if defined(INET) || defined(RFILE)

#include <sys/fcntl.h>

char * _gmon_dir;

/* 
 * Try to use remote file access to save it.
 */

int
_gmoncreat (const char *name)
{
    char buf[1024];
    int fd;

    /* 
     * Note: some tftp servers don't allow relative filenames when
     * writing files.  The only way around this is for the programmer
     * to explicitly point _gmon_dir to the directory which should be 
     * used.
     */
#ifdef INET
    strcpy (buf, "/tftp/");
#else
    strcpy (buf, "/rfile/");
#endif
    if (_gmon_dir) {
	strcat (buf, _gmon_dir);
	strcat (buf, "/");
    }
    strcat (buf, name);
    fd = open (buf, O_WRONLY | O_BINARY | O_TRUNC | O_CREAT, 0666);
    if (fd < 0) {
	perror (buf);
	(void) gmemcreat (name);
	fd = 1000;
    }

    return fd;
}

ssize_t
_gmonwrite (int fd, const void *data, size_t n)
{
    if (fd == 1000)
	return gmemwrite (0, data, n);
    else
	return write (fd, data, n);
}


int
_gmonclose (int fd)
{
    if (fd == 1000)
	return gmemclose (0);
    else
	return close (fd);
}
#endif
