#ident "kit/idtsim/simmon.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1998-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * idtsim/simmon.c: low-level output to console, avoiding termio etc.
 */


#include "kit_impl.h"
#include "sim.h"

void
_mon_putc (char c)
{
    char cb = c;
    sim_write (1, &cb, 1);
}


void
_mon_puts (const char *s)
{
    const char *e;

    while (*s) {
	for (e = s; *e && *e != '\n'; e++)
	    continue;
	sim_write (1, s, e - s);
	if (*e == '\n') {
	    sim_write (1, "\r\n", 2);
	    e++;
	}
	s = e;
    }
}

void 
_mon_write (const char *buf, unsigned int nb)
{
    while (nb != 0) {
	const char *e;
	unsigned int n;
	for (e = buf, n = 0; n < nb && *e != '\n'; e++, n++)
	    continue;
	sim_write (1, buf, n);
	if (n < nb && *e == '\n') {
	    sim_write (1, "\r\n", 2);
	    n++;
	}
	buf += n;
	nb -= n;
    }
}


int
_mon_getc (int canblock)
{
    do {
	char cb;
	int n = sim_read (0, &cb, 1);
	if (n > 0)
	    return cb;
	else if (n < 0)
	    break;
    } while (canblock);
    return -1;
}
