/*
 * Copyright (c) 2001-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * gt64120.h: Galileo GT64120A PCI system controller
 */

#define GT_BASE		0x1be00000

#ifdef __ASSEMBLER__

/* offsets from base register */
#define GT64120(x)		(x)

/* device is littleendian, so data may need to be swapped */
#if #endian(big)
#define htoll(x) ((((x) & 0x00ff) << 24) | \
		  (((x) & 0xff00) <<  8) | \
		  (((x) >> 8)  & 0xff00) | \
		  (((x) >> 24) & 0x00ff))
/*#define ltohl(x) htoll(x)*/
#else
#define htoll(x) (x)
/* #define ltohl(x) (x) */
#endif

#else /* !__ASSEMBLER */

/* offsets from base pointer, this construct allows optimisation */
static char * const _gt64120p = PA_TO_KVA1(GT_BASE);

#define GT64120(x)	*(volatile unsigned long *)(_gt64120p + (x))

/* device is littleendian, so data has to be swapped */
#include <sys/endian.h>
#define htoll(x)	htole32(x)
#define ltohl(x)	letoh32(x)

#endif /* __ASSEMBLER__ */

/* CPU configuration */
#define GT_CPU_CFG			GT64120(0x000)

#define GT_CPU_CFG_CacheOpMap		(0xff<<0)
#define GT_CPU_CFG_CachePres		(1<<8)
#define GT_CPU_CFG_WriteMode		(1<<11)
# define  GT_CPU_CFG_WriteMode_R4K	 (1<<11)
# define  GT_CPU_CFG_WriteMode_Pipeline	 (0<<11)
#define GT_CPU_CFG_Endianess		(1<<12)
# define  GT_CPU_CFG_Endianess_EL	 (1<<12)
# define  GT_CPU_CFG_Endianess_EB	 (0<<12)
#define GT_CPU_CFG_R5KL2Pres		(1<<14)
#define GT_CPU_CFG_ExtHitDelay		(1<<15)
#define GT_CPU_CFG_WriteRate		(1<<16)
# define  GT_CPU_CFG_WriteRate_DD	 (1<<16)
# define  GT_CPU_CFG_WriteRate_DxD	 (0<<16)
#define GT_CPU_CFG_StopRetry		(1<<17)
#define GT_CPU_CFG_MultiGT		(1<<18)
#define GT_CPU_CFG_SysADCValid		(1<<19)
#define GT_CPU_CFG_PCI0_Ovride		(3<<20)
# define  GT_CPU_CFG_PCI0_Ovride_Normal	 (0<<20)
# define  GT_CPU_CFG_PCI0_Ovride_1GB	 (1<<20)
# define  GT_CPU_CFG_PCI0_Ovride_2GB	 (2<<20)
#define GT_CPU_CFG_PCI1_Ovride		(3<<24)
# define  GT_CPU_CFG_PCI1_Ovride_Normal	 (0<<24)
# define  GT_CPU_CFG_PCI1_Ovride_1GB	 (1<<24)
# define  GT_CPU_CFG_PCI1_Ovride_2GB	 (2<<24)

/* Multi-controller setup */
#define GT_MULTIGT			GT64120(0x120)
#define GT_MULTIGT_MultiGTAct		(3 << 0)

/* Processor (CPU) Address Space */

/* local chip selects */
#define GT_PAS_SCS10LO			GT64120(0x008)
#define GT_PAS_SCS10HI			GT64120(0x010)
#define GT_PAS_SCS32LO			GT64120(0x018)
#define GT_PAS_SCS32HI			GT64120(0x020)
#define GT_PAS_CS20LO			GT64120(0x028)
#define GT_PAS_CS20HI			GT64120(0x030)
#define GT_PAS_CS3BOOTLO		GT64120(0x038)
#define GT_PAS_CS3BOOTHI		GT64120(0x040)

/* PCI0 addresses */
#define GT_PAS_PCIIOLO			GT64120(0x048)
#define GT_PAS_PCIIOHI			GT64120(0x050)
#define GT_PAS_PCIMEMLO			GT64120(0x058)
#define GT_PAS_PCIMEMHI			GT64120(0x060)
#define GT_PAS_PCIMEM1LO 		GT64120(0x080)
#define GT_PAS_PCIMEM1HI 		GT64120(0x088)

/* PCI1 addresses */
#define GT_PAS_PCI1IOLO			GT64120(0x090)
#define GT_PAS_PCI1IOHI			GT64120(0x098)
#define GT_PAS_PCI1MEMLO		GT64120(0x0a0)
#define GT_PAS_PCI1MEMHI		GT64120(0x0a8)
#define GT_PAS_PCI1MEM1LO 		GT64120(0x0b0)
#define GT_PAS_PCI1MEM1HI		GT64120(0x0b8)

/* PAS register fields */
#define GT_PAS_LOMASK_Low		0x7fff
#define GT_PAS_LOSHIFT_Low		0
#define GT_PAS_HIMASK_High		0x07f
#define GT_PAS_HISHIFT_High		0

/* Internal decode */
#define GT_PAS_INTDEC			GT64120(0x068)

/* Address Remapping */
#define GT_REMAP_SCS10			GT64120(0x0d0)
#define GT_REMAP_SCS32			GT64120(0x0d8)
#define GT_REMAP_CS20			GT64120(0x0e0)
#define GT_REMAP_CS3BOOT		GT64120(0x0e8)
#define GT_REMAP_PCIIO			GT64120(0x0f0)
#define GT_REMAP_PCIMEM0		GT64120(0x0f8)
#define GT_REMAP_PCIMEM1		GT64120(0x100)
#define GT_REMAP_PCI1IO			GT64120(0x108)
#define GT_REMAP_PCI1MEM0		GT64120(0x110)
#define GT_REMAP_PCI1MEM1		GT64120(0x118)

/* Remapping fields */
#define GT_REMAP_MASK			0x3ff
#define GT_REMAP_SHIFT			0

/* CPU Errors */
#define GT_ERR_ADDRLO			GT64120(0x070)
#define GT_ERR_ADDRHI			GT64120(0x078)
#define GT_ERR_DATALO			GT64120(0x128)
#define GT_ERR_DATAHI			GT64120(0x130)
#define GT_ERR_PARITY			GT64120(0x138)

/* CPU Sync barrier */
#define GT_SYNCBARRIER_0		GT64120(0x0c0)
#define GT_SYNCBARRIER_1		GT64120(0x0c8)

/* DRAM and Device Address Space decode */
#define GT_DDAS_RAS0LO			GT64120(0x400)
#define GT_DDAS_RAS0HI			GT64120(0x404)
#define GT_DDAS_RAS1LO			GT64120(0x408)
#define GT_DDAS_RAS1HI			GT64120(0x40c)
#define GT_DDAS_RAS2LO			GT64120(0x410)
#define GT_DDAS_RAS2HI			GT64120(0x414)
#define GT_DDAS_RAS3LO			GT64120(0x418)
#define GT_DDAS_RAS3HI			GT64120(0x41c)
#define GT_DDAS_CS0LO			GT64120(0x420)
#define GT_DDAS_CS0HI			GT64120(0x424)
#define GT_DDAS_CS1LO			GT64120(0x428)
#define GT_DDAS_CS1HI			GT64120(0x42c)
#define GT_DDAS_CS2LO			GT64120(0x430)
#define GT_DDAS_CS2HI			GT64120(0x434)
#define GT_DDAS_CS3LO			GT64120(0x438)
#define GT_DDAS_CS3HI			GT64120(0x43c)
#define GT_DDAS_BOOTCSLO		GT64120(0x440)
#define GT_DDAS_BOOTCSHI		GT64120(0x444)
#define GT_DDAS_ERROR			GT64120(0x470)

#define GT_DDAS_LOMASK_Low		0xff
#define GT_DDAS_LOSHIFT_Low		0
#define GT_DDAS_HIMASK_High		0xff
#define GT_DDAS_HISHIFT_High		0

/* DRAM Configuration */
#define GT_DRAM_CFG			GT64120(0x448)

#define GT_DRAM_CFG_RefIntCntMASK	0x00003fff
# define GT_DRAM_CFG_RefIntCntSHIFT	0
# define GT_DRAM_CFG_RefIntCnt(x)	(((x)<<GT_DRAM_CFG_RefIntCntSHIFT)&\
					 GT_DRAM_CFG_RefIntCntMASK)
#define GT_DRAM_CFG_Interleave		(1<<14)
#define GT_DRAM_CFG_RMW			(1<<15)
#define GT_DRAM_CFG_StagRef		(1<<16)
# define GT_DRAM_CFG_StagRefOn		 (0<<16)
# define GT_DRAM_CFG_StagRefAll		 (1<<16)
#define GT_DRAM_CFG_CPUtoDRAMErr	(1<<17)
#define GT_DRAM_CFG_EccInt		(1<<18)
# define  GT_DRAM_CFG_EccInt_Two	 (0<<18)
# define  GT_DRAM_CFG_EccInt_Any	 (1<<18)
#define GT_DRAM_CFG_DupCntl		(1<<19)
#define GT_DRAM_CFG_DupBA		(1<<20)
#define GT_DRAM_CFG_DupEOT0		(1<<21)
#define GT_DRAM_CFG_DupEOT1		(1<<22)
#define GT_DRAM_CFG_RegSDRAM		(1<<23)
#define GT_DRAM_CFG_Dadr12Sel		(1<<24)

/* SDRAM Operation Mode */
#define GT_SDRAM_OP	 		GT64120(0x474)

#define GT_SDRAM_OP_SDRAMOP		(7<<0)
# define GT_SDRAM_OP_SDRAMOP_Normal	 (0<<0)
# define GT_SDRAM_OP_SDRAMOP_NOP	 (1<<0)
# define GT_SDRAM_OP_SDRAMOP_Precharge   (2<<0)
# define GT_SDRAM_OP_SDRAMOP_ModeRegCmd  (3<<0)
# define GT_SDRAM_OP_SDRAMOP_CBRCycle    (4<<0)

/* SDRAM Burst Mode */
#define GT_SDRAM_BURST	 		GT64120(0x478)

#define GT_SDRAM_BURST_Order		(1<<2)
# define GT_SDRAM_BURST_Order_Linear	 (0<<2)
# define GT_SDRAM_BURST_Order_SubBlock	 (1<<2)

/* SDRAM Address Decode */
#define GT_SDRAM_ADDR	 		GT64120(0x47c)

#define GT_SDRAM_ADDR_AddrDecode	(7<<0)

/* DRAM Parameters */
#define GT_SDRAM_PAR_BANK0 		GT64120(0x44c)
#define GT_SDRAM_PAR_BANK1 		GT64120(0x450)
#define GT_SDRAM_PAR_BANK2 		GT64120(0x454)
#define GT_SDRAM_PAR_BANK3 		GT64120(0x458)

#define GT_SDRAM_PAR_CASLat		(3<<0)
# define GT_SDRAM_PAR_CASLat2		 (1<<0)
# define GT_SDRAM_PAR_CASLat3		 (2<<0)
#define GT_SDRAM_PAR_FlowThrough	(1<<2)
# define GT_SDRAM_PAR_FlowThrough_One	 (0<<2)
# define GT_SDRAM_PAR_FlowThrough_None	 (1<<2)
#define GT_SDRAM_PAR_SRASPrecharge	(1<<3)
# define GT_SDRAM_PAR_SRASPrecharge_2	 (0<<3)
# define GT_SDRAM_PAR_SRASPrecharge_3	 (1<<3)
#define GT_SDRAM_PAR_64bitInt		(1<<5)
#define  GT_SDRAM_PAR_64bitInt_2way	 (0<<5)
#define  GT_SDRAM_PAR_64bitInt_4way	 (1<<5)
#define GT_SDRAM_PAR_BankWidth		(1<<6)
# define GT_SDRAM_PAR_BankWidth_32	 (0<<6)
# define GT_SDRAM_PAR_BankWidth_64	 (1<<6)
#define GT_SDRAM_PAR_BankLoc		(1<<7)
# define GT_SDRAM_PAR_BankLoc_Even	 (0<<7)
# define GT_SDRAM_PAR_BankLoc_Odd	 (1<<7)
#define GT_SDRAM_PAR_ECC		(1<<8)
#define GT_SDRAM_PAR_Bypass		(1<<9)
#define GT_SDRAM_PAR_SRAStoSCAS		(1<<10)
# define GT_SDRAM_PAR_SRAStoSCAS_2	 (0<<10)
# define GT_SDRAM_PAR_SRAStoSCAS_3	 (1<<10)
#define GT_SDRAM_PAR_Size0		(1<<11)
# define GT_SDRAM_PAR_Size0_16Mb	 (0<<11)
# define GT_SDRAM_PAR_Size0_not16Mb	 (1<<11)
#define GT_SDRAM_PAR_BurstLen		(1<<13)
# define GT_SDRAM_PAR_BurstLen_8	 (0<<13)
# define GT_SDRAM_PAR_BurstLen_4	 (1<<13)
#define GT_SDRAM_PAR_Size1		(1<<11)
# define GT_SDRAM_PAR_Size1_not256Mb	 (0<<11)
# define GT_SDRAM_PAR_Size1_256Mb	 (1<<11)

/* bank specific */
#define GT_SDRAM_PAR1_Refresh_Disable	(1<<4)
#define GT_SDRAM_PAR2_TREQ_Enable	(1<<4)

/* ECC */
#define GT_ECC_ERRDATA_HI		GT64120(0x480)
#define GT_ECC_ERRDATA_LO		GT64120(0x484)
#define GT_ECC_MEM			GT64120(0x488)
#define GT_ECC_CALC			GT64120(0x48c)
#define GT_ECC_ERRADDR			GT64120(0x490)

#define GT_ECC_ERRADDR_ERR		(3<<0)
# define GT_ECC_ERRADDR_ERR_None	 (0<<0)
# define GT_ECC_ERRADDR_ERR_One		 (1<<0)
# define GT_ECC_ERRADDR_ERR_Two		 (2<<0)
#define GT_ECC_ERRADDR_ADDR		0fffffffc

/* Device Parameters */
#define GT_DEVPAR_BANK0			GT64120(0x45c)
#define GT_DEVPAR_BANK1			GT64120(0x460)
#define GT_DEVPAR_BANK2			GT64120(0x464)
#define GT_DEVPAR_BANK3			GT64120(0x468)
#define GT_DEVPAR_BOOT			GT64120(0x46c)

#define GT_DEVPAR_TurnOffSHIFT		0
#define GT_DEVPAR_TurnOffMASK		(7<<0)
# define GT_DEVPAR_TurnOff(x)		 ((x)<<0)
#define GT_DEVPAR_AccToFirstSHIFT	3
#define GT_DEVPAR_AccToFirstMASK	(15<<3)
# define GT_DEVPAR_AccToFirst(x)	 ((x)<<3)
#define GT_DEVPAR_AccToNextSHIFT	7
#define GT_DEVPAR_AccToNextMASK		(15<<7)
# define GT_DEVPAR_AccToNext(x)		 ((x)<<7)
#define GT_DEVPAR_ALEtoWrSHIFT		11
#define GT_DEVPAR_ALEtoWrMASK		(7<<11)
# define GT_DEVPAR_ALEtoWr(x)		 ((x)<<11)
#define GT_DEVPAR_WrActiveSHIFT		14
#define GT_DEVPAR_WrActiveMASK		(7<<14)
# define GT_DEVPAR_WrActive(x)		 ((x)<<14)
#define GT_DEVPAR_WrHighSHIFT		17
#define GT_DEVPAR_WrHighMASK		(7<<17)
# define GT_DEVPAR_WrHigh(x)		 ((x)<<17)
#define GT_DEVPAR_DevWidthSHIFT		20	
#define GT_DEVPAR_DevWidthMASK		(3<<20)
# define GT_DEVPAR_DevWidth8		 (0<<20)
# define GT_DEVPAR_DevWidth16		 (1<<20)
# define GT_DEVPAR_DevWidth32		 (2<<20)
# define GT_DEVPAR_DevWidth64		 (3<<20)
#define GT_DEVPAR_DevLoc		(1<<23)
# define GT_DEVPAR_DevLoc_Even		 (0<<23)
# define GT_DEVPAR_DevLoc_Odd		 (1<<23)
#define GT_DEVPAR_DMAFlyBy_SHIFT	26
#define GT_DEVPAR_DMAFlyBy_MASK		(15<<26)
# define GT_DEVPAR_DMAFlyBy(x)		 ((x)<<26)

#define GT_DEVPAR_ReservedMASK		0x3d400000
#define GT_DEVPAR_Reserved		0x14400000

/* Conter Timers */
#define GT_TIMER0			GT64120(0x850)
#define GT_TIMER0_MASK			0xffffffff
#define GT_TIMER1			GT64120(0x854)
#define GT_TIMER1_MASK			0xffffff
#define GT_TIMER2			GT64120(0x858)
#define GT_TIMER2_MASK			0xffffff
#define GT_TIMER3			GT64120(0x85c)
#define GT_TIMER3_MASK			0xffffff

#define GT_TIMER_CTRL			GT64120(0x864)
#define GT_TIMER_CTRL_EnTC0		(1<<0)
#define GT_TIMER_CTRL_SelTC0		(1<<1)
# define GT_TIMER_CTRL_SelTC0_Counter	 (0<<1)
# define GT_TIMER_CTRL_SelTC0_Timer	 (1<<1)
#define GT_TIMER_CTRL_EnTC1		(1<<2)
#define GT_TIMER_CTRL_SelTC1		(1<<3)
# define GT_TIMER_CTRL_SelTC1_Counter	 (0<<3)
# define GT_TIMER_CTRL_SelTC1_Timer	 (1<<3)
#define GT_TIMER_CTRL_EnTC2		(1<<4)
#define GT_TIMER_CTRL_SelTC2		(1<<5)
# define GT_TIMER_CTRL_SelTC2_Counter	 (0<<5)
# define GT_TIMER_CTRL_SelTC2_Timer	 (1<<5)
#define GT_TIMER_CTRL_EnTC3		(1<<6)
#define GT_TIMER_CTRL_SelTC3		(1<<7)
# define GT_TIMER_CTRL_SelTC3_Counter	 (0<<7)
# define GT_TIMER_CTRL_SelTC3_Timer	 (1<<7)

/* PCI Internal */
#define GT_IPCI_0_CMD			GT64120(0xc00)
#define GT_IPCI_1_CMD			GT64120(0xc80)

#define GT_IPCI_CMD_ByteSwap		(1<<0)
# define GT_IPCI_CMD_ByteSwapOn		 (0<<0)
# define GT_IPCI_CMD_ByteSwapOff	 (1<<0)
#define GT_IPCI_CMD_SyncModeSHIFT	1
#define GT_IPCI_CMD_SyncModeMASK	(3<<1)
# define GT_IPCI_CMD_SyncModeStd	 (0<<1)
# define GT_IPCI_CMD_SyncMode1		 (1<<1)
# define GT_IPCI_CMD_SyncMode2		 (2<<1)
# define GT_IPCI_CMD_SyncMode5		 (5<<1)
# define GT_IPCI_CMD_SyncMode6		 (6<<1)
#define GT_IPCI_CMD_MWordSwap		(1<<10)
#define GT_IPCI_CMD_SWordSwap		(1<<11)
#define GT_IPCI_CMD_SSBWordSwap		(1<<12)
#define GT_IPCI_CMD_SByteSwap		(1<<16)
#define GT_IPCI_CMD_DRAMtoPCIErr	(1<<26)

/* Timeout and Retry */
#define GT_IPCI_0_TOR			GT64120(0xc04)
#define GT_IPCI_1_TOR			GT64120(0xc84)

#define GT_IPCI_TOR_Timeout0MASK	(255<<0)
#define GT_IPCI_TOR_Timeout0SHIFT	0
#define GT_IPCI_TOR_Timeout0(x)	((x)<<0)
#define GT_IPCI_TOR_Timeout1MASK	(255<<8)
#define GT_IPCI_TOR_Timeout1SHIFT	8
#define GT_IPCI_TOR_Timeout1(x)	((x)<<8)
#define GT_IPCI_TOR_RetryCtrMASK	(255<<16)
#define GT_IPCI_TOR_RetryCtrSHIFT	16
#define GT_IPCI_TOR_RetryCtr(x)		((x)<<16)

#define GT_IPCI_0_SCS10SIZE		GT64120(0xc08)
#define GT_IPCI_0_SCS32SIZE		GT64120(0xc0c)
#define GT_IPCI_0_CS20SIZE		GT64120(0xc10)
#define GT_IPCI_0_CS3BOOTSIZE		GT64120(0xc14)

#define GT_IPCI_1_SCS10SIZE		GT64120(0xc88)
#define GT_IPCI_1_SCS32SIZE		GT64120(0xc8c)
#define GT_IPCI_1_CS20SIZE		GT64120(0xc90)
#define GT_IPCI_1_CS3BOOTSIZE		GT64120(0xc94)

#define GT_IPCI_SIZE_BankSizeMASK	(0xfffff<<12)
#define GT_IPCI_SIZE_BankSizeSHIFT	12

#define GT_IPCI_0_BAREN			GT64120(0xc3c)
#define GT_IPCI_1_BAREN			GT64120(0xcbc)

#define GT_IPCI_BAREN_SwCS3BootDis	(1<<0)
#define GT_IPCI_BAREN_SwCS32Dis		(1<<1)
#define GT_IPCI_BAREN_SwCS10Dis		(1<<2)
#define GT_IPCI_BAREN_IntIODis		(1<<3)
#define GT_IPCI_BAREN_IntMemDis		(1<<4)
#define GT_IPCI_BAREN_CS3BootDis	(1<<5)
#define GT_IPCI_BAREN_CS20Dis		(1<<6)
#define GT_IPCI_BAREN_SCS32Dis		(1<<7)
#define GT_IPCI_BAREN_SCS10Dis		(1<<8)

#define GT_IPCI_0_PREFETCH		GT64120(0xc40)
#define GT_IPCI_1_PREFETCH		GT64120(0xcc0)

#define GT_IPCI_PREFETCH_Dram0Max	(1<<0)
# define GT_IPCI_PREFETCH_Dram0Max_4	 (0<<0)
# define GT_IPCI_PREFETCH_Dram0Max_8	 (1<<0)
#define GT_IPCI_PREFETCH_Dram1Max	(1<<1)
# define GT_IPCI_PREFETCH_Dram1Max_4	 (0<<1)
# define GT_IPCI_PREFETCH_Dram1Max_8	 (1<<1)
#define GT_IPCI_PREFETCH_Dev0Max	(1<<2)
# define GT_IPCI_PREFETCH_Dev0Max_4	 (0<<2)
# define GT_IPCI_PREFETCH_Dev0Max_8	 (1<<2)
#define GT_IPCI_PREFETCH_Dev1Max	(1<<3)
# define GT_IPCI_PREFETCH_Dev1Max_4	 (0<<3)
# define GT_IPCI_PREFETCH_Dev1Max_8	 (1<<3)
#define GT_IPCI_PREFETCH_RdMemPref	(1<<4)
# define GT_IPCI_PREFETCH_RdMemPref_Reg  (0<<4)
# define GT_IPCI_PREFETCH_RdMemPref_Aggr (1<<4)
#define GT_IPCI_PREFETCH_RdLnPref	(1<<5)
# define GT_IPCI_PREFETCH_RdLnPref_Reg   (0<<5)
# define GT_IPCI_PREFETCH_RdLnPref_Aggr  (1<<5)
#define GT_IPCI_PREFETCH_RdMulPref	(1<<6)
# define GT_IPCI_PREFETCH_RdMulPref_Reg  (0<<6)
# define GT_IPCI_PREFETCH_RdMulPref_Aggr (1<<6)

#define GT_IPCI_0_REMAP_SCS10		GT64120(0xc48)
#define GT_IPCI_0_REMAP_SCS32		GT64120(0xc4c)
#define GT_IPCI_0_REMAP_SWAPSCS10	GT64120(0xc58)
#define GT_IPCI_0_REMAP_SWAPSCS32	GT64120(0xc5c)
#define GT_IPCI_0_REMAP_CS20		GT64120(0xc50)
#define GT_IPCI_0_REMAP_CS3BOOT		GT64120(0xc54)
#define GT_IPCI_0_REMAP_SWAPCS3BOOT	GT64120(0xc64)

#define GT_IPCI_1_REMAP_SCS10		GT64120(0xcc8)
#define GT_IPCI_1_REMAP_SCS32		GT64120(0xccc)
#define GT_IPCI_1_REMAP_SWAPSCS10	GT64120(0xcd8)
#define GT_IPCI_1_REMAP_SWAPSCS32	GT64120(0xcdc)
#define GT_IPCI_1_REMAP_CS20		GT64120(0xcd0)
#define GT_IPCI_1_REMAP_CS3BOOT		GT64120(0xcd4)
#define GT_IPCI_1_REMAP_SWAPCS3BOOT	GT64120(0xce4)

#define GT_IPCI_REMAP_MASK		0xfffff000
#define GT_IPCI_REMAP_SHIFT		12

#define GT_IPCI_0_CFGADDR		GT64120(0xcf8)
#define GT_IPCI_0_CFGDATA		GT64120(0xcfc)
#define GT_IPCI_1_CFGADDR		GT64120(0xcf0)
#define GT_IPCI_1_CFGDATA		GT64120(0xcf4)

#define GT_IPCI_CFGADDR_RegNumMASK	(0x3f<<2)
#define GT_IPCI_CFGADDR_RegNumSHIFT	2
#define GT_IPCI_CFGADDR_RegNum(x)	((x)<<2)
#define GT_IPCI_CFGADDR_FunctNumMASK	(0x7<<8)
#define GT_IPCI_CFGADDR_FunctNumSHIFT	8
#define GT_IPCI_CFGADDR_FunctNum(x)	((x)<<8)
#define GT_IPCI_CFGADDR_DevNumMASK	(0x1f<<11)
#define GT_IPCI_CFGADDR_DevNumSHIFT	11
#define GT_IPCI_CFGADDR_DevNum(x)	((x)<<11)
#define GT_IPCI_CFGADDR_BusNumMASK	(0xff<<16)
#define GT_IPCI_CFGADDR_BusNumSHIFT	16
#define GT_IPCI_CFGADDR_BusNum(x)	((x)<<16)
#define GT_IPCI_CFGADDR_ConfigEn	(1<<31)

#define GT_IPCI_0_INTACK		GT64120(0xc34)
#define GT_IPCI_1_INTACK		GT64120(0xc30)

#define GT_INTRCAUSE			GT64120(0xc18)
#define GT_INTRCAUSEHI			GT64120(0xc98)
#define GT_INTRMASK			GT64120(0xc1c)
#define GT_INTRMASKHI			GT64120(0xc9c)

#define  GT_INTR_INTSUM			0x00000001
#define  GT_INTR_MEMOUT			0x00000002
#define  GT_INTR_DMAOUT			0x00000004
#define  GT_INTR_CPUOUT			0x00000008
#define  GT_INTR_DMA0COMP		0x00000010
#define  GT_INTR_DMA1COMP		0x00000020
#define  GT_INTR_DMA2COMP		0x00000040
#define  GT_INTR_DMA3COMP		0x00000080
#define  GT_INTR_T0EXP			0x00000100
#define  GT_INTR_T1EXP			0x00000200
#define  GT_INTR_T2EXP			0x00000400
#define  GT_INTR_T3EXP			0x00000800
#define  GT_INTR_MASRDERR		0x00001000
#define  GT_INTR_SLVWRERR		0x00002000
#define  GT_INTR_MASWRERR		0x00004000
#define  GT_INTR_SLVRDERR		0x00008000
#define  GT_INTR_ADDRERR		0x00010000
#define  GT_INTR_MEMERR			0x00020000
#define  GT_INTR_MASABORT		0x00040000
#define  GT_INTR_TARABORT		0x00080000
#define  GT_INTR_RETRYCTR0		0x00100000
#define  GT_INTR_PMCINT0		0x00200000
#define  GT_INTR_CPU2PCIA		0x00400000
#define  GT_INTR_CPU2PCIB		0x00800000
#define  GT_INTR_CPU2PCIC		0x01000000
#define  GT_INTR_CPU2PCID		0x02000000
#define  GT_INTR_PCI2CPUA		0x04000000
#define  GT_INTR_PCI2CPUB		0x08000000
#define  GT_INTR_PCI2CPUC		0x10000000
#define  GT_INTR_PCI2CPUD		0x20000000
#define  GT_INTR_CPUINTSUM		0x40000000
#define  GT_INTR_PCIINTSUM		0x80000000


#define GT_CPU_INTRSELECT		GT64120(0xc70)
#define GT_PCI_0_INTRSELECT		GT64120(0xc74)

#define GT_INTRSELECT_Aliased_Mask	0x3ffffffe
#define GT_INTRSELECT_SelectCause	0x40000000
# define GT_INTRSELECT_SelectCause_Lo	 0x00000000
# define GT_INTRSELECT_SelectCause_Hi	 0x40000000
#define GT_INTRSELECT_LowAndHigh	0x80000000

#define GT_PCI_0_INTMASK		GT64120(0xc24)
#define GT_PCI_0_INTMASKHI		GT64120(0xca4)

#define GT_IPCI_SERR_0_MASK		GT64120(0xc28)
#define GT_IPCI_SERR_1_MASK		GT64120(0xca8)

#define GT_IPCI_SERRMASK_AddrErr	(1<<0)
#define GT_IPCI_SERRMASK_MasWrErr	(1<<1)
#define GT_IPCI_SERRMASK_MasRdErr	(1<<2)
#define GT_IPCI_SERRMASK_MemErr		(1<<3)
#define GT_IPCI_SERRMASK_MasAbort	(1<<4)
#define GT_IPCI_SERRMASK_TarAbort	(1<<5)

