/*
 * Copyright (c) 2002-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * i8259.h: definitions for i8259 interrupt controller
 */


#ifndef __ASSEMBLER__
struct i8259reg {
#if #endian(little)
    unsigned char	i8259_cmd;
    unsigned char	i8259_dat;
#else
    unsigned char       dummy1;
    unsigned char       dummy2;
    unsigned char       i8259_dat;
    unsigned char       i8259_cmd;
#endif
};

#define i8259_rr	i8259_cmd
#define i8259_ocw1	i8259_dat

#endif

/* Initialisation Control Word 1 */
#define I8259_A765	0xe0		/* bits 7,6,5 of vector */
#define I8259_A76	0xc0		/* bits 7,6  of vector */
#define I8259_ICW1	0x10		/* identifies this as ICW1 */
#define I8259_LTIM	0x08		/* level triggered mode */
#define I8259_ADI8	0x04		/* call address interval of 8 */
#define I8259_ADI4	0x00		/* call address interval of 4 */
#define I8259_SNGL	0x02		/* single mode */
#define I8259_NEEDICW4	0x01		/* need icw4 */

/* Initialisation Control Word 4 */
#define I8259_SFNM	0x10		/* special fully nested mode */
#define I8259_BUFFERED	0x08
#define I8259_MASTER	0x04		/* buffered mode: master */
#define I8259_NOBUF	0x00		/* non-buffered mode */
#define I8259_AEOI	0x02		/* auto eoi */
#define I8259_8086	0x01		/* i8086/i8088 mode */

/* Operation Control Word 2 */
#define I8259_ROTATE	0x80		/* rotate priorities */
#define I8259_SPECIFIC	0x40		/* specific variant */
#define I8259_EOI	0x20		/* end-of-interrupt */
#define I8259_OCW2	0x00		/* identify this as ocw2 */
#define I8259_LEVEL	0x07		/* specific eoi level */

/* Operation Control Word 3 */
#define I8259_ESMM	0x40		/* enable special mask mode */
#define	I8259_SMM	0x20		/* set/clear special mask */
#define I8259_OCW3	0x08		/* identify this as ocw3 */
#define I8259_POLL	0x04		/* poll command */
#define I8259_RR	0x02		/* read register command */
#define I8259_RIS	0x01		/* select isr */
#define I8259_RIR	0x00		/* select irr */

/* Poll command bit fields */
#define I8259_INTREQ	0x80		/* int is pending */
#define I8259_INTVEC	0x07		/* int vector */
