/*
 * Copyright (c) 2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * malta/msc01_memmap.h: Memory map defines for SOC-it
 */

#ifndef MSC01_MEMMAP_H
#define MSC01_MEMMAP_H

#ifndef KSEG1_BASE
# define KSEG0_BASE      	0x80000000	/* unmapped, cachable	*/
# define KSEG1_BASE      	0xa0000000	/* unmapped, uncached	*/
#endif

/*****************************************************************************
 * IPIF generic register space addresses, unmapped & uncached access via kseg1
 ****************************************************************************/

#define	MSC01_REGSPACE_MC	0
#define	MSC01_REGSPACE_IC	1
#define	MSC01_REGSPACE_BIU	2
#define	MSC01_REGSPACE_IP1	3
#define	MSC01_REGSPACE_IP2	4
#define	MSC01_REGSPACE_IP3	5

#define	MSC01_REGADDR_RESET	(KSEG1_BASE + 0x1f800000) /* power-up value */
#define	MSC01_REGADDR_SIZE	0x00040000		  /* 256 KByte	    */
#define	MSC01_REGADDR(regspace)	(MSC01_REGADDR_BASE	\
				 +(regspace)*MSC01_REGADDR_SIZE)



/*****************************************************************************
 * Default register base addresses (can be modified at run-time by software)
 ****************************************************************************/

#define	MSC01_REGADDR_BASE	(KSEG1_BASE + 0x1bc00000) /* programmed val */

#define MSC01_MC_REG_BASE	MSC01_REGADDR(MSC01_REGSPACE_MC)
#define MSC01_IC_REG_BASE	MSC01_REGADDR(MSC01_REGSPACE_IC)
#define MSC01_BIU_REG_BASE	MSC01_REGADDR(MSC01_REGSPACE_BIU)
#define MSC01_SC_REG_BASE	MSC01_REGADDR(MSC01_REGSPACE_BIU)
#define MSC01_IP1_REG_BASE	MSC01_REGADDR(MSC01_REGSPACE_IP1)
#define MSC01_IP2_REG_BASE	MSC01_REGADDR(MSC01_REGSPACE_IP2)
#define MSC01_IP3_REG_BASE	MSC01_REGADDR(MSC01_REGSPACE_IP3)


/*****************************************************************************
 * Power-up memory space (can be modified at run-time by software)
 ****************************************************************************/

#define	MSC01_PHYSBASE_MC	0x00000000
#define	MSC01_PHYSBASE_IP1	0x10000000
#define	MSC01_PHYSBASE_IP2	0x14000000
#define	MSC01_PHYSBASE_IP3	0x18000000

#define	MSC01_KSEG0BASE_MC	(KSEG0_BASE + MSC01_PHYSBASE_MC)
#define	MSC01_KSEG0BASE_IP1	(KSEG0_BASE + MSC01_PHYSBASE_IP1)
#define	MSC01_KSEG0BASE_IP2	(KSEG0_BASE + MSC01_PHYSBASE_IP2)
#define	MSC01_KSEG0BASE_IP3	(KSEG0_BASE + MSC01_PHYSBASE_IP3)

#endif

