/*
 * Copyright (c) 2003-2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * product.h: MIPS Technologies Malta product revision information header file
 */

#ifndef PRODUCT_H
#define PRODUCT_H


#define MIPS_REVISION              0x1FC00010 /* REVISION              */

#define MSK(X) ((1<<X)-1)

/* field: PROID */
#define MIPS_REVISION_PROID_SHF	   4
#define MIPS_REVISION_PROID_MSK	   (MSK(4) << MIPS_REVISION_PROID_SHF)

/* field: PRORV */
#define MIPS_REVISION_PRORV_SHF	   0
#define MIPS_REVISION_PRORV_MSK	   (MSK(4) << MIPS_REVISION_PRORV_SHF)

/* field: MANID (in case of 3rd party product) */
#define MIPS_REVISION_MANID_SHF	   8
#define MIPS_REVISION_MANID_MSK    (MSK(8) << MIPS_REVISION_MANID_SHF)

/* field: MANPD (in case of 3rd party product) */
#define MIPS_REVISION_MANPD_SHF	   16
#define MIPS_REVISION_MANPD_MSK    (MSK(8) << MIPS_REVISION_MANPD_SHF)

/* field: CORID (Atlas/Malta) */
#define MIPS_REVISION_CORID_SHF    10
#define MIPS_REVISION_CORID_MSK	   (MSK(6) << MIPS_REVISION_CORID_SHF)

/* field: CORRV (Atlas/Malta) */
#define MIPS_REVISION_CORRV_SHF    8
#define MIPS_REVISION_CORRV_MSK    (MSK(2) << MIPS_REVISION_CORRV_SHF)

/* field: SCON (Atlas/Malta) */
#define MIPS_REVISION_SCON_SHF    24
#define MIPS_REVISION_SCON_MSK    (MSK(8) << MIPS_REVISION_SCON_SHF)

/**** Encoding of corid field ****/

#define MIPS_REVISION_CORID_QED_RM5261     0
#define MIPS_REVISION_CORID_CORE_LV        1
#define MIPS_REVISION_CORID_BONITO64	   2
#define MIPS_REVISION_CORID_CORE_20K	   3
#define MIPS_REVISION_CORID_CORE_FPGA      4
#define MIPS_REVISION_CORID_CORE_SYS       5
#define MIPS_REVISION_CORID_CORE_EMUL      6
#define MIPS_REVISION_CORID_CORE_FPGA2     7
#define MIPS_REVISION_CORID_CORE_FPGAr2    8
#define MIPS_REVISION_CORID_CORE_FPGA3     9


/**** Artificial corid defines ****/
/*
 *  CoreEMUL with   Bonito   System Controller is treated like a Core20K
 *  CoreEMUL with SOC-it 101 System Controller is treated like a CoreSYS
 */
#define MIPS_REVISION_CORID_CORE_EMUL_20K  0x63
#define MIPS_REVISION_CORID_CORE_EMUL_SYS  0x65
/*
 *  To ease code sharing with CoreSYS, this value is stored to
 *  variable sys_corecard when MIPS SOC-it 101 is used on SEAD:
 */
#define MIPS_REVISION_CORID_SEAD_MSC01 0x305

/* Following value used if no core card is available */
#define MIPS_REVISION_CORID_NA	   0xffffffff

/* Encoding of proid field */
#define PRODUCT_ATLASA_ID		 0x0		/* Atlas board   */
#define PRODUCT_SEAD_ID			 0x1		/* SEAD board    */
#define PRODUCT_MALTA_ID		 0x2		/* Malta board   */
#define PRODUCT_SEAD2_ID		 0x3		/* SEAD-2	 */
#define PRODUCT_THIRD_PARTY_ID		 0xe		/* Third party	 */
#define PRODUCT_UNKNOWN			 0xffffffff

/**** Encoding of SCON field ****/

#define MIPS_REVISION_SCON_OTHER	0x0
#define MIPS_REVISION_SCON_SOCITSC	0x1
#define MIPS_REVISION_SCON_SOCITSCP	0x2
#define MIPS_REVISION_SCON_UNKNOWN	0xffffffff

#endif /* #ifndef PRODUCT_H */
