/*
 * Copyright (c) 2001-2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * sbd.h: MIPS Technologies Malta board definition header file
 */

#ifndef __MALTA_SBD_H
#define __MALTA_SBD_H

#ifndef MHZ
/* fastest possible pipeline clock */
#define MHZ		400
#endif

#define RAMCYCLE	60			/* 60ns dram cycle */
#define ROMCYCLE	150			/* ~150ns rom cycle */
#define CACHECYCLE	(1000/MHZ) 		/* pipeline clock */
#define CYCLETIME	CACHECYCLE
#define CACHEMISS	(CYCLETIME * 6)

/*
 * Address Map
 */

#define DRAM_BASE	0x00000000	/* 128MB */
#define DRAM_SIZE	 0x8000000

#ifndef __ASSEMBLER__
extern unsigned long _sbd_pci_io_space;
extern unsigned long _sbd_pci_io_size;
extern unsigned long _sbd_pci_mem_space;
extern unsigned long _sbd_pci_mem_size;
#endif

#define PCI_IO_SPACE  _sbd_pci_io_space
#define PCI_MEM_SPACE _sbd_pci_mem_space
#define PCI_IO_SIZE   _sbd_pci_io_size
#define PCI_MEM_SIZE  _sbd_pci_mem_size

#define UART_CAN_MOVE

#define FLASH_BASE	0x1e000000
#define FLASH_SIZE	  0x400000
#define FPGA_BASE	0x1f000000
#define BOOTPROM_BASE	0x1fc00000
#define BOOTPROM_SIZE	  0x400000
#define REVISION_ADDR	0x1fc00010

/* ISA i/o and memory address mapping */
#define ISAPORT_BASE(x)	(PCI_IO_SPACE + (x))
#define ISAMEM_BASE(x)	(PCI_MEM_SPACE + (x))

/* Well-known ISA i/o ports */
#define DMA1_PORT	0x000
#define ICU1_PORT	0x020
#define CTC_PORT		0x040
#define DIAG_PORT	0x061
#define RTC_ADDR_PORT	0x070 
#define RTC_DATA_PORT	0x071 
#define KEYBD_PORT	0x060
#define DMAPAGE_PORT	0x080
#define SYSC_PORT	0x092
#define ICU2_PORT	0x0a0
#define DMA2_PORT	0x0c0
#define IDE_PORT		0x1f0
#define UART1_PORT	0x2f8
#define UART0_PORT	0x3f8
#define ECP_PORT		0x378
#define FDC_PORT		0x3f0
#define ELCR1_PORT	0x4d0	/* PIIX4 icu edge/level control */
#define ELCR2_PORT	0x4d1
#define SMB_PORT		0x7000  /* Intel convention? */
#define GPIO_PORT	0xe000	/* FIXME: where should this go? */
#define BMDMA_PORT	0xf000	/* Intel convention? */

/* interrupt controller */
#define NMISTATUS_ADDR	(FPGA_BASE + 0x00000024)	/* nmp status */
#define NMIACK_ADDR	(FPGA_BASE + 0x00000104)	/* nmi ack */

#define NMI_SB		0x00000002			/* from south bridge */
#define NMI_ONNMI	0x00000001			/* from on switch */

/* switch readback register */
#define SWITCH_ADDR	(FPGA_BASE + 0x00000200)

#define SWITCH_S2	0x0000000f	/* switch S2-7:0 */

/* board status register (r/o) */
#define STATUS_ADDR	(FPGA_BASE + 0x00000208)

#define STATUS_MFWR	0x00000010	/* monitor flash write enabled */
#define STATUS_S54	0x00000008	/* switch S5-4: reset environ */
#define STATUS_S53	0x00000004	/* switch S5-3 */
#define STATUS_BIGEND	0x00000002	/* S5-2: big endian */
#define STATUS_RESV	0x00000001	/* reserved */

/* jumpbers readback register */
#define JMPRS_ADDR	(FPGA_BASE + 0x00000210)

#define JMPRS_PCICLK	0x0000001c	/* PCI frequency */
#define JMPRS_EELOCK	0x00000002	/* EEPROM write protected */
#define JMPRS_RESV	0x00000001

/* green led */
#define LEDGREEN_ADDR	(FPGA_BASE + 0x00000400)

/* 8-bar led array */
#define LEDBAR_ADDR	(FPGA_BASE + 0x00000408)

/* 8-character ASCII led array */
#define ASCIIWORD_ADDR	(FPGA_BASE + 0x00000410)
#define ASCII_BASE	(FPGA_BASE + 0x00000418)
#define  ASCII_LED(n)	((n)*8)

/* software reset control */
#define SOFTRES_ADDR	(FPGA_BASE + 0x00000500)
#define SOFTRES_DATA	0x42

/* break signal reset timer */
#define BRKRES_ADDR	(FPGA_BASE + 0x00000508)

/* local gpio */
#define GPOUT_ADDR	(FPGA_BASE + 0x00000a00)
#define GPIN_ADDR	(FPGA_BASE + 0x00000a08)

/* i2c bus */
#define I2CINP_ADDR	(FPGA_BASE + 0x00000b00)
#define I2COE_ADDR	(FPGA_BASE + 0x00000b08)
#define I2COUT_ADDR	(FPGA_BASE + 0x00000b10)
# define I2CSCL		 0x2
# define I2CSDA		 0x1
#define I2CSEL_ADDR	(FPGA_BASE + 0x00000b18)
# define I2CFPGA	 	 0x1

/* 
 * UART definitions
 */
#define NS16550_HZ	(24000000/13)

#if #endian(big)
#define NSREG(x)        ((x) ^ 3)
#else
#define NSREG(x)	(x)
#endif

#if 0
/* register mapping is different from UART 0 & 1 (8 bytes) */
#define UART2_BASE	0x1f000900
#endif

/* board attributes */
#define DEFSPEED	38400
/*#define _SBD_PANEL*/

/*
 * rough scaling factors for 2 instruction DELAY loop to get 1ms and 1us delays
 */
#define ASMDELAY(ns,icycle)	\
	(((ns) + (icycle)) / ((icycle) * 2))

#define CACHENS(ns)	ASMDELAY((ns), CACHECYCLE)
#define RAMNS(ns)	ASMDELAY((ns), CACHEMISS+RAMCYCLE)
#define ROMNS(ns)	ASMDELAY((ns), CACHEMISS+ROMCYCLE)
#define CACHEUS(us)	ASMDELAY((us)*1000, CACHECYCLE)
#define RAMUS(us)	ASMDELAY((us)*1000, CACHEMISS+RAMCYCLE)
#define ROMUS(us)	ASMDELAY((us)*1000, CACHEMISS+ROMCYCLE)
#define CACHEMS(ms)	((ms) * ASMDELAY(1000000, CACHECYCLE))
#define RAMMS(ms)	((ms) * ASMDELAY(1000000, CACHEMISS+RAMCYCLE))
#define ROMMS(ms)	((ms) * ASMDELAY(1000000, CACHEMISS+ROMCYCLE))

#ifndef __ASSEMBLER__
extern void _sbd_nsdelay (unsigned long);
#define nsdelay(ns)	_sbd_nsdelay (ns)
#define usdelay(us)	_sbd_nsdelay ((us) * 1000)

extern int _sbd_sysctrl_type;
extern int _sbd_discover_sysctrl (void);
#endif

#define SYSCTRL_IS_UNKNOWN -1
#define SYSCTRL_IS_GALILEO 0
#define SYSCTRL_IS_BONITO  1
#define SYSCTRL_IS_SOCIT   2
#define SYSCTRL_IS_SOCITSC 3

#define PCI_MEM_SPACE_GAL	0x08000000	/* 256MB: s/w configurable */
#define PCI_MEM_SIZE_GAL	0x10000000
#define PCI_IO_SPACE_GAL	0x18000000	/*  62MB: s/w configurable */
#define PCI_IO_SIZE_GAL		0x03e00000

#define PCI_MEM_SPACE_BON	0x10000000
#define PCI_MEM_SIZE_BON	(192*1024*1024)
#define PCI_IO_SPACE_BON	0x1fd00000
#define PCI_IO_SIZE_BON		(1024*1024)

#define PCI_MEM_SPACE_SOCIT     0x10000000
#define PCI_MEM_SIZE_SOCIT      0x08000000
#define PCI_IO_SPACE_SOCIT      0x1b000000
#define PCI_IO_SIZE_SOCIT       0x00800000

#define PCI_MEM_SPACE_SOCITSC   0x10000000
#define PCI_MEM_SIZE_SOCITSC    0x07000000
#define PCI_IO_SPACE_SOCITSC    0x17000000
#define PCI_IO_SIZE_SOCITSC     0x00800000

#define UART0_BASE	ISAPORT_BASE(UART0_PORT)
#define UART1_BASE	ISAPORT_BASE(UART1_PORT)

#endif
