/*
 * Copyright (c) 2002-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * malta/sbdicu.h: interrupt numbering for MIPS Malta board
 */


/* MIPS RTPX specials */
#define INTR_CLOCK	-1
#define INTR_NONE	0

#define INTR_IS_CPU(n)		(((n) & 0x38) == 0x08)
#define INTR_IS_ISA(n)		(((n) & 0x30) == 0x10)
#define INTR_IS_ISA_MASTER(n)	(((n) & 0x38) == 0x10)
#define INTR_IS_ISA_SLAVE(n)	(((n) & 0x38) == 0x18)
#define INTR_IS_SYSCTRL(n)	((n) & 0x20)

/* CPU interrupts */
#define INTR_SINT0	8	/* SINT0 */
#define INTR_SINT1	9	/* SINT1 */
#define INTR_ISAMASTER	10	/* HINT0 */
#define INTR_SMI	11	/* HINT1 */
#define INTR_TTY2	12	/* HINT2 */
#define INTR_COREHI	13	/* HINT3 */
#define INTR_CORELO	14	/* HINT4 */
#define INTR_TIMER	15	/* HINT5 */

/* i8259 master interrupts */
#define INTR_ISAIRQ0	16
#define INTR_ISAIRQ1	17
#define INTR_ISAIRQ2	18
#define INTR_ISAIRQ3	19
#define INTR_ISAIRQ4	20
#define INTR_ISAIRQ5	21
#define INTR_ISAIRQ6	22
#define INTR_ISAIRQ7	23

/* i8259 slave interrupts */
#define INTR_ISAIRQ8	24
#define INTR_ISAIRQ9	25
#define INTR_ISAIRQ10	26
#define INTR_ISAIRQ11	27
#define INTR_ISAIRQ12	28
#define INTR_ISAIRQ13	29
#define INTR_ISAIRQ14	30
#define INTR_ISAIRQ15	31

/* Conventional ISA IRQ usage */
#define INTR_PTIMER	INTR_ISAIRQ0
#define INTR_KEYBD	INTR_ISAIRQ1
#define INTR_ISASLAVE	INTR_ISAIRQ2
#define INTR_COM2	INTR_ISAIRQ3
#define INTR_COM1	INTR_ISAIRQ4
#define INTR_UART1	INTR_ISAIRQ3	/* alias com2 */
#define INTR_UART0	INTR_ISAIRQ4	/* alias com1 */
#define INTR_LPT1	INTR_ISAIRQ5	/* unused? */
#define INTR_FDC	INTR_ISAIRQ6
#define INTR_LPT0	INTR_ISAIRQ7
#define INTR_RTC	INTR_ISAIRQ8
#define INTR_SMB	INTR_ISAIRQ9
#define INTR_MOUSE	INTR_ISAIRQ12
#define INTR_IDE0	INTR_ISAIRQ14
#define INTR_IDE1	INTR_ISAIRQ15

/* YAMON's default PCI -> ISA IRQ mapping */
#define INTR_PCI_A	INTR_ISAIRQ10	/* PCI IRQA */
#define INTR_PCI_B	INTR_ISAIRQ10	/* PCI IRQB */
#define INTR_PCI_C	INTR_ISAIRQ11	/* PCI IRQC */
#define INTR_PCI_D	INTR_ISAIRQ11	/* PCI IRQD */

#define INTR_COREHI_0	32

/* Bonito interrupts */
#define INTR_BONITO_MBOX0	32
#define INTR_BONITO_MBOX1	33
#define INTR_BONITO_MBOX2	34
#define INTR_BONITO_MBOX3	35
#define INTR_BONITO_DMARDY	36
#define INTR_BONITO_DMAEMPTY	37
#define INTR_BONITO_COPYRDY	38
#define INTR_BONITO_COPYEMPTY	39
#define INTR_BONITO_COPYERR	40
#define INTR_BONITO_PCIIRQ	41
#define INTR_BONITO_MASTERERR	42
#define INTR_BONITO_SYSTEMERR	43
#define INTR_BONITO_DRAMPERR	44
#define INTR_BONITO_RETRYERR	45
#define INTR_BONITO_GPIO0	48
#define INTR_BONITO_GPIO1	49
#define INTR_BONITO_GPIO2	50
#define INTR_BONITO_GPIO3	51
#define INTR_BONITO_GPIN0	57
#define INTR_BONITO_GPIN1	58
#define INTR_BONITO_GPIN2	59
#define INTR_BONITO_GPIN3	60
#define INTR_BONITO_GPIN4	61
#define INTR_BONITO_GPIN5	62
#define INTR_BONITO_MAX	        63

/* Galileo GT64120 interrupts */
#define INTR_GALILEO_INTSUM		32
#define INTR_GALILEO_MEMOUT		33
#define INTR_GALILEO_DMAOUT		34
#define INTR_GALILEO_CPUOUT		35
#define INTR_GALILEO_DMA0COMP		36
#define INTR_GALILEO_DMA1COMP		37
#define INTR_GALILEO_DMA2COMP		38
#define INTR_GALILEO_DMA3COMP		39
#define INTR_GALILEO_T0EXP		40
#define INTR_GALILEO_T1EXP		41
#define INTR_GALILEO_T2EXP		42
#define INTR_GALILEO_T3EXP		43
#define INTR_GALILEO_MASRDERR		44
#define INTR_GALILEO_SLVWRERR		45
#define INTR_GALILEO_MASWRERR		46
#define INTR_GALILEO_SLVRDERR		47
#define INTR_GALILEO_ADDRERR		48
#define INTR_GALILEO_MEMERR		49
#define INTR_GALILEO_MASABORT		50
#define INTR_GALILEO_TARABORT		51
#define INTR_GALILEO_RETRYCTR0		52
#define INTR_GALILEO_PMCINT0		53
#define INTR_GALILEO_CPU2PCIA		54
#define INTR_GALILEO_CPU2PCIB		55
#define INTR_GALILEO_CPU2PCIC		56
#define INTR_GALILEO_CPU2PCID		57
#define INTR_GALILEO_PCI2CPUA		58
#define INTR_GALILEO_PCI2CPUB		59
#define INTR_GALILEO_PCI2CPUC		60
#define INTR_GALILEO_PCI2CPUD		61
#define INTR_GALILEO_CPUINTSUM		62
#define INTR_GALILEO_PCIINTSUM		63
#define	INTR_GALILEO_MAX		64

/* Overall maximum interrupt used */

#define INTR_MAX	INTR_GALILEO_MAX
