/*
 * Copyright (c) 2003-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * malta/sbdpci-table.h: indirect PCI access routines
 */


#ifndef SBD_PCI_TABLE_H
#define SBD_PCI_TABLE_H


typedef struct {
  int (*_pci_hwinit) (int);
  void (*_pci_hwreinit) (void);
  vm_offset_t (*_pci_dmamap) (vm_offset_t,unsigned int);
  vm_offset_t (*_pci_cpumap) (vm_offset_t,unsigned int);
  pcitag_t (*_pci_make_tag) (int,int,int);
  void (*_pci_break_tag) (pcitag_t,int*,int*,int*);
  pcireg_t (*_pci_conf_read) (pcitag_t,int);
  void (*_pci_conf_write) (pcitag_t,int,pcireg_t);
  int (*_pci_canscan) (pcitag_t);
  int (*_pci_map_io) (pcitag_t,int,vm_offset_t*,vm_offset_t*);
  int (*_pci_map_mem) (pcitag_t,int,vm_offset_t*,vm_offset_t*);
  int (*_pci_map_int) (pcitag_t);
  pcireg_t (*_pci_allocate_mem) (pcitag_t,size_t);
  pcireg_t (*_pci_allocate_io) (pcitag_t,size_t);
  void* (*_isa_map_io) (unsigned int);
  void* (*_isa_map_mem) (vm_offset_t);
} _pci_funcs_t;

#define REGISTER_PCI_FUNC(T,X) _sbd_pci_table.X = __##T##X

#define REGISTER_ALL_PCI_FUNCS(T)              \
REGISTER_PCI_FUNC(T,_pci_hwinit);              \
REGISTER_PCI_FUNC(T,_pci_hwreinit);            \
REGISTER_PCI_FUNC(T,_pci_dmamap);              \
REGISTER_PCI_FUNC(T,_pci_cpumap);              \
REGISTER_PCI_FUNC(T,_pci_make_tag);            \
REGISTER_PCI_FUNC(T,_pci_break_tag);           \
REGISTER_PCI_FUNC(T,_pci_conf_read);           \
REGISTER_PCI_FUNC(T,_pci_conf_write);          \
REGISTER_PCI_FUNC(T,_pci_canscan);             \
REGISTER_PCI_FUNC(T,_pci_map_io);              \
REGISTER_PCI_FUNC(T,_pci_map_mem);             \
REGISTER_PCI_FUNC(T,_pci_map_int);             \
REGISTER_PCI_FUNC(T,_pci_allocate_mem);        \
REGISTER_PCI_FUNC(T,_pci_allocate_io);         \
REGISTER_PCI_FUNC(T,_isa_map_io);              \
REGISTER_PCI_FUNC(T,_isa_map_mem)

#define REGISTER_STD_PCI_FUNCS(T)              \
REGISTER_PCI_FUNC(T,_pci_hwinit);              \
REGISTER_PCI_FUNC(T,_pci_hwreinit);            \
REGISTER_PCI_FUNC(T,_pci_dmamap);              \
REGISTER_PCI_FUNC(T,_pci_cpumap);              \
REGISTER_PCI_FUNC(T,_pci_make_tag);            \
REGISTER_PCI_FUNC(T,_pci_break_tag);           \
REGISTER_PCI_FUNC(T,_pci_conf_read);           \
REGISTER_PCI_FUNC(T,_pci_conf_write);          \
REGISTER_PCI_FUNC(T,_pci_canscan);             \
REGISTER_PCI_FUNC(T,_pci_map_io);              \
REGISTER_PCI_FUNC(T,_pci_map_mem)


extern _pci_funcs_t _sbd_pci_table;

#endif /* SBD_PCI_TABLE_H */
