#ident "kit/malta/sbdrtpx.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2002-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * sbdrtpx.c: low-level interrupt and timer handling for MIPS Malta board
 */


#include <sys/types.h>
#include <mips/cpu.h>
#include <mips/prid.h>
#include <mips/xcpt.h>
#include "kit_impl.h"
#include "sbd.h"
#include "sbdicu.h"
#include "i8259.h"
#include "gt64120.h"
#include "bonito.h"

#include "pthread_impl.h"

#include <sys/init.h>

/* 
 * Interrupt wiring under MIPS RTPX:
 *  HINT5 - cpu clock
 *  HINT4 - core lo
 *  HINT3 - core hi (sys ctrlr)
 *  HINT2 - tty2 (discrete ns16550)
 *  HINT1 - smi
 *  HINT0 - piix icu
 *  SINT1 - unused/reserved
 *  SINT0 - unused/reserved
 */

/* the interrupt mask when at "user" level (ipl0) */
_sbd_intrmask_t		_sbd_ipl0_mask;

/* the CPU status register when at scheduler level (ipl1) */
unsigned int 		_sbd_ipl1_sr;

/* the CPU status register when at interrupt blocking level (iplhi) */
unsigned int		_sbd_iplhi_sr;

/* the interrupt mask with which we were entered (e.g. from TAMON),
   see .../kit/yamon/yamonlow.sx */
unsigned int 		_sr_base __attribute__((common));

/* tell TAMON glue code that we handle all interrupts,
   see .../kit/yamon/yamonxlow.sx */
unsigned 		_intrmask = CR_IMASK;

/* table mapping CPU interrupt cause register bits to interrupt numbers */
static signed char	crToIntr[256];

/* pointer to Bonito registers */
static volatile char * const _bonito = PA_TO_KVA1 (BONITO_REG_BASE);

/* Bonito board-specific interrupt requests */
#define BONITO_ICU_PCI_IRQA    BONITO_ICU_GPIO(0)
#define BONITO_ICU_PCI_IRQB    BONITO_ICU_GPIO(1)
#define BONITO_ICU_PCI_IRQC    BONITO_ICU_GPIO(2)
#define BONITO_ICU_PCI_IRQD    BONITO_ICU_GPIO(3)
#define BONITO_ICU_NMI_IRQ     BONITO_ICU_GPIN(0)
#define BONITO_ICU_ISA_IRQ     BONITO_ICU_GPIN(1)
#define BONITO_ICU_ETH_IRQ     BONITO_ICU_GPIN(2)
#define BONITO_ICU_BIDE_IRQ    BONITO_ICU_GPIN(3)
#define BONITO_ICU_UART1_IRQ   BONITO_ICU_GPIN(4)
#define BONITO_ICU_UART0_IRQ   BONITO_ICU_GPIN(5)

/* pointers to ISA interrupt controllers */
static volatile struct i8259reg * isamaster;
static volatile struct i8259reg * isaslave;
static volatile unsigned char * elcr1;
static volatile unsigned char * elcr2;

static void
_rtpx_init (void)
{
    switch (_sbd_sysctrl_type) {
    case SYSCTRL_IS_BONITO:
    case SYSCTRL_IS_GALILEO:
	break;
    default:
        abort(); /* rtpx only supports bonito and galileo */
    }

    isamaster = PA_TO_KVA1 (ISAPORT_BASE (ICU1_PORT));
    isaslave = PA_TO_KVA1 (ISAPORT_BASE (ICU2_PORT));
#if #endian(big)
    elcr1 = PA_TO_KVA1 (ISAPORT_BASE (ELCR1_PORT) ^ 3);
    elcr2 = PA_TO_KVA1 (ISAPORT_BASE (ELCR2_PORT) ^ 3);
#else
    elcr1 = PA_TO_KVA1 (ISAPORT_BASE (ELCR1_PORT));
    elcr2 = PA_TO_KVA1 (ISAPORT_BASE (ELCR2_PORT));
#endif
}

DECL_INIT_FUNC (IF_DEVICE, _rtpx_init);

#ifndef _SBD_CLK_DIV
extern long _sbd_cyclespercount (void);
#define _SBD_CLK_DIV	_sbd_cyclespercount ()
#endif

#ifndef _SBD_CLK_MULT
#define _SBD_CLK_MULT	1
#endif

/* 
 * Return number of timer ticks per second
 */
unsigned int
_sbd_tickspersec (void)
{
    unsigned int freq = _sbd_pipefreq ();

    /* Convert from pipeline frequency to count register frequency. */
    freq = freq * _SBD_CLK_MULT / _SBD_CLK_DIV;

    /* Initialise on-chip counter interrupt */
#ifdef mips_bicconfig
    switch ((mips_getprid () >> 8) & 0xff) {
    case PRID_20KC:
    case PRID_25KF:
#define M20KC_CFG0_TI		0x01000000
	/* Wire timer interrupt to IP5 */
	mips_bicconfig (M20KC_CFG0_TI);
	break;
    }
#endif

    return freq;
}


/*
 * Return current value of timer (upcount)
 */
unsigned int
_sbd_gettickval (void)
{
    return mips_getcount();
}


/*
 * Set value at which timer should generate interrupt, and
 * clear any pending timer interrupt.
 */
void
_sbd_settickcmp (unsigned int v)
{
    /* set next tick time, and clear interrupt */
    mips_setcompare(v);
}


int 
_sbd_intr_howmany ()
{
    return INTR_MAX + 1;
}


/* disable interrupt and clear edge detect latches */
static __inline__ void
intrmaskandclear (int intr)
{
    /* note this is called at iplhi, so we don't need to use atomic
       updates on the mask values */

    if (INTR_IS_CPU (intr)) {
	_sbd_ipl0_mask.sr &= ~(1 << intr);
	/* clear software interrupts */
	if (intr <= INTR_SINT1)
	    (void) mips_biccr (1 << intr);
    }
    else if (INTR_IS_SYSCTRL (intr)) {
	_sbd_ipl0_mask.sysctrl &= ~(1 << (intr & 31));
	switch (_sbd_sysctrl_type) {
	case SYSCTRL_IS_BONITO:
            /* disable interrupt and clear edge-detect latch in one go */
            BONITO_INTENCLR = 1 << (intr & 31);
	    break;
	case SYSCTRL_IS_GALILEO:
            /* clear interrupt latch and set new mask */
            GT_INTRCAUSE = htoll (~(1 << (intr & 31)));
            GT_INTRMASK = htoll (_sbd_ipl0_mask.sysctrl);
	    break;
        }
	mips_wbflush ();
    }
    else if (INTR_IS_ISA_SLAVE (intr)) {
	_sbd_ipl0_mask.isaslave &= ~(1 << (intr & 7));
	isaslave->i8259_ocw1 = ~_sbd_ipl0_mask.isaslave;
	isaslave->i8259_cmd = I8259_OCW2 | I8259_SPECIFIC | I8259_EOI
	    | (intr & 7);
	isamaster->i8259_cmd = I8259_OCW2 | I8259_SPECIFIC | I8259_EOI 
	    | 2;
        _mips_sync();
	(void) isamaster->i8259_rr;	/* ensure last PCI cycle complete */
        _mips_sync();
    }
    else {
	_sbd_ipl0_mask.isamaster &= ~(1 << (intr & 7));
	isamaster->i8259_ocw1 = ~_sbd_ipl0_mask.isamaster;
	isamaster->i8259_cmd = I8259_OCW2 | I8259_SPECIFIC | I8259_EOI
	    | (intr & 7);
        _mips_sync();
	(void) isamaster->i8259_rr;	/* ensure last PCI cycle complete */
        _mips_sync();
    }
}


/* 
 * Poll specific interrupt, returns 0 if interrupt is not active.
 * Called at ipl1.
 */
int 
_sbd_intrpoll (int intr)
{
    unsigned int ret;

    if (INTR_IS_CPU (intr)) {
	if (intr <= INTR_SINT1) {
	    /* read and clear software interrupts (atomically) */
	    mips_setsr (_sbd_iplhi_sr);
	    ret = mips_biccr (1 << intr);
	    mips_setsr (_sbd_ipl1_sr);
	}
	else {
	    /* read hardware interrupts */
	    ret = mips_getcr();
	}
	ret &= (1 << intr);
    }
    else if (INTR_IS_SYSCTRL (intr)) {
	switch (_sbd_sysctrl_type) {
	case SYSCTRL_IS_BONITO:
            ret = BONITO_INTISR & (1 << (intr & 31));
            if (ret) {
                /* clear edge-detect latch */
                BONITO_INTENCLR = ret;
		mips_wbflush ();
	    }
	    break;
	case SYSCTRL_IS_GALILEO:
            ret = ltohl (GT_INTRCAUSE) & (1 << (intr & 31));
            if (ret) {
                /* clear edge-detect latch */
                GT_INTRCAUSE = htoll (~ret);
		mips_wbflush ();
	    }
	    break;
        }
    }
    else if (INTR_IS_ISA_SLAVE (intr)) {
	ret = isaslave->i8259_rr & (1 << (intr & 7));
	if (ret) {
	    /* clear interrupt */
	    isaslave->i8259_cmd = I8259_OCW2 | I8259_SPECIFIC | I8259_EOI
		| (intr & 7);
	    isamaster->i8259_cmd = I8259_OCW2 | I8259_SPECIFIC | I8259_EOI 
		| 2;
            _mips_sync();
	    (void) isamaster->i8259_rr;	/* ensure last PCI cycle complete */
            _mips_sync();
	}
    }
    else {
	ret = isamaster->i8259_rr & (1 << (intr & 7));
	if (ret) {
	    /* clear interrupt */
	    isamaster->i8259_cmd = I8259_OCW2 | I8259_SPECIFIC | I8259_EOI
		| (intr & 7);
            _mips_sync();
	    (void) isamaster->i8259_rr;	/* ensure last PCI cycle complete */
            _mips_sync();
	}
    }

    return ret != 0;
}


/*
 * Enable ipl0 interupts which have been blocked, in particular
 * when interrupt handler first installed.
 * Called at ipl1.
 */
void 
_sbd_intrenable (int intr)
{
    /* We must use atomic operations on mask "registers" because they
       may be shared with interrupt code running at higher "priority"
       than ipl1. */
  
    if (INTR_IS_CPU (intr)) {
	/* when enabled at ipl0, make sure it's disabled at ipl1 */
	mips_atomic_bis (&_sbd_ipl0_mask.sr, 1 << intr);
	mips_atomic_bic (&_sbd_ipl1_sr, 1 << intr);
    }
    else if (INTR_IS_SYSCTRL (intr)) {
	mips_setsr (_sbd_iplhi_sr);
	_sbd_ipl0_mask.sysctrl |= 1 << (intr & 31);
	switch (_sbd_sysctrl_type) {
	case SYSCTRL_IS_BONITO:
            /* Bonito implements atomic set of mask */
            BONITO_INTENSET = 1 << (intr & 31);
	    break;
	case SYSCTRL_IS_GALILEO:
            /* Galileo */
            GT_INTRMASK = htoll (_sbd_ipl0_mask.sysctrl);
	    break;
        }
	mips_wbflush ();
	mips_setsr (_sbd_ipl1_sr);
    }
    else if (INTR_IS_ISA_MASTER (intr)) {
	mips_setsr (_sbd_iplhi_sr);
	_sbd_ipl0_mask.isamaster |= 1 << (intr & 7);
	isamaster->i8259_ocw1 = ~_sbd_ipl0_mask.isamaster;
	(void) isamaster->i8259_rr;
	_mips_sync ();
	mips_setsr (_sbd_ipl1_sr);
    }
    else {
	mips_setsr (_sbd_iplhi_sr);
	_sbd_ipl0_mask.isaslave |= 1 << (intr & 7);
	isaslave->i8259_ocw1 = ~_sbd_ipl0_mask.isaslave;
	(void) isamaster->i8259_rr;
        _mips_sync();
	mips_setsr (_sbd_ipl1_sr);
    }
}


/* 
 * Take a guess at which ipl this CP0 status register represents. If
 * in doubt return 0. 
 */
int
_sbd_probable_ipl (unsigned int sr)
{
    return !(sr & SR_IE) ? 2 : (sr & SR_HINT5) ? 0 : 1;
}


/* 
 * Return next pending interrupt for this cpu, or zero if none.
 * If idle != 0 then we can spin waiting for an interrupt 
 * (possibly going into low-power mode) 
 * Called at ipl1.
 */
int 
_sbd_intrnext (int idle)
{
  int intr;

  /* switch to "hi" interrupt priority (all interrupts disabled) */
  mips_setsr (_sbd_iplhi_sr);

  while (1) {
      unsigned int pend;

      /* find interrupts which we handle that would be pending at ipl0 */
      pend = mips_getcr() & _sbd_ipl0_mask.sr & _intrmask;

      /* quick map from cause register bits to interrupt number */
      intr = crToIntr[pend >> 8];
      
      if (intr == INTR_COREHI) {
	  /* demultiplex core system controller interrupts */
	  unsigned int irr;
	  switch (_sbd_sysctrl_type) {
	  case SYSCTRL_IS_BONITO:
              irr = BONITO_INTISR & _sbd_ipl0_mask.sysctrl;
              intr = INTR_COREHI_0 + 31 - mips_clz (irr);
	      break;
	  case SYSCTRL_IS_GALILEO:
              irr = ltohl (GT_INTRCAUSE) & _sbd_ipl0_mask.sysctrl;
              intr = INTR_COREHI_0 + 31 - mips_clz (irr);
	      break;
	  }
      }
      else if (intr  == INTR_ISAMASTER) {
	  /* demultiplex PIIX i8259 icu interrupts */
	  unsigned int mirr;
	  mirr = isamaster->i8259_rr & _sbd_ipl0_mask.isamaster;
	  intr = 0;
	  while (mirr != 0 && intr == 0) {
	      int mintr = 31 ^ mips_clz (mirr);
	      mirr &= ~(1 << mintr);
	      intr = INTR_ISAIRQ0 + mintr;
	      if (intr == INTR_ISASLAVE) {
		  unsigned int sirr;
		  sirr = isaslave->i8259_rr & _sbd_ipl0_mask.isaslave;
		  if (sirr != 0)
		      intr = INTR_ISAIRQ8 + 31 - mips_clz (sirr);
		  else
		      intr = 0;
	      }
	  }
      }

      if (intr != 0 || !idle)
	  break;

      /* return to ipl1 to allow non-RTPX interrupts to be serviced */
      mips_setsr (_sbd_ipl1_sr);
      mips_setsr (_sbd_iplhi_sr);
  }

  /* disable and clear the detected interrupt at ipl0; note that clock i/u
     (intr == -1) is never disabled. */
  if (intr > 0)
      intrmaskandclear (intr);

  /* return to ipl1 */
  mips_setsr (_sbd_ipl1_sr);
  return intr;
}


/* 
 * Board-specific interrupt handler, return 1 if interrupt handled.
 * Called at iplhi.
 */
int 
_sbd_intrhandler (unsigned int pend, struct xcptcontext *xcp)
{
    /* everything else is passed to the scheduler */
    return 0;
}


int
_sbd_intrtype (int intr, const intrtype_t *new, intrtype_t *old)
{
    if (old) {
	if (INTR_IS_CPU (intr))
	    old->mode = INTRTYPE_LEVEL_HI;
	else if (INTR_IS_SYSCTRL (intr)) {
	    unsigned int bit = 1 << (intr & 31);
	    switch (_sbd_sysctrl_type) {
	    case SYSCTRL_IS_BONITO:
		old->mode = (BONITO_INTEDGE & bit)
		    ? ((BONITO_INTPOL & bit)
		       ? INTRTYPE_EDGE_RISING : INTRTYPE_EDGE_FALLING)
		    : ((BONITO_INTPOL & bit)
		       ? INTRTYPE_LEVEL_HI : INTRTYPE_LEVEL_LO);
		break;
	    case SYSCTRL_IS_GALILEO:
		break;
	    }
	}
	else if (INTR_IS_ISA_MASTER (intr)) {
	    old->mode = (*elcr1 & (1 << (intr & 7)))
		? INTRTYPE_LEVEL_HI : INTRTYPE_EDGE_RISING;
	}
	else {
	    old->mode = (*elcr2 & (1 << (intr & 7)))
		? INTRTYPE_LEVEL_HI : INTRTYPE_EDGE_RISING;
	}
    }
	
    /* note that we can't change the active polarity, user must
       just get this right. */
    if (new) {
	if (INTR_IS_SYSCTRL (intr)) {
	    unsigned int bit = 1 << (intr & 31);
	    switch (_sbd_sysctrl_type) {
	    case SYSCTRL_IS_BONITO:
		switch (new->mode) {
		case INTRTYPE_LEVEL_LO:
		    BONITO_INTPOL &= ~bit;
		    BONITO_INTEDGE &= ~bit;
		    break;
		case INTRTYPE_LEVEL_HI:
		    BONITO_INTPOL |= bit;
		    BONITO_INTEDGE &= ~bit;
		    break;
		case INTRTYPE_EDGE_FALLING:
		    BONITO_INTPOL &= ~bit;
		    BONITO_INTEDGE |= bit;
		    break;
		case INTRTYPE_EDGE_RISING:
		    BONITO_INTPOL |= bit;
		    BONITO_INTEDGE |= bit;
		    break;
		default:
		    return 1;
		}
		break;
	    case SYSCTRL_IS_GALILEO:
		break;
          }
	}
	else if (INTR_IS_ISA_MASTER (intr)) {
	    unsigned int bit = 1 << (intr & 7);
	    if (new->mode == INTRTYPE_LEVEL_HI)
		*elcr1 |= bit;
	    else if (new->mode == INTRTYPE_EDGE_RISING)
		*elcr1 &= ~bit;
	    else
		return 1;
	}
	else if (INTR_IS_ISA_SLAVE (intr)) {
	    unsigned int bit = 1 << (intr & 7);
	    if (new->mode == INTRTYPE_LEVEL_HI)
		*elcr2 |= bit;
	    else if (new->mode == INTRTYPE_EDGE_RISING)
		*elcr2 &= ~bit;
	    else
		return 1;
	}
	else if (new->mode != INTRTYPE_LEVEL_HI)
	    return 1;
    }

    return 0;
}


/* 
 * Initialise interrupt handling for this board 
 * Called at undefined ipl, should return at iplhi
 */
void 
_sbd_intrinit(void)
{
    int i;

    /* disable all interrupts at interrupt level and set cpu to iplhi */
    _sbd_iplhi_sr = _sr_base & ~(SR_CU1 | SR_IMASK | SR_IE);
    mips_setsr (_sbd_iplhi_sr);

    /* disable clock, piix & sysctrl interrupt in scheduler */
    _sbd_ipl1_sr = (_sr_base & ~(SR_HINT5 | SR_HINT3 | SR_HINT0)) | SR_IE;

    /* enable clock, piix & sysctrl interrupt group at user level */
    _sbd_ipl0_mask.sr = (_sr_base & SR_IMASK) | SR_HINT5 | SR_HINT3 | SR_HINT0;

    /* enable only ISA slave icu in master icu mask */
    _sbd_ipl0_mask.isamaster = 1 << 2;

    /* all interrupts disabled in ISA slave icu */
    _sbd_ipl0_mask.isaslave = 0;
    
    switch (_sbd_discover_sysctrl()) {
    case SYSCTRL_IS_BONITO:
        /* enable only ISA master icu in bonito mask */
        _sbd_ipl0_mask.bonito = BONITO_ICU_ISA_IRQ;
        
        /* initialise Bonito icu edge and level detection */
        BONITO_INTEDGE 	= (BONITO_ICU_SYSTEMERR | BONITO_ICU_MASTERERR
			   | BONITO_ICU_RETRYERR | BONITO_ICU_MBOXES);
        BONITO_INTPOL 	= (BONITO_ICU_UART0_IRQ | BONITO_ICU_UART1_IRQ
			   | BONITO_ICU_BIDE_IRQ | BONITO_ICU_ISA_IRQ
			   | BONITO_ICU_NMI_IRQ);

        /* initialise Bonito interrupt routing */
        BONITO_INTSTEER	= 0;		/* all routed to CPU Intr0 */
        BONITO_INTENCLR	= ~0;
        BONITO_INTENSET	= _sbd_ipl0_mask.bonito;
	break;

    case SYSCTRL_IS_GALILEO:
        GT_INTRMASK	= 0;
        GT_INTRMASKHI	= 0;
        GT_INTRCAUSE	= 0;
        GT_INTRCAUSEHI	= 0;
	break;
    }

    /* initialise ISA slave icu */
    isaslave->i8259_dat	= I8259_ICW1 | I8259_NEEDICW4;	/* icw1 */
    isaslave->i8259_dat	= 0x08;				/* icw2 - vector */
    isaslave->i8259_dat	= 0x02;				/* icw3 - slave id */
    isaslave->i8259_dat	= I8259_8086 | I8259_NOBUF;	/* icw4 */
    isaslave->i8259_cmd	= (I8259_OCW3 			/* ocw3 */
			   | I8259_ESMM | I8259_SMM	/* special mask mode */
			   | I8259_RR | I8259_RIR);	/* read IR reg */
    isaslave->i8259_ocw1= ~_sbd_ipl0_mask.isaslave;	/* ocw1 - set mask */
    
    /* initialise ISA master icu */
    isamaster->i8259_cmd = I8259_ICW1 | I8259_NEEDICW4;	/* icw1 */
    isamaster->i8259_dat = 0x00;			/* icw2 - vector */
    isamaster->i8259_dat = 0x04;			/* icw3 - slave id */
    isamaster->i8259_dat = I8259_8086 | I8259_NOBUF;	/* icw4 */
    isamaster->i8259_cmd = (I8259_OCW3 			/* ocw3 */
			   | I8259_ESMM | I8259_SMM	/* special mask mode */
			   | I8259_RR | I8259_RIR);	/* read IR reg */
    isamaster->i8259_ocw1= ~_sbd_ipl0_mask.isamaster;	/* ocw1 - set mask */
    
    _mips_sync();
    
    /* build Cause bits to interrupt number mapping table */
    for (i = 0; i < 256; i++) {
	unsigned int cr = i << 8;
	int intr;
	/* test in "priority" order */
	if (cr & CR_HINT5)
	    intr = INTR_CLOCK;
	else if (cr & CR_HINT4)
	    intr = INTR_CORELO;
	else if (cr & CR_HINT3)
	    intr = INTR_COREHI;
	else if (cr & CR_HINT2)
	    intr = INTR_TTY2;
	else if (cr & CR_HINT1)
	    intr = INTR_SMI;
	else if (cr & CR_HINT0)
	    intr = INTR_ISAMASTER;
	else if (cr & CR_SINT1)
	    intr = INTR_SINT1;
	else if (cr & CR_SINT0)
	    intr = INTR_SINT0;
	else
	    intr = 0;
	crToIntr[i] = intr;
    }

    /* set a "safe" compare value and clear timer interrupt */
    mips_setcompare (mips_getcount() + INT_MAX);

    /* disable auto-reset on rs232 break */
    *(volatile unsigned int *) PA_TO_KVA1 (BRKRES_ADDR) = 0;
    mips_wbflush ();
}


/* 
 * Called by remote debug stub when first initialised to enable "debug"
 * interrupts (NMI?).
 */
void
_dbg_signals (void)
{
}
