#ident "kit/malta/sbdsysctrl.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2003-2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * malta/sbdpci.c: Machine-specific functions for PCI autoconfiguration.
 */

#include <sys/types.h>
#include <stdlib.h>
#include <mips/xcpt.h>
#include <mips/cpu.h>

#include "sbd.h"
#include "product.h"

#include <sys/init.h>

int _sbd_sysctrl_type = SYSCTRL_IS_UNKNOWN;

unsigned long _sbd_pci_io_space;
unsigned long _sbd_pci_io_size;
unsigned long _sbd_pci_mem_space;
unsigned long _sbd_pci_mem_size;

/* Some very basic stuff for seeing if a bonito is present */
#define BONITO_REG_BASE              0x1fe00000
#define _bonito                      PA_TO_KVA1(BONITO_REG_BASE)
#define BONITO(x)                    *(volatile unsigned int *)(_bonito + (x))
#define BONITO_PCICONFIGBASE         0x00
#define BONITO_PCI_REG(x)            BONITO(BONITO_PCICONFIGBASE + (x))
#define BONITO_PCIDID                BONITO_PCI_REG(0x00)

int
_sbd_discover_sysctrl(void)
{
    unsigned int mipsrev;

    if (_sbd_sysctrl_type != SYSCTRL_IS_UNKNOWN)
	return _sbd_sysctrl_type;
	
    mipsrev = *((volatile unsigned int*) PA_TO_KVA1(MIPS_REVISION));

    switch ((mipsrev & MIPS_REVISION_CORID_MSK) >> MIPS_REVISION_CORID_SHF) {

    case MIPS_REVISION_CORID_QED_RM5261:
    case MIPS_REVISION_CORID_CORE_LV:
    case MIPS_REVISION_CORID_CORE_FPGA:
    case MIPS_REVISION_CORID_CORE_FPGAr2:
        _sbd_sysctrl_type = SYSCTRL_IS_GALILEO;
        break;

    case MIPS_REVISION_CORID_BONITO64:
    case MIPS_REVISION_CORID_CORE_20K:
        _sbd_sysctrl_type = SYSCTRL_IS_BONITO;
        break;

    case MIPS_REVISION_CORID_CORE_EMUL:
        /* CORE_EMUL is either bonito or socit */
        switch (BONITO_PCIDID) {
	case 0x0001df53:
	case 0x0003df53:
            _sbd_sysctrl_type = SYSCTRL_IS_BONITO;
	    break;
        default: /* XXX */
            _sbd_sysctrl_type = SYSCTRL_IS_SOCIT;
	    break;
	}
	break;

    case MIPS_REVISION_CORID_CORE_SYS:
    case MIPS_REVISION_CORID_CORE_FPGA2:
    case MIPS_REVISION_CORID_CORE_FPGA3:
    default:
    switch ((mipsrev & MIPS_REVISION_SCON_MSK) >> MIPS_REVISION_SCON_SHF) {
	case MIPS_REVISION_SCON_SOCITSC:
	case MIPS_REVISION_SCON_SOCITSCP:
	    _sbd_sysctrl_type = SYSCTRL_IS_SOCITSC;
	    break;
	case MIPS_REVISION_SCON_OTHER:
	default:
	    /* default to SOCit */
	    _sbd_sysctrl_type = SYSCTRL_IS_SOCIT;
	    break;
	}
        break;
    }
    
    /* sort out the PCI spaces based on the sysctrl type */
    switch (_sbd_sysctrl_type) {
    case SYSCTRL_IS_SOCITSC:
        PCI_IO_SPACE = PCI_IO_SPACE_SOCITSC;
        PCI_IO_SIZE = PCI_IO_SIZE_SOCITSC;
        PCI_MEM_SPACE = PCI_MEM_SPACE_SOCITSC;
        PCI_MEM_SIZE = PCI_MEM_SIZE_SOCITSC;
	break;
    case SYSCTRL_IS_SOCIT:
        PCI_IO_SPACE = PCI_IO_SPACE_SOCIT;
        PCI_IO_SIZE = PCI_IO_SIZE_SOCIT;
        PCI_MEM_SPACE = PCI_MEM_SPACE_SOCIT;
        PCI_MEM_SIZE = PCI_MEM_SIZE_SOCIT;
	break;
    case SYSCTRL_IS_BONITO:
        PCI_IO_SPACE  = PCI_IO_SPACE_BON;
        PCI_MEM_SPACE = PCI_MEM_SPACE_BON;
        PCI_IO_SIZE   = PCI_IO_SIZE_BON;
        PCI_MEM_SIZE  = PCI_MEM_SIZE_BON;
	break;
    case SYSCTRL_IS_GALILEO:
        PCI_IO_SPACE  = PCI_IO_SPACE_GAL;
        PCI_MEM_SPACE = PCI_MEM_SPACE_GAL;
        PCI_IO_SIZE   = PCI_IO_SIZE_GAL;
        PCI_MEM_SIZE  = PCI_MEM_SIZE_GAL;
	break;
    default:
	break;
    }

    return _sbd_sysctrl_type;
}

static void 
_sbd_init_sysctrl(void)
{
    (void) _sbd_discover_sysctrl ();
}

DECL_INIT_FUNC (IF_SYSCTRL, _sbd_init_sysctrl)
