/*
 * Copyright (c) 2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * malta/ssc_memmap.h: Memory map defines for SOC-it SC
 */

#ifndef SSC_MEMMAP_H
#define SSC_MEMMAP_H

#ifndef KSEG1_BASE
# define KSEG0_BASE      	0x80000000	/* unmapped, cachable	*/
# define KSEG1_BASE      	0xa0000000	/* unmapped, uncached	*/
#endif

/*****************************************************************************
 * Memory space
 ****************************************************************************/

#define SSC_PHYSBASE_DRAM	0x00000000
#define SSC_PHYSBASE_OCP0	0x10000000
#define SSC_PHYSBASE_OCP1	0x18000000
#define SSC_PHYSBASE_OCP2	0x1c000000
#define SSC_PHYSBASE_OCP0_BOOT	0x1fc00000
#define SSC_PHYSBASE_OCP1_BOOT	0x1fd00000
#define SSC_PHYSBASE_OCP2_BOOT	0x1fe00000
#define SSC_PHYSBASE_REGS	0x1ff00000
#define SSC_PHYSBASE_OCP0_BIG	0x20000000
#define SSC_PHYSBASE_OCP1_BIG	0x40000000
#define SSC_PHYSBASE_OCP2_BIG	0x60000000
#define SSC_PHYSBASE_DRAM_BIG	0x80000000

#define SSC_KSEG0BASE_DRAM	(KSEG0_BASE + SSC_PHYSBASE_DRAM)
#define SSC_KSEG0BASE_OCP0	(KSEG0_BASE + SSC_PHYSBASE_OCP0)
#define SSC_KSEG0BASE_OCP1	(KSEG0_BASE + SSC_PHYSBASE_OCP1)
#define SSC_KSEG0BASE_OCP2	(KSEG0_BASE + SSC_PHYSBASE_OCP2)
#define SSC_KSEG0BASE_OCP0_BOOT	(KSEG0_BASE + SSC_PHYSBASE_OCP0_BOOT)
#define SSC_KSEG0BASE_OCP1_BOOT	(KSEG0_BASE + SSC_PHYSBASE_OCP1_BOOT)
#define SSC_KSEG0BASE_OCP2_BOOT	(KSEG0_BASE + SSC_PHYSBASE_OCP2_BOOT)
#define SSC_KSEG0BASE_REGS	(KSEG0_BASE + SSC_PHYSBASE_REGS)

 
/*****************************************************************************
 * Generic register space addresses, unmapped & uncached access via kseg1
 ****************************************************************************/

#define SSC_REGSPACE_OCP0	0
#define SSC_REGSPACE_OCP1	2
#define SSC_REGSPACE_OCP2	4
#define SSC_REGSPACE_SMX	6
#define SSC_REGSPACE_DRAM	7

#define SSC_REGADDR_SIZE	0x00020000		/* 128 KByte */
#define SSC_REGADDR(regspace)	(SSC_REGADDR_BASE +		\
				 (regspace) * SSC_REGADDR_SIZE)


/*****************************************************************************
 * Register base addresses
 ****************************************************************************/

#define SSC_REGADDR_BASE	(KSEG1_BASE + SSC_PHYSBASE_REGS)

#define SSC_OCP0_REG_BASE	SSC_REGADDR(SSC_REGSPACE_OCP0)
#define SSC_OCP1_REG_BASE	SSC_REGADDR(SSC_REGSPACE_OCP1)
#define SSC_OCP2_REG_BASE	SSC_REGADDR(SSC_REGSPACE_OCP2)
#define SSC_SMX_REG_BASE	SSC_REGADDR(SSC_REGSPACE_SMX)
#define SSC_DRAM_REG_BASE	SSC_REGADDR(SSC_REGSPACE_DRAM)

#endif
