/*
 * Copyright (c) 2005-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * MIPS SDE kit MIPS MTSPMON interface definitions
 */


#define MTSP_SYSCALL_BASE	0

#define MTSP_SYSCALL_EXIT    	(MTSP_SYSCALL_BASE + 0)
#define MTSP_SYSCALL_OPEN    	(MTSP_SYSCALL_BASE + 1)
#define MTSP_SYSCALL_READ    	(MTSP_SYSCALL_BASE + 2)
#define MTSP_SYSCALL_WRITE   	(MTSP_SYSCALL_BASE + 3)
#define MTSP_SYSCALL_CLOSE   	(MTSP_SYSCALL_BASE + 4)
#define MTSP_SYSCALL_LSEEK32 	(MTSP_SYSCALL_BASE + 5)
#define MTSP_SYSCALL_ISATTY  	(MTSP_SYSCALL_BASE + 6)
#define MTSP_SYSCALL_GETTIME 	(MTSP_SYSCALL_BASE + 7)
#define MTSP_SYSCALL_PIPEFREQ 	(MTSP_SYSCALL_BASE + 8)
#define MTSP_SYSCALL_GETTOD  	(MTSP_SYSCALL_BASE + 9)
#define MTSP_SYSCALL_IOCTL  	(MTSP_SYSCALL_BASE + 10)

/* linux side may only support 4 args */
struct mtsp_syscall_generic {
    int arg0;
    int arg1;
    int arg2;
    int arg3;
    int arg4;
    int arg5;
    int arg6;
};

struct mtsp_syscall_open {
    const char *name;
    int flags;
    int mode;
};

struct mtsp_syscall_read {
    int fd;
    void *buff;
    int len;
};

struct mtsp_syscall_write {
    int fd;
    const void *buff;    
    int len;
};

struct mtsp_syscall_close {
    int fd;
};

struct mtsp_syscall_lseek32 {
    int fd;
    long offset;
    int where;
};

struct mtsp_syscall_ioctl {
    int fd;
    int cmd;
    void *arg;
};

struct mtsp_syscall_exit {
    int exitcode;
};

struct mtsp_syscall_ret {
	int retval;
	int errno;
};

struct mtsp_syscall {
    int cmd;
    unsigned char abi;
    unsigned char size;
};

#define MTSP_ABI_O32	0
#define MTSP_ABI_N32	1
#define MTSP_ABI_N64	2
#define MTSP_ABI_O64	3
#define MTSP_ABI_EABI32	4
#define MTSP_ABI_EABI64	5
#define MTSP_ABI_NUBI32	6
#define MTSP_ABI_NUBI64	7

/* Define current MTSP ABI */
#if _MIPS_SIM == _ABI32
# define MTSP_ABI	MTSP_ABI_O32
#elif _MIPS_SIM == _ABIN32
# define MTSP_ABI	MTSP_ABI_N32
#elif _MIPS_SIM == _ABI64
# define MTSP_ABI	MTSP_ABI_N64
#elif _MIPS_SIM == _ABIO64
# define MTSP_ABI	MTSP_ABI_O64
#elif _MIPS_SIM == _ABIEABI && __mips64
# define MTSP_ABI	MTSP_ABI_EABI64
#elif _MIPS_SIM == _ABIEABI
# define MTSP_ABI	MTSP_ABI_EABI32
#else
# define MTSP_ABI	MTSP_ABI_O32  /* default */
#endif
    
#ifndef __ASSEMBLER__
struct timeval;

extern int _mtsp_syscall (const struct mtsp_syscall *, 
			  const void *, struct mtsp_syscall_ret *);
extern int	_mtsp_open (const char *, int, int);
extern int	_mtsp_close (int);
extern int	_mtsp_read (int, void *, int);
extern int	_mtsp_write (int, const void *, int);
extern long	_mtsp_lseek32 (int, long, int);
extern int	_mtsp_ioctl(int, int, void *);
extern int 	_mtsp_isatty(int);
extern int 	_mtsp_gettime(struct timeval *);
extern void 	_mtsp_exit(int);
#endif
