#ident "kit/share/dbgsup.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2005-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * mtspdbgsup.c: implementation dependent remote debugger support for MIPS SDE
 */


#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <mips/cpu.h>
#include <mips/rtlx.h>
#include "kit_impl.h"

void
_dbg_panic (const char *fmt, ...)
{
    va_list ap;
    _mon_puts ("\r\n_dbg_panic: ");
    va_start (ap, fmt);
    _mon_vprintf (fmt, ap);
    va_end (ap);
    _mon_puts ("\r\n");
    abort ();
}

void
_dbg_printf (const char *fmt, ...)
{
    va_list ap;
    va_start (ap, fmt);
    _mon_vprintf (fmt, ap);
    va_end (ap);
}



/* 
 * Debug i/o stubs
 */

#include "sbdser.h"

int _stub_sbd_setdbgport (int *);
int
_stub_sbd_setdbgport (int *speed)
{
	return(0);
}


int
_dbg_init (void)
{
    return (_rtlx_open_dbg ());
}


int
_dbg_inch (void)
{
    char c;

    while ((_rtlx_read_dbg (&c, 1)) == 0)
        continue;

    return (c & 0xff);
}

int
_dbg_inpoll (void)
{
    char c;

    if (_rtlx_read_dbg (&c, 1) == 0)
        return -1;

    return (c & 0xff);
}


void
_dbg_outch (int c)
{
    char cb = c;
    _rtlx_write_dbg (&cb, 1);
}


void
_dbg_drain ()
{
}
