/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* p4032/icu.h: interrupt control unit support */

#ifndef _SBD_ICU_H_
#define _SBD_ICU_H_

#ifndef __ASSEMBLER__
typedef union {
    struct {
	unsigned int dev;
	unsigned int err;
	unsigned int pci;
    } irr;
    struct {
	unsigned int mask;
	unsigned int clear;
	unsigned int pcimask;
	unsigned int devxbar0;
	unsigned int devxbar1;
	unsigned int pcixbar;
    } ctrl;
} p4032icu;
#else
#define ICU_IRR_DEV		0
#define ICU_IRR_ERR		4
#define ICU_IRR_PCI		8
#define ICU_CTRL_MASK		0
#define ICU_CTRL_CLEAR		4
#define ICU_CTRL_PCIMASK	8
#define ICU_CTRL_DEVXBAR0	12
#define ICU_CTRL_DEVXBAR1	16
#define ICU_CTRL_PCIXBAR	20
#endif

/* device interrupt request and mask bits */
#define ICU_DEV_RTC	0x80
#define ICU_DEV_GPIO	0x40
#define ICU_DEV_CENT	0x20	/* also the "clear" bit */
#define ICU_DEV_UART1	0x10
#define ICU_DEV_UART0	0x08
#define ICU_DEV_KBD	0x04
#define ICU_DEV_FDC	0x02
#define ICU_DEV_V96	0x01

/* error interrupt request and clear bits */
#define ICU_ERR_BUSERR	0x04
#define ICU_ERR_ACFAIL	0x02
#define ICU_ERR_DEBUG	0x01

/* PCI request and mask bits */
#define ICU_PCI_INT3	0x80
#define ICU_PCI_INT2	0x40
#define ICU_PCI_INT1	0x20
#define ICU_PCI_INT0	0x10
#define ICU_PCI_FDCDMA	0x08	/* write only to enable DMA IRQ on HWINT3 */

/* devxbar0 shifts */
#define ICU_DEVX0_V96	0
#define ICU_DEVX0_FDC	2
#define ICU_DEVX0_KBD	4
#define ICU_DEVX0_UART0 6

/* devxbar1 shifts */
#define ICU_DEVX1_UART1 0
#define ICU_DEVX1_CENT	2
#define ICU_DEVX1_GPIO	4
#define ICU_DEVX1_RTC	6

/* pcixbar shifts */
#define ICU_PCIX_0	0
#define ICU_PCIX_1	2
#define ICU_PCIX_2	4
#define ICU_PCIX_3	6

/* interrupt selects */
#define ICU_HWINT0	0
#define ICU_HWINT1	1
#define ICU_HWINT2	2
#define ICU_HWINT3	3
#define ICU_HWNONE	3	/* if ICU_PCI_FDCDMA is set */
#define ICU_HWMASK	3

/* 
 * The software interrupt numbers are purely a convention, but
 * they should not be changed lightly.
 */

#ifndef NEWINTRUPT
/* error interrupts are not handled as normal interrupts in "new" scheme" */
#define INTR_MAX	28
#define INTR_BUSERR	27
#define INTR_ACFAIL	26
#define INTR_DEBUG	25
#else
#define INTR_MAX	25
#endif

/* 24-21 correspond to the PCI IRR register bits 7-4 */
#define INTR_PCI3	24
#define INTR_PCI2	23
#define INTR_PCI1	22
#define INTR_PCI0	21

/* 16-9 correspond to the CPU Cause register bits 15-8*/ 
#define INTR_PANIC	16	/* HINT5 */
#define INTR_TIMER	15	/* HINT4 */
#define INTR_FDCDMA	14	/* HINT3 */
#define INTR_CENT	13	/* HINT2 */
#define INTR_PCIMUX	12	/* HINT1 */
#define INTR_DEVMUX	11	/* HINT0 */
#define INTR_SINT1	10
#define INTR_SINT0	9

/* 8-1 correspond to the DEV IRR register bits 7-0 */
#define INTR_RTC	8
#define INTR_GPIO	7
#define INTR_UART1	5
#define INTR_UART0	4
#define INTR_KBD	3
#define INTR_FDC	2
#define INTR_V96	1

#define INTR_NONE	0
#define INTR_CLOCK	-1

#if !defined(__ASSEMBLER__) && !defined(NEWINTRUPT)
int	_icu_map (int inum, int hwint);
int	_icu_mask (int inum, int enable);
int	_icu_devintr (void);
int	_icu_pciintr (void);
void	_icu_init (void);
#endif

#endif /* _SBD_ICU_H_ */
