#ident "kit/p4032/sbd.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * p4032/sbd.c: low-level interface to P4032 board registers
 */


#include <stddef.h>
#include <mips/cpu.h>

#include "sbd.h"
#include "rtc.h"
#include "icu.h"

static inline void
crwrite (volatile unsigned int *cr, unsigned int oval, unsigned int nval) 
{
    unsigned int chg = oval ^ nval;

    while (chg != 0) {
	if (chg & 1)
	    *cr = nval & 1;
	cr++;
	nval >>= 1;
	chg >>= 1;
    }
}


unsigned int 
_bcr_get (void)
{
    return _rtc_get (RTC_BCR);
}


unsigned int
_bcr_set (unsigned int nbcr)
{
    unsigned int obcr = _rtc_set (RTC_BCR, nbcr);
    crwrite (PA_TO_KVA1 (BCR_BASE), obcr, nbcr);
    return obcr;
}


unsigned int 
_bcr_bic (unsigned int bits)
{
    unsigned int obcr = _rtc_bic (RTC_BCR, bits);
    unsigned int nbcr = obcr & ~bits;
    crwrite (PA_TO_KVA1 (BCR_BASE), obcr, nbcr);
    return obcr;
}


unsigned int 
_bcr_bis (unsigned int bits)
{
    unsigned int obcr = _rtc_bis (RTC_BCR, bits);
    unsigned int nbcr = obcr | bits;
    crwrite (PA_TO_KVA1 (BCR_BASE), obcr, nbcr);
    return obcr;
}



unsigned int 
_dcr_get (void)
{
    return _rtc_get (RTC_DCR);
}


unsigned int
_dcr_set (unsigned int ndcr)
{
    unsigned int odcr = _rtc_set (RTC_DCR, ndcr);
    crwrite (PA_TO_KVA1 (DCR_BASE), odcr, ndcr);
    return odcr;
}


unsigned int 
_dcr_bic (unsigned int bits)
{
    unsigned int odcr = _rtc_bic (RTC_DCR, bits);
    unsigned int ndcr = odcr & ~bits;
    crwrite (PA_TO_KVA1 (DCR_BASE), odcr, ndcr);
    return odcr;
}


unsigned int 
_dcr_bis (unsigned int bits)
{
    unsigned int odcr = _rtc_bis (RTC_DCR, bits);
    unsigned int ndcr = odcr | bits;
    crwrite (PA_TO_KVA1 (DCR_BASE), odcr, ndcr);
    return odcr;
}



unsigned int 
_imask_get (void)
{
    return _rtc_get (RTC_IMASK);
}


unsigned int 
_imask_set (unsigned int nmask)
{
    volatile p4032icu * const icu = PA_TO_KVA1 (ICU_BASE);
    unsigned int omask = _rtc_set (RTC_IMASK, nmask);
    icu->ctrl.mask = nmask;
    return omask;
}


unsigned int 
_imask_bic (unsigned int bits)
{
    unsigned int omask = _rtc_bic (RTC_IMASK, bits);
    unsigned int nmask = omask & ~bits;

    if (nmask != omask) {
	volatile p4032icu * const icu = PA_TO_KVA1 (ICU_BASE);
	icu->ctrl.mask = nmask;
    }
    return omask;
}


unsigned int 
_imask_bis (unsigned int bits)
{
    unsigned int omask = _rtc_bis (RTC_IMASK, bits);
    unsigned int nmask = omask | bits;

    if (nmask != omask) {
	volatile p4032icu * const icu = PA_TO_KVA1 (ICU_BASE);
	icu->ctrl.mask = nmask;
    }
    return omask;
}

#include <pci/pcivar.h>

void *
ioport_map (unsigned int port)
{
    return _isa_map_io (port);
}


unsigned char
inb (unsigned int port)
{
    return *(volatile unsigned char *) _isa_map_io (port);
}

unsigned short
inw (unsigned int port)
{
    return *(volatile unsigned short *) _isa_map_io (port);
}

unsigned long
inl (unsigned int port)
{
    return *(volatile unsigned long *) _isa_map_io (port);
}

void
outb (unsigned int port, unsigned char val)
{
    *(volatile unsigned char *) _isa_map_io (port) = val;
    mips_wbflush ();
}

void
outw (unsigned int port, unsigned short val)
{
    *(volatile unsigned short *) _isa_map_io (port) = val;
    mips_wbflush ();
}

void
outl (unsigned int port, unsigned long val)
{
    *(volatile unsigned long *) _isa_map_io (port) = val;
    mips_wbflush ();
}

