/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * sbd.h: Algorithmics P4032 board definition header file
 */


#ifndef MHZ
/* fastest possible pipeline clock (75MHz * 3, 62MHz * 4) */
#define MHZ		250
#endif

#define RAMCYCLE	60			/* 60ns dram cycle */
#define ROMCYCLE	750			/* ~750ns rom cycle */
#define CACHECYCLE	(1000/MHZ) 		/* pipeline clock */
#define CYCLETIME	CACHECYCLE
#define CACHEMISS	(CYCLETIME * 6)

/*
 * rough scaling factors for 2 instruction DELAY loop to get 1ms and 1us delays
 */
#define ASMDELAY(ns,icycle)	\
	(((ns) + (icycle)) / ((icycle) * 2))

#define CACHENS(ns)	ASMDELAY((ns), CACHECYCLE)
#define RAMNS(ns)	ASMDELAY((ns), CACHEMISS+RAMCYCLE)
#define ROMNS(ns)	ASMDELAY((ns), CACHEMISS+ROMCYCLE)
#define CACHEUS(us)	ASMDELAY((us)*1000, CACHECYCLE)
#define RAMUS(us)	ASMDELAY((us)*1000, CACHEMISS+RAMCYCLE)
#define ROMUS(us)	ASMDELAY((us)*1000, CACHEMISS+ROMCYCLE)
#define CACHEMS(ms)	((ms) * ASMDELAY(1000000, CACHECYCLE))
#define RAMMS(ms)	((ms) * ASMDELAY(1000000, CACHEMISS+RAMCYCLE))
#define ROMMS(ms)	((ms) * ASMDELAY(1000000, CACHEMISS+ROMCYCLE))

#ifndef __ASSEMBLER__
extern void _sbd_nsdelay (unsigned long);
#define nsdelay(ns)	_sbd_nsdelay (ns)
#define usdelay(us)	_sbd_nsdelay ((us) * 1000)
#endif

/* Our local to PCI bus apertures are set up with the following sizes */
#define PCI_MEM_SPACE_SIZE	(128 * 1024 * 1024)
#define PCI_IO_SPACE_SIZE_NEW	( 16 * 1024 * 1024)
#define PCI_IO_SPACE_SIZE_OLD	(  1 * 1024 * 1024)
#define PCI_CONF_SPACE_SIZE	(  1 * 1024 * 1024)

#define DRAM_BASE	0x00000000
#define FSIMM_BASE	0x04000000
#define FSIMM_SIZE	0x04000000
#define PCI_MEM_SPACE	0x10000000	/* 128MB: s/w configurable */
#define PCI_IO_SPACE_NEW 0x1d000000	/*  16MB: s/w configurable */
#define PCI_IO_SPACE_OLD 0x1ed00000	/*   1MB: s/w configurable */
#define PCI_CONF_SPACE	0x1ee00000	/*   1MB: s/w configurable */
#define V96XPBC_BASE	0x1ef00000	/*  64KB: s/w configurable */
#define BOOTPROM_BASE	0x1fc00000
#define EPROM_BASE	0x1fd00000
#define FLASH_BASE	0x1fe00000
#define RTC_BASE	0x1ff00000
#define  RTC_ADDR	 0
#define  RTC_DATA	 4
#define KEYBD_BASE	0x1ff10000
#define LED_BASE	0x1ff20010
#define  LED(n)	((3-(n))*4)
#define LCD_BASE	0x1ff30000
#define ZPIO_BASE	0x1ff40000
#define ZPIO_IACK	0x1ff50000
#define ZPIO_RETI	0x1ff50000
#define COMBI_BASE	0x1ff80000
#define ICU_BASE	0x1ff90000
#define FDC_DACK	0x1ffa0000
#define BCR_BASE	0x1ffb0000
#define DCR_BASE	0x1ffc0000
#define OPTION_BASE	0x1ffd0000

/* simple 8-bit local devices are connected to bits D7:0 */
#if #endian(little)
#define ioaddrw(base,o)	((base)+(o)*4)
#define ioaddrb(base,o)	ioaddrw(base, o)
#define ioaddrh(base,o)	ioaddrw(base, o)
#else
#define ioaddrw(base,o)	((base)+(o)*4)
#define ioaddrb(base,o)	(ioaddrw(base, o) + 3)
#define ioaddrh(base,o)	(ioaddrw(base, o) + 2)
#endif

/* Winbond chip subsystems */
#define UART0_BASE	ioaddrw(COMBI_BASE, 0x3f8)
#define UART1_BASE	ioaddrw(COMBI_BASE, 0x2f8)
#define ECP_BASE	ioaddrw(COMBI_BASE, 0x378)
#define FDC_BASE	ioaddrw(COMBI_BASE, 0x3f0)
#define IDE_BASE	ioaddrw(COMBI_BASE, 0x1f0)

/* NS16550 parameters & (endian independent) register layout */
#define NS16550_HZ	(24000000/13)
#define NSREG(x)	((x)*4)
#ifdef __ASSEMBLER__
#define NS16550_INB(dst,offs,base) \
	lw	dst,offs(base)
#define NS16550_OUTB(src,offs,base) \
	sw	src,offs(base)
#else
#define NS16550_INB(base, offs) \
	*((volatile unsigned int *)((char *)(base) + (offs)))
#define NS16550_OUTB(base, offs, data) \
	do { \
	  *((volatile unsigned int *)((char *)(base) + (offs))) = (data); \
	} while (0)
#endif

/* DRAM control register (8 bits, 1 word per bit) */
#define DCR_SIMM1_DRAM	0x80	/* simm1 is DRAM */
#define DCR_TYPE	0x60	/* simm0/1 type */
#define  DCR_BURSTEDO	 0x60	  /* burst edo */
#define  DCR_EDO	 0x40	  /* normal edo */
#define  DCR_FASTPAGE1	 0x20	  /* fast page mode */
#define  DCR_FASTPAGE2	 0x00	  /* fast page mode */
#define DCR_DRAMFAST	0x10	/* select fast(60ns)/slow(70ns) DRAM timings */
#define DCR_SIMM1_SGL	0x08	/* simm1 is single-sided */
#define DCR_SIMM0_SGL	0x04	/* simm0 is single-sided */
#define DCR_SIMM0_SIZE	0x03	/* simm0 size (x2 if double-sided) */
#define  DCR_4MB	 0x1
#define  DCR_8MB	 0x2
#define  DCR_16MB	 0x3

/* Board option links */
#define OPTION_SIMM1	0x80	/* see DCR_SIMM1_DRAM */
#define OPTION_DRAMTYPE	0x60	/* see DCR_TYPE */
#define OPTION_DRAMFAST	0x10	/* see DCR_DRAMFAST */
#define OPTION_BE	0x08	/* endianness */
#define OPTION_ENVNRST	0x04	/* ~ force env reset */
#define OPTION_USER	0x03
#define OPTION_USER1	0x02
#define OPTION_USER0	0x01

/* Board control register (8 bits, 1 word per bit) */
#define BCR_FLASH_WE	0x80 	/* flash prom write enable */
#define BCR_IO_LE	0x40	/* little-endian i/o system */
#define BCR_ETH_ENABLE	0x20 	/* ethernet chip enable (!reset) */
#define BCR_FDC_TC	0x10 	/* floppy dma terminal count */
#define BCR_SCSI_ENABLE	0x08	/* scsi chip enable (!reset) */
#define BCR_LED_ON	0x04	/* led on (!blank) */
#define BCR_AUTO_BUSY	0x02	/* enable auto busy generation */
#define BCR_V96X_ENABLE	0x01	/* enable V96x pci chip (!reset) */

/* board attributes */
#define _SBD_PANEL

#if !defined(__ASSEMBLER__) && !defined(inb)
/* i/o port access ala 80x86 for ISA bus peripherals */
unsigned char	inb (unsigned int);
unsigned short	inw (unsigned int);
unsigned long	inl (unsigned int);
void		outb (unsigned int, unsigned char);
void		outw (unsigned int, unsigned short);
void		outl (unsigned int, unsigned long);
void *		ioport_map (unsigned int);
#endif

#ifndef __ASSEMBLER__
/* safe programmatic interface to write-only board registers */

unsigned int _dcr_get (void);
unsigned int _dcr_set (unsigned int);
unsigned int _dcr_bis (unsigned int);
unsigned int _dcr_bic (unsigned int);

unsigned int _bcr_get (void);
unsigned int _bcr_set (unsigned int);
unsigned int _bcr_bis (unsigned int);
unsigned int _bcr_bic (unsigned int);

unsigned int _imask_get (void);
unsigned int _imask_set (unsigned int);
unsigned int _imask_bis (unsigned int);
unsigned int _imask_bic (unsigned int);

#ifdef FLASHDEV_OK
extern flashcookie_t	_sbd_fromopen (paddr_t);
extern flashcookie_t	_sbd_fsimmopen (paddr_t);
#endif

#endif
