#ident "kit/p4032/sbdfrom.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * P4032/sbdfrom.c: Flash ROM support for P4032
 */


#ifdef IN_PMON
#include "pmon.h"
#include "sde-compat.h"
#include <assert.h>
#else
#include <sys/types.h>
#include <mips/cpu.h>
#include <stdlib.h>
#include <string.h>
#include "kit_impl.h"
#endif

#include "sbdfrom.h"
#include "flashdev.h"
#define FLASHROM_INLINE
#include "flashrom.h"
#include "sbd.h"

/* writable cookie */
static struct fromcookie flash_cookie;
static struct fromcookie prom_cookie;

/* board-specific code */
static int fromgetparts (flashcookie_t, struct flashparts *);
static int fromsetparts (flashcookie_t, const struct flashparts *);

static const struct flashboard fromboard = {
    fromgetparts,
    fromsetparts
};


flashcookie_t
_sbd_fromopen (paddr_t devaddr)
{
    struct fromcookie *fcp;
    paddr_t base;

    /* Referencing the BOOTPROM could access either the PROM
       socket or the Flash, depending on a board link setting.
       Unfortunately we've no way of telling which way it is set.  

       We also have the problem that we cannot execute code running at
       the explicit flash address: it has to be linked to boot from
       flash. and we cannot program the flash via boot prom addresses.

       All in all pretty useless, but to support debugging of code in
       flash we'll just assume that if we're given an address in the
       bootprom region, then the user has set the board to boot
       from the flash.  */

    if (devaddr == 0
	 || (devaddr >= FLASH_BASE && devaddr < FLASH_BASE + 0x100000)) {
	/* only safe to access via flash space when little endian */
	fcp = &flash_cookie;
	base = FROM_MAPPABLE ? FLASH_BASE : ~0;
    }
    else if (devaddr >= BOOTPROM_BASE && devaddr < BOOTPROM_BASE + 0x100000) {
	/* always safe to access via boot prom */
	fcp = &prom_cookie;
	base = BOOTPROM_BASE;
    }
    else
	return FLASH_UNKNOWN;

    if (fcp->size == 0) {
	/* always program via flash address */
	fcp->pbase = PA_TO_KVA1 (FLASH_BASE);

	/* read from there too, uncached also */
	fcp->rbase = PA_TO_KVA1 (FLASH_BASE);
	
	fcp->nbanks = 1;
	
	/* probe for device */
	if (_flashrom_probe (fcp, 0) < 0)
	    return FLASH_MISSING;
	
	/* compute total size */
	fcp->size = fcp->devsize * FROM_NCOLS;

	/* device may also be mappable */
	fcp->mapbase = base;
    }

    /* check we still fit */
    if ((devaddr & 0xfffff) >= fcp->size)
	return FLASH_MISSING;

    /* plug in board-specific functions */
    fcp->common.board = &fromboard;

    return &fcp->common;
}


/*
 * Return flash partition table 
 */
static int
fromgetparts (flashcookie_t cookie, struct flashparts *parts)
{
    struct fromcookie *fcp = (struct fromcookie *)cookie;

    memset (parts, 0, sizeof (*parts));
    _flashdev_setpart (parts, FLASHPART_RAW, FLASHPART_RAW, 0, fcp->size);
    _flashdev_setpart (parts, FLASHPART_BOOT, FLASHPART_BOOT, 0, 0x68000);
    _flashdev_setpart (parts, FLASHPART_POST, FLASHPART_POST, 
		       0x68000, 0x18000);
    if (fcp->size > 0x800000)
	/* spare partition */
	_flashdev_setpart (parts, FLASHPART_FFS, FLASHPART_UNDEF, 
			   0x80000, fcp->size - 0x80000);

    return FLASHDEV_OK;
}


static int
fromsetparts (flashcookie_t cookie, const struct flashparts *parts)
{
    /* not supported yet, sorry */
    return FLASHDEV_FAIL;
}


/* include code inline since sbdfsimm.c will need a different copy */
#include "../share/flashrom.c"
