/*
 * Copyright (c) 1999-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * p4032/sbdfrom.h: onboard FROM support for Algorithmics P4032
 */


/* Range of device families supported on this board */
#define FROM_FAMILY_ATMEL	1
#define FROM_FAMILY_INTEL	1
#define FROM_FAMILY_AMD		1

/* Fixed device geometry on this board */
#define FROM_WIDTH		1	/* dev width = byte */
#define FROM_NCOLS		1	/* 1 device in parallel */
#define FROM_GROUP		1	/* 1 device per access */

/* "Variable" device geometry */
/*#define FROM_SECSIZE(fcp)*/
/*#define FROM_DEVSIZE(fcp)*/
/*#define FROM_DEVSIZE_LOG2(fcp)*/
#define FROM_NBANKS(fcp)	1	/* 1 sequential bank */

/* Define to enable debug messages */
/*#define FROM_DEBUG*/

/* Define if code (in particular flash programming code) 
   will never executed be from the flash rom. */
/*#define FROM_NEVER_SELF_PROGRAM*/

/* The device is only accessible as normal memory when
   when running little-endian */
#if #endian(little)
#define FROM_MAPPABLE	1
#else
#define FROM_MAPPABLE	0
#endif

/* The device is readable directly without address munging
   when not reading big-endian code. */
#define FROM_READ_CONTIGUOUS(flags) \
	(!((flags) & FLASHDEV_READ_CODE_EB))

/* If required, any special twiddling of the read or programming 
   address (as line or column byte offset) can be done here. */

/* When programming big-endian code into flash we must munge its
   byte-in-word address, to match how the byte-packer operates when
   reading in big-endian mode. */
#define frompline(x,flags) \
    ((x) ^ (((flags) & FLASHDEV_PROG_CODE_EB) ? 3 : 0))

/* Reverse the action when reading */
#define fromrline(x,flags) \
    ((x) ^ (((flags) & FLASHDEV_READ_CODE_EB) ? 3 : 0))

/* No column index munging required (only 1 column per line) */
/*#define fromrcol(x,flags)*/
/*#define frompcol(x,flags)*/
