#ident "kit/p4032/sbdfsimm.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2005 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * P4032/sbdfsimm.c: Flash SIMM support for P4032
 */


#ifdef P4032_FSIMM

#ifdef IN_PMON
#include "pmon.h"
#include "sde-compat.h"
#include <assert.h>
#else
#include <sys/types.h>
#include <mips/cpu.h>
#include <stdlib.h>
#include <string.h>
#include "kit_impl.h"
#endif

#include "sbdfsimm.h"
#include "flashdev.h"
#define FLASHROM_INLINE
#include "flashrom.h"
#include "sbd.h"


/* writable cookie */
static struct fromcookie cookie;

/* board-specific code */
static int fsimmgetparts (flashcookie_t, struct flashparts *);
static int fsimmsetparts (flashcookie_t, const struct flashparts *);

static const struct flashboard fsimmboard = {
    fsimmgetparts,
    fsimmsetparts
};


flashcookie_t
_sbd_fsimmopen (paddr_t devaddr)
{
    struct fromcookie *fcp = &cookie;

    if (!(devaddr == 0 ||
	  (devaddr >= FSIMM_BASE && devaddr < FSIMM_BASE + FSIMM_SIZE)))
	return FLASH_UNKNOWN;

    /* check flash simm link is in, and force single-sided mode */
    if (_dcr_get () & DCR_SIMM1_DRAM)
	return FLASH_MISSING;
    _dcr_bis (DCR_SIMM1_SGL);

    if (fcp->size == 0) {
	/* programming base address may be different from read address */
	fcp->pbase = PA_TO_KVA1 (FSIMM_BASE);
	if (IS_KVA0 (fcp))
	    fcp->rbase = PA_TO_KVA0 (FSIMM_BASE);
	else
	    fcp->rbase = PA_TO_KVA1 (FSIMM_BASE);

	/* probe for device */
	if (_flashrom_probe (fcp, 0) < 0)
	    return FLASH_MISSING;

	/* probe for second bank */
	if (_flashrom_probe (fcp, fcp->devsize * 2) < 0)
	    return FLASH_MISSING;

	fcp->nbanks = 2;
	fcp->size = fcp->devsize * 4;
    }

    /* check we still fit */
    if (devaddr >= FSIMM_BASE + fcp->size)
	return FLASH_MISSING;

    /* plug in board-specific functions */
    fcp->common.board = &fsimmboard;

    return &fcp->common;
}


/*
 * Return flash partition table 
 */
static int
fsimmgetparts (flashcookie_t cookie, struct flashparts *parts)
{
    struct fromcookie *fcp = (struct fromcookie *)cookie;

    memset (parts, 0, sizeof (*parts));
    _flashdev_setpart (parts, FLASHPART_RAW, FLASHPART_RAW, 0, fcp->size);
    _flashdev_setpart (parts, FLASHPART_FFS, FLASHPART_UNDEF, 0, fcp->size);

    return FLASHDEV_OK;
}


static int
fsimmsetparts (flashcookie_t cookie, const struct flashparts *parts)
{
    /* not supported yet, sorry */
    return FLASHDEV_FAIL;
}


/* include code inline since sbdfrom.c will need a different copy */
#include "../share/flashrom.c"

#endif /* P4032_FSIMM */
