#ident "kit/p4032/sbdicu.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * p4032/sbdicu.c: interrupt control unit support
 */


#include <stddef.h>
#include <mips/cpu.h>

#include "sbd.h"
#include "icu.h"

static p4032icu icu_softc;

struct intrmap {
    int	intr_num;
    int intr_shift;
    int	intr_mask;
    enum {Xbar_none, Xbar_dev0, Xbar_dev1, Xbar_pci} intr_xbar;
};

static const struct intrmap intrmap[] = {
    { INTR_BUSERR, 0, 0, Xbar_none },
    { INTR_ACFAIL, 0, 0, Xbar_none },
    { INTR_DEBUG, 0, 0, Xbar_none },
    { INTR_FDCDMA, 0, 0, Xbar_none },

    { INTR_RTC, ICU_DEVX1_RTC, ICU_DEV_RTC, Xbar_dev1 },
    { INTR_GPIO, ICU_DEVX1_GPIO, ICU_DEV_GPIO, Xbar_dev1 },
    { INTR_CENT, ICU_DEVX1_CENT, ICU_DEV_CENT, Xbar_dev1 },
    { INTR_UART1, ICU_DEVX1_UART1, ICU_DEV_UART1, Xbar_dev1 },

    { INTR_UART0, ICU_DEVX0_UART0, ICU_DEV_UART0, Xbar_dev0 },
    { INTR_KBD, ICU_DEVX0_KBD, ICU_DEV_KBD, Xbar_dev0 },
    { INTR_FDC, ICU_DEVX0_FDC, ICU_DEV_FDC, Xbar_dev0 },
    { INTR_V96, ICU_DEVX0_V96, ICU_DEV_V96, Xbar_dev0 },

    { INTR_PCI0, ICU_PCIX_0, ICU_PCI_INT0, Xbar_pci },
    { INTR_PCI1, ICU_PCIX_1, ICU_PCI_INT1, Xbar_pci },
    { INTR_PCI2, ICU_PCIX_2, ICU_PCI_INT2, Xbar_pci },
    { INTR_PCI3, ICU_PCIX_3, ICU_PCI_INT3, Xbar_pci },

};
#define NINTRMAP (sizeof (intrmap) / sizeof (intrmap[0]))

/* map logical interrupt number onto hardware interrupt */
int
_icu_map (int inum, int hwint)
{
    const struct intrmap * mp;
    volatile p4032icu * const hicu = PA_TO_KVA1 (ICU_BASE);
    volatile p4032icu * const sicu = &icu_softc;
    unsigned int mask, omask;

    if (inum < 0 || inum >= INTR_MAX) {
	/* log (LOG_WARNING, "intr_map: illegal interrupt %d\n", inum); */
	return 0;
    }

    /* assume the table is ordered, but don't fail if it is not */
    if (inum < NINTRMAP && intrmap[inum].intr_num == inum)
	mp = &intrmap[inum];
    else {
	/* log (LOG_WARNING, "intr_map: intrmap mismatch for interrupt %d\n", inum); */
	for (mp = intrmap; mp < &intrmap[NINTRMAP]; mp++)
	    if (mp->intr_num == inum)
		break;
	if (mp >= &intrmap[NINTRMAP]) {
	    /* log (LOG_WARNING, "intr_map: bad interrupt %d\n", inum); */
	    return 0;
	}
    }

    switch (mp->intr_xbar) {
    default:
    case Xbar_none:
	break;
    case Xbar_dev0:
	omask = sicu->ctrl.devxbar0;
	mask = (omask & ~(ICU_HWMASK << mp->intr_shift)) |
	    ((hwint & ICU_HWMASK) << mp->intr_shift);
	if (mask != omask) {
	    sicu->ctrl.devxbar0 = mask;
	    hicu->ctrl.devxbar0 = mask;
	}
	break;
    case Xbar_dev1:
	omask = sicu->ctrl.devxbar1;
	mask = (omask & ~(ICU_HWMASK << mp->intr_shift)) |
	    ((hwint & ICU_HWMASK) << mp->intr_shift);
	if (mask != omask) {
	    sicu->ctrl.devxbar1 = mask;
	    hicu->ctrl.devxbar1 = mask;
	}
	break;
    case Xbar_pci:
	omask = sicu->ctrl.pcixbar;
	mask = (omask & ~(ICU_HWMASK << mp->intr_shift)) |
	    ((hwint & ICU_HWMASK) << mp->intr_shift);
	if (mask != omask) {
	    sicu->ctrl.pcixbar = mask;
	    hicu->ctrl.pcixbar = mask;
	}
	break;
    }
    return (1);
}

/* set/reset interrupt mask mask */
int
_icu_mask (int inum, int enable)
{
    const struct intrmap *mp;
    volatile p4032icu * const hicu = PA_TO_KVA1 (ICU_BASE);
    volatile p4032icu * const sicu = &icu_softc;
   unsigned int mask, omask;

    if (inum < 0 || inum >= INTR_MAX) {
	/* log (LOG_WARNING, "intr_enable: illegal interrupt %d\n", inum); */
	return 0;
    }

    /* assume the table is ordered, but don't fail if it is not */
    if (inum < NINTRMAP && intrmap[inum].intr_num == inum)
	mp = &intrmap[inum];
    else {
	/* log (LOG_WARNING, "intr_enable: intrmap mismatch for interrupt %d\n", inum); */
	for (mp = intrmap; mp < &intrmap[NINTRMAP]; mp++)
	    if (mp->intr_num == inum)
		break;
	if (mp >= &intrmap[NINTRMAP]) {
	    /* log (LOG_WARNING, "intr_map: bad interrupt %d\n", inum); */
	    return 0;
	}
    }

    switch (mp->intr_xbar) {
    default:
    case Xbar_none:
	break;
    case Xbar_dev0:
    case Xbar_dev1:
	omask = sicu->ctrl.mask;
	if (enable)
	    mask = omask | mp->intr_mask;
	else
	    mask = omask & ~mp->intr_mask;
	if (mask != omask) {
	    sicu->ctrl.mask = mask;
	    hicu->ctrl.mask = mask;
	}
	break;
    case Xbar_pci:
	omask = sicu->ctrl.pcimask;
	if (enable)
	    mask = omask | mp->intr_mask;
	else
	    mask = omask & ~mp->intr_mask;
	if (mask != omask) {
	    sicu->ctrl.pcimask = mask;
	    hicu->ctrl.pcimask = mask;
	}
	break;
    }
    return (1);
}

/* return active device interrupts */
int
_icu_devintr ()
{
    volatile p4032icu * const hicu = PA_TO_KVA1 (ICU_BASE);
    volatile p4032icu * const sicu = &icu_softc;
    return (hicu->irr.dev & sicu->ctrl.mask);
}

/* return active PCI interrupts */
int
_icu_pciintr ()
{
    volatile p4032icu * const hicu = PA_TO_KVA1 (ICU_BASE);
    volatile p4032icu * const sicu = &icu_softc;
    return (hicu->irr.pci & sicu->ctrl.pcimask);
}


#define XBAR(xbarshift,xbarval) \
	((ICU_##xbarval)<<(ICU_##xbarshift))
 
void
_icu_init ()
{
  volatile p4032icu * const icu = PA_TO_KVA1 (ICU_BASE);
  static int icuinitialised = 0;

  if (icuinitialised)
      return;

  /* intialise icu_softc */
  icu_softc.ctrl.mask = 0;
  icu_softc.ctrl.pcimask = ICU_PCI_FDCDMA;
  icu_softc.ctrl.devxbar0 = XBAR(DEVX0_V96,HWNONE) | XBAR(DEVX0_FDC,HWNONE) |
			    XBAR(DEVX0_KBD,HWNONE) | XBAR(DEVX0_UART0,HWNONE);
  icu_softc.ctrl.devxbar1 = XBAR(DEVX1_UART1,HWNONE) | XBAR(DEVX1_CENT,HWNONE) |
			    XBAR(DEVX1_GPIO,HWNONE) | XBAR(DEVX1_RTC,HWNONE);
  icu_softc.ctrl.pcixbar =  XBAR(PCIX_0,HWNONE) | XBAR(PCIX_1,HWNONE) |
			    XBAR(PCIX_2,HWNONE) | XBAR(PCIX_3,HWNONE);

  /* copy soft copy into hardware icu */
  icu->ctrl.mask = icu_softc.ctrl.mask;
  icu->ctrl.clear = ICU_ERR_BUSERR | ICU_ERR_ACFAIL | ICU_ERR_DEBUG;
  icu->ctrl.pcimask = icu_softc.ctrl.pcimask;
  icu->ctrl.devxbar0 = icu_softc.ctrl.devxbar0;
  icu->ctrl.devxbar1 = icu_softc.ctrl.devxbar1;
  icu->ctrl.pcixbar = icu_softc.ctrl.pcixbar;

  icuinitialised = 1;

}

#include <sys/init.h>

DECL_INIT_FUNC (IF_DEVICE, _icu_init)
