#ident "kit/p4032/sbdpio.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * P4032/sbdpio.c: low-level interface to P4032 PIO chip
 */


#include <stddef.h>
#include <mips/cpu.h>

#include "sbd.h"
#include "z80pio.h"


/* two uncached reads give pio time to stop driving the i/o bus */
#define ZPIO_DELAY() \
  do { \
    (void) *(volatile unsigned char *) PA_TO_KVA1 (0); \
    (void) *(volatile unsigned char *) PA_TO_KVA1 (0); \
  } while (0);


unsigned int 
_zpiob_get (void)
{
    volatile zpiodev * const zpio = PA_TO_KVA1 (ZPIO_BASE);
    unsigned char val = zpio->b_dat;
    ZPIO_DELAY();
    return val;
}


unsigned int 
_zpiob_set (unsigned int new)
{
    volatile zpiodev * const zpio = PA_TO_KVA1 (ZPIO_BASE);
    unsigned char old = zpio->b_dat;
    ZPIO_DELAY();
    zpio->b_dat = new;
    ZPIO_DELAY();
    return old;
}


unsigned int 
_zpiob_bic (unsigned int bits)
{
    volatile zpiodev * const zpio = PA_TO_KVA1 (ZPIO_BASE);
    unsigned char old = zpio->b_dat;
    ZPIO_DELAY();
    zpio->b_dat = old & ~bits;
    ZPIO_DELAY();
    return old;
}


unsigned int 
_zpiob_bis (unsigned int bits)
{
    volatile zpiodev * const zpio = PA_TO_KVA1 (ZPIO_BASE);
    unsigned char old = zpio->b_dat;
    ZPIO_DELAY();
    zpio->b_dat = old | bits;
    ZPIO_DELAY();
    return old;
}



void 
_zpioa_setctl (unsigned int ctl)
{
    volatile zpiodev * const zpio = PA_TO_KVA1 (ZPIO_BASE);
    zpio->a_ctl = ctl;
    ZPIO_DELAY();
}

unsigned int 
_zpioa_get (void)
{
    volatile zpiodev * const zpio = PA_TO_KVA1 (ZPIO_BASE);
    unsigned char val = zpio->a_dat;
    ZPIO_DELAY();
    return val;
}

unsigned int 
_zpioa_set (unsigned int new)
{
    volatile zpiodev * const zpio = PA_TO_KVA1 (ZPIO_BASE);
    unsigned char old = zpio->a_dat;
    ZPIO_DELAY();
    zpio->a_dat = new;
    ZPIO_DELAY();
    return old;
}

unsigned int 
_zpioa_bic (unsigned int bits)
{
    volatile zpiodev * const zpio = PA_TO_KVA1 (ZPIO_BASE);
    unsigned char old = zpio->a_dat;
    ZPIO_DELAY();
    zpio->a_dat = old & ~bits;
    ZPIO_DELAY();
    return old;
}


unsigned int 
_zpioa_bis (unsigned int bits)
{
    volatile zpiodev * const zpio = PA_TO_KVA1 (ZPIO_BASE);
    unsigned char old = zpio->a_dat;
    ZPIO_DELAY();
    zpio->a_dat = old | bits;
    ZPIO_DELAY();
    return old;
}


/* generate a fake Z80 M1 signal */
void
_zpio_m1 (void)
{
    volatile unsigned int * const zpiom1 = PA_TO_KVA1 (ZPIO_IACK);
    (void) *zpiom1;
    ZPIO_DELAY();
}
