#ident "kit/p4032/sbdrtc.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * P4032/sbdrtc.c: low-level interface to P4032 real-time clock chip
 */


#include <stddef.h>
#include <mips/cpu.h>

#include "sbd.h"
#include "rtc.h"

static volatile unsigned int * const rtcaddr = PA_TO_KVA1 (RTC_BASE+RTC_ADDR);
static volatile unsigned int * const rtcdata = PA_TO_KVA1 (RTC_BASE+RTC_DATA);

unsigned int
_rtc_get (int reg)
{
    *rtcaddr = reg;
    return (*rtcdata & 0xff);
}


unsigned int
_rtc_set (int reg, unsigned int val)
{
    unsigned int o;
    *rtcaddr = reg;
    o = *rtcdata & 0xff;
    if (val != o)
	*rtcdata = val;
    return (o);
}

unsigned int
_rtc_bis (int reg, unsigned int val)
{
    unsigned int o, n;

    *rtcaddr = reg;
    o = *rtcdata & 0xff;
    n = o | val;
    if (n != o)
	*rtcdata = n;
    return (o);
}


unsigned int
_rtc_bic (int reg, unsigned int val)
{
    unsigned int o, n;

    *rtcaddr = reg;
    o = *rtcdata & 0xff;
    n = o & ~val;
    if (n != o)
	*rtcdata = n;
    return (o);
}
