/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * z80pio.h: Z80 parallel i/o chip register definitions
 */


#ifndef __ASSEMBLER__
typedef struct {
    unsigned int	a_dat;
    unsigned int	b_dat;
    unsigned int	a_ctl;
    unsigned int	b_ctl;
} zpiodev;
#endif

/* register offsets */
#define ZPIO_A_DAT	0
#define ZPIO_B_DAT	4
#define ZPIO_A_CTL	8
#define ZPIO_B_CTL	12

#define ZPIO_MODE_OUT	0x0f
#define ZPIO_MODE_IN	0x4f
#define ZPIO_MODE_BIDIR	0x8f
#define ZPIO_MODE_CTRL	0xcf

#define ZPIO_ICW_ENABLE	0x80
#define ZPIO_ICW_AND	0x40
#define ZPIO_ICW_HIGH	0x20
#define ZPIO_ICW_MASK	0x10
#define ZPIO_ICW_SET	0x07

#define ZPIO_INT_ENB	0x83
#define ZPIO_INT_DIS	0x03

#define ZPIOB_E2_DO	0x80	/* in:  EEPROM data out */
#define ZPIOB_E2_SK	0x40	/* out: EEPROM clock */
#define ZPIOB_E2_CS	0x20	/* out: EEPROM chip select */
#define ZPIOB_E2_DI	0x10	/* out: EEPROM data in */
#define ZPIOB_FPGA_DMASK 0x0c	/* out: FPGA JTAG data mask */
#define ZPIOB_FPGA_DSHFT 2	/* out: FPGA JTAG data shift */
#define ZPIOB_FPGA_TCK	0x02	/* out: FPGA JTAG clock */
#define ZPIOB_NFAULT	0x01	/* out: Centronics peripheral mode nFault */

#ifndef __ASSEMBLER__
/* safe programmatic interface to pio */

unsigned int _zpiob_get (void);
unsigned int _zpiob_set (unsigned int);
unsigned int _zpiob_bis (unsigned int);
unsigned int _zpiob_bic (unsigned int);

void _zpioa_setctl (unsigned int);
unsigned int _zpioa_get (void);
unsigned int _zpioa_set (unsigned int);
unsigned int _zpioa_bis (unsigned int);
unsigned int _zpioa_bic (unsigned int);

void _zpio_m1 (void);
#endif
