/*
 * Copyright (c) 1997-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * p5064/from.h: onboard FLASH ROM support for P5064 board
 */


/* The sequence of addresses used to control the devices changes
 * slightly between the 4Mb and 8Mb devices 
 */
#define UNLOCK1_4Mb_OFFS	0xaaaa
#define UNLOCK1_8Mb_OFFS	0x5555
#define UNLOCK1_DATA		0xaa

#define UNLOCK2_4Mb_OFFS	0x5555
#define UNLOCK2_8Mb_OFFS	0x2aaa
#define UNLOCK2_DATA		0x55

/* Command codes are the same for all */
#define FROM_RESET		0xf0
#define FROM_AUTOSELECT 	0x90
#define FROM_PROGRAM		0xa0
#define FROM_ERASE		0x80
#define FROM_ERASECHIP		0x10
#define FROM_ERASESECT		0x30

/* Manufacturer codes */
#define MAN_AMD			0x01
#define MAN_FUJITSU		0x04
#define MAN_ATMEL		0x1f
#define MAN_MICRON		0x2c
#define MAN_INTEL		0x89

/* data bit definitions for "embedded programming algorithm" */
#define DQPOLL			0x80
#define DQTOGGLE		0x40
#define DQTIMEEXCEEDED		0x20
#define DQSECTERASETIMER 	0x08

#define FROMMAXSIZE		(0x100000)	/* 1MB max flash size */
#define FROMMAXSECSIZE		(0x010000)	/* 64KB max sector size */
#define FROMMAXSECTORS		19

/* P5064 flash roms are all byte wide */
typedef unsigned char from_t;
