#ident "kit/p5064/sbdapc.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1998-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * P5064/sbdrtc.c: low-level interface to P5064 APC module
 */


#include <stddef.h>
#include <mips/cpu.h>

#include "sbd.h"
#include "rtc.h"
#include "apc.h"

unsigned int
_apc_get (int reg)
{
    unsigned int rtcsa, v;

    if (BOARD_REVISION < 'C')
	return ~0;

    outb (RTC_ADDR_PORT, RTC_STATUSA);
    rtcsa = inb (RTC_DATA_PORT);
    if ((rtcsa & RTCSA_DVMASK) != RTC_DV2_OSC_ON)
	outb (RTC_DATA_PORT, (rtcsa & ~RTCSA_DVMASK) | RTC_DV2_OSC_ON);

    outb (RTC_ADDR_PORT, reg);
    v = inb (RTC_DATA_PORT);

    /* paranoia - switch back to bank 0 */
    outb (RTC_ADDR_PORT, RTC_STATUSA);
    outb (RTC_DATA_PORT, (rtcsa & ~RTCSA_DVMASK) | RTC_DV0_OSC_ON);

    return (v);
}

unsigned int
_apc_set (int reg, unsigned int val)
{
    unsigned int rtcsa;
    unsigned int o;

    if (BOARD_REVISION < 'C')
	return ~0;

    outb (RTC_ADDR_PORT, RTC_STATUSA);
    rtcsa = inb (RTC_DATA_PORT);
    if ((rtcsa & RTCSA_DVMASK) != RTC_DV2_OSC_ON)
	outb (RTC_DATA_PORT, (rtcsa & ~RTCSA_DVMASK) | RTC_DV2_OSC_ON);

    outb (RTC_ADDR_PORT, reg);
    o = inb (RTC_DATA_PORT);
    if (val != o)
	outb (RTC_DATA_PORT, val);

    /* paranoia - switch back to bank 0 */
    outb (RTC_ADDR_PORT, RTC_STATUSA);
    outb (RTC_DATA_PORT, (rtcsa & ~RTCSA_DVMASK) | RTC_DV0_OSC_ON);

    return (o);
}


unsigned int
_apc_bis (int reg, unsigned int val)
{
    unsigned int rtcsa, o, n;

    if (BOARD_REVISION < 'C')
	return ~0;

    outb (RTC_ADDR_PORT, RTC_STATUSA);
    rtcsa = inb (RTC_DATA_PORT);
    if ((rtcsa & RTCSA_DVMASK) != RTC_DV2_OSC_ON)
	outb (RTC_DATA_PORT, (rtcsa & ~RTCSA_DVMASK) | RTC_DV2_OSC_ON);

    outb (RTC_ADDR_PORT, reg);
    o = inb (RTC_DATA_PORT);
    n = o | val;
    if (o != n)
	outb (RTC_DATA_PORT, n);

    /* paranoia - switch back to bank 0 */
    outb (RTC_ADDR_PORT, RTC_STATUSA);
    outb (RTC_DATA_PORT, (rtcsa & ~RTCSA_DVMASK) | RTC_DV0_OSC_ON);

    return (o);
}

unsigned int
_apc_bic (int reg, unsigned int val)
{
    unsigned int rtcsa, o, n;

    if (BOARD_REVISION < 'C')
	return ~0;

    outb (RTC_ADDR_PORT, RTC_STATUSA);
    rtcsa = inb (RTC_DATA_PORT);
    if ((rtcsa & RTCSA_DVMASK) != RTC_DV2_OSC_ON)
	outb (RTC_DATA_PORT, (rtcsa & ~RTCSA_DVMASK) | RTC_DV2_OSC_ON);

    outb (RTC_ADDR_PORT, reg);
    o = inb (RTC_DATA_PORT);
    n = o & ~val;
    if (o != n)
	outb (RTC_DATA_PORT, n);

    /* paranoia - switch back to bank 0 */
    outb (RTC_ADDR_PORT, RTC_STATUSA);
    outb (RTC_DATA_PORT, (rtcsa & ~RTCSA_DVMASK) | RTC_DV0_OSC_ON);

    return (o);
}
