#ident "kit/p5064/sbdpio.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1997-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * P5064/sbdpio.c: low-level interface to PIO chips
 */


#include <stddef.h>
#include <mips/cpu.h>
#include <sys/init.h>
#include "sbd.h"
#include "z80pio.h"
#include "gpio.h"
#include "pio.h"

struct piofunctions {
    void		(*pioa_dir)(unsigned int);
    unsigned int	(*pioa_get)(void);
    unsigned int	(*pioa_set)(unsigned int);
    unsigned int	(*pioa_bic)(unsigned int);
    unsigned int	(*pioa_bis)(unsigned int);
    void		(*piob_dir)(unsigned int);
    unsigned int	(*piob_get)(void);
    unsigned int	(*piob_set)(unsigned int);
    unsigned int	(*piob_bic)(unsigned int);
    unsigned int	(*piob_bis)(unsigned int);
};

static struct piofunctions pio_revab = {
    _zpioa_dir,
    _zpioa_get,
    _zpioa_set,
    _zpioa_bic,
    _zpioa_bis,
    _zpiob_dir,
    _zpiob_get,
    _zpiob_set,
    _zpiob_bic,
    _zpiob_bis
};

static struct piofunctions pio_revc = {
    _gpioa_dir,
    _gpioa_get,
    _gpioa_set,
    _gpioa_bic,
    _gpioa_bis,
    _gpiob_dir,
    _gpiob_get,
    _gpiob_set,
    _gpiob_bic,
    _gpiob_bis
};

static struct piofunctions *pio;

void 
_pioa_dir (unsigned int outputs)
{
    (*pio->pioa_dir)(outputs);
}

unsigned int 
_pioa_get (void)
{
    return (*pio->pioa_get)();
}

unsigned int 
_pioa_set (unsigned int new)
{
    return (*pio->pioa_set)(new);
}

unsigned int 
_pioa_bic (unsigned int bits)
{
    return (*pio->pioa_bic)(bits);
}


unsigned int 
_pioa_bis (unsigned int bits)
{
    return (*pio->pioa_bis)(bits);
}

void 
_piob_dir (unsigned int outputs)
{
    (*pio->piob_dir)(outputs);
}

unsigned int 
_piob_get (void)
{
    return (*pio->piob_get)();
}

unsigned int 
_piob_set (unsigned int new)
{
    return (*pio->piob_set)(new);
}

unsigned int 
_piob_bic (unsigned int bits)
{
    return (*pio->piob_bic)(bits);
}


unsigned int 
_piob_bis (unsigned int bits)
{
    return (*pio->piob_bis)(bits);
}

static void
_pio_init (void)
{
    if (BOARD_REVISION >= 'C')
	pio = &pio_revc;
    else
	pio = &pio_revab;
}

DECL_INIT_FUNC (IF_DRIVER, _pio_init)
