#ident "kit/p6032/sbd.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2000-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * p6032/sbd.c: miscellaneous low-level board functions 
 */


#include <stddef.h>
#include <mips/cpu.h>
#include "sbd.h"
#include "i82371eb.h"

unsigned int
_sbd_pcbrev(void)
{
    unsigned int gpi = (inb (PM_PORT + I82371_PM_GPIREG2) << 16)
	| (inb (PM_PORT + I82371_PM_GPIREG1) << 8)
	| (inb (PM_PORT + I82371_PM_GPIREG0));
    /* printf ("sbd_pcbrev: gpi=0x%x, pcbrev=%d\n", gpi, (gpi >> 17) & 7); */
    return (gpi >> 17) & 7;
}

unsigned int
_sbd_modrev(void)
{
    unsigned int gpi = (inb (PM_PORT + I82371_PM_GPIREG2) << 16)
	| (inb (PM_PORT + I82371_PM_GPIREG1) << 8)
	| (inb (PM_PORT + I82371_PM_GPIREG0));
    /* printf ("sbd_modrev: gpi=0x%x, pcbrev=%d\n", gpi, (gpi >> 13) & 15); */
    return (gpi >> 13) & 15;
}

unsigned int
_sbd_boardtype (void)
{
    return (_sbd_pcbrev() & 4) ? 64 : 32;
}

unsigned int
_sbd_switches (void)
{
    unsigned short cpldrev = *(unsigned short *)(PA_TO_KVA1(CPLD_BASE + CPLD_SWITCHES));
    if (cpldrev < 2)
	return *(unsigned short *)(PA_TO_KVA1(CPLD_BASE + CPLD_SWITCHES_PREREV2));
    else
	return *(unsigned short *)(PA_TO_KVA1(CPLD_BASE + CPLD_SWITCHES));
}


void *
ioport_map (unsigned int port)
{
    return PA_TO_KVA1 (ISAPORT_BASE(port));
}


unsigned char
inb (unsigned int port)
{
#if #endian(big) && defined(P6032)
    return *(volatile unsigned char *) PA_TO_KVA1 (ISAPORT_BASE(port^3));
#else
    return *(volatile unsigned char *) PA_TO_KVA1 (ISAPORT_BASE(port));
#endif
}

unsigned short
inw (unsigned int port)
{
#if #endian(big) && defined(P6032)
    return *(volatile unsigned short *) PA_TO_KVA1 (ISAPORT_BASE(port^2));
#else
    return *(volatile unsigned short *) PA_TO_KVA1 (ISAPORT_BASE(port));
#endif
}

unsigned long
inl (unsigned int port)
{
    return *(volatile unsigned long *) PA_TO_KVA1 (ISAPORT_BASE(port));
}

void
outb (unsigned int port, unsigned char val)
{
#if #endian(big) && defined(P6032)
    *(volatile unsigned char *) PA_TO_KVA1 (ISAPORT_BASE(port)^3) = val;
#else
    *(volatile unsigned char *) PA_TO_KVA1 (ISAPORT_BASE(port)) = val;
#endif
    mips_wbflush ();
}

void
outw (unsigned int port, unsigned short val)
{
#if #endian(big) && defined(P6032)
    *(volatile unsigned short *) PA_TO_KVA1 (ISAPORT_BASE(port^2)) = val;
#else
    *(volatile unsigned short *) PA_TO_KVA1 (ISAPORT_BASE(port)) = val;
#endif
    mips_wbflush ();
}

void
outl (unsigned int port, unsigned long val)
{
    *(volatile unsigned long *) PA_TO_KVA1 (ISAPORT_BASE(port)) = val;
    mips_wbflush ();
}
