/*
 * Copyright (c) 2000-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * p6032/sbdicu.h: interrupn numbering for P-6032 board
 */


/* MIPS RTPX specials */
#define INTR_CLOCK	-1
#define INTR_NONE	0

#define INTR_IS_CPU(n)		(((n) & 0x30) == 0)
#define INTR_IS_ISA(n)		(((n) & 0x30) == 0x10)
#define INTR_IS_ISA_MASTER(n)	(((n) & 0x38) == 0x10)
#define INTR_IS_ISA_SLAVE(n)	(((n) & 0x38) == 0x18)
#define INTR_IS_BONITO(n)	(((n) & 0x20) != 0)

/* CPU interrupts */
#define INTR_SINT0	8
#define INTR_SINT1	9
#define INTR_BONITO0	10
#define INTR_BONITO1	11
#define INTR_HINT2      12
#define INTR_DEBUG	13
#define INTR_HINT4	14
#define INTR_TIMER	15

/* i8259 master interrupts */
#define INTR_ISAIRQ0	16
#define INTR_ISAIRQ1	17
#define INTR_ISAIRQ2	18
#define INTR_ISAIRQ3	19
#define INTR_ISAIRQ4	20
#define INTR_ISAIRQ5	21
#define INTR_ISAIRQ6	22
#define INTR_ISAIRQ7	23

/* i8259 slave interrupts */
#define INTR_ISAIRQ8	24
#define INTR_ISAIRQ9	25
#define INTR_ISAIRQ10	26
#define INTR_ISAIRQ11	27
#define INTR_ISAIRQ12	28
#define INTR_ISAIRQ13	29
#define INTR_ISAIRQ14	30
#define INTR_ISAIRQ15	31

/* conventional ISA usage */
#define INTR_PTIMER	INTR_ISAIRQ0
#define INTR_KEYBD	INTR_ISAIRQ1
#define INTR_ISASLAVE	INTR_ISAIRQ2
#define INTR_COM2	INTR_ISAIRQ3
#define INTR_COM1	INTR_ISAIRQ4
#define INTR_LPT1	INTR_ISAIRQ5
#define INTR_FDC	INTR_ISAIRQ6
#define INTR_LPT0	INTR_ISAIRQ7
#define INTR_RTC	INTR_ISAIRQ8
#define INTR_IDE0	INTR_ISAIRQ14
#define INTR_IDE1	INTR_ISAIRQ15

/* Bonito interrupts */
#define INTR_MBOX0	32
#define INTR_MBOX1	33
#define INTR_MBOX2	34
#define INTR_MBOX3	35
#define INTR_DMARDY	36
#define INTR_DMAEMPTY	37
#define INTR_COPYRDY	38
#define INTR_COPYEMPTY	39
#define INTR_COPYERR	40
#define INTR_PCIIRQ	41
#define INTR_MASTERERR	42
#define INTR_SYSTEMERR	43
#define INTR_DRAMPERR	44
#define INTR_RETRYERR	45
#define INTR_GPIO0	48
#define INTR_GPIO1	49
#define INTR_GPIO2	50
#define INTR_GPIO3	51
#define INTR_GPIN0	57
#define INTR_GPIN1	58
#define INTR_GPIN2	59
#define INTR_GPIN3	60
#define INTR_GPIN4	61
#define INTR_GPIN5	62
#define INTR_MAX	63

/* P-6032 specific wiring */
#define INTR_PCI_A	INTR_GPIO0	/* PCI IRQA */
#define INTR_PCI_B	INTR_GPIO1	/* PCI IRQB */
#define INTR_PCI_C	INTR_GPIO2	/* PCI IRQC */
#define INTR_PCI_D	INTR_GPIO3	/* PCI IRQD */
#define INTR_ISANMI	INTR_GPIN0	/* ISA NMI signal */
#define INTR_ISAMASTER	INTR_GPIN1	/* ISA i8259 master */
#define INTR_ETH	INTR_GPIN2	/* Onboard ethernet */
#define INTR_BONIDE	INTR_GPIN3	/* Bonito IDE */
#define INTR_UART1	INTR_GPIN4	/* ISA IRQ3 = COM2 = UART1 */
#define INTR_UART0	INTR_GPIN5	/* ISA IRQ4 = COM1 = UART0 */
