#ident "kit/p6032/sbdrtc.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2000-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * p6032/sbdrtc.c: low-level interface to real-time clock chip
 */


#ifdef IN_PMON
#include <mips.h>
#include <pmon.h>
#else
#include <stddef.h>
#include <mips/cpu.h>
#endif

#include "sbd.h"
#include "rtc.h"

static void
setaddress(int reg)
{
    unsigned char rtca;

    /* PC97307 RTC */
    if (reg < 0x40)
	outb(RTC_ADDR_PORT, reg);	/* accessible from any bank */
    else {
	outb (RTC_ADDR_PORT, RTC_STATUSA);
	rtca = inb (RTC_DATA_PORT);
	if (reg < 0x80) {
	    /* accessible from bank 0 */
	    if ((rtca & RTCSA_DVMASK) != RTC_DV0_OSC_ON)
		outb (RTC_DATA_PORT, (rtca & ~RTCSA_DVMASK) | RTC_DV0_OSC_ON);
	    outb (RTC_ADDR_PORT, reg);
	}
	else {
	    /* accessible indirectly via bank 1 */
	    if ((rtca & RTCSA_DVMASK) != RTC_DV1_OSC_ON)
		outb (RTC_DATA_PORT, (rtca & ~RTCSA_DVMASK) | RTC_DV1_OSC_ON);
	    outb (RTC_ADDR_PORT, RTC_BANK1_URADDR);
	    outb (RTC_DATA_PORT, reg - 0x80);
	    outb (RTC_ADDR_PORT, RTC_BANK1_URDATA);
	}
    }
}

unsigned int
_rtc_get (int reg)
{
    setaddress (reg);
    return inb (RTC_DATA_PORT);
}

unsigned int
_rtc_set (int reg, unsigned int val)
{
    unsigned int o;

    setaddress (reg);
    o = inb (RTC_DATA_PORT);
    if (val != o)
	outb (RTC_DATA_PORT, val);
    return (o);
}

unsigned int
_rtc_bis (int reg, unsigned int val)
{
    unsigned int o, n;

    setaddress (reg);
    o = inb (RTC_DATA_PORT);
    n = o | val;
    if (n != o)
	outb (RTC_DATA_PORT, n);
    return (o);
}

unsigned int
_rtc_bic (int reg, unsigned int val)
{
    unsigned int o, n;

    setaddress (reg);
    o = inb (RTC_DATA_PORT);
    n = o & ~val;
    if (n != o)
	outb (RTC_DATA_PORT, n);
    return (o);
}
