#ident "kit/p6032/sbdsmb.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2000-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * p6032/sbdsmb.c: low level reset code for P-6032
 */

#include <sys/types.h>
#include <mips/cpu.h>

#include "sbd.h"
#include "smb.h"

#include "i82371eb.h"

#define smbinb(r) inb(__CONCAT(I82371_SMB_SMB,r))
#define smboutb(r,v) outb(__CONCAT(I82371_SMB_SMB,r),v)

static int
smb_ready(void)
{
    /* host busy ? */
    if (smbinb(HSTSTS) & I82371_SMB_HOST_BUSY)
	return 0;
    /* slave busy ? (shouldn't happen) */
    if (smbinb(SLVSTS) & I82371_SMB_SLV_BSY)
	return 0;

    /* reset interrupts */
    smboutb(HSTSTS, I82371_SMB_FAILED | I82371_SMB_BUS_ERR |
	    I82371_SMB_DEV_ERR | I82371_SMB_INTER);

    return 1;
}

static void
smb_start(unsigned int protocol)
{
    unsigned int cnt;
    /* clears the index pointer */
    cnt = smbinb(HSTCNT);
    /* set protocol and start */
    smboutb(HSTCNT, (cnt & 0xa0) | protocol | I82371_SMB_START);
}

#define SMB_ALLINTS (I82371_SMB_FAILED|I82371_SMB_BUS_ERR|I82371_SMB_DEV_ERR|I82371_SMB_INTER)

static int
smb_wait (void)
{
    unsigned int status;

    /*
     * There is latency on the HOST_BUSY bit, so monitor the interrupt
     * lines instead
     */

    do {
	status = smbinb(HSTSTS);
    } while ((status & SMB_ALLINTS) == 0); 

    smboutb(HSTSTS, status & SMB_ALLINTS);

    if (status & I82371_SMB_INTER)
	return 1;

    return 0;

}


int
_sbd_smbread(int addr, int protocol, int cmd, void *buf, int *len)
{
    unsigned char *bp = buf;
    int i;
    if (!smb_ready ())
	return 0;

    smboutb(HSTADD, addr | 1);	/* host address */
    smboutb(HSTCMD, cmd);	/* command byte */

    smb_start (protocol);
    if (smb_wait () == 0)
	return 0;

    switch (protocol) {
    case I82371_SMB_QRW:
	break;
    case I82371_SMB_BRW:
    case I82371_SMB_BDRW:
	*len = 1;
	*bp = smbinb(HSTDAT0);
	break;
    case I82371_SMB_WDRW:
	*len = 2;
	/* FIXME: endianess? */
	*bp++ = smbinb(HSTDAT0);
	*bp = smbinb(HSTDAT1);
	break;
    case I82371_SMB_BKRW:
	*len = smbinb(HSTDAT0);
	for (i = *len; i; i--)
	    *bp++ = smbinb(BLKDAT);
	break;
    }
    return 1;
}

int
_sbd_smbwrite(int addr, int protocol, int cmd, const void *buf, int len)
{
    const unsigned char *bp = buf;
    int i;
    if (!smb_ready ())
	return 0;

    smboutb(HSTADD, addr & ~1);	/* host address */
    smboutb(HSTCMD, cmd);		/* command byte */

    switch (protocol) {
    case I82371_SMB_QRW:
	break;
    case I82371_SMB_BRW:
    case I82371_SMB_BDRW:
	smboutb(HSTDAT0, *bp);
	break;
    case I82371_SMB_WDRW:
	/* FIXME: endianess ? */
	smboutb(HSTDAT0, *bp++);
	smboutb(HSTDAT1, *bp);
	break;
    case I82371_SMB_BKRW:
	smboutb(HSTDAT0, len);
	for (i = len; i; i--)
	    smboutb(BLKDAT, *bp++);
	break;
    }

    smb_start (protocol);
    return smb_wait ();
}
