/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * MIPS SDE kit pmon interface definitions
 */


/* this is the structure passed to PMON's onintr() function */
struct pmonhandler {
    struct pmonhandler *prev;
    void	(*addr)();
    int		enabled;	/* this is our own flag */
};


extern int pmon_open (const char *name, int flags);
extern int pmon_read (int fd, void *buf, int);
extern int pmon_write (int fd, const void *buf, int);
extern int pmon_ioctl (int fd, int cmd, void *arg);
extern int pmon_close (int fd);
extern void pmon_onintr (int xcptno, struct pmonhandler *h);

extern char *pmon_getenv (const char *s);

/* pmon_ioctl opcodes */
#define PMON_TCGETA	1
#define PMON_TCSETAF	2
#define PMON_TCSETAW	3
#define PMON_SETINTR	4
#define PMON_SETSANE	5
#define PMON_FIONREAD	6
#define PMON_GETINTR	7
#define PMON_SETNCNE	10
#define PMON_CBREAK	11

/* iflags */
#define PMON_ISTRIP	0x0020
#define PMON_ICRNL	0x0040
#define PMON_IXON	0x0400
#define PMON_IXANY	0x0800
#define PMON_IXOFF	0x1000

/* oflags */
#define PMON_ONLCR	0x0004

/* lflags */
#define PMON_ISIG	0x0001
#define PMON_ICANON	0x0002
#define PMON_ECHO	0x0008
#define PMON_ECHOE	0x0010

/* cflags */
#define	PMON_CBAUD	0000017		
#define	PMON_B0		0000000
#define	PMON_B50	0000001	
#define	PMON_B75	0000002
#define	PMON_B110	0000003
#define	PMON_B134	0000004
#define	PMON_B150	0000005
#define	PMON_B200	0000006
#define	PMON_B300	0000007
#define	PMON_B600	0000010
#define	PMON_B1200	0000011
#define	PMON_B1800	0000012
#define	PMON_B2400	0000013
#define	PMON_B4800	0000014
#define	PMON_B9600	0000015
#define	PMON_B19200	0000016
#define	PMON_B38400	0000017

/* cc definitions */
#define PMON_VINTR	0
#define PMON_VERASE	2
#define PMON_VEOL	5
#define PMON_VEOL2	6
#define PMON_VSTART	8
#define PMON_VSTOP	9
#define PMON_NCC 	23

struct pmontermio {
    unsigned short	c_iflag;
    unsigned short	c_oflag;
    unsigned short	c_cflag;
    unsigned short	c_lflag;
    unsigned char	c_cc[PMON_NCC];
};
