#ident "kit/pmon/pmonroot.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * pmonroot.c: a root for an emulated Posix-like file system under LSI PMON
 */


#define _KERNEL
#include <sys/types.h>
#include <sys/stat.h>
#include <sdethread.h>
#include "file.h"

/* external file systems */
extern const struct fileops _mfs_ops;
#ifdef INET
extern const struct fileops _tftp_ops;
#endif

/* global lock */
__sdethread_mutex_t _pmon_lock = __SDETHREAD_MUTEX_INITIALIZER("pmon");

/* forward declarations */
static struct dirnode dev_dir[];
static struct dirnode root_dir[];

/* directories:	drwxr-xr-x */
#define DIRMODE	(S_IFDIR|S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH)

/* /dev/tty*:	crw-rw-r-- */
#define TTYMODE	(S_IFCHR|S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IWOTH)


/* _root must be global, everything else is static */
struct dirnode _root = 
    DIRDESC(0,		0, DIRMODE, 0, _mfs_ops, root_dir);

static struct dirnode root_dir[] = {
    DIRDESC(".",	0, DIRMODE, 0, _mfs_ops, root_dir),
    DIRDESC("..",	0, DIRMODE, 0, _mfs_ops, root_dir),
    DIRDESC("dev",	0, DIRMODE, 0, _mfs_ops, dev_dir),
#ifdef INET
    DIRDESC("tftp",	0, DIRMODE, 0, _tftp_ops, 0),
#endif
    {0}
};

static struct dirnode dev_dir[] = {
    DIRDESC(".",	0, DIRMODE, 0, _mfs_ops, dev_dir),
    DIRDESC("..",	0, DIRMODE, 0, _mfs_ops, root_dir),
    {0}
};
