/*
 * Copyright (c) 2001-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * sbd.h: MIPS Technologies SEAD-2 board definition header file
 */


#ifndef MHZ
/* fastest possible pipeline clock */
#define MHZ		50
#endif

#define RAMCYCLE	60			/* 60ns dram cycle */
#define ROMCYCLE	150			/* ~150ns rom cycle */
#define CACHECYCLE	(1000/MHZ) 		/* pipeline clock */
#define CYCLETIME	CACHECYCLE
#define CACHEMISS	(CYCLETIME * 6)

/*
 * Address Map
 */

#define DRAM_BASE	0x00000000	/* 32MB DRAM */
#define DRAM_SIZE	 0x2000000
#define FLASH_BASE	0x1c000000	/* 32MB Flash */
#define FLASH_SIZE	  0x800000
#define FPGA_BASE	0x1f000000
#define BOOTPROM_BASE	0x1fc00000	/* Boot region */
#define BOOTPROM_SIZE	  0x400000
#define REVISION_ADDR	0x1fc00010

/* NMI controller */
#define NMISTATUS_ADDR	(FPGA_BASE + 0x00000040)	/* nmp status */
#define NMIACK_ADDR	(FPGA_BASE + 0x00000048)	/* nmi ack */
#define NMI_WERR	0x00000002
#define NMI_FLAG	0x00000001

/* software reset */
/* software reset control */
#define SOFTRES_ADDR	(FPGA_BASE + 0x00000050)
#define SOFTRES_DATA	0x42

/* green led */
/*#define LEDGREEN_ADDR	(FPGA_BASE + 0x00000400)*/

/* 8-bar led array */
#define LEDBAR_ADDR	(FPGA_BASE + 0x00000210)

/* 8-character ASCII led array */
/*#define ASCIIWORD_ADDR	(FPGA_BASE + 0x00000410)*/
#define ASCII_BASE	(FPGA_BASE + 0x000005c0)
#define  ASCII_LED(n)	((n)*8)

/* 
 * UART definitions
 */
#define UART0_BASE	(FPGA_BASE + 0x00000800)
#define UART1_BASE	(FPGA_BASE + 0x00000900)
#define NS16550_HZ	3686400
#define NSREG(x)	((x)*8)		/* regs mapped every 8 bytes */

/* use word accesses for endian independence */
#ifdef __ASSEMBLER__
#define NS16550_INB(dst,offs,base) \
	lw	dst,offs(base)
#define NS16550_OUTB(src,offs,base) \
	sw	src,offs(base)
#else
#define NS16550_INB(base, offs) \
	*((volatile unsigned int *)((char*)(base) + (offs)))
#define NS16550_OUTB(base, offs, data) \
	do { \
	  *((volatile unsigned int *)((char*)(base) + (offs))) = (data); \
	} while (0)
#endif

/* board attributes */
#define DEFSPEED	38400
/*#define _SBD_PANEL*/

/*
 * rough scaling factors for 2 instruction DELAY loop to get 1ms and 1us delays
 */
#define ASMDELAY(ns,icycle)	\
	(((ns) + (icycle)) / ((icycle) * 2))

#define CACHENS(ns)	ASMDELAY((ns), CACHECYCLE)
#define RAMNS(ns)	ASMDELAY((ns), CACHEMISS+RAMCYCLE)
#define ROMNS(ns)	ASMDELAY((ns), CACHEMISS+ROMCYCLE)
#define CACHEUS(us)	ASMDELAY((us)*1000, CACHECYCLE)
#define RAMUS(us)	ASMDELAY((us)*1000, CACHEMISS+RAMCYCLE)
#define ROMUS(us)	ASMDELAY((us)*1000, CACHEMISS+ROMCYCLE)
#define CACHEMS(ms)	((ms) * ASMDELAY(1000000, CACHECYCLE))
#define RAMMS(ms)	((ms) * ASMDELAY(1000000, CACHEMISS+RAMCYCLE))
#define ROMMS(ms)	((ms) * ASMDELAY(1000000, CACHEMISS+ROMCYCLE))

#ifndef __ASSEMBLER__
extern void _sbd_nsdelay (unsigned long);
#define nsdelay(ns)	_sbd_nsdelay (ns)
#define usdelay(us)	_sbd_nsdelay ((us) * 1000)
#endif
