#ident "kit/sead2/sbdfreq.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2001-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * sead2/sbdfreq.c: CPU frequency determination for MIPS SEAD-2 board
 */


#include <stddef.h>
#include <time.h>
#include <errno.h>
#include <mips/cpu.h>
#include <mips/prid.h>

#include "kit_impl.h"
#include "sbd.h"
#include "ns16550.h"

 /*
  *  The console tty port is used to estimate the CPU frequency as
  *  follows:
  *
  *  2000 characters are printed to the UART (in loopback mode) with
  *  flowcontrol turned off, and the time needed for this is measured. 
  *  The elapsed time is scaled with a constant, to yield the estimated
  *  CPU frequency.
  */

long
_sbd_pipefreq (void)
{
    static long pipefreq = 0;
    void * dp;
    unsigned long count;
    unsigned char ocfcr, omcr, odll, odlm;
    int i;

    if (pipefreq > 0)
	return pipefreq;

    dp = PA_TO_KVA1 (UART0_BASE);

    /* wait for tx fifo to fully empty (flush any pending output) */
    while (!(NS16550_INB (dp, NS16550_LSR) & LSR_TSRE))
	continue;

    /* 
     * Setup this UART to 38400, no flow control, 1 stop bit, no
     * parity, no auto flowcontrol, loopback mode.
     */
    
    /* record old settings */
    omcr = NS16550_INB (dp, NS16550_MCR);
    ocfcr = NS16550_INB (dp, NS16550_CFCR);
    NS16550_OUTB (dp, NS16550_CFCR, CFCR_DLAB);
    odll = NS16550_INB (dp, NS16550_DLL);
    odlm = NS16550_INB (dp, NS16550_DLM);
    /* set to 38400 baud */
    NS16550_OUTB (dp, NS16550_DLL, 0x06);
    NS16550_OUTB (dp, NS16550_DLM, 0x00);
    /* set to 8,N,1 mode */
    NS16550_OUTB (dp, NS16550_CFCR, CFCR_8BITS);

    /* initialise MCR - no auto flowcontrol */
    NS16550_OUTB (dp, NS16550_MCR, MCR_LOOPBACK | MCR_DTR | MCR_RTS);
	
    /* poll until space in fifo */
    while (!(NS16550_INB (dp, NS16550_LSR) & LSR_TXRDY))
	continue;

    /* write first byte of data to fifo */
    NS16550_OUTB (dp, NS16550_DATA, 'M');

    /* start timing */
    count = mips_getcount ();

    /* write remaining bytes */
    for (i = 1;  i < 2000;  i++)
    {
	/* poll until space in fifo */
	while (!(NS16550_INB (dp, NS16550_LSR) & LSR_TXRDY))
	    continue;
	/* write another byte of data */
	NS16550_OUTB (dp, NS16550_DATA, 'M');
    }

    /* wait for tx fifo to fully empty */
    while (!(NS16550_INB (dp, NS16550_LSR) & LSR_TSRE))
	continue;

    /* stop timing */
    count = mips_getcount () - count;

    /* calculate CPU pipeline frequency */
    pipefreq = (count * _sbd_cyclespercount () / 5204) * 10000;

    /* empty read fifo, filled by loopback */			
    while (NS16550_INB (dp, NS16550_LSR) & LSR_RXRDY)
	(void) NS16550_INB (dp, NS16550_DATA);

    /* reprogram old uart settings */
    NS16550_OUTB (dp, NS16550_MCR, omcr);
    NS16550_OUTB (dp, NS16550_CFCR, CFCR_DLAB);
    NS16550_OUTB (dp, NS16550_DLL, odll);
    NS16550_OUTB (dp, NS16550_DLM, odlm);
    NS16550_OUTB (dp, NS16550_CFCR, ocfcr);

    return pipefreq;
}


/*
 * Return the CPU bus clock frequency.
 */

long
_sbd_cpufreq (void)
{
    /* Bus frequency always equals CPU frequency on SEAD-2 */
    /* XXX is this true? */
    return _sbd_pipefreq ();
}
