#ident "kit/sead2/sbdfrom.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2001-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * sead2/sbdfrom.c: Flash ROM support for MIPS SEAD-2 board
 */


#ifdef IN_PMON
#include "pmon.h"
#include "sde-compat.h"
#include <assert.h>
#else
#include <sys/types.h>
#include <mips/cpu.h>
#include <stdlib.h>
#include "kit_impl.h"
#endif

#include "sbdfrom.h"
#include "flashdev.h"
#include "flashrom.h"
#include "sbd.h"
#include <string.h>

/* writable cookie */
static struct fromcookie flash_cookie;

/* board-specific code */
static int fromgetparts (flashcookie_t, struct flashparts *);
static int fromsetparts (flashcookie_t, const struct flashparts *);

static const struct flashboard fromboard = {
    fromgetparts,
    fromsetparts
};


flashcookie_t
_sbd_flashopen (paddr_t devaddr)
{
    struct fromcookie *fcp;

    if (devaddr == 0
	 || (devaddr >= FLASH_BASE
	     && devaddr < FLASH_BASE + FLASH_SIZE))
	fcp = &flash_cookie;
    else
	return FLASH_UNKNOWN;

    if (fcp->size == 0) {
	/* always program via flash address */
	fcp->pbase = PA_TO_KVA1 (FLASH_BASE);

	/* read from there too */
	if (IS_KVA0 (fcp))
	    fcp->rbase = PA_TO_KVA0 (FLASH_BASE);
	else
	    fcp->rbase = PA_TO_KVA1 (FLASH_BASE);

	/* probe for device */
	fcp->nbanks = 1;
	if (_flashrom_probe (fcp, 0) < 0)
	    return FLASH_MISSING;
	
	/* compute total size */
	fcp->size = fcp->devsize * FROM_NCOLS;

	fcp->mapbase = FLASH_BASE;
    }

    /* plug in board-specific functions */
    fcp->common.board = &fromboard;

    return &fcp->common;
}


/*
 * Return flash partition table 
 */
static int
fromgetparts (flashcookie_t cookie, struct flashparts *parts)
{
    struct fromcookie *fcp = (struct fromcookie *)cookie;

    memset (parts, 0, sizeof (*parts));

    _flashdev_setpart (parts, FLASHPART_RAW, FLASHPART_RAW, 0, fcp->size);

    /* monitor region: top 4M with top 128K reserved for environment */
    _flashdev_setpart (parts, FLASHPART_BOOT, FLASHPART_BOOT, 
		       0x400000, fcp->size - 0x020000);
    _flashdev_setpart (parts, FLASHPART_ENV, FLASHPART_ENV, 
		       fcp->size - 0x020000, 0x020000);

    return FLASHDEV_OK;
}


static int
fromsetparts (flashcookie_t cookie, const struct flashparts *parts)
{
    /* not supported yet, sorry */
    return FLASHDEV_FAIL;
}
