.ident "kit/share/cache.h: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * cache.h: switchable cache support for MIPS SDE kit
 */

#define _BOOTCODE
#include <mips/asm.h>
#include <mips/regdef.h>
#include <mips/prid.h>

/*	
 * The function of this module is to support boards which can handle
 * multiple MIPS cpus with different cache architectures.
 */	
	
#ifndef _SDE_CACHE_EXTRA	
	
#if defined(IN_PMON) || defined(ITROM)
#define DECL(x, val)		\
	BSS(x,4)
#else	
#define DECL(x, val)		\
	.sdata;			\
	EXPORTS(x, 4);		\
	.word	val
#endif	

#endif
	
#if #cache(r5k)+#cache(rm7k)+#cache(r54)+#cache(m32)+#cache(rc32)+#cache(lr30)+#cache(cw10)+#cache(cw01)+#cache(r4k)+#cache(r3k) > 1
# define MULTICACHE	
# define GETPRID 		\
	.set	noreorder;	\
	mfc0	t1,$15;		\
	nop; nop;		\
	srl	t0,t1,8;	\
	and	t0,0xff;	\
	.set	reorder;
	
# define SWTEST(prid,dest) 	\
	beq	t0,prid,dest;
	
#else	
	
# define GETPRID
	
# define SWTEST(prid,dest) 	\
	b	dest;
	
#endif	
	
#if #cache(r5k)
#define R5KSWITCH(x)	SWTEST(PRID_R5000,   _ASMCONCAT(r5k_, x)) \
			SWTEST(PRID_RM52XX,  _ASMCONCAT(r5k_, x)) \
			SWTEST(PRID_RC6457X, _ASMCONCAT(r5k_, x))
#else
#define R5KSWITCH(x)
#endif
	
#if #cache(rm7k)
#define RM7KSWITCH(x)	SWTEST(PRID_RM7000, _ASMCONCAT(rm7k_, x))
#else
#define RM7KSWITCH(x)
#endif
	
#if #cache(r54)
#define R54SWITCH(x)	SWTEST(PRID_R5400, _ASMCONCAT(r54_, x)) \
			SWTEST(PRID_R5500, _ASMCONCAT(r54_, x))
#else
#define R54SWITCH(x)
#endif
	
#if #cache(rc32)
#define RC32SWITCH(x)	SWTEST(PRID_RC32364, _ASMCONCAT(rc32_, x)) \
			SWTEST(PRID_RC3233X, _ASMCONCAT(rc32_, x))
#else
#define RC32SWITCH(x)
#endif
	
/* MIPS32/64 can be determined by any non-zero value in PrID bits 16:23 */
#if #cache(m32)
# if defined(MULTICACHE)
#  define MIPS32SWITCH(x)	\
	srl	t2,t1,16; 	\
	and	t2,0xff; 	\
	bnez	t2,_ASMCONCAT(m32_, x);
# else	
#  define MIPS32SWITCH(x)	\
	b	_ASMCONCAT(m32_, x);
# endif	
#else
# define MIPS32SWITCH(x)
#endif
	
/* LSI LR33K is a default fall-through if enabled */
#if #cache(lr30)
#define LR30SWITCH(x)	b	_ASMCONCAT(lr30_, x);
#else
#define LR30SWITCH(x)
#endif
	
/* LSI CW4010 is a default fall-through if enabled */
#if #cache(cw10)
#define CW10SWITCH(x)	b	_ASMCONCAT(cw10_, x);
#else
#define CW10SWITCH(x)
#endif
	
/* LSI CW4001 is a default fall-through if enabled */
#if #cache(cw01)
#define CW01SWITCH(x)	b	_ASMCONCAT(cw01_, x);
#else
#define CW01SWITCH(x)
#endif
			
/* Vanilla R4000 is a default fall-through if enabled */
#if #cache(r4k)
#define R4KSWITCH(x)	b	_ASMCONCAT(r4k_, x);
#else
#define R4KSWITCH(x)
#endif
	
/* Vanilla R3000 is a default fall-through if enabled */
#if #cache(r3k)
#define R3KSWITCH(x)	b	_ASMCONCAT(r3k_, x);
#else
#define R3KSWITCH(x)
#endif
	
#define SWCACHE(name) 		\
LEAF(_ASMCONCAT(mips_, name));	\
	GETPRID			\
	R5KSWITCH(name)		\
	RM7KSWITCH(name)	\
	R54SWITCH(name)		\
	RC32SWITCH(name)	\
	MIPS32SWITCH(name)	\
	CW01SWITCH(name)	\
	CW10SWITCH(name)	\
	LR30SWITCH(name)	\
	R4KSWITCH(name)		\
	R3KSWITCH(name)		\
	j	$ra;		\
END(_ASMCONCAT(mips_, name))
