#ident "kit/share/dbgnrom.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2000-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * dbgnrom.c: NetROM dual-port remote debugger support for MIPS SDE
 */


#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "kit_impl.h"


void
_dbg_panic (const char *fmt, ...)
{
    va_list ap;
    _mon_puts ("\r\n_dbg_panic: ");
    va_start(ap, fmt);
    _mon_vprintf (fmt, ap);
    va_end(ap);
    _mon_puts ("\r\n");
    abort ();
}


void
_dbg_printf (const char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    _mon_vprintf (fmt, ap);
    va_end(ap);
}




/* 
 * Debug i/o stubs
 */

/* NetROM header files */
#include "dpconfig.h"  /* This file must be ported to the new target */
#include <netrom/dptarget.h>

#ifndef NETROM_DBGCHAN
#define NETROM_DBGCHAN	0
#endif

int
_dbg_init (void)
{
    nr_ConfigDP ((uInt32)DP_BASE, ROMWORDWIDTH, POD_0_INDEX);
    nr_setBlockIO (NETROM_DBGCHAN, 1);
}


int
_dbg_inch (void)
{
    int c;
    nr_FlushTX (NETROM_DBGCHAN);
    while ((c = nr_Getch (NETROM_DBGCHAN)) < 0)
	continue;
    return c;
}


int
_dbg_inpoll (void)
{
    nr_FlushTX (NETROM_DBGCHAN);
    if (nr_Poll (NETROM_DBGCHAN) <= 0)
	return -1;
    return nr_Getch (NETROM_DBGCHAN);
}


void
_dbg_outch (int c)
{
    nr_Putch (NETROM_DBGCHAN, c);
}


void
_dbg_drain ()
{
    nr_FlushTx (NETROM_DBGCHAN);
}
