#ident "kit/share/dbgsup.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * dbgsup.c: implementation dependent remote debugger support for MIPS SDE
 */


#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <mips/cpu.h>

#include "kit_impl.h"

static int dbgport;

void
_dbg_panic (const char *fmt, ...)
{
    va_list ap;
    _mon_puts ("\r\n_dbg_panic: ");
    va_start(ap, fmt);
    _mon_vprintf (fmt, ap);
    va_end(ap);
    _mon_puts ("\r\n");
    abort ();
}

void
_dbg_printf (const char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    _mon_vprintf (fmt, ap);
    va_end(ap);
}



/* 
 * Debug i/o stubs
 */

#include "sbdser.h"

#pragma weak _sbd_setdbgport=_stub_sbd_setdbgport
int _stub_sbd_setdbgport(int *);
int
_stub_sbd_setdbgport (int *speed)
{
    int port = _sbddbgport;
    *speed = _sbdspeed[port];
    return port;
}


int
_dbg_init (void)
{
    const struct sbdser *ss;
    char *s;
    int dbgspeed;

    /* allow debug port override from environment */
    if ((s = _sbd_getenv ("HOSTPORT")) || (s = _sbd_getenv ("hostport"))
	|| (s = _sbd_getenv ("DBGPORT")) || (s = _sbd_getenv ("dbgport"))) {
	int n;
	dbgport = _sbddbgport;
	dbgspeed = _sbdspeed[dbgport];
	if (strncmp (s, "tty", 3) == 0)
	    s += 3;
	while (isdigit (*s))
	    n += *s++ - '0';
	if (*s == '\0' && n < _sbdnser)
	    dbgport = n;
	if (*s++ == ',') {
	    dbgspeed = 0;
	    while (isdigit (*s))
		dbgspeed += *s++ - '0';
	}
    }
    else {
	dbgport = _sbd_setdbgport (&dbgspeed);
    }

    ss = &_sbdsertab[dbgport];
    (*ss->ss_init) (ss->ss_data, 0);
    return (*ss->ss_params) (ss->ss_data, dbgspeed, 0);
}


int
_dbg_inch (void)
{
    const struct sbdser *ss = &_sbdsertab[dbgport];
    int c;

    while ((c = (*ss->ss_rxpoll) (ss->ss_data, 1, 0)) == 0)
	continue;
    return c & 0xff;
}

int
_dbg_inpoll (void)
{
    const struct sbdser *ss = &_sbdsertab[dbgport];
    int c;

    c = (*ss->ss_rxpoll) (ss->ss_data, 1, 0);
    if (c == 0)
	return -1;
    return c & 0xff;
}


void
_dbg_outch (int c)
{
    const struct sbdser *ss = &_sbdsertab[dbgport];

    (*ss->ss_tx)(ss->ss_data, c, 0);
}


void
_dbg_drain ()
{
    const struct sbdser *ss = &_sbdsertab[dbgport];
    (*ss->ss_params)(ss->ss_data, 0, 0);
}

