#ident "kit/share/dirops.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2001-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * dirops.c: directory level file operations
 */


#define _KERNEL
#include <sys/cdefs.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/filedesc.h>
#undef _KERNEL
#define _XOPEN_SOURCE 500
#include <sys/stat.h>
#include <unistd.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <assert.h>

#define _KERNEL
#include "file.h"
#include "kit_impl.h"

/* from <sys/fcntl.h>, etc */
int	symlink (const char *name1, const char *name2);

static int 
fremove (const char *name, int isrmdir)
{
#if 1
    /* the remove operation is not supported by any filesystems yet,
       but then neither can you create files or directories, so just
       dummy this for now. */
    errno = EACCES;
    return -1;
#else
    dir_t *fdp, *pdp;
    int err;

    _syscall_enter ();
    /* XXX this interface to _fileio_lookup, returning the
       parent dirnode, is not implemented yet */
    err = _fileio_lookup (&fdp, &pdp, name, FNODREF | FREAD, 0);
    if (err == 0) {
	if (!pdp || !(pdp->d_mode & S_IWUSR) || !pdp->d_ops->fo_remove)
	    /* parent directory not writable or not supporting remove */
	    err = EACCES;
	else if (S_ISDIR (fdp->d_mode) && !isrmdir)
	    /* can't unlink directory */
	    err = !isrmdir ? EISDIR : ENOTEMPTY;
	else if (S_ISDIR (fdp->d_mode) && fdp->d_nlink > 2)
	    /* can't remove non-empty directory */
	    err = ENOTEMPTY;
	else if ((pdp->d_mode & S_ISTXT) && !(fdp->d_mode & S_IWUSR))
	    /* dir marked sticky and file not writable by user */
	    err = EPERM;
	else
	    /* ok, have a go at removing the file */
	    err = ((*pdp->d_ops->fo_remove) (ddp, fdp));
	drelease (pdp);
	drelease (fdp);
    }
    return _syscall_leave (err, 0);
#endif
}

int 
unlink (const char *name)
{
    return fremove (name, 0);
}


int 
rmdir (const char *name)
{
    return fremove (name, 1);
}


int 
mkdir (const char *name, mode_t mode)
{
    /* not supported yet */
    errno = EACCES;
    return -1;
}


int 
link (const char *name1, const char *name2)
{
    /* not supported yet */
    errno = EACCES;
    return -1;
}


int 
symlink (const char *name1, const char *name2)
{
    /* not supported yet */
    errno = EACCES;
    return -1;
}


