/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * ecc.h: information regarding cache/ecc errors
 */


#define NECCMSG		4

struct eccinfo {
    struct cxcptcontext	cxcp;
    int			ctype;
    unsigned int 	sc_tag;
    unsigned int 	pc_tag;
    unsigned char 	sc_ecc;
    unsigned char 	pc_ecc;
    unsigned char	linesize;
    unsigned char	flags;
    unsigned int	pidx;
    unsigned int	sidx;
    unsigned long	pa;
    unsigned long	sva;
    unsigned long long	data[16];
    unsigned long long	mdata[16];
    unsigned char	ecc[16];
    int			nmsg;
    const char *	msgs[NECCMSG];
};

#define EF_GOTDATA	0x01
#define EF_FIXED	0x02
#define EF_MSGOFLO	0x04

struct eccinfo *	eccnext (void);
int			eccpending (void);
int			ecctotal (void);
void			eccflush (void);

unsigned int		ecctagcalc (unsigned int);
unsigned int		eccdatacalc (int, unsigned long long);

/*
 * Cache selection
 */
#define CS_LEVEL	0x03
#define  CS_PCACHE	 0
#define  CS_SCACHE	 1
#define  CS_TCACHE	 2
#define  CS_MEM		 3
#define CS_TYPE		0x04
#define  CS_DCACHE	 0x0
#define  CS_ICACHE	 0x4
#define CS_WHICH	0x07
#define CS_WORD		0x08
#define  CS_LO		 0x0
#define  CS_HI		 0x8
#define CS_NOERRS	0x10

#define TDCACHE		(CS_TCACHE | CS_DCACHE)
#define TICACHE		(CS_TCACHE | CS_ICACHE)
#define SDCACHE		(CS_SCACHE | CS_DCACHE)
#define SICACHE		(CS_SCACHE | CS_ICACHE)
#define PDCACHE		(CS_PCACHE | CS_DCACHE)
#define PICACHE		(CS_PCACHE | CS_ICACHE)
#define MEMORY		CS_MEM

#define IS_MEMORY(x)	(((x) & CS_LEVEL) == CS_MEM)
#define IS_TCACHE(x)	(((x) & CS_LEVEL) == CS_TCACHE)
#define IS_SCACHE(x)	(((x) & CS_LEVEL) == CS_SCACHE)
#define IS_PCACHE(x)	(((x) & CS_LEVEL) == CS_PCACHE)
#define IS_ICACHE(x)	(((x) & CS_TYPE) == CS_ICACHE)
#define IS_DCACHE(x)	(((x) & CS_TYPE) == CS_DCACHE)

#define CACHENAME(x)	(IS_TCACHE(x) ? (IS_ICACHE(x) ? "TI" : "TD") : \
			 IS_SCACHE(x) ? (IS_ICACHE(x) ? "SI" : "SD") : \
			 IS_PCACHE(x) ? (IS_ICACHE(x) ? "PI" : "PD") : \
    			 "??")

#define CACHELEVEL(x)	(IS_TCACHE(x) ? 't' : \
			 IS_SCACHE(x) ? 's' : \
			 IS_ICACHE(x) ? 'i' : 'd')
