#ident "kit/share/ecchandler.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2005 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * ecchandler.c: attempt to correct and/or report cache/ecc errors
 */



#include <stddef.h>
#include <string.h>
#include <mips/cpu.h>
#include <mips/xcpt.h>

/* only works for full R4000 cache model at present */
#if defined(XCPC_CACHEERR) && defined(Create_Dirty_Exc_SD) && !__mips16

#include <mips/prid.h>
#include <sys/signal.h>
#include "kit_impl.h"
#include "ecc.h"

extern int mips_scache_split;

struct cxcptcontext _cxcpt;

#ifdef MEMTEST
#include "memtest.h"
#endif

#define ucp(x)	(__typeof__(x) *)KVA0_TO_KVA1(&(x))
#define uc(x)	(*ucp(x))

#define stringify(x)	#x

#define mkcacheop(x,y) \
static inline void x (volatile void *va) \
{ \
    asm volatile ("%(nop;nop;cache %0,0(%1);nop;nop%)" : \
		  : "JK" (y), "r" (va)); \
}

mkcacheop(index_invalidate_i, Index_Invalidate_I)
mkcacheop(index_writeback_inv_d, Index_Writeback_Inv_D)
mkcacheop(index_invalidate_si, Index_Invalidate_SI)
mkcacheop(index_writeback_inv_sd, Index_Writeback_Inv_SD)
mkcacheop(index_load_tag_i, Index_Load_Tag_I)
mkcacheop(index_load_tag_d, Index_Load_Tag_D)
mkcacheop(index_load_tag_si, Index_Load_Tag_SI)
mkcacheop(index_load_tag_sd, Index_Load_Tag_SD)
mkcacheop(index_store_tag_i, Index_Store_Tag_I)
mkcacheop(index_store_tag_d, Index_Store_Tag_D)
mkcacheop(index_store_tag_si, Index_Store_Tag_SI)
mkcacheop(index_store_tag_sd, Index_Store_Tag_SD)
mkcacheop(create_dirty_exc_d, Create_Dirty_Exc_D)
mkcacheop(create_dirty_exc_sd, Create_Dirty_Exc_SD)
mkcacheop(hit_invalidate_i, Hit_Invalidate_I)
mkcacheop(hit_invalidate_d, Hit_Invalidate_D)
mkcacheop(hit_invalidate_si, Hit_Invalidate_SI)
mkcacheop(hit_invalidate_sd, Hit_Invalidate_SD)
mkcacheop(hit_writeback_inv_d, Hit_Writeback_Inv_D)
mkcacheop(hit_writeback_inv_sd, Hit_Writeback_Inv_SD)
mkcacheop(hit_writeback_sd, Hit_Writeback_SD)
mkcacheop(fill_i, Fill_I)
mkcacheop(hit_writeback_d, Hit_Writeback_D)
mkcacheop(hit_writeback_i, Hit_Writeback_I)


#define NERR	16		/* must be power of two */

struct eccring {
    unsigned char	flags;
    unsigned char	first;
    unsigned char	count;
    unsigned int	total;
    struct eccinfo	info[NERR];
} eccring;

#define	ECCF_OVERFLO	0x01



/*0->63*/
static const unsigned char data_eccs[64] = {
	0x13, 0x23, 0x43, 0x83, 0x2f, 0xf1, 0x0d, 0x07,
	0xd0, 0x70, 0x4f, 0xf8, 0x61, 0x62, 0x64, 0x68,
	0x1c, 0x2c, 0x4c, 0x8c, 0x15, 0x25, 0x45, 0x85,
	0x19, 0x29, 0x49, 0x89, 0x1a, 0x2a, 0x4a, 0x8a,
	0x51, 0x52, 0x54, 0x58, 0x91, 0x92, 0x94, 0x98,
	0xa1, 0xa2, 0xa4, 0xa8, 0x31, 0x32, 0x34, 0x38,
	0x16, 0x26, 0x46, 0x86, 0x1f, 0xf2, 0x0b, 0x0e,
	0xb0, 0xe0, 0x8f, 0xf4, 0xc1, 0xc2, 0xc4, 0xc8,
};

/*0->24*/
static const unsigned char tag_eccs[25] = {
	0x07, 0x16, 0x26, 0x46, 0x0d, 0x0e, 0x1c, 0x4c,
	0x31, 0x32, 0x38, 0x70, 0x61, 0x62, 0x64, 0x68,
	0x0b, 0x15, 0x23, 0x45, 0x29, 0x51, 0x13, 0x49,
	0x25
};


#ifdef TAG_ECC_MASK
static unsigned int
stagvtop (unsigned int v)
{
    /* swap Scache TagLo register bit order into h/w bit order */
    return ((((v & TAG_STAG_MASK) >> TAG_STAG_SHIFT) << 0)
	    | (((v & TAG_VINDEX_MASK) >> TAG_VINDEX_SHIFT) << 19)
	    | (((v & TAG_SSTATE_MASK) >> TAG_SSTATE_SHIFT) << 22)
	    | (((v & TAG_ECC_MASK) >> TAG_ECC_SHIFT) << 25)
	);
}
#endif

#if 0
static unsigned int
ptagvtop (unsigned int v)
{
    /* swap Pcache TagLo register bit order into h/w bit order */
    return ((((v & TAG_PTAG_MASK) >> TAG_PTAG_SHIFT) << 0) |
	    (((v & TAG_PSTATE_MASK) >> TAG_PSTATE_SHIFT) << 24) |
	    (((v & TAG_PARITY_MASK) >> TAG_PARITY_SHIFT) << 26));
}
#endif

#ifdef TAG_ECC_MASK
static unsigned int
stagptov (unsigned int x)
{
    /* swap Scache h/w bit order into TagLo register bit order */
    return ((((x >> 0) << TAG_STAG_SHIFT) & TAG_STAG_MASK)
	    | (((x >> 19) << TAG_VINDEX_SHIFT) & TAG_VINDEX_MASK)
	    | (((x >> 22) << TAG_SSTATE_SHIFT) & TAG_SSTATE_MASK)
	    | (((x >> 25) << TAG_ECC_SHIFT) & TAG_ECC_MASK)
	);
}
#endif

#if 0
static unsigned int
ptagptov (unsigned int x)
{	
    /* swap Pcache h/w bit order into TagLo register bit order */
    return ((((x >> 0) << TAG_PTAG_SHIFT) & TAG_PTAG_MASK) |
	    (((x >> 24) << TAG_PSTATE_SHIFT) & TAG_PSTATE_MASK) |
	    (((x >> 26) << TAG_PARITY_SHIFT) & TAG_PARITY_MASK));
}
#endif



unsigned int
ecctagcalc (unsigned int tag)
{
    const unsigned char *eccs = ucp(tag_eccs[0]);
    unsigned char ecc = 0;

    while (tag != 0) {
	if (tag & 1)
	    ecc ^= *eccs;
	eccs++;
	tag >>= 1;
    }
    return ecc;
}


unsigned int
eccdatacalc (int ctype, unsigned long long data)
{
    const unsigned char *eccs = ucp(data_eccs[0]);
    unsigned char ecc = 0;

    if (IS_SCACHE (ctype)) {
	/* calc ECC for 64-bit Scache data word */
	while (data != 0) {
	    if (data & 1)
		ecc ^= *eccs;
	    eccs++;
	    data >>= 1;
	}
    }
    else {
	/* calc Parity for 64-bit Pcache data word */
	unsigned int pbit;
	for (pbit = 0; data != 0; pbit++, data >>= 8) {
	    /* compute 1 byte's worth of parity */
	    unsigned int pv = data;
	    pv ^= pv >> 4;
	    pv ^= pv >> 2;
	    pv ^= pv >> 1;
	    ecc |= (pv & 1) << pbit;
	}
    }
    return ecc;
}


#ifdef TAG_ECC_MASK
static int
ecctagbit (unsigned char ecc)
{
    const unsigned char *eccs = ucp(tag_eccs[0]);
    int bit;

    for (bit = 0; bit < sizeof(tag_eccs); bit++)
	if (eccs[bit] == ecc)
	    return bit;
    return -1;
}
#endif

static int
eccdatabit (unsigned char ecc)
{
    const unsigned char *eccs = ucp(data_eccs[0]);
    int bit;

    for (bit = 0; bit < sizeof(data_eccs); bit++)
	if (eccs[bit] == ecc)
	    return bit;
    return -1;
}


static void
eccaddmsg (struct eccinfo *einfo, const char *msg)
{
    if (einfo->nmsg < NECCMSG)
	einfo->msgs[einfo->nmsg++] = msg;
    else
	einfo->flags |= EF_MSGOFLO;
}


static int
eccfixmem (struct xcptcontext *xcp, struct eccinfo *einfo)
{
    unsigned int *va;

    va = PA_TO_KVA1 (einfo->pa & ~(einfo->linesize - 1));
#ifdef MEMTEST
    {
	int n;
	extern void *first_addr, *last_addr;
	if (einfo->pa < KVA_TO_PA (first_addr)
	    || einfo->pa > KVA_TO_PA (last_addr)) {
	    eccaddmsg (einfo, "Error address out of range");
	    return SIGKILL;
	}
	/* just zap the line so that we can notice it later */
	for (n = einfo->linesize / sizeof(*va); n; n--)
	    *va++ = 0xbadf00d;
    }
    return 0;
#else
    /* XXX most memory only has parity, so we can't fix a memory error */
    return SIGKILL;
#endif
}


static int
eccfixctag (struct xcptcontext *xcp, struct eccinfo *einfo)
{
    unsigned char ecc;
    unsigned int *va;
    int fixed = 0;
    int sig = 0;

    mips_settaglo (0);
    if (IS_PCACHE (einfo->ctype)) {
	ecc = 1;
	va = (unsigned int *) PA_TO_KVA0 (einfo->pidx);
	if (IS_ICACHE (einfo->ctype))
	    index_store_tag_i (va);
	else
	    index_store_tag_d (va);
    }
    else {
#ifdef TAG_ECC_MASK
	unsigned int tag = stagvtop (einfo->sc_tag & ~TAG_ECC_MASK);
	ecc = einfo->sc_tag & TAG_ECC_MASK;
	ecc ^= ecctagcalc (tag);
	if (ecc != 0) {
	    if ((ecc & (ecc - 1)) == 0) {
		/* faulty ecc bit: just rewrite tag */
		mips_settaglo (einfo->sc_tag);
		fixed = 1;
	    }
	    else {
		/* look for single-bit error */
		int bit = ecctagbit (ecc);
		if (bit >= 0) {
		    mips_settaglo (stagptov (tag ^ (1 << bit)));
		    fixed = 1;
		}
	    }

	    va = (unsigned int *) PA_TO_KVA0 (einfo->sidx);
	    if (IS_ICACHE (einfo->ctype))
		index_store_tag_si (va);
	    else
		index_store_tag_sd (va);
	}
#endif
	ecc = 0;
    }
	
    if (fixed)
	eccaddmsg (einfo, "Fixed single-bit tag error");
    else if (ecc)
	eccaddmsg (einfo, "Zapped faulty tag (DANGER!)");
    else
	eccaddmsg (einfo, "No error found, tag not changed!");

    return sig;
}


static int
eccfixcdata (struct xcptcontext *xcp, struct eccinfo *einfo)
{
    volatile unsigned long long *lva;
    volatile unsigned long long *pva;
    unsigned char anyecc = ~0;
    int sig = 0;
    int fixed = 0;
    int failed = 0;
    int i;

    /* KSEG0 is forced to be cacheable by the low-level handler */
    lva = PA_TO_KVA0 (einfo->sva & ~(einfo->linesize - 1));
    pva = PA_TO_KVA0 (einfo->pidx & ~(einfo->linesize - 1));

    /* take a copy of the faulty line before zapping it */
    if (IS_DCACHE (einfo->ctype)
	 || (IS_SCACHE (einfo->ctype) && !uc(mips_scache_split))) {
	volatile unsigned long long *mva = KVA0_TO_KVA1 (lva);
	for (i = 0; i < einfo->linesize/sizeof(long long); i++) {
	    switch (einfo->ctype) {
	    case SDCACHE:
		index_load_tag_sd (lva + i);
		break;
	    case PDCACHE:
		index_load_tag_d (pva + i);
		break;
	    case SICACHE:
		index_load_tag_si (lva + i);
		break;
	    case PICACHE:
		index_load_tag_i (pva + i);
		break;
	    }	
	    einfo->ecc[i] = mips_getecc ();
	}

	anyecc = 0;
	for (i = 0; i < einfo->linesize/sizeof(long long); i++) {
	    unsigned char ecc;
	    einfo->mdata[i] = *mva++;
	    einfo->data[i] = lva[i];
	    ecc = eccdatacalc (einfo->ctype, einfo->data[i]);
	    anyecc |= ecc ^ einfo->ecc[i];
	}
	einfo->flags |= EF_GOTDATA;
    }

    mips_settaglo (0);
    if (anyecc == 0) {
	eccaddmsg (einfo, "No error found, cache not modified!");
    } 
    else if (IS_PCACHE (einfo->ctype)) {
	/* clear tag, refill with valid parity, then clear tag again */
	/* XXX will this work for two way set caches! */
	if (IS_ICACHE (einfo->ctype)) {
	    index_store_tag_i (pva);
	    fill_i (pva);		/* refill with valid parity */
	    index_store_tag_i (pva);
	} else {
	    /* this will be fatal if cache line was dirty! */
	    index_store_tag_d (pva);
	    (void)*lva;			/* refill with valid parity */
	    index_store_tag_d (pva);
	}
    }
    else {
	unsigned char *sva;
	unsigned int dlinesize = uc(mips_dcache_linesize);
	unsigned int ilinesize = uc(mips_dcache_linesize);

	sva = PA_TO_KVA0 (einfo->sidx & ~(einfo->linesize - 1));
	if (IS_ICACHE (einfo->ctype) && uc(mips_scache_split)) {
	    /* clear stags */
	    index_store_tag_si (sva);
	    for (i = 0; i < einfo->linesize; i += ilinesize) {
		index_store_tag_i (sva + i);
		/* refill primary with good parity from secondary */
		fill_i (sva + i);
		/* write "good" data back to secondary */
		hit_writeback_i (sva + i);
		/* leave ptag empty */
		index_store_tag_i (sva + i);
	    }
	    /* clear stags again */
	    index_store_tag_si (sva);
	} else {
	    /* clear tags */
	    index_store_tag_sd (sva);
#if 0
	    /* don't zap dtags, they may contain valid dirty data */
	    for (i = 0; i < einfo->linesize; i += dlinesize)
		index_store_tag_d (sva + i);
#endif

	    /* create dirty lines */
	    create_dirty_exc_sd (lva);
	    for (i = 0; i < einfo->linesize; i += dlinesize)
		create_dirty_exc_d (lva + i);

	    if (einfo->flags & EF_GOTDATA) {
		/* put back whatever data we collected earlier */
		for (i = 0; i < einfo->linesize/sizeof(long long); i++) {
		    unsigned long long word = einfo->data[i];
		    unsigned char ecc = eccdatacalc (einfo->ctype, word);
		    ecc ^= einfo->ecc[i];
		    if (ecc != 0) {
			int bit = eccdatabit (ecc);
			if ((ecc & (ecc - 1)) == 0) {
			    /* single-bit ecc error */
			    fixed++;
			}
			else if ((bit = eccdatabit (ecc)) >= 0) {
			    /* single-bit data error */
			    word ^= 1LL << bit;
			    fixed++;
			} else {
			    /* multi-bit error */
			    failed++;
			}
		    }
		    lva[i] = word;
		}
	    }
	    else {
		/* zap the line with a recognisable pattern */
		for (i = 0; i < einfo->linesize/sizeof(long long); i++)
		    lva[i] = 0xacedbabeacedbabeLL;
		failed++;
	    }

	    /* move primary cache line to secondary and clear ptag */
	    for (i = 0; i < einfo->linesize; i += dlinesize) {
		hit_writeback_inv_d (lva + i);
		index_store_tag_d (lva + i);
	    }
	}
    }

    if (anyecc && fixed)
	eccaddmsg (einfo, "Fixed single-bit ecc error(s)");

    if (failed)
	eccaddmsg (einfo, "Couldn't fix multi-bit ecc error(s)");

    if ((anyecc && !fixed) || failed) {
	if (IS_SCACHE (einfo->ctype)) {
#ifdef TAG_SSTATE_MASK
#if defined(SSTATE_DIRTY_EXCL) && defined(SSTATE_DIRTY_SHARED)
	    unsigned int state;
	    state = (einfo->sc_tag & TAG_SSTATE_MASK) >> TAG_SSTATE_SHIFT;

	    if (state == SSTATE_DIRTY_EXCL || state == SSTATE_DIRTY_SHARED) {
		eccaddmsg (einfo, "ZAPPED DIRTY SC DATA!!");
		failed = 1;
	    }
	    else
#endif
		eccaddmsg (einfo, "Zapped clean SC data");
#else
	    eccaddmsg (einfo, "ERROR IN MISSING SC!!");
#endif
	}
	else {
#if defined(PSTATE_DIRTY_EXCL) && defined(PSTATE_SHARED)
	    unsigned int state;
	    state = (einfo->pc_tag & TAG_PSTATE_MASK) >> TAG_PSTATE_SHIFT;
	    if (state == PSTATE_DIRTY_EXCL || state == PSTATE_SHARED) {
		eccaddmsg (einfo, "ZAPPED DIRTY PC DATA!!");
		failed = 1;
	    }
	    else
#endif
		eccaddmsg (einfo, "Zapped clean PC data");
	}
    }

    if (failed) {
#ifdef MEMTEST    
	/* check for fatal error outside of test region */
	extern void *first_addr, *last_addr;
	if (einfo->pa < KVA_TO_PA (first_addr)
	    || einfo->pa > KVA_TO_PA (last_addr)) {
	    eccaddmsg (einfo, "Error address out of range");
	    sig = SIGKILL;
	}
#else
	/* any fatal error is really fatal */
	sig = SIGKILL;
#endif
    }

    return sig;

}


static int
eccfixcache (struct xcptcontext *xcp, struct eccinfo *einfo)
{
    int sig = 0;

    if (IS_PCACHE (einfo->ctype)) {
	unsigned int prid = (mips_getprid() >> 8) & 0xff;
	unsigned int size, idx, pg, tag;
	
	if (prid == PRID_R4000 && uc(mips_scache_size) > 0) {
	    /* An R4000 bug means that we can't rely on the cacherr.vidx field
	       and have to search for the matching primary cache line. */
	    size = IS_ICACHE (einfo->ctype) ? uc(mips_icache_size) 
		: uc(mips_dcache_size);
	    idx = einfo->pidx & 0xfff;
	    for (pg = 0; pg < size; pg += 0x1000) {
		if (IS_ICACHE (einfo->ctype))
		    index_load_tag_i (PA_TO_KVA0 (pg | idx));
		else
		    index_load_tag_d (PA_TO_KVA0 (pg | idx));
		tag = mips_gettaglo ();
		if ((tag & TAG_PSTATE_MASK) != (PSTATE_INVAL << TAG_PTAG_SHIFT)
		    && ((tag & TAG_PTAG_MASK) << 4) == (einfo->pa & ~0xfff))
		    break;
	    }
	    if (pg >= size)
		eccaddmsg (einfo, "Couldn't find matching primary tag");
	    else
		einfo->pidx = pg | idx;
	}
	else {
	    /* assume pidx is correct and fetch the tag */
	    if (IS_ICACHE (einfo->ctype))
		index_load_tag_i (PA_TO_KVA0 (einfo->pidx));
	    else
		index_load_tag_d (PA_TO_KVA0 (einfo->pidx));
	    tag = mips_gettaglo ();

	    /* build full physical address from cache tag */
	    einfo->pa = ((tag & TAG_PTAG_MASK) << 4) | (einfo->pa & 0xfff);
	    einfo->sva = einfo->pa;
	}
    }
    else {
#ifdef TAG_VINDEX
	/* construct pcache index from secondary cache tag */
	if (uc(mips_scache_size) > 0)
	    einfo->pidx = 
		((einfo->sc_tag & TAG_VINDEX_MASK) << (12 - TAG_VINDEX_SHIFT)) |
		(einfo->pidx & 0xfff);
	else
#else
	eccaddmsg (einfo, "Unexpected SC error");
#endif
    }

    if (IS_ICACHE (einfo->ctype))
	index_load_tag_i (PA_TO_KVA0 (einfo->pidx));
    else
	index_load_tag_d (PA_TO_KVA0 (einfo->pidx));
    einfo->pc_tag = mips_gettaglo ();
    einfo->pc_ecc = mips_getecc ();

    if (einfo->cxcp.cacheerr & CACHEERR_TAG) 
	sig = eccfixctag (xcp, einfo);

    if (einfo->cxcp.cacheerr & CACHEERR_DATA && sig == 0) 
	sig = eccfixcdata (xcp, einfo);

    if (IS_ICACHE (einfo->ctype)) {
	/* Always zap primary instruction cache on an instruction cache err */
	mips_settaglo (0);
	index_store_tag_i (PA_TO_KVA0 (einfo->pidx));
    } 
#ifdef CACHEERR_STORE
    else if (einfo->cxcp.cacheerr & CACHEERR_STORE) {
	/* Scache error on Pcache store refill: kill the Pcache line */
	mips_settaglo (0);
	index_store_tag_d (PA_TO_KVA0 (einfo->pidx));
    }
#endif

    return sig;
}


/*
 * The top level cache error exception handler.  Install with
 *     xcption (XCPTCACHERR, ecchandler);
 */

static int
ecchandler (int xcptno, struct xcptcontext *xcp)
{
    struct cxcptcontext *cxcp = ucp(_cxcpt);
    struct eccring *ering = ucp(eccring);
    struct eccinfo *einfo;
    int sig, n;

    /* grab next entry in the ecc error ring */
    n = ering->count;
    if (n == NERR) {
	ering->flags |= ECCF_OVERFLO;
	n = NERR - 1;
    }
    else {
	ering->count = n + 1;
    }
    einfo = &ering->info[(ering->first + n) & (NERR - 1)];
    ering->total++;

    /* initialise counters and flags */
    memset (einfo, 0, sizeof (*einfo));
    /*einfo->nmsg = einfo->flags = 0;*/

    /* get the scache index (physaddr 21..3) from CacheErr */
    einfo->sidx = cxcp->cacheerr & CACHEERR_SIDX_MASK;

    /* Read the failing line's scache tag */
    if (cxcp->cacheerr & CACHEERR_TYPE)
	index_load_tag_sd (PA_TO_KVA0 (einfo->sidx));
    else
	index_load_tag_si (PA_TO_KVA0 (einfo->sidx));
    einfo->sc_tag = mips_gettaglo ();
    einfo->sc_ecc = mips_getecc ();

    /* get primary cache index (not necessarily the right va) */
    einfo->pidx = ((cxcp->cacheerr & CACHEERR_PIDX_MASK) << 12) |
	(einfo->sidx & 0xfff);
	
#ifdef TAG_STAG_MASK
    if (uc(mips_scache_size) > 0) {
	/* get PA32..22 from TagLo and PA21..3 from CacheErr */
	einfo->pa = (((einfo->sc_tag & TAG_STAG_MASK) << 4) & 0xffc00000) |
	    einfo->sidx;

	/* calc scache addr for actual cache line from tag & cacherr */
	einfo->sva = ((einfo->sc_tag & TAG_STAG_MASK) << 4) |
	    (einfo->sidx & 0x1ffff);
    }
    else 
#endif
    {
	/* only an approximation here */
	einfo->pa = einfo->sidx; 	/* get PA21..4 from cacherr */
	einfo->sva = einfo->pa;		/* use PA as VA */
    }

    /* switch off "both" indicator if it's a data error */
    if ((cxcp->cacheerr & (CACHEERR_BOTH | CACHEERR_DATA)) ==
	(CACHEERR_BOTH | CACHEERR_DATA))
	cxcp->cacheerr &= ~CACHEERR_BOTH;

    /* save error registers */
    einfo->cxcp = *cxcp;

    if (cxcp->cacheerr & CACHEERR_BUS) {
	/* SysAD error */
	einfo->ctype = MEMORY;
	n = uc(mips_scache_linesize);
	if (n < uc(mips_dcache_linesize))
	    n = uc(mips_dcache_linesize);
	if (n < uc(mips_icache_linesize))
	    n = uc(mips_icache_linesize);
	einfo->linesize = n;
    }
    else if (cxcp->cacheerr & CACHEERR_LEVEL) {
	/* secondary cache error */
	if (cxcp->cacheerr & CACHEERR_TYPE)
	    einfo->ctype = SDCACHE;
	else
	    einfo->ctype = SICACHE;
	einfo->linesize = uc(mips_scache_linesize);
    }
    else {
	/* primary cache error */
	if (cxcp->cacheerr & CACHEERR_TYPE) {
	    einfo->ctype = PDCACHE;
	    einfo->linesize = uc(mips_dcache_linesize);
	}
	else {
	    einfo->ctype = PICACHE;
	    einfo->linesize = uc(mips_icache_linesize);
	}
    }

    if (IS_MEMORY (einfo->ctype)) {
	sig = eccfixmem (xcp, einfo);
    }
    else {
	sig = eccfixcache (xcp, einfo);
    }

    if (cxcp->cacheerr & CACHEERR_BOTH) {
	eccaddmsg (einfo, "Flushed all caches due to CacheErr.EB");
	mips_flush_cache ();
    }

    /* Handle repeated errors */
    if (ering->count > 1) {
	struct eccinfo *eprev;
	eprev = &ering->info[(ering->first + ering->count - 2) & (NERR - 1)];
	if (cxcp->errpc == eprev->cxcp.errpc
	    && cxcp->cacheerr == eprev->cxcp.cacheerr) {
	    /* Exactly the same error again */
#ifdef MEMTEST
	    /* check whether in cache test subroutine */
	    if (_inptrops (REG_TO_VA (cxcp->errpc))) {
		/* force return(0xdeadbad) from test routine */
		xcp->regs[2] = 0x0deadbad;	/* $v0 = 0xdeadbad */
		cxcp->errpc = xcp->regs[31];	/* $errpc = $ra */
	    }
#endif
	}
    }

    return sig;
}

int 
ecctotal ()
{
    return uc(eccring.total);
}


int 
eccpending ()
{
    return uc(eccring.count);
}

void
eccflush ()
{
    struct eccring *ering = ucp(eccring);
    ering->count = 0;
    ering->flags = 0;
}

struct eccinfo *
eccnext ()
{
    struct eccring *ering = ucp(eccring);
    unsigned int i;

    if (ering->count == 0)
	return 0;
    if (ering->flags & ECCF_OVERFLO) {
	_mon_puts ("NOTE: ecc error ring overflowed\n");
	ering->flags &= ~ECCF_OVERFLO;
    }
    i = ering->first;
    ering->first = (ering->first + 1) & (NERR - 1);
    ering->count--;
    return &ering->info[i];
}


void
_mips_ecchandler_init (void)
{
    xcption (XCPTCACHEERR, ecchandler);
}
#endif
