#ident "kit/share/getdir.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2001-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * getdir.c: getdirentries() system call for MIPS SDE
 */


#define _KERNEL
#include <sys/fcntl.h>
#include <sys/dirent.h>
#undef _KERNEL
#include <unistd.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <assert.h>

#define _KERNEL
#include "file.h"
#include "kit_impl.h"

int
getdirentries (int fd, char *buf, int len, long *basep)
{
    file_t *fp;
    struct uio uio;
    struct iovec iov[1];
    int ret = -1;
    int err;

    _syscall_enter ();

    err = _fileio_fget (fd, &fp);
    if (err)
	goto done;
    if (!(fp->f_flag & FREAD)) {
	err = EACCES;
	goto done;
    }
    if (!fp->f_dp || !S_ISDIR (fp->f_dp->d_mode)) {
	err = ENOTDIR;
	goto done;
    }

    if (basep)
	*basep = fp->f_offset;

    iov[0].iov_base = buf;
    iov[0].iov_len = len;
    uio.uio_iov = iov;
    uio.uio_iovcnt = 1;
    uio.uio_offset = fp->f_offset;
    uio.uio_resid = len;
    uio.uio_segflg = UIO_USERSPACE;
    uio.uio_rw = UIO_READ;

    err = (*fp->f_ops->fo_read) (fp, &uio);

    fp->f_offset = uio.uio_offset;
    ret = len - uio.uio_resid;

 done:
    _fileio_frelease (fp);
    return _syscall_leave (err, ret);
}
