#ident "kit/share/initfini.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Call .init/.fini functions in reverse order when exiting
 */

#include <sys/init.h>
#include <sys/cdefs.h>
#include "kit_impl.h"

/*
 * Call each function in the .sdeinit list
 */
void
_call_init_funcs (int type)
{
    struct ifent *ifp;

    for (ifp = __SDEINIT_LIST__; ifp->if_type != IF_END; ifp++) {
	if (ifp->if_type != type)
	    continue;
	(*ifp->if_func) ();
    }
}

/*
 * Call each function in the .sdefini list
 */
void
_call_fini_funcs (void)
{
    struct ifent *ifp, *eifp, *bifp;
    int type;

    /* find end of table */
    for (eifp = bifp = __SDEFINI_LIST__; eifp->if_type != IF_END; eifp++)
	continue;

    if (eifp == bifp)
	return;

    for (type = IF_MAX; type != 0; type--) {
	ifp = eifp;
	while (ifp-- != bifp) {
	    if (ifp->if_type == type)
		(*ifp->if_func) ();
	}
    }
}
