.ident "kit/share/m82510.s: Copyright (c) MIPS Technologies, Inc. All rights reserved."
	
/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * share/m82510.s: Generic 82510 UART driver.
 */

	
/* These routines can be called very early in the life of the PROM.
 * - They cannot use dram / stack etc.  
 * - They may only use registers AT, v0-v1, a0-a3, t0-t3
 */

#include <mips/asm.h>
#include <mips/regdef.h>
#include <mips/cpu.h>

#include "sbdser.h"
#include "m82510.h"

#ifndef M82510_CLK
#error M82510_CLK must be defined
#endif
			
#ifdef DUART2_BASE
#define	NSER	2
#else
#define	NSER	1
#endif

#define TXWAIT	10000		/* TxRdy wait timeout */

/* These macros allow for any special action needed before accessing device. */
#ifndef PRE_M82510
#define PRE_M82510
#define POST_M82510
#endif
	
#ifdef WDELAY
#define DELAY 			\
	.set	noreorder;	\
        .set    noat; 		\
	li	AT,WDELAY;	\
10:	bnez	AT,10b;		\
	subu	AT,1;		\
        .set    at; 		\
	.set	reorder
#else
#define DELAY
#endif
		
#ifndef DEFSPEED
#define DEFSPEED	9600	/* default console speed */
#endif
	
#ifndef DBGSPEED
#define DBGSPEED	9600	/* default debug speed */
#endif
	
#ifndef DBGPORT	
#define DBGPORT		(NSER-1)
#endif
	
	.sdata
/* remote debugging port number */
EXPORTS(_sbddbgport, 4)
	.word	DBGPORT
	
	.rdata
/* device table */
EXPORTS(_sbdsertab, SS_SIZE * NSER)
	.word	0x0, reset, init, params, rxpoll, txrdy, tx
#ifdef DUART2_BASE
	.word	0x4, reset, init, params, rxpoll, txrdy, tx
#endif


/* number of ports */
EXPORTS(_sbdnser, 4)
	.word	NSER

/* default initial port speeds */
EXPORTS(_sbdspeed, NSER*4)
#if NSER > 1	
	.word	DEFSPEED : NSER - 1
	.word	DBGSPEED
#else	
	.word	DEFSPEED
#endif	


	.rdata

/* UART address table */
_addrs:
	.word	PA_TO_KVA1(DUART_BASE)
#ifdef DUART2_BASE
	.word	PA_TO_KVA1(DUART2_BASE)
#endif


/*
 * reset (chan, reloc)
 *
 * Once-only device reset at power-on
 */
SLEAF(reset)
	la	t0,_addrs	# uart address table
	addu	t0,a1		# relocation
	addu	t0,a0		# calc &_addrs[chan]
	lw	a0,0(t0)	# get address

	PRE_M82510

	li	t0,BANK_WRK
	sb	t0,BANK(a0)	# select work bank
	li	t0,ICM_SRST
	sb	t0,WRK_ICM(a0)	# software reset
	li	t0,TCM_TXDI
	sb	t0,WRK_TCM(a0)	# disable Tx
	li	t0,RCM_RXDI
	sb	t0,WRK_RCM(a0)	# disable rx
	WBFLUSH
	DELAY
	
	POST_M82510
	j	ra
SEND(reset)



	.rdata
	
/* Channel initialisation table */
_inittab:
	.word	BANK, BANK_MCFG			/* select modem config bank */
	.word	0, 0
 	.word	MCFG_CLCF, CLCF_RXCS | CLCF_TXCS/* 16x BRGA clock */
	.word	MCFG_BBCF, BBCF_BBCS_CLOCK	/* BRGB is timer */
	.word	MCFG_BACF, BACF_BACS_CLOCK | BACF_BAM/* BRGA is BRG */
	.word	MCFG_MIE,  0			/* no modem interrupts */
	.word	0, 0
	
	.word	BANK, BANK_GCFG			/* select work bank */
	.word	0, 0
	.word	GCFG_IMD, IMD_IAM | IMD_RFD_4	/* auto intack; 4-deep rx fifo */
	.word	GCFG_FMD, (3 << FMD_TFT_SHIFT) | (0 << FMD_RFT_SHIFT)
	.word	GCFG_RIE, 0				/* no rx interrupts */
	.word	0, 0

	.word	BANK, BANK_WRK			/* select work bank */
	.word	0, 0
	.word	WRK_TCM, TCM_TXEN	/* enable Tx */
	.word	WRK_RCM, RCM_RXEN	/* enable Rx */
	.word	0, 0

	.word	-1,-1


/*
 * init (chan, reloc)
 *
 * Per channel initialisation.
 */
SLEAF(init)
	la	t0,_addrs	# uart address table
	addu	t0,a1		# relocation
	addu	t0,a0		# calc _addrs[chan]
	lw	a0,0(t0)	# get address

	la	t0,_inittab	# initialisation table
	addu	a1,t0		# add relocation

	PRE_M82510

1:	lw	t0,0(a1)	# register offset
	lw	t1,4(a1)	# register value
	addu	a1,8
	
	bnez	t0,2f		# WBFLUSH request?
	WBFLUSH
	b	1b
	
	beq	t0,-1,3f	# end of table?

	addu	t0,a0		# add uart base address
	sb	t1,(t0)		# write uart register
	DELAY

	b	1b

3:	POST_M82510
	j	ra
SEND(init)




/*
 * Baud rate conversion table
 */
#define SPD(speed) \
	.half	speed, BRTC(M82510_CLK,speed)

#define SPEED	0
#define BRGVAL	2

	.rdata
_speedtab:
	SPD(50)
	SPD(75)
	SPD(110)
	SPD(134)
	SPD(150)
	SPD(220)
	SPD(300)
	SPD(600)
	SPD(1050)
	SPD(1200)
	SPD(2400)
 	SPD(4800)
	SPD(1800)
	SPD(7200)
	SPD(9600)
	SPD(19200)
	SPD(38400)
	SPD(0xffff)



/*
 * params (chan, speed, reloc)
 *
 * program new speed, return 0 on success
 */
SLEAF(params)
	beqz	a1,..fail	# don't want divide by zero err
	li	t2,M82510_CLK/16
	divu	t2,t2,a1	# brtc = M82510_CLK / 16 / speed
	blez	t2,..fail	# brtc must be > 0
	
	/* program the channel with the requested baud rate */
	la	v1,_addrs	# uart address table base
	addu	v1,a2		# relocate table base
	addu	a2,v1,a0	# calc &_addrs[chan]
	lw	a2,0(a2)	# channel address in a2

	PRE_M82510

	li	t0,BANK_NAS
	sb	t0,BANK(a2)			# select 8250 compat bank
	WBFLUSH
	
	li	t0,LCR_DLAB | LCR_PARNONE | LCR_CL8 # no parity: 8 bits/char
	sb	t0,NAS_LCR(a2)
	WBFLUSH
	sb	t2,NAS_BAL(a2)
	WBFLUSH
	srl	t2,8
	sb	t2,NAS_BAH(a2)
	WBFLUSH
	li	t0,LCR_PARNONE | LCR_CL8	# no parity: 8 bits/char
	sb	t0,NAS_LCR(a2)
	WBFLUSH

	li	t0,BANK_WRK
	sb	t0,BANK(a2)
	WBFLUSH
	
	POST_M82510

	move	v0,zero	
	j	ra

..fail:	li	v0,1
	j	ra
SEND(params)



/*
 * rxpoll (chan, canblock, reloc) 
 *
 * poll for received character, return zero if none available
 */
SLEAF(rxpoll)
	la	t0,_addrs	# uart address table
	addu	t0,a2		# relocate
	addu	t0,a0		# calc &_addrs[chan]
	lw	a0,0(t0)	# channel address

	PRE_M82510
	lbu	v0,WRK_GSR(a0)
	and	v0,GSR_RFIR
	beqz	v0,1f
	lbu	v0,WRK_DAT(a0)
	or	v0,0x100
1:	POST_M82510
	j	ra
SEND(rxpoll)


/* 
 * txrdy (chan, reloc) 
 *
 * return non-zero if ready to transmit
 */
SLEAF(txrdy)
	la	t0,_addrs	# uart address table
	addu	t0,a1		# relocate
	addu	t0,a0		# calc &_addrs[chan]
	lw	a0,0(t0)	# channel address

	PRE_M82510
	lbu	v0,WRK_GSR(a0)
	and	v0,GSR_TFIR
	POST_M82510
	j	ra
SEND(txrdy)


/* 
 * tx (chan, data, reloc) 
 *
 * transmit data
 */
SLEAF(tx)
	la	t0,_addrs	# uart address table
	addu	t0,a2		# relocate
	addu	t0,a0		# calc &_addrs[chan]
	lw	a0,0(t0)	# channel address

	PRE_M82510

	li	t0,TXWAIT	# TxRdy timeout
1:	lbu	t1,WRK_GSR(a0)
	and	t1,GSR_TFIR
	bnez	t1,2f
	subu	t0,1
	bnez	t0,1b

2:	sb	a1,WRK_DAT(a0)
	POST_M82510
	j	ra
SEND(tx)
