.ident "kit/share/memprobe.s: Copyright (c) MIPS Technologies, Inc. All rights reserved."
	
/*
 * Copyright (c) 1996-2005 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * memprobe.s:	generic low-level memory sizer and clearer 
 *
 * NOTE: uses 32-bit address space (KSEG1), so limited to 512MB
 */

	
/*
 * We *must* initialise memory so that it has good parity/ecc, 
 * and this must be done before the caches are used
 */

/*		
 * Define BUSWIDTH to usually be real buswidth X 2 (i.e assuming 
 * 2-way interleaving).  This is so that the test pattern and
 * inverted pattern are written to the same bank of memory, which
 * prevents us reading back data sitting in the dram buffers and 
 * getting a false match.
 */

#include <mips/asm.h>
#include <mips/regdef.h>
	
#ifndef BUSWIDTH
# if #cpu(r4000) || #cpu(r5000) || #cpu(rm7000) || __mips==64
#  define BUSWIDTH	16		/* 64 bit memory, bank interleaved */
# else
#  define BUSWIDTH	8		/* 32 bit memory, bank interleaved */
# endif
#endif

#ifndef RAM_BASE
#define RAM_BASE	KSEG1_BASE
#endif
			
#ifndef MEMSTART
#define MEMSTART	0x0		/* start of physical memory */
#endif
		
#ifndef MEMINCR
# define MEMINCR	0x10000		/* work up in 64Kb increments */
#endif

#ifndef MEMLIMIT	
# define MEMLIMIT	0x10000000	/* 256 Mb max memory */
#endif
		
	
/*
 * unsigned int size_mem_limit (unsigned int *base, unsigned int *lim)
 */
SLEAF(size_mem_limit)
	mfc0	t8,C0_STATUS
	
#ifdef SR_DE
	/* disable cache and memory parity checking */
	.set	noreorder
	nop
	or	t0,t8,SR_DE
	mtc0	t0,C0_STATUS
	.set	reorder
#endif

	move	t0,a0			# start at bottom of phys mem
	li	t2,0xaa55aa55		# pattern 
	not	t3,t2			# ~pattern

	la	ta0,..fail		# bus error exception catcher
	addu	k0,ta0,s8		# RELOC 

	/* fill first MEMINCR bytes with zero (for cache init) */
	move	ta0,t0
	addu	ta1,ta0,MEMINCR
	.set	noreorder
1:	sw	zero,0(ta0)
	sw	zero,4(ta0)
	sw	zero,8(ta0)
	addu	ta0,16
	bne	ta0,ta1,1b
	sw	zero,-4(ta0)		# BDSLOT
	.set	reorder
	
..loop:
	move	ta0,t0

	/* store pattern in bank 0, line 0 */
	sw	t2,0(ta0)
	addu	ta0,4

#if BUSWIDTH > 4
	/* fill remainder of line with zeros */
	li	ta1,BUSWIDTH-4
1:	sw	zero,0(ta0)
	subu	ta1,4
	addu	ta0,4
	bnez	ta1,1b
#endif

	/* store inverse pattern in bank 0, line 1 */
	sw	t3,0(ta0)
	addu	ta0,4

#if BUSWIDTH > 4
	/* fill remainder of line with zeros */
	li	ta1,BUSWIDTH-4
1:	sw	zero,0(ta0)
	subu	ta1,4
	addu	ta0,4
	bnez	ta1,1b
#endif

	/* defeat write buffering */
#if __mips >= 2
	sync
#endif
	lw	zero,-4(ta0)

	lw	ta0,0(t0)		# read first word of line
	lw	ta1,0(a0)		# read start of memory (should be zero)
	bne	ta0,t2,..fail		# this line wrong?
	
	bne	t0,a0,1f		# start of memory?
	sw	zero,0(a0)		# yes - put zero back at start
	move	ta1,zero
	
1:	bnez    ta1,..fail		# start of mem overwritten?

	addu    t0,MEMINCR		# bump address
	bltu	t0,a1,..loop		# until end of region

..fail:
	/* restore Status register */
	move	k0,zero
	mtc0	t8,C0_STATUS

	/* return memory size */
	subu 	v0,t0,a0
	j	ra
END(size_mem_limit)


/*
 * unsigned int size_mem (void)
 */
SLEAF(size_mem)
	li	a0,RAM_BASE+MEMSTART	# start at bottom of phys mem
	addu	a1,a0,MEMLIMIT
	b	size_mem_limit
SEND(size_mem)

	
		
/* 
 * void clear_mem_limit (void *base, void *lim)
 */
SLEAF(clear_mem_limit)
	bgeu	a0,a1,9f

	.set noreorder	
#if (__mips >= 3 && __mips != 32) || __mips64
1:	sd	zero,0(a0)
	sd	zero,8(a0)
	sd	zero,16(a0)
	sd	zero,24(a0)
	sd	zero,32(a0)
	sd	zero,40(a0)
	sd	zero,48(a0)
	addu	a0,64
	bne	a0,a1,1b
	sd	zero,-8(a0)		# BDSLOT
#else	
1:	sw	zero,0(a0)
	sw	zero,4(a0)
	sw	zero,8(a0)
	sw	zero,12(a0)
	sw	zero,16(a0)
	sw	zero,20(a0)
	sw	zero,24(a0)
	sw	zero,28(a0)
	addu	a0,32
	bne	a0,a1,1b
	sw	zero,-4(a0)		# BDSLOT
#endif	
	.set	reorder

9:	j	ra
END(clear_mem_limit)

	
/* 
 * void clear_mem (unsigned int size)
 */
SLEAF(clear_mem)	
	move	a1,a0
	li	a0,RAM_BASE+MEMSTART	# start at bottom of phys mem
	addu	a1,a0
	b	clear_mem_limit
SEND(clear_mem)	
	
	
