.ident "kit/share/mpsc.s: Copyright (c) MIPS Technologies, Inc. All rights reserved."
	
/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * share/mpsc.s: NEC uPD72001 multi protocol serial controller driver
 * low level shared by other Algorithmics boards.
 */

	
/*	
 * These routines can be called very early in the life of the PROM.
 * - They cannot use memory or stack.  
 * - They may only use registers AT, v0-v1, a0-a3, t0-t3
 */

#include <mips/asm.h>
#include <mips/regdef.h>
#include "sbdser.h"

#ifndef MPSC_CHAN0
#define MPSC_CHAN0	(MPSC_BASE+0)
#define MPSC_CHAN1	(MPSC_BASE+4)
#endif

#ifndef MPSC_HZ
#define MPSC_HZ		6144000
#endif

#ifndef DEFSPEED
#define DEFSPEED	9600	/* default console speed */
#endif
	
#ifndef DBGSPEED
#define DBGSPEED	9600	/* default debug speed */
#endif
	
	.sdata
/* remote debugging port number */
EXPORTS(_sbddbgport, 4)
	.word	1

	.rdata
/* device table */
EXPORTS(_sbdsertab, SS_SIZE * 2)
	.word	PA_TO_KVA1(MPSC_CHAN0), reset, init, params, rxpoll, txrdy, tx
	.word	PA_TO_KVA1(MPSC_CHAN1),     0, init, params, rxpoll, txrdy, tx

/* number of ports */
EXPORTS(_sbdnser, 4)
	.word	2

/* default initial port speeds */
EXPORTS(_sbdspeed, 2*4)
	.word	DEFSPEED, DBGSPEED
	

/* macros for programming uPD72001 */
#define MPSCPUTREG(n,v) \
	li a1,n; li a2,v; bal _mpscputreg

#define MPSCGETREG(n) \
	li a1,n; bal _mpscgetreg

#define MPSCPUTBAUD(n,v) \
	li a1,n; move a2,v; bal	_mpscputbaud

#define TXWAIT		10000


/*
 * reset (chan, reloc)
 *
 * Once only device reset at power-on
 */
SLEAF(reset)
	move	t3,ra				# save return address

	li	t0,PA_TO_KVA1(MPSC_BASE)	# channel A
	li	t1,PA_TO_KVA1(MPSC_BASE+4)	# channel B

	/* single read to get in known state */
	move	a0,t0
	MPSCGETREG(mSR0)
	move	a0,t1
	MPSCGETREG(mSR0)

	/* force out of standby mode by writing to register 0 */
	move	a0,t0
	MPSCPUTREG(mCR0, mCR0_NOP)

	/* global chip reset */
	move	a0,t0
	MPSCPUTREG(mCR0, mCR0_RESETCHN)
	move	a0,t1
	MPSCPUTREG(mCR0, mCR0_RESETCHN)

        /* select interrupt operation (!) and make receivers have priority */
	move	a0,t0
	MPSCPUTREG(mCR2A, mCR2A_PRIRX|mCR2A_INTAINTB|mCR2A_SVEC|mCR2A_B1)
	move	a0,t1
	MPSCPUTREG(mCR2B, 0)

	move	ra,t3
	j	ra
SEND(reset)


/*
 * init (chan, reloc)
 *
 * Per channel initialisation.
 */
SLEAF(init)
	move	t3,ra				# save return address
	MPSCGETREG(mSR0)			# get into known state
	MPSCPUTREG(mCR0, mCR0_NOP)		# get out of standby mode
	MPSCPUTREG(mCR0, mCR0_RESETCHN)		# reset channel
	li	t2,9600
	b	2f
SEND(init)


/*
 * params (chan, speed, reloc)
 *
 * program new speed, return 0 on success
 */
SLEAF(params)
	move	t3,ra				# save return address
	move	t2,a1				# save baud rate

	/* wait for all tx data to be sent */
	li	t0,TXWAIT
1:	MPSCGETREG(mSR1)
	and	v0,mSR1_ALLSENT
	bnez	v0,2f
	subu	t0,1
	bnez	t0,1b
	
	/* convert baud rate in t2 into register value */
2:	beqz	t2,..fail
	li	t1,MPSC_HZ/(MPSCX*2)
	divu	t2,t1,t2
	subu	t2,2
	blez	t2,..fail
	
	/* disable interrupts (sic) */
	MPSCPUTREG(mCR1, 0)

	/* fixed 8-N-1 mode */
	MPSCPUTREG(mCR4, mCR4_CLKX | mCR4_STOP1)
	MPSCPUTREG(mCR5, mCR5_DTR | mCR5_RTS | mCR5_TXEN | mCR5_8BIT)
	MPSCPUTREG(mCR3, mCR3_AUTO | mCR3_RXEN | mCR3_8BIT)

	/* stop BRG */
	MPSCPUTREG(mCR14, mCR14_NOP)

	/* set BRG */
	MPSCPUTBAUD(mCR12_TXBRGSET, t2)
	MPSCPUTBAUD(mCR12_RXBRGSET, t2)
	MPSCPUTREG(mCR15, mCR15_RXRXBRG |mCR15_TXTXBRG | \
				mCR15_TRXCO | mCR15_TRXCTXCLK)

	/* restart BRG */
	MPSCPUTREG(mCR14, mCR14_NOP | mCR14_BRGSYS | \
				mCR14_BRGRX | mCR14_BRGTX)

	MPSCPUTREG(mCR10, mCR10_NRZ)
	MPSCPUTREG(mCR12, mCR12_TXTRXC)
	MPSCPUTREG(mCR11, 0);

	move	ra,t3
	move	v0,zero				# indicate success
	j	ra
	
..fail:	li	v0,1				# indicate failure
	j	ra
SEND(params)


/*
 * rxpoll (chan, canblock, reloc) 
 *
 * poll for received character, return zero if none available
 */
SLEAF(rxpoll)
	lw	v0,CMD(a0)			# get SR0
	and	v0,mSR0_RXRDY			# rx ready?
	beqz	v0,1f				# no - return 0
	lw	v0,DATA(a0)			# yes - get data
	srl	v0,24				# extract character 
	or	v0,0x100			# make it non-zero
1:	j	ra
SEND(rxpoll)


/* 
 * txrdy (chan, reloc) 
 *
 * return non-zero if ready to transmit
 */
SLEAF(txrdy)
	lw	v0,CMD(a0)			# get SR0
	and	v0,mSR0_TXEMPTY			# txrdy?
	j	ra
SEND(txrdy)


/* 
 * tx (chan, data, reloc) 
 *
 * transmit data
 */
SLEAF(tx)
	/* blocking transmit, with timeout */
	li	t0,TXWAIT			# timeout
1:	lw	v0,CMD(a0)			# get SR0
	and	v0,mSR0_TXEMPTY			# txrdy?
	bnez	v0,1f				# yup - go and write
	subu	t0,1				# continue until timeout
	bnez	t0,1b

1:	sll	t0,a1,24			# endianness independent
	sw	t0,DATA(a0)			#   byte write!
	WBFLUSH
	j	ra
SEND(tx)



/*
 * write uPD72001 register
 * trashes: a1
 * depth: 0
 */
SLEAF(_mpscputreg)
	beq	a1,mCR0,1f
	sll	a1,24
	or	a1,mCR0_REG	/* set register number */
	sw	a1,CMD(a0)
	WBFLUSH
1:	sw	a2,CMD(a0)
	WBFLUSH
	j	ra
END(_mpscputreg)

/*
 * read uPD72001 register
 * trashes: a1,v0
 * depth: 0
 */
SLEAF(_mpscgetreg)
	beq	a1,mSR0,1f
	sll	a1,24
	or	a1,mCR0_REG	/* set register number */
	sw	a1,CMD(a0)
	WBFLUSH
1:	lw	v0,CMD(a0)
	j	ra
END(_mpscgetreg)


/*
 * write uPD72001 baudrate
 * trashes: v0
 * depth: 0
 */
SLEAF(_mpscputbaud)
	li	v0,((mCR12<<24)|mCR0_REG)
	sw	v0,CMD(a0)
	WBFLUSH
	sw	a1,CMD(a0)
	WBFLUSH
	sll	v0,a2,24
	sw	v0,CMD(a0)
	WBFLUSH
	sll	v0,a2,16
	sw	v0,CMD(a0)
	WBFLUSH
	lw	zero,CMD(a0)
	j	ra
END(_mpscputbaud)
