.ident "kit/share/ns16550.s: Copyright (c) MIPS Technologies, Inc. All rights reserved."
	
/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * share/ns16550.s: NS16550/450 uart driver
 */


/*	
 * These routines can be called very early in the life of the PROM.
 * - They cannot use memory or stack.  
 * - They may only use registers AT, v0-v1, a0-a3, t0-t3

 * All functions here are called via _sbdsertab, of which entry
 * number 0 is the console.
 *
 * Each function is called with its first argument being the first
 * word from its table entry.  In this case the KSEG1 address
 * of the UART.
 *
 * Each function is called with its last argument being the romlow
 * relocation factor, which should be added to any addresses in
 * .text or .rdata (after clearing bit 0).  If it is non-zero then
 * DRAM may not have been initialised yet (in particular the 
 * .data segment may not have been copied there) so don't try to 
 * use it.
 */

#include <mips/asm.h>
#include <mips/regdef.h>
#include <mips/cpu.h>
	
#include "sbdser.h"
#include "sbd.h"
#include "ns16550.h"

#if defined(UART3_BASE)
#define NUART 		4
#elif defined(UART2_BASE)
#define NUART		3
#elif defined(UART1_BASE)
#define NUART		2				
#else
#define NUART 		1
#endif	

#ifndef NS16550_HZ
#define NS16550_HZ	1843200
#endif

#ifndef DEFSPEED
#define DEFSPEED	9600		/* default console speed */
#endif
	
#ifndef DBGSPEED
#define DBGSPEED	DEFSPEED	/* default debug speed */
#endif
	
#ifndef DBGPORT	
#define DBGPORT		(NUART-1)
#endif
	
	.sdata
/* remote debugging port number (can be patched) */
EXPORTS(_sbddbgport, 4)
	.word	DBGPORT
	
	.rdata
/* device table */
EXPORTS(_sbdsertab, SS_SIZE * NUART)
	.word	PA_TO_KVA1(UART0_BASE), reset, init, params, rxpoll, txrdy, tx
#ifdef UART1_BASE	
	.word	PA_TO_KVA1(UART1_BASE), reset, init, params, rxpoll, txrdy, tx
#endif
#ifdef UART2_BASE	
	.word	PA_TO_KVA1(UART2_BASE), reset, init, params, rxpoll, txrdy, tx
#endif
#ifdef UART3_BASE	
	.word	PA_TO_KVA1(UART3_BASE), reset, init, params, rxpoll, txrdy, tx
#endif
	
/* number of ports */
EXPORTS(_sbdnser, 4)
	.word	NUART

/* default initial port speeds */
EXPORTS(_sbdspeed, NUART*4)
#if NUART > 1
	.word	DEFSPEED : NUART - 1
	.word	DBGSPEED
#else	
	.word	DEFSPEED
#endif
	
#ifdef NS16550_TXWAIT	
#define TXWAIT		NS16550_TXWAIT
#else	
#define TXWAIT		10000
#endif

#ifndef NS16550_INB
#define NS16550_INB(dst,offs,base) \
	lbu	dst,offs(base)
#endif
	
#ifndef NS16550_OUTB
#define NS16550_OUTB(src,offs,base) \
	sb	src,offs(base)
#endif
	
	
/*
 * reset (chan, reloc)
 *
 * Once only device reset at power-on
 */
SLEAF(reset)
#ifdef NS16550_RESET	
	NS16550_RESET(a0)
#endif		
	NS16550_OUTB(zero,IER,a0)	
	NS16550_OUTB(zero,CFCR,a0)	
	j	ra
SEND(reset)


/*
 * init (chan, reloc)
 *
 * Per channel (re)initialisation.
 */
SLEAF(init)
	# enable 16550 fifo if it is there
	li	t0,FIFO_ENABLE|FIFO_RCV_RST|FIFO_XMT_RST|FIFO_TRIGGER_4
	NS16550_OUTB(t0,FIFO,a0)
	
	# 100us delay 
	.set	noreorder
	li	t0,CACHEUS(100)
1:	bnez	t0,1b
	subu	t0,1
	.set	reorder
	
	li	a1,DEFSPEED
	/* drop through */
SEND(init)


/*
 * params (chan, speed, reloc)
 *
 * program new speed, return 0 on success
 */
SLEAF(params)
	/* wait for all tx data to be sent */
	li	t0,TXWAIT
	li	t2,(LSR_TXRDY|LSR_TSRE)		# tx ready and empty?
1:	NS16550_INB(t1,LSR,a0)
	and	t1,t2
#if defined(IDT334A) || defined(IDT334) || defined(IDT332) || defined(IDT33X)
	beq	t1,t2,2f			# both should be set
#else	
	bnez	t1,2f
#endif	
	subu	t0,1
	bnez	t0,1b
	
	/* convert baud rate in a1 into register value */
2:	beqz	a1,..fail			# don't want divide error
	li	t2,NS16550_HZ/16		# brtc = CLK/16/speed
	divu	t2,a1
	blez	t2,..fail			# brtc must be > 0
	
	li	t0,CFCR_DLAB			# select brtc divisor
	NS16550_OUTB(t0,CFCR,a0)			
	NS16550_OUTB(t2,DLL,a0)			# store divisor lsb
	srl	t2,8	
	NS16550_OUTB(t2,DLM,a0)			# store divisor msb
	li	t0,CFCR_8BITS			# set 8N1 mode
	NS16550_OUTB(t0,CFCR,a0)
	
	li	t0,MCR_DTR|MCR_RTS|MCR_IENABLE	# enable DTR & RTS
	NS16550_OUTB(t0,MCR,a0)
	
	li	t0,IER_ERXRDY			# enable receive interrupt(!)
	NS16550_OUTB(t0,IER,a0)
	
	move	v0,zero				# indicate success
	j	ra
	
..fail:	li	v0,1
	j	ra
SEND(params)


/*
 * rxpoll (chan, canblock, reloc) 
 *
 * poll for received character, return zero if none available
 */
SLEAF(rxpoll)
	NS16550_INB(v0,LSR,a0)			# get LSR
	and	v0,LSR_RXRDY			# rx ready?
	beqz	v0,1f				# no - return 0
	NS16550_INB(v0,DATA,a0)			# yes - get data
	and	v0,0xff				# extract character
	or	v0,0x100			# make it non-zero
1:	j	ra
SEND(rxpoll)


/* 
 * txrdy (chan, reloc) 
 *
 * return non-zero if ready to transmit
 */
SLEAF(txrdy)
	NS16550_INB(v0,LSR,a0)			# get LSR
	li	t1,(LSR_TXRDY|LSR_TSRE)		# tx ready or empty?
	and	v0,t1
#if defined(IDT334A) || defined(IDT334) || defined(IDT332) || defined(IDT33X)
	seq	v0,v0,t1			# both bits must be set
#endif	
	j	ra
SEND(txrdy)


/* 
 * tx (chan, data, reloc) 
 *
 * transmit data
 */
SLEAF(tx)
	/* blocking transmit, with timeout */
	li	t0,TXWAIT			# timeout
	li	t1,(LSR_TXRDY|LSR_TSRE)		# tx ready or empty?
1:	NS16550_INB(v0,LSR,a0)			# get LSR
	and	v0,t1
#if defined(IDT334A) || defined(IDT334) || defined(IDT332) || defined(IDT33X)
	beq	v0,t1,1f			# both bits must be set
#else	
	bnez	v0,1f				# either set is OK
#endif	
	subu	t0,1				# continue until timeout
	bnez	t0,1b
1:	NS16550_OUTB(a1,DATA,a0)		# write data
	j	ra
SEND(tx)
