#ident "kit/share/nsdelay.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2001-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * share/nsdelay.c: generic low-level delay function
 */


#include <stddef.h>
#include <mips/cpu.h>
#include <mips/prid.h>
#include "kit_impl.h"
#include "sbd.h"

#ifdef C0_COUNT

#ifndef _SBD_CLK_DIV
#define _SBD_CLK_DIV	_sbd_cyclespercount ()
#endif

#ifndef _SBD_CLK_MULT
#define _SBD_CLK_MULT	1
#endif

void
_sbd_nsdelay (unsigned long ns)
{
    static unsigned int nspercount;
    unsigned long start, count;

    if (nspercount == 0) {
	nspercount = 1000000000 / _sbd_pipefreq ();
	nspercount = (nspercount * _SBD_CLK_DIV) / _SBD_CLK_MULT;
    }

    /* calculate number of counter ticks (rounding up) */
    count = (ns + nspercount / 2) / nspercount;

    /* discount cost of divide (~32 cycles) */
    if (count < 32)
	return;
    count -= 32;

    start = mips_getcount ();
    while ((mips_getcount () - start) < count)
	continue;
}

void (_sbd_usdelay) (unsigned long us)
{
    _sbd_nsdelay (us * 1000);
}

void (_sbd_msdelay) (unsigned long ms)
{
    while (ms-- != 0)
	_sbd_nsdelay (1000000);
}

#endif /* C0_COUNT */
