/*
 * Copyright (c) 1997-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * nvenv.h: generic Algo-style environment in r/w non-volatile memory.
 */


#ifndef _ENVIRON_H_
#define _ENVIRON_H_

#ifndef ENVOFFSET
#define ENVOFFSET 0
#endif

/* default offsets of reserved locations */
#ifndef ENVOFF_MAGIC
#define ENVOFF_MAGIC		0	/* 2 magic value */
#define ENVOFF_CSUM		2	/* 2 environment checksum */
#define ENVOFF_SIZE		4	/* 2 size of environment */
#ifdef ENVOFF_P4000_COMPAT
/* Algorithmics P4000 (old layout) */
#define ENVOFF_ETHADDR		6	/*  6 decoded ethernet address */
#define ENVOFF_STATE  		12	/*  1 state of NVRAM */
#define ENVOFF_CENTURY		13	/*  1 century byte */
#define ENVOFF_TEST		14	/*  2 cold start test bytes */
#define ENVOFF_SPARE  		16	/* 48 spare bytes */
#define ENVOFF_BASE		64	/*  ? start of environment */
#else /* !ENVOFF_P4000_COMPAT */
/* Algorithmics P4032 (new layout) */
#define ENVOFF_STATE  		6	/* 1 state of environment */
#define ENVOFF_SPARE  		7	/* 1 spare byte */
#define ENVOFF_TEST  		8	/* 2 test word */
#define ENVOFF_BASE		10	/* ? start of environment proper */
#endif /* !ENVOFF_P4000_COMPAT */
#endif /* !ENVOFF_NAGIC */

/* value for ENVOFF_MAGIC */
#define ENV_MAGIC		0xdeaf

/* values for ENVOFF_STATE */
#define ENVSTATE_UNKNOWN	-1
#define ENVSTATE_OK		0
#define ENVSTATE_UNINITIALISED	1
#define ENVSTATE_RECOVERABLE	2
#define ENVSTATE_UNRECOVERABLE	3
#define ENVSTATE_FORCERESET	4

#endif /*_ENVIRON_H_*/
