#ident "kit/share/r361clk.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * r361clk.c: generic R36100 timing support functions
 */


/* 
 * This file will usually be #include'd into an SBD skeleton file
 * which must define _sbd_cpufreq () which returns the frequency
 * at which this R36100's SYSCLK is driven.
 */

#include <stddef.h>
#include <sys/time.h>
#include <sys/clock.h>
#include <mips/cpu.h>
#include <mips/xcpt.h>
#include <assert.h>
#include "kit_impl.h"

#include <idt/r36100.h>

#define counttousec(tick)	(((tick) * 100) / (_clk_tck / 10000))
#define usectocount(usecs)	((_clk_tck / 10000) * ((usecs) / 100))

static clock_t		(*tickfunc)(long, struct xcptcontext *);
static clock_t		tick;
static clock_t		maxtick;
static unsigned short	counter;

/* real timer resolution in Hz */
clock_t			_clk_tck;

void
_sbd_settimer (clock_t newtick)
{
    p36100;
    assert (newtick <= maxtick);
    if (newtick != tick) {
        tick = newtick;
	/* reprogram timer (note this resets timer, so don't do it too often) */
	counter = usectocount(newtick);
	TimerControlReg0 |= TimerControlTimerDis;
	TimerCompareReg0 = counter;
	TimerControlReg0 &= ~TimerControlTimerDis; /* restart clock */
    }
}

static int
clkintr (int arg, struct xcptcontext *xcp)
{
    p36100;
    clock_t newtick;

    /* Acknowledge timer interrupt */
    ExpIntPendReg0 = ExpInt0TimerTC0;

    /* call timer tick function */
    newtick = (*tickfunc) (1, xcp);
    
    /* update parameters if they have changed */
    if (newtick != tick)
      _sbd_settimer (newtick);

    return 0;
}


clock_t
_sbd_gettimer ()
{
    p36100;
    clock_t usec;
    unsigned short count;

#if 1
    count = TimerCountReg0;
    if (ExpIntPendReg0 & ExpInt0TimerTC0 && count < counter)
      /* counter rolled over, but no interrupt taken yet */
      count += counter;

    usec = counttousec(count);
    return usec;
#else
    return 0;
#endif
}


clock_t
_sbd_inittimer (clock_t (*func)(long, struct xcptcontext *))
{
    p36100;
    int prescale;
    unsigned long freq;

    if (!_clk_tck) {
	/* Clock runs at CPU frequency */
        freq = _sbd_cpufreq ();

	/* adjust prescaler so clock ticks about 1MHz */
	prescale = 0;
	while (freq >= 2000000) {
	  freq /= 2;
	  prescale++;
	}
	_clk_tck = freq;

	/* work out timing parameters based on that */
	maxtick = counttousec(65535);

	TimerControlReg0 = TimerControlTC | TimerControlTimerDis | prescale;
	ExpIntMaskReg0 = ExpInt0TimerTC0;

    }

    tickfunc = func;
    intrupt (7, clkintr, 0);	/* assume no one else wants to use it... */
    mips_bissr (SR_IM7);	/* enable interrupt now */

    return maxtick;
}
