#ident "kit/share/r4kwatch.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2000-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * r4kwatch.c: R4000 family watchpoint support for MIPS SDE kit
 */


#if #cache(r4k) || #cache(r54)
#include <sys/types.h>
#include <mips/r4kc0.h>
#include <mips/r4kxcpt.h>
#include <mips/cpu.h>
#include <mips/xcpt.h>
#include <mips/prid.h>
#include <mips/watchpoint.h>

#include "kit_impl.h"

static int	r4kwp_init (unsigned int);
static void	r4kwp_setup (struct mips_watchpoint *);
static int 	r4kwp_hit (const struct xcptcontext *, vaddr_t *, size_t *);
static int	r4kwp_set (struct mips_watchpoint *);
static int	r4kwp_clear (struct mips_watchpoint *);
static void	r4kwp_remove (void);
static void	r4kwp_insert (void);
static void	r4kwp_reset (void);

const struct mips_watchpoint_funcs _r4k_watchpoint_funcs = {
    r4kwp_init,
    r4kwp_setup,
    r4kwp_hit,
    r4kwp_set,
    r4kwp_clear,
    r4kwp_remove,
    r4kwp_insert,
    r4kwp_reset
};

static unsigned long	r4kwp_reg;
static unsigned int	r4kwp_type;

static int	
r4kwp_init (unsigned int prid)
{
    switch ((prid >> 8) & 0xff) {
    case PRID_R4000:
    case PRID_R4100:
    case PRID_R4200:
    case PRID_R4300:
    case PRID_R5400:
	/* r4000 watchpoint supported */
	return 1;
    case PRID_R4600:
    case PRID_R4700:
    case PRID_RC6447X:
	/* recognised, but don't support watchpoints */
	return 0;
    default:
	return -1;
    }
}


static void
r4kwp_setup (struct mips_watchpoint *wp)
{
    wp->capabilities = MIPS_WATCHPOINT_DWORD | MIPS_WATCHPOINT_R
	| MIPS_WATCHPOINT_W;
}


static int
r4kwp_hit (const struct xcptcontext *xcp, vaddr_t *pva, size_t *plen)
{
    int hit = 0;

    if (XCP_CODE (xcp) == XCPTWATCH)
	hit = _mips_watchpoint_address (xcp, r4kwp_type, pva, plen);

    return hit;
}


static int
r4kwp_set (struct mips_watchpoint *wp)
{
    if (((wp->va & 7) + wp->len - 1) > 7)
	return 0;
    wp->mask = 7;

    r4kwp_reg = wp->pa & WATCHLO_PA;
    r4kwp_type = wp->type;
    if (wp->type & MIPS_WATCHPOINT_R)
	r4kwp_reg |= WATCHLO_R;
    if (wp->type & MIPS_WATCHPOINT_W)
	r4kwp_reg |= WATCHLO_W;
    return 1;
}


static int
r4kwp_clear (struct mips_watchpoint *wp)
{
    r4kwp_reg = 0;
    return 1;
}


static void	
r4kwp_remove (void)
{
    if (r4kwp_reg) {
	mips_setwatchlo (0);
	_mips_watchlo = 0;
    }
}


static void
r4kwp_insert (void)
{
    /* exception handler restores watchpoint reg */
    if (r4kwp_reg)
	_mips_watchlo = r4kwp_reg;
}


static void
r4kwp_reset (void)
{
    r4kwp_reg = 0;
    _mips_watchlo = 0;
    mips_setwatchlo (0);
    mips_setwatchhi (0);
}
#endif
