.ident "kit/share/r54cache.h: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * r54cache.h: Vr5400 cache support functions for MIPS SDE
 */


#if #cache(r54)
		
#define _BOOTCODE
#include <mips/asm.h>
#include <mips/regdef.h>
	
#assert cpu(r54xx)	
#include <mips/r5kc0.h>


/*
 * R5400 cache operations.
 *
 * The _flush and _clean functions are complex composites that do whatever
 * is necessary to flush/clean ALL caches, in the quickest possible way.
 * The other functions are targetted explicitly at a particular cache
 * I or D; it is up to the user to call the correct set of functions
 * for a given system.
 *
 * NOTE:	
 * This code is untested - we haven't seen real silicon yet.
 */

IMPORT(mips_icache_size,4)
IMPORT(mips_icache_linesize,4)
IMPORT(mips_icache_ways,4)
	
IMPORT(mips_dcache_size,4)
IMPORT(mips_dcache_linesize,4)
IMPORT(mips_dcache_ways,4)
	
/* linesize is fixed at 32 bytes for all caches */	
#define LINESIZE	32
	
/*
 * Macros to automate cache operations
 */

#define addr	t0
#define maxaddr	t1
#define mask	t2

/* virtual cache op: no limit on size of region */
#define vcacheop(kva, n, op)	\
	.set	noreorder ;		\
	/* check for bad size */	\
 	blez	n,11f ;			\
	addu	maxaddr,kva,n ;		\
	/* align to line boundaries */	\
	li	mask,~(LINESIZE-1) ;	\
	and	addr,kva,mask ;		\
	addu	maxaddr,-1 ;		\
	and	maxaddr,mask ;		\
	/* the cacheop loop */		\
10: 	cache	op,0(addr) ;	 	\
	bne     addr,maxaddr,10b ;	\
	addu   	addr,LINESIZE ;		\
11:	.set	reorder

/* indexed cache op: no limit on size of region */
/* NOTE both sets are operated on */
#define icacheop(kva, n, op)	\
	.set	noreorder ;		\
	/* check for bad size */	\
 	blez	n,11f ;			\
	srl	maxaddr,n,1 ;		\
	addu	maxaddr,kva ;		\
	addu	maxaddr,-1 ;		\
	/* align to line boundaries */	\
	li	mask,~(LINESIZE-1) ;	\
	and	addr,kva,mask ;		\
	and	maxaddr,mask ;		\
	/* the cacheop loop */		\
10: 	cache	op,0(addr) ;	 	\
	nop;				\
	cache	op,1(addr);		\
	bne     addr,maxaddr,10b ;	\
	addu   	addr,LINESIZE ;		\
11:	.set	reorder

/* indexed cache op: region limited to set size */
/* NOTE both sets are operated on */
#define icacheop_lim(kva, n, cachesize, op) \
	move	t3,n ;			\
	srl	t2,cachesize,1 ;	\
	bltu	t3,t2,12f ;		\
	move	t3,t2 ;			\
12:	sll	t3,1 ;			\
	icacheop(kva,t3,op)
	
	
#if defined(IN_PMON) || defined(ITROM) || defined(_SDE_CACHE_EXTRA)
/* caches are always sized first */
#define SIZE_CACHE(reg,which)		\
	lw	reg,which;		\
	blez	reg,9f
#else	
#define SIZE_CACHE(reg,which)		\
	sync;				\
	li	a2,LINESIZE;		\
	lw	reg,which;		\
	move	v1,ra;			\
	bgez	reg,9f;			\
	bal	r54_size_cache;		\
	lw	reg,which;		\
	move	ra,v1;			\
9:	blez	reg,9f
#endif
	


#ifndef _SDE_CACHE_EXTRA
	
#define icachesize	t2
#define dcachesize	t3

#endif /* _SDE_CACHE_EXTRA*/
		
#endif /* #cache(r54) */
