/*
 * Copyright (c) 1998-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * share/reginit.h: generic hardware initialisation engine
 */


#ifndef __REGINIT_H
#ifdef __cplusplus
extern "C" {
#endif
#define __REGINIT_H

#define MOD_MASK	0x03
#define MOD_B		0x00 /* byte "modifier" */
#define MOD_H		0x01 /* halfword "modifier" */
#define MOD_W		0x02 /* word "modifier" */
#if __mips64
#define MOD_D		0x03 /* doubleword "modifier" */
#endif

#define OP_MASK		0x1c
#define	OP_EXIT		0x00 /* exit (status) */
#define OP_DELAY	0x04 /* delay (cycles) */
#define OP_RD		0x08 /* read (addr) */
#define OP_WR		0x0c /* write (addr, val) */
#define OP_RMW		0x10 /* read-modify-write (addr, and, or) */
#define OP_WAIT		0x14 /* wait (addr, mask, value) */
#define OP_JUMP		0x18 /* jump (addr) */

#define A1_MASK		0x40
#define A1_NONE		0x00
#define A1_MOD	        0x40

#define A2_MASK		0x80
#define A2_NONE		0x00
#define A2_MOD		0x80

#ifdef __ASSEMBLER__

#if __mips64
#define rword	.dword
#define lr	ld
#else
#define rword	.word
#define lr	lw
#endif

#define REGINIT_DATA_B	.byte
#define REGINIT_DATA_H	.half
#define REGINIT_DATA_W	.word
#define REGINIT_DATA_D	.dword

#define WR_INIT(mod,addr,val) \
        .align	0; \
	.byte	OP_WR|_ASMCONCAT(MOD_,mod)|A1_MOD|A2_NONE; \
	.word	PA_TO_KVA1(addr);\
    	_ASMCONCAT(REGINIT_DATA_,mod) (val); \
	.rdata
	
#define RD_INIT(mod,addr) \
        .align	0; \
    	.byte	OP_RD|_ASMCONCAT(MOD_,mod)|A1_NONE|A2_NONE; \
	.word	PA_TO_KVA1(addr); \
	.rdata

#define RMW_INIT(mod,addr,and,or) \
        .align	0; \
	.byte	OP_RMW|_ASMCONCAT(MOD_,mod)|A1_MOD|A2_MOD; \
	.word	PA_TO_KVA1(addr); \
	_ASMCONCAT(REGINIT_DATA_,mod) (or),(and); \
	.rdata
	
#define WAIT_INIT(mod,addr,mask,val) \
        .align	0; \
	.byte	OP_WAIT|_ASMCONCAT(MOD_,mod)|A1_MOD|A2_MOD; \
	.word	PA_TO_KVA1(addr); \
	_ASMCONCAT(REGINIT_DATA_,mod) 	(val),(mask); \
	.rdata

#define JUMP_INIT(addr) \
        .align	0; \
88:	.byte	OP_JUMP|A1_NONE|A2_NONE; \
	.word	addr; \
	.rdata
	
#define DELAY_INIT(cycles) \
        .align	0; \
	.byte	OP_DELAY|A1_NONE|A2_NONE; \
	.word	(cycles); \
	.rdata
	
#define EXIT_INIT(status) \
        .align	0; \
	.byte	OP_EXIT|A1_NONE|A2_NONE; \
	.word	(status); \
	.rdata

#else

#error reginit.h not defined for C code yet

#endif

#ifdef __cplusplus
}
#endif
#endif /* __REGINIT_H */


