.ident "kit/share/reginit.s: Copyright (c) MIPS Technologies, Inc. All rights reserved."
	
/*
 * Copyright (c) 1998-2005 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * share/reginit.s: generic hardware initialisation engine
 */

	
#include "reginit.h"	
	
#define reg_ptr		a0
#define reg_op		t0
#define reg_size	t1
#define reg_tmp		t2
#define reg_a0		t3
#define reg_a1		ta0
#define reg_a2		ta1
#define reg_val		ta2
				
#define	reg_ifsize(mod,label) \
	and	reg_tmp,reg_size,(1 << (mod)); \
	bnez	reg_tmp,label

LEAF(reginit)			/* local name */
XLEAF(_sbd_reginit)		/* global name */
	
..next:	lbu	reg_op, 0(reg_ptr)
	ulw	reg_a0, 1(reg_ptr)
	addu	reg_ptr,5
	
	and	reg_tmp,reg_op,MOD_MASK
	li	reg_size,1
	sll	reg_size,reg_tmp
	
	and	reg_tmp,reg_op,A1_MASK|A2_MASK
	beq	reg_tmp,A2_NONE|A1_NONE,..argsdone
	beq	reg_tmp,A2_NONE|A1_MOD,..a1mod
	/*beq	reg_tmp,A2_MOD|A1_MOD,..a1a2mod*/
	
..a1a2mod:
#if __mips64	
	reg_ifsize(MOD_D, ..a2d)
#endif	
	reg_ifsize(MOD_W, ..a2w)
	reg_ifsize(MOD_H, ..a2h)
..a2b:	lbu	reg_a2,0(reg_ptr)	# second arg comes first
	addu	reg_ptr,1
	b	..a1b	
..a2h:	ulhu	reg_a2,0(reg_ptr)	
	addu	reg_ptr,2
	b	..a1h
..a2w:	ulw	reg_a2,0(reg_ptr)
	addu	reg_ptr,4
#if __mips64	
	dsll	reg_a2,32		# truncate
	dsrl	reg_a2,32
#endif	
	b	..a1w
#if __mips64	
..a2d:	uld	reg_a2,0(reg_ptr)	
	addu	reg_ptr,8
	b	..a1d
#endif
	
..a1mod:
#if __mips64	
	reg_ifsize(MOD_D, ..a1d)
#endif	
	reg_ifsize(MOD_W, ..a1w)
	reg_ifsize(MOD_H, ..a1h)
..a1b:	lbu	reg_a1,0(reg_ptr)
	addu	reg_ptr,1
	b	..argsdone
..a1h:	ulhu	reg_a1,0(reg_ptr)	
	addu	reg_ptr,2
	b	..argsdone
..a1w:	ulw	reg_a1,0(reg_ptr)	
	addu	reg_ptr,4
#if __mips64	
	dsll	reg_a1,32		# truncate
	dsrl	reg_a1,32
#endif	
	b	..argsdone
#if __mips64	
..a1d:	uld	reg_a1,0(reg_ptr)	
	addu	reg_ptr,8
	b	..argsdone
#endif
		
..argsdone:		
	and	reg_tmp,reg_op,OP_MASK
	beq	reg_tmp,OP_EXIT,..op_exit
	beq	reg_tmp,OP_DELAY,..op_delay
	beq	reg_tmp,OP_RD,..op_rd
	beq	reg_tmp,OP_WR,..op_wr
	beq	reg_tmp,OP_RMW,..op_rmw
	beq	reg_tmp,OP_WAIT,..op_wait
	/*beq	reg_tmp,OP_JUMP,..op_jump*/
	
	/* 
	 * JUMP(ADDR) 
	 */
..op_jump:		
	move	reg_ptr,reg_a0
	b	..next
	
	/* 
	 * DELAY(CYCLES) 
	 */
..op_delay:		
	.set noreorder
1:	bnez	reg_a0,1b
	subu	reg_a0,1
	.set reorder
	b	..next
	
	/* 
	 * READ(ADDR) 
	 */
..op_rd:		
#if __mips64	
	reg_ifsize(MOD_D, ..rd_d)
#endif	
	reg_ifsize(MOD_W, ..rd_w)
	reg_ifsize(MOD_H, ..rd_h)
..rd_b:	lbu	reg_val,0(reg_a0)
	b	..next
..rd_h:	lhu	reg_val,0(reg_a0)
	b	..next
..rd_w:	lw	reg_val,0(reg_a0)
	b	..next
#if __mips64
..rd_d:	ld	reg_val,0(reg_a0)
	b	..next
#endif
	
	/* 
	 * WRITE(ADDR,VAL) 
	 */
..op_wr:		
#if __mips64	
	reg_ifsize(MOD_D, ..wr_d)
#endif	
	reg_ifsize(MOD_W, ..wr_w)
	reg_ifsize(MOD_H, ..wr_h)
..wr_b:	sb	reg_a1,0(reg_a0)
	b	..next
..wr_h:	sh	reg_a1,0(reg_a0)
	b	..next
..wr_w:	sw	reg_a1,0(reg_a0)	
	b	..next
#if __mips64	
..wr_d:	sd	reg_a1,0(reg_a0)
	b	..next
#endif
		
	/* 
	 * RMW(ADDR,AND,OR) 
	 */
..op_rmw:		
#if __mips64	
	reg_ifsize(MOD_D, ..rmw_d)
#endif	
	reg_ifsize(MOD_W, ..rmw_w)
	reg_ifsize(MOD_H, ..rmw_h)
..rmw_b:lbu	reg_val,0(reg_a0)
	and	reg_a1,reg_val
	or	reg_a1,reg_a2
	b	..wr_b
..rmw_h:lhu	reg_val,0(reg_a0)
	and	reg_a1,reg_val
	or	reg_a1,reg_a2
	b	..wr_h
..rmw_w:lw	reg_val,0(reg_a0)
	and	reg_a1,reg_val
	or	reg_a1,reg_a2
	b	..wr_w
#if __mips64
..rmw_d:ld	reg_val,0(reg_a0)
	and	reg_a1,reg_val
	or	reg_a1,reg_a2
	b	..wr_d
#endif
	
	/* 
	 * WAIT(ADDR,MASK,VAL) 
	 */
..op_wait:		
#if __mips64	
	reg_ifsize(MOD_D, ..wait_d)
#endif	
	reg_ifsize(MOD_W, ..wait_w)
	reg_ifsize(MOD_H, ..wait_h)
..wait_b:lbu	reg_val,0(reg_a0)
	b	1f
..wait_h:lh	reg_val,0(reg_a0)
	b	1f
..wait_w:lw	reg_val,0(reg_a0)
	b	1f
#if __mips64	
..wait_d:ld	reg_val,0(reg_a0)
#endif	
1:	and	reg_val,reg_a1
	bne	reg_val,reg_a2,..op_wait
	b	..next
	
	/* 
	 * EXIT(STATUS)
	 */
..op_exit:		
	move	v0,reg_a0
	
..done:	j	ra	
SEND(reginit)
