/*
 * Copyright (c) 1999-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * share/s2681.s: Signetics SCN2681 / Motorola 68681 / UMC UM2681 UART header
 */


#ifndef _S2681_H_
#define _S2681_H_

#ifndef S2681REGSZ
/* bytes between each register */
#error please define S2681REGSZ
#endif

#ifndef S2681BANK
/* distance between banks of registers (usually 4 * register size) */
#define S2681BANK	(S2681REGSZ * 4)
#endif

#ifndef S2681GAP
/* gap from end of one bank to start of next (usually 0) */
#define S2681GAP	(S2681BANK - (S2681REGSZ * 4))
#endif

#ifndef S2681XTAL
/* baud rate oscillaror frequency */
#define S2681XTAL	3686400
#endif

#ifndef S2681_WDELAY
/* recovery delay between register writes */
#define S2681_WDELAY	32
#endif

#ifndef S2681REG
#define S2681REG(bank,reg)	(((bank) * S2681BANK) + ((reg) * S2681REGSZ))
#endif

#ifndef __ASSEMBLER__

#ifndef s2681reg
# if S2681REGSZ == 8
#  define s2681reg(x)	unsigned long long x
# elif S2681REGSZ == 4
#  define s2681reg(x)	unsigned int x
# elif S2681REGSZ == 2
#  define s2681reg(x)	unsigned short x
# elif S2681REGSZ == 1
#  define s2681reg(x)	unsigned char x
# else
#  error bad S2681REGSZ value
# endif
#endif

struct s2681dev {
    struct {
	s2681reg(reg0);
	s2681reg(reg1);
	s2681reg(reg2);
	s2681reg(reg3);
#if S2681GAP > 0
	unsigned char	gap[S2681GAP];
#endif
    } bank[4];
};

#endif

/* channel bank bases */
#define CHA	S2681REG(0,0)
#define CHB	S2681REG(2,0)

/* per-channel register offsets */
#define MR	S2681REG(0,0)	/* r/w: mode register */
#define SR	S2681REG(0,1)	/* r/o: status register */
#define CSR	S2681REG(0,1)	/* w/o: clock select register */
#define CR	S2681REG(0,2)	/* w/o: command register */     
#define RHR	S2681REG(0,3)	/* r/o: rx holding register */  
#define THR	S2681REG(0,3)	/* w/o: tx holding register */  

/* global registers */
#define IPCR	S2681REG(1,0)	/* r/o: input port change reg */
#define ACR	S2681REG(1,0)	/* w/o: aux control reg */
#define ISR	S2681REG(1,1)	/* r/o: interrupt status reg */
#define IMR	S2681REG(1,1)	/* w/o: interrupt mask reg */
#define CTU	S2681REG(1,2)	/* r/w: counter/timer upper */
#define CTL	S2681REG(1,3)	/* r/w: counter/timer lower */

#define IPORT	S2681REG(3,0)	/* r/o: input port */
#define OPCR	S2681REG(3,1)	/* w/o: output port conf */
#define CTSTART	S2681REG(3,2)	/* r/o: start counter cmd */
#define SETOP	S2681REG(3,2)	/* w/o: set output port bits */
#define CTSTOP	S2681REG(3,3)	/* r/o: stop counter cmd */
#define RSTOP	S2681REG(3,3)	/* w/o: reset output port bits */

#ifndef __ASSEMBLER__

/* channel bank base */
#define cha	bank[0]
#define chb	bank[2]

/* per-channel registers */
#define ch_mr	reg0		/* r/w: mode register */
#define ch_sr	reg1		/* r/o: status register */
#define ch_csr	reg1		/* w/o: clock select register */
#define ch_cr	reg2		/* w/o: command register */     
#define ch_rhr	reg3		/* r/o: rx holding register */  
#define ch_thr	reg3		/* w/o: tx holding register */  

/* global registers */
#define g_ipcr	bank[1].reg0	/* r/o: input port change reg */
#define g_acr	bank[1].reg0	/* w/o: aux control reg */
#define g_isr	bank[1].reg1	/* r/o: interrupt status reg */
#define g_imr	bank[1].reg1	/* w/o: interrupt mask reg */
#define g_ctu	bank[1].reg2	/* r/w: counter/timer upper */
#define g_ctl	bank[1].reg3	/* r/w: counter/timer lower */

#define g_iport	bank[3].reg1	/* r/o: input port */
#define g_opcr	bank[3].reg1	/* w/o: output port conf */
#define g_start bank[3].reg2	/* r/o: start counter cmd */
#define g_setop	bank[3].reg2	/* w/o: set output port bits */
#define g_stop	bank[3].reg3	/* r/o: stop counter cmd */
#define g_rstop	bank[3].reg3	/* w/o: reset output port bits */

#endif

#define MR1_RXRTS	0x80
#define MR1_FFULL	0x40
#define MR1_RXRDY	0x00
#define MR1_BLKERR	0x20
#define MR1_PARON	0x00
#define MR1_PARFORCE	0x08
#define MR1_PAROFF	0x10
#define MR1_MDROP	0x18
#define MR1_PARODD	0x04
#define MR1_5BITS	0x00
#define MR1_6BITS	0x01
#define MR1_7BITS	0x02
#define MR1_8BITS	0x03

/* channel mode */
#define MR2_NORMAL	0x00
#define MR2_AUTOECHO	0x40
#define MR2_LOCALLOOP	0x80
#define MR2_REMLOOP	0xc0
/* flow control */
#define MR2_TXRTS	0x20	/* TxRTS control */
#define MR2_CTS		0x10	/* CTS enables Tx */
/* stop bit length */
#define MR2_STOP1	0x07
#define MR2_STOP2	0x0f

/* command codes */
#define CR_NOP		0x00	/* no-operation */
#define CR_RSTMR	0x10	/* reset MR pointer */
#define CR_RSTRX	0x20	/* reset receiver */
#define CR_RSTTX	0x30	/* reset transmitter */
#define CR_RSTERR	0x40	/* reset error status */
#define CR_RSTBRK	0x50	/* reset break change status */
#define CR_STARTBRK	0x60	/* start sending break */
#define CR_STOPBRK	0x70	/* stop sending break */
/* individual control bits */
#define CR_DISTX	0x08	/* disable tx */
#define CR_ENBTX	0x04	/* enable tx */
#define CR_DISRX	0x02	/* disable rx */
#define CR_ENBRX	0x01	/* enable tx */

#define SR_BREAK	0x80	/* break detected */
#define SR_FRAME	0x40	/* framing error */
#define SR_PARITY	0x20	/* parity error */
#define SR_OFLO		0x10	/* fifo overflow */
#define SR_TXEMT	0x08	/* tx empty */
#define SR_TXRDY	0x04	/* tx ready */
#define SR_FFULL	0x02	/* rx fifo full */
#define SR_RXRDY	0x01	/* rx ready */

/* baud rate sets */
#define ACR_SET1	0x00	/* set 1 */
#define ACR_SET2	0x80	/* set 2 */
#define ACR_CTMODE	0x70	/* counter timer mode bits */
/* counter mode: clock source */
#define ACR_C_IP2	0x00	/* pin IP2 */
#define ACR_C_TXCA	0x10	/* chan A tx clock */
#define ACR_C_TXCB	0x20	/* chan B tx clock */
#define ACR_C_XTAL_16	0x30	/* crystal / 16 */
/* timer mode: clock source */
#define ACR_T_IP2	0x40	/* pin IP2 */
#define ACR_T_IP2_16	0x50	/* pin IP2 / 16 */
#define ACR_T_XTAL	0x60	/* crystal */
#define ACR_T_XTAL_16	0x70	/* crystal / 16 */
/* input port delta interrupt enables */
#define ACR_DELTA3	0x08
#define ACR_DELTA2	0x04
#define ACR_DELTA1	0x02
#define ACR_DELTA0	0x01

/* interrupt status/mask bits */
#define ISR_IPORT	0x80	/* input port change */
#define ISR_BREAKB	0x40	/* break change (chan B) */
#define ISR_RXB		0x20	/* rxrdy/ffull (chan B) */
#define ISR_TXB		0x10	/* txrdy (chan B) */
#define ISR_CT		0x08	/* counter timer */
#define ISR_BREAKA	0x04	/* break change (chan A) */
#define ISR_RXA		0x02	/* rxrdy/ffull (chan A) */
#define ISR_TXA		0x01	/* txrdy (chan A) */

#endif /* _S2681_H_ */
