.ident "kit/share/s2681.s: Copyright (c) MIPS Technologies, Inc. All rights reserved."
	
/*
 * Copyright (c) 1996-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * share/s2681.s: Signetics SCN2681 / Motorola 68681 / UMC UM2681 UART driver.
 */


/* 
 * These routines can be called very early in the life of the PROM.
 * - They cannot use dram / stack etc.  
 * - They may only use registers AT, v0-v1, a0-a3, t0-t3
 */

#include <mips/asm.h>
#include <mips/regdef.h>
#include <mips/cpu.h>
	
#include "sbdser.h"
#include "sbd.h"
#include "s2681.h"
	
#ifdef DUART2_BASE
#define	NSER	4
#else
#define	NSER	2
#endif

#define TXWAIT	10000		/* TxRdy wait timeout */

#ifndef DEFSPEED
#define DEFSPEED	9600	/* default console speed */
#endif
	
#ifndef DBGSPEED
#define DBGSPEED	9600	/* default debug speed */
#endif
	
#ifndef DBGPORT	
#define DBGPORT		1
#endif
	
/* These macros allow for any special action needed before accessing device. */
#ifndef PRE_S2681
#define PRE_S2681
#define POST_S2681
#endif
	
/* Allow ld/sd in this module if S2681 registers are 8 bytes apart */
#if S2681_REGSZ == 8 && __mips >= 3 && __mips != 32
#undef __mips64
#define __mips64 1
#endif
		
/* These macros allow for special handling of input and output */
#ifndef S2681_INB
# if S2681_REGSZ == 8 && __mips64
#  define S2681_INB(dst,offs,base) 	ld	dst,offs(base)
# else	
#  define S2681_INB(dst,offs,base) 	lw	dst,offs(base)
# endif	
#endif
	
#ifndef S2681_OUTB
# if S2681_REGSZ == 8 && __mips64
#  define S2681_OUTB(src,offs,base) 	sd	src,offs(base)
# else	
#  define S2681_OUTB(src,offs,base) 	sw	src,offs(base)
# endif
#endif
		
#define DELAY 			\
	.set	noreorder;	\
        .set    noat; 		\
	li	AT,S2681_WDELAY;\
10:	bnez	AT,10b;		\
	subu	AT,1;		\
        .set    at; 		\
	.set	reorder

#ifndef WBFLUSH	
	.lcomm	wbfltmp,4
#define	WBFLUSH  \
	la	t3,wbfltmp; \
	or	t3,KSEG1_BASE; \
	sw	t3,0(t3); \
	lw	t3,0(t3); \
	addu	t3,1
#endif
		
/* remote debugging port number */
	.sdata
EXPORTS(_sbddbgport, 4)
	.word	DBGPORT
		
	.rdata
/* device table */
EXPORTS(_sbdsertab, SS_SIZE * NSER)
	.word	0x0, reset, init, params, rxpoll, txrdy, tx
	.word	0x4, 0,     init, params, rxpoll, txrdy, tx
#ifdef DUART2_BASE
	.word	0x8, reset, init, params, rxpoll, txrdy, tx
	.word	0xc, 0,     init, params, rxpoll, txrdy, tx
#endif

/* number of ports */
EXPORTS(_sbdnser, 4)
	.word	NSER

/* default initial port speeds */
EXPORTS(_sbdspeed, NSER*4)
	.word	DEFSPEED, DBGSPEED
#if NSER > 2
	.word	DEFSPEED, DEFSPEED
#endif


	.rdata

/* UART address table */
_addrs:
	.word	PA_TO_KVA1(DUART_BASE)+CHA
	.word	PA_TO_KVA1(DUART_BASE)+CHB
#ifdef DUART2_BASE
	.word	PA_TO_KVA1(DUART2_BASE)+CHA
	.word	PA_TO_KVA1(DUART2_BASE)+CHB
#endif

/* Per-UART reset table (both channels) */
_resettab:
	.word	IMR,0		# interrupts off
	.word	ACR,0		# ACR = 0
	.word	OPCR,0		# enable status o/p pins
	.word	RSTOP,0xff	# reset all o/p pins
	.word	SETOP,0x80	# set OP7 (for some LSI boards)
	.word	-1,-1
	
/*
 * reset (chan, reloc)
 *
 * Once-only device reset at power-on
 */
SLEAF(reset)
	and	a1,~1		# clear bit 0 of reloc
	la	t0,_addrs(a1)	# uart address table
	addu	t0,a0		# calc &_addrs[chan]
	lw	a0,0(t0)	# get address

	la	a1,_resettab(a1) # initialisation table
	
	PRE_S2681

1:	lw	t0,0(a1)	# register offset
	lw	t1,4(a1)	# register value
	addu	a1,8
	bltz	t0,3f		# end of table?

	addu	t0,a0		# add uart base address
	S2681_OUTB(t1,0,t0)	# write uart register
	WBFLUSH
	DELAY

	b	1b
	
3:	POST_S2681
	j	ra
SEND(reset)



	.rdata
	
/* Per-channel initialisation table */
_inittab:
	.word	CR,CR_DISTX|CR_DISRX	# disable tx & rx
	.word	CR,CR_RSTMR		# reset MR ptr
	.word	CR,CR_RSTRX		# reset rx
	.word	CR,CR_RSTTX		# reset tx
	.word	CR,CR_STOPBRK		# stop break
	.word 	MR,MR1_PAROFF|MR1_8BITS	# no parity, 8 bits data
	.word	MR,MR2_NORMAL|MR2_STOP1	# 1 stop bit
	.word	CR,CR_ENBTX|CR_ENBRX	# enable tx & rx
	.word	-1,-1


/*
 * init (chan, reloc)
 *
 * Per channel initialisation.
 */
SLEAF(init)
	and	a1,~1		# clear bit 0 of reloc
	la	t0,_addrs(a1)	# uart address table
	addu	t0,a0		# calc _addrs[chan]
	lw	a0,0(t0)	# get address

	la	a1,_inittab(a1)	# initialisation table

	PRE_S2681

	/* Wait for Tx fifo to completely empty or time out */
	li	t0,TXWAIT
1:	S2681_INB(t1,SR,a0)
	and	t1,SR_TXEMT
	bnez	t1,2f
	subu	t0,1
	bnez	t0,1b

2:	DELAY
	
1:	lw	t0,0(a1)	# register offset
	lw	t1,4(a1)	# register value
	addu	a1,8
	bltz	t0,3f		# end of table?

	addu	t0,a0		# add uart base address
	S2681_OUTB(t1,0,t0)	# write uart register
	WBFLUSH
	DELAY

	b	1b

3:	POST_S2681
	j	ra
SEND(init)




/*
 * Baud rate conversion table
 */
#define SPD(speed, csrval, acr7) \
	.half	speed; \
	.byte	csrval; \
	.byte	acr7

	.struct 0
SPEED:	.half	0
CSRVAL:	.byte	0
ACR7:	.byte	0
	
	.rdata
_speedtab:
	SPD(50,		0x00, 0)
	SPD(75,		0x00, 1)
	SPD(110,	0x11, 2)
	SPD(134,	0x22, 2)
	SPD(150,	0x33, 1)
	SPD(220,	0x33, 0)
	SPD(300,	0x44, 2)
	SPD(600,	0x55, 2)
	SPD(1050,	0x66, 1)
	SPD(1200,	0x66, 0)
	SPD(2400, 	0x88, 2)
 	SPD(4800,	0x99, 2)
	SPD(1800,	0xaa, 1)
	SPD(7200,	0xaa, 0)
	SPD(9600,	0xbb, 2)
	SPD(19200,	0xcc, 1)
	SPD(38400,	0xcc, 0)
	SPD(0xffff,	0x00, 0)

/*
 * Pointers to current speed for each channel
 */
	.data
_speeds:	
	.word	0 : NSER

/*
 * Soft ACR register for each UART (shared with s2681clk.c)
 */
	BSS(_s2681_acr, NSER / 2)



/*
 * params (chan, speed, reloc)
 *
 * program new speed, return 0 on success
 */
SLEAF(params)
	move	a3,a2		# temporary copy of reloc
	and	a2,~1		# clear bit zero of reloc
	
	la	t0,_addrs(a2)	# uart address table
	addu	t0,a0		# calc &_addrs[chan]
	lw	t0,0(t0)	# channel address

	/* Wait for Tx fifo to completely empty or time out */
	PRE_S2681
	li	t1,TXWAIT
1:	S2681_INB(t2,SR,t0)
	and	t2,SR_TXEMT
	bnez	t2,2f
	subu	t1,1
	bnez	t1,1b
2:	DELAY
	POST_S2681

	/* get pointer to speed table */
	la	v0,_speedtab(a2)
	
	/* search for matching speed */
1:	lhu	t1,SPEED(v0)
	beq	t1,a1,1f
	addu	v0,4
	bne	t1,0xffff,1b

	/* no match */
	b	..fail

	/* are we being called from romlow (.data not initialised yet)? */
1:	beqz	a3,2f

	/* in romlow: just get the required ACR, ignoring the other channel */
	lbu	a3,ACR7(v0)
	sll	a3,7		# move ACR[7] to top bit
	b	8f

	/* 
	 * The requested speed is only allowed if the other channel's
	 * ACR[7] bit is compatible.  
	 */
2:	lbu	t2,ACR7(v0)	# new->ACR7

	xor	t0,a0,4		# calc "other channel" index
	lw	t0,_speeds(t0)	# get _speeds[other]
	beqz	t0,7f		# not programmed yet?

	/* mismatch if (new->ACR7 + other->ACR7) == 1 */
	lbu	t3,ACR7(t0)	# other->ACR7
	addu	t3,t2
	beq	t3,1,..fail

	/* save our new speed in table */
7:	sw	v0,_speeds(a0)	

	/* update soft copy of ACR */
	srl	t3,a0,3		# get "channel A" index (chan / 8)
	lbu	a3,_s2681_acr(t3)

	/* don't change it if new->ACR7 == 2 (don't care) */
	beq	t2,2,8f
	
	/* insert new->ACR7 into top bit of ACR and save it */
	sll	t2,7
	and	a3,0x7f
	or	a3,t2
	sb	a3,_s2681_acr(t3)

	/* 
	 * All checks OK: program the channel baud rate.
 	 */
8:	la	v1,_addrs(a2)	# uart address table base
	addu	a2,v1,a0	# calc &_addrs[chan]
	lw	a2,0(a2)	# channel address in a2

	PRE_S2681

	/* Disable transmitter and receiver */
	li	t1,CR_DISTX|CR_DISRX
	S2681_OUTB(t1,CR,a2)
	WBFLUSH
	DELAY

	and	t0,a0,~4	# get channel A offset
	addu	t0,v1		# add relocated address table base
	lw	t0,0(t0)	# channel A address in t0
	S2681_OUTB(a3,ACR,t0)	# write ACR
	WBFLUSH
	DELAY

1:	lbu	t1,CSRVAL(v0)	# get new baudrate value
	S2681_OUTB(t1,CSR,a2)	# write clock select reg
	WBFLUSH
	DELAY

	/* Enable transmitter and receiver */
	li	t1,CR_ENBTX|CR_ENBRX
	S2681_OUTB(t1,CR,a2)
	WBFLUSH
	DELAY
	
	POST_S2681

	move	v0,zero	
	j	ra

..fail:	li	v0,1
	j	ra
SEND(params)



/*
 * rxpoll (chan, canblock, reloc) 
 *
 * poll for received character, return zero if none available
 */
SLEAF(rxpoll)
	and	a2,~1		# clear bit 0 of reloc
	la	t0,_addrs(a2)	# uart address table
	addu	t0,a0		# calc &_addrs[chan]
	lw	a0,0(t0)	# channel address

	PRE_S2681
	S2681_INB(v0,SR,a0)
	and	v0,SR_RXRDY
	beqz	v0,1f
	S2681_INB(v0,RHR,a0)
	and	v0,0xff
	or	v0,0x100
1:	POST_S2681
	j	ra
SEND(rxpoll)


/* 
 * txrdy (chan, reloc) 
 *
 * return non-zero if ready to transmit
 */
SLEAF(txrdy)
	and	a1,~1		# clear bit 0 of reloc
	la	t0,_addrs(a1)	# uart address table
	addu	t0,a0		# calc &_addrs[chan]
	lw	a0,0(t0)	# channel address

	PRE_S2681
	S2681_INB(v0,SR,a0)
	and	v0,SR_TXRDY
	POST_S2681
	j	ra
SEND(txrdy)


/* 
 * tx (chan, data, reloc) 
 *
 * transmit data
 */
SLEAF(tx)
	and	a2,~1		# clear bit 0 of reloc
	la	t0,_addrs(a2)	# uart address table
	addu	t0,a0		# calc &_addrs[chan]
	lw	a0,0(t0)	# channel address

	PRE_S2681

	li	t0,TXWAIT	# TxRdy timeout
1:	S2681_INB(t1,SR,a0)
	and	t1,SR_TXRDY
	bnez	t1,2f
	subu	t0,1
	bnez	t0,1b

2:	S2681_OUTB(a1,THR,a0)
	POST_S2681
	j	ra
SEND(tx)
