/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * sbdser.h: MIPS SDE board specific serial i/o dispatch table
 */


#ifndef _sbdser_H_
#define _sbdser_H_

/* 
 * The final parameter for each function has the following meaning:
 *  zero	code is fully relocated, and data segment initialised;
 *  non-zero	relocation factor to get at rom .text/.rdata
 *
 * For more details see romlow.S.
 */

#ifndef __ASSEMBLER__
struct sbdser {
    void	*ss_data;			/* private data for routines */
    int		(*ss_reset)(void *, long);	/* power-on reset (rom only) */
    int		(*ss_init)(void *, long);	/* initialise channel */
    int		(*ss_params)(void *, unsigned int, long);/* set speed */
    int		(*ss_rxpoll)(void *, int, long);/* get rx data (canblock) */
    int		(*ss_txrdy)(void *, long);	/* tx possible? */
    void	(*ss_tx)(void *, int, long);	/* send tx data */
};

/* These objects are in .rodata */
extern const struct sbdser	_sbdsertab[];	/* table of serial devices */
extern const int		_sbdspeed[];	/* default initial baud rate */
extern const int		_sbdnser;	/* num of entries in table */

/* These may be patchable */
extern const int		_sbddbgport;	/* remote dbg index # */
extern int			_sbd_setdbgport(int *);

struct termios;
struct tty;

/* Interrupt-driven serial port interface */
struct sbdserint {
    const char *si_devname;
    int		si_nchan;
    int		(*si_init) (int, int);
    int		(*si_open) (int, struct tty **);
};

#else

/* structure offsets for assembler */
	.struct	0
SS_DATA:	.word	0
SS_RESET:	.word	0
SS_INIT:	.word	0
SS_PARAMS:	.word	0
SS_RXPOLL:	.word	0
SS_TXRDY:	.word	0
SS_TX:		.word	0
	.previous

#define SS_SIZE	(7*4)

#endif

#endif /* _sbdser_H_ */
