#ident "kit/share/stubs.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1998-2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * Thread stubs 
 */

#include <sys/cdefs.h>
#include <sdethread.h>
#include <libc_thread.h>

#pragma weak __isthreaded
int __isthreaded = 0;

WEAK_ALIAS(__sdethread_mutex_init);
WEAK_PROTOTYPE(__sdethread_mutex_init);
int
WEAK_NAME(__sdethread_mutex_init) (__sdethread_mutex_t *mx, const char *name)
{
  return 0;
}

WEAK_ALIAS(__sdethread_mutex_destroy);
WEAK_PROTOTYPE(__sdethread_mutex_destroy);
int
WEAK_NAME(__sdethread_mutex_destroy) (__sdethread_mutex_t *mx)
{
  return 0;
}

WEAK_ALIAS(__sdethread_mutex_lock);
WEAK_PROTOTYPE(__sdethread_mutex_lock);
int
WEAK_NAME(__sdethread_mutex_lock) (__sdethread_mutex_t *mx)
{
  return 0;
}

WEAK_ALIAS(__sdethread_mutex_trylock);
WEAK_PROTOTYPE(__sdethread_mutex_trylock);
int
WEAK_NAME(__sdethread_mutex_trylock) (__sdethread_mutex_t *mx)
{
  return 0;
}

WEAK_ALIAS(__sdethread_mutex_unlock);
WEAK_PROTOTYPE(__sdethread_mutex_unlock);
int
WEAK_NAME(__sdethread_mutex_unlock) (__sdethread_mutex_t *mx)
{
  return 0;
}

WEAK_ALIAS(__sdethread_sigdisable);
WEAK_PROTOTYPE(__sdethread_sigdisable);
void
WEAK_NAME(__sdethread_sigdisable) (__sdethread_sigstate_t *state) 
{/* no signals */}

WEAK_ALIAS(__sdethread_sigrestore);
WEAK_PROTOTYPE(__sdethread_sigrestore);
void
WEAK_NAME(__sdethread_sigrestore) (const __sdethread_sigstate_t *state) 
{/* no signals */}

WEAK_ALIAS(___sdethread_once);
WEAK_PROTOTYPE(___sdethread_once);
int
WEAK_NAME(___sdethread_once) (__sdethread_once_t *once, void (*fn)(void))
{
    if (once->so_state == __SDETHREAD_ONCE_NEVER) {
	once->so_state = __SDETHREAD_ONCE_DONE;
	(*fn) ();
    }
    return 0;
}

#undef __sdethread_once
__c_weak_alias(__sdethread_once, WEAK_NAME(___sdethread_once));

WEAK_ALIAS(_libc_private_storage);
WEAK_PROTOTYPE(_libc_private_storage);
void *
WEAK_NAME(_libc_private_storage)(struct _thread_private_key_struct *key,
				 void *storage, size_t size, void *err)
{
    /* return static storage only */
    return storage;
}

#include <unistd.h>

WEAK_ALIAS(getpid);
WEAK_PROTOTYPE(getpid);
pid_t
WEAK_NAME(getpid)(void)
{
    return 1;
}


#include <errno.h>
#undef errno

WEAK_ALIAS(__sdethread_errno_pointer);
WEAK_PROTOTYPE(__sdethread_errno_pointer);
int *
WEAK_NAME(__sdethread_errno_pointer) (void)
{
    return &errno;
}

__attribute__((__nomips16__)) WEAK_ALIAS(__sdethread_set_errno);
__attribute__((__nomips16__)) WEAK_PROTOTYPE(__sdethread_set_errno);
__attribute__((__nomips16__))
int
WEAK_NAME(__sdethread_set_errno) (int err, int rv)
{
    errno = err;
    return rv;
}

WEAK_ALIAS(fork);
WEAK_PROTOTYPE(fork);
pid_t	WEAK_NAME(fork)(void)
{
	errno = ENOSYS;
	return -1;
}

WEAK_ALIAS(execl);
WEAK_PROTOTYPE(execl);
int	WEAK_NAME(execl)(const char *path, const char *arg, ...)
{
	errno = ENOSYS;
	return -1;
}

WEAK_ALIAS(execv);
WEAK_PROTOTYPE(execv);
int	WEAK_NAME(execv)(const char *path, const char * const *argv)
{
	errno = ENOSYS;
	return -1;
}

#include <sys/wait.h>

WEAK_ALIAS(waitpid);
WEAK_PROTOTYPE(waitpid);
pid_t	WEAK_NAME(waitpid)(pid_t pid, int *status, int options)
{
	errno = ENOSYS;
	return -1;
}

