#ident "kit/share/sysconf.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2000-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * sysconf.c: return POSIX sysconf() function
 */


#define _XOPEN_SOURCE 500
#include <unistd.h>
#include <limits.h>
#include <stdio.h>
#include <errno.h>
#include <pthread.h>
#include "kit_impl.h"

#pragma weak pthread_sysconf_np

long
sysconf (int name)
{
    /* let thread library have first shot, if present */
    if (pthread_sysconf_np) {
	int rv = pthread_sysconf_np (name);
	if (rv >= 0)
	    return rv;
    }

    switch (name) {
    case _SC_ARG_MAX:
	return ARG_MAX;
    case _SC_ATEXIT_MAX:
	return ATEXIT_MAX;
    case _SC_AVPHYS_PAGES:
	return _sbrk_availpages();
    case _SC_CHILD_MAX:
	return 0;
    case _SC_CLK_TCK:
	return CLK_TCK;
    case _SC_FSYNC:
	return -1;
    case _SC_JOB_CONTROL:
	return 0;
    case _SC_NGROUPS_MAX:
	return 0;
    case _SC_OPEN_MAX:
	return OPEN_MAX;
    case _SC_PAGESIZE:
	return getpagesize();
    case _SC_PHYS_PAGES:
	return _sbrk_totalpages();
    case _SC_SAVED_IDS:
	return 0;
    case _SC_STREAM_MAX:
	return FOPEN_MAX;
    case _SC_TTY_NAME_MAX:
	return TTY_NAME_MAX;
    case _SC_TZNAME_MAX:
	return TZNAME_MAX;
    case _SC_XOPEN_VERSION:
	return -1;
    case _SC_VERSION:
	return _POSIX_VERSION;

    case _SC_NPROCESSORS_CONF:
    case _SC_NPROCESSORS_ONLN:
	return 1;

   /* no real-time POSIX support by default */
    case _SC_AIO_LISTIO_MAX:
    case _SC_AIO_MAX:
    case _SC_AIO_PRIO_DELTA_MAX:
    case _SC_ASYNCHRONOUS_IO:
    case _SC_DELAYTIMER_MAX:
    case _SC_MAPPED_FILES:
    case _SC_MEMLOCK:
    case _SC_MEMLOCK_RANGE:
    case _SC_MEMLOCK_PROTECTION:
    case _SC_MEMORY_PROTECTION:
    case _SC_MESSAGE_PASSING:
    case _SC_MQ_OPEN_MAX:
    case _SC_MQ_PRIO_MAX:
    case _SC_PRIORITIZED_IO:
    case _SC_PRIORITY_SCHEDULING:
    case _SC_REALTIME_SIGNALS:
    case _SC_RTSIG_MAX:
    case _SC_SEMAPHORES:
    case _SC_SEM_NSEMS_MAX:
    case _SC_SEM_VALUE_MAX:
    case _SC_SHARED_MEMORY_OBJECTS:
    case _SC_SYNCHRONIZED_IO:
    case _SC_SIGQUEUE_MAX:
    case _SC_TIMER_MAX:
    case _SC_TIMERS:
	return -1;
    }

    errno = EINVAL;
    return -1;
}
