.ident "kit/share/vacser.s: Copyright (c) MIPS Technologies, Inc. All rights reserved."
	
/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * share/vacser.s: VAC serial port driver shared by Algorithmics VME boards.
 */
	
	
/* 
 * These routines can be called very early in the life of the PROM.
 * - They cannot use memory or stack.  
 * - They may only use registers AT, v0-v1, a0-a3, t0-t3
 */

#include <mips/asm.h>
#include <mips/regdef.h>
#include <mips/cpu.h>
	
#include "sbd.h"
#include "vac068.h"

#include "sbdser.h"
	
#define TXWAIT	10000		/* TxRdy wait timeout */

	.sdata
/* remote debugging port number */
EXPORTS(_sbddbgport, 4)
	.word	1

	.rdata

/* device table */
EXPORTS(_sbdsertab, SS_SIZE * 2)
	.word	vaca, 	0, init, params, rxpoll, txrdy, tx
	.word	vacb,   0, init, params, rxpoll, txrdy, tx

/* number of ports */
EXPORTS(_sbdnser, 4)
	.word	2

/* default initial port speeds */
EXPORTS(_sbdspeed, 2*4)
	.word	9600, 9600
	
/* register address table */
vaca:	.word	VACMRA, VACTXA, VACRXA, VACISA
vacb:	.word	VACMRB, VACTXB, VACRXB, VACISB
	
#define MR	0
#define TX	4
#define RX	8
#define IS	12
	

SLEAF(reset)
	/* nothing to do (done in board reset) */
	j	ra
SEND(reset)
		

SLEAF(init)
	/* nothing to do */
	j	ra
SEND(init)
		

SLEAF(params)
	sne	v0,a1,9600	# fail if not 9600 baud!
	j	ra
SEND(params)


SLEAF(rxpoll)
	addu	a0,a2
	lw	a3,IS(a0)
	lw	v0,0(a3)	
	and	v0,(VAC_UIS_RXC|VAC_UIS_FIFOF)
	beqz	v0,1f
	
	lw	a3,RX(a0)
	lw	v0,0(a3)
	srl	v0,16
	and	v0,0xff
	or	v0,0x100
	
1:	j	ra
SEND(rxpoll)


SLEAF(txrdy)
	addu	a0,a1
	lw	a3,IS(a0)
	lw	v0,0(a3)
	and	v0,VAC_UIS_TXR
	j	ra
SEND(txrdy)


SLEAF(tx)
	addu	a0,a2
	lw	a3,IS(a0)

	li	t0,TXWAIT
1:	lw	t1,0(a3)
	and	t1,VAC_UIS_TXR
	bnez	t1,2f
	subu	t0,1
	bnez	t0,1b

2:	lw	a3,TX(a0)
	sll	a1,24
	sw	a1,0(a3)
	WBFLUSH
	j	ra
SEND(tx)

