.ident "kit/share/xcptcache.s: Copyright (c) MIPS Technologies, Inc. All rights reserved."
	
/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * xcptcache.s: R4000 cache error handler.  
 */
	
	
/*
 * We may only access uncached memory.  The low-level trap saves
 * AT and k0 in low memory and calls other code to do the
 * hard work.  We build an exception context similarly to
 * the standard handler, but add some extra cache error status.
 */

#ifndef C0_CACHEERR
#error CacheErr handling not available
#endif
.ifndef XCPC_CACHEERR
.error CacheErr handling not available
.endif	
		
#include <mips/prid.h>
	
#define AT_SAVE		(0x180-0x08)
#define SP_SAVE	 	(0x180-0x10)
#define ECC_STACKSIZE	1024
	
LEAF(_xcpt_cache) 
	.set	noat
	sr	AT,AT_SAVE(zero)
	sr	sp,SP_SAVE(zero)
	.set	at
XLEAF(_xcpt_cache_local)
	la	sp,_eccstack+ECC_STACKSIZE
	or	sp,KSEG1_BASE

	/* save enough registers to get by */
	.set	noat
	lr	AT,AT_SAVE(zero)
	sr	AT,XCP_AT(sp)
	.set	at
	sr	v0,XCP_V0(sp)
	sr	v1,XCP_V1(sp)
	lr	v0,SP_SAVE(zero)
	sr	a0,XCP_A0(sp)
	sr	a1,XCP_A1(sp)
	sr	a2,XCP_A2(sp)
	sr	a3,XCP_A3(sp)
	sr	v0,XCP_SP(sp)
	
	sr	k0,XCP_K0(sp)
	sr	k1,XCP_K1(sp)
	
	/* save cache related coprocessor registers */
	mfc0	a0,C0_CACHEERR
	rmfc0	a1,C0_ERRPC
#if defined(C0_ECC)
	mfc0	a2,C0_ECC
#elif defined(C0_ERRCTL)	
	mfc0	a2,C0_ERRCTL
#endif	
	mfc0	a3,C0_TAGLO
	mfc0	v1,C0_CONFIG
	
	la	v0,_cxcpt
	or	v0,KSEG1_BASE
	sr	a0,CXCP_CACHEERR(v0)
#ifdef C0_TAGHI	
	mfc0	a0,C0_TAGHI
#else	
	move	a0,zero
#endif	
	sr	a1,CXCP_ERRPC(v0)
	mfc0	a1,C0_PRID
#if defined(C0_ECC) || defined(C0_ERRCTL)	
	sr	a2,CXCP_ECC(v0)
#endif	
	sr	a3,CXCP_TAGLO(v0)
	sr	a0,CXCP_TAGHI(v0)
	
	/* Make sure that KSEG0 is now cacheable.
	   XXX this is so that a higher level ecchandler can access
	   the caches via KSEG0 */
	srl	a1,8
	and	a1,0xff
	beq	a1,PRID_R4650,1f
	
	/* R4xxx: make kseg0 cacheable */
	and	a1,v1,~CFG_K0MASK
	or	a1,CFG_C_NONCOHERENT
	mtc0	a1,C0_CONFIG
	b	2f

	/* R4650: make kseg0 cacheable */
1:	mfc0	v1,$17
	li	a1,0x22233333
	mtc0	a1,$17
		
	/* note exception class */
2:	sr	v1,CXCP_CONFIG(v0)
	
	/* get coprocessor 0 exception state */
	mfc0	a0,C0_CR
	mfc0	a1,C0_SR
	rmfc0	a2,C0_BADVADDR
	rmfc0	a3,C0_EPC

#ifdef SR_DE
	/* switch off ERL and ECC flipping, but leave cache errors and 
	   i/us disabled. */
	and	v0,a1,~(SR_CE|SR_IE|SR_ERL|SR_EXL)
	or	v0,SR_DE
	mtc0	v0,C0_SR
#else 
	/* XXX if we can't switch off cache error detection using SR_DE
	   then how do we get at kuseg addresses addresses without
	   remapping them to kseg2 or something horrid. */	
#endif
	
	/* save coprocessor exception state */
	sr	a0,XCP_CR(sp)
	sr	a1,XCP_SR(sp)
	sr	a2,XCP_VADDR(sp)
	sr	a3,XCP_EPC(sp)

	/* note exception class */
	li	v0,XCPC_CACHEERR
	sw	v0,XCP_XCLASS(sp)
	sw	zero,XCP_PREV(sp)

	/* mdhi and mdlo */
	mfhi	v0
	mflo	v1
	sr	v0,XCP_MDHI(sp)
	sr	v1,XCP_MDLO(sp)

	/* 
	 * Save all the other general registers.
	 */
	sr	$0,XCP_R0(sp)		/* zero */
	sr	$8,XCP_R8(sp)		/* t0(o32)	a4/ta0(n32/n64) */
	sr	$9,XCP_R9(sp)		/* t1(o32)	a5/ta1(n32/n64) */
	sr	$10,XCP_R10(sp)		/* t2(o32)	a6/ta2(n32/n64) */
	sr	$11,XCP_R11(sp)		/* t3(o32)	a7/ta3(n32/n64) */
	sr	$12,XCP_R12(sp)		/* t4/ta0(o32)	t0(n32/n64) */
	sr	$13,XCP_R13(sp)		/* t5/ta1(o32)	t1(n32/n64) */
	sr	$14,XCP_R14(sp)		/* t6/ta2(o32)	t2(n32/n64) */
	sr	$15,XCP_R15(sp)		/* t7/ta3(o32)	t3(n32/n64) */
	sr	$16,XCP_R16(sp)		/* s0 */
	sr	$17,XCP_R17(sp)		/* s1 */
	sr	$18,XCP_R18(sp)		/* s2 */
	sr	$19,XCP_R19(sp)		/* s3 */
	sr	$20,XCP_R20(sp)		/* s4 */
	sr	$21,XCP_R21(sp)		/* s5 */
	sr	$22,XCP_R22(sp)		/* s7 */
	sr	$23,XCP_R23(sp)		/* s7 */
	sr	$24,XCP_R24(sp)		/* t8 */
	sr	$25,XCP_R25(sp)		/* t9 */
	sr	$28,XCP_R28(sp)		/* gp */
	sr	$30,XCP_R30(sp)		/* s8/fp */ 
	sr	$31,XCP_R31(sp)		/* ra */

	/* load our _gp pointer */
	la	gp,_gp
	or	gp,KSEG1_BASE	# just in case

	/* and call the C exception handler */
	move	a0,sp		# arg1 = &xcp
#if ARGSIZE > 0	
	PTR_SUBU sp,ARGSIZE	# (arg save area)
#endif
	
	bal	_xcptcall
	
#if ARGSIZE > 0	
	PTR_ADDU sp,ARGSIZE	# (arg save area)
#endif

	/*
	 * Restore all state (sp points at exception frame again)
	 */

	/* restore most general regs */
	lr	v0,XCP_MDHI(sp)
	lr	v1,XCP_MDLO(sp)
	
	lr	$4,XCP_R4(sp)	/* a0 */
	lr	$5,XCP_R5(sp)	/* a1 */
	lr	$6,XCP_R6(sp)	/* a2 */
	lr	$7,XCP_R7(sp)	/* a3 */
	lr	$8,XCP_R8(sp)	/* t0(o32)	a4/ta0(n32/n64) */
	lr	$9,XCP_R9(sp)	/* t1(o32)	a5/ta1(n32/n64) */
	lr	$10,XCP_R10(sp)	/* t2(o32)	a6/ta2(n32/n64) */
	lr	$11,XCP_R11(sp)	/* t3(o32)	a7/ta3(n32/n64) */
	lr	$12,XCP_R12(sp)	/* t4/ta0(o32)	t0(n32/n64) */
	lr	$13,XCP_R13(sp)	/* t5/ta1(o32)	t1(n32/n64) */
	lr	$14,XCP_R14(sp)	/* t6/ta2(o32)	t2(n32/n64) */
	lr	$15,XCP_R15(sp)	/* t7/ta3(o32)	t3(n32/n64) */
	lr	$16,XCP_R16(sp)	/* s0 */
	lr	$17,XCP_R17(sp)	/* s1 */
	lr	$18,XCP_R18(sp)	/* s2 */
	lr	$19,XCP_R19(sp)	/* s3 */
	lr	$20,XCP_R20(sp)	/* s4 */
	lr	$21,XCP_R21(sp)	/* s5 */
	lr	$22,XCP_R22(sp)	/* s6 */
	lr	$23,XCP_R23(sp)	/* s7 */
	lr	$25,XCP_R24(sp)	/* t8 */
	lr	$25,XCP_R25(sp)	/* t9 */
	lr	$29,XCP_R28(sp)	/* gp */
	lr	$30,XCP_R30(sp)	/* s8/fp */
	lr	$31,XCP_R31(sp)	/* ra */
	
	/* mdhi and mdlo */
	mthi	v0
	mtlo	v1
	
	/* exception time $sr & $epc */
	lr	v0,XCP_SR(sp)
	mtc0	v0,C0_SR
	
	/* cache related coprocessor registers */
	la	v0,_cxcpt
	or	v0,KSEG1_BASE
	
	lr	v1,XCP_EPC(sp)
	lr	k1,CXCP_ERRPC(v0)
	rmtc0	v1,C0_EPC
	rmtc0	k1,C0_ERRPC
	lr	v1,CXCP_CACHEERR(v0)
#if defined(C0_ECC) || defined(C0_ERRCTL)	
	lr	k1,CXCP_ECC(v0)
#endif	
	mtc0	v1,C0_CACHEERR
#if defined(C0_ECC)
	mtc0	k1,C0_ECC
#elif defined(C0_ERRCTL)
	mtc0	k1,C0_ERRCTL
#endif	
	lr	v1,CXCP_TAGHI(v0)
	lr	k1,CXCP_TAGLO(v0)
#ifdef C0_TAGHI
	mtc0	v1,C0_TAGHI
#endif	
	mtc0	k1,C0_TAGLO
	
	.set noreorder
	nop
	mfc0	v1,C0_PRID
	.set reorder
	
	lr	k1,CXCP_CONFIG(v0)
	lr	v0,XCP_V0(sp)
	
	srl	v1,8
	and	v1,0xff
	.set	noreorder
	beq	v1,PRID_R4650,1f
	lr	v1,XCP_V1(sp)		# BDSLOT
	.set	reorder
	
	/* R4XXX - restore $config */
	mtc0	k1,C0_CONFIG
	lr	k1,XCP_K1(sp)
	.set	noreorder
	b	2f
	lr	k0,XCP_K0(sp)		# BDSLOT
	.set	reorder
		
	/* R4650 - restore $calg */
1:	mtc0	k1,$17
	lr	k1,XCP_K1(sp)
	lr	k0,XCP_K0(sp)		
	
	/* Restore remaining scratch registers and return */
2:	.set	noat
	lr	AT,XCP_AT(sp)
	lr	sp,XCP_SP(sp)
	eret
	.set	at
END(_xcpt_cache) 
		

/* cache exception code (copied to 0xa0000100) */
_xcpt_veccache:
	.set	noreorder
	.set	noat
	sr	AT,AT_SAVE(zero)
	.set	at
	sr	sp,SP_SAVE(zero)
	la	sp,_xcpt_cache_local
	or	sp,KSEG1_BASE		# must stay uncached
	j	sp
	nop
	.set	reorder
_xcpt_endcache:

/* private stack for cache/ecc error handling */
LBSS(_eccstack,ECC_STACKSIZE+XCP_SIZE)

/* cache/ecc error context */
BSS(_cxcpt,CXCP_SIZE)
