#ident "kit/share/xcptshow.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * xcptshow.c: display an exception 
 */


#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <mips/cpu.h>
#include <mips/xcpt.h>
#include <sys/signal.h>
#include "kit_impl.h"

#undef DBGXCPT
#ifdef DBGXCPT
#define DBG(fmt, args...) \
	_mon_printf (fmt , ## args);
#else
#define DBG(fmt, args...)
#endif

const char *const _sys_xcptclass[] =
{
#define XCPCDEF(NAME, NUMBER, SIG, DESC) DESC ,
#include <mips/xcptcpu.h>
#undef XCPCDEF
    0
};

const char *const _sys_xcptlist[] =
{
#define XCPTDEF(NAME, NUMBER, SIG, BRIEF, DESC)	DESC ,
#include <mips/xcptcpu.h>
#undef XCPTDEF
    0
};


#if XCP_RSIZE == 8
#define REGPERLINE	4
#else
#define REGPERLINE	8
#endif

static const char *const xcptregnames[32/REGPERLINE] = {
#if REGPERLINE == 8
# if _MIPS_SIM==_ABIO32
/* standard o32 */
"       zero      at       v0       v1       a0       a1       a2       a3",
"        t0       t1       t2       t3       t4       t5       t6       t7",
"        s0       s1       s2       s3       s4       s5       s6       s7",
"        t8       t9       k0       k1       gp       sp       s8       ra"
# else
/* 32-bit eabi */
"       zero      at       v0       v1       a0       a1       a2       a3",
"        a4       a5       a6       a7       t0       t1       t2       t3",
"        s0       s1       s2       s3       s4       s5       s6       s7",
"        t8       t9       k0       k1       gp       sp       s8       ra"
# endif
#else /* REGPERLINE != 8 */
# if _MIPS_SIM==_ABIN32 || _MIPS_SIM==_ABI64 || _MIPS_SIM==_ABIEABI
/* n32 / n64 / 64-bit eabi */
"           zero              at               v0               v1",
"            a0               a1               a2               a3",
"            a4               a5               a6               a7",
"            t0               t1               t2               t3",
"            s0               s1               s2               s3",
"            s4               s5               s6               s7",
"            t8               t9               k0               k1",
"            gp               sp               s8               ra"
# else
/* o32+ / o64 */
"           zero              at               v0               v1",
"            a0               a1               a2               a3",
"            t0               t1               t2               t3",
"            t4               t5               t6               t7",
"            s0               s1               s2               s3",
"            s4               s5               s6               s7",
"            t8               t9               k0               k1",
"            gp               sp               s8               ra"
# endif
#endif /* REGPERLINE != 8 */
};


static void
xcptputreg (reg_t val, int nb)
{
  char buf[17], *b = &buf[16];
  int i;

  *--b = '\0';
  for (i = nb * 2; i != 0; i--) {
      *--b = "0123456789abcdef"[val & 0xf];
      val >>= 4;
  }
  _mon_puts (b);
}


static void
xcptwritehex (const char *name, unsigned long val)
{
  _mon_printf ("%s: %08lx\n", name, val);
}


#define PCPERLINE 8
unsigned int _xcptstktracedepth = 4 * PCPERLINE;


/*
 * Dummy version (see xcptstack.c for full implementation)
 */
#pragma weak xcptstackinfo
enum XcptStackTraceStates
xcptstackinfo (struct xcptcontext *xcp, int atdepth, void **vap)
{
    return XcptStackTraceNA;
}


/*
 * Print a stack backtrace.
 */
#pragma weak xcptstacktrace
void
xcptstacktrace (struct xcptcontext *xcp)
{
    enum XcptStackTraceStates state = XcptStackTracePC;
    int depth;
    void *va;

    _mon_puts ("Call stack trace (bottom to top):");

    for (depth = 0; state == XcptStackTracePC
	     && depth <= _xcptstktracedepth; depth++) {

	state = xcptstackinfo (xcp, depth, &va);

	switch (state) {
	case XcptStackTraceLast:
	case XcptStackTracePC:
	    if (depth % PCPERLINE == 0)
		_mon_puts ("\r\n    ");
	    if (depth == _xcptstktracedepth
		&& state != XcptStackTraceLast)
		_mon_puts (" (more)");
	    else
		_mon_printf (" %08lx", (unsigned long)va);
	    break;

	case XcptStackTraceLoop:
	    _mon_printf ("\n     Stack loop!");
	    break;

	case XcptStackTraceBadPC:
	    _mon_printf ("\n     Invalid pc=%08lx", (unsigned long)va);
	    break;

	case XcptStackTraceNA:
	    _mon_printf ("\n     Stack trace module not loaded");
	    break;
	}
    }

    _mon_puts ("\r\n");
}



/*
 * Display cause of exception on console (without using printf).
 */
void 
xcpt_show (struct xcptcontext *xcp)
{
  int regno;

  _mon_printf ("SDE %s xcption (%s)\n",
	       _sys_xcptclass[XCP_CLASS (xcp)], _sys_xcptlist[XCP_CODE (xcp)]);
  xcptwritehex ("Cause ", XCP_CAUSE (xcp));
  xcptwritehex ("Status", XCP_STATUS (xcp));
  xcptwritehex ("EPC   ", xcp->epc);

  switch (_xcpt_signo (xcp)) {
  case SIGBUS:
#if defined(XCPTIBE) && defined(mips_geterraddr)
    xcptwritehex ("ErrAdd", mips_geterraddr());
    /* fall through */
#endif
  case SIGSEGV:
    xcptwritehex ("BadVa ", xcp->vaddr);
    break;
  default:
    break;
  }

  _mon_puts ("Register dump:");
  for (regno = 0; regno < 32; regno++) {
      if (regno % REGPERLINE == 0)
	  _mon_printf ("\n%s\n-%02d-", xcptregnames[regno / REGPERLINE], regno);
      _mon_putc (' ');
      xcptputreg (xcp->regs[regno], XCP_RSIZE);
  }
  _mon_printf ("\n");

  xcptstacktrace (xcp);
}

