#ident "kit/share/xcptshowmin.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2004-2005 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * xcptshowmin.c: display an exception in short form for MINKIT builds
 */

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <mips/cpu.h>
#include <mips/xcpt.h>
#include "kit_impl.h"

static const char *const xcptlist[] =
{
#define XCPTDEF(NAME, NUMBER, SIG, BRIEF, DESC)	BRIEF ,
#include <mips/xcptcpu.h>
#undef XCPTDEF
    0
};


static void
xcptputreg (reg_t val, int nb)
{
  char buf[17], *b = &buf[16];
  int i;

  *--b = '\0';
  for (i = nb * 2; i != 0; i--) {
      *--b = "0123456789abcdef"[val & 0xf];
      val >>= 4;
  }
  _mon_puts (b);
}


static void
xcptwritehex (const char *name, unsigned long val)
{
  _mon_puts (name);
  _mon_puts (": ");
  xcptputreg (val, sizeof (val));
  _mon_puts ("\n");
}

/*
 * Display cause of exception on console (without using printf).
 */
void 
xcpt_showmin (struct xcptcontext *xcp)
{
  _mon_puts ("SDE ");
  _mon_puts (xcptlist[XCP_CODE (xcp)]);
  _mon_puts (" exception\n");

  xcptwritehex ("Cause ", XCP_CAUSE (xcp));
  xcptwritehex ("Status", XCP_STATUS (xcp));
  xcptwritehex ("EPC   ", xcp->epc);
  xcptwritehex ("RA    ", xcp->regs[31]);

  switch (XCP_CODE (xcp)) {
#if defined(XCPTTLBL)
  case XCPTMOD:	case XCPTTLBL:	case XCPTTLBS: 
#elif defined(XCPTIBOUND)
  case XCPTIBOUND: case XCPTDBOUND:
#endif
  case XCPTADEL: case XCPTADES:
    xcptwritehex ("BadVa ", xcp->vaddr);
    break;
#if defined(XCPTIBE) && defined(mips_geterraddr)
  case XCPTIBE:
  case XCPTDBE:
    xcptwritehex ("ErrAdd", mips_geterraddr());
    break;
#endif
  default:
    break;
  }
}

