.ident "kit/share/z8530.s: Copyright (c) MIPS Technologies, Inc. All rights reserved."
	
/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * share/z8530.s: AMD 85c30 / Zilog z8530 / IDT R36100 UART driver.
 */

	
/*	
 * These routines can be called very early in the life of the PROM.
 * - They cannot use dram / stack etc.  
 * - They may only use registers AT, v0-v1, a0-a3, t0-t3
 */

#include <mips/asm.h>
#include <mips/regdef.h>
#include <mips/cpu.h>
	
#include "sbd.h"
#include "sbdser.h"
#include "z8530.h"

#ifdef DUART2_BASE
#define	NSER	4
#else
#define	NSER	2
#endif

#define TXWAIT	10000		/* TxRdy wait timeout */

#ifndef DEFSPEED
#define DEFSPEED	9600	/* default console speed */
#endif
	
#ifndef DBGSPEED
#define DBGSPEED	9600	/* default debug speed */
#endif
	
/* These macros allow for any special action needed before accessing device. */
#ifndef PRE_Z8530
#define PRE_Z8530
#define POST_Z8530
#endif
	
/* Z8530 requires 1-2 us of recovery time between each register access.
   If this isn't enforced by h/w then we have to do it in software.
   NB we assume we're running cached, which will give longer delays 
   when running uncached, but so what! */
	
#if Z8530DELAY > 0	
#define DELAY 			\
        .set    noat; 		\
	li	AT,CACHEUS(Z8530DELAY);	\
	.set	noreorder;	\
10:	bnez	AT,10b;		\
	subu	AT,1;		\
        .set    at; 		\
	.set	reorder
#else
#define DELAY
#endif
		
#ifndef WBFLUSH	
#if __mips >= 2
#define	WBFLUSH 	sync; lb t3,RVEC_BASE; addu t3,1
#else	
#define	WBFLUSH 	lb t3,RVEC_BASE; addu t3,1
#endif	
#endif
		
	.sdata
/* remote debugging port number */
EXPORTS(_sbddbgport, 4)
	.word	1
	
	.rdata
/* device table */
EXPORTS(_sbdsertab, SS_SIZE * NSER)
	.word	 0, z8reset, z8init, z8params, z8rxpoll, z8txrdy, z8tx
	.word	 4, 0,       z8init, z8params, z8rxpoll, z8txrdy, z8tx
#ifdef DUART2_BASE
	.word	 8, z8reset, z8init, z8params, z8rxpoll, z8txrdy, z8tx
	.word	12, 0,       z8init, z8params, z8rxpoll, z8txrdy, z8tx
#endif

/* number of ports */
EXPORTS(_sbdnser, 4)
	.word	NSER

/* default initial port speeds */
EXPORTS(_sbdspeed, NSER*4)
	.word	DEFSPEED, DBGSPEED
#if NSER > 2
	.word	DEFSPEED, DEFSPEED
#endif


	.rdata

/* UART address table (indexed by _sbdsertab[x].ss_data above) */
_addrs:
	.word	PA_TO_KVA1(DUART_BASE)+CHA
	.word	PA_TO_KVA1(DUART_BASE)+CHB
#ifdef DUART2_BASE
	.word	PA_TO_KVA1(DUART2_BASE)+CHA
	.word	PA_TO_KVA1(DUART2_BASE)+CHB
#endif


/* macros for programming z8530 */
#define ZPUTREG(dp,offs,val) \
	li	v0,val; sb v0,offs(dp); WBFLUSH; DELAY
	
#define ZGETREG(dp,offs) \
	lbu	v0,offs(dp); DELAY
	
#define ZPUTCMD(dp,wr,cmd) \
	ZPUTREG(dp,UCMD,wr); ZPUTREG(dp,UCMD,cmd)
	
#define ZPUTCMDREG(dp,wr,cmdr) \
	ZPUTREG(dp,UCMD,wr); sb cmdr,UCMD(dp); WBFLUSH; DELAY
	
#define ZGETSTAT(dp,rr) \
	ZPUTREG(dp,UCMD,rr); ZGETREG(dp,UCMD)
	
	
/*
 * reset (chan, reloc)
 *
 * Once-only device reset at power-on
 */
SLEAF(z8reset)
	and	a1,~1		# clear bit 0 of reloc
	la	t0,_addrs(a1)	# uart address table
	addu	t0,a0		# calc &_addrs[chan]
	lw	a0,0(t0)	# get address

	PRE_Z8530
	
#ifdef Z8530RESET
#if CHA > CHB
	/* only do h/w reset using base of first channel */
	subu	a0,CHA-CHB
#endif
	# h/w reset & enable chip
	Z8530RESET(a0)
	DELAY
#else
	# read RR0 and RR1 to get in sync
	ZGETREG(a0,UCMD)
	ZGETSTAT(a0,RR_ERROR)
	
	# s/w reset chip
	ZPUTCMD(a0,WR_INTCNTRL,UC_HWRESET)
#endif	
	
	# read RR0 and RR1 to get in sync (should check returned values)
	ZGETREG(a0,UCMD)
	ZGETSTAT(a0,RR_ERROR)
		
	POST_Z8530
	j	ra
SEND(z8reset)



/* Channel initialisation table */
	.rdata
_inittab:
#ifdef Z8530INITTAB
	Z8530INITTAB
#else
	/* Z8530 manual specifies order of first 3 writes only... */	
	/* set basic asynchronous mode, 8 bits, 1 stop bit, no parity */
	.byte	WR_MODE, UM_16X_CLK | UM_1STOPBIT | UM_PARITYOFF
	.byte	WR_RXBITS, UM_8BITS_RX
	.byte	WR_TXBITS, UM_8BITS_TX | UM_DTR | UM_RTS
	
	/* Tx & Rx are clocked from baud rate generator */
	.byte	WR_CLKMODE, UM_RXCISBRG | UM_TXCISBRG
	
	/* enable master interrupt, without Z80 style vectoring */
	.byte	WR_INTCNTRL, UM_MIE | UM_NOVEC
	
	/* disable Tx, Rx & Status interrupts */
	.byte	WR_TXRXINT, 0
	.byte	WR_STATCNTRL, 0
	
	/* clear status interrupt latch */
	.byte	WR_COMMAND, UC_RESIST
	.byte	WR_COMMAND, UC_RESIST
	
	/* initialise baud rate generator to 9600 baud */
	.byte	WR_BRGCNT, 0
	.byte	WR_BRGTC_LO, BRTC(Z8530CLOCK, 9600) & 0xff
	.byte	WR_BRGTC_HI, (BRTC(Z8530CLOCK, 9600) >> 8) & 0xff
	.byte	WR_BRGCNT, UM_BRGENABLE
	
	/* finally enable Tx & Rx  */
	.byte	WR_TXBITS, UM_8BITS_TX | UM_DTR | UM_RTS | UM_TXENABLE
	.byte	WR_RXBITS, UM_8BITS_RX | UM_RXENABLE
#endif	
	.byte	-1,-1


/*
 * init (chan, reloc)
 *
 * Per channel initialisation.
 */
SLEAF(z8init)
	and	a1,~1		# clear bit 0 of reloc
	la	t0,_addrs(a1)	# uart address table
	addu	t0,a0		# calc _addrs[chan]
	lw	a0,0(t0)	# get address

	la	a1,_inittab(a1)	# initialisation table

	PRE_Z8530
	
	lbu	zero,UCMD(a0)	# initial read to synchronise
	
1:	lb	t0,0(a1)	# register offset
	lbu	t1,1(a1)	# register value
	addu	a1,2
	bltz	t0,3f		# end of table?

	sb	t0,UCMD(a0)	# select register
	WBFLUSH; DELAY
	sb	t1,UCMD(a0)	# write uart register
	WBFLUSH; DELAY

	b	1b

3:	POST_Z8530
	move	v0,zero
	j	ra
SEND(z8init)




/*
 * params (chan, speed, reloc)
 *
 * program new speed, return 0 on success
 */
SLEAF(z8params)
	move	a3,a2		# temporary copy of reloc
	and	a2,~1		# clear bit zero of reloc
	
	blez	a1,..fail	# don't want divide by zero err
	li	t2,Z8530CLOCK/32 # t2 = Z8530CLK / 32 / speed
	divu	t2,t2,a1
	subu	t2,2		# brtc = (Z8530CLK / 32 / speed) - 2
	blez	t2,..fail	# brtc must be > 0
	
	la	a2,_addrs(a2)	# uart address table base &_addrs[0]
	addu	a2,a0		# calc &_addrs[chan]
	lw	a2,0(a2)	# get channel address in a2

	PRE_Z8530

	/* Wait for Tx to completely finish */
	li	t0,TXWAIT
1:	ZGETSTAT(a2,RR_ERROR)
	and	v0,US_ALLSENT
	bnez	v0,2f
	subu	t0,1
	bnez	t0,1b

2:	ZPUTCMD(a2, WR_BRGCNT, 0)
	ZPUTCMDREG(a2, WR_BRGTC_LO, t2)
	srl	t2,8
	ZPUTCMDREG(a2, WR_BRGTC_HI, t2)
	ZPUTCMD(a2, WR_BRGCNT, UM_BRGENABLE)
					
	POST_Z8530

	move	v0,zero	
	j	ra

..fail:	li	v0,1
	j	ra
SEND(z8params)



/*
 * rxpoll (chan, canblock, reloc) 
 *
 * poll for received character, return zero if none available
 */
SLEAF(z8rxpoll)
	and	a2,~1		# clear bit 0 of reloc
	la	t0,_addrs(a2)	# uart address table
	addu	t0,a0		# calc &_addrs[chan]
	lw	a0,0(t0)	# channel address

	PRE_Z8530
	
	ZGETREG(a0, UCMD)
	and	v0,US_RXRDY
	beqz	v0,1f
	
	ZGETREG(a0, UDATA)
	and	v0,0xff
	or	v0,0x100
	
1:	POST_Z8530
	j	ra
SEND(z8rxpoll)


/* 
 * txrdy (chan, reloc) 
 *
 * return non-zero if ready to transmit
 */
SLEAF(z8txrdy)
	and	a1,~1		# clear bit 0 of reloc
	la	t0,_addrs(a1)	# uart address table
	addu	t0,a0		# calc &_addrs[chan]
	lw	a0,0(t0)	# channel address

	PRE_Z8530
	
	ZGETREG(a0, UCMD)
	and	v0,US_TXRDY
	
	POST_Z8530
	j	ra
SEND(z8txrdy)


/* 
 * tx (chan, data, reloc) 
 *
 * transmit data
 */
SLEAF(z8tx)
	and	a2,~1		# clear bit 0 of reloc
	la	t0,_addrs(a2)	# uart address table
	addu	t0,a0		# calc &_addrs[chan]
	lw	a0,0(t0)	# channel address

	PRE_Z8530

	li	t0,TXWAIT	# TxRdy timeout
1:	ZGETREG(a0, UCMD)
	and	v0,US_TXRDY
	bnez	v0,2f
	subu	t0,1
	bnez	t0,1b

2:	sb	a1,UDATA(a0)
	WBFLUSH; DELAY

	POST_Z8530
	j	ra
SEND(z8tx)
