#ident "kit/yamon/yamonmem.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2000-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * yamonmem.c: determine memory layout under YAMON
 */


#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <mips/cpu.h>
#include "kit_impl.h"

/*
 * _sbd_memlayout returns a pointer to a table of memory regions.
 */

extern char **_yamon_env;
extern void *_maxbrk;
extern void *_maxbrk;
extern paddr_t _mem_top;

const struct sbd_region *
_sbd_memlayout (void)
{
    static struct sbd_region mem[2];

    if (mem[0].type == SBD_MEM_END) {
	char **envp;
	char *sz = NULL;

	/* search yamon environment (non-standard layout) */
	for (envp = _yamon_env; envp && *envp; envp += 2) {
	    if (strcmp (envp[0], "memsize") == 0) {
		sz = envp[1];
		break;
	    }
	}

	mem[0].base = 0x00000000;
	mem[0].type = SBD_MEM_RAM;
	if (_mem_top)
	    /* use saved $a3 from YAMON, if we got it */
	    mem[0].size = _mem_top;
	else if (sz)
	    /* use $memsize if found */
	    mem[0].size = strtoul (sz, NULL, 16);
	else if (_maxbrk)
	    /* use top of heap */
	    mem[0].size = KVA_TO_PA (_maxbrk);
	else
	    /* give up and take an educated guess */
	    /* XXX Could read sys ctrlr memory regs here */
#ifdef SEAD	
	    mem[0].size = 32*1024*1024;
#else
	    mem[0].size = 16*1024*1024;
#endif
    }

    return mem;
}
