#ident "lib/c/arch/mips/divdi.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1998-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * divdi.c: 64bit / 64bit -> 64quo, 64mod division
 */



#include <machine/types.h>
#include <machine/limits.h>

#ifndef LLONG_MIN
#define LLONG_MAX	9223372036854775807LL
#define LLONG_MIN	(-LLONG_MAX-1)
#define ULLONG_MAX	18446744073709551615ULL
#endif

#ifdef TEST
#define fn(x)	test ## x
#else
#define fn(x)	x
#endif

u_int64_t
fn(__udivmoddi4) (u_int64_t num, u_int64_t den, u_int64_t *modp)
{
  u_int64_t bit = 1;
  u_int64_t res = 0;

  if (den == 0) {
#if __mips
      __asm__ __volatile__ ("break 7");
#else
      abort();
#endif
  }

  /* check for obvious short cuts */
  if (num == den) {
      if (modp) *modp = 0;
      return 1;
  }

  if (num < den) {
      if (modp) *modp = num;
      return 0;
  }

  if ((den & (den - 1)) == 0)
    {
      /* quick power of 2 */
      if (modp) *modp = num & (den - 1);
      while (den >= 0x10)
        {
	  den >>=4;
	  num >>=4;
	}
      while (den != 1)
        {
	  den >>=1;
	  num >>=1;
        }
      return num;
    }

  while (den < num && den < 0x1000000000000000LL)
    {
      den <<=4;
      bit <<=4;
    }
  while (den < num && !(den & (1LL<<63)))
    {
      den <<=1;
      bit <<=1;
    }

  while (bit)
    {
      if (num >= den)
	{
	  num -= den;
	  res |= bit;
	}
      bit >>=1;
      den >>=1;
    }

  if (modp) *modp = num;
  return res;
}

u_int64_t
fn(__udivdi3) (u_int64_t a, u_int64_t b)
{
    return fn(__udivmoddi4) (a, b, 0);
}

u_int64_t
fn(__umoddi3) (u_int64_t a, u_int64_t b)
{
    u_int64_t mod;
    (void) fn(__udivmoddi4) (a, b, &mod);
    return mod;
}


u_int64_t
fn(__divdi3) (int64_t a, int64_t b)
{
    u_int64_t quo;
    int sign;

    if (b == -1 && a == LLONG_MIN) {
#if __mips
	__asm__ __volatile__ ("break 6");
#else
	abort ();
#endif
    }

    if (sign = (a < 0))
	a = -a;
    if (b < 0) {
	b = -b;
	sign ^= 1;
    }

    quo = fn(__udivmoddi4) (a, b, 0);
    return sign ? -quo : quo;
}

int64_t
fn(__moddi3) (int64_t a, int64_t b)
{
    u_int64_t mod;
    int sign;

    if (b == -1 && a == LLONG_MIN) {
#if __mips
	__asm__ __volatile__ ("break 6");
#else
	abort ();
#endif
    }

    if (sign = (a < 0))
	a = -a;
    if (b < 0)
	b = -b;

    (void) fn(__udivmoddi4) (a, b, &mod);
    return sign ? -mod : mod;
}

#ifdef TEST
#include <stdlib.h>
#include <stdio.h>

main ()
{
    int i;
    srandom (time (0));
    for (i = 1; i < 100000; i++) {
	int64_t a = (int64_t) random () << (random() & 31);
	int64_t b = (int64_t) random () << (random() & 31);
	int64_t v, want;

	if (random () & 1)
	    a = -a;
	if (random () & 2)
	    b = -b;

	/* don't allow error conditions */
	if (b == 0)
	    b = 1;
	else if (b == -1 && a == LLONG_MIN)
	    b = -12345;

	want = a / b;
	if ((v = test__divdi3 (a, b)) != want)
	    fprintf (stderr, "__divdi3 (%lld,%lld) = %lld, want %lld\n",
		     a, b, v, want);

#if 0
	want = a % b;
	if ((v = test__moddi3 (a, b)) != want)
	    fprintf (stderr, "__moddi3 (%lld,%lld) = %lld, want %lld\n",
		     a, b, v, want);
#endif

	want = (u_int64_t)a / (u_int64_t)b;
	if ((v = test__udivdi3 (a, b)) != want)
	    fprintf (stderr, "__udivdi3 (%llu,%llu) = %llu, want %llu\n",
		     a, b, v, want);

	want = (u_int64_t)a % (u_int64_t)b;
	if ((v = test__umoddi3 (a, b)) != want)
	    fprintf (stderr, "__umoddi3 (%llu,%llu) = %llu, want %llu\n",
		     a, b, v, want);
    }
    exit (0);
}
#endif
