#ident "lib/c/arch/mips/divsi.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1998-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * divsi.c: 32bit / 32bit -> 32quo, 32mod division
 */



#include <machine/types.h>
#include <machine/limits.h>

#ifdef TEST
#define fn(x)	test ## x
#else
#define fn(x)	x
#endif

unsigned int
fn(__udivmodsi4) (unsigned int num, unsigned int den, unsigned int *modp)
{
  unsigned int bit = 1;
  unsigned int res = 0;

  if (den == 0) {
#if __mips
      __asm__ __volatile__ ("break 7");
#else
      abort();
#endif
  }

  /* check for obvious short cuts */
  if (num == den) {
      if (modp) *modp = 0;
      return 1;
  }

  if (num < den) {
      if (modp) *modp = num;
      return 0;
  }

  if ((den & (den - 1)) == 0)
    {
      /* quick power of 2 */
      if (modp) *modp = num & (den - 1);
      while (den >= 0x10)
        {
	  den >>=4;
	  num >>=4;
	}
      while (den != 1)
        {
	  den >>=1;
	  num >>=1;
        }
      return num;
    }

  while (den < num && den < 0x10000000)
    {
      den <<=4;
      bit <<=4;
    }
  while (den < num && !(den & (1L<<31)))
    {
      den <<=1;
      bit <<=1;
    }

  while (bit)
    {
      if (num >= den)
	{
	  num -= den;
	  res |= bit;
	}
      bit >>=1;
      den >>=1;
    }

  if (modp) *modp = num;
  return res;
}

unsigned int
fn(__udivsi3) (unsigned int a, unsigned int b)
{
    return fn(__udivmodsi4) (a, b, 0);
}

unsigned int
fn(__umodsi3) (unsigned int a, unsigned int b)
{
    unsigned int mod;
    (void) fn(__udivmodsi4) (a, b, &mod);
    return mod;
}


unsigned int
fn(__divsi3) (int a, int b)
{
    unsigned int quo, sign;

    if (b == -1 && a == INT_MIN) {
#if __mips
	__asm__ __volatile__ ("break 6");
#else
	abort ();
#endif
    }

    if ((sign = (a < 0)))
	a = -a;
    if (b < 0) {
	b = -b;
	sign ^= 1;
    }

    quo = fn(__udivmodsi4) (a, b, 0);
    return sign ? -quo : quo;
}

unsigned int
fn(__modsi3) (int a, int b)
{
    unsigned int mod, sign;

    if (b == -1 && a == INT_MIN) {
#if __mips
	__asm__ __volatile__ ("break 6");
#else
	abort ();
#endif
    }

    if ((sign = (a < 0)))
	a = -a;
    if (b < 0)
	b = -b;

    (void) fn(__udivmodsi4) (a, b, &mod);
    return sign ? -mod : mod;
}

#ifdef TEST
#include <stdlib.h>
#include <stdio.h>

main ()
{
    int i;
    srandom (time (0));
    for (i = 1; i < 1000000; i++) {
	long a = random ();
	long b = random ();
	long v, want;

	if (random () & 1)
	    a = -a;
	if (random () & 2)
	    b = -b;

	/* don't allow error conditions */
	if (b == 0)
	    b = 1;
	else if (b == -1 && a == INT_MIN)
	    b = -2;

	want = a / b;
	if ((v = test__divsi3 (a, b)) != want)
	    fprintf (stderr, "__divsi3 (%ld,%ld) = %ld, want %ld\n",
		     a, b, v, want);

	want = a % b;
	if ((v = test__modsi3 (a, b)) != want)
	    fprintf (stderr, "__modsi3 (%ld,%ld) = %ld, want %ld\n",
		     a, b, v, want);

	want = (unsigned int)a / (unsigned int)b;
	if ((v = test__udivsi3 (a, b)) != want)
	    fprintf (stderr, "__udivsi3 (%lu,%lu) = %lu, want %lu\n",
		     a, b, v, want);

	want = (unsigned int)a % (unsigned int)b;
	if ((v = test__umodsi3 (a, b)) != want)
	    fprintf (stderr, "__umodsi3 (%lu,%lu) = %lu, want %lu\n",
		     a, b, v, want);
    }
    exit (0);
}
#endif
