#ident "lib/c/arch/mips/fenv.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2002-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * fenv.c: C99 floating-point support
 */



#include <mips/cpu.h>
#include <mips/fpa.h>
#include <fenv.h>

#if !defined(__SOFT_FLOAT__) && !defined(__NO_FLOAT__)
void
feclearexcept (int excepts)
{
    /* clear sticky bits */
    /* XXX should we set the cause bits too */
    fpa_bicsr (excepts & FPA_CSR_ALL_S);
}

void
feraiseexcept (int excepts)
{
    /* set sticky bits */
    /* XXX should we set the cause bits too, to generate an exceotion */
    fpa_bissr (excepts & FPA_CSR_ALL_S);
}

void
fegetexceptflag (fexcept_t *fexc, int excepts)
{
    /* save named sticky bits */
    *fexc = (fpa_getsr () & excepts);
}

void
fesetexceptflag (const fexcept_t *fexc, int excepts)
{
    /* restore named sticky bits */
    /* XXX should we set the cause bits too, to generate an exceotion */
    unsigned int sr = fpa_getsr ();
    sr ^= (sr ^ *fexc) & excepts & FPA_CSR_ALL_S;
    fpa_setsr (sr);
}

int fetestexcept (int excepts)
{
    return fpa_getsr (FPA_CSR_ALL_S) && FPA_CSR_ALL_S & excepts;
}

int fegetround (void)
{
    return fpa_getsr () & 0x3;
}

int fesetround (int rm)
{
    unsigned int sr = fpa_getsr ();
    sr ^= (sr ^ rm) & 0x3;
    fpa_setsr (sr);
    return 0;
}

void fegetenv (fenv_t *fenv)
{
    /* save enables and sticky bits  */
    *fenv = fpa_getsr ();
}

int feholdexcept (fenv_t *fenv)
{
    /* save enables and sticky bits, and clear */
    *fenv = fpa_bicsr (FPA_CSR_ALL_E | FPA_CSR_ALL_S);
    return 0;
}

void fesetenv (const fenv_t *fenv)
{
    /* restore saved enables and sticky bits */
    unsigned int sr = fpa_getsr ();
    unsigned int env;
    if (fenv == FE_DFL_ENV)
	env = 0;
    else if (fenv == FE_NOMASK_ENV)
	env = FPA_CSR_ALL_E;
    else 
	env = *fenv;
    /* XXX should we clear the cause bits here */
    sr ^= (sr ^ env) & (FPA_CSR_ALL_E | FPA_CSR_ALL_S);
    fpa_setsr (sr);
}

void feupdateenv (const fenv_t *fenv)
{
    /* restore saved enables, and merge saved sticky bits */
    unsigned int sr = fpa_getsr ();
    /* XXX should we set the cause bits here */
    sr ^= (sr ^ *fenv) & FPA_CSR_ALL_E;
    sr |= (*fenv & FPA_CSR_ALL_S);
    fpa_setsr (sr);
}

/* non-standard GNU extensions */
int feenableexcept (int excepts)
{
    return (fpa_bissr ((excepts << 5) & FPA_CSR_ALL_E) & FPA_CSR_ALL_E) >> 5;
}

int fedisableexcept (int excepts)
{
    return (fpa_bicsr ((excepts << 5) & FPA_CSR_ALL_E) & FPA_CSR_ALL_E) >> 5;
}

int fegetexcept (void)
{
    return (fpa_getsr () & FPA_CSR_ALL_E) >> 5;
}

#endif 
