#ident "lib/c/arch/mips/gccfpe.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1998-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if defined (__NO_FLOAT)

void FNNAME (void)
{
    abort ();
}

#elif defined (__SOFT_FLOAT) || defined (__SINGLE_FLOAT)

#include <ieee754.h>

#ifdef L_adddf3
double __adddf3(double x,double y)
{
    ieee754dp sx, sy;
    sx.d = x; sy.d = y;
    return ieee754dp_add(sx,sy).d;
}
#endif

#ifdef L_subdf3
double __subdf3(double x,double y)
{
    ieee754dp sx, sy;
    sx.d = x; sy.d = y;
    return ieee754dp_sub(sx,sy).d;
}
#endif

#ifdef L_muldf3
double __muldf3(double x,double y)
{
    ieee754dp sx, sy;
    sx.d = x; sy.d = y;
    return ieee754dp_mul(sx,sy).d;
}
#endif

#ifdef L_divdf3
double __divdf3(double x,double y)
{
    ieee754dp sx, sy;
    sx.d = x; sy.d = y;
    return ieee754dp_div(sx,sy).d;
}
#endif

#ifdef L_negdf2
double __negdf2(double x)
{
    ieee754dp sx;
    sx.d = x;
    return ieee754dp_neg(sx).d;
}
#endif

#ifdef L_absdf2
double __absdf2(double x)
{
    ieee754dp sx;
    sx.d = x;
    return ieee754dp_abs(sx).d;
}
#endif

#ifdef L_srtqdf2
double __sqrtdf2(double x)
{
    ieee754dp sx;
    sx.d = x;
    return ieee754dp_sqrt(sx).d;
}
#endif

#ifdef L_sqrtsf2
float __sqrtsf2(float x)
{
    ieee754sp sx;
    sx.f = x;
    return ieee754sp_sqrt(sx).f;
}
#endif

#ifdef L_fixdfsi
int __fixdfsi(double x)
{
    ieee754dp sx;
    sx.d = x;
    return ieee754dp_tint(sx);
}
#endif

#ifdef L_fixunsdfsi
unsigned int __fixunsdfsi(double x)
{
    ieee754dp sx;
    sx.d = x;
    return ieee754dp_tuns(sx);
}
#endif

#ifdef L_fixdfdi
long long __fixdfdi(double x)
{
    ieee754dp sx;
    sx.d = x;
    return ieee754dp_tlong(sx);
}
#endif

#ifdef L_fixunsdfdi
unsigned long long __fixunsdfdi(double x)
{
    ieee754dp sx;
    sx.d = x;
    return ieee754dp_tulong(sx);
}
#endif

#ifdef L_floatsidf
double __floatsidf(int x)
{
    return ieee754dp_fint(x).d;
}
#endif

#ifdef L_floatdidf
double __floatdidf(long long x)
{
    return ieee754dp_flong(x).d;
}
#endif

#ifdef L_extendsfdf2
double __extendsfdf2(float x)
{
    ieee754sp sx;
    sx.f = x;
    return ieee754dp_fsp(sx).d;
}
#endif

/* dp compares with gcc funny return value 
*/
#ifdef L_eqdf2
int __eqdf2(double x,double y)
{
    ieee754dp sx, sy;
    sx.d = x; sy.d = y;
    return !ieee754dp_eq(sx,sy);
}
#endif

#ifdef L_nedf2
int __nedf2(double x,double y)
{
    ieee754dp sx, sy;
    sx.d = x; sy.d = y;
    return ieee754dp_ne(sx,sy);
}
#endif

#ifdef L_gtdf2
int __gtdf2(double x,double y)
{
    ieee754dp sx, sy;
    sx.d = x; sy.d = y;
    return ieee754dp_gt(sx,sy);
}
#endif

#ifdef L_gedf2
int __gedf2(double x,double y)
{
    ieee754dp sx, sy;
    sx.d = x; sy.d = y;
    return ieee754dp_ge(sx,sy) - 1;
}
#endif

#ifdef L_ltdf2
int __ltdf2(double x,double y)
{
    ieee754dp sx, sy;
    sx.d = x; sy.d = y;
    return -ieee754dp_lt(sx,sy);
}
#endif

#ifdef L_ledf2
int __ledf2(double x,double y)
{
    ieee754dp sx, sy;
    sx.d = x; sy.d = y;
    return 1-ieee754dp_le(sx,sy);
}
#endif

#ifdef L_unordsf2
int __unorddf2(double x,double y)
{
    ieee754dp sx, sy;
    sx.d = x; sy.d = y;
    return (ieee754dp_isnan(sx) || ieee754dp_isnan(sy));
}
#endif

#ifdef L_addsf3
float __addsf3(float x,float y)
{
    ieee754sp sx, sy;
    sx.f = x; sy.f = y;
    return ieee754sp_add(sx,sy).f;
}
#endif

#ifdef L_subsf3
float __subsf3(float x,float y)
{
    ieee754sp sx, sy;
    sx.f = x; sy.f = y;
    return ieee754sp_sub(sx,sy).f;
}
#endif

#ifdef L_mulsf3
float __mulsf3(float x,float y)
{
    ieee754sp sx, sy;
    sx.f = x; sy.f = y;
    return ieee754sp_mul(sx,sy).f;
}
#endif

#ifdef L_divsf3
float __divsf3(float x,float y)
{
    ieee754sp sx, sy;
    sx.f = x; sy.f = y;
    return ieee754sp_div(sx,sy).f;
}
#endif

#ifdef L_negsf2
float __negsf2(float x)
{
    ieee754sp sx;
    sx.f = x;
    return ieee754sp_neg(sx).f;
}
#endif

#ifdef L_abssf2
float __abssf2(float x)
{
    ieee754sp sx;
    sx.f = x;
    return ieee754sp_abs(sx).f;
}
#endif

#ifdef L_fixsfsi
int __fixsfsi(float x)
{
    ieee754sp sx;
    sx.f = x;
    return ieee754sp_tint(sx);
}
#endif

#ifdef L_fixunssfsi
unsigned int __fixunssfsi(float x)
{
    ieee754sp sx;
    sx.f = x;
    return ieee754sp_tuns(sx);
}
#endif

#ifdef L_floatsisf
float __floatsisf(int x)
{
    return ieee754sp_fint(x).f;
}
#endif

#ifdef L_fixsfdi
long long __fixsfdi(float x)
{
    ieee754sp sx;
    sx.f = x;
    return ieee754sp_tlong(sx);
}
#endif

#ifdef L_fixunssfdi
unsigned long long __fixunssfdi(float x)
{
    ieee754sp sx;
    sx.f = x;
    return ieee754sp_tulong(sx);
}
#endif

#ifdef L_floatdisf
float __floatdisf(long long x)
{
    return ieee754sp_flong(x).f;
}
#endif

#ifdef L_truncdfsf2
float __truncdfsf2(double x)
{
    ieee754dp sx;
    sx.d = x;
    return ieee754sp_fdp(sx).f;
}
#endif

/* sp compares with gcc funny return value 
*/
#ifdef L_eqsf2
int __eqsf2(float x,float y)
{
    ieee754sp sx, sy;
    sx.f = x; sy.f = y;
    return !ieee754sp_eq(sx,sy);
}
#endif

#ifdef L_nesf2
int __nesf2(float x,float y)
{
    ieee754sp sx, sy;
    sx.f = x; sy.f = y;
    return ieee754sp_ne(sx,sy);
}
#endif

#ifdef L_gtsf2
int __gtsf2(float x,float y)
{
    ieee754sp sx, sy;
    sx.f = x; sy.f = y;
    return ieee754sp_gt(sx,sy);
}
#endif

#ifdef L_gesf2
int __gesf2(float x,float y)
{
    ieee754sp sx, sy;
    sx.f = x; sy.f = y;
    return ieee754sp_ge(sx,sy) - 1;
}
#endif

#ifdef L_ltsf2
int __ltsf2(float x,float y)
{
    ieee754sp sx, sy;
    sx.f = x; sy.f = y;
    return -ieee754sp_lt(sx,sy);
}
#endif

#ifdef L_lesf2
int __lesf2(float x,float y)
{
    ieee754sp sx, sy;
    sx.f = x; sy.f = y;
    return 1-ieee754sp_le(sx,sy);
}
#endif

#ifdef L_unordsf2
int __unordsf2(float x,float y)
{
    ieee754sp sx, sy;
    sx.f = x; sy.f = y;
    return (ieee754sp_isnan(sx) || ieee754sp_isnan(sy));
}
#endif

#endif /* !NOFLOAT */
