#ident "lib/c/arch/mips/mulsi.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1998-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * mulsi.c: 32bit x 32bit -> 32bit multiplication
 */


extern const unsigned char __mul4x4tab[256];

unsigned int
__mulsi3 (unsigned int a, unsigned int b)
{	
   unsigned int ret = 0;
   int i, j;

   for (i = 0; a != 0; i += 4)
     {
       unsigned a_part = a & 0x0f;
       a >>= 4;
       if (a_part != 0) {
	   unsigned int b_tmp;
	   a_part <<= 4;
	   for (j = i, b_tmp = b; b_tmp != 0 && j < 32; j += 4, b_tmp >>= 4)
	       ret += __mul4x4tab[a_part + (b_tmp & 0x0f)] << j;
       }
    }

   return ret;
}


#ifdef TEST
#include <stdlib.h>
#include <stdio.h>

main ()
{
    int i;
    srandom (time (0));
    for (i = 1; i < 100000; i++) {
	long a = random ();
	long b = random ();
	long v, want;
	long long lwant, lv, la, lb;

	if (random () & 1)
	    a = -a;
	if (random () & 2)
	    b = -b;

	want = a * b;
	if ((v = __mulsi3 (a, b)) != a * b)
	    fprintf (stderr, "_mulsi3 (%ld,%ld) = %ld, want %ld\n",
		     a, b, v, want);
	else if ((v = __mulsi3 (b, a)) != a * b)
	    fprintf (stderr, "_mulsi3 (%ld,%ld) = %ld, want %ld\n",
		     b, a, v, want);

	la = (unsigned long)a;
	lb = (unsigned long)b;
	lwant = la * lb;
	if ((lv = __umulsidi3 (a, b)) != lwant)
	    fprintf (stderr, "_umulsidi3 (%ld,%ld) = 0x%016llx, want 0x%016llx\n",
		     a, b, lv, lwant);
	else if ((lv = __umulsidi3 (b, a)) != lwant)
	    fprintf (stderr, "_umulsidi3 (%ld,%ld) = 0x%016llx, want 0x%016llx\n",
		     b, a, lv, lwant);

	la = a;
	lb = b;
	lwant = la * lb;
	if ((lv = __mulsidi3 (a, b)) != lwant)
	    fprintf (stderr, "_mulsidi3 (%ld,%ld) = 0x%016llx, want 0x%016llx\n",
		     a, b, lv, lwant);
	else if ((lv = __mulsidi3 (b, a)) != lwant)
	    fprintf (stderr, "_mulsidi3 (%ld,%ld) = 0x%016llx, want 0x%016llx\n",
		     b, a, lv, lwant);
    }
    exit (0);
}
#endif
