#ident "lib/c/arch/mips/mulsidi.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1998-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * mulsidi.c: 32bit x 32bit -> 64bit multiplication
 */


extern const unsigned char __mul4x4tab[256];


unsigned long long
__umulsidi3 (unsigned int a, unsigned int b)
{	
   unsigned long long ret = 0;
   int i, j;

   for (i = 0; a != 0; i += 4)
     {
       unsigned a_part = a & 0x0f;
       a >>= 4;
       if (a_part != 0) {
	   unsigned int b_tmp;
	   a_part <<= 4;
	   for (j = i, b_tmp = b; b_tmp != 0 && j < 64; j += 4, b_tmp >>= 4)
	       ret += (unsigned long long)__mul4x4tab[a_part + (b_tmp & 0x0f)] << j;
       }
    }

   return ret;
}


unsigned long long
__mulsidi3 (int a, int b)
{	
   unsigned long long ret;
   int sign = 0;

   if (a < 0) {
       a = -a;
       sign = 1;
   }
   if (b < 0) {
       b = -b;
       sign ^= 1;
   }
       
   ret = __umulsidi3 (a, b);
   return sign ? -ret : ret;
}
