/*
 * Copyright (c) 2001-2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * libc_private.h: per-thread and per-process support
 */


#ifndef __LIBC_PRIVATE_H__
#define __LIBC_PRIVATE_H__

#include <libc_thread.h>

#ifdef __cplusplus
extern "C" {
#endif

/* The atexit() list is maintained in a list of these structures */
#define _ATEXIT_CHUNK	32

struct _exitargs {
    void *fnargs[_ATEXIT_CHUNK];
    void *dso[_ATEXIT_CHUNK];
    unsigned long fnargs_p;
    unsigned long cxa_p;
};

struct _atexit {
    struct _atexit	*next;
    int			nexfun;
    union _funtab {
	void	(*f0)(void);
	void	(*f1)(void *args);
	void	(*f2)(void *args, int status);
    } funtab[_ATEXIT_CHUNK];
    struct _exitargs	exitargs;
};

enum _atexit_types { __t_atexit, __t_cxa };

extern int __register_exitfun (int type, void (*fun)(void), void *arg, void *dso);
extern void __call_exitfuns (int code, void *dso);

extern __sdethread_mutex_t __exfun_mx;

#ifdef __cplusplus
}
#endif
#endif /* __LIBC_PRIVATE_H__ */
