#ident "lib/c/px/execvp.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * execvp.c : execvp POSIX function
 */

#define _POSIX_SOURCE
#include <stddef.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>

/* XXX oops non POSIX */
#include <alloca.h>

int execvp(const char *name, const char * const * argv)
{
    char	*path;
    char	*filebuf;

    if(strchr(name,'/') ) 
      return execv(name,argv);
    
    if((path = getenv("PATH")) == NULL)
      path = ":/bin:/usr/bin";
    
    /* find longest directory component of path 
     * and alloc buffer big enough
     */
    {
	int maxent = 0;
	char *ep = path;

	while(ep) {
	    char *nep;
	    int	  len;
	    
	    if(nep = strchr(ep,':')) {
		len = nep - ep;
		ep = nep+1;
	    } else {
		len = strlen(ep);
		ep = NULL;
	    }
	    if(len > maxent)
	      maxent = len;
	}
	
	/* 1 for a possible '/' and 1 for the terminating null */
	
	filebuf = alloca(maxent + 1 + strlen(name) + 1);
    }
    if(filebuf == NULL) {
	errno = ENOMEM;
	return -1;
    }
    
    while(*path) {
	char *fp = filebuf;
	while(*path) {
	    if(*path == ':') {
		path++;
		break;
	    }
	    *fp++ = *path++;
	}

	/* add a slash if neccessary */
	if(fp[-1] != '/')
	  *fp++ = '/';
	
	/* and add filename */
	strcpy(fp,name);

	execv(filebuf,argv);

	/* try as shell script ?? 
	 */

	/*
	 * keeep going for certain errors only
	 */
	switch(errno) {
	case ENOENT:
	    continue;
	default:
	    return -1;
	}
    }
    return -1;
}
