#ident "lib/c/px/getgrent.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * getgrent.c: POSIX library - get entry from group file
 */


/* TODO
 * make thread safe
 */

#define _POSIX_SOURCE
#include <sys/types.h>
#include <fcntl.h>
#include <grp.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>


static char _gr_file[] = "/etc/group";
static char _grbuf[256];
static char _buffer[1024];
static char *_pnt;
static char *_buf;
static int  _gfd = -1;
static int  _bufcnt;
static struct group grp;
static char *_mem = NULL;

static int setgrent()
{
  if (_gfd >= 0)
	lseek (_gfd, 0L, SEEK_SET);
  else
	_gfd = open(_gr_file, O_RDONLY);

  _bufcnt = 0;
  return (_gfd);
}


static void endgrent() 
{
  if (_gfd >= 0)
	close (_gfd);

  _gfd = -1;
  _bufcnt = 0;
}


static int _getline() 
{
  if (_gfd < 0 && setgrent() < 0)
	return 0;

  _buf = _grbuf;
  do {
	if (--_bufcnt <= 0) {
		if ((_bufcnt = read(_gfd, _buffer, 1024)) <= 0)
			return 0;
		else
			_pnt = _buffer;
	}
	*_buf++ = *_pnt++;
  } while (*_pnt != '\n');
  _pnt++;
  _bufcnt--;
  *_buf = 0;
  _buf = _grbuf;
  return 1;
}

static void _skip_period() 
{
  while ((*_buf) && (*_buf != ':'))
	_buf++;
  *_buf++ = '\0';
}

static struct group *getgrent() 
{
  if (_getline() == 0)
       return 0;

  grp.gr_name = _buf;
  _skip_period();
  grp.gr_passwd = _buf;
  _skip_period();
  grp.gr_gid = (gid_t) atoi (_buf);
  _skip_period();
  return &grp;
}

static void _grp_members()  /* PENDING - needs to be rather more elaborate */
{
  grp.gr_mem = &_mem;
}

struct group *getgrnam(const char *name)
{
  struct group *grp;

  setgrent();
  while ((grp = getgrent()) != 0)
	if (!strcmp(grp->gr_name, name))
		break;
  endgrent();
  _grp_members();
  return grp;
}

struct group *getgrgid(gid_t gid)
{
  struct group *grp;

  setgrent();
  while ((grp = getgrent()) != 0)
	if (grp->gr_gid == gid)
		break;
  endgrent();
  _grp_members();
  return grp;
}
