#ident "lib/c/px/getpwent.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * getpwent.c: POSIX library - get entry from password file
 */


/* TODO
 * make thread safe
 */

#define _POSIX_SOURCE
#include <sys/types.h>
#include <fcntl.h>
#include <pwd.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#define PRIVATE static

static char  _pw_file[] = "/etc/passwd";
static char  _pwbuf[256];
static char  _buffer[1024];
static char *_pnt;
static char *_buf;
static int   _pw = -1;
static int   _bufcnt;
static struct passwd pwd;

static int setpwent() 
{
  if (_pw >= 0)
	lseek(_pw, 0L, SEEK_SET);
  else
	_pw = open(_pw_file, O_RDONLY);

  _bufcnt = 0;
  return _pw;
}


static void endpwent() 
{
  if (_pw >= 0)
	close (_pw);

  _pw = -1;
  _bufcnt = 0;
}

static int _getline() 
{
  if (_pw < 0 && setpwent() < 0)
	return 0;
  _buf = _pwbuf;
  do {
	if (--_bufcnt <= 0) {
		if ((_bufcnt = read(_pw, _buffer, 1024)) <= 0)
			return 0;
		else
			_pnt = _buffer;
	}
	*_buf++ = *_pnt++;
  } while (*_pnt != '\n');
  _pnt++;
  _bufcnt--;
  *_buf = 0;
  _buf = _pwbuf;
  return 1;
}

static void _skip_period() 
{
  while (*_buf != ':')
	_buf++;
  *_buf++ = '\0';
}

static struct passwd *getpwent() 
{
  if (_getline() == 0)
	return 0;

  pwd.pw_name = _buf;
  _skip_period();
  pwd.pw_passwd = _buf;
  _skip_period();
  pwd.pw_uid = (uid_t) atoi (_buf);
  _skip_period();
  pwd.pw_gid = (gid_t) atoi (_buf);
  _skip_period();
  pwd.pw_gecos = _buf;
  _skip_period();
  pwd.pw_dir = _buf;
  _skip_period();
  pwd.pw_shell = _buf;

  return &pwd;
}

struct passwd *getpwnam(const char *name)
{
  struct passwd *pwd;

  setpwent();
  while ((pwd = getpwent()) != 0)
	if (!strcmp(pwd->pw_name, name))
		break;
  endpwent();
  return pwd;
}

struct passwd *getpwuid(uid_t uid)
{
  struct passwd *pwd;

  setpwent();
  while ((pwd = getpwent()) != 0)
	if (pwd->pw_uid == uid)
		break;
  endpwent();
  return pwd;
}
