#ident "lib/c/px/popen.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * popen.c: POSIX library - run a command with a pipe
 */


#define _POSIX_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

/* association list FILE * with the pid 
*/
struct porec {
    FILE		*fp;
    int			pid;
    struct porec	*next;
};

static struct porec *poreclist;

FILE *popen(const char *program,const char *type)
{
    FILE 	*fp;
    int 	pdes[2];
    pid_t 	pid;
    struct porec *porec;
    
    if ( (type[0] != 'r' && type[0] != 'w') || type[1])
      return (NULL);

    if( (porec=malloc(sizeof(*porec))) == NULL)
      return NULL;
    
    if (pipe(pdes) < 0)
      return (NULL);
    
    switch (pid = fork()) {
    case -1:			/* error */
	(void) close(pdes[0]);
	(void) close(pdes[1]);
	free(porec);
	return (NULL);
	/* NOTREACHED */
    case 0:			/* child */
	if (type[0] == 'r') {
	    if (pdes[1] != STDOUT_FILENO) {
		(void) dup2(pdes[1], STDOUT_FILENO);
		(void) close(pdes[1]);
	    }
	    (void) close(pdes[0]);
	} else {
	    if (pdes[0] != STDIN_FILENO) {
		(void) dup2(pdes[0], STDIN_FILENO);
		(void) close(pdes[0]);
	    }
	    (void) close(pdes[1]);
	}
	execl("/bin/sh", "sh", "-c", program, NULL);
	_exit(127);
	/* NOTREACHED */
    }
    
    /* XXX fdopen fails then the command runs but never 
     * gets any input !!
     */
    
    if (type[0] == 'r') {
	fp = fdopen(pdes[0], type);
	(void) close(pdes[1]);
    } else {
	fp = fdopen(pdes[1], type);
	(void) close(pdes[0]);
    }
    porec->fp = fp;
    porec->pid = pid;
    
    porec->next = poreclist;
    poreclist = porec;
    
    return (fp);
}

int pclose(FILE *fp)
{
    sigset_t omask,nmask;
    int stat;
    pid_t pid;
    
    /*
     * pclose returns -1 if stream is not associated with a
     * `popened' command, if already `pclosed', or waitpid
     * returns an error.
     */
    {
	struct porec *pporec = 0;
	struct porec	*porec;
	
	for(porec=poreclist; porec; pporec=porec,porec = porec->next)
	  if(porec->fp == fp)
	    break;
	if(porec == NULL)
	  return (-1);
	
	pid = porec->pid;
	
	if(pporec)
	  pporec->next = porec->next;
	else
	  poreclist=porec->next;
	
	
	free(porec);
    }
    
    (void) fclose(fp);
    
    sigemptyset(&nmask);
    sigaddset(&nmask,SIGINT);
    sigaddset(&nmask,SIGQUIT);
    sigaddset(&nmask,SIGHUP);
    
    sigprocmask(SIG_BLOCK,&nmask,&omask);
    
    while(waitpid(pid,&stat,0) == -1 ) {
	if(errno != EINTR) {
	    stat = -1;
	    break;
	}
    }
    sigprocmask(SIG_SETMASK,&omask,0);
    
    return stat;
}
