#ident "lib/c/px/setlocale.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * setlocale.c: POSIX library 
 */


#include <stddef.h>
#include <locale.h>
#include <limits.h>
#include "mbctype.h"

#define sdata __attribute__ ((section (".sdata")))

/* internal to libc */
sdata char __lc_ctype[12] = "C";

/* visible via MB_CUR_MAX */
sdata int __mb_cur_max = 1;

/* should these be thread local? */
static char *charset = "ISO-8859-1";
static int cur_category;
static const char *cur_locale;

static const struct lconv lconv = 
{
  ".", "", "", "", "", "", "", "", "", "",
  CHAR_MAX, CHAR_MAX, CHAR_MAX, CHAR_MAX,
  CHAR_MAX, CHAR_MAX, CHAR_MAX, CHAR_MAX,
};

char *setlocale(int category, const char *locale)
{
#ifndef _MB_SUPPORT
  if (locale != NULL
      && locale[0] != '\0' 
      && (locale[0] != 'C' || locale[1] != '\0')
      && strcmp (locale, "POSIX") != 0) 
    return NULL;
  cur_category = category;  
  cur_locale = "C";
#else
  static char last_lc_ctype[12] = "C";
  static char lc_messages[12] = "C";
  static char last_lc_messages[12] = "C";

  if (locale)
    {
      char *locale_name = (char *)locale;
      if (category != LC_CTYPE && category != LC_MESSAGES) 
        { 
	    if (locale[0] != '\0' 
		&& (locale[0] != 'C' || locale[1] != '\0')
		&& strcmp (locale, "POSIX") != 0) 
	      return NULL;
          if (category == LC_ALL)
            {
              strcpy (last_lc_ctype, __lc_ctype);
              strcpy (__lc_ctype, "C");
              strcpy (last_lc_messages, lc_messages);
              strcpy (lc_messages, "C");
              __mb_cur_max = 1;
            }
        }
      else
        {
          if (locale[0] == 'C' && locale[1] == '-')
            {
              switch (locale[2])
                {
                case 'U':
                  if (strcmp (locale, "C-UTF-8"))
                    return 0;
		  break;
                case 'J':
                  if (strcmp (locale, "C-JIS"))
                    return 0;
		  break;
                case 'E':
                  if (strcmp (locale, "C-EUCJP"))
                    return 0;
		  break;
                case 'S':
                  if (strcmp (locale, "C-SJIS"))
                    return 0;
		  break;
                case 'I':
                  if (strcmp (locale, "C-ISO-8859-1"))
                    return 0;
		  break;
                default:
                  return 0;
                }
            }
          else 
            {
	      if (locale[0] != '\0' 
		  && (locale[0] != 'C' || locale[1] != '\0')
		  && strcmp (locale, "POSIX") != 0) 
		return NULL;
              locale_name = "C"; /* C is always the default locale */
            }

          if (category == LC_CTYPE)
            {
              strcpy (last_lc_ctype, __lc_ctype);
              strcpy (__lc_ctype, locale_name);

              __mb_cur_max = 1;
              if (locale[1] == '-')
                {
                  switch (locale[2])
                    {
                    case 'U':
                      __mb_cur_max = 6;
		      break;
                    case 'J':
                      __mb_cur_max = 8;
		      break;
                    case 'E':
                      __mb_cur_max = 2;
		      break;
                    case 'S':
                      __mb_cur_max = 2;
		      break;
                    case 'I':
                    default:
                      __mb_cur_max = 1;
		      break;
                    }
                }
            }
          else
            {
              strcpy (last_lc_messages, lc_messages);
              strcpy (lc_messages, locale_name);

              charset = "ISO-8859-1";
              if (locale[1] == '-')
                {
                  switch (locale[2])
                    {
                    case 'U':
                      charset = "UTF-8";
                    break;
                    case 'J':
                      charset = "JIS";
                    break;
                    case 'E':
                      charset = "EUCJP";
                    break;
                    case 'S':
                      charset = "SJIS";
                    break;
                    case 'I':
                      charset = "ISO-8859-1";
                    break;
                    default:
                      return NULL;
                    }
                }
            }
        }

      cur_category = category;  
      cur_locale = locale;

      if (category == LC_CTYPE)
        return last_lc_ctype;
      else if (category == LC_MESSAGES)
        return last_lc_messages;
    }
  else
    {
      if (category == LC_CTYPE)
        return __lc_ctype;
      else if (category == LC_MESSAGES)
        return lc_messages;
    }
#endif
    return "C";
}

struct lconv *
localeconv (void)
{
  return (struct lconv *) &lconv;
}

char * __locale_charset (void)
{
  return charset;
}
