#ident "lib/c/std/_call_exitfun.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * _call_exitfun.c : stdlib function
 */

#include <stddef.h>
#include <stdlib.h>
#include <unistd.h>
#include "libc_private.h"

__sdethread_mutex_t __exfun_mx = __SDETHREAD_MUTEX_INITIALIZER("atexit");

void __call_exitfun (int status, void *dso)
{
    struct _atexit *ex;
    struct _exitargs *args;

    /* Potentially unsafe: what if exit is called while atexit() is
       suspended in malloc() with this mutex still locked, but then
       any of the functions called from here might block. */
    __sdethread_mutex_lock (&__exfun_mx);
    _libc_data->_exiting = 1;
    __sdethread_mutex_unlock (&__exfun_mx);

    for (ex = _libc_data->_atexit; ex; ex = ex->next) {
	int n = ex->nexfun;
	while (n) {
	    args = &ex->exitargs;
	    n--;
	    /* If there is a DSO, skip functions which aren't in this DSO. */
	    if (dso && (args->dso[n] != dso)) {
		continue;
	    }
	    if (args->fnargs_p) {
		if (args->cxa_p & (1 << n))
		    (ex->funtab[n].f2) (args->fnargs[n], status);
		else
		    (ex->funtab[n].f1) (args->fnargs[n]);
	    } else
		(ex->funtab[n].f0) ();
	}
    }
    if (!dso) {
	struct _atexit *tmp;
	ex = _libc_data->_atexit->next;
	while (ex) {
	    tmp = ex;
	    ex = ex->next;
	    free (tmp);
	}
    }
}
