#ident "lib/c/std/_register_exitfun.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * _register_exitfun.c : stdlib function
 */

#include <stddef.h>
#include <stdlib.h>
#include "libc_private.h"

int __register_exitfun (int type, void (*fun)(void), void *arg, void *dso)
{
    struct _atexit *ex;
    struct _exitargs *args;

    __sdethread_mutex_lock (&__exfun_mx);
    if (_libc_data->_exiting) {
	__sdethread_mutex_unlock (&__exfun_mx);
	return -1;
    }

    ex = _libc_data->_atexit;
    if (ex->nexfun >= _ATEXIT_CHUNK) {
	if (! (ex = calloc (1, sizeof (*ex)))) {
	    __sdethread_mutex_unlock (&__exfun_mx);
	    return -1;
	}
	ex->nexfun = 0;
	ex->next = _libc_data->_atexit;
	_libc_data->_atexit = ex;
    }
    if (type == __t_cxa) {
	args = &ex->exitargs;
	args->fnargs[ex->nexfun] = arg;
	args->dso[ex->nexfun] = dso;
	args->fnargs_p |= (1 << ex->nexfun);
	args->cxa_p |= (1 << ex->nexfun);
    }
    ex->funtab[ex->nexfun++].f0 = fun;

    __sdethread_mutex_unlock (&__exfun_mx);
    return 0;
}
