#ident "lib/c/std/frexp.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * ./frexp.c : stdlib function
 */


#include <stdlib.h>

#if defined(__NO_FLOAT)

double frexp(double v,int *eptr)
{
    abort ();
}

#elif defined(__SOFT_FLOAT) || defined(__SINGLE_FLOAT)

double frexp(double v,int *eptr)
{
    extern double __frexp(double,int *);
    return __frexp(v, eptr);
}

#else

#include <math.h>
#include <machine/endian.h>

#define DBL_EBIAS 1023

double frexp(double v,int *eptr)
{
    union {
	struct {
#if BYTE_ORDER == LITTLE_ENDIAN
	    unsigned mlo:32;
	    unsigned mhi:20;
	    unsigned bexp:11;
	    unsigned s:1;
#else
	    unsigned s:1;
	    unsigned bexp:11;
	    unsigned mhi:20;
	    unsigned mlo:32;
#endif
	} parts;
	double v;
    } u;
    
    if(v == 0.0 || !finite(v)) {
	*eptr = 0;
	return v;
    } else {
	u.v = v;
	if(u.parts.bexp == 0) {
	    u.v = ldexp(u.v,100);	/* mult 2^100 */
	    *eptr = (int)u.parts.bexp - DBL_EBIAS+1 - 100;
	} else {
	    *eptr = (int)u.parts.bexp - DBL_EBIAS+1;
	}
	u.parts.bexp = DBL_EBIAS-1;
	return u.v;
    }
}

#endif
