#ident "lib/c/std/ldexp.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * ldexp.c : stdlib function
 */


#include <stdlib.h>
#include <stdint.h>
#include <machine/endian.h>
#include <values.h>

#if _IEEE == 1

#if defined (__NO_FLOAT)

double
ldexp (double v, int n)
{
    abort ();
}

#elif defined(__SOFT_FLOAT) || defined(__SINGLE_FLOAT)

double
ldexp (double v, int n)
{
    extern double __ldexp (double, int);
    return __ldexp (v, n);
}

#else

#define mexp 	0x7ff0	/* the 'top' short when masked is the exponent bits */
#define msign	0x7fff	/* the 'top' short when masked is the non sign bits */
#define gap	4	/* trailing bits at the end of the short */
#define bias 	1023	/* the exponent bias */
#define prep1	54	/* mantissa size + 2 (??) */
#define zero	0.0

/* most-significant 16-bit word */
#if BYTE_ORDER == LITTLE_ENDIAN
#define ix	3
#else
#define ix	0
#endif

static const double novf = DBL_MAX;
static const double nunf = DBL_MIN;

double ldexp (double x, int N)
{
    union {
	double 	d;
	int16_t h[4];
    } xu;
    int k;

    if (x == zero)
      return (x); 

    xu.d = x;
    if ((k = xu.h[ix] & mexp) != mexp) {	/* not Nan or Inf */

	if (N < -2100) 
	  return (nunf*nunf);	/* cause underflow */
	else if (N > 2100) 
	  return (novf+novf);	/* cause overflow  */
	
	if (k == 0) {		/* denorm */
	    x *= ldexp (1.0, prep1);
	    return (ldexp (x, N - prep1));
	}
	    
	if ((k = (k>>gap)+ N) > 0)
	  if (k < (mexp >> gap)) 
	    xu.h[ix] = (xu.h[ix] & ~mexp) | (k << gap);
	  else 
	    xu.d =novf+novf;	/* overflow */
	else if (k > -prep1)  {
	    /* gradual underflow */
	    xu.h[ix] = (xu.h[ix] & ~mexp) | (1 << gap);
	    xu.d *= ldexp (1.0, k-1);
	} else
	  return (nunf*nunf);
    }

    return (xu.d);
}

#endif /* __SOFT_FLOAT */

#endif /* _IEEE == 1 */

