#ident "lib/c/std/ldexpf.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * ldexpf.c : stdlib function
 */


#include <stdlib.h>
#include <stdint.h>
#include <machine/endian.h>
#include <values.h>
#include <float.h>

#if _IEEE == 1

#if defined (__NO_FLOAT)

float
ldexpf (float v, int n)
{
    abort ();
}

#elif defined(__SOFT_FLOAT)

float
ldexpf (float v, int n)
{
    extern float __ldexpf (float, int);
    return __ldexpf (v, n);
}

#else

#define mexp 	0x7f800000	/* the mask for the exponent bits */
#define msign	0x7fffffff	/* the mask for the non sign bits */
#define gap	23		/* the mantissa bits */
#define bias 	127		/* the exponent bias */
#define prep1	25		/* mantissa size + 2 (??) */
#define zero	0.0f

static const float novf = FLT_MAX;
static const float nunf = FLT_MIN;


float ldexpf (float x, int N)
{
    union {
	float 	f;
	int32_t i;
    } xu;
    int k;

    if (x == zero)
      return (x); 

    xu.f = x;
    if ((k = xu.i & mexp) != mexp) {	/* not Nan or Inf */

	if (N < -279)
	  return(nunf*nunf);	/* cause underflow */
	else if (N > 279) 
	  return(novf+novf);	/* cause overflow  */
	
	if (k == 0) {		/* denorm */
	    x *= ldexpf (1.0f, prep1);
	    return (ldexpf (x, N - prep1));
	}
	    
	if ((k = (k >> gap) + N) > 0)
	  if (k < (mexp >> gap)) 
	    xu.i = (xu.i & ~mexp) | (k << gap);
	  else 
	    xu.f =novf+novf;	/* overflow */
	else if (k > -prep1)  {
	    /* gradual underflow */
	    xu.i = (xu.i & ~mexp) | (1 << gap);
	    xu.f *= ldexpf (1.0f, k - 1);
	} else
	  return (nunf*nunf);
    }
    return (xu.f);
}

#endif /* __SOFT_FLOAT */

#endif /* _IEEE == 1 */

