#ident "lib/c/std/mbstowcs.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * mbstowcs.c: multi-byte to wchar string conversion
 */

	
#include <stdlib.h>
#include <libc_thread.h>
#include "mbctype.h"

#ifndef MB_SUPPORT
size_t	mbstowcs (wchar_t *pwcs, const char *s, size_t n)
{
  int count = 0;
  
  if (n != 0) {
    do {
      if ((*pwcs++ = (wchar_t) *s++) == 0)
	break;
      count++;
    } while (--n != 0);
  }
  
  return count;
}
#else
size_t	mbstowcs (wchar_t *pwcs, const char *s, size_t n)
{
    static _mbstate_t mbstate;
    _THREAD_PRIVATE_KEY(mbstowcs);
    _mbstate_t *state = _THREAD_PRIVATE(mbstowcs, mbstate, NULL);
    return _mbstowcs_r (pwcs, s, n, state);
}

size_t	_mbstowcs_r (wchar_t *pwcs, const char *s, size_t n, _mbstate_t *state)
{
  wchar_t *ptr = pwcs;
  size_t max = n;
  char *t = (char *)s;
  int bytes;

  while (n > 0)
    {
      bytes = _mbtowc_r (ptr, t, MB_CUR_MAX, state);
      if (bytes < 0)
	{
	  state->__count = 0;
	  return -1;
	}
      else if (bytes == 0)
        return ptr - pwcs;
      t += bytes;
      ++ptr;
      --n;
    }

  return max;
}
#endif
