#ident "lib/c/std/modf.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * ./modf.c : stdlib function
 */


#include <stdlib.h>
#include <values.h>

#if _IEEE == 1

#if defined (__NO_FLOAT)

double
modf (double d, double *ipart)
{
    abort ();
}

#elif defined (__SOFT_FLOAT) || defined(__SINGLE_FLOAT)

double
modf (double d, double *ipart)
{
    extern double __modf (double, double *);
    return __modf (d, ipart);
}

#else

#include <math.h>

#ifdef __GNUC__
#define fabs(x) __builtin_fabs((x))
#endif

double modf(double d, double *ipart)
{
    double absval;
    
    absval = fabs(d);

    if (absval >= DMAXPOWTWO) {
	*ipart = d;
	return 0.0;
    } else {
	double ival = absval;
	
	ival += DMAXPOWTWO;
	ival -= DMAXPOWTWO;
	
	while (ival > absval)
	  ival -= 1.0;
	if (d < 0)
	  ival = -ival;
	*ipart = ival;
	return d - ival;
    }
}

#endif /* __SOFT_FLOAT */

#endif /* _IEEE == 1 */
