#ident "lib/c/std/setenv.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * ./setenv.c : stdlib function
 */

/*
 * Copyright (c) 1987 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#include <string.h>
#include <stdlib.h>

extern char **environ;
static int alloced;			/* if allocated space before */

extern char *_findenv(const char *,int *);
/*
 * setenv --
 *	Set the value of the environmental variable "name" to be
 *	"value".  If rewrite is set, replace any current value.
 */
int setenv(const char *name, const char *value, int rewrite)
{
    char *entry;
    int l_value;
    int l_name;
    int offset;
	
    if (*value == '=')		/* no `=' in value */
      ++value;
	
    l_value = strlen(value);
    
    if ((entry = _findenv(name, &offset))) { /* find if already exists */
	if (!rewrite)
	  return(0);
	if (strlen(entry) >= l_value) { /* old larger; copy over */
	    strcpy(entry,value);
	    return(0);
	}
    } else {			/* create new slot */
	int	cnt;
	char	**envp;

	/* count entries in environ */
	for (envp = environ, cnt = 0; *envp; ++envp, ++cnt);
	
	if (alloced) {		/* just increase size */
	    envp = (char **)realloc((char *)environ,
				       (sizeof(char *) * (cnt + 2)));
	    if (!envp)
	      return(-1);
	} else {			/* get new space */
	    alloced = 1;	/* copy old entries into it */
	    envp = (char **)malloc((sizeof(char *) * (cnt + 2)));
	    if (!envp)
	      return(-1);
	    memcpy(envp, environ, cnt * sizeof(char *));
	}
	environ = envp;
	environ[cnt + 1] = NULL;
	offset = cnt;
    }
    
    /* no `=' in name */
    {
	const char *s;
	
	for(s=name; *s && *s != '='; s++)
	  continue;
	l_name = s - name;
    }
	
    
    /* name + `=' + value */
    if (!(entry = malloc(l_name + l_value + 2)))
      return(-1);
    
    environ[offset] = entry;
    
    strncpy(entry,name,l_name);
    entry += l_name;
    *entry++ = '=';
    strcpy(entry,value);

    return(0);
}

/*
 * unsetenv(name) --
 *	Delete environmental variable "name".
 */
void unsetenv(const char *name)
{
    char	**envp;
    int	offset;

    while (_findenv(name, &offset)) /* if set multiple times */
      
      for (envp = &environ[offset]; envp[0] = envp[1]; ++envp)
	continue;
}
