#ident "lib/c/std/strtoll.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * ./strtoll.c : stdlib function
 */

#include <ctype.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>

long long int strtoll(const char *nptr,char **endptr,int base)
{
    const char *s = nptr;
    int c;
    long long result = 0LL;
    long long limit;
    int negative = 0;
    int overflow = 0;
    int some = 0;
    int digit;

    while ((c = *s) && isspace(c)) /* skip leading white space */
      s++;
    if ((c = *s) == '+' || c == '-') { /* handle signs */
	negative = (c == '-');
	s++;
    }
    if (base == 0) {		/* determine base if unknown */
	base = 10;
	if (*s == '0') {
	    base = 8;
	    s++;
	    some = 1;	    	/* just "0" on its own counts as valid input */
	    if ((c = *s) == 'x' || c == 'X') {
		base = 16;
		s++;
	    }
	}
    }
    else
      if (base == 16 && *s == '0') {	/* discard 0x/0X prefix if hex */
	  s++;
	  some = 1;	    		/* just "0" on its own counts as valid input */
	  if (((c = *s) == 'x') || c == 'X')
	    s++;
      }

    limit = LONG_LONG_MAX / base;	/* ensure no overflow */

    s--;			/* convert the number */
    while ((c = *++s)) {
	if (isdigit(c))
	  digit = c - '0';
	else if(isalpha(c))
	  digit = c - (isupper(c) ? 'A' : 'a') + 10;
	else
	  break;
	if (digit < 0 || digit >= base)
	  break;
	some = 1;
	if (result > limit)
	  overflow = 1;
	if (!overflow) {
	    result = base * result;
	    if (digit > LONG_LONG_MAX - result)
	      overflow = 1;
	    else	
	      result += digit;
	}
    }
    if (negative && !overflow)
      result = 0LL - result;
    if (overflow) {
	errno = ERANGE;
	if (negative)
	  result = LONG_LONG_MIN;
	else
	  result = LONG_LONG_MAX;
    }

    if (endptr != NULL)		/* set up return values */
      *endptr = (char *) (some ? s : nptr);
    return result;
}
