#ident "lib/c/std/wctomb.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * wcstombs.c: multi-byte / wchar string conversion
 */

	
#include <stdlib.h>
#include <libc_thread.h>
#include "mbctype.h"

#ifndef MB_SUPPORT
size_t wcstombs (char *s, const wchar_t *pwcs, size_t n)
{
  int count = 0;
  
  if (n != 0) {
    do {
      if ((*s++ = (char) *pwcs++) == 0)
	break;
      count++;
    } while (--n != 0);
  }
  
  return count;
}
#else
size_t wcstombs (char *s, const wchar_t *pwcs, size_t n)
{
    static _mbstate_t mbstate;
    _THREAD_PRIVATE_KEY(wcstombs);
    _mbstate_t *state = _THREAD_PRIVATE(wcstombs, mbstate, NULL);
    return _wcstombs_r (s, pwcs, n, state);
}

size_t _wcstombs_r (char *s, const wchar_t *pwcs, size_t n, _mbstate_t *state)
{
  char *ptr = s;
  size_t max = n;
  char buff[8];
  int i, num_to_copy;

  while (n > 0)
    {
      int bytes = _wctomb_r (buff, *pwcs, state);
      if (bytes == -1)
        return -1;
      num_to_copy = (n > bytes ? bytes : (int)n);
      for (i = 0; i < num_to_copy; ++i)
        *ptr++ = buff[i];
          
      if (*pwcs == 0x00)
        return ptr - s - (n >= bytes);
      ++pwcs;
      n -= num_to_copy;
    }

  return max;
} 
#endif /* MB_SUPPORT */
