#ident "lib/c/stdio/_bufproc.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * _bufproc.c : standard I/O library
 */

/* --- _bufproc.c --- */
/* Manages buffers for stdio */

#include <stdlib.h>
#include <stdio.h>
#include "stdiolib.h"

/* _bufalloc - allocate a new buffer				*/
/* Input:  A valid stream with _io_bufsiz appropriately set	*/
/* Output: Maybe a buffer, maybe not				*/

void _bufalloc(FILE *stream)
{
    if (stream->_io_bufsiz > 0 &&
	(stream->_io_buf = malloc(stream->_io_bufsiz)) != NULL) {
	_io_setflag(stream, _IO_MYBUF);
	stream->_io._io_ptr = stream->_io_buf;
    } else {
	stream->_io_bufsiz = 0;
	stream->_io._io_char = EOF;
    }
}

/* _buffree - free an existing buffer if allocated by _bufalloc	*/
/* Input:  A valid stream					*/
/* Output: A valid unbuffered stream				*/

void _buffree(FILE *stream)
{
    if (stream->_io_buf != NULL && _io_testflag(stream, _IO_MYBUF)) {
	free(stream->_io_buf);
	stream->_io_buf = NULL;
	stream->_io_count = stream->_io_bufsiz = 0;
	stream->_io._io_char = EOF;
	_io_clearflag(stream, (_IO_LBUF | _IO_MYBUF));
    }
}
