#ident "lib/c/stdio/_doscan.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * _doscan.c : standard I/O library
 */


/*
 * Formatted input workhorse 
 * Desperately needs tidying
 */

#include <assert.h>
#include <ctype.h>
#include <limits.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "stdiolib.h"

/* --- Constants --- */
#define OK			0	/* return values from input eaters */
#define INPUT_FAILURE		EOF
#define MATCHING_FAILURE	1 
#define UNIMPLEMENTED		2

#define POINTER_BASE		16	/* constants for %p conversion */

#define NO_NULL			0	/* constants for _strtostr() */
#define ADD_NULL		1
#define NO_COMPARE		0
#define WHITE_COMPARE		1
#define SET_COMPARE		2
#define CHAR_SET_SIZE		256

#define MAX_CONVERSION 	128
/*#define MAX_FCONVERSION 512*/

#define SIZET_MAX	((size_t) LONG_MAX)	/* probably */

/* --- input state --- */
struct doscan_state {
  FILE *f;		/* pointer to input file */
  int input_char;	/* buffer for next input character */
  const char *fp;	/* format pointer */
  va_list ap;		/* argument pointer */
  int chars_read;	/* number of characters read from the stream (%n) */
  int items_converted;	/* successful conversions */
  int items_assigned;	/* successful assignments */
};


/* --- Macros --- */
#define BACKUP_INPUT(c)	((gs->input_char = (c)), (gs->chars_read--))

/* --- Input functions --- */
static int gsgetc();
#define GSGETC(gs) 	gsgetc(gs)
#define GSGETCMAX(gs,c,max) \
  ((max > 0 && (c=GSGETC(gs)) != EOF) ? \
   (max--,c): \
   (c=EOF))

/* --- Input processors --- */
static int _eat_white(struct doscan_state *gs);
static int _parse_format(struct doscan_state *gs);


static int _eat_one(int,struct doscan_state *gs);
static int _strtostr(int method,int string_end,int flags,size_t,struct doscan_state *gs);

/* convert integer formats */
static int _strtol(int,int,size_t,struct doscan_state *gs);

#ifndef __NO_FLOAT
/* convert floating formats */
static int _strtod(int,size_t,struct doscan_state *gs);	
#endif

#define _IO_ERRVAL EOF

int vfscanf (FILE *stream, const char *format, va_list arg)
{
    register int c;
    register int error = OK;
    struct doscan_state	as;
    struct doscan_state	*gs = &as;
    
    _arg_assert(format);

    flockfile (stream);

    _io_assert(stream, _IO_WRITE);

    gs->f = stream;
    gs->input_char = EOF;
    gs->fp = format;
    gs->ap = arg;
    gs->items_converted = 0;
    gs->items_assigned = 0;
    gs->chars_read = 0;

    while ((c = (int) *gs->fp++) && (error == OK)) {
	if( c == '%') {
	    error = _parse_format(gs);
	} else if (isspace(c)) {
	    error = _eat_white(gs);
	} else {
	    error = _eat_one(c,gs);
	}
    }
    if (gs->input_char != EOF && !(stream->_io_flags & _IO_STRING))
      ungetc(gs->input_char, gs->f);
    funlockfile (stream);
    return (error == INPUT_FAILURE && gs->items_converted == 0)
      ? EOF : gs->items_assigned;
}

static int gsgetc(struct doscan_state *gs)
{
  int rc;

  if (gs->input_char != EOF) {
      rc = gs->input_char;
      gs->input_char = EOF;
  }
  else {
      rc = getc_unlocked(gs->f);
  }
  if (rc != EOF)
      gs->chars_read++;
  return rc;
}

#define TESTFLAG(x)	(flags & (x))
#define CLEARFLAG(x)	(flags &= ~(x))
#define SETFLAG(x)	(flags |= (x))

#define NOASG		0x0001
#define CHAR		0x0080	/* length modifiers */
#define SHORT		0x0100
#define LONG		0x0200
#define LONGDOUBLE	0x0400
#define LONGLONG	0x0800
#define SIGNED		0x1000	/* signed integer argument */

static int _parse_format(struct doscan_state *gs)
{
    register int c;
    int error = OK;
    int base = -1;
    int flags = 0;
    size_t maxwidth = SIZET_MAX;
    
    if ((c = *gs->fp++) == '%')
      return _eat_one(c,gs);

    if (c == '*') {
	SETFLAG(NOASG);
	c = *gs->fp++;
    } 
    
    if (isdigit(c)) {
	gs->fp--;
	maxwidth = strtoul(gs->fp,(char **)&gs->fp,10);
	c = *gs->fp++;
    }
  
    switch(c) {
    case 'h':
	c = *gs->fp++;
	if (c == 'h') {
	    c = *gs->fp++;
	    SETFLAG(CHAR);
	} else
	  SETFLAG(SHORT);
	break;
    case 'l':
	c = *gs->fp++;
	if (c == 'l') {
	    c = *gs->fp++;
	    SETFLAG(LONGLONG);
	} else
	  SETFLAG(LONG);
	break;
    case 'L':
	SETFLAG(LONGDOUBLE);
	c = *gs->fp++;
	break;
    case 'q':
	/* BSD unix quad format */
	SETFLAG(LONGLONG);
	c = *gs->fp++;
	break;
    }
    
    switch (c) {
    case 'i':
	SETFLAG(SIGNED);
	base = 0;
	goto donum;
	
    case 'D':			/* not ANSI */
	SETFLAG(LONG);
    case 'd':
	SETFLAG(SIGNED);
	base = 10;
	goto donum;
	
    case 'O':			/* not ANSI */
	SETFLAG(LONG);
    case 'o':
	base = 8;
	goto donum;
	
    case 'U':			/* not ANSI */
	SETFLAG(LONG);
    case 'u':
	base = 10;
	goto donum;
	
    case 'X':			
    case 'x':
	base = 16;
	goto donum;
	
    case 'p':
	base = POINTER_BASE;
	goto donum;
	
    donum:
	if( (error = _eat_white(gs)) == 0)
	  error = _strtol(base, flags, maxwidth,gs);
	break;
    case 'F':
    case 'E':
    case 'G':
    case 'f':
    case 'e':
    case 'g':
#ifdef __NO_FLOAT
	error = UNIMPLEMENTED;
#else
	if( (error = _eat_white(gs)) == 0)
	  error = _strtod(flags,maxwidth,gs);
#endif
	break;
    case 's':
        if( (error = _eat_white(gs)) == 0)
	  error = _strtostr(WHITE_COMPARE, ADD_NULL,flags,maxwidth,gs);
	break;
    case '[':
	error = _strtostr(SET_COMPARE, ADD_NULL,flags,maxwidth,gs);
	break;
    case 'c':
	if (maxwidth == SIZET_MAX)
	  maxwidth = 1;
	error = _strtostr(NO_COMPARE, NO_NULL,flags, maxwidth,gs);
	break;
    case 'n':
	if (!TESTFLAG(NOASG)) {
	    int result = gs->chars_read;
	    if (TESTFLAG(LONGLONG))
	      *va_arg(gs->ap, long long *)  = (long long) result;
	    else if (TESTFLAG(LONG))
	      *va_arg(gs->ap, long *)  = (long) result;
	    else if (TESTFLAG(SHORT))
	      *va_arg(gs->ap, short *) = (short) result;
	    else if (TESTFLAG(CHAR))
	      *va_arg(gs->ap, char *) = (char) result;
	    else
	      *va_arg(gs->ap, int *)   = (int) result;
	}
	error = OK;
	break;
    default:
	error = MATCHING_FAILURE;
	break;
    }
    return error;
}

/* --- Input processors --- */
static int _eat_white(struct doscan_state *gs)
{
  register int c;

  while ((c = GSGETC(gs)) != EOF && isspace(c)) ;
  BACKUP_INPUT(c);
  return (c == EOF) ? INPUT_FAILURE : OK;
}

static int _eat_one(int c,struct doscan_state *gs)
{
    register int i;

    if ((i = GSGETC(gs)) == EOF)
      return INPUT_FAILURE;
    else if(i == c)
      return OK;
    else {
	BACKUP_INPUT(i);
	return MATCHING_FAILURE;
    } 
}

/* scan a syntatic number and convert with strtol/strtoul 
 */
static int _strtol(int base,int flags,size_t maxwidth,struct doscan_state *gs)
{
    register int c;
    int original_width = maxwidth;
    char	buf[MAX_CONVERSION];
    char 	*bp = buf;
    const char 	*cset;
    long result;
    long long llresult;
    
    /* --- Handle signs --- */
    GSGETCMAX(gs,c,maxwidth);
    if (c == '+' || c == '-') {
	*bp++ = c;
	GSGETCMAX(gs,c,maxwidth);
    }

    if (base == 0) {
	/* --- Accept leading base specifier
	 */
	base = 10;
	if (c == '0') {
	    *bp++ = c;
	    base = 8;
	    GSGETCMAX(gs,c,maxwidth);
	    if (c == 'x' || c == 'X') {
		base = 16;
		GSGETCMAX(gs,c,maxwidth);
	    }
	}
    }
    else if (base == 16 && c == '0') {	
	/* discard 0x/0X prefix if hex */
	*bp++ = c;
	GSGETCMAX(gs,c,maxwidth);
	if (c == 'x' || c == 'X') {
	    GSGETCMAX(gs,c,maxwidth);
	}
    }
    switch(base) {
    case 8:
	cset = "01234567";
	break;
    case 16:
	cset = "0123456789ABCDEFabcdef";
	break;
    case 10:
    default:	/* cant happen */
	cset = "0123456789";
	break;
    }
    
    while(c != EOF && strchr(cset,c) != NULL) {
	*bp++ = c;
	GSGETCMAX(gs,c,maxwidth);
    }
    *bp = '\0';
    if(c != EOF) {
	maxwidth++;
	BACKUP_INPUT(c);
    }
    
    if (TESTFLAG(LONGLONG)) {
	if(TESTFLAG(SIGNED))
	  llresult = strtoll(buf,&bp,base);
	else
	  llresult = strtoull(buf,&bp,base);
    } else {
	if(TESTFLAG(SIGNED))
	  result = strtol(buf,&bp,base);
	else
	  result = strtoul(buf,&bp,base);
    }
    assert(*bp == '\0');
  
    gs->items_converted++;

    /* --- Stash the result --- */
    if (maxwidth == original_width)
      return (c == EOF) ? INPUT_FAILURE : MATCHING_FAILURE;
    
    if (!TESTFLAG(NOASG)) {
	if (TESTFLAG(SIGNED)) {
	    if (TESTFLAG(LONGLONG))
	      *va_arg(gs->ap, long long *)  = (long long)  llresult;
	    else if (TESTFLAG(LONG))
	      *va_arg(gs->ap, long *)  = (long)  result;
	    else if (TESTFLAG(SHORT))
	      *va_arg(gs->ap, short *) = (short) result;
	    else if (TESTFLAG(CHAR))
	      *va_arg(gs->ap, char *) = (char) result;
	    else
	      *va_arg(gs->ap, int *)   = (int)   result;
	} else {
	    if (TESTFLAG(LONGLONG))
	      *va_arg(gs->ap, unsigned long long *)  = (unsigned long long) llresult;
	    else if (TESTFLAG(LONG))
	      *va_arg(gs->ap, unsigned long *)  = (unsigned long)  result;
	    else if( TESTFLAG(SHORT) )
	      *va_arg(gs->ap, unsigned short *) = (unsigned short) result;
	    else if( TESTFLAG(CHAR) )
	      *va_arg(gs->ap, unsigned char *) = (unsigned char) result;
	    else
	      *va_arg(gs->ap, unsigned int *)   = (unsigned int)   result;
	}
	gs->items_assigned++;
    }
    return OK;
}

#ifndef __NO_FLOAT
/* read the syntactic number 
*/

static int _strtod(int flags,size_t maxwidth,struct doscan_state *gs)
{
    char	ibuf[MAX_CONVERSION];
    char	*pbuf = ibuf;
    char	*xbuf = NULL;
    int		bufsiz = MAX_CONVERSION;
    int		bufcnt = 0;
    int		res = OK;
    int		c;
    int		sign;

    int putget (int c)
    {
	if (bufcnt == bufsiz) {
	    bufsiz += MAX_CONVERSION;
	    if (xbuf)
		xbuf = realloc (xbuf, bufsiz);
	    else {
		xbuf = malloc (bufsiz);
		memcpy (xbuf, pbuf, MAX_CONVERSION);
	    }
	    pbuf = xbuf;
	}
	pbuf[bufcnt++] = c;
	if (c)
	    GSGETCMAX(gs,c,maxwidth);
	return c;
    }

    GSGETCMAX(gs,c,maxwidth);

    /* --- Handle signs --- */
    if (sign = (c == '+' || c == '-')) {
	c = putget (c);
    }
    
    while(isdigit(c)) {
	c = putget (c);
    }
    if( c == '.') {		/* have fractional part */
	c = putget (c);

	while( isdigit(c) ) {
	    c = putget (c);
	}
    }
    if(bufcnt - sign > 0
       && (c == 'e' || c == 'E')) {	/* have exponent */
	c = putget (c);

	if( c == '-' || c == '+') {
	    c = putget (c);
	}
	
	while(isdigit(c)) {
	    c = putget (c);
	}
    }
    (void) putget ('\0');
    
    if (c != EOF) {
	maxwidth++;
	BACKUP_INPUT(c);		/* term char */
    }
    
    if (bufcnt - sign > 0) {
	/* convert using strtof/strtod, and plant it */
	gs->items_converted++;
	if (!TESTFLAG(NOASG)) {
	    if( TESTFLAG(LONGDOUBLE))
		*va_arg(gs->ap, long double *) =
		    (long double)strtod(pbuf, NULL);
	    else if( TESTFLAG(LONG))
		*va_arg(gs->ap, double *) = strtod(pbuf, NULL);
	    else
		*va_arg(gs->ap, float *) = strtof(pbuf, NULL);
	    gs->items_assigned++;
	}
    }
    else {
	/* no float found */
	res = MATCHING_FAILURE;
    }

    if (xbuf)
	free (xbuf);
    return res;
}
#endif


static int _strtostr(int method,int string_end,int flags,size_t maxwidth,
		     struct doscan_state *gs)
{
  int negated;
  register int c;
  register unsigned char *target;
  int original_width = maxwidth;
  char valid_char[CHAR_SET_SIZE];

/* --- Set up for comparisions --- */
  if (method == SET_COMPARE) {
	negated = 0;
	if (*gs->fp == '^') {
		negated = 1;
		gs->fp++;
	}
	memset(valid_char, negated, CHAR_SET_SIZE);
	negated = !negated;
	c = EOF;
	do {
		register int next = gs->fp[1];

		if (c != EOF && *gs->fp == '-' && next != ']') {
			for (c++ ; c <= next; c++)
				valid_char[c] = negated;
			gs->fp++;
			c = EOF;
		}
		else
			valid_char[c = *gs->fp] = negated;
	} while (*++gs->fp != ']');
	gs->fp++;
  }

/* --- Copy the string --- */
  if (!TESTFLAG(NOASG))
    target = va_arg(gs->ap, unsigned char *);
  
  while( GSGETCMAX(gs,c,maxwidth) != EOF) {
      if (method == WHITE_COMPARE && isspace(c))
	break;
      else if (method == SET_COMPARE && !valid_char[c])
	break;
      
      if (!TESTFLAG(NOASG))
	*target++ = c;
  }
  if(c != EOF) {
      maxwidth++;
      BACKUP_INPUT(c);
  }
  gs->items_converted++;

/* --- Finish up --- */
  if (!TESTFLAG(NOASG)) {
      if (string_end == ADD_NULL)
	*target = '\0';
      gs->items_assigned++;
  }
  if (maxwidth == original_width)
    return (c == EOF) ? INPUT_FAILURE : MATCHING_FAILURE;
  return OK;
}
