#ident "lib/c/stdio/_fflush.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * _fflush.c : standard I/O library
 */

#include <sys/types.h>
#include <stdio.h>
#include <unistd.h>
#include "stdiolib.h"

int _fflush(FILE *stream)
{
    int rc = 0;

    if (stream->_io_count == _IO_NEVER)
	return rc;

    if (stream->_io_bufsiz != 0) {
	if (_io_testflag(stream, _IO_WRITE)) {
	    /* flush write stream */
	    rc = _flspbuf(stream);
	}
	else if (_io_testflag(stream, _IO_READ)) {
	    int count = stream->_io_count;
	    /* reset pointers now in case of signal/longjmp */
	    stream->_io_count = 0;
	    stream->_io._io_ptr = stream->_io_buf;
	    /* backup over buffered but unread data */
	    if (count > 0
		&& lseek(stream->_io_fd, 0 - count, SEEK_CUR) < 0)
		rc = EOF;
	}
    }
    else
	/* flush ungetc buffer */
	stream->_io._io_char = EOF;

    if (_io_testflag(stream, _IO_UPDATE)) {
	/* ready to change direction on next i/o */
	_io_clearflag(stream, (_IO_READ | _IO_WRITE));
	stream->_io_count = _IO_NEVER;
    }

    return rc;
}
