#ident "lib/c/stdio/_filbuf.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * _filbuf.c : standard I/O library
 */

/* --- _filbuf.c --- */
/* Read from the file associated with a stream */
/* Notes:  Will flush stdout if reading stdin and both are tty's	*/

#include <sys/types.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include "stdiolib.h"

int _filbuf(FILE *stream)
{
    int c;

    /* --- Virgin stream: set things up --- */
    /* note _io_count has just been decremented */
    if (stream->_io_count == _IO_NEVER - 1)
      _i_init(stream);

    /* --- Not SysV compatible --- */
    if (_io_testflag(stream, _IO_EOF))
	return EOF;

    /* --- Read a bufferful --- */
    do {
	/* flush all line-buffered output before reading from terminal */
	if (_io_testflag(stream, _IO_LBUF))
	    _fwalk (_flslbuf);
	if (stream->_io_bufsiz == 0) { /* unbuffered */
	    char c1;
	    
	    if (stream->_io._io_char != EOF) { /* check for ungetc()ed char */
		c = stream->_io._io_char;
		stream->_io._io_char = EOF;
		return c;
	    }
	    stream->_io_count = read(stream->_io_fd, &c1, 1);
	    c = c1;
	} else {
	    stream->_io_count =
		read(stream->_io_fd, stream->_io_buf, stream->_io_bufsiz);
	    stream->_io._io_ptr = stream->_io_buf;
	}
	/* an interrupt is not an error */
    } while (stream->_io_count < 0 && errno == EINTR);

    if (stream->_io_count < 0) {
	_io_setflag(stream, _IO_ERR);
	return EOF;
    }
    if (stream->_io_count == 0) {
	_io_setflag(stream, _IO_EOF);
	return EOF;
    }
    stream->_io_count--;
    return (stream->_io_bufsiz == 0)
      ? c
	: *stream->_io._io_ptr++;
}
