#ident "lib/c/stdio/_flsbuf.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * _flsbuf.c : standard I/O library
 */

/* --- _flsbuf.c --- */
/* Flushes full stream buffers */

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include "stdiolib.h"

/* stream must be valid, writable */
int _flsbuf(int c,FILE *stream)
{
    int ret;

    /* --- Can't flush string */
    if (_io_testflag(stream, _IO_STRING))
	return c;
	
    /* --- Virgin stream: set things up --- */
    /* note _io_count has just been decremented */
    if (++stream->_io_count == _IO_NEVER)
      _o_init(stream);

    /* --- Unbuffered stream: write one character --- */
    if (stream->_io_bufsiz == 0) {
	char	c1 = c;
	stream->_io_count = 0;
	do {
	    ret = write (stream->_io_fd, &c1, 1);
	} while (ret < 0 && errno == EINTR);
	if (ret != 1) {
	    _io_setflag(stream, _IO_ERR);
	    return EOF;
	}
	return c;
    }

    /* --- Buffered stream: write out a full buffer --- */
    if (stream->_io_count == 0) {
	char *bp = stream->_io_buf;
	int bytes = stream->_io_bufsiz;

	/* set these now to avoid screwing up on longjmp */
	stream->_io._io_ptr = stream->_io_buf;
	stream->_io_count = stream->_io_bufsiz;

	do {
	    ret = write(stream->_io_fd, bp, bytes);
	    if (ret < 0 && errno != EINTR)
		break;
	    if (ret > 0) {
		bp += ret;
		bytes -= ret;
	    }
	} while (bytes != 0);

	if (bytes != 0) {
	    _io_setflag(stream, _IO_ERR);
	    return EOF;
	}
    }

    /* --- Buffered stream: start filling new buffer --- */
    --stream->_io_count;
    *stream->_io._io_ptr++ = c;

    /* --- Line buffered stream: flush a partial buffer --- */
    if (_io_testflag(stream, _IO_LBUF) && c == '\n' && _flspbuf(stream) != 0)
      return EOF;

    return c;
}



