#ident "lib/c/stdio/_iobstralloc.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * _iobstralloc.c: io functions for asprintf
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "stdiolib.h"

static int 
_filbuf_stralloc (struct _iob *stream __attribute__((__unused__)))
{
    return -1;
}

static int 
_flsbuf_stralloc (int c, struct _iob *stream)
{
    size_t pos = stream->_io._io_ptr - stream->_io_buf;
    unsigned char *buf;
    int size;
    
    /* Allocate space exponentially. */
    size = (stream->_io_bufsiz << 1) + 1;
    buf = realloc (stream->_io_buf, size + 1);
    if (buf == NULL) {
	_io_setflag (stream, _IO_ERR);
	return EOF;
    }
    stream->_io_count += size - stream->_io_bufsiz;
    stream->_io_bufsiz = size;
    stream->_io_buf = buf;
    stream->_io._io_ptr = buf + pos;
    *stream->_io._io_ptr++ = c;
    return c;
}

static int
_fflush_stralloc (struct _iob *stream __attribute__((__unused__)))
{
    /* shouldn't happen */
    return 0;
}

static size_t 
_fwrite_stralloc (const void *ptr __attribute__((__unused__)),
		  size_t size __attribute__((__unused__)), 
		  size_t nmemb  __attribute__((__unused__)),
		  struct _iob *stream __attribute__((__unused__)))
{
    /* shouldn't happen */
    return 0;
}
  
static int 
_fsync_stralloc (struct _iob *stream __attribute__((__unused__)), 
		 int mbz __attribute__((__unused__)))
{
    return 1;
}

const struct _iobfuncs _io_strallocfuncs = {
    .filbuf = _filbuf_stralloc,
    .flsbuf = _flsbuf_stralloc,
    .fflush = _fflush_stralloc,
    .fwrite = _fwrite_stralloc,
    .fsync = _fsync_stralloc,
};
