#ident "lib/c/stdio/_ioinit.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * _ioinit.c : standard I/O library
 */

/* --- _ioinit.c --- */
/* Checks for a whole bunch of initial conditions on i/o */
/* Notes:  Includes _i_init() and _o_init() */

#include <sys/types.h>
#include <stdio.h>
#include <unistd.h>
#include <sdethread.h>
#include "stdiolib.h"

void _i_init(FILE *stream)
{
/* --- Allocate a buffer if necessary --- */
  if (stream->_io_buf == NULL)
	_bufalloc(stream);
  if (stream->_io_bufsiz == 0)
      stream->_io._io_char = EOF;
  else
      stream->_io._io_ptr = stream->_io_buf;
  stream->_io_count = 0;
  _io_setflag(stream, _IO_READ);

/* --- Line buffering: check once and turn it off if stream is not a tty --- */
  if (_io_testflag(stream, _IO_TTY)) {
	if (_io_testflag(stream, _IO_LBUF) && !isatty(stream->_io_fd))
		_io_clearflag(stream, _IO_LBUF);
	_io_clearflag(stream, _IO_TTY);
  }
}

void _o_init(FILE *stream)
{
/* --- Register cleanup function so that exit() can find it --- */
  _libc_data->_cleanup = _cleanup;

/* --- Allocate a buffer if necessary --- */
  if (stream->_io_buf == NULL)
	_bufalloc(stream);
  if (stream->_io_bufsiz != 0)
      stream->_io._io_ptr = stream->_io_buf;
  stream->_io_count = stream->_io_bufsiz;
  _io_setflag(stream, _IO_WRITE);

/* --- Line buffering: check once and turn it off if stream is not a tty --- */
  if (_io_testflag(stream, _IO_TTY)) {
	if (_io_testflag(stream, _IO_LBUF) && !isatty(stream->_io_fd))
		_io_clearflag(stream, _IO_LBUF);
	_io_clearflag(stream, _IO_TTY);
  }
}
