#ident "lib/c/stdio/_libcproc.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2001-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * _libcproc.c : C library per-process data
 */


/* --- _libcprot.c --- */

#include <stdio.h>
#include <stdlib.h>
#include "stdiolib.h"
#include "libc_private.h"

static const FILE _stdin = STDIN_INIT;
static const FILE _stdout = STDOUT_INIT;
static const FILE _stderr = STDERR_INIT;


struct _libc_data *	
_libc_new ()
{
    struct _libc_data *	libc;
    struct _atexit *atex;
    struct _iov *iov;

    libc = calloc (1, sizeof (struct _libc_data));
    if (!libc)
	return NULL;

    iov = calloc (1, sizeof (struct _iov));
    if (!iov) {
	free (libc);
	return NULL;
    }

    atex = calloc (1, sizeof (struct _atexit));
    if (!atex) {
	free (iov);
	free (libc);
	return NULL;
    }

    memcpy (&iov->iob[0], &_stdin, sizeof (FILE));
    memcpy (&iov->iob[1], &_stdout, sizeof (FILE));
    memcpy (&iov->iob[2], &_stderr, sizeof (FILE));
    iov->iob[3]._io_count = _IO_FREE;

    libc->_stdin = &iov->iob[0];
    libc->_stdout = &iov->iob[1];
    libc->_stderr = &iov->iob[2];
    libc->_iov = iov;
    libc->_atexit = atex;
    libc->_opterr = libc->_optreset = 1;

    return libc;
}


void
_libc_delete (struct _libc_data *libc)
{
    struct _atexit *ex, *nex;
    struct _iov *iov, *niov;
    FILE *stream;

    /* free atexit list */
    for (ex = libc->_atexit; ex; ex = nex) {
	nex = ex->next;
	free (ex);
    }

    /* free stream tables */
    for (iov = libc->_iov; iov; iov = niov) {
	/* free stream buffers */
	for (stream = iov->iob; stream < &iov->iob[IOV_CHUNK]; stream++)
	    if (_io_valid(stream))
		_buffree(stream);
	niov = iov->next;
	free (iov);
    }

    /* free the whole structure */
    free (libc);
}

