#ident "lib/c/stdio/_valmode.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * _valmode.c : standard I/O library
 */

/* --- _valmode.c --- */
/* Validates mode for stream opening functions	*/

/* Input:  mode - a mode string for fopen()		*/
/* Output: EOF for a bad string				*/
/*	   0 for a good string without a '+'		*/
/*	   _IO_UPDATE for a good string with a '+'	*/

#include <stdio.h>
#include "stdiolib.h"

int _valmode(const char *mode)
{
  int rc;

/* --- Check basic mode --- */
  switch (*mode) {
  case 'r':
      rc = _IO_READ;
      break;
  case 'a':
  case 'w':
      rc = _IO_WRITE;
      break;
  default:
      goto err;
  }

  if (*++mode == '\0')
	return rc;

/* --- Check second character of mode string --- */
  switch (*mode) {
  case '+':
      rc |= _IO_UPDATE;
      break;
  case 'b':
      rc |= _IO_BINARY;
      break;
  default:
      goto err;
  }

  if (*++mode == '\0')
	return rc;

/* --- Check third character of mode string --- */
  switch (*mode) {
  case '+':
      if (rc & _IO_UPDATE)
	  goto err;
      rc |= _IO_UPDATE;
      break;
  case 'b':
      if (!rc & _IO_BINARY)
	  goto err;
      rc |= _IO_BINARY;
      break;
  default:
      goto err;
  }

  if (*++mode == '\0')
	return rc;

/* --- Mode string too long --- */
 err:
  errno = EINVAL;
  return EOF;
}
