#ident "lib/c/stdio/asprintf.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2003-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * asprintf.c : standard I/O library
 */

#include <limits.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include "stdiolib.h"
#include <errno.h>

  
int __vasprintf (char **str, const char *format, va_list arg)
{
    FILE fake;
    unsigned char *buf;
    int n;

    fake._io_fd = -1;
    fake._io_flags = _IO_WRITE | _IO_STRING | _IO_STRALLOC;
    fake._io._io_ptr = fake._io_buf = (unsigned char *) malloc (128);
    fake._io_bufsiz = fake._io_count = 127; /* room for nul */
    fake._io_funcs = &_io_strallocfuncs;
    fake._io_lock = NULL;
    
    n = vfprintf (&fake, format, arg);
    if (n < 0)
	goto err;

    /* terminate string */
    *fake._io._io_ptr = '\0';

    /* shrink buffer to actual length */
    buf = realloc (fake._io_buf, n + 1);
    if (buf == NULL)
	goto err;

    /* return buffer and count */
    *str = (char *)buf;
    return n;

err:
    if (fake._io_buf)
	free(fake._io_buf);
    *str = NULL;
    errno = ENOMEM;
    return -1;
}


int vasprintf (char **str, const char *format, va_list arg)
{
    return __vasprintf (str, format, arg);
}


int asprintf (char **str, const char *format, ...)
{
    va_list arg;
    int n;

    va_start (arg, format);
    n = __vasprintf (str, format, arg);
    va_end (arg);
    return n;
}

