#ident "lib/c/stdio/fdopen.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * fdopen.c : standard I/O library
 */

/* --- fdopen.c --- */
/* Associates a stream with an already open file */
/* Notes:  Open mode of file must agree with open mode specified	*/
/*	   POSIX function						*/
#define _POSIX_SOURCE
#include <sys/types.h>
#include <stdio.h>
#include <unistd.h>
#include "stdiolib.h"

#define _IO_ERRVAL NULL


FILE *(fdopen)(int fd,const char *mode)
{
  int flags;
  register FILE *stream;

  if (fd < 0) {
      errno = EBADF;
      return _IO_ERRVAL;
  }
      
  if ((flags = _valmode(mode)) < 0)
	return _IO_ERRVAL;

  if ((stream = _ioballoc()) == NULL)
	return _IO_ERRVAL;

  if (isatty(fd))
	flags |= _IO_LBUF;

  if (flags & _IO_UPDATE)
      flags &= ~(_IO_READ | _IO_WRITE);

  stream->_io_fd = fd;
  stream->_io_flags = flags;
  stream->_io_count = _IO_NEVER;
  stream->_io_bufsiz = BUFSIZ;
  stream->_io_funcs = &_io_funcs;
  return stream;
}
