#ident "lib/c/stdio/fflush.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * fflush.c : standard I/O library
 */


/* Flushes a stream's buffer */

/* Notes:  Given a NULL pointer as a stream, fflush() flushes all streams. */
/*	   Flushing a readable stream discards the buffer contents	   */
/*		and lseek()'s the underlying file to meet expectations.    */

#include <sys/types.h>
#include <stdio.h>
#include <unistd.h>
#include "stdiolib.h"

#define _IO_ERRVAL EOF

/* flush line-buffered output streams */
int _flslbuf (FILE *stream)
{
    /* XXX don't block if another thread is locking this file */
    if (ftrylockfile(stream) >= 0) {
	if (_io_testflag(stream, _IO_LBUF) && _io_testflag(stream, _IO_WRITE))
	    stream->_io_funcs->fflush(stream);
	funlockfile (stream);
    }
    return 0;
}

/* flush output streams */
static int _flsobuf (FILE *stream)
{
    int rv = 0;
    /* XXX don't block if another thread is locking this file */
    if (ftrylockfile(stream) >= 0) {
	if (_io_testflag(stream, _IO_WRITE))
	    rv = stream->_io_funcs->fflush(stream);
	funlockfile (stream);
    }
    return rv;
}


int (fflush)(FILE *stream)
{
    int rv;
    if (stream == NULL) {
	rv = _fwalk (_flsobuf);
    } else {
	flockfile (stream);
	_io_assert(stream, 0);
	rv = stream->_io_funcs->fflush(stream);
	funlockfile (stream);
    }
    return rv;
}
