#ident "lib/c/stdio/fseek.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * fseek.c : standard I/O library
 */

/* --- fseek.c --- */
/* Sets the position of a stream */
/* Notes:  Always flushes a stream's buffer				*/
/*	   Includes fsetpos() and rewind()				*/
/*	   fsetpos() is a backstop for a macro in <stdio.h>		*/

#include <sys/types.h>
#include <stdio.h>
#include <unistd.h>
#include "stdiolib.h"

#define _IO_ERRVAL EOF

int fseek(FILE *stream,long offset,int whence)
{
    off_t new = offset;

    _arg_assert(whence == SEEK_SET || whence == SEEK_CUR || whence == SEEK_END);
    flockfile (stream);

    _io_assert(stream, 0);

    fflush(stream);

    _io_clearflag(stream, _IO_EOF);

    new = lseek(stream->_io_fd, new, whence);

    funlockfile (stream);
    return (new < (off_t)0) ? _IO_ERRVAL : 0;
}


int (fsetpos)(FILE *stream, const fpos_t *pos)
{
    off_t new;

    flockfile (stream);

    _io_assert(stream, 0);

    fflush(stream);

    _io_clearflag(stream, _IO_EOF);

    new = lseek (stream->_io_fd, *pos, SEEK_SET);

    funlockfile (stream);
    return (new < (off_t)0) ? _IO_ERRVAL : 0;
}


void (rewind)(FILE *stream)
{
    flockfile (stream);
    (void) fseek(stream, 0L, SEEK_SET);
    clearerr(stream);
    funlockfile (stream);
}
