#ident "lib/c/stdio/ftell.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * ftell.c : standard I/O library
 */

/* --- ftell.c --- */
/* Returns the position of a stream */
/* Notes:  Includes fgetpos(), which backstops a macro in <stdio.h>	*/

#include <errno.h>
#include <sys/types.h>
#include <stdio.h>
#include <unistd.h>
#include <limits.h>
#include "stdiolib.h"

#define _IO_ERRVAL -1

int (fgetpos) (FILE *stream, fpos_t *pos)
{
  extern int errno;
  int adjustment;
  off_t offs;
  int rv;

  flockfile (stream);

  _io_assert (stream, 0);

  if (stream->_io_count == _IO_NEVER)
      adjustment = 0;
  else if (_io_testflag(stream, _IO_READ))
      adjustment = -stream->_io_count;
  else if (_io_testflag(stream, _IO_WRITE))
      adjustment = stream->_io_bufsiz - stream->_io_count;
  else
      adjustment = 0;		/* ??? */

  offs = lseek(stream->_io_fd, (off_t)0, SEEK_CUR);

  funlockfile (stream);

  if (offs < (off_t)0)
      return EOF;

  offs += adjustment;
  if (offs < (off_t)0)
      return EOF;

  *pos = offs;
  return 0;
}


long (ftell)(FILE *stream)
{
    fpos_t pos;

    if (fgetpos (stream, &pos) < 0)
	return -1L;
    if (pos > LONG_MAX) {
	errno = ERANGE;
	return -1L;
    }
    return (long)pos;
}
