#ident "lib/c/stdio/setvbuf.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * setvbuf.c : standard I/O library
 */

/* --- setvbuf.c --- */
/* Associates a buffer with a stream and sets buffering modes */
/* Notes:  Does not actually allocate a buffer; just sets things up	*/
/*	   Includes setbuf(), which backstops a macro in <stdio.h>	*/

#include <stdio.h>
#include "stdiolib.h"

#define _IO_ERRVAL EOF


int (setvbuf)(FILE *stream,char *buffer,int type,size_t size)
{
  int rv;
  
    _arg_assert(type == _IONBF || size != 0);

    flockfile(stream);

    _io_assert(stream, 0);

    if (stream->_io_count != _IO_NEVER)
	goto err;

    fflush(stream);
    _buffree(stream);
    _io_clearflag(stream, (_IO_LBUF | _IO_MYBUF));
    
    switch (type) {
    case _IONBF:
	stream->_io_bufsiz = 0;
	stream->_io_buf = NULL;
	stream->_io._io_char = EOF;
	stream->_io_count = _IO_NEVER;
	rv = 0;
	break;
    case _IOLBF:
    case _IOFBF:
	stream->_io_bufsiz = size;
	stream->_io._io_ptr = stream->_io_buf = (unsigned char *) buffer;
	stream->_io_count = _IO_NEVER;
	_io_setflag(stream, type);
	rv = 0;
	break;
    default:
    err:
      rv = _IO_ERRVAL;
      break;
    }
    
    funlockfile(stream);
    return rv;
}


/* ANSI requires function definition 
 * (even if the default is a macro)
*/
void (setbuf)(FILE *stream,char *buffer)
{
  (void) setvbuf(stream, buffer, (buffer == NULL) ? _IONBF : _IOFBF, BUFSIZ);
}
