#ident "lib/c/stdio/tmpnam.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * tmpnam.c : standard I/O library
 */

/* --- tmpnam.c --- */
/* Generates a unique file name */
/* Notes:  File name is of the form Tmpxxxxx.yyyyy	*/
/*	   Not reentrant				*/

#include <sys/types.h>
#include <string.h>
#include <unistd.h>
#include <libc_thread.h>

#include <stdio.h>		/* L_tmpnam !! */
#include "../stdio/stdiolib.h"	/* P_tmpnam !! */

#define DIGITS	5		/* maximum digits converted */

char *(tmpnam) (char *s)
{
  static __sdethread_mutex_t tmpnam_mx = __SDETHREAD_MUTEX_INITIALIZER("tmpnam");
  _THREAD_PRIVATE_KEY(tmpnam);
  static char buf[L_tmpnam];
  int pid;
  int length;
  register int width;
  register char *p;
  int suffix;

  if (s == NULL) {
      s = _THREAD_PRIVATE(tmpnam, buf, NULL);
      if (!s)
	  return NULL;
  }

  __sdethread_mutex_lock (&tmpnam_mx);
  if (_libc_data->_tmpnamsuf >= TMP_MAX) {
      	__sdethread_mutex_unlock (&tmpnam_mx);
	*s = '\0';
	return NULL;
  }
  suffix = _libc_data->_tmpnamsuf++;
  __sdethread_mutex_unlock (&tmpnam_mx);

  pid = getpid();
  strcpy(s, P_tmpdir);
  strcat(s, "/Tmp00000.00000");
  length = strlen(s);
  p = s + (length - 1 - DIGITS - 1);	/* pid ends here */
  width = DIGITS;
  do {
	*p-- = pid % 10 + '0';
  } while ((pid /= 10) && (--width > 0));
  p = s + (length - 1);			/* suffix ends here */
  width = DIGITS;
  do {
	*p-- = suffix % 10 + '0';
  } while ((suffix /= 10) && (--width > 0));
  return s;
}
