#ident "lib/c/stdio/ungetc.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * ungetc.c : standard I/O library
 */

/* --- ungetc.c --- */
/* Pushes one character back into an input stream */
/* Notes:  Will not push back EOF					*/
/*	   Can fail if tried too many times				*/

#include <stdio.h>
#include "stdiolib.h"

#define _IO_ERRVAL EOF

int (ungetc)(int c,FILE *stream)
{
    if (c == EOF)
      return _IO_ERRVAL;

    flockfile (stream);

    _io_assert(stream,  _IO_WRITE);

    if (stream->_io_count == _IO_NEVER)
	/* (re)initialise input stream */
	_i_init(stream);

    if (stream->_io_bufsiz == 0) {
	if (stream->_io._io_char == EOF) {
	    stream->_io._io_char = c;
	    _io_clearflag(stream, _IO_EOF);
	} else {
	    c = _IO_ERRVAL;
	}
    } else if (stream->_io._io_ptr > stream->_io_buf
	       || stream->_io_count == 0) {
	if (stream->_io._io_ptr > stream->_io_buf)
	    --stream->_io._io_ptr;
	stream->_io_count++;
	/* may be a string const: don't rewrite if no change */
	if (*stream->_io._io_ptr != c)
	    *stream->_io._io_ptr = c;
	_io_clearflag(stream, _IO_EOF);
    }

    funlockfile (stream);
    return c;
}
