#ident "lib/c/stdio/vsprintf.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * vsprintf.c : standard I/O library
 */

#include <limits.h>
#include <stdarg.h>
#include <stdio.h>
#include "stdiolib.h"

int __vsnprintf (char *s, size_t size, const char *format, va_list arg)
{
    FILE fake;
    int n;

    fake._io_fd = -1;
    fake._io_flags = _IO_WRITE | _IO_STRING;
    fake._io_bufsiz = fake._io_count = size - 1; /* room for nul */
    fake._io._io_ptr = fake._io_buf = (unsigned char *) s;
    fake._io_funcs = &_io_strfuncs;
    fake._io_lock = NULL;

    n = vfprintf (&fake, format, arg);

    /* terminate string */
    *fake._io._io_ptr = '\0';

    /* return count, excluding terminating null */
    return n;
}

int vsprintf (char *s, const char *format, va_list arg)
{
    return __vsnprintf (s, INT_MAX, format, arg);
}
