#ident "lib/c/str/strcmp.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * strcmp.c : string function
 */

#include <string.h>
#include "stringlib.h"

int strcmp(const char *s1,const char *s2)
{
    /* If s1 and s2 are word-aligned, compare them a word at a time. */
    if (ALIGNEDPAIR (s1, s2)) {  
	const unsigned long *a1 = (unsigned long *)s1;
	const unsigned long *a2 = (unsigned long *)s2;
	while (*a1 == *a2) {
	    /* To get here, *a1 == *a2, thus if we find a null in *a1,
	       then the strings must be equal, so return zero.  */
	    if (DETECTNULL (*a1))
		return 0;
	    a1++;
	    a2++;
        }
	
	/* Now search bytewise. */
	s1 = (char *)a1;
	s2 = (char *)a2;
    }
    
    while (*s1 == *s2) {
	if (*s1 == 0)
	  return 0;
	s1++;
	s2++;
    }

    return (*s1 - *s2);
}
