#ident "lib/c/time/asctime.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * asctime.c : ANSI C function
 */

#include <stdio.h>
#include <time.h>
#include <libc_thread.h>
#include "tzfile.h"

#if __GNUC__ >= 3
#define sprintf __builtin_sprintf
#endif

char *asctime_r (const struct tm *tp, char *buf)
{
    static const char	wday_name[DAYS_PER_WEEK][3] = {
	"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
      };
    static const char	mon_name[MONS_PER_YEAR][3] = {
	"Jan", "Feb", "Mar", "Apr", "May", "Jun",
	"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
      };
    const char *wn, *mn;
    if (tp->tm_wday < 0 || tp->tm_wday >= DAYS_PER_WEEK)
	wn = "???";
    else
	wn = wday_name[tp->tm_wday];
    if (tp->tm_mon < 0 || tp->tm_mon >= MONS_PER_YEAR)
	mn = "???";
    else 
	mn = mon_name[tp->tm_mon];
    (void) sprintf(buf, "%.3s %.3s%3d %02d:%02d:%02d %d\n",
		   wn, mn,
		   tp->tm_mday, tp->tm_hour,
		   tp->tm_min, tp->tm_sec,
		   TM_YEAR_BASE + tp->tm_year);
    return buf;
}

char *asctime (const struct tm *tp)
{
    static char	result[26];
    _THREAD_PRIVATE_KEY(asctime);
    char *s = _THREAD_PRIVATE(asctime, result, NULL);
    if (s)
	return asctime_r (tp, s);
    return NULL;
}
