#ident "lib/c/time/strftime.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * strftime.c : POSIX function
 */


#define _POSIX_SOURCE
#include <time.h>
#include <string.h>
#include <stdio.h>

#define DAYS_PER_WEEK	7
#define MONS_PER_YEAR	12
#define DAYS_PER_NYEAR	365
#define DAYS_PER_LYEAR	366

#define isleap(y) (((y) % 4) == 0 && (((y) % 100) != 0 || ((y) % 400) == 0))

#if __GNUC__ >= 3
#define sprintf __builtin_sprintf
#endif

static const char	awday_names[DAYS_PER_WEEK][3] = {
    "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
  };

static const char	amon_names[MONS_PER_YEAR][3] = {
    "Jan", "Feb", "Mar", "Apr", "May", "Jun",
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
  };

static const char *const fwday_names[DAYS_PER_WEEK] = {
    "Sunday", "Monday", "Tuesday", "Wednesday", 
    "Thursday", "Friday", "Saturday"
  };
	
static const char	*const fmon_names[MONS_PER_YEAR] = {
    "January", "Feburary", "March", "April", "May", "June",
    "July", "August", "September", "October", "November", "December"
  };

#define MAX_CONVERSION 32

#define SUN_WEEK(t)	(((t)->tm_yday + 7 - \
				((t)->tm_wday)) / 7)
#define MON_WEEK(t)	(((t)->tm_yday + 7 - \
				((t)->tm_wday ? (t)->tm_wday - 1 : 6)) / 7)

size_t strftime(char *buf,size_t n,const char *format,const struct tm *tm)
{
    size_t avail = n;
    
    while(1) {
	int c;
	
	while( (c=*format++) && c != '%') {
	    if(avail <= 0) {
		return 0;
	    } else {
		*buf++ = c; 
		avail--;
	    }
	}
	if(c == 0 || (c=*format++) == 0) {
	    if(avail > 0)
	      *buf++ = 0;	/* assist caller */
	    return n-avail;
	} else {
	    char cnvbuf[MAX_CONVERSION];
	    const char *cnv = cnvbuf;
	    size_t len = 0;
	    int alt = 0;
#define _ALT_E	0x01
#define _ALT_O	0x02

	next:
	    switch(c) {
	    default:
		cnvbuf[0] = c; cnvbuf[1] = 0;
		break;
	    case 'a':
		cnv = awday_names[tm->tm_wday];
		len = 3;
		break;
	    case 'A':
		cnv = fwday_names[tm->tm_wday];
		break;
	    case 'b':
		cnv = amon_names[tm->tm_mon];
		len = 3;
		break;
	    case 'B':
		cnv = fmon_names[tm->tm_mon];
		break;
	    case 'c':		/* XXX current locale */
		strftime(cnvbuf,sizeof(cnvbuf),	"%a %b %e %T %Y", tm);
		break;
	    case 'C':
		sprintf(cnvbuf,"%02d",(1900+tm->tm_year)/100);
		break;
	    case 'd':
		sprintf(cnvbuf,"%02d",tm->tm_mday);
		break;
	    case 'D':
		strftime(cnvbuf,sizeof(cnvbuf),"%m/%d/%y", tm);
		break;
	    case 'e':
		sprintf(cnvbuf,"% 2d",tm->tm_mday);
		break;
	    case 'E':	/* modifier */
		alt |= _ALT_E;
		if ((c=*format) != 0) {
		    ++format;
		    goto next;
		}
		break;
	    case 'g':	/* ISO 8601 year (two digits) */
	    case 'G':	/* ISO 8601 year (four digits) */
	    case 'V':	/* ISO 8601 week number */
	    {
		/* ISO 8601 Week Of Year:
		   If the week (Monday - Sunday) containing
		   January 1 has four or more days in the new 
		   year, then it is week 1; otherwise it is 
		   week 53 of the previous year and the next
		   week is week one. */
		/* Algorithm from Arnold Robbins' strftime version 3.0 */
		int year = tm->tm_year + 1900;
		int yday = tm->tm_yday;
		int wday = tm->tm_wday;
		int w;
		for ( ; ; ) {
		    int	len;
		    int	bot;
		    int	top;
		    
		    len = isleap(year) ?
			DAYS_PER_LYEAR :
			DAYS_PER_NYEAR;
		    /*
		    ** What yday (-3 ... 3) does
		    ** the ISO year begin on?
		    */
		    bot = ((yday + 11 - wday) %
			   DAYS_PER_WEEK) - 3;
		    /*
		    ** What yday does the NEXT
		    ** ISO year begin on?
		    */
		    top = bot -
			(len % DAYS_PER_WEEK);
		    if (top < -3)
			top += DAYS_PER_WEEK;
		    top += len;
		    if (yday >= top) {
			++year;
			w = 1;
			break;
		    }
		    if (yday >= bot) {
			w = 1 + ((yday - bot) /
				 DAYS_PER_WEEK);
			break;
		    }
		    --year;
		    yday += isleap(year) ?
			DAYS_PER_LYEAR :
			DAYS_PER_NYEAR;
		}
		switch (c) {
		case 'V':
		    sprintf (cnvbuf, "%02d", w);
		    break;
		case 'g':
		    sprintf (cnvbuf, "%02d", year % 100);
		    break;
		case 'G':
		    sprintf (cnvbuf, "%04d", year);
		    break;
		}
		break;
	    }
	    case 'h':		/* XXX current locale */
		cnv = amon_names[tm->tm_mon];
		len = 3;
		break;
	    case 'H':
		sprintf(cnvbuf,"%02d",tm->tm_hour);
		break;
	    case 'I':
		sprintf(cnvbuf,"%02d", (tm->tm_hour%12)?(tm->tm_hour%12):12);
		break;
	    case 'j':
		sprintf(cnvbuf,"%03d", tm->tm_yday+1);
		break;
	    case 'k':
		sprintf(cnvbuf,"% 2d",tm->tm_hour);
		break;
	    case 'l':
		sprintf(cnvbuf,"% 2d", (tm->tm_hour%12)?(tm->tm_hour%12):12);
		break;
	    case 'm':
		sprintf(cnvbuf,"%02d", tm->tm_mon + 1);
		break;
	    case 'M':
		sprintf(cnvbuf,"%02d", tm->tm_min);
		break;
	    case 'n':
		cnv = "\n";
		break;
	    case 'O':	/* modifier */
		alt |= _ALT_O;
		if ((c=*format) != 0) {
		    ++format;
		    goto next;
		}
		break;
	    case 'p':		/* XXX current locale */
		cnv = (tm->tm_hour < 12) ? "AM" : "PM";
		break;
	    case 'P':		/* XXX current locale */
		cnv = (tm->tm_hour < 12) ? "am" : "pm";
		break;
	    case 'r':
		strftime(cnvbuf,sizeof(cnvbuf),"%I:%M:%S %p", tm);
		break;
	    case 'R':
		strftime(cnvbuf,sizeof(cnvbuf),"%H:%M", tm);
		break;
	    case 's':
	    {
		struct tm safe = *tm;
		sprintf(cnvbuf,"%lu", mktime (&safe));
		break;
	    }
	    case 'S':
		sprintf(cnvbuf,"%02d", tm->tm_sec);
		break;
	    case 't':
		cnv = "\t";
		break;
	    case 'T':
		strftime(cnvbuf,sizeof(cnvbuf),"%H:%M:%S", tm);
		break;
	    case 'u':
		sprintf(cnvbuf, "%d", (tm->tm_wday == 0) ? 7 
			: tm->tm_wday);
		break;
	    case 'U':
		sprintf(cnvbuf, "%02d", SUN_WEEK(tm));
		break;
	    case 'W':
		sprintf(cnvbuf, "%02d", MON_WEEK(tm));
		break;
	    case 'w':
		sprintf(cnvbuf, "%d", tm->tm_wday);
		break;
	    case 'x':		/* XXX current locale */
		strftime (cnvbuf,sizeof(cnvbuf),"%m/%d/%y",tm);
		break;
	    case 'X':		/* XXX current locale */
		strftime(cnvbuf,sizeof(cnvbuf),"%H:%M:%S",tm);
		break;
	    case 'y':
		sprintf(cnvbuf,"%02d", tm->tm_year % 100);
		break;
	    case 'Y':
		sprintf(cnvbuf,"%04d", tm->tm_year + 1900);
		break;
	    case 'z':
	    {
		extern long _tz_stdoffset;	/* XXX */
		extern long _tz_dstoffset;	/* XXX */
		char sign = '+';
		int diff = 0;
		if (tm->tm_isdst > 0)
		    diff = tm->tm_isdst ? -_tz_dstoffset : -_tz_stdoffset;
		if (diff < 0) {
		    sign = '-';
		    diff = -diff;
		} 
		sprintf(cnvbuf,"%c%02d%02d", sign, diff / 60, diff % 60);
		break;
	    }
	    case 'Z':
		cnv = tm->tm_zone;
		if (!cnv || !*cnv)
		    cnv = tzname[tm->tm_isdst != 0];
		break;
	    case '+':
		/* date(1) format */
		strftime(cnvbuf,sizeof(cnvbuf),	"%a %b %e %H:%M:%S %Z %Y", tm);
		break;
	    }

	    /* try to fit cnv into buf with avail bytes space */
	    if (len == 0)
		len = strlen(cnv);
	    if (len > avail) {
		if (avail)
		    *buf++ = 0;
		return 0;
	    }
	    memcpy(buf,cnv,len);
	    buf += len;
	    avail -= len;
	}
    }
}
