#ident "lib/ieee/dp_add.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/* IEEE754 floating point arithmetic
 * double precision: common utilities
 */
/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <limits.h>
#include <assert.h>
#include <stdarg.h>
#include "ieee754dp.h"

ieee754dp ieee754dp_add(ieee754dp x, ieee754dp y)
{
    COMPXDP;
    COMPYDP;
    int s;
    
    EXPLODEXDP;
    EXPLODEYDP;
    
    CLEARCX;

    if (! CLPAIR_NORM_P (cp)) {
#if IEEE754_CLASS_CASE
	switch(cp) {
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_SNAN):
	    return ieee754dp_nanxcpt(ieee754dp_bestnan(x,y),"add",x,y);
	    
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_SNAN):
	    return ieee754dp_nanxcpt(y,"add",x,y);
	    
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_DNORM):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_INF):
	    return ieee754dp_nanxcpt(x,"add",x,y);
	    
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_QNAN):
	return ieee754dp_bestnan(x,y);
	
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_QNAN):
	    return ieee754dp_samenan(y);
	    
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_DNORM):
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_INF):
	    return ieee754dp_samenan(x);
      
    /* Infinity handling */

	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_INF):
	    if(xs == ys)
		return x;
	    SETCX(IEEE754_INVALID_OPERATION);
	    return ieee754dp_xcpt(ieee754dp_indef(),"add",x,y);
      
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_INF):
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_INF):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_INF):
	    return y;
	    
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_DNORM):
	    return x;
	    
      /* Zero handling */
      
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_ZERO):
	    if(xs == ys)
		return x;
	    return ieee754dp_zero(IEEE754_GET_RM () == IEEE754_RD);
	    
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_ZERO):
	    return x;
	    
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_DNORM):
	    return y;
	    
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_DNORM):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_DNORM):
	    /* no need to normalise denormals */
	    break;
	}
#else
	if (LIKELY (cp == (IEEE754_CLASS_ZERO | IEEE754_CLASS_NORM))
	    || cp == (IEEE754_CLASS_ZERO | IEEE754_CLASS_DNORM)) {
	    if ((xc & yc & IEEE754_CLASS_ZERO) && xs != ys)
		/* Zero - Zero = +/-Zero */
		return ieee754dp_zero (IEEE754_GET_RM () == IEEE754_RD);
	    /* Zero + Y or X + Zero = Y or X */
	    return (xc == IEEE754_CLASS_ZERO) ? y : x;
	}

	if (cp & IEEE754_CLASS_SNAN) {
	    /* SNaN handling */
	    return ieee754dp_nanxcpt(ieee754dp_bestnan(x,y),"add",x,y);
	}
	    
	if (cp & IEEE754_CLASS_QNAN) {
	    /* QNaN handling */
	    return ieee754dp_bestnan(x,y);
	}
	    
	if (cp & IEEE754_CLASS_INF) {
	    /* Inf handling */
	    if (xc & yc) {
		/* Inf + Inf = Inf */
		if (xs == ys)
		    return x;
		/* Inf - Inf = Indef */
		SETCX(IEEE754_INVALID_OPERATION);
		return ieee754dp_xcpt(ieee754dp_indef(),"add",x,y);
	    }
	    /* Inf + Y or X + Inf - return the Inf */
	    return (xc == IEEE754_CLASS_INF) ? x : y;
	}

	/* Denorms can be handled without normalisation */
	assert (cp == IEEE754_CLASS_DNORM);
#endif
    }

//    assert(xm & DP_HIDDEN_BIT);
//    assert(ym & DP_HIDDEN_BIT);

    /* provide guard, round and stick bit space */
    xm <<= DP_GUARDBITS;
    ym <<= DP_GUARDBITS;
    
    s = xe - ye;
    if (s > 0) {
	/* have to shift y fraction right to align */
	ym = XDPSRS(ym, s);
    } else if (s < 0) {
	/* have to shift x fraction right to align */
	xm = XDPSRS(xm, -s);
	xe = ye;
    }
    assert(xe <= DP_EMAX);
    
    if(xs == ys) {
	/* generate 28 bit result of adding two 27 bit numbers
	 * leaving result in xm,xs,xe
	 */
	xm = xm + ym;
	if(xm & DP_MBIT(DP_MBITS+1+DP_GUARDBITS)) { 	/* carry out */
	    xm = XDPSRS1(xm);
	    xe++;
	}
    } else {
	xm = xm - ym;

	if (xm == 0)
	  return ieee754dp_zero(IEEE754_GET_RM () == IEEE754_RD);

	if ((int64_t)xm < 0) {
	    xm = -xm;
	    xs = ys;
	}

	/* normalize in extended double precision */
	DP_NORM (xm, xe, DP_GUARDBITS);
    }
    DPNORMRET2(xs,xe,xm,"add",x,y);
}

ieee754dp ieee754dp_sub(ieee754dp x, ieee754dp y)
{
    return ieee754dp_add (x, y ^ DP_SIGN_BIT);
}
