#ident "lib/ieee/dp_fint.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/* IEEE754 floating point arithmetic
 * double precision: common utilities
 */
/*
 * Copyright (c) 1996-2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <limits.h>
#include <assert.h>
#include <stdarg.h>
#include <stdlib.h>
#include "ieee754dp.h"

ieee754dp ieee754dp_fint(int32_t x)
{
    COMPXDP;
    uint32_t xu32;

    CLEARCX;

    if (x == 0)
	/* zero must be handled specially */
	return ieee754dp_zero(0);

    xs = (x < 0);
    if (xs) {
	if(x == (1<<31))
	  xu32 = (1U<<31); /* max neg can't be safely negated */
	else
	  xu32 = -x;
    } else {
	xu32 = x;
    }

    if (xu32 == 1)
	return ieee754dp_one(xs);      
#ifndef ieee754_clz
    if (xu32 == 10)
	return ieee754dp_ten(xs);
#endif

    /* normalize and extend to 64-bits */
    /* result can never be inexact or overflow */

    xe = DP_MBITS;
#ifdef ieee754_clz
    {
	int z = ieee754_clz (xu32);
	assert (z < 31);
	xu32 = xu32 << (z + 1);
	xm = (uint64_t)xu32 << (32 - (64 - DP_MBITS));
	xe -= z + 32 - (64 - (1 + DP_MBITS));
    }
#else
    xm = xu32;
    DP_NORM_PACK (xm, xe, 0);
#endif
    return builddp (xs, xe + DP_EBIAS, xm);
}

ieee754dp ieee754dp_funs(uint32_t u)
{
    if((int32_t)u < 0) 
      return ieee754dp_add(ieee754dp_1e31(0), 
			   ieee754dp_fint(clearmsb32 (u, 1)));
    return ieee754dp_fint(u);
}
