#ident "lib/ieee/dp_flong.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/* IEEE754 floating point arithmetic
 * double precision: common utilities
 */
/*
 * Copyright (c) 1996-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <limits.h>
#include <assert.h>
#include <stdarg.h>
#include "ieee754dp.h"

ieee754dp ieee754dp_flong(int64_t x)
{
    COMPXDP;

    CLEARCX;

    if(x == 0)
      return ieee754dp_zero(0);

    xs = (x < 0);

    if (x == 1 || x == -1)
      return ieee754dp_one(xs);      
    if (x == 10 || x == -10)
      return ieee754dp_ten(xs);

    if(xs) {
	if(x == (1ULL << 63))
	  xm = (1ULL << 63); /* max neg can't be safely negated */
	else
	  xm = -x;
    } else {
	xm = x;
    }
    xe = DP_MBITS+DP_GUARDBITS;

#ifdef ieee754_dclz
    {
	int shift = (64 - (DP_MBITS + 1 + DP_GUARDBITS)) - ieee754_dclz (xm);
	if (shift > 0) {
	    /* shunt out least sig overflow bits */
	    assert (shift < DP_MBITS + DP_GUARDBITS);
	    xm = XDPSRS_safe(xm, shift);
	}
	else if (shift < 0) {
	    /* normalize in grs extended double precision */
	    xm <<= -shift;
	}
	xe += shift;
    }
#else    
    if(xm >> (DP_MBITS+1+DP_GUARDBITS)) {
	/* shunt out overflow bits */
	while(xm >> (DP_MBITS+1+DP_GUARDBITS)) {
	    XDPSRSX1();
	}
    } 
    else {
	/* normalize in grs extended double precision */
	DP_NORM (xm, xe, DP_GUARDBITS);
    }
#endif
    DPNORMRET1(xs,xe,xm,"dp_flong",x);
}

ieee754dp ieee754dp_fulong(uint64_t u)
{
    if((int64_t)u < 0) 
      return ieee754dp_add(ieee754dp_1e63(0), 
			   ieee754dp_flong(clearmsb64 (u, 1)));
    return ieee754dp_flong(u);
}
