#ident "lib/ieee/dp_fsp.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/* IEEE754 floating point arithmetic
 * double precision: common utilities
 */
/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <limits.h>
#include <assert.h>
#include <stdarg.h>
#include "ieee754dp.h"
#include "ieee754sp.h"

ieee754dp ieee754dp_fsp(ieee754sp x)
{
    COMPXSP;
    
    CLEARCX;
    
    EXPLODEXSP;
    
    if (! CLASS_NORM_P (xc)) {
#if IEEE754_CLASS_CASE
	switch(xc) {
#if IEEE754_NO_STICKY_NANS
	case IEEE754_CLASS_QNAN:
	    return ieee754dp_qnan ();
	case IEEE754_CLASS_SNAN:
	    return ieee754dp_nanxcpt (ieee754dp_snan(), "fsp", x);
#else
	case IEEE754_CLASS_QNAN:
	case IEEE754_CLASS_SNAN:
	    return ieee754dp_nanxcpt(builddp(xs,
					     DP_EMAX+1+DP_EBIAS,
					     ((uint64_t)xm << (DP_MBITS-SP_MBITS))),
				     "fsp",x);
#endif
	case IEEE754_CLASS_INF:
	    return ieee754dp_inf(xs);
	case IEEE754_CLASS_ZERO:
	    return ieee754dp_zero(xs);
	case IEEE754_CLASS_DNORM:
	    /* normalize */
	    SP_NORM (xm, xe, 0);
	    break;
	}
#else
	if (xc & IEEE754_CLASS_ZERO)
	    return ieee754dp_zero(xs);
#if IEEE754_NO_STICKY_NANS
	if (xc & IEEE754_CLASS_QNAN)
	    return ieee754dp_qnan (0);
	if (xc & IEEE754_CLASS_SNAN)
	    return ieee754dp_nanxcpt (ieee754dp_snan(), "fdp",x);
#else
	if (xc & (IEEE754_CLASS_QNAN | IEEE754_CLASS_SNAN)) {
	    return ieee754dp_nanxcpt(builddp(xs,
					     DP_EMAX+1+DP_EBIAS,
					     ((uint64_t)xm << (DP_MBITS-SP_MBITS))),
				     "fsp",x);
	}
#endif
	if (xc & IEEE754_CLASS_INF)
	    return ieee754dp_inf(xs);
	/* normalize */
	assert (xc & IEEE754_CLASS_DNORM);
	SP_NORM (xm, xe, 0);
#endif
    }
    
    /* CAN'T possibly overflow, underflow or need rounding */
    
    /* drop the hidden bit and above */
    xm = clearmsb32 (xm, 32 - SP_MBITS);
    
    return builddp(xs, xe + DP_EBIAS,
		   (uint64_t)xm << (DP_MBITS-SP_MBITS));
}
