#ident "lib/ieee/dp_fstr.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <ctype.h>
#include <errno.h>
#include <assert.h>

#define _IEEE

#ifdef _IEEE
/* avoid compiler/assembler breaking constants by building them by
 * hand.
 */
#define DOIEEEHEX

#include "ieee754int.h"
#include "ieee754dp.h"
#endif

/*
 * strtod for IEEE doubles 
*/

#if defined(DOIEEEHEX)

#include <machine/endian.h>

#if BYTE_ORDER == LITTLE_ENDIAN
struct ieeehex { unsigned long l,h; };
#define IEEEHEXK(hi,lo)	{(lo),(hi)}
#else
struct ieeehex { unsigned long h,l; };
#define IEEEHEXK(hi,lo)	{(hi),(lo)}
#endif

static const struct ieeehex f5pow[] = {
    IEEEHEXK( 0x40140000, 0x00000000 ),	/* 5 */
    IEEEHEXK( 0x40390000, 0x00000000 ),	/* 25 */
    IEEEHEXK( 0x40838800, 0x00000000 ),	/* 625 */
    IEEEHEXK( 0x4117d784, 0x00000000 ),	/* 390625 */
    IEEEHEXK( 0x4241c379, 0x37e08000 ),	/* 152587890625 */
    IEEEHEXK( 0x4493b8b5, 0xb5056e17 ),	/* 2.3283064365387e+022 */
    IEEEHEXK( 0x49384f03, 0xe93ff9f5 ),	/* 5.42101086242753e+044 */
    IEEEHEXK( 0x52827748, 0xf9301d32 ),	/* 2.93873587705572e+089 */
    IEEEHEXK( 0x65154fdd, 0x7f73bf3c )	/* 8.63616855509445e+178 */
};

#define F5POW(x)	(*(ieee754dp *)&f5pow[x])	/* 5^x */
#define F5POW_MAXX	(sizeof(f5pow)/sizeof(f5pow[0]))

#else
static const ieee754dp	f5pow[]	= {
	5.,
	25.,
	625.,
	390625.,
	152587890625.,
	23283064365386962890625.,
#ifdef _IEEE
	5.4210108624275231e+044,
	2.9387358770557196e+089,
	8.6361685550944492e+178,
#endif
};
#define F5POW		(f5pow[x])
#define F5POW_MAXX	(sizeof(f5pow)/sizeof(f5pow[0]))
#endif

#ifdef	HAVE_IEEE
/* required constats 
 */
#define FZERO	0.
#define FONE	1.
#define FTEN	10.

/* required operations 
*/
#define FLO(i)		((double)(i))
#define FMUL(x,y)	((x)*(y))
#define FDIV(x,y)	((x)/(y))
#define FADD(x,y)	((x)+(y))
#define FCMPLT(x,y)	((x) < (y))
#define FNEG(x)		(-(x))
#define ISINF(x)	isinf(x)
#else

/* required constats 
 */
#define FZERO		ieee754dp_zero(0)
#define FONE		ieee754dp_one(0)
#define FTEN		ieee754dp_ten(0)

/* XXX target has ops 
 */
#define FADD(x,y)	ieee754dp_add(x,y)
#define FNEG(x)		ieee754dp_neg(x)
#define FMUL(x,y)	ieee754dp_mul(x,y)
#define FDIV(x,y)	ieee754dp_div(x,y)
#define FCMPLT(x,y)	(ieee754dp_order(x,y) < 0)
#define FLO(i)		ieee754dp_fint(i)
#define ISINF(x)	ieee754dp_isinf(x)
#endif

#define FSCALB(x,n)	ieee754dp_nldexp(x,n)

ieee754dp ieee754dp_nfstr(const char *nptr, char **endp)
{
    const char *s = nptr;
    int mneg = 0;
    int expt = 0;
    int nz   = 0;
    int some = 0;
    ieee754dp	v = FZERO;

    while(isspace(*s) ) 
      s++;
    
    if(*s == '-') {
	mneg = 1;
	s++;
    }else if(*s == '+'){
           s++;
    }
    
    /* grab 
     */
    while( isdigit(*s) ) {
	int inc = *s++ - '0';
	nz |= inc;
#ifdef notdef
	if(FCMPLT(v,F2EM))
	   v = FADD(FMUL(v,FTEN),FLO(inc));
	else
	  expt++;
#else
	v = FADD(FMUL(v,FTEN),FLO(inc));
#endif
	some = 1;
    }
    
    if( *s == '.') {
	s++;
	
	/* grab fractional part decrementing exponent
	 */
	while( isdigit(*s) ) {
	    int inc = *s++ - '0';
	    nz |= inc;
#ifdef notdef
	    if(FCMPLT(v,F2EM)) {
		v = FADD(FMUL(v,FTEN),FLO(inc));
		expt--;
	    }
#else
	    v = FADD(FMUL(v,FTEN),FLO(inc));
	    expt--;
#endif
	    some = 1;
	}
    }

    if(some && (*s == 'e' || *s == 'E')) {
	int e 		= 0;
	int eneg	= 0;
	
	s++;
	
	/* grab exponent */
	if( *s == '-') {
	    eneg = 1;
	    s++;
	} else if( *s == '+') {
	    s++;
	}
	while(isdigit(*s))
	  e = e * 10 + (*s++ - '0');
	if(eneg)
	  e = -e;
	expt += e;
    }
    
    if(endp)
      *endp = (char *) (some ? s : nptr);

    if(expt) {
	int c;
	ieee754dp	flexp = FONE;
	int	bexp = expt;

	if(expt < 0)
	  expt = -expt;
	
#ifndef notdef
	for (c = F5POW_MAXX; c;  c--) {
	    if (expt & (1 << (c-1)))
	      flexp = FMUL(flexp,F5POW(c-1));
	}
#else
	while(expt--)
	  flexp = FMUL(flexp,F5POW(0));
#endif	
	if (bexp < 0) 
	  v = FDIV(v,flexp);
	else 
	  v = FMUL(v,flexp);
	
	v = FSCALB(v, bexp);
    }
    
    if (UNLIKELY (ISINF (v))) {
	errno = ERANGE;
    }
    else if (UNLIKELY (v == FZERO && nz)) {
	v = FZERO;
	errno = ERANGE;
    }

    if (mneg)
      v = FNEG(v);
    
    return v;
}

#if ! IEEE754_GCCSOFTFLOAT
ieee754dp_u ieee754dp_fstr(const char *nptr, char **endp)
{
    ieee754dp_u r;
    r.bits = ieee754dp_nfstr(nptr, endp);
    return r;
}
#endif
