#ident "lib/ieee/dp_logb.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/* IEEE754 floating point arithmetic
 * double precision: common utilities
 */
/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <limits.h>
#include <assert.h>
#include <stdarg.h>
#include "ieee754dp.h"

ieee754dp ieee754dp_logb(ieee754dp x)
{
    COMPXDP;

    CLEARCX;

    EXPLODEXDP;

    if (! CLASS_NORM_P (xc)) {
#if IEEE754_CLASS_CASE
	switch(xc) {
	case IEEE754_CLASS_SNAN:
	    return ieee754dp_nanxcpt(x,"logb",x);
	case IEEE754_CLASS_QNAN:
	    return x;
	case IEEE754_CLASS_INF:
	    return ieee754dp_inf(0);
	case IEEE754_CLASS_ZERO:
	    return ieee754dp_inf(1);
	case IEEE754_CLASS_DNORM:
	    DPDNORMX;
	    break;
	}
#else
	if (xc & IEEE754_CLASS_INF)
	    return ieee754dp_inf(0);
    	if (xc & IEEE754_CLASS_ZERO)
	    return ieee754dp_inf(1);
	if (xc & IEEE754_CLASS_QNAN)
	    return x;
	if (xc & IEEE754_CLASS_SNAN)
	    return ieee754dp_nanxcpt(x,"logb",x);
	assert (xc & IEEE754_CLASS_DNORM);
	DPDNORMX;
#endif
    }

    return ieee754dp_fint(xe);
}
