#ident "lib/ieee/dp_modf.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/* IEEE754 floating point arithmetic
 * double precision: common utilities
 */
/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <limits.h>
#include <assert.h>
#include <stdarg.h>
#include "ieee754dp.h"

/* modf function is always exact for a finite number
*/
ieee754dp ieee754dp_nmodf(ieee754dp x,ieee754dp *ip)
{
    COMPXDP;
    uint64_t tm;
    unsigned int bits;

    CLEARCX;
    
    EXPLODEXDP;

    if (! CLASS_NORM_P (xc)) {
#if IEEE754_CLASS_CASE
	switch(xc) {
	case IEEE754_CLASS_SNAN:
	case IEEE754_CLASS_QNAN:
	case IEEE754_CLASS_INF:
	case IEEE754_CLASS_ZERO:
	    *ip = x;
	    return x;
	case IEEE754_CLASS_DNORM:
	    /* far too small */
	    *ip = ieee754dp_zero(xs);
	    return x;
	}
#else
	if (xc & (IEEE754_CLASS_SNAN | IEEE754_CLASS_QNAN
		  | IEEE754_CLASS_INF | IEEE754_CLASS_ZERO)) {
	    *ip = x;
	    return x;
	}

	/* far too small */
	assert (xc & IEEE754_CLASS_DNORM);
	*ip = ieee754dp_zero(xs);
	return x;
#endif
    }

    if(xe < 0) {
	*ip = ieee754dp_zero(xs);
	return x;
    }
    if(xe >= DP_MBITS) {
	*ip = x;
	return ieee754dp_zero(xs);
    } 

    /* generate ipart mantissa by clearing bottom bits */
    bits = DP_MBITS - xe;
    tm = (xm >> bits) << bits;
    tm &= ~DP_HIDDEN_BIT;
    *ip = builddp(xs, xe + DP_EBIAS, tm);
    
    /* generate fpart mantissa by clearing top bits
     * and normalizing (must be able to normalize)
     */
    xm = (xm << (64 - bits)) >> (64 - bits);
    if(xm == 0) 
      return ieee754dp_zero(xs);
    
    DP_NORM_PACK (xm, xe, 0);
    return builddp(xs, xe+DP_EBIAS, xm);
}

#if ! IEEE754_GCCSOFTFLOAT
ieee754dp_u ieee754dp_modf(ieee754dp_u x, ieee754dp_u *ip)
{
    ieee754dp_u r;
    r.bits = ieee754dp_nmodf (x.bits, &ip->bits);
    return r;
}
#endif
