#ident "lib/ieee/dp_mul.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/* IEEE754 floating point arithmetic
 * double precision: common utilities
 */
/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <limits.h>
#include <assert.h>
#include <stdarg.h>
#include "ieee754dp.h"

ieee754dp ieee754dp_mul(ieee754dp x, ieee754dp y)
{
    COMPXDP;
    COMPYDP;

    CLEARCX;

    EXPLODEXDP;
    EXPLODEYDP;
    
    if (! CLPAIR_NORM_P (cp)) {
#if IEEE754_CLASS_CASE
	switch(cp) {
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_SNAN):
	    return ieee754dp_nanxcpt(ieee754dp_bestnan(x,y),"mul",x,y);
	    
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_SNAN):
	    return ieee754dp_nanxcpt(y,"mul",x,y);
	    
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_DNORM):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_INF):
	    return ieee754dp_nanxcpt(x,"mul",x,y);
	    
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_QNAN):
	    return ieee754dp_bestnan(x,y);
	    
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_QNAN):
	    return ieee754dp_samenan(y);
      
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_DNORM):
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_INF):
	    return ieee754dp_samenan(x);
	    
	    
    /* Infinity handling */

	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_INF):
	    SETCX(IEEE754_INVALID_OPERATION);
	    return ieee754dp_xcpt(ieee754dp_indef(),"mul",x,y);
	    
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_INF):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_INF):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_DNORM):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_INF):
	    return ieee754dp_inf(xs ^ ys);
	    
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_DNORM):
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_ZERO):
	    return ieee754dp_zero(xs ^ ys);
	
      
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_DNORM):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_NORM):
	    DPDNORMX;
	    if (yc != IEEE754_CLASS_DNORM)
		break;
	    /* fall through */
	    
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_DNORM):
	    DPDNORMY;
	    break;
	}
#else
	if (LIKELY (cp == IEEE754_CLASS_ZERO))
	    /* Norm * Zero, or  Zero * Norm, or Zero * Zero */
	    return ieee754dp_zero(xs ^ ys);
	
	if (cp & IEEE754_CLASS_SNAN) {
	    /* SNaN handling */
	    return ieee754dp_nanxcpt(ieee754dp_bestnan(x,y),"mul",x,y);
	}

	if (cp & IEEE754_CLASS_QNAN) {
	    /* QNaN handling */
	    return ieee754dp_bestnan(x,y);
	}
      
	if (cp & IEEE754_CLASS_INF) {
	    /* Inf handling */
	    if (cp & IEEE754_CLASS_ZERO) {
		/* Inf * Zero = Indef */
		SETCX(IEEE754_INVALID_OPERATION);
		return ieee754dp_xcpt(ieee754dp_indef(),"mul",x,y);
	    }
	    /* Inf * X = Inf */
	    return ieee754dp_inf(xs ^ ys);
	}

	assert (cp & IEEE754_CLASS_DNORM);

	if (cp & IEEE754_CLASS_ZERO)
	    /* DNorm * Zero = Zero */
	    return ieee754dp_zero(xs ^ ys);

	if (xc & IEEE754_CLASS_DNORM)
	    DPDNORMX;
	
	if (yc & IEEE754_CLASS_DNORM)
	    DPDNORMY;
#endif
    }

    assert(xm & DP_HIDDEN_BIT);
    assert(ym & DP_HIDDEN_BIT);

    /* rm = xm * ym, re = xe+ye basically */
    {
	int re = xe + ye;
	int rs = xs ^ ys;
	uint64_t rm;
	
	/* shunt to top of word */
	xm <<= 64-(DP_MBITS+1);
	ym <<= 64-(DP_MBITS+1);
    
	/* multiply 64bits xm x ym to give high 64bits rm with stickiness */
#if __mips64
	{
	    /* 64 * 64 => 128 */
	    uint64_t lrm;
	    __asm__ ("dmultu %2,%3; mflo %1; mfhi %0"
		     : "=d" (rm), "=&d" (lrm)
		     : "d" (xm), "d" (ym)
		     : "hi", "lo");
	    rm |= (lrm != 0);
	}
#else	
	{
	    uint64_t rm_h;
	    uint32_t rm_lh;
	    uint32_t rm_ll;
	    uint32_t xm_l = xm;
	    uint32_t xm_h = xm >> 32;
	    uint32_t ym_l = ym;
	    uint32_t ym_h = ym >> 32;
	    uint32_t carry;

#define DPXMULT(x,y)	((x) * (uint64_t)(y))  /* 32 * 32 => 64 */

	    {
		uint64_t rm_l;
		rm_l = DPXMULT(xm_l,ym_l);
		rm_lh = rm_l >> 32;
		rm_ll = rm_l;
	    }
	    rm_h = DPXMULT(xm_h,ym_h);

	    {
		uint64_t x0 = DPXMULT(xm_l,ym_h);
		uint32_t at = rm_lh + (uint32_t)x0;
		rm_h += (x0 >> 32);
		carry = at < rm_lh; /* lrm carry out */
		rm_lh = at;
	    }

	    {
		uint64_t x1 = DPXMULT(xm_h,ym_l);	
		uint32_t at = rm_lh + (uint32_t)x1;
		rm_h += (x1 >> 32);
		carry += at < rm_lh; /* lrm carry out */
		rm_lh = at;
	    }

	    /* add carry */
	    rm_h += carry;

	    /* add stickiness */
	    rm = rm_h | ((rm_lh | rm_ll) != 0);
	}
#endif
	
	/*
	 * sticky shift down to normal rounding precision
	 */
	if((int64_t)rm < 0) {
	    rm = XDPSRS (rm, 64-(DP_MBITS+1+DP_GUARDBITS));
	    re++;
	} else {
	    rm = XDPSRS (rm, 64-(DP_MBITS+1+DP_GUARDBITS+1));
	}

	assert(rm & (DP_HIDDEN_BIT << DP_GUARDBITS));
	DPNORMRET2(rs,re,rm,"mul",x,y);
    }
}
