#ident "lib/ieee/dp_tint.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/* IEEE754 floating point arithmetic
 * double precision: convert to 32-bit signed int
 */
/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <limits.h>
#include <assert.h>
#include <stdarg.h>
#include "ieee754dp.h"

int32_t ieee754dp_tint(ieee754dp x)
{
    COMPXDP;
    
    CLEARCX;
    
    EXPLODEXDP;
    
    if (! CLASS_NORM_P (xc)) {
#if IEEE754_CLASS_CASE
	switch(xc) {
	case IEEE754_CLASS_SNAN:
	case IEEE754_CLASS_QNAN:
	    SETCX(IEEE754_INVALID_OPERATION);
	    return ieee754si_xcpt(ieee754si_indef(),"fixdp",x);
	case IEEE754_CLASS_INF:
	    SETCX(IEEE754_OVERFLOW);
	    return ieee754si_xcpt(ieee754si_indef(),"fixdp",x);
	case IEEE754_CLASS_ZERO:
	    return 0;
	case IEEE754_CLASS_DNORM:		/* much too small */
	    SETCX(IEEE754_UNDERFLOW);
	    return ieee754si_xcpt(0,"fixdp",x);
	}
#else
        if (xc & IEEE754_CLASS_ZERO)
	    return 0;
	if (xc & (IEEE754_CLASS_SNAN | IEEE754_CLASS_QNAN)) {
	    SETCX(IEEE754_INVALID_OPERATION);
	    return ieee754si_xcpt(ieee754si_indef(),"fixdp",x);
	}
        if (xc & IEEE754_CLASS_INF) {
	    SETCX(IEEE754_OVERFLOW);
	    return ieee754si_xcpt(ieee754si_indef(),"fixdp",x);
	}
	/* much too small */
	assert (xc & IEEE754_CLASS_DNORM);
	SETCX(IEEE754_UNDERFLOW);
	return ieee754si_xcpt(0,"fixdp",x);
#endif
    }

    if(xe >= 31) {
	SETCX(IEEE754_OVERFLOW);
	return ieee754si_xcpt(ieee754si_indef(),"fix",x);
    }
    if(xe < 0) {
	SETCX(IEEE754_UNDERFLOW);
	return ieee754si_xcpt(0,"fix",x);
    }

    if(xe > DP_MBITS) {
	xm <<= xe - DP_MBITS;
    } else if(xe < DP_MBITS) {
	/* XXX no rounding */
	xm >>= DP_MBITS - xe;
    }

    return xs ? -xm : xm;
}
