#ident "lib/ieee/dp_tstr.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stddef.h>
#include <stdlib.h>
#include "ieee754.h"

/*
 * have a number x == m2 * 2**e2
 *
 * 1) must be able to print any number we can scan
 * 2) must be able to scan any number we can print
*/

#define MAX_EDIGITS	3
#define MAX_MDIGITS	30
#define MAX_ECONV	(1+MAX_MDIGITS+1+1+MAX_EDIGITS+1)

#include <machine/endian.h>

#ifdef HAVE_IEEE
#define FZERO		0.0
#define FONE		1.0
#define FTEN		10.0

#define FNAN(x)		((x) != (x))
#define FLTZ(x)		((x) < 0.0)
#define FEQZ(x)		((x) == 0.0)
#define FADD(x,y)	((x)+(y))
#define FNEG(x)		(-(x)
#define FMUL(x,y)	((x) * (y))
#define FDIV(x,y)	((x) / (y))
#define FIX(x)		((int)(x))
#define MODF(x,ip)	ieee754dp_nmodf(x,ip)
#else
#define FZERO		ieee754dp_zero(0)
#define FONE		ieee754dp_one(0)
#define FTEN		ieee754dp_ten(0)

#define FNAN(x)		ieee754dp_ne(x,x)
#define FLTZ(x)		ieee754dp_lt(x,FZERO)
#define FEQZ(x)		ieee754dp_eq(x,FZERO)
#define FADD(x,y)	ieee754dp_add(x,y)
#define FNEG(x)		ieee754dp_neg(x)
#define FMUL(x,y)	ieee754dp_mul(x,y)
#define FDIV(x,y)	ieee754dp_div(x,y)
#define FIX(x)		ieee754dp_tint(x)
#define MODF(x,ip)	ieee754dp_nmodf(x,ip)
#endif

char *ieee754dp_tstr(ieee754dp x,int prec,int fmt,int af)
{
    int 	sign;
    int		expt = 0;
    
    if(prec < 0)
      prec = 0;
    
    if(!ieee754dp_finite(x)) {
	if(FNAN(x))
	  return "Nan";
	else if( FLTZ(x) )
	  return "-Infinity";
	else
	  return "+Infinity";
    }
    
    if(FLTZ(x)) {
	x = FNEG(x);
	sign = 1;
    } else {
	sign = 0;
    }
    
    
    if(!FEQZ(x)) {
	ieee754dp	ip;
	ieee754dp	f;
	
	f = MODF(x,&ip);
	if(!FEQZ(f)) {
	    ieee754dp	scale = FONE;
	    ieee754dp	five = ieee754dp_nldexp(ieee754dp_ten(0),-1);
	    
	    /* scale up small values to integer
	     */
	    do {
		expt--;
		scale = FMUL(scale,five);
		x = ieee754dp_nldexp(x,1);
	    } while(!FEQZ(f=MODF(ieee754dp_mul(x,scale),&ip)));
	    x = FMUL(x,scale);
	} else {
	    ieee754dp	scale = FONE;
	    
	    /* scale till x/10 produces fract
	     */
	    while(FEQZ(MODF(FDIV(x,FMUL(scale,FTEN)),&ip))) {
		expt++;
		scale = FMUL(scale,FTEN);
	    }
	    x = FDIV(x,scale);
	}
    }
    
    /* x is integer(ish) in double precision format 
     * value is x * 10**expt
     */
    {
	char *buf;
	char *r;
	int ndig;
	int nsd;
	
	buf = malloc (MAX_ECONV);
	if (!buf)
	    return NULL;

	r = &buf[MAX_ECONV];

	/* reverse print x into back of buffer (r)
	 */
	{
	    ieee754dp fudge;	/* 0.015 fix truncates to int */
	    
	    fudge = ieee754dp_nldexp(ieee754dp_one(0),-6);
	    
	    *--r = '\0';
	    for(ndig=0; ndig<(MAX_ECONV-1) && !FEQZ(x); ndig++) {
		ieee754dp df = MODF(FDIV(x,FTEN),&x);
		
		int 	   di = FIX(FMUL(FADD(df,fudge),FTEN));
		
		*--r =  di + '0';
	    }
	}
	expt += ndig;

	/* value repr is 0.rrrrrrr * 10**expt 
	 * there are ndig digits in r (ndig>=0)
	 */
	nsd = prec+1;
	
	/* round r to nsd
	 * NOTE rounding needs 1 extra r space
	 */
	{
	    if(nsd <= 0) {
		*r = '\0';
	    } else if(ndig > nsd) {
		int carry;
		int ndig = nsd;
		
		carry = r[ndig] >= '5';	/* hmm */
		r[ndig] = '\0';
		ndig--;
		while(carry && ndig >= 0) {
		    if(r[ndig] < '9') {
			r[ndig]++;
			carry = 0;
		    } else {
			r[ndig]='\0';
			carry = 1;
		    }
		    ndig--;
		}
		if(carry) {
		    *--r = '1'; expt++;
		}
	    }
	}

	/* create in e/f format */
	{
	    char *p = buf;
	    
	    if(sign)
	      *p++='-';
	
	    if(*r) {
		*p++ = *r++; expt--; 
	    } else {
		*p++ = '0';
	    }
	    if(prec > 0 || af)
	      *p++ = '.';
	    while(*r && prec > 0) {
		*p++ = *r++;
		prec--;
	    }
	    while(prec > 0) {
		*p++ = '0';
		prec--;
	    }
	    *p = '\0';
	    
	    /* add integer exponent */
	    *p++ = (fmt == 'E' || fmt == 'G')?'E':'e';
	    if(expt<0) {
		expt = -expt;
		*p++ ='-';
	    } else
	      *p++ = '+';
	    
	    {
		char	ebuf[MAX_EDIGITS];
		char	*r;
		int 	ndigit = 0;
		
		for(r=&ebuf[MAX_EDIGITS]; expt; expt /= 10) {
		    *--r = expt%10 + '0';
		    ndigit++;
		}
		
		if(ndigit == 0) {
		    *p++ = '0';
		    *p++ = '0';
		} else {
		    if(ndigit < 2)	/* !!!! */
		      *p++ ='0';	
		    
		    while(ndigit--) 
		      *p++ = *r++;
		}
	    }
	    *p = '\0';
	}
	return buf;
    }
}
