#ident "lib/ieee/gccfpe.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if defined (__NO_FLOAT)

void __nofloat (void);

#ifdef L_nofloat

static const char warning[] __attribute__((section(".gnu.warning"))) 
    = "warning: linking code which uses floating-point with -mno-float library";

void __nofloat (void)
{
    abort ();
}

#else

void FNNAME (void)
{
    __nofloat ();
}

#endif

#elif defined (__SOFT_FLOAT) || defined (__SINGLE_FLOAT)

#include "gccfpe.h"

#ifdef L_adddf3
#include "dp_add.c"
#endif

#ifdef L_subdf3
#include "dp_sub.c"
#endif

#ifdef L_muldf3
#include "dp_mul.c"
#endif

#ifdef L_divdf3
#include "dp_div.c"
#endif

#ifdef L_negdf2
#include "dp_neg.c"
#endif

#ifdef L_absdf2
#include "dp_abs.c"
#endif

#ifdef L_sqrtdf2
#include "dp_sqrt.c"
#endif

#ifdef L_sqrtsf2
#include "sp_sqrt.c"
#endif

#ifdef L_ldexp
#include "dp_scalb.c"
#endif

#ifdef L_ldexpf
#include "sp_scalb.c"
#endif

#ifdef L_modf
#include "dp_modf.c"
#endif

#ifdef L_frexp
#include "dp_frexp.c"
#endif

#ifdef L_strtod
#include "dp_fstr.c"
#endif

#ifdef L_strtof
#include "sp_fstr.c"
#endif

#ifdef L_fixdfsi
#include "dp_tint.c"
#endif

#ifdef L_fixunsdfsi
#include "dp_tuns.c"
#endif

#ifdef L_fixdfdi
#include "dp_tlong.c"
#endif

#ifdef L_fixunsdfdi
#include "dp_tulong.c"
#endif

#ifdef L_floatsidf
#include "dp_fint.c"
#endif

#ifdef L_floatdidf
#include "dp_flong.c"
#endif

#ifdef L_extendsfdf2
#include "dp_fsp.c"
#endif

#ifdef L_cmpdf2
#include "dp_order.c"
#endif

#ifdef L_eqdf2
#include "ieee754dp.h"
int __eqdf2(ieee754dp x,ieee754dp y)
{
  if (UNLIKELY (ieee754dp_isnan (x) || ieee754dp_isnan (y))) {
      return 1;			/* ne */
  }
  return ieee754dp_order (x, y);
}
#endif

#ifdef L_nedf2
#include "ieee754dp.h"
int __nedf2(ieee754dp x,ieee754dp y)
{
  if (UNLIKELY (ieee754dp_isnan (x) || ieee754dp_isnan (y))) {
      return 1;			/* ne */
  }
  return ieee754dp_order (x, y);
}
#endif

#ifdef L_gtdf2
#include "ieee754dp.h"
int __gtdf2(ieee754dp x,ieee754dp y)
{
  if (UNLIKELY (ieee754dp_isnan (x) || ieee754dp_isnan (y))) {
      if (SETCX (IEEE754_INVALID_OPERATION))
	  return ieee754si_xcpt (-1, "gtf2", x, y);
      return -1;		/* lt */
  }
  return ieee754dp_order (x, y);
}
#endif

#ifdef L_gedf2
#include "ieee754dp.h"
int __gedf2(ieee754dp x,ieee754dp y)
{
  if (UNLIKELY (ieee754dp_isnan (x) || ieee754dp_isnan (y))) {
      if (SETCX (IEEE754_INVALID_OPERATION))
	  return ieee754si_xcpt (-1, "gedf2", x, y);
      return -1;		/* lt */
  }
  return ieee754dp_order (x, y);
}
#endif

#ifdef L_ltdf2
#include "ieee754dp.h"
int __ltdf2(ieee754dp x,ieee754dp y)
{
  if (UNLIKELY (ieee754dp_isnan (x) || ieee754dp_isnan (y))) {
      if (SETCX (IEEE754_INVALID_OPERATION))
	  return ieee754si_xcpt (1, "ltdf2", x, y);
      return 1;			/* gt */
  }
  return ieee754dp_order (x, y);
}
#endif

#ifdef L_ledf2
#include "ieee754dp.h"
int __ledf2(ieee754dp x,ieee754dp y)
{
  if (UNLIKELY (ieee754dp_isnan (x) || ieee754dp_isnan (y))) {
      if (SETCX (IEEE754_INVALID_OPERATION))
	  return ieee754si_xcpt (1, "ledf2", x, y);
      return 1;			/* gt */
  }
  return ieee754dp_order (x, y);
}
#endif

#if defined (__SOFT_FLOAT) || defined (__SINGLE_FLOAT)
#include "ieee754dp.h"
/* hard-ieee754sp version is in libgcc.a */
#ifdef L_unorddf2
int __unorddf2(ieee754dp x,ieee754dp y)
{
    return (ieee754dp_isnan(x) || ieee754dp_isnan(y));
}
#endif
#endif

#ifdef L_addsf3
#include "sp_add.c"
#endif

#ifdef L_subsf3
#include "sp_sub.c"
#endif

#ifdef L_mulsf3
#include "sp_mul.c"
#endif

#ifdef L_divsf3
#include "sp_div.c"
#endif

#ifdef L_negsf2
#include "sp_neg.c"
#endif

#ifdef L_abssf2
#include "sp_abs.c"
#endif

#ifdef L_fixsfsi
#include "sp_tint.c"
#endif

#ifdef L_fixunssfsi
#include "sp_tuns.c"
#endif

#ifdef L_fixsfdi
#include "sp_tlong.c"
#endif

#ifdef L_fixunssfdi
#include "sp_tulong.c"
#endif

#ifdef L_floatsisf
#include "sp_fint.c"
#endif

#ifdef L_floatdisf
#include "sp_flong.c"
#endif

#ifdef L_truncdfsf2
#include "sp_fdp.c"
#endif

#ifdef L_cmpsf2
#include "sp_order.c"
#endif

#ifdef L_eqsf2
#include "ieee754sp.h"
int __eqsf2(ieee754sp x,ieee754sp y)
{
  if (UNLIKELY (ieee754sp_isnan (x) || ieee754sp_isnan (y))) {
      return 1;			/* ne */
  }
  return ieee754sp_order (x, y);
}
#endif

#ifdef L_nesf2
#include "ieee754sp.h"
int __nesf2(ieee754sp x,ieee754sp y)
{
  if (UNLIKELY (ieee754sp_isnan (x) || ieee754sp_isnan (y))) {
      return 1;			/* ne */
  }
  return ieee754sp_order (x, y);
}
#endif

#ifdef L_gtsf2
#include "ieee754sp.h"
int __gtsf2(ieee754sp x,ieee754sp y)
{
  if (UNLIKELY (ieee754sp_isnan (x) || ieee754sp_isnan (y))) {
      if (SETCX (IEEE754_INVALID_OPERATION))
	  return ieee754si_xcpt (-1, "gtf2", x, y);
      return -1;		/* lt */
  }
  return ieee754sp_order (x, y);
}
#endif

#ifdef L_gesf2
#include "ieee754sp.h"
int __gesf2(ieee754sp x,ieee754sp y)
{
  if (UNLIKELY (ieee754sp_isnan (x) || ieee754sp_isnan (y))) {
      if (SETCX (IEEE754_INVALID_OPERATION))
	  return ieee754si_xcpt (-1, "gesf2", x, y);
      return -1;		/* lt */
  }
  return ieee754sp_order (x, y);
}
#endif

#ifdef L_ltsf2
#include "ieee754sp.h"
int __ltsf2(ieee754sp x,ieee754sp y)
{
  if (UNLIKELY (ieee754sp_isnan (x) || ieee754sp_isnan (y))) {
      if (SETCX (IEEE754_INVALID_OPERATION))
	  return ieee754si_xcpt (1, "ltsf2", x, y);
      return 1;			/* gt */
  }
  return ieee754sp_order (x, y);
}
#endif

#ifdef L_lesf2
#include "ieee754sp.h"
int __lesf2(ieee754sp x,ieee754sp y)
{
  if (UNLIKELY (ieee754sp_isnan (x) || ieee754sp_isnan (y))) {
      if (SETCX (IEEE754_INVALID_OPERATION))
	  return ieee754si_xcpt (1, "lesf2", x, y);
      return 1;			/* gt */
  }
  return ieee754sp_order (x, y);
}
#endif

#if defined (__SOFT_FLOAT)
#include "ieee754sp.h"
/* hard-ieee754sp version is in libgcc.a */
#ifdef L_unordsf2
int __unordsf2(ieee754sp x,ieee754sp y)
{
    return (ieee754sp_isnan(x) || ieee754sp_isnan(y));
}
#endif
#endif

#endif /* !NOFLOAT */
