#ident "lib/ieee/ieee754d.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/* some debug functions
*/
/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <stdio.h>
#include <stdarg.h>

#include "ieee754int.h"

ieee754dp ieee754dp_dump(char *m,ieee754dp x)
{
    int c, i;

    printf("%s",m);
    printf("<%016llx>\n", x);
    printf("\t=");
    switch(c = ieee754dp_class(x)) {
    default:
	abort();
#if ! IEEE754_NO_SNANS
    case IEEE754_CLASS_SNAN:
#endif
    case IEEE754_CLASS_QNAN:
	printf("%cNan %c", c == IEEE754_CLASS_QNAN ? 'Q' : 'S',
	       DPSIGN(x)?'-':'+');
	for(i=DP_FBITS-1;i>=0;i--)
	  printf("%c", DPMANT(x) & DP_MBIT(i)?'1':'0');
	break;
    case IEEE754_CLASS_INF:
	printf("%cInfinity", DPSIGN(x)?'-':'+');
	break;
    case IEEE754_CLASS_ZERO:
	printf("%cZero", DPSIGN(x)?'-':'+');
	break;
    case IEEE754_CLASS_DNORM:
	printf("%c0.",  DPSIGN(x)?'-':'+');
	for(i=DP_FBITS-1;i>=0;i--)
	  printf("%c", DPMANT(x) & DP_MBIT(i)?'1':'0');
	printf("e%d",DPBEXP(x) - DP_EBIAS);
	break;
    case IEEE754_CLASS_NORM:
	printf("%c1.",  DPSIGN(x)?'-':'+');
	for(i=DP_FBITS-1;i>=0;i--)
	  printf("%c", DPMANT(x) & DP_MBIT(i)?'1':'0');
	printf("e%d", DPBEXP(x) - DP_EBIAS);
	break;
    }
    printf("\n");
    return x;
}

ieee754sp ieee754sp_dump(char *m,ieee754sp x)
{
    int c;
    int i;

    printf("%s=",m);
    printf("<%08x>\n", x);
    printf("\t=");
    switch(c = ieee754sp_class(x)) {
    default:
	abort();
#if ! IEEE754_NO_SNANS
    case IEEE754_CLASS_SNAN:
#endif
    case IEEE754_CLASS_QNAN:
	printf("%cNan %c", c == IEEE754_CLASS_QNAN ? 'Q' : 'S',
	       SPSIGN(x)?'-':'+');
	for(i=SP_FBITS-1;i>=0;i--)
	  printf("%c", SPMANT(x) & SP_MBIT(i)?'1':'0');
	break;
    case IEEE754_CLASS_INF:
	printf("%cInfinity", SPSIGN(x)?'-':'+');
	break;
    case IEEE754_CLASS_ZERO:
	printf("%cZero", SPSIGN(x)?'-':'+');
	break;
    case IEEE754_CLASS_DNORM:
	printf("%c0.",  SPSIGN(x)?'-':'+');
	for(i=SP_FBITS-1;i>=0;i--)
	  printf("%c", SPMANT(x) & SP_MBIT(i)?'1':'0');
	printf("e%d",SPBEXP(x) - SP_EBIAS);
	break;
    case IEEE754_CLASS_NORM:
	printf("%c1.",  SPSIGN(x)?'-':'+');
	for(i=SP_FBITS-1;i>=0;i--)
	  printf("%c", SPMANT(x) & SP_MBIT(i)?'1':'0');
	printf("e%d",SPBEXP(x) - SP_EBIAS);
	break;
    }
    printf("\n");
    return x;
}
