ieee754dp ieee754dp_format(int sn, int xe, uint64_t xm)
{
    assert(xm);	/* we dont gen exact zeros (probably should) */
    
    assert((xm >> (DP_MBITS+1+DP_GUARDBITS)) == 0);	/* no excess */
//    assert(xm & (DP_HIDDEN_BIT<<DP_GUARDBITS));
    
    xe += DP_EBIAS;

    if (UNLIKELY (xe < DP_EBIAS + DP_EMIN)) {
	/* strip lower bits */
	int es;
	
	if (IEEE754_NO_DENORMS ()) {
	    SETCX(IEEE754_UNDERFLOW);
	    return ieee754dp_zero(sn);
	}

	/* sticky right shift es bits */
	es = DP_EBIAS + DP_EMIN - xe;
	xm = XDPSRS(xm,es);
	xe += es;

	assert((xm & (DP_HIDDEN_BIT<<DP_GUARDBITS)) == 0);
	assert(xe == DP_EBIAS + DP_EMIN);
    }

    if ((uint32_t)xm & ((1 << DP_GUARDBITS) - 1)) {
	SETCX(IEEE754_INEXACT);
	/* inexact, must round off guard bits */
	switch(IEEE754_GET_RM ()) {
	case IEEE754_RZ:
	    break;
	case IEEE754_RN:
	    xm += ((0x1 << (DP_GUARDBITS - 1)) - 1)
		+ (((uint32_t)xm >> DP_GUARDBITS) & 1);
	    break;
	case IEEE754_RU:	/* toward +Infinity */
	    if(!sn)
	      xm+=(0x1 << DP_GUARDBITS);
	    break;
	case IEEE754_RD:	/* toward -Infinity */
	    if(sn) 
	      xm+=(0x1 << DP_GUARDBITS);
	    break;
	}
	/* adjust exponent for rounding overflow */
	if(xm & DP_MBIT(DP_MBITS+DP_GUARDBITS+1))
	  {xm >>=1; xe++;}
    } 

    /* strip grs bits */
    xm >>= DP_GUARDBITS;
    
    assert((xm >> (DP_MBITS+1)) == 0);	/* no excess */
    assert(xe >= DP_EBIAS + DP_EMIN - 1);
    
    if (UNLIKELY (xe > DP_EBIAS + DP_EMAX)) {
	SETCX(IEEE754_OVERFLOW);
	/* -O can be table indexed by (rm,sn) */
	switch(IEEE754_GET_RM ()) {
	case IEEE754_RN:
	    return ieee754dp_inf(sn);
	case IEEE754_RZ:
	    return ieee754dp_max(sn);
	case IEEE754_RU:	/* toward +Infinity */
	    if(sn == 0)
	      return ieee754dp_inf(0);
	    else
	      return ieee754dp_max(1);
	case IEEE754_RD:	/* toward -Infinity */
	    if(sn == 0)
	      return ieee754dp_max(0);
	    else
	      return ieee754dp_inf(1);
	}
    }

    /* gen norm/denorm/zero */
    if (UNLIKELY ((xm & DP_HIDDEN_BIT) == 0)) {
	/* we underflow (tiny/zero) */
	SETCX(IEEE754_UNDERFLOW);
	assert(xe == DP_EBIAS + DP_EMIN);
	xe = DP_EBIAS + DP_EMIN - 1;
    } 
    else
	xm &= DP_MMASK;

    return builddp (sn, xe, xm);
}
