#ident "lib/ieee/ieee754sp.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/* IEEE754 floating point arithmetic
 * single precision
 */
/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <limits.h>
#include <assert.h>
#include <stdarg.h>

#include "ieee754sp.h"

int ieee754sp_class(ieee754sp x)
{
    COMPXSP;
    EXPLODEXSP;
    return xc;
}

#if ! IEEE754_NO_XCPT
ieee754sp ieee754sp_xcpt(ieee754sp r,const char *op, ...)
{
    struct ieee754xctx ax;
    
    if(!TSTX()) 
      return r;
    
    ax.op = op;
    ax.rt = IEEE754_RT_SP;
    ax.rv.sp = r;
    va_start(ax.ap,op);
    ieee754_xcpt(&ax);
    return ax.rv.sp;
}

ieee754sp ieee754sp_nanxcpt(ieee754sp r,const char *op, ...)
{
    struct ieee754xctx ax;

    assert(ieee754sp_isnan(r));
    
    if(!ieee754sp_issnan(r))	/* QNAN does not cause invalid op !! */
      return r;
    
    if(!SETCX(IEEE754_INVALID_OPERATION)) {
	/* not enabled convert to a quiet NaN */
	if (IEEE754_NO_QNANS ())
	  return r;
	r &= ~SP_MBIT(SP_MBITS-1);
	return r;
    }
    
    ax.op = op;
    ax.rt = 0;
    ax.rv.sp = r;
    va_start(ax.ap,op);
    ieee754_xcpt(&ax);
    return ax.rv.sp;
}
#endif

#if ! (IEEE754_NO_FLAGS && IEEE754_FIXED_ROUNDING)
#include "ieee754spfmt.c"
#endif
