/* generate a normal/denormal number with over,under handeling
 * sn is sign
 * xe is an unbiased exponent
 * xm is 3bit extended precision value.
 */
ieee754sp ieee754sp_format(int sn, int xe, uint32_t xm)
{
    assert(xm);	/* we dont gen exact zeros (probably should) */
    
    assert((xm >> (SP_MBITS+1+SP_GUARDBITS)) == 0);	/* no excess */
//    assert(xm & (SP_HIDDEN_BIT<<SP_GUARDBITS));
    
    xe += SP_EBIAS;

    if (UNLIKELY (xe < SP_EBIAS + SP_EMIN)) {
	/* strip lower bits */
	int es;
	
	if (IEEE754_NO_DENORMS ()) {
	    SETCX(IEEE754_UNDERFLOW);
	    return ieee754sp_zero(sn);
	}
	
	/* sticky right shift es bits 
	 */
	es = SP_EBIAS + SP_EMIN - xe;
	SPXSRSXn(es);
	
	assert((xm & (SP_HIDDEN_BIT<<SP_GUARDBITS)) == 0);
	assert(xe == SP_EBIAS + SP_EMIN);
    }

    if (xm & (SP_MBIT(SP_GUARDBITS)-1)) {
	SETCX(IEEE754_INEXACT);
	/* inexact must round off guard bits */
	switch(IEEE754_GET_RM ()) {
	case IEEE754_RZ:
	    break;
	case IEEE754_RN:
	    xm += ((0x1 << (SP_GUARDBITS - 1)) - 1)
		+ ((xm >> SP_GUARDBITS) & 1);
	    break;
	case IEEE754_RU:	/* toward +Infinity */
	    if(!sn)
	      xm+=(0x1 << SP_GUARDBITS);
	    break;
	case IEEE754_RD:	/* toward -Infinity */
	    if(sn)
	      xm+=(0x1 << SP_GUARDBITS);
	    break;
	}
	/* adjust exponent for rounding overflow */
	if(xm >> (SP_MBITS+1+SP_GUARDBITS))
	  {xm >>=1; xe++;}
    } 

    /* strip grs bits */
    xm >>= SP_GUARDBITS;

    assert((xm >> (SP_MBITS+1)) == 0);	/* no excess */
    assert(xe >= SP_EBIAS + SP_EMIN);
    
    if (UNLIKELY (xe > SP_EBIAS + SP_EMAX)) {
	SETCX(IEEE754_OVERFLOW);
	/* -O can be table indexed by (rm,sn) */
	switch(IEEE754_GET_RM ()) {
	case IEEE754_RN:
	    return ieee754sp_inf(sn);
	case IEEE754_RZ:
	    return ieee754sp_max(sn);
	case IEEE754_RU:	/* toward +Infinity */
	    if(sn == 0)
	      return ieee754sp_inf(0);
	    else
	      return ieee754sp_max(1);
	case IEEE754_RD:	/* toward -Infinity */
	    if(sn == 0)
	      return ieee754sp_max(0);
	    else
	      return ieee754sp_inf(1);
	}
    }

    /* gen norm/denorm/zero */
    if (UNLIKELY ((xm & SP_HIDDEN_BIT) == 0)) {
	/* we underflow (tiny/zero) */
	SETCX(IEEE754_UNDERFLOW);
	assert(xe == SP_EBIAS + SP_EMIN);
	xe = SP_EBIAS + SP_EMIN - 1;
    } 
    else 
	xm &= SP_MMASK;

    return buildsp (sn, xe, xm);
}
