#ident "lib/ieee/ieeetst.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <assert.h>
#include <signal.h>
#include <setjmp.h>
#include "ieee754.h"

typedef ieee754sp f32;
typedef ieee754dp f64;

jmp_buf jb;

void fpfault(int s)
{
    longjmp(jb,1);
}

#define SPEQ(x,y)	(x.bits == y.bits)
#define DPEQ(x,y)	(x.bits == y.bits)

int main()
{
    long long int iter;

    printf("sizeof(sp)=%d\n", sizeof(ieee754sp));
    printf("sizeof(dp)=%d\n", sizeof(ieee754dp));
    
#ifdef notdef
    srandom(time(0));
#endif

    ieee754sp_dump("positive sp infinity", ieee754sp_inf(0));
    ieee754sp_dump("negative sp infinity", ieee754sp_inf(1));
    ieee754sp_dump("positive sp zero", ieee754sp_zero(0));
    ieee754sp_dump("negative sp zero", ieee754sp_zero(1));
    ieee754sp_dump("positive sp one", ieee754sp_one(0));
    ieee754sp_dump("negative sp one", ieee754sp_one(1));
    
    assert(SPEQ(ieee754sp_neg(ieee754sp_one(0)),ieee754sp_one(1)));
    assert(SPEQ(ieee754sp_neg(ieee754sp_one(1)),ieee754sp_one(0)));
    assert(SPEQ(ieee754sp_abs(ieee754sp_one(0)),ieee754sp_one(0)));
    assert(SPEQ(ieee754sp_abs(ieee754sp_one(1)),ieee754sp_one(0)));
    assert(SPEQ(ieee754sp_fint(1),ieee754sp_one(0)));
    assert(SPEQ(ieee754sp_fint(-1),ieee754sp_one(1)));
    
    ieee754sp_dump("fldexpf(1,24)",
		       ieee754sp_nldexp(ieee754sp_one(0),24));
    ieee754sp_dump("fldexpf(1,53)",
		       ieee754sp_nldexp(ieee754sp_one(0),53));
    ieee754sp_dump("(float)10",ieee754sp_fint(10));
    ieee754sp_dump("(float)-10",ieee754sp_fint(-10));
    ieee754sp_dump("1.0e31",ieee754sp_nldexp(ieee754sp_one(0),31));
    
    {
	ieee754sp fx;
	ieee754sp fi;
	ieee754sp ff;

	fx = ieee754sp_one(0);
	ff = ieee754sp_nmodf(fx,&fi);
	assert(SPEQ(ff,ieee754sp_zero(0)));
	assert(SPEQ(fi,ieee754sp_one(0)));
    }

    ieee754dp_dump("positive dp infinity", ieee754dp_inf(0));
    ieee754dp_dump("negative dp infinity", ieee754dp_inf(1));
    ieee754dp_dump("positive dp zero", ieee754dp_zero(0));
    ieee754dp_dump("negative dp zero", ieee754dp_zero(1));
    ieee754dp_dump("positive dp one", ieee754dp_one(0));
    ieee754dp_dump("negative dp one", ieee754dp_one(1));

    ieee754dp_dump("(double)1",ieee754dp_fint(1));
    ieee754dp_dump("(double)-1",ieee754dp_fint(-1));
    ieee754dp_dump("(double)10",ieee754dp_fint(10));
    ieee754dp_dump("(double)-10",ieee754dp_fint(-10));
    ieee754dp_dump("fneg(1.0)",
		       ieee754dp_neg(ieee754dp_one(0)));
    ieee754dp_dump("fneg(-1.0)",
		       ieee754dp_neg(ieee754dp_one(1)));
    ieee754dp_dump("fabs(1.0)",
		       ieee754dp_abs(ieee754dp_one(0)));
    ieee754dp_dump("fabs(-1.0)",
		       ieee754dp_abs(ieee754dp_one(1)));
    ieee754dp_dump("fldexp(1,24)",
		       ieee754dp_nldexp(ieee754dp_one(0),24));
    ieee754dp_dump("fldexp(1,53)",
		       ieee754dp_nldexp(ieee754dp_one(0),53));
    ieee754dp_dump("1.0e31",ieee754dp_nldexp(ieee754dp_one(0),31));
    
#define MAXLINE	4096
    {
	char line[MAXLINE];
	while(fgets(line,MAXLINE,stdin)) {
	    ieee754dp_dump(line,ieee754dp_nfstr(line,0));
	}
    }

#ifdef ns32000
    ieee754_csr.mx |= IEEE754_OVERFLOW;
    /* ieee754_csr.mx |= IEEE754_UNDERFLOW; */
    ieee754_csr.mx |= IEEE754_ZERO_DIVIDE;
    ieee754_csr.mx |= IEEE754_INVALID_OPERATION;
    ieee754_csr.nod = 1;
#endif
    ieee754_csr.noq = 1;	/* dont convert snan -> qnan */

    signal(SIGFPE,fpfault);

    
    for(iter=0;1;iter++) {
	int ix,iy,Ix,Iy;
	f32 fx,fy;
	f32 fr;
	f32 Fr;
	f64 dr;
	f64 Dr;
	
	if( (iter & 0x3ff) == 0 ) {
	    printf("Iter %08x\r", (unsigned)(iter>>10)); fflush(stdout);
	}
	
	*(int *)&fx = random() * random();
	*(int *)&fy = random() * random();
	
	if(ieee754_csr.nod && (ieee754sp_class(fx) == IEEE754_CLASS_DNORM ||
			       ieee754sp_class(fy) == IEEE754_CLASS_DNORM))
	  continue;
	      
	switch(ieee754sp_class(fx)) {
	case IEEE754_CLASS_QNAN:
	case IEEE754_CLASS_SNAN:
	    assert(!ieee754sp_eq(fx,fx) );
	    break;
	default:
	    /* for any !Nan x */
	    assert(ieee754sp_eq(fx,fx) );
	    assert(ieee754sp_eq(ieee754sp_add(fx,ieee754sp_zero(0)),fx));
	    assert(ieee754sp_eq(ieee754sp_add(fx,ieee754sp_zero(1)),fx));
	    assert(ieee754sp_eq(ieee754sp_mul(fx,ieee754sp_one(0)),fx));
	    {
		ieee754sp fi;
		ieee754sp ff;
	    
		ff = ieee754sp_nmodf(fx,&fi);
		assert(SPEQ(fx,ieee754sp_add(fi,ff)));
	    }
	    if(ieee754sp_finite(fx)) {
		ieee754sp fr1;
		ieee754sp fr2;
		int exp;
		fr1 = ieee754sp_nfrexp(fx,&exp);
		fr2 = ieee754sp_nldexp(fr1,exp);
		assert(SPEQ(fx,fr2));
		if(!ieee754sp_cmp(fx,ieee754sp_zero(0),
				  IEEE754_CEQ|IEEE754_CUN)) {
		    ieee754sp fa = ieee754sp_abs(fr1);
		    assert(ieee754sp_lt(fa,ieee754sp_one(0)));
		    assert(ieee754sp_ge(fa,
					ieee754sp_nldexp(ieee754sp_one(0),-1)));
		}
	    }
		   
	    break;
	}
		   
	/* single precision op must give result of truncated double precion op
	 */
	{
	    ieee754dp	dn = ieee754dp_fsp(fx);
	    ieee754sp	sn = ieee754sp_fdp(dn);
	    assert(SPEQ(fx,sn));
	}
	assert(SPEQ(fx,ieee754sp_fdp(ieee754dp_fsp(fx))));
	
	assert(ieee754sp_fix(fx) == ieee754dp_fix(ieee754dp_fsp(fx)));
	
	assert(SPEQ(ieee754sp_neg(fx),
			    ieee754sp_fdp(ieee754dp_neg(ieee754dp_fsp(fx)))));
	
	assert(SPEQ(ieee754sp_abs(fx),
			    ieee754sp_fdp(ieee754dp_abs(ieee754dp_fsp(fx)))));

	assert(SPEQ(ieee754sp_add(fx,fy),
			    ieee754sp_fdp(ieee754dp_add(
							ieee754dp_fsp(fx),
							ieee754dp_fsp(fy)
							))));
	assert(SPEQ(ieee754sp_sub(fx,fy),
			    ieee754sp_fdp(ieee754dp_sub(
							ieee754dp_fsp(fx),
							ieee754dp_fsp(fy)
							))));
	assert(SPEQ(ieee754sp_div(fx,fy),
			    ieee754sp_fdp(ieee754dp_div(
							ieee754dp_fsp(fx),
							ieee754dp_fsp(fy)
							))));
	assert(SPEQ(ieee754sp_mul(fx,fy),
			    ieee754sp_fdp(ieee754dp_mul(
							ieee754dp_fsp(fx),
							ieee754dp_fsp(fy)
							))));
			     
#ifdef CANCHECK
	fr = ieee754sp_add(fx,fy);
	if(setjmp(jb)) {
	    if( (ieee754_csr.cx & ieee754_csr.mx) == 0)
	      printf("Add failure\n");
	} else {
	    *(float *)&Fr = *(float *)&fx + *(float *)&fy;

	    if(!SPEQ(Fr,fr)) {
		f64 Dx,Dy,Dr;
		*(double *)&Dx = *(float *)&fx;
		*(double *)&Dy = *(float *)&fy;
		*(double *)&Dr = *(double *)&Dx + *(double *)&Dy;
	    
		printf("Add failure\n");
		ieee754sp_dump("fx",fx);
		ieee754dp_dump("Dx",Dx);
		ieee754sp_dump("fy",fy);
		ieee754dp_dump("Dy",Dy);
		ieee754sp_dump("fr",fr);
		ieee754sp_dump("Fr",Fr);
		ieee754dp_dump("Dr",Dr);
	    }
	}
#endif
	
#ifdef CANCHECK
	fr = ieee754sp_sub(fx,fy);
	if(setjmp(jb)) {
	    if( (ieee754_csr.cx & ieee754_csr.mx) == 0)
	      printf("Sub failure\n");
	} else {
	    *(float *)&Fr = *(float *)&fx - *(float *)&fy;

	    if( !SPEQ(Fr,fr)) {
		f64 Dx,Dy;
		*(double *)&Dx = *(float *)&fx;
		*(double *)&Dy = *(float *)&fy;
		*(double *)&Dr = *(double *)&Dx - *(double *)&Dy;
	    
		printf("Sub failure\n");
		ieee754sp_dump("fx",fx);
		ieee754dp_dump("Dx",Dx);
		ieee754sp_dump("fy",fy);
		ieee754dp_dump("Dy",Dy);
		ieee754sp_dump("fr",fr);
		ieee754sp_dump("Fr",Fr);
		ieee754dp_dump("Dr",Dr);
	    }
	}
#endif

#ifdef CANCHECK
	fr = ieee754sp_mul(fx,fy);
	if(setjmp(jb)) {
	    if( (ieee754_csr.cx & ieee754_csr.mx) == 0)
	      printf("Mul failure\n");
	} else {
	    *(float *)&Fr = *(float *)&fx * *(float *)&fy;

	    if( !SPEQ(Fr,fr)) {
		f64 Dx,Dy;
		*(double *)&Dx = *(float *)&fx;
		*(double *)&Dy = *(float *)&fy;
		*(double *)&Dr = *(double *)&Dx * *(double *)&Dy;
	    
		printf("Mul failure\n");
		ieee754sp_dump("fx",fx);
		ieee754dp_dump("Dx",Dx);
		ieee754sp_dump("fy",fy);
		ieee754dp_dump("Dy",Dy);
		ieee754sp_dump("fr",fr);
		ieee754sp_dump("Fr",Fr);
		ieee754dp_dump("Dr",Dr);
	    }
	}
#endif

#ifdef CANCHECK
	fr = ieee754sp_div(fx,fy);
	if(setjmp(jb)) {
	    if( (ieee754_csr.cx & ieee754_csr.mx) == 0) {
		printf("Div failure\n");
		ieee754sp_dump("fx",fx);
		ieee754sp_dump("fy",fy);
	    }
	} else {
	    *(float *)&Fr = *(float *)&fx / *(float *)&fy;

	    if( !SPEQ(Fr,fr) ) {
		f64 Dx,Dy;
		*(double *)&Dx = *(float *)&fx;
		*(double *)&Dy = *(float *)&fy;
		*(double *)&Dr = *(double *)&Dx / *(double *)&Dy;
	    
		printf("Div failure\n");
		ieee754sp_dump("fx",fx);
		ieee754dp_dump("Dx",Dx);
		ieee754sp_dump("fy",fy);
		ieee754dp_dump("Dy",Dy);
		ieee754sp_dump("fr",fr);
		ieee754sp_dump("Fr",Fr);
		ieee754dp_dump("Dr",Dr);
	    }
	}
#endif

#ifdef CANCHECK
	dr = ieee754dp_fsp(fx);
	if(setjmp(jb)) {
	    if( (ieee754_csr.cx & ieee754_csr.mx) == 0) {
		printf("ieee754dp_fsp xcpt failure\n");
		ieee754sp_dump("fx",fx);
		ieee754dp_dump("dr",dr);
	    }
	} else {
	    *(double *)&Dr = *(float *)&fx;
	    if( !DPEQ(Dr,dr) ) {
		f64 Dx;
		*(double *)&Dx = *(float *)&fx;
		
		printf("ieee754dp_fsp failure\n");
		ieee754sp_dump("fx",fx);
		ieee754dp_dump("Dx",Dx);
		ieee754dp_dump("dr",dr);
		ieee754dp_dump("Dr",Dr);
	    }
	}
#endif	
#ifdef CANCHECK
	ix = ieee754sp_fix(fx);
	if(setjmp(jb)) {
	    if( (ieee754_csr.cx & ieee754_csr.mx) == 0) {
		printf("Fix32 xcpt failure\n");
		ieee754sp_dump("fx",fx);
		printf("ix %d\n", ix);
	    }
	} else {
	    Ix = *(float *)&fx;
	    if(ix != Ix) {
		printf("Fix32 failure\n");
		ieee754sp_dump("fx",fx);
		printf("ix %d\n", ix);
		printf("Ix %d\n", Ix);
	    }
	}
#endif
    }

    return 0;
}

