all: libe.a

include		$(ROOT)/../defines.mk

# Force expensive optimisation, but no inlining/unrolling
OPTIM	=-O3 -fno-inline-functions -fno-unroll-loops \
	 -foptimize-sibling-calls \
#	 -fno-reorder-blocks 

vpath	%.S	$(ROOT)
vpath	%.c	$(ROOT)

CFLAGS	+=$(VAR)
ASFLAGS	+=$(VAR)
FP	 =soft

LIB1OBJ=dp_fstr.o dp_tstr.o ieee754m.o ieee754d.o \
	ieee754dp.o ieee754sp.o ieee754.o ieee754xcpt.o \
	dp_frexp.o dp_modf.o dp_div.o dp_mul.o dp_add.o dp_fsp.o \
	dp_cmp.o dp_order.o dp_logb.o dp_scalb.o \
	dp_simple.o dp_abs.o dp_neg.o \
	dp_tint.o dp_tuns.o dp_tlong.o dp_tulong.o dp_fint.o dp_flong.o \
	sp_fstr.o sp_frexp.o sp_modf.o sp_div.o sp_mul.o sp_add.o sp_fdp.o \
	sp_cmp.o sp_order.o sp_logb.o sp_scalb.o \
	sp_simple.o sp_abs.o sp_neg.o \
	sp_tint.o sp_tuns.o sp_tlong.o sp_tulong.o sp_fint.o sp_flong.o \
	dp_sqrt.o sp_sqrt.o

LIB2OBJ = \
	_divdf3.o _muldf3.o _negdf2.o _absdf2.o _adddf3.o _subdf3.o \
	_fixdfsi.o _fixsfsi.o _fixunsdfsi.o _fixunssfsi.o \
	_floatsidf.o _floatsisf.o \
	_fixdfdi.o _fixsfdi.o _fixunsdfdi.o _fixunssfdi.o \
	_floatdidf.o _floatdisf.o \
	_truncdfsf2.o _extendsfdf2.o \
	_addsf3.o _negsf2.o _abssf2.o _subsf3.o _mulsf3.o _divsf3.o \
	_cmpdf2.o _eqdf2.o _nedf2.o _gtdf2.o _gedf2.o _ltdf2.o _ledf2.o \
	_cmpsf2.o _eqsf2.o _nesf2.o _gtsf2.o _gesf2.o _ltsf2.o _lesf2.o \
	_unorddf2.o _unordsf2.o \
	_sqrtdf2.o _sqrtsf2.o \
	_ldexp.o _ldexpf.o _modf.o _frexp.o _strtod.o _strtof.o \
	_nofloat.o

LIBDPCOBJ = dp_zero.o dp_one.o dp_ten.o dp_inf.o dp_qnan.o \
	    dp_snan.o dp_max.o dp_min.o dp_mind.o \
	    dp_1e31.o dp_1e63.o

LIBSPCOBJ = sp_zero.o sp_one.o sp_ten.o sp_inf.o sp_qnan.o \
	    sp_snan.o sp_max.o sp_min.o sp_mind.o \
	    sp_1e31.o sp_1e63.o

LIBOBJ	+=$(LIB1OBJ) $(LIB2OBJ) $(LIBDPCOBJ) $(LIBSPCOBJ)

libe.a: $(LIBOBJ)
	$(AR) rcs $@ $?

$(LIB2OBJ): gccfpe.c $(LIB1OBJ:.o=.c)
	$(CC) $(CFLAGS) -DL$(subst .o,,$@) -DFNNAME=_$(subst .o,,$@) -c $< -o $@

$(LIBDPCOBJ): ieeeconst.c
	$(CC) $(CFLAGS) -DTYPE=ieee754dp -DFNNAME=ieee754$(subst .o,,$@) -c $< -o $@

$(LIBSPCOBJ): ieeeconst.c
	$(CC) $(CFLAGS) -DTYPE=ieee754sp -DFNNAME=ieee754$(subst .o,,$@) -c $< -o $@


install: libe.a
	if [ -d $(DESTDIR)/$(SUBDIR) ]; then \
		rm -f $(DESTDIR)/$(SUBDIR)/libe.a; \
	else \
		mkdir -p $(DESTDIR)/$(SUBDIR); \
	fi
	cp -p libe.a $(DESTDIR)/$(SUBDIR)/libe.a

clean:
	rm -f libe.a *.o core*

$(LIB2OBJ): $(ROOT)/ieee754.h
$(LIBOBJ): $(ROOT)/ieee754.h $(ROOT)/ieee754int.h \
	$(ROOT)/ieee754dp.h $(ROOT)/ieee754sp.h \
	$(ROOT)/ieee754dpfmt.c $(ROOT)/ieee754spfmt.c
