#ident "lib/ieee/sp_add.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/* IEEE754 floating point arithmetic
 * single precision
 */
/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <limits.h>
#include <assert.h>
#include <stdarg.h>

#include "ieee754sp.h"

ieee754sp ieee754sp_add(ieee754sp x, ieee754sp y)
{
    COMPXSP;
    COMPYSP;
    int s;
    
    EXPLODEXSP;
    EXPLODEYSP;
    
    CLEARCX;

    if (! CLPAIR_NORM_P (cp)) {
#if IEEE754_CLASS_CASE
	switch(CLPAIR(xc,yc)) {
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_SNAN):
	    return ieee754sp_nanxcpt(ieee754sp_bestnan(x,y),"add",x,y);
	    
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_SNAN):
	    return ieee754sp_nanxcpt(y,"add",x,y);
	    
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_DNORM):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_INF):
	    return ieee754sp_nanxcpt(x,"add",x,y);
	    
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_QNAN):
	    return ieee754sp_bestnan(x,y);
	    
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_QNAN):
	    return y;
      
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_DNORM):
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_INF):
	    return x;
      
      /* Inifity handling */
      
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_INF):
	    if(xs == ys)
		return x;
	    SETCX(IEEE754_INVALID_OPERATION);
	    return ieee754sp_xcpt(ieee754sp_indef(),"add",x,y);
	    
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_INF):
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_INF):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_INF):
	    return y;
	    
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_DNORM):
	    return x;

      /* Zero handling */
      
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_ZERO):
	    if(xs == ys)
		return x;
	    return ieee754sp_zero(IEEE754_GET_RM () == IEEE754_RD);
	    
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_ZERO):
	    return x;
	    
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_DNORM):
	    return y;
	    
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_DNORM):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_DNORM):
	    /* no need to normalise denormals */
	    break;
	    
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_NORM):
	    break;
	}
#else
	if (LIKELY (cp == (IEEE754_CLASS_ZERO | IEEE754_CLASS_NORM))
	    || cp == (IEEE754_CLASS_ZERO | IEEE754_CLASS_DNORM)) {
	    if ((xc & yc & IEEE754_CLASS_ZERO) && xs != ys)
		/* Zero - Zero = +/-Zero */
		return ieee754dp_zero (IEEE754_GET_RM () == IEEE754_RD);
	    /* Zero + Y or X + Zero - return non-zero value */
	    return (xc == IEEE754_CLASS_ZERO) ? y : x;
	}

	if (cp & IEEE754_CLASS_SNAN) {
	    /* SNaN handling */
	    return ieee754sp_nanxcpt(ieee754sp_bestnan(x,y),"add",x,y);
	}
	
	if (cp & IEEE754_CLASS_QNAN) {
	    /* QNaN handling */
	    return ieee754sp_bestnan(x,y);
	}
      
	if (cp & IEEE754_CLASS_INF) {
	    /* Inf handling */
	    if (xc & yc) {
		/* Inf + Inf = Inf */
		if (xs == ys)
		    return x;
		/* Inf - Inf = Indef */
		SETCX(IEEE754_INVALID_OPERATION);
		return ieee754sp_xcpt(ieee754sp_indef(),"add",x,y);
	    }
	    /* Inf + Y or X + Inf - return the Inf */
	    return (xc == IEEE754_CLASS_INF) ? x : y;
	}
	    
	/* Denorms can be handled without normalisation */
#endif
    }

//    assert(xm & SP_HIDDEN_BIT);
//    assert(ym & SP_HIDDEN_BIT);

    /* provide guard,round and stick bit space */
    xm <<= SP_GUARDBITS;
    ym <<= SP_GUARDBITS;
    
    s = xe - ye;
    if (s > 0) {
	/* have to shift y fraction right to align */
	SPXSRSYn(s);
    } else if (s < 0) {
	/* have to shift x fraction right to align */
	SPXSRSXn(-s);
    }

    assert(xe == ye);
    assert(xe <= SP_EMAX);
    
    if(xs == ys) {
	/* generate 28 bit result of adding two 27 bit numbers
	 * leaving result in xm,xs,xe
	 */
	xm = xm + ym;
	if(xm >> (SP_MBITS+1+SP_GUARDBITS)) { 	/* carry out */
	    SPXSRSX1();
	}
    } else {
	xm = xm - ym;

	if(xm == 0)
	  return ieee754sp_zero(IEEE754_GET_RM () == IEEE754_RD);

	if ((int32_t)xm < 0) {
	    xm = -xm;
	    xs = ys;
	}

	/* normalize in extended single precision */
	SP_NORM (xm, xe, SP_GUARDBITS);
    }
    SPNORMRET2(xs,xe,xm,"add",x,y);
}

ieee754sp ieee754sp_sub(ieee754sp x, ieee754sp y)
{
    return ieee754sp_add (x, y ^ SP_SIGN_BIT);
}
