#ident "lib/ieee/sp_div.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/* IEEE754 floating point arithmetic
 * single precision
 */
/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <limits.h>
#include <assert.h>
#include <stdarg.h>

#include "ieee754sp.h"

ieee754sp ieee754sp_div(ieee754sp x, ieee754sp y)
{
    COMPXSP;
    COMPYSP;
    
    CLEARCX;

    EXPLODEXSP;
    EXPLODEYSP;

    if (! CLPAIR_NORM_P (cp)) {
#if IEEE754_CLASS_CASE
	switch(cp) {
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_SNAN):
	    return ieee754sp_nanxcpt(ieee754sp_bestnan(x,y),"div",x,y);
	    
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_SNAN):
	    return ieee754sp_nanxcpt(y,"div",x,y);
	    
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_DNORM):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_INF):
	    return ieee754sp_nanxcpt(x,"div",x,y);
	    
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_QNAN):
	    return ieee754sp_bestnan(x,y);
	    
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_QNAN):
	    return y;
      
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_DNORM):
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_INF):
	    return x;
      
      
      /* Infinity handling */
      
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_INF):
	    SETCX(IEEE754_INVALID_OPERATION);
	    return ieee754sp_xcpt(ieee754sp_indef(),"div",x,y);
	    
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_INF):
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_INF):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_INF):
	    return ieee754sp_zero(xs^ys);
	    
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_DNORM):
	    return ieee754sp_inf(xs^ys);

      /* Zero handling */
      
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_ZERO):
	    SETCX(IEEE754_INVALID_OPERATION);
	    return ieee754sp_xcpt(ieee754sp_indef(),"div",x,y);
	    
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_ZERO):
	    SETCX(IEEE754_ZERO_DIVIDE);
	    return ieee754sp_xcpt(ieee754sp_inf(xs^ys),"div",x,y);
	    
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_DNORM):
	    return ieee754sp_zero(xs^ys);
      
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_DNORM):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_NORM):
	    SPDNORMX;
	    if (yc != IEEE754_CLASS_DNORM)
		break;
	    /* fall through */

	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_DNORM):
	    SPDNORMY;
	    break;
	}
#else
	if (cp & IEEE754_CLASS_SNAN) {
	    /* SNaN handling */
	    return ieee754sp_nanxcpt(ieee754sp_bestnan(x,y),"div",x,y);
	}

	if (cp & IEEE754_CLASS_QNAN) {
	    /* QNaN handling */
	    return ieee754sp_bestnan(x,y);
	}
      
	if (cp & IEEE754_CLASS_INF) {
	    /* Inf handling */
	    if (xc & yc) {
		/* Inf / Inf = Indef */
		SETCX(IEEE754_INVALID_OPERATION);
		return ieee754sp_xcpt(ieee754sp_indef(),"div",x,y);
	    }
	    else if (yc == IEEE754_CLASS_INF)
		/* X / Inf  = Zero */
		return ieee754sp_zero(xs ^ ys);
	    else
		/* Inf / X = Inf */
		return ieee754sp_inf(xs ^ ys);
	}

	if (cp & IEEE754_CLASS_ZERO) {
	    /* Zero handling */
	    if (yc == IEEE754_CLASS_ZERO) {
		if (xc == IEEE754_CLASS_ZERO) {
		    /* Zero / Zero = Indef */
		    SETCX(IEEE754_INVALID_OPERATION);
		    return ieee754sp_xcpt(ieee754sp_indef(),"div",x,y);
		}
		/* X / Zero = Inf */
		SETCX(IEEE754_ZERO_DIVIDE);
		return ieee754sp_xcpt(ieee754sp_inf(xs^ys),"div",x,y);
	    }
	    /* Zero / X = Zero */
	    return ieee754sp_zero(xs^ys);
	}

	if (xc & IEEE754_CLASS_DNORM)
	    SPDNORMX;
	
	if (yc & IEEE754_CLASS_DNORM)
	    SPDNORMY;
#endif
    }

    assert(xm & SP_HIDDEN_BIT);
    assert(ym & SP_HIDDEN_BIT);

    /* provide rounding space */
    xm <<= SP_GUARDBITS;
    ym <<= SP_GUARDBITS;

    {
	/* now the dirty work */

	uint32_t rm = 0;
	uint32_t bm;
	int 	 re = xe - ye;
    
	for(bm= SP_MBIT(SP_MBITS+SP_GUARDBITS-1); bm; bm >>= 1) {
	    if(xm >= ym) {
		xm -= ym;
		rm |= bm;
		if(xm == 0)
		  break;
	    } 
	    xm <<= 1;
	}
	rm <<= 1;
	if(xm)
	  rm |= 1;      /* have remainder, set sticky */
	
	assert(rm);
	
	/* normalise rm to rounding precision ? */
	SP_NORM (rm, re, SP_GUARDBITS);

	SPNORMRET2(xs^ys,re,rm,"div",x,y);
    }
}
