#ident "lib/ieee/sp_fdp.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/* IEEE754 floating point arithmetic
 * single precision
 */
/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <limits.h>
#include <assert.h>
#include <stdarg.h>

#include "ieee754sp.h"
#include "ieee754dp.h"

ieee754sp ieee754sp_fdp(ieee754dp x)
{
    COMPXDP;
    
    CLEARCX;
    
    EXPLODEXDP;
    
    if (! CLASS_NORM_P (xc)) {
#if IEEE754_CLASS_CASE
	switch(xc) {
#if IEEE754_NO_STICKY_NANS
	case IEEE754_CLASS_QNAN:
	    return ieee754sp_qnan ();
	case IEEE754_CLASS_SNAN:
	    return ieee754sp_nanxcpt (ieee754sp_snan(), "fdp",x);
#else
	case IEEE754_CLASS_QNAN:
	case IEEE754_CLASS_SNAN:
	    return ieee754sp_nanxcpt(buildsp(xs,
					     SP_EMAX+1+SP_EBIAS,
					     (unsigned long)
					     (xm >> (DP_MBITS-SP_MBITS))),
				     "fdp",x);
#endif
	case IEEE754_CLASS_INF:
	    return ieee754sp_inf(xs);
	case IEEE754_CLASS_ZERO:
	    return ieee754sp_zero(xs);
	case IEEE754_CLASS_DNORM:
	    /* cant possibly be sp representable */
	    SETCX(IEEE754_UNDERFLOW);
	    return ieee754sp_xcpt(ieee754sp_zero(xs),"fdp",x);
	}
#else
        if (xc & IEEE754_CLASS_ZERO)
	    return ieee754sp_zero(xs);
#if IEEE754_NO_STICKY_NANS
	if (xc & IEEE754_CLASS_QNAN)
	    return ieee754sp_qnan (0);
	if (xc & IEEE754_CLASS_SNAN)
	    return ieee754sp_nanxcpt (ieee754sp_snan(), "fdp",x);
#else
	if (xc & (IEEE754_CLASS_QNAN | IEEE754_CLASS_SNAN))
	    return ieee754sp_nanxcpt(buildsp(xs,
					     SP_EMAX+1+SP_EBIAS,
					     (uint32_t)
					     (xm >> (DP_MBITS-SP_MBITS))),
				     "fdp",x);
#endif
	if (xc & IEEE754_CLASS_INF)
	    return ieee754sp_inf(xs);

	/* dp denorm can't possibly be sp representable */
	assert (xc & IEEE754_CLASS_DNORM);
	SETCX(IEEE754_UNDERFLOW);
	return ieee754sp_xcpt(ieee754sp_zero(xs),"fdp",x);
#endif
    }
    
    {
	/* convert from DP_MBITS to SP_MBITS+3 with sticky right shift */
	uint32_t rm = XDPSRS(xm, DP_MBITS-(SP_MBITS+SP_GUARDBITS));
	SPNORMRET1(xs,xe,rm,"fdp",x);
    }
}
