#ident "lib/ieee/sp_fint.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/* IEEE754 floating point arithmetic
 * single precision
 */
/*
 * Copyright (c) 1996-2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <limits.h>
#include <assert.h>
#include <stdarg.h>

#include "ieee754sp.h"

ieee754sp ieee754sp_fint(int32_t x)
{
    COMPXSP;

    CLEARCX;

    if(x == 0)
      return ieee754sp_zero(0);

    xs = (x < 0);
    if (x == 1 || x == -1)
      return ieee754sp_one(xs);      
    if (x == 10 || x == -10)
      return ieee754sp_ten(xs);
    
    if(xs) {
	if(x == (1<<31))
	  xm = (1U<<31); /* max neg can't be safely negated */
	else
	  xm = -x;
    } else {
	xm = x;
    }
    xe = SP_MBITS+SP_GUARDBITS;
    
    if (xm >> (SP_MBITS+1+SP_GUARDBITS)) {
	/* shunt out overflow bits */
	while(xm >> (SP_MBITS+1+SP_GUARDBITS)) {
	    SPXSRSX1();
	}
    } else {
	/* normalize in grs extended single precision */
	SP_NORM (xm, xe, SP_GUARDBITS);
    }
    SPNORMRET1(xs,xe,xm,"fint",x);
}

ieee754sp ieee754sp_funs(uint32_t u)
{
    if((int32_t)u < 0) 
      return ieee754sp_add(ieee754sp_1e31(0), 
			   ieee754sp_fint(clearmsb32 (u, 1)));
    return ieee754sp_fint(u);
}
