#ident "lib/ieee/sp_frexp.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/* IEEE754 floating point arithmetic
 * single precision
 */
/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <limits.h>
#include <assert.h>
#include <stdarg.h>

#include "ieee754sp.h"

/* close to ieeep754sp_logb 
*/
ieee754sp ieee754sp_nfrexp(ieee754sp x,int *eptr)
{
    COMPXSP;
    CLEARCX;
    EXPLODEXSP;

    if (! CLASS_NORM_P (xc)) {
#if IEEE754_CLASS_CASE
	switch(xc) {
	case IEEE754_CLASS_SNAN:
	case IEEE754_CLASS_QNAN:
	case IEEE754_CLASS_INF:
	case IEEE754_CLASS_ZERO:
	    *eptr = 0;
	    return x;
	case IEEE754_CLASS_DNORM:
	    SPDNORMX;
	    break;
	case IEEE754_CLASS_NORM:
	    break;
	}
#else
	if (xc & (IEEE754_CLASS_SNAN | IEEE754_CLASS_QNAN
		  | IEEE754_CLASS_INF | IEEE754_CLASS_ZERO)) {
	    *eptr = 0;
	    return x;
	}
	/* must be denormal */
	SPDNORMX;
#endif
    }

    *eptr = xe+1;
    return buildsp(xs,-1+SP_EBIAS,xm & ~SP_HIDDEN_BIT);
}

ieee754sp_u ieee754sp_frexp(ieee754sp_u x,int *eptr)
{
    ieee754sp_u r;
    r.bits = ieee754sp_nfrexp (x.bits, eptr);
    return r;
}
