#ident "lib/ieee/sp_modf.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/* IEEE754 floating point arithmetic
 * single precision
 */
/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <limits.h>
#include <assert.h>
#include <stdarg.h>

#include "ieee754sp.h"

/* modf function is always exact for a finite number
*/
ieee754sp ieee754sp_nmodf(ieee754sp x,ieee754sp *ip)
{
    COMPXSP;
    uint32_t tm;
    unsigned int bits;

    CLEARCX;
    
    EXPLODEXSP;

    if (! CLASS_NORM_P (xc)) {
#if IEEE754_CLASS_CASE
	switch(xc) {
	case IEEE754_CLASS_SNAN:
	case IEEE754_CLASS_QNAN:
	case IEEE754_CLASS_INF:
	case IEEE754_CLASS_ZERO:
	    *ip = x;
	    return x;
	case IEEE754_CLASS_DNORM:
	    /* far too small */
	    *ip = ieee754sp_zero(xs);
	    return x;
	}
#else
	if (xc & (IEEE754_CLASS_SNAN | IEEE754_CLASS_QNAN
		  | IEEE754_CLASS_INF | IEEE754_CLASS_ZERO)) {
	    *ip = x;
	    return x;
	}

	/* far too small */
	assert (xc & IEEE754_CLASS_DNORM);
	*ip = ieee754sp_zero(xs);
	return x;
#endif
    }

    if(xe < 0) {
	*ip = ieee754sp_zero(xs);
	return x;
    }

    if(xe >= SP_MBITS) {
	*ip = x;
	return ieee754sp_zero(xs);
    } 

    /* generate ipart mantissa by clearing bottom bits */
    bits = SP_MBITS - xe;
    tm = (xm >> bits) << bits;
    tm &= ~SP_HIDDEN_BIT;
    *ip = builddp(xs, xe + SP_EBIAS, tm);
    
    /* generate fpart mantissa by clearing top bits
     * and normalizing (must be able to normalize)
     */
    xm = (xm << (32 - bits)) >> (32 - bits);
    if(xm == 0) 
      return ieee754sp_zero(xs);
    
    SP_NORM_PACK (xm, xe, 0);
    return buildsp(xs, xe+SP_EBIAS, xm);
}

ieee754sp_u ieee754sp_modf(ieee754sp_u x, ieee754sp_u *ip)
{
    ieee754sp_u r;
    r.bits = ieee754sp_nmodf (x.bits, &ip->bits);
    return r;
}
