#ident "lib/ieee/sp_mul.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/* IEEE754 floating point arithmetic
 * single precision
 */
/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <limits.h>
#include <assert.h>
#include <stdarg.h>

#include "ieee754sp.h"

ieee754sp ieee754sp_mul(ieee754sp x, ieee754sp y)
{
    COMPXSP;
    COMPYSP;

    CLEARCX;

    EXPLODEXSP;
    EXPLODEYSP;
    
    if (! CLPAIR_NORM_P (cp)) {
#if IEEE754_CLASS_CASE
	switch(cp) {
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_SNAN):
	    return ieee754sp_nanxcpt(ieee754sp_bestnan(x,y),"mul",x,y);
      
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_SNAN):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_SNAN):
	    return ieee754sp_nanxcpt(y,"mul",x,y);
	    
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_DNORM):
	case CLPAIR(IEEE754_CLASS_SNAN,IEEE754_CLASS_INF):
	    return ieee754sp_nanxcpt(x,"mul",x,y);
	    
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_QNAN):
	    return ieee754sp_bestnan(x,y);
	    
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_QNAN):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_QNAN):
	    return y;
	    
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_DNORM):
	case CLPAIR(IEEE754_CLASS_QNAN,IEEE754_CLASS_INF):
	    return x;
      
      
      /* Infinity handling */

	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_INF):
	    SETCX(IEEE754_INVALID_OPERATION);
	    return ieee754sp_xcpt(ieee754sp_indef(),"mul",x,y);
	    
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_INF):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_INF):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_DNORM):
	case CLPAIR(IEEE754_CLASS_INF,IEEE754_CLASS_INF):
	    return ieee754sp_inf(xs ^ ys);
	    
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_NORM):
	case CLPAIR(IEEE754_CLASS_ZERO,IEEE754_CLASS_DNORM):
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_ZERO):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_ZERO):
	    return ieee754sp_zero(xs ^ ys);
	
	    
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_DNORM):
	case CLPAIR(IEEE754_CLASS_DNORM,IEEE754_CLASS_NORM):
	    SPDNORMX;
	    if (yc != IEEE754_CLASS_DNORM)
		break;
	    /* fall through */
      
	case CLPAIR(IEEE754_CLASS_NORM,IEEE754_CLASS_DNORM):
	    SPDNORMY;
	    break;
	}
#else
	if (LIKELY (cp == IEEE754_CLASS_ZERO))
	    /* Norm * Zero, or  Zero * Norm, or Zero * Zero */
	    return ieee754sp_zero(xs ^ ys);

	if (cp & IEEE754_CLASS_SNAN) {
	    /* SNaN handling */
	    return ieee754sp_nanxcpt(ieee754sp_bestnan(x,y),"mul",x,y);
	}

	if (cp & IEEE754_CLASS_QNAN) {
	    /* QNaN handling */
	    return ieee754sp_bestnan(x,y);
	}
      
	if (cp & IEEE754_CLASS_INF) {
	    /* Inf handling */
	    if (cp & IEEE754_CLASS_ZERO) {
		/* Inf * Zero */
		SETCX(IEEE754_INVALID_OPERATION);
		return ieee754sp_xcpt(ieee754sp_indef(),"mul",x,y);
	    }
	    return ieee754sp_inf(xs ^ ys);
	}

	assert (cp &  IEEE754_CLASS_DNORM);

	if (cp & IEEE754_CLASS_ZERO)
	    /* DNorm * Zero */
	    return ieee754sp_zero(xs ^ ys);

	if (xc & IEEE754_CLASS_DNORM)
	    SPDNORMX;
	
	if (yc & IEEE754_CLASS_DNORM)
	    SPDNORMY;
#endif
    }

    /* rm = xm * ym, re = xe+ye basicly */
    assert(xm & SP_HIDDEN_BIT);
    assert(ym & SP_HIDDEN_BIT);

    {
	int re = xe + ye;
	int rs = xs ^ ys;
	uint32_t rm;
	
	/* shunt to top of word */
	xm <<= 32-(SP_MBITS+1);
	ym <<= 32-(SP_MBITS+1);
    
	/* multiply 32bits xm x ym to give high 32bits rm with stickiness */
	{
	    uint64_t tm = xm * (uint64_t)ym;
	    rm = (uint32_t)(tm >> 32) | ((uint32_t)tm != 0);
	}

	/*
	 * sticky shift down to normal rounding precision
	 */
	if((int32_t)rm < 0) {
	    rm = XSPSRS (rm, 32-(SP_MBITS+1+SP_GUARDBITS));
	    re++;
	} else {
	    rm = XSPSRS (rm, 32-(SP_MBITS+1+SP_GUARDBITS+1));
	}
	assert(rm & (SP_HIDDEN_BIT << SP_GUARDBITS));
	SPNORMRET2(rs,re,rm,"mul",x,y);
    }
}
