#ident "lib/ieee/tstsqrt.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <float.h>
#include "ieee754.h"

extern int random();

extern double sqrt(double x);

double __sqrtdf2(double x)
{
    ieee754dp_u sx;
    sx.d = x;
    sx.bits = ieee754dp_nsqrt (sx.bits);
    return sx.d;
}

#define LOOPS 20000

main ()
{
    double ldexp();
    double mant, op, old, new, diff;
    double maxdiff, maxop, min, max;
    int exp;
    struct rusage start, end;
    double nt, ot;
    int i;

    srandom (time(0));
    maxdiff = 0.0;
    min = DBL_MAX;
    max = DBL_MIN;
    for (i = 1; i < LOOPS; i++) {
	op = ldexp ((double)random(), (random() % 2048) - 1025);
	if (op > max)
	  max = op;
	if (op < min)
	  min = op;
	old = sqrt(op);
	new = __sqrtdf2(op);
	diff = old - new;
	if (diff < 0)
	  diff = -diff;
	if (diff > maxdiff) {
	    maxdiff = diff;
	    maxop = op;
	}
    }

    if (maxdiff != 0.0)
      printf ("maxdiff=%e: sqrt(%e) => %e\n", maxdiff, maxop, __sqrtdf2(maxop));
    else
      printf ("exactly matching result\n");
    printf ("range covered: %g - %g\n", min, max);
    srandom (100);
    getrusage (RUSAGE_SELF, &start);
    for (i = 1; i < LOOPS; i++) {
	op = ldexp ((double)random(), (random() % 1900) - 1024);
	old = sqrt(op);
    }
    getrusage (RUSAGE_SELF, &end);
    ot = (double)(end.ru_utime.tv_sec
		 - start.ru_utime.tv_sec)
      + (double)(end.ru_utime.tv_usec
		 - start.ru_utime.tv_usec) * 1e-6;
    printf ("old  = %f secs/call\n", ot / LOOPS);

    srandom (100);
    getrusage (RUSAGE_SELF, &start);
    for (i = 1; i < LOOPS; i++) {
	op = ldexp ((double)random(), (random() % 1900) - 1024);
	old = __sqrtdf2(op);
    }
    getrusage (RUSAGE_SELF, &end);
    nt = (double)(end.ru_utime.tv_sec
		 - start.ru_utime.tv_sec)
      + (double)(end.ru_utime.tv_usec
		 - start.ru_utime.tv_usec) * 1e-6;
    printf ("new  = %f secs/call\n", nt / LOOPS);

    if (ot > nt)
      printf ("speed up = %.1f X\n", ot / nt);
    else
      printf ("slow down = %.1f X\n", nt / ot);
    exit (0);

}

