#ident "lib/math/mips/test.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * test.c: test sqrt performance
 */

	
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>

extern int random();

extern double sqrt(double x);
extern double newsqrt(double x);

#define LOOPS 100000

main ()
{
    double op, old, new, diff;
    double maxdiff, maxop;
    struct rusage start, end;
    double nt, ot;
    int i;

    srandom (100);
    maxdiff = 0.0;
    for (i = 1; i < LOOPS; i++) {
	op = (double)random() * 1.234e-5;
	old = sqrt(op);
	new = newsqrt(op);
	diff = old - new;
	if (diff < 0)
	  diff = -diff;
	if (diff > maxdiff) {
	    maxdiff = diff;
	    maxop = op;
	}
    }

    if (maxdiff != 0.0)
      printf ("maxdiff=%e: sqrt(%e) => %e\n", maxdiff, maxop, newsqrt(maxop));
    else
      printf ("exactly matching result\n");

    srandom (100);
    getrusage (RUSAGE_SELF, &start);
    for (i = 1; i < LOOPS; i++) {
	op = (double)random() * 1.234e-5;
	old = sqrt(op);
    }
    getrusage (RUSAGE_SELF, &end);
    ot = (double)(end.ru_utime.tv_sec
		 - start.ru_utime.tv_sec)
      + (double)(end.ru_utime.tv_usec
		 - start.ru_utime.tv_usec) * 1e-6;
    printf ("old  = %f secs/call\n", ot / LOOPS);

    srandom (100);
    getrusage (RUSAGE_SELF, &start);
    for (i = 1; i < LOOPS; i++) {
	op = (double)random() * 1.234e-5;
	old = newsqrt(op);
    }
    getrusage (RUSAGE_SELF, &end);
    nt = (double)(end.ru_utime.tv_sec
		 - start.ru_utime.tv_sec)
      + (double)(end.ru_utime.tv_usec
		 - start.ru_utime.tv_usec) * 1e-6;
    printf ("new  = %f secs/call\n", nt / LOOPS);

    printf ("speed up = %.1f X\n", ot / nt);
    exit (0);

}

