/*
 * Copyright (c) 1983, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#if defined(LIBC_SCCS) && !defined(lint)
static char rcsid[] = "$OpenBSD: getprotoent.c,v 1.4 1999/09/03 16:23:18 millert Exp $";
#endif /* LIBC_SCCS and not lint */

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "thread_private.h"

#define	MAXALIASES	35

#if __AlgNET__
/* Builtin table of common protocols in case there's no file system */
static const char protocols[] =
 	"ip	0	IP\n"
	"icmp	1	ICMP\n"
	"igmp	2	IGMP\n"
	"ggp	3	GGP\n"
	"ipencap 4	IP-ENCAP\n"
	"tcp	6	TCP\n"
	"egp	8	EGP\n"
	"igp	9	IGP\n"
	"udp	17	UDP\n"
	"mux	18	MUX\n"
	"ipv6	41	IPv6\n"
	"gre	47	GRE\n"
	"mhrp	48	MHRP\n"
	"esp 	50	IPSEC-ESP\n"
	"ah	51	IPSEC-AH\n"
	"swipe	53	SWIPE\n"
	"mobileip 55	MOBILEIP\n"
	"skip	57	SKIP\n"
	"ipv6-icmp 58	IPv6-ICMP\n"
	"ipv6-nonxt 59	IPv6-NoNxt\n"
	"ipv6-opts 60	IPv6-Opts\n"
	"iso-ip	80	ISO-IP\n"
	"ospf	89	OSPFIGP\n"
	"mtp	92	MTP\n"
	"ipip	94	IPIP\n"
	"micp	95	MICP\n"
	"etherip 97	ETHERIP\n"
	"encap	98	ENCAP\n"
	"gmtp	100	GMTP\n"
	"pim	103	PIM\n"
	"ipcomp	108	IPComp\n"
	"vrrp	112	VRRP\n";
#endif

struct protodata {
    FILE	*protof;
    char 	line[BUFSIZ+1];
#if __AlgNET__
    const char	*protos;
#endif
    struct protoent proto;
    char 	*aliases[MAXALIASES];
    int 	stayopen;
};

_THREAD_PRIVATE_KEY(getprotoent);
static struct protodata pdata;

void
setprotoent(f)
	int f;
{
	struct protodata *pd = _THREAD_PRIVATE(getprotoent, pdata, NULL);
	if (!pd)
	    return;
	if (pd->protof == NULL)
		pd->protof = fopen(_PATH_PROTOCOLS, "r" );
	else
		rewind(pd->protof);
#if __AlgNET__
	if (pd->protof == NULL)
	    pd->protos = protocols;
#endif
	if (f > 0)
	    pd->stayopen |= f;
}

void
endprotoent()
{
	struct protodata *pd = _THREAD_PRIVATE(getprotoent, pdata, NULL);
	if (!pd)
	    return;
	if (pd->protof) {
		fclose(pd->protof);
		pd->protof = NULL;
	}
#if __AlgNET__
	pd->protos = NULL;
#endif
	pd->stayopen = 0;
}

void
_endprotoent()
{
	struct protodata *pd = _THREAD_PRIVATE(getprotoent, pdata, NULL);
	if (pd && !pd->stayopen)
	    endprotoent ();
}

struct protoent *
getprotoent()
{
	struct protodata *pd = _THREAD_PRIVATE(getprotoent, pdata, NULL);
	char *p, *cp, **q, *endp;
	long l;
	size_t len;

	if (!pd)
	    return (NULL);

	if (pd->protof == NULL 
#if __AlgNET__
	    && pd->protos == NULL
#endif
	    )
	    setprotoent (0);

	if (pd->protof == NULL 
#if __AlgNET__
	    && pd->protos == NULL
#endif
	    )
	    return (NULL);
	    
again:
#if __AlgNET__
	if (pd->protos) {
	    p = (char *) pd->protos;
	    if (!*p)
		return (NULL);
	    pd->protos = strchr (p, '\n') + 1;
	    len = pd->protos - p;
	}
	else
#endif
	if ((p = fgetln(pd->protof, &len)) == NULL)
		return (NULL);
	if (p[len-1] == '\n')
		len--;
	if (len >= sizeof(pd->line) || len == 0)
		goto again;
	p = memcpy(pd->line, p, len);
	pd->line[len] = '\0';
	if (*p == '#')
		goto again;
	if ((cp = strchr(p, '#')) != NULL)
		*cp = '\0';
	pd->proto.p_name = p;
	cp = strpbrk(p, " \t");
	if (cp == NULL)
		goto again;
	*cp++ = '\0';
	while (*cp == ' ' || *cp == '\t')
		cp++;
	p = strpbrk(cp, " \t");
	if (p != NULL)
		*p++ = '\0';
	l = strtol(cp, &endp, 10);
	if (endp == cp || *endp != '\0' || l < 0 || l >= INT_MAX)
		goto again;
	pd->proto.p_proto = l;
	q = pd->proto.p_aliases = pd->aliases;
	if (p != NULL) {
		cp = p;
		while (cp && *cp) {
			if (*cp == ' ' || *cp == '\t') {
				cp++;
				continue;
			}
			if (q < &pd->aliases[MAXALIASES - 1])
				*q++ = cp;
			cp = strpbrk(cp, " \t");
			if (cp != NULL)
				*cp++ = '\0';
		}
	}
	*q = NULL;
	return (&pd->proto);
}
