/*
 * Copyright (c) 1983, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#if defined(LIBC_SCCS) && !defined(lint)
static char rcsid[] = "$OpenBSD: getservent.c,v 1.5 1999/09/03 16:23:19 millert Exp $";
#endif /* LIBC_SCCS and not lint */

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "thread_private.h"

#define	MAXALIASES	35

#if __AlgNET__
/* Builtin table of common services in case there's no file system */
static const char services[] = 
	"tcpmux		1/tcp\n"
	"echo		7/tcp\n"
	"echo		7/udp\n"
	"discard	9/tcp		sink null\n"
	"discard	9/udp		sink null\n"
	"systat		11/tcp		users\n"
	"daytime	13/tcp\n"
	"daytime	13/udp\n"
	"netstat	15/tcp\n"
	"chargen	19/tcp		ttytst source\n"
	"chargen	19/udp		ttytst source\n"
	"ftp-data	20/tcp\n"
	"ftp		21/tcp\n"
	"telnet		23/tcp\n"
	"smtp		25/tcp		mail\n"
	"time		37/tcp		timserver\n"
	"time		37/udp		timserver\n"
	"whois		43/tcp		nicname\n"
	"domain		53/tcp		nameserver\n"
	"domain		53/udp		nameserver\n"
	"tftp		69/udp\n"
	"finger		79/tcp\n"
	"www		80/tcp		http\n"
	"www		80/udp\n"
	"pop2		109/tcp		pop-2	postoffice\n"
	"pop2		109/udp		pop-2\n"
	"pop3		110/tcp		pop-3\n"
	"pop3		110/udp		pop-3\n"
	"sunrpc		111/tcp		portmapper\n"
	"sunrpc		111/udp		portmapper\n"
	"auth		113/tcp		authentication tap ident\n"
	"nntp		119/tcp		readnews untp\n"
	"ntp		123/tcp\n"
	"ntp		123/udp\n"
	"snmp		161/udp\n"
	"snmp-trap	162/udp		snmptrap\n"
	"bgp		179/tcp\n"
	"bgp		179/udp\n"
	"smux		199/tcp\n"
	"smux		199/udp\n"
	"imap3		220/tcp\n"
	"imap3		220/udp\n"
	"https		443/tcp\n"
	"https		443/udp\n"
	"exec		512/tcp\n"
	"biff		512/udp		comsat\n"
	"login		513/tcp\n"
	"who		513/udp		whod\n"
	"shell		514/tcp		cmd\n"
	"syslog		514/udp\n"
	"printer	515/tcp		spooler\n"
	"route		520/udp		router routed\n"
	"timed		525/udp		timeserver\n"
	"gdbremote	2159/udp	gdb\n"
	"gdbremote	2159/tcp	gdb\n";
#endif

struct servdata {
    FILE 	*servf;
#if __AlgNET__
    const char	*servs;
#endif
    char	line[BUFSIZ+1];
    struct servent serv;
    char	*aliases[MAXALIASES];
    int		stayopen;
};

_THREAD_PRIVATE_KEY(getservent);
static struct servdata sdata;

void
setservent(f)
	int f;
{
	struct servdata *sd = _THREAD_PRIVATE(getservent, sdata, NULL);
	if (!sd)
	    return;
	if (sd->servf == NULL)
		sd->servf = fopen(_PATH_SERVICES, "r" );
	else
		rewind(sd->servf);
#if __AlgNET__
	if (sd->servf == NULL)
	    sd->servs = services;
#endif
	if (f > 0)
	    sd->stayopen |= f;
}

void
endservent()
{
	struct servdata *sd = _THREAD_PRIVATE(getservent, sdata, NULL);
	if (!sd)
	    return;
	if (sd->servf) {
		fclose(sd->servf);
		sd->servf = NULL;
	}
#if __AlgNET__
	sd->servs = NULL;
#endif
	sd->stayopen = 0;
}

void
_endservent()
{
	struct servdata *sd = _THREAD_PRIVATE(getservent, sdata, NULL);
	if (sd && !sd->stayopen)
	    endservent ();
}


struct servent *
getservent()
{
	struct servdata *sd = _THREAD_PRIVATE(getservent, sdata, NULL);
	char *p, *cp, **q, *endp;
	long l;
	size_t len;

	if (!sd)
	    return (NULL);

	if (sd->servf == NULL 
#if __AlgNET__
	    && sd->servs == NULL
#endif
	    )
	    setservent (0);

	if (sd->servf == NULL 
#if __AlgNET__
	    && sd->servs == NULL
#endif
	    )
	    return (NULL);
	    
again:
#if __AlgNET__
	if (sd->servs) {
	    p = (char *) sd->servs;
	    if (!*p)
		return (NULL);
	    sd->servs = strchr (p, '\n') + 1;
	    len = sd->servs - p;
	}
	else
#endif
	if ((p = fgetln(sd->servf, &len)) == NULL)
		return (NULL);
	if (p[len-1] == '\n')
		len--;
	if (len >= sizeof(sd->line) || len == 0)
		goto again;
	p = memcpy(sd->line, p, len);
	sd->line[len] = '\0';
	if (*p == '#')
		goto again;
	if ((cp = strchr(p, '#')) != NULL)
		*cp = '\0';
	sd->serv.s_name = p;
	p = strpbrk(p, " \t");
	if (p == NULL)
		goto again;
	*p++ = '\0';
	while (*p == ' ' || *p == '\t')
		p++;
	cp = strpbrk(p, ",/");
	if (cp == NULL)
		goto again;
	*cp++ = '\0';
	l = strtol(p, &endp, 10);
	if (endp == p || *endp != '\0' || l < 0 || l > USHRT_MAX)
		goto again;
	sd->serv.s_port = htons((in_port_t)l);
	sd->serv.s_proto = cp;
	q = sd->serv.s_aliases = sd->aliases;
	cp = strpbrk(cp, " \t");
	if (cp != NULL)
		*cp++ = '\0';
	while (cp && *cp) {
		if (*cp == ' ' || *cp == '\t') {
			cp++;
			continue;
		}
		if (q < &sd->aliases[MAXALIASES - 1])
			*q++ = cp;
		cp = strpbrk(cp, " \t");
		if (cp != NULL)
			*cp++ = '\0';
	}
	*q = NULL;
	return (&sd->serv);
}
