# Copyright (c) 2001-2007 MIPS Technologies, Inc.
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above
#      copyright
#        notice, this list of conditions and the following disclaimer
#        in the documentation and/or other materials provided with
#        the distribution.
#      * Neither the name of MIPS Technologies Inc. nor the names of its
#        contributors may be used to endorse or promote products derived
#        from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Copyright (c) 2001, Algorithmics Ltd.  All rights reserved.
#
# net/net.mk
#
# Shared subsidiary make include file for AlgNET network stack
# (included by kit/kit.mk)
#

ifneq ($(SBDNET),)

PKGFEATURES += network

# The SBDNET variable should be set in your board's sbd.mk to a list
# of the following options. The irreducible setting is SBDNET=INET.
# If you change any of these you will need to do a "make clean".
#  INET			IP + ICMP + TCP + UDP
#  GATEWAY		Packet forwarding
#  MULTICAST		Multicast support
#  MROUTING		Multicast routing
#  IPSEC		IPsec
#  CRYPTO		Cryptographic framework (not implemented)
#  KEY			PF_KEY (requires and implied by IPsec)
#  INET6		IPv6 (not tested)
#  TCP6			TCPv6 (not tested)
#  TCP_SACK		TCP selective ack
#  TCP_FACK		TCP forward ack (requires TCP_SACK)
#  TCP_SIGNATURE	TCP signatures (requires IPSEC)
#  NS			XNS
#  NSIP			XNS tunneling over IP
#  NETATALK		Appletalk
#  IPX			IPX+SPX
#  IPXIP		IPX tunneling over IP
#  ISO TPIP		OSI networking
#  EON			OSI tunneling over IP
#  CCITT LLC HDLC	X.25
#  IPFILTER		IP security packet filter (not implemented)
#  IPFILTER_LOG		Log IPF to /dev/ipl (not implemented)
#  PPP_BSDCOMP		PPP BSD compression
#  PPP_DEFLATE		PPP deflate compression

# The SBDNETCOUNT variable can be set in your board's sbd.mk to a list
# of the following options, which select the number of each
# type of "pseudo-device". The defaults are listed here, and 
# see also the NETCOUNTDEF variable below.
# If you change any of these you will need to do a "make clean".
#  NPTY=4 		Pseudo-tty devices
#  NLOOP=1		Standard loopback interfaces
#  NETHER=1		Ethernet support
#  NBPFILTER=0		Berkeley Packet Filter devices (not implemented)
#  NENC=0		Encapsulation interfaces (untested)
#  NFAITH=0		Loopback driver for testing/timing (untested)
#  NGIF=0		More encapsulation interfaces (untested)
#  NGRE=0		More encapsulation interfaces (untested)
#  NPPP=0		PPP serial line IP interfaces (not implemented)
#  NSL=0 		SLIP serial line IP interfaces (not implemented)
#  NSTRIP=0		Starmode Radio IP interfaces (not implemented)
#  NTUN=0		Tunnel interface (not implemented)
#  NVLAN=0 		Virtual LAN (not implemented)

# Miscellaneous kernel options
ifndef SBDNETKERNOPTS
SBDNETKERNOPTS	=-DDIAGNOSTIC -DKMEMSTATS
endif

# List of header files meant to contain counts
NETCOUNTF	=pty.h bpfilter.h ether.h loop.h sl.h strip.h 	\
		 ppp.h tun.h bridge.h vlan.h enc.h gre.h 	\
		 faith.h gif.h

# Default count values
NETCOUNTDEF	=NPTY=4 NETHER=1 NLOOP=1 NSL=0 NBPFILTER=0 	\
		 NSTRIP=0 NPPP=0 NTUN=0 NVLAN=0 NENC=0 NGRE=0 	\
		 NFAITH=0 NGIF=0

NETDIR = 	../../net
SYSDIR = 	$(NETDIR)/sys
SYSARCHSBD =	$(SYSDIR)/arch/algrtx/$(notdir $(SBDDIRS))

# Kernel objects
_VPATH := $(_VPATH):$(SYSDIR)/kern
SYSOBJS +=	init_main.o kern_malloc.o kern_sig.o kern_subr.o 	\
		kern_synch.o kern_sysctl.o kern_time.o kern_timeout.o	\
		subr_autoconf.o subr_log.o subr_prf.o subr_md5.o 	\
		sys_socket.o tty_pty.o 					\
		uipc_domain.o uipc_mbuf.o uipc_proto.o uipc_socket.o	\
		uipc_socket2.o uipc_syscalls.o uipc_usrreq.o vm_kern.o

# Machine dependent objects
_VPATH := $(_VPATH):$(SYSDIR)/arch/algrtx/algrtx:$(SYSDIR)/arch/algrtx/pci
_VPATH := $(_VPATH):$(SYSARCHSBD)
SYSOBJS +=	machdep.o sdepci.o sbdpcibus.o sbdbus.o ioconf.o

# PCI glue
ifneq (,$(filter yes,$(PCI)))
_VPATH := $(_VPATH):$(SYSDIR)/dev/pci
SYSOBJS +=	pci.o pci_map.o pci_subr.o
endif

# Device drivers
_VPATH := $(_VPATH):$(SYSDIR)/dev/mii:$(SYSDIR)/dev/isa
_VPATH := $(_VPATH):$(SYSDIR)/dev/ic:$(SYSDIR)/dev
SYSOBJS +=	$(SBDNETDRV)

# Network support objects
_VPATH := $(_VPATH):$(SYSDIR)/net
SYSOBJS +=	if.o if_ethersubr.o if_gif.o if_gre.o 			\
		if_loop.o if_media.o if_tun.o if_vlan.o 		\
		net_osdep.o radix.o raw_cb.o raw_usrreq.o rnd.o 	\
		route.o rtsock.o 

# IPv4 objects
_VPATH := $(_VPATH):$(SYSDIR)/netinet
SYSOBJS +=	if_ether.o in.o in_cksum.o 				\
		ip_icmp.o ip_input.o ip_output.o ip_ipip.o 		\
		igmp.o ip_ecn.o ip_id.o raw_ip.o 			\
		in_proto.o in_pcb.o in.o 				\
		tcp_debug.o tcp_input.o tcp_output.o tcp_subr.o 	\
		tcp_timer.o tcp_usrreq.o udp_usrreq.o			\
		if_ppp.o ppp_tty.o if_sl.o

ifneq (,$(filter MROUTING,$(SBDNET)))
# Multicast routing
SYSOBJS +=	ip_mroute.o 
endif

ifneq (,$(filter PPP_BSDCOMP,$(SBDNET)))
# PPP BSD compression
SYSOBJS +=	bsd-comp.o
endif

ifneq (,$(filter PPP_DEFLATE,$(SBDNET)))
# PPP ZIP compression
SYSOBJS +=	ppp-deflate.o zlib.o
endif

ifneq (,$(filter NPPP=% NSL=%,$(SBDNETCOUNT)))
# PPP || SLIP
SYSOBJS +=	slcompress.o
endif

ifneq (,$(filter IPFILTER,$(SBDNET)))
# In-kernel IP filtering and NAT (not implemented)
SYSOBJS +=	fil.o ip_fil.o ip_log.o ip_nat.o ip_frag.o 		\
		ip_state.o ip_proxy.o ip_auth.o ip_log.o
endif

ifneq (,$(filter IPSEC,$(SBDNET)))
# IP security (untested)
SYSOBJS +=	ipsec_input.o ipsec_output.o ip_esp.o ip_ah.o 		\
		pfkey.o pfkeyv2.o pfkeyv2_parsemessage.o
endif

ifneq (,$(filter IPSEC TCP_SIGNATURE,$(SBDNET)))
# IPSEC || TCP_SIGNATURE
SYSOBJS +=	ip_ipsp.o ip_spd.o
endif

ifneq (,$(filter CRYPTO,$(SBDNET)))
# Cryptography framework (not implemented)
_VPATH := $(_VPATH):$(SYSDIR)/crypto
SYSOBJS +=	rijndael.o rmd160.o sha1.o case.o skipjack.o 		\
		ecb_enc.o set_key.o ecb3_enc.o crypto.o 		\
		cryptosoft.o mbuf.o xform.o
endif

ifneq (,$(filter INET6,$(SBDNET)))
# IPv6 (untested)
_VPATH := $(_VPATH):$(SYSDIR)/netinet6
SYSOBJS +=	in6_pcb.o raw_ipv6.o in6.o in6_ifattach.o		\
		in6_cksum.o in6_src.o in6_prefix.o in6_proto.o		\
		dest6.o frag6.o icmp6.o ip6_input.o			\
		ip6_forward.o ip6_mroute.o ip6_output.o			\
		route6.o mld6.o nd6.o nd6_nbr.o nd6_rtr.o		\
		udp6_output.o	
endif

ifneq (,$(filter TCP6,$(SBDNET)))
# TCPv6 (untested)
SYSOBJS +=	raw_ip6.o tcp6_debug.o tcp6_input.o tcp6_output.o	\
		tcp6_subr.o tcp6_timer.o tcp6_usrreq.o	
endif

ifneq (,$(filter NETATALK,$(SBDNET)))
# Appletalk (untested)
_VPATH := $(_VPATH):$(SYSDIR)/netatalk
SYSOBJS +=	aarp.o at_control.o at_proto.o 				\
		ddp_input.o ddp_output.o ddp_usrreq.o
endif

ifneq (,$(filter ISO,$(SBDNET)))
# ISO protocol (untested)
_VPATH := $(_VPATH):$(SYSDIR)/netiso
SYSOBJS +=	clnp_debug.o clnp_er.o clnp_frag.o clnp_input.o 	\
		clnp_options.o clnp_output.o clnp_raw.o clnp_subr.o 	\
		clnp_timer.o cltp_usrreq.o esis.o idrp_usrreq.o 	\
		if_cons.o 						\
		iso.o iso_chksum.o iso_pcb.o iso_proto.o iso_snpac.o 	\
		tp_cons.o tp_iso.o
endif

ifneq (,$(filter ISO TPIP,$(SBDNET)))
# OSI TP or TP-over-IP (untested)
SYSOBJS +=	tp_astring.o tp_driver.o tp_emit.o 			\
		tp_inet.o tp_input.o tp_meas.o tp_output.o 		\
		tp_pcb.o tp_subr.o tp_subr2.o tp_timer.o tp_trace.o 	\
		tp_usrreq.o 
endif

ifneq (,$(filter ISO TUBA,$(SBDNET)))
# OSI Tuba or Tuba-over-IP (untested)
SYSOBJS +=	tuba_subr.o tuba_table.o tuba_usrreq.o
endif

ifneq (,$(filter EON,$(SBDNET)))
# OSI tunneling over IP (untested)
SYSOBJS +=	if_eon.o
endif

ifneq (,$(filter NS,$(SBDNET)))
# XNS protocol (untested)
_VPATH := $(_VPATH):$(SYSDIR)/netns
SYSOBJS +=	idp_usrreq.o ns.o ns_error.o ns_input.o			\
		ns_ip.o ns_output.o ns_pcb.o ns_proto.o			\
		spp_debug.o spp_usrreq.o		
endif

ifneq (,$(filter IPX,$(SBDNET)))
# IPX/SPX protocol (not implemented)
_VPATH := $(_VPATH):$(SYSDIR)/netipx
SYSOBJS +=	ipx.o ipx_input.o ipx_ip.o ipx_outputfl.o		\
		ipx_pcb.o ipx_proto.o ipx_tun.o ipx_usrreq.o		\
		ipx_cksum.o spx_debug.o spx_usrreq.o
endif

ifneq (,$(filter CCITT,$(SBDNET)))
# CCITT protocol (untested)
_VPATH := $(_VPATH):$(SYSDIR)/netccitt
SYSOBJS += 	ccitt_proto.o if_x25subr.o				\
		pk_acct.o pk_debug.o pk_input.o 			\
		pk_output.o pk_subr.o pk_timer.o pk_usrreq.o
endif

ifneq (,$(filter LLC,$(SBDNET)))
# LLC protocol (untested)
SYSOBJS +=	llc_input.o llc_output.o llc_subr.o llc_timer.o		\
		pk_llcsubr.o
endif

# Network "C" library
_VPATH := $(_VPATH):$(NETDIR)/libc/arch/mips/net:$(NETDIR)/libc/net:$(NETDIR)/libc/gen:$(NETDIR)/libc/thread
LIBCNETOBJS =	base64.o ethers.o freeaddrinfo.o		\
		gai_strerror.o getaddrinfo.o gethostnamadr.o		\
		getifaddrs.o getnameinfo.o getnetbyaddr.o getnetbyname.o \
		getnetent.o getnetnamadr.o getproto.o getprotoent.o	\
		getprotoname.o getservbyname.o getservbyport.o		\
		getservent.o herror.o htonl.o htons.o			\
		if_indextoname.o if_nameindex.o if_nametoindex.o	\
		inet_addr.o inet_lnaof.o inet_makeaddr.o 		\
		inet_net_ntop.o inet_net_pton.o inet_neta.o 		\
		inet_netof.o inet_network.o inet_ntoa.o inet_ntop.o	\
		inet_pton.o ip6opt.o ipx_addr.o ipx_ntoa.o		\
		iso_addr.o linkaddr.o net_addrcmp.o			\
		ns_addr.o ns_ntoa.o nsap_addr.o	recv.o			\
		res_comp.o res_data.o res_debug.o res_init.o		\
		res_mkquery.o res_query.o res_random.o res_send.o	\
		rthdr.o send.o sethostent.o syslog.o	\
		sigcompat.o vars6.o					\
		err.o errx.o warn.o warnx.o 				\
		verr.o verrx.o vwarn.o vwarnx.o				\
		_err.o _errx.o _warn.o _warnx.o				\
		_verr.o _verrx.o _vwarn.o _vwarnx.o

##		ntohl.o ntohs.o 		

SRCS += $(SYSOBJS:.o=.c) $(LIBCNETOBJS:.o=.c)

KERNOBJS =	$(addprefix netobj/kernel/,$(SYSOBJS))

NETHDRS =	$(addprefix netobj/kernel/,$(NETCOUNTF))

LIBCOBJS =	$(addprefix netobj/libc/,$(LIBCNETOBJS))

INCDIRS += -I $(SYSARCHSBD) \
		-I $(SYSDIR)/arch/algrtx \
		-I netobj/kernel \
		-I $(NETDIR)/libc/include \
		-idirafter $(SYSDIR) \
		-idirafter $(SYSDIR)/sys \
		-idirafter $(SYSDIR)/arch \
		-I $(KITDIR)

NETCPPFLAGS =	-D__AlgNET__

NETOPTIONS	=$(SBDNETKERNOPTS) 

# Add all SBDNET options to compiler command with -D prepended
NETOPTIONS	+=$(patsubst %,-D%,$(SBDNET))

# BSD kernel has to be compiled with -fcommon
NETOPTIONS	+=-fcommon

RETAIN		=-Wl,--retain-symbols-file,$(SYSDIR)/retainsyms

CPPFLAGS 	+=-D_NETWORK

ALL 		+=libnet.a

NETSUBDIRS 	=netobj netobj/kernel netobj/libc

net: libnet.a

.NOTPARALLEL:

$(NETSUBDIRS):
	mkdir -p $@

libnet.a: netobj/kernel/kernel.o $(LIBCOBJS)
	$(AR) $(ARFLAGS) $@ $?

netobj/kernel/kernel.o: $(KERNOBJS)
	$(CC) $(LDFLAGS) -Wl,-r $(RETAIN) -nostartfiles -nostdlib -o $@ $^

netobj/kernel/%.o: %.c
	$(COMPILE.c) -D_KERNEL $(NETOPTIONS) $(NETCPPFLAGS) $< $(OUTPUT_OPTION)

netobj/libc/%.o: %.S
	$(COMPILE.S) $(NETCPPFLAGS) $< $(OUTPUT_OPTION)

netobj/libc/%.o: %.c
	$(COMPILE.c) -I$(?D) $(NETCPPFLAGS) $< $(OUTPUT_OPTION)

$(KERNOBJS): $(NETHDRS) netobj/kernel/machine

$(NETHDRS): $(NETSUBDIRS)
	@echo "$@ $(NETCOUNTDEF) $(SBDNETCOUNT)" \
		| awk -f $(NETDIR)/mkcount.awk >$@.$$$$ \
		&& if cmp -s $@.$$$$ $@ ; \
		then rm $@.$$$$ ; \
		else mv $@.$$$$ $@ ; fi

netobj/kernel/machine:
	ln -s ../../$(SYSARCHSBD) $@
	@if [ ! -e $(SYSDIR)/arch/algrtx/machine ] ; then  \
	  echo "ln -s include $(SYSDIR)/arch/algrtx/machine"; \
          ln -s include $(SYSDIR)/arch/algrtx/machine; \
        fi

CLEAN += netclean
.PHONY: netclean
netclean:
	rm -rf libnet.a netobj

LIB_TEST +=net-test
net-test:
	@$(MAKE) SBD=$(SBD) netclean
	@$(MAKE) SBD=$(SBD) net
	@touch $@

APP_TEST +=netmon-test
netmon-test:
	@$(MAKE) -C ../../netxampl/netmon SBD=$(SBD) KITBUILT=y
	@touch $@

endif
