#include <sys/param.h>
#include <sys/systm.h>
#include <sys/device.h>
#include <vm/vm.h>
#include <vm/pmap.h>
#include <machine/pio.h>
#include <machine/intr.h>
#include <machine/autoconf.h>

#if 0
struct cfdata {
	struct	cfattach *cf_attach;	/* config attachment */
	struct	cfdriver *cf_driver;	/* config driver */
	short	cf_unit;		/* unit number */
	short	cf_fstate;		/* finding state (below) */
	int	*cf_loc;		/* locators (machine dependent) */
	int	cf_flags;		/* flags from config */
	short	*cf_parents;		/* potential parents */
	int	cf_locnames;		/* start of names */
	void	(**cf_ivstubs)		/* config-generated vectors, if any */
			__P((void));
	short	cf_starunit1;		/* 1st usable unit number by STAR */
};
extern struct cfdata cfdata[];
#define FSTATE_NOTFOUND	0	/* has not been found */
#define	FSTATE_FOUND	1	/* has been found */
#define	FSTATE_STAR	2	/* duplicable */
#define FSTATE_DNOTFOUND 3	/* has not been found, and is disabled */
#define FSTATE_DSTAR	4	/* duplicable, and is disabled */
#endif

#define NF	FSTATE_NOTFOUND	/* has not been found */
#define	STAR	FSTATE_STAR	/* duplicable */

/* The mainbus (CPU bus) */
extern struct cfattach	mainbus_ca;
extern struct cfdriver	mainbus_cd;

/* The mainbus to PCI bus bridge */
extern struct cfattach	pcibr_ca;
extern struct cfdriver  pcibr_cd;
static short		pcibr_pa[];

/* The PCI bus */
extern struct cfattach	pci_ca;
extern struct cfdriver  pci_cd;
static short		pci_pa[];
static int		pci_loc[] = {-1};	/* bus=-1 */

/* DEC ethernet driver (typical of all PCI bus devices) */
extern struct cfattach	de_ca;
extern struct cfdriver  de_cd;
static short		de_pa[];
static int		de_loc[] = {-1, -1};	/* dev=-1 func=-1 */

/* AMD PCInet ethernet driver */
extern struct cfattach	le_pci_ca;
extern struct cfdriver  le_cd;
static short		le_pa[];
static int		le_loc[] = {-1, -1};	/* dev=-1 func=-1 */

struct cfdata cfdata[] = {
    /* attach		driver		unit	fstate	loc   flg parents */
#define CF_MAINBUS	0
    {&mainbus_ca,	&mainbus_cd,	0,	NF,	0,  	0,  0},
#define CF_PCIBR	1
    {&pcibr_ca, 	&pcibr_cd,	0,	NF,	0,  	0,  pcibr_pa},
#define CF_PCI		2
    {&pci_ca, 		&pci_cd,	0,	NF,	pci_loc,0,  pci_pa},
#define CF_DE		3
    {&de_ca, 		&de_cd,		0,	STAR,	de_loc, 0,  de_pa},
#define CF_LE		4
    {&le_pci_ca,	&le_cd,		0,	STAR,	le_loc, 0,  le_pa},

    {0}
};

/* root devices */
short cfroots[]	= {CF_MAINBUS, -1};

/* potential parents of each device */
static short pcibr_pa[] = {CF_MAINBUS, -1};
static short pci_pa[]	= {CF_PCIBR, -1};
static short de_pa[]	= {CF_PCI, -1};
static short le_pa[] 	= {CF_PCI, -1};

/* pseudo device initialisation */
extern void loopattach __P((int));
extern void bpfilterattach __P((int));
extern void slattach __P((int));
extern void pppattach __P((int));
extern void stripattach __P((int));
extern void tunattach __P((int));
extern void ptyattach __P((int));
extern void vlanattach __P((int));
extern void encattach __P((int));
extern void greattach __P((int));
extern void faithattach __P((int));
extern void gifattach __P((int));

const struct pdevinit pdevinit[] = {
#include "loop.h"
#if NLOOP > 0
    {loopattach,	NLOOP},
#endif
#include "bpfilter.h"
#if NBPFILTER > 0
    {bpfilterattach,	NBPFILTER},
#endif
#include "bridge.h"
#if NBRIDGE > 0
    {bridgeattach,	NBRIDGE},
#endif
#include "sl.h"
#if NSL > 0
    {slattach,		NSL},
#endif
#include "ppp.h"
#if NPPP > 0
    {pppattach,		NPPP},
#endif
#include "strip.h"
#if NSTRIP > 0
    {stripattach,	NSTRIP},
#endif
#include "tun.h"
#if NTUN > 0
    {tunattach,		NTUN},
#endif
#include "pty.h"
#if NPTY > 0
    {ptyattach,		NPTY},
#endif
#include "vlan.h"
#if NVLAN > 0
    {vlanattach,	NVLAN},
#endif
#include "enc.h"
#if NENC > 0
    {encattach,		NENC},
#endif
#include "gre.h"
#if NGRE > 0
    {greattach,		NGRE},
#endif
#include "faith.h"
#if NFAITH > 0
    {faithattach,	NFAITH},
#endif
#include "gif.h"
#if NGIF > 0
    {gifattach,		NGIF},
#endif
    {0, 0}
};
