/* 
 * PCI bus glue for BSD kernel running on IDT 79S33X board.
 *
 * Copyright (c) 2001 Algorithmics Ltd
 *
 * This program is NOT free software, it is supplied under the terms
 * of the AlgNET License Agreement, a copy of which is available at:
 *
 *  http://www.algor.co.uk/algor/info/algnet-license.pdf
 *
 * Any company which has obtained and signed a valid AlgNET license
 * may use and modify this software internally and use (without
 * restrictions) any derived binary.  You may not, however,
 * redistribute this in whole or in part as source code, nor may you
 * modify or remove any part of this copyright message.  
 */

/*
 * Copyright (c) 1996, 1997, 1998 Per Fogelstrom, Opsycon AB
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed under OpenBSD by
 *	Per Fogelstrom, Opsycon AB.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#define _ALGRTX_BUS_DMA_PRIVATE
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/device.h>
#include <machine/pio.h>
#include <machine/intr.h>
#include <machine/autoconf.h>
#include <machine/bus.h>

/*#include <sbdbus.h>*/

#include <dev/pci/pcireg.h>
#include <dev/pci/pcivar.h>

#include <pci/pci_machdep.h>

#include <pci/pcivar.h>

#include <mips/cpu.h>
#include <idt/rc3233x.h>

#include "sbd.h"
#include "kit_impl.h"

#include <vm/pmap.h>

static bus_addr_t idt33x_bus_dmamap_vtop __P((bus_dma_tag_t, caddr_t,
					      bus_size_t));

struct algrtx_bus_dma_tag pci_bus_dma_tag = {
	NULL,			/* _cookie */
	_bus_dmamap_create, 
	_bus_dmamap_destroy,
	_bus_dmamap_load,
	_bus_dmamap_load_mbuf,
	_bus_dmamap_load_uio,
	_bus_dmamap_load_raw,
	_bus_dmamap_unload,
	_bus_dmamap_sync,
	idt33x_bus_dmamap_vtop,
	_bus_dmamem_alloc,
	_bus_dmamem_free,
	_bus_dmamem_map,
	_bus_dmamem_unmap,
	_bus_dmamem_mmap,
};

static void idt33x_sync_cache __P((vm_offset_t, int));

void
_sbd_pcibus_attach (struct pcibus_attach_args *pba)
{
    printf (": IDT RC3233x\n");

    pba->pba_memt->bus_base = (u_int32_t) PCI_MEM_SPACE;  /* note in KUSEG */
    pba->pba_memt->bus_sparse1 = 0;
    pba->pba_memt->bus_sparse2 = 0;
    pba->pba_memt->bus_sparse4 = 0;
    pba->pba_memt->bus_sparse8 = 0;

    pba->pba_iot->bus_base = (u_int32_t) PA_TO_KVA1 (PCI_IO_SPACE);
    pba->pba_iot->bus_sparse1 = 0;
    pba->pba_iot->bus_sparse2 = 0;
    pba->pba_iot->bus_sparse4 = 0;
    pba->pba_iot->bus_sparse8 = 0;

    /* set cache coherence hooks, as required */
    pba->pba_pc->pc_sync_cache = idt33x_sync_cache;
    pba->pba_pc->pc_postsync_cache = 0;	/* none */

    /* indicate what PCI read/write commands the bridge supports,
       only "memory read multiple" gives any useful benefit over plain
       "read" */
    pba->pba_flags |= 0
    	/* | PCI_FLAGS_MRL_OKAY*/
	| PCI_FLAGS_MRM_OKAY
        /* | PCI_FLAGS_MWI_OKAY */
	;
}


/* Convert CPU virtual address to physical PCI address, used only
   by if_de.c driver. */

vm_offset_t 
_pci_vtop (struct device *dv, vm_offset_t addr)
{
    /* call MIPS SDE PCI mapper */
    return _pci_dmamap (addr, 1);
}


/* Convert CPU virtual address to physical address, old interface.  In
   theory this is meant to be the CPU physical address, but we'll
   assume that this an address about to be passed to a PCI master,
   and pass the PCI physical address. */

/* XXX most drivers calling this function won't be handling caches
   properly. They should be using the bus_dmamap_... interfaces */

vm_offset_t 
_kern_vtophys (vm_offset_t addr)
{
    /* call MIPS SDE PCI mapper */
    return _pci_dmamap (addr, 1);
}

/* Preferred interface, called from ../algrtx/machdep.c */

static bus_addr_t
idt33x_bus_dmamap_vtop (bus_dma_tag_t t, caddr_t va, bus_size_t nb)
{
    /* call MIPS SDE PCI mapper */
    return _pci_dmamap ((vm_offset_t) va, nb);
}


/* Flush caches associated with CPU virtual address range [addr, addr + len) */

static void
idt33x_sync_cache (vm_offset_t addr, int len)
{
    if (IS_KVA0 (addr))
	mips_clean_cache ((vaddr_t)addr, len);
}


int
_pci_ether_hw_addr (u_int8_t *p, int bus, int devno)
{
    /* Nothing to do */
    return 0;
}
