/* 
 * PCI bus glue for BSD kernel running on MIPS Malta.
 *
 * Copyright (c) 2002 Algorithmics Ltd
 *
 * This program is NOT free software, it is supplied under the terms
 * of the AlgNET License Agreement, a copy of which is available at:
 *
 *  http://www.algor.co.uk/algor/info/algnet-license.pdf
 *
 * Any company which has obtained and signed a valid AlgNET license
 * may use and modify this software internally and use (without
 * restrictions) any derived binary.  You may not, however,
 * redistribute this in whole or in part as source code, nor may you
 * modify or remove any part of this copyright message.  
 */

/*
 * Copyright (c) 1996, 1997, 1998 Per Fogelstrom, Opsycon AB
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed under OpenBSD by
 *	Per Fogelstrom, Opsycon AB.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#define _ALGRTX_BUS_DMA_PRIVATE
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/device.h>
#include <machine/pio.h>
#include <machine/intr.h>
#include <machine/autoconf.h>
#include <machine/bus.h>

#include <sys/init.h>

/*#include <sbdbus.h>*/

#include <dev/pci/pcireg.h>
#include <dev/pci/pcivar.h>

#include <pci/pci_machdep.h>

#include <pci/pcivar.h>

#include <mips/cpu.h>
#include "sbd.h"

#include "bonito.h"
#include "gt64120.h"

#include "kit_impl.h"

#include <vm/pmap.h>

static void malta_bus_dmamap_sync __P((bus_dma_tag_t, bus_dmamap_t,
				       bus_addr_t, bus_size_t, int));

static bus_addr_t malta_bus_dmamap_vtop __P((bus_dma_tag_t, caddr_t,
					     bus_size_t));

struct algrtx_bus_dma_tag pci_bus_dma_tag = {
	NULL,			/* _cookie */
	_bus_dmamap_create, 
	_bus_dmamap_destroy,
	_bus_dmamap_load,
	_bus_dmamap_load_mbuf,
	_bus_dmamap_load_uio,
	_bus_dmamap_load_raw,
	_bus_dmamap_unload,
	malta_bus_dmamap_sync,
	malta_bus_dmamap_vtop,
	_bus_dmamem_alloc,
	_bus_dmamem_free,
	_bus_dmamem_map,
	_bus_dmamem_unmap,
	_bus_dmamem_mmap,
};

static char * const _bonito = PA_TO_KVA1(BONITO_REG_BASE);
static void malta_postsync_cache __P((vm_offset_t, int));
static void malta_presync_cache __P((vm_offset_t, int));

void
_sbd_pcibus_attach (struct pcibus_attach_args *pba)
{
    if (_sbd_sysctrl_type == SYSCTRL_IS_BONITO) {
      unsigned int bonrev = PCI_REVISION(BONITO_PCICLASS);
      printf (": Bonito revision: %s %x.%x\n",
	      (bonrev & 0x80) ? "FPGA" : "ASIC",
	      (bonrev >> 4) & 0xf, bonrev & 0xf);
    } else {
      printf (": Galileo GT64120\n");
    }

    pba->pba_iot->bus_base = (u_int32_t) PA_TO_KVA1 (PCI_IO_SPACE);
    pba->pba_iot->bus_sparse1 = 0;
    pba->pba_iot->bus_sparse2 = 0;
    pba->pba_iot->bus_sparse4 = 0;
    pba->pba_iot->bus_sparse8 = 0;

    pba->pba_memt->bus_base = (u_int32_t) PA_TO_KVA1 (PCI_MEM_SPACE);
    pba->pba_memt->bus_sparse1 = 0;
    pba->pba_memt->bus_sparse2 = 0;
    pba->pba_memt->bus_sparse4 = 0;
    pba->pba_memt->bus_sparse8 = 0;

    /* set cache coherence hooks */
    pba->pba_pc->pc_sync_cache = malta_presync_cache;
    if (_sbd_sysctrl_type == SYSCTRL_IS_BONITO)
        pba->pba_pc->pc_postsync_cache = malta_postsync_cache;
    else
	pba->pba_pc->pc_postsync_cache = 0;
      
    /* indicate what PCI read/write modes the bridge supports */
#if 0
    /* doesn't matter, Bonito always does read-ahead/write-behind */
    pba->pba_flags |= 0
    	| PCI_FLAGS_MRL_OKAY
	| PCI_FLAGS_MRM_OKAY
        | PCI_FLAGS_MWI_OKAY
	;
#endif
}


/* Convert CPU virtual address to physical PCI address, used only
   by if_de.c driver. */

vm_offset_t 
_pci_vtop (struct device *dv, vm_offset_t addr)
{
    vm_offset_t pa = _pci_dmamap (addr, 1);
    return pa;
}


/* Convert CPU virtual address to physical address, old interface.  In
   theory this is meant to be the CPU physical address, but we'll
   assume that this an address about to be passed to a PCI master,
   and pass the PCI physical address. */

/* XXX most drivers calling this function won't be handling caches
   properly. They should be using the bus_dmamap_... interfaces */

vm_offset_t 
_kern_vtophys (vm_offset_t addr)
{
    return _pci_dmamap (addr, 1);
}


/* Flush caches associated with CPU virtual address range [addr, addr + len) */

void
_pci_sync_cache (vm_offset_t addr, int len)
{
    mips_clean_cache ((vaddr_t)addr, len);
}


/* Preferred interface, called from ../algrtx/machdep.c */

static bus_addr_t
malta_bus_dmamap_vtop (bus_dma_tag_t t, caddr_t va, bus_size_t nb)
{
    /* call MIPS SDE PCI mapper */
    return _pci_dmamap ((vm_offset_t) va, nb);
}



/*
 * Bonito i/o buffer cache handling
 *
 * The i/o buffer cache snoops on uncached cycles by the MIPS cpu, so
 * we don't need to do anything for uncached CPU acceses (e.g. command
 * and status descriptors). But we do have to explicitly write-back
 * and invalidate buffer cache lines if we have used cached CPU accesses.
 */

/* FIXME these definitions should be in bonito.h! */
#define CACHECMD_INVAL	0
#define CACHECMD_WBINV	1
#define CACHECMD_RDTAG	2
#define CACHECMD_WQFLUSH 3

#define TAG_LOCK	0x80000000
#define TAG_WBACK	0x40000000
#define TAG_PFPEND	0x20000000
#define TAG_PEND	0x10000000
#define TAG_MOD		0x08000000
#define TAG_PFDVAL	0x04000000
#define TAG_DVAL	0x02000000
#define TAG_AVAL	0x01000000
#define TAG_ADDR	0x00ffffff


/*
 * Execute an i/o buffer cache command 
 */
 
static void
bonito_iobc_cmd (unsigned int cmd, unsigned int line)
{
    unsigned int ctrl;
	
    ctrl = (cmd << BONITO_PCICACHECTRL_CACHECMD_SHIFT)
	| (line << BONITO_PCICACHECTRL_CACHECMDLINE_SHIFT);
    
    BONITO_PCICACHECTRL = ctrl;
    BONITO_PCICACHECTRL = ctrl | BONITO_PCICACHECTRL_CMDEXEC;
    while (BONITO_PCICACHECTRL & BONITO_PCICACHECTRL_CMDEXEC)
	continue;
    BONITO_PCICACHECTRL = ctrl;
}


/*
 * Writeback and invalidate any i/o buffer cache lines which overlap
 * the specified address range. 
 */
 
static void
_bonito_iobc_wbinv (unsigned int pa, size_t nb)
{
    unsigned int line;
    unsigned int tag;
    
    for (line = 0; line < 4; line++) {
	bonito_iobc_cmd (CACHECMD_RDTAG, line);
	tag = BONITO_PCICACHETAG;
	if (tag & TAG_AVAL) {
	    unsigned int tagaddr = (tag & TAG_ADDR) << 5;
	    if (nb == ~(size_t)0 ||
		(tagaddr < pa + nb && tagaddr + 32 > pa))
		bonito_iobc_cmd (CACHECMD_WBINV, line);
	}
    }
    bonito_iobc_cmd (CACHECMD_WQFLUSH, 0);
}

/* Flush caches associated with CPU virtual address range [addr, addr + len),
   before a transfer. */

static void
malta_presync_cache (vm_offset_t addr, int len)
{
    if (IS_KVA0 (addr)) {
        if (_sbd_sysctrl_type == SYSCTRL_IS_BONITO)
	    _bonito_iobc_wbinv (KVA_TO_PA (addr), len);
	mips_clean_cache (addr, len);
    }
}

/* Flush caches associated with CPU virtual address range [addr, addr + len),
   after a transfer. */

static void
malta_postsync_cache (vm_offset_t addr, int len)
{
    if (IS_KVA0 (addr))
	/* no mips cpu cache flush required */
	_bonito_iobc_wbinv (KVA_TO_PA (addr), len);
}

/*
 * Malta PCI specific for DMA cache synchronization.  
 */
static void
malta_bus_dmamap_sync (t, map, offset, len, ops)
	bus_dma_tag_t t;
	bus_dmamap_t map;
	bus_addr_t offset;
	bus_size_t len;
	int ops;
{
    /* call generic function */
    _bus_dmamap_sync(t, map, offset, len, ops);

    /* nothing more needed for cache coherent (uncached) mappings */
    if (map->_dm_flags & ALGOR_DMAMAP_COHERENT)
	return;

    /* nothing more needed if not Bonito */
    if (_sbd_sysctrl_type != SYSCTRL_IS_BONITO)
	return;

    /* flush the Bonito i/o buffer cache (readahead/writebehind) */
    if (ops & (BUS_DMASYNC_PREWRITE | BUS_DMASYNC_POSTREAD)) {
	int i;
	for (i = 0; i < map->dm_nsegs && len != 0; i++) {
	    bus_size_t minlen;

	    /* Find the beginning segment. */
	    if (offset >= map->dm_segs[i].ds_len) {
		offset -= map->dm_segs[i].ds_len;
		continue;
	    }
	    /*
	     * Now at the first segment to sync; nail
	     * each segment until we have exhausted the
	     * length.
	     */
	    minlen = len < map->dm_segs[i].ds_len - offset ?
		len : map->dm_segs[i].ds_len - offset;

	    /* ds_addr holds physaddr */
	    _bonito_iobc_wbinv (map->dm_segs[i].ds_addr + offset, len);
	    offset = 0;
	    len -= minlen;
	}
    }
}



/*
 * return ethernet address associated with a particular device 
 */
int
_pci_ether_hw_addr (u_int8_t *p, int bus, int devno)
{
    /* No special support needed */
    return 0;
}
