/* 
 * PCI bus glue for BSD kernel running on P-4032.
 *
 * Copyright (c) 2001 Algorithmics Ltd
 *
 * This program is NOT free software, it is supplied under the terms
 * of the AlgNET License Agreement, a copy of which is available at:
 *
 *  http://www.algor.co.uk/algor/info/algnet-license.pdf
 *
 * Any company which has obtained and signed a valid AlgNET license
 * may use and modify this software internally and use (without
 * restrictions) any derived binary.  You may not, however,
 * redistribute this in whole or in part as source code, nor may you
 * modify or remove any part of this copyright message.  
 */

/*
 * Copyright (c) 1996, 1997, 1998 Per Fogelstrom, Opsycon AB
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed under OpenBSD by
 *	Per Fogelstrom, Opsycon AB.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#define _ALGRTX_BUS_DMA_PRIVATE
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/device.h>
#include <machine/pio.h>
#include <machine/intr.h>
#include <machine/autoconf.h>
#include <machine/bus.h>

/*#include <sbdbus.h>*/

#include <dev/pci/pcireg.h>
#include <dev/pci/pcivar.h>

#include <pci/pci_machdep.h>

#include <pci/pcivar.h>

#include <mips/cpu.h>
#include "sbd.h"
#include "v96xpbc.h"
#include "kit_impl.h"

#include <vm/pmap.h>

static unsigned int v96x_vrev;

static void get_eth_hw_addr __P((char *));

char *getenv __P((const char *env));

/* Hardware ethernet address */
static char eth_hw_addr[6];	

static bus_addr_t p4032_bus_dmamap_vtop __P((bus_dma_tag_t, caddr_t,
					     bus_size_t));

struct algrtx_bus_dma_tag pci_bus_dma_tag = {
	NULL,			/* _cookie */
	_bus_dmamap_create, 
	_bus_dmamap_destroy,
	_bus_dmamap_load,
	_bus_dmamap_load_mbuf,
	_bus_dmamap_load_uio,
	_bus_dmamap_load_raw,
	_bus_dmamap_unload,
	_bus_dmamap_sync,
	p4032_bus_dmamap_vtop,
	_bus_dmamem_alloc,
	_bus_dmamem_free,
	_bus_dmamem_map,
	_bus_dmamem_unmap,
	_bus_dmamem_mmap,
};

static void p4032_sync_cache __P((vm_offset_t, int));

void
_sbd_pcibus_attach (struct pcibus_attach_args *pba)
{
    unsigned char * const _v96xp = PA_TO_KVA1(V96XPBC_BASE);
    int pcioldmap;
    char *s;

    /* get PCI bridge chip revision */
    v96x_vrev = V96X_PCI_CC_REV & V96X_PCI_CC_REV_VREV;
    printf (": V3 V962PBC, Rev %x.\n", v96x_vrev);

    pba->pba_memt->bus_base = (u_int32_t) PA_TO_KVA1 (V96X_LB_BASE1 & V96X_LB_BASEx_ADR_BASE);
    pba->pba_memt->bus_sparse1 = 0;
    pba->pba_memt->bus_sparse2 = 0;
    pba->pba_memt->bus_sparse4 = 0;
    pba->pba_memt->bus_sparse8 = 0;

    /*
     * Only newer revisions of the V3 bridge can
     * provide simultansous access to Memory band IO space
     */
    if (((s = getenv ("pcimap")) || (s = getenv ("PCIMAP")))
	&& strcmp (s, "old") == 0)
	pcioldmap = 1;
    else
	pcioldmap = 0;

    pba->pba_iot->bus_base = 
	(v96x_vrev >= V96X_VREV_B2 && !pcioldmap)
	?  (u_int32_t) PA_TO_KVA1 (V96X_LB_BASE2 & V96X_LB_BASEx_ADR_BASE)
	: 0xa11dead;	/* hopefully cause a trap if it is used. */
    pba->pba_iot->bus_sparse1 = 0;
    pba->pba_iot->bus_sparse2 = 0;
    pba->pba_iot->bus_sparse4 = 0;
    pba->pba_iot->bus_sparse8 = 0;

    /* set cache coherence hooks, as required */
    pba->pba_pc->pc_sync_cache = p4032_sync_cache;
    pba->pba_pc->pc_postsync_cache = 0;	/* none */
	
    /* indicate what PCI read/write modes the bridge supports */
    pba->pba_flags |= 0
    	/*| PCI_FLAGS_MRL_OKAY*/
	| PCI_FLAGS_MRM_OKAY
        /*| PCI_FLAGS_MWI_OKAY*/
	;

    /* ethernet address is stored in environment */
    get_eth_hw_addr(getenv("ethaddr"));
}


/* Convert CPU virtual address to physical PCI address, used only
   by if_de.c driver. */

vm_offset_t 
_pci_vtop (struct device *dv, vm_offset_t addr)
{
    vm_offset_t pa = _pci_dmamap (addr, 1);
    if (dv->dv_class == DV_IFNET && v96x_vrev < V96X_VREV_C0) { 
	/* BUG in early V962PBC's */
	pa |= 0xc0000000;	/* Use aparture 2 */
    }
    return pa;
}


/* Convert CPU virtual address to physical address, old interface.  In
   theory this is meant to be the CPU physical address, but we'll
   assume that this an address about to be passed to a PCI master,
   and pass the PCI physical address. */

/* XXX most drivers calling this function won't be handling caches
   properly. They should be using the bus_dmamap_... interfaces */

vm_offset_t 
_kern_vtophys (vm_offset_t addr)
{
    return _pci_dmamap (addr, 1);
}


/* Preferred interface, called from ../algrtx/machdep.c */

static bus_addr_t
p4032_bus_dmamap_vtop (bus_dma_tag_t t, caddr_t va, bus_size_t nb)
{
    /* call MIPS SDE PCI mapper */
    return _pci_dmamap ((vm_offset_t) va, nb);
}


/* Flush caches associated with CPU virtual address range [addr, addr + len) */

static void
p4032_sync_cache (vm_offset_t addr, int len)
{
    if (IS_KVA0 (addr))
	mips_clean_cache ((vaddr_t)addr, len);
}


/*
 * Code from "pci/if_de.c" used to calculate crc32 of ether rom data.
 * Another example can be found in document EC-QPQWA-TE from DEC.
 */
#define      TULIP_CRC32_POLY  0xEDB88320UL
static __inline__ unsigned
srom_crc32(
    const unsigned char *databuf,
    size_t datalen)
{
    u_int idx, bit, data, crc = 0xFFFFFFFFUL;

    for (idx = 0; idx < datalen; idx++)
        for (data = *databuf++, bit = 0; bit < 8; bit++, data >>= 1)
            crc = (crc >> 1) ^ (((crc ^ data) & 1) ? TULIP_CRC32_POLY : 0);
    return crc;
}

/*
 * Convert "xx:xx:xx:xx:xx:xx" string to ethernet hardware address.
 */
static void
get_eth_hw_addr(s)
	char *s;
{
    extern unsigned long strtoul (const char *, char **, int);
    char *e;
    int i;
    if(s != NULL) {
	for(i = 0; i < 6; i++) {
	    eth_hw_addr[i] = strtoul (s, &e, 16);
	    s = e + 1;
	}
    }
}

/*
 * Build the serial rom info normaly stored in an EEROM on
 * PCI DEC21x4x boards. Cheapo designs skips the rom so
 * we do the job here. The setup is not 100% correct but
 * close enough to make the driver happy!  */

int
_pci_ether_hw_addr (u_int8_t *p, int bus, int devno)
{
	int i;

	/* check that this is our local device */
	if (bus != 0 || devno != 5)
	    return 0;

	for(i = 0; i < 128; i++)
		p[i] = 0x00;
	p[18] = 0x03;	/* Srom version. */
	p[19] = 0x01;	/* One chip. */

	/* Next six, ethernet address. */
	bcopy(eth_hw_addr, &p[20], 6);

	p[26] = 0x00;	/* Chip 0 device number */
	p[27] = 30;		/* Descriptor offset */
	p[28] = 00;
	p[29] = 00;		/* MBZ */
					/* Descriptor */
	p[30] = 0x00;	/* Autosense. */
	p[31] = 0x08;
#if 1
	/* P4032/P5064 */
		p[32] = 0x01;	/* Block cnt */
		p[33] = 0x02;	/* Medium type is AUI */
#else
		p[32] = 0xff;	/* GP cntrl */
		p[33] = 0x01;	/* Block cnt */
#define GPR_LEN 0
#define	RES_LEN 0
		p[34] = 0x80 + 12 + GPR_LEN + RES_LEN;
		p[35] = 0x01;	/* MII PHY type */
		p[36] = 0x00;	/* PHY number 0 */
		p[37] = 0x00;	/* GPR Length */
		p[38] = 0x00;	/* Reset Length */
		p[39] = 0x00;	/* Media capabilities */
		p[40] = 0x78;	/* Media capabilities */
		p[41] = 0x00;	/* Autoneg advertisment */
		p[42] = 0x78;	/* Autoneg advertisment */
		p[43] = 0x00;	/* Full duplex map */
		p[44] = 0x50;	/* Full duplex map */
		p[45] = 0x00;	/* Treshold map */
		p[46] = 0x18;	/* Treshold map */
#endif

	i = (srom_crc32(p, 126) & 0xFFFF) ^ 0xFFFF;
	p[126] = i;
	p[127] = i >> 8;

	return 1;
}
