/*	$OpenBSD: pci_machdep.h,v 1.4 1998/03/18 12:06:18 pefo Exp $	*/
/*	AlgRTX version */

/*
 * Copyright (c) 1996 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Chris G. Demetriou
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie the
 * rights to redistribute these changes.
 */

#ifndef _ALGRTX_PCI_MACHDEP_H_
#define _ALGRTX_PCI_MACHDEP_H_

/*
 * Machine-specific definitions for PCI autoconfiguration.
 */

#include <machine/vmparam.h>	/* for vm_offset_t */

/*
 * Types provided to machine-independent PCI code
 */
typedef struct alg_pci_chipset *pci_chipset_tag_t;
typedef u_int32_t pcitag_t;
typedef u_int32_t pci_intr_handle_t;

extern struct algrtx_bus_dma_tag pci_bus_dma_tag;

/*
 * machine-specific PCI structure and type definitions.
 * NOT TO BE USED DIRECTLY BY MACHINE INDEPENDENT CODE.
 */
struct alg_pci_chipset {
    /* config space */
	void		*pc_conf_v;
	void		(*pc_attach_hook) __P((struct device *,
			    struct device *, struct pcibus_attach_args *));
	int		(*pc_bus_maxdevs) __P((void *, int));
	pcitag_t	(*pc_make_tag) __P((void *, int, int, int));
	void		(*pc_decompose_tag) __P((void *, pcitag_t, int *,
			    int *, int *));
	pcireg_t	(*pc_conf_read) __P((void *, pcitag_t, int));
	void		(*pc_conf_write) __P((void *, pcitag_t, int, pcireg_t));
    /* interrupt handling */
	void		*pc_intr_v;
	int		(*pc_intr_map) __P((void *, pcitag_t, int, int,
			    pci_intr_handle_t *));
	const char	*(*pc_intr_string) __P((void *, pci_intr_handle_t));
	void		*(*pc_intr_establish) __P((void *, pci_intr_handle_t,
			    int, int (*)(void *), void *, char *));
	void		(*pc_intr_disestablish) __P((void *, void *));

    /* miscellaneous */
	int		(*pc_ether_hw_addr) __P((u_int8_t *, int, int));
	void		(*pc_sync_cache) __P((vm_offset_t, int));
	void		(*pc_postsync_cache) __P((vm_offset_t, int));
	vm_offset_t	(*pc_vtop) __P((struct device *, vm_offset_t));

        int		pc_maxburst_log2;
        int		pc_cacheline_log2;
};

/*
 * Functions provided to machine-independent PCI code.
 */
#define	pci_attach_hook(p, s, pba)					\
    (*(pba)->pba_pc->pc_attach_hook)((p), (s), (pba))
#define	pci_bus_maxdevs(c, b)						\
    (*(c)->pc_bus_maxdevs)((c)->pc_conf_v, (b))
#define	pci_make_tag(c, b, d, f)					\
    (*(c)->pc_make_tag)((c)->pc_conf_v, (b), (d), (f))
#define	pci_decompose_tag(c, t, bp, dp, fp)				\
    (*(c)->pc_decompose_tag)((c)->pc_conf_v, (t), (bp), (dp), (fp))
#define	pci_conf_read(c, t, r)						\
    (*(c)->pc_conf_read)((c)->pc_conf_v, (t), (r))
#define	pci_conf_write(c, t, r, v)					\
    (*(c)->pc_conf_write)((c)->pc_conf_v, (t), (r), (v))
#define	pci_intr_map(c, it, ip, il, ihp)				\
    (*(c)->pc_intr_map)((c)->pc_intr_v, (it), (ip), (il), (ihp))
#define	pci_intr_string(c, ih)						\
    (*(c)->pc_intr_string)((c)->pc_intr_v, (ih))
#define	pci_intr_establish(c, ih, l, h, a, nm)				\
    (*(c)->pc_intr_establish)((c)->pc_intr_v, (ih), (l), (h), (a), (nm))
#define	pci_intr_disestablish(c, iv)					\
    (*(c)->pc_intr_disestablish)((c)->pc_intr_v, (iv))
#define	pci_ether_hw_addr(c, p, b, d)					\
    (*(c)->pc_ether_hw_addr)((p), (b), (d))
#define	pci_sync_cache(c, p, s)						\
    if ((c)->pc_sync_cache)						\
      (*(c)->pc_sync_cache)((p), (s))
#define	pci_postsync_cache(c, p, s)					\
    if ((c)->pc_postsync_cache)						\
      (*(c)->pc_postsync_cache)((p), (s))
#define	pci_vtop(c, dv, va)						\
    (*(c)->pc_vtop)((dv), (va))

/* KLUDGE KLUDGE */
#define NEED_PCI_ETHER_HW_ADDR_FUNC
#define	TULIP_KVATOPHYS(sc, va) \
	pci_vtop(sc->tulip_pc, &sc->tulip_dev, (vm_offset_t)va)
#define	TULIP_BURSTSIZE(unit)	\
	MIN(TULIP_UNIT_TO_SOFTC(unit)->tulip_pc->pc_maxburst_log2, 5)
#define	NCR_KVATOPHYS(sc, va)	\
	pci_vtop(sc->sc_pc, &sc->sc_dev, (vm_offset_t)va)

#define PHYS_TO_UNCACHED(pa)	((paddr_t)(pa) | 0xa0000000)
#define PHYS_TO_CACHED(pa)	((paddr_t)(pa) | 0x80000000)
#define CACHED_TO_PHYS(va)	((vaddr_t)(va) & 0x1fffffff)
#define UNCACHED_TO_PHYS(va)	((vaddr_t)(va) & 0x1fffffff)
#define CACHED_TO_UNCACHED(va)	((vaddr_t)(va) | 0x20000000)
#define UNCACHED_TO_CACHED(va)	((vaddr_t)(va) & ~0x20000000)

void 		_sbd_pcibus_attach (struct pcibus_attach_args *);

vm_offset_t 	_pci_vtop (struct device *, vm_offset_t);
void		_pci_sync_cache (vm_offset_t, int);
int		_pci_ether_hw_addr (u_int8_t *, int, int);

#endif /* _ALGRTX_PCI_MACHDEP_H_ */
