/*  
 * Generic PCI bridge driver glue for BSD kernel running on top of
 * MIPS SDE PCI support code.
 *
 * Copyright (c) 2001 Algorithmics Ltd
 *
 * This program is NOT free software, it is supplied under the terms
 * of the AlgNET License Agreement, a copy of which is available at:
 *
 *  http://www.algor.co.uk/algor/info/algnet-license.pdf
 *
 * Any company which has obtained and signed a valid AlgNET license
 * may use and modify this software internally and use (without
 * restrictions) any derived binary.  You may not, however,
 * redistribute this in whole or in part as source code, nor may you
 * modify or remove any part of this copyright message.  */

/*
 * Copyright (c) 1997, 1998 Per Fogelstrom, Opsycon AB
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed under OpenBSD by
 *	Per Fogelstrom, Opsycon AB.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/kernel.h>
#include <sys/malloc.h>
#include <sys/device.h>
#include <sys/proc.h>
#include <vm/vm.h>

#include <machine/autoconf.h>
#define _ALGRTX_BUS_DMA_PRIVATE
#include <machine/bus.h>

#include <dev/pci/pcireg.h>
#include <dev/pci/pcivar.h>

#include <pci/pci_machdep.h>
#include <algrtx/pci/pci_machdep.h>
#include <algrtx/pci/pcibrvar.h>
#include <pci/pcivar.h>


static int	pcibrmatch __P((struct device *, void *, void *));
static void	pcibrattach __P((struct device *, struct device *, void *));
static int	pcibrprint __P ((void *, const char *));

struct cfattach pcibr_ca = {
        sizeof(struct pcibr_softc), pcibrmatch, pcibrattach,
};

struct cfdriver pcibr_cd = {
	NULL, "pcibr", DV_DULL,
};


/*
 * Functions provided to machine-independent PCI code.
 */
static void	pbc_attach_hook __P((struct device *, struct device *,
				      struct pcibus_attach_args *));
static int	pbc_bus_maxdevs __P((void *, int));
static pcitag_t	pbc_make_tag __P((void *, int, int, int));
static void 	pbc_decompose_tag __P((void *, pcitag_t, int *,
					int *, int *));
static pcireg_t pbc_conf_read __P((void *, pcitag_t, int));
static void 	pbc_conf_write __P((void *, pcitag_t, int, pcireg_t));
static int 	pbc_intr_map __P((void *, pcitag_t, int, int,
				   pci_intr_handle_t *));
static const char * pbc_intr_string __P((void *, pci_intr_handle_t));
static void * 	pbc_intr_establish __P((void *, pci_intr_handle_t,
					 int, int (*)(void *),
					 void *, char *));
static void 	pbc_intr_disestablish __P((void *, void *));
static int 	pbc_ether_hw_addr __P((u_int8_t *, int, int));
static vm_offset_t pbc_vtop __P((struct device *, vm_offset_t));

static struct pcibr_config pbc_config;

#if 0
/*
 * PCI doesn't have any special needs; just use the generic versions
 * of these functions.
 */
static struct algrtx_bus_dma_tag pci_bus_dma_tag = {
	NULL,			/* _cookie */
	_bus_dmamap_create, 
	_bus_dmamap_destroy,
	_bus_dmamap_load,
	_bus_dmamap_load_mbuf,
	_bus_dmamap_load_uio,
	_bus_dmamap_load_raw,
	_bus_dmamap_unload,
	_bus_dmamap_sync,	
	_bus_dmamem_alloc,
	_bus_dmamem_free,
	_bus_dmamem_map,
	_bus_dmamem_unmap,
	_bus_dmamem_mmap,
};
#endif

static int
pcibrmatch(parent, match, aux)
	struct device *parent;
	void *match, *aux;
{
	struct confargs *ca = aux;

	/* Make sure that we're looking for a PCI bridge. */
	if (strcmp(ca->ca_name, pcibr_cd.cd_name) != 0)
		return (0);

	return (1);
}

static void
pcibrattach(parent, self, aux)
	struct device *parent, *self;
	void *aux;
{
	struct pcibr_softc *sc = (struct pcibr_softc *)self;
	struct pcibr_config *lcp;
	struct pcibus_attach_args pba;

	lcp = sc->sc_pcibr = &pbc_config;

	lcp->lc_pc.pc_conf_v = lcp;
	lcp->lc_pc.pc_attach_hook = pbc_attach_hook;
	lcp->lc_pc.pc_bus_maxdevs = pbc_bus_maxdevs;
	lcp->lc_pc.pc_make_tag = pbc_make_tag;
	lcp->lc_pc.pc_decompose_tag = pbc_decompose_tag;
	lcp->lc_pc.pc_conf_read = pbc_conf_read;
	lcp->lc_pc.pc_conf_write = pbc_conf_write;

	lcp->lc_pc.pc_intr_v = lcp;
	lcp->lc_pc.pc_intr_map = pbc_intr_map;
	lcp->lc_pc.pc_intr_string = pbc_intr_string;
	lcp->lc_pc.pc_intr_establish = pbc_intr_establish;
	lcp->lc_pc.pc_intr_disestablish = pbc_intr_disestablish;

	lcp->lc_pc.pc_ether_hw_addr = pbc_ether_hw_addr;
	lcp->lc_pc.pc_sync_cache = 0;
	lcp->lc_pc.pc_postsync_cache = 0;
	lcp->lc_pc.pc_vtop = pbc_vtop;
	
	lcp->lc_pc.pc_maxburst_log2 = _pci_maxburst_log2();
	lcp->lc_pc.pc_cacheline_log2 = _pci_cacheline_log2();

	pba.pba_busname = "pci";
	pba.pba_iot = &sc->sc_iobus_space;
	pba.pba_memt = &sc->sc_membus_space;
	pba.pba_dmat = &pci_bus_dma_tag;	/* XXX not used yet */
	pba.pba_pc = &lcp->lc_pc;
	pba.pba_bus = 0;
	pba.pba_flags = 0;
	
	/* board specific attach hook */
	_sbd_pcibus_attach (&pba);

	config_found(self, &pba, pcibrprint);
}

static int
pcibrprint(aux, pnp)
	void *aux;
	const char *pnp;
{
	struct pcibus_attach_args *pba = aux;

	if(pnp)
		printf("%s at %s", pba->pba_busname, pnp);
	printf(" bus %d", pba->pba_bus);
	return(UNCONF);
}


static void
pbc_attach_hook(parent, self, pba)
	struct device *parent, *self;
	struct pcibus_attach_args *pba;
{
    /* nothing */
}

static int
pbc_bus_maxdevs(cpv, busno)
	void *cpv;
	int busno;
{
    return 32;
}

static pcitag_t
pbc_make_tag(cpv, bus, dev, fnc)
	void *cpv;
	int bus, dev, fnc;
{
    return _pci_make_tag (bus, dev, fnc);
}

static void
pbc_decompose_tag(cpv, tag, busp, devp, fncp)
	void *cpv;
	pcitag_t tag;
	int *busp, *devp, *fncp;
{
    return _pci_break_tag (tag, busp, devp, fncp);
}

static pcireg_t
pbc_conf_read(cpv, tag, offset)
	void *cpv;
	pcitag_t tag;
	int offset;
{
    return _pci_conf_read (tag, offset);
}

static void
pbc_conf_write(cpv, tag, offset, data)
	void *cpv;
	pcitag_t tag;
	int offset;
	pcireg_t data;
{
    _pci_conf_write (tag, offset, data);
}

/* Return the interrupt "id" or "handle" for device TAG in *IHP.
   Note that IPIN and ILINE are reread by _pci_map_int, so these
   args are ignored. */

static int
pbc_intr_map (void *lcv, pcitag_t tag, int ipin, int iline,
	       pci_intr_handle_t *ihp)
{
    int intrnum;

    intrnum = _pci_map_int (tag);
    if (intrnum < 0)
	return 1;

    *ihp = intrnum;
    return 0;
}



/* Return a string representing the "interrupt handle" IH. */

const char *
pbc_intr_string (void *lcv, pci_intr_handle_t ih)
{
    static char str[16];
    sprintf (str, "intr%d", ih);
    return str;
}


/* Install an interrupt handler routine HANDLER for the interrupt
   handle IH, to be run at priority level IPL. */

static void *
pbc_intr_establish (void *lcv, pci_intr_handle_t ih, int ipl,
		     int (*handler)(void *), 
		     void *arg, char *name)
{
    return _kern_intr_establish (ih, ipl, handler, arg, name);
}


/* Remove interrupt handler COOKIE */

static void
pbc_intr_disestablish (void *lcv, void *cookie)
{
    return _kern_intr_disestablish (cookie);
}



static int
pbc_ether_hw_addr(u_int8_t *addr, int bus, int devno)
{
    return _pci_ether_hw_addr (addr, bus, devno);
}


static vm_offset_t
pbc_vtop (struct device *dv, vm_offset_t va)
{
    return _pci_vtop (dv, va);
}


