/*	$OpenBSD: if_le_pci.c,v 1.12.8.1 2001/02/01 02:01:09 jason Exp $	*/
/*	$NetBSD: if_le_pci.c,v 1.13 1996/10/25 21:33:32 cgd Exp $	*/

/*-
 * Copyright (c) 1995 Charles M. Hannum.  All rights reserved.
 * Copyright (c) 1992, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Ralph Campbell and Rick Macklem.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	@(#)if_le.c	8.2 (Berkeley) 11/16/93
 */

#include "bpfilter.h"

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/mbuf.h>
#include <sys/syslog.h>
#include <sys/socket.h>
#include <sys/device.h>

#include <vm/vm.h>
#include <vm/vm_kern.h>
#include <vm/vm_param.h>

#include <net/if.h>
#include <net/if_media.h>

#ifdef INET
#include <netinet/in.h>
#include <netinet/if_ether.h>
#endif

#include <vm/vm.h>

#if !defined(__AlgNET__)
#include <machine/cpu.h>
#endif
#include <machine/bus.h>
#include <machine/intr.h>

#include <dev/pci/pcireg.h>
#include <dev/pci/pcivar.h>
#include <dev/pci/pcidevs.h>

#include <dev/ic/am7990reg.h>
#include <dev/ic/am7990var.h>

#include <dev/pci/if_levar.h>

#ifdef __alpha__			/* XXX */
/* XXX XXX NEED REAL DMA MAPPING SUPPORT XXX XXX */ 
#undef vtophys
#define	vtophys(va)	alpha_XXX_dmamap((vm_offset_t)(va))
#endif

int le_pci_match __P((struct device *, void *, void *));
void le_pci_attach __P((struct device *, struct device *, void *));

struct cfattach le_pci_ca = {
	sizeof(struct le_softc), le_pci_match, le_pci_attach
};

#ifdef __mips__
#define DWIO			/* use DWIO accesses */
#endif

#ifdef DWIO
#define bus_space_read_r	bus_space_read_4
#define bus_space_write_r	bus_space_write_4
#else
#define bus_space_read_r	bus_space_read_2
#define bus_space_write_r	bus_space_write_2
#endif

hide void le_pci_wrcsr __P((struct am7990_softc *, u_int16_t, u_int16_t));
hide u_int16_t le_pci_rdcsr __P((struct am7990_softc *, u_int16_t));

/*
 * PCI constants.
 * XXX These should be in a common file!
 */
#define PCI_CBIO	0x10		/* Configuration Base IO Address */

hide void
le_pci_wrcsr(sc, port, val)
	struct am7990_softc *sc;
	u_int16_t port, val;
{
	struct le_softc *lesc = (struct le_softc *)sc;
	bus_space_tag_t iot = lesc->sc_iot;
	bus_space_handle_t ioh = lesc->sc_ioh;

	bus_space_write_r(iot, ioh, lesc->sc_rap, port);
	bus_space_write_r(iot, ioh, lesc->sc_rdp, val);
}

hide u_int16_t
le_pci_rdcsr(sc, port)
	struct am7990_softc *sc;
	u_int16_t port;
{
	struct le_softc *lesc = (struct le_softc *)sc;
	bus_space_tag_t iot = lesc->sc_iot;
	bus_space_handle_t ioh = lesc->sc_ioh;

	bus_space_write_r(iot, ioh, lesc->sc_rap, port);
	return bus_space_read_r(iot, ioh, lesc->sc_rdp);
}

int
le_pci_match(parent, match, aux)
	struct device *parent;
	void *match, *aux;
{
	struct pci_attach_args *pa = aux;

	if (PCI_VENDOR(pa->pa_id) != PCI_VENDOR_AMD)
		return (0);

	switch (PCI_PRODUCT(pa->pa_id)) {
	case PCI_PRODUCT_AMD_PCNET_PCI:
	case PCI_PRODUCT_AMD_PCHOME_PCI:
		return (1);
	}

	return (0);
}

static u_int16_t
le_pci_rdphy (sc, phyid, anr)
    struct am7990_softc *sc;
    int phyid, anr;
{
    struct le_softc *lesc = (struct le_softc *)sc;
    bus_space_tag_t iot = lesc->sc_iot;
    bus_space_handle_t ioh = lesc->sc_ioh;

    /* set PHY address register (PHY & ANR) */
    bus_space_write_r(iot, ioh, lesc->sc_rap, 33);
    bus_space_write_r(iot, ioh, lesc->sc_bdp, (phyid << 5) | anr);
    /* get PHY management data */
    bus_space_write_r(iot, ioh, lesc->sc_rap, 34);
    return bus_space_read_r(iot, ioh, lesc->sc_bdp);
}


static void
le_pci_hwinit (sc)
    struct am7990_softc *sc;
{
    struct le_softc *lesc = (struct le_softc *)sc;
    bus_space_tag_t iot = lesc->sc_iot;
    bus_space_handle_t ioh = lesc->sc_ioh;
    int timeout = 2400;	
    int phyid;

    /* get PHY address register (initialised by EEROM) */
    bus_space_write_r(iot, ioh, lesc->sc_rap, 33);
    phyid = (bus_space_read_r(iot, ioh, lesc->sc_bdp) >> 5) & 0x1f;

    /* latch new value */
    (void) le_pci_rdphy (sc, phyid, 1);

    /* wait for the link to come up */
    while (timeout > 0) {
	u_int16_t status;
	/* read PHY status reg */
	status = le_pci_rdphy (sc, phyid, 1);
	if (status & 0x4) {
	    /* link is up: read PHY summary status reg */
	    status = le_pci_rdphy (sc, phyid, 24);
	    sc->sc_arpcom.ac_if.if_baudrate = IF_Mbps((status & 1) ? 100 : 10);
	    if (!lesc->sc_linkup)
		printf ("%s: %dMb/s %cD link up\n", sc->sc_dev.dv_xname, 
			(status & 1) ? 100 : 10, (status & 4) ? 'F' : 'H');
	    lesc->sc_linkup = 1;
	    return;
	}
	DELAY (100000);
	timeout -= 100;
    }

    if (lesc->sc_linkup)
	printf ("%s: link down\n", sc->sc_dev.dv_xname);
    lesc->sc_linkup = 0;
}


void
le_pci_attach(parent, self, aux)
	struct device *parent, *self;
	void *aux;
{
	struct le_softc *lesc = (void *)self;
	struct am7990_softc *sc = &lesc->sc_am7990;
	struct pci_attach_args *pa = aux;
	pci_intr_handle_t ih;
	bus_addr_t iobase;
	bus_size_t iosize;
	bus_space_handle_t ioh;
	bus_space_tag_t iot = pa->pa_iot;
	pci_chipset_tag_t pc = pa->pa_pc;
	pcireg_t csr;
	int i, rseg;
	const char *intrstr;
	bus_dma_segment_t seg;
	caddr_t kva;

	switch (PCI_PRODUCT(pa->pa_id)) {
	case PCI_PRODUCT_AMD_PCNET_PCI:
	case PCI_PRODUCT_AMD_PCHOME_PCI:
#ifdef DWIO
		lesc->sc_rap = PCNET_PCI_DWIO_RAP;
		lesc->sc_rdp = PCNET_PCI_DWIO_RDP;
		lesc->sc_bdp = PCNET_PCI_DWIO_BDP;
#else
		lesc->sc_rap = PCNET_PCI_RAP;
		lesc->sc_rdp = PCNET_PCI_RDP;
		lesc->sc_bdp = PCNET_PCI_BDP;
#endif
		break;
	}

	if (pci_io_find(pc, pa->pa_tag, PCI_CBIO, &iobase, &iosize)) {
		printf(": can't find I/O base\n");
		return;
	}
	if (bus_space_map(iot, iobase, iosize, 0, &ioh)) {
		printf(": can't map I/O space\n");
		return;
	}

	/*
	 * Extract the physical MAC address from the ROM.
	 */
	for (i = 0; i < sizeof(sc->sc_arpcom.ac_enaddr); i++)
		sc->sc_arpcom.ac_enaddr[i] = bus_space_read_1(iot, ioh, i);

	/*
	 * Allocate a DMA area for the card.
	 */
	if (bus_dmamem_alloc(pa->pa_dmat, PCNET_MEMSIZE, PAGE_SIZE,
	    0, &seg, 1, &rseg, BUS_DMA_NOWAIT)) {
		printf(": couldn't allocate memory for card\n");
		return;
	}
	if (bus_dmamem_map(pa->pa_dmat, &seg, rseg, PCNET_MEMSIZE,
	    &kva, BUS_DMA_NOWAIT|BUS_DMA_COHERENT)) {
		printf(": couldn't map memory for card\n");
		bus_dmamem_free(pa->pa_dmat, &seg, rseg);
		return;
	}

	/*
	 * Create and load the DMA map for the DMA area.
	 */
	if (bus_dmamap_create(pa->pa_dmat, PCNET_MEMSIZE, 1, PCNET_MEMSIZE,
	    0, BUS_DMA_NOWAIT, &lesc->sc_dmamap)) {
		printf(": couldn't create dma map\n");
		bus_dmamem_unmap(pa->pa_dmat, kva, PCNET_MEMSIZE);
		bus_dmamem_free(pa->pa_dmat, &seg, rseg);
		return;
	}
	if (bus_dmamap_load(pa->pa_dmat, lesc->sc_dmamap, kva, PCNET_MEMSIZE,
	    NULL, BUS_DMA_NOWAIT)) {
		printf(": couldn't load dma map\n");
		bus_dmamap_destroy(pa->pa_dmat, lesc->sc_dmamap);
		bus_dmamem_unmap(pa->pa_dmat, kva, PCNET_MEMSIZE);
		bus_dmamem_free(pa->pa_dmat, &seg, rseg);
		return;
	}

	sc->sc_mem = kva;
	bzero(sc->sc_mem, PCNET_MEMSIZE);

	printf("\n");

	lesc->sc_iot = iot;
	lesc->sc_ioh = ioh;
	lesc->sc_dmat = pa->pa_dmat;

	/* DXSUFLO enables recovery from Tx underflow */
	sc->sc_conf3 = 1 << 6;		/* DXSUFLO */
	sc->sc_mode = LE_MODE_PORTSEL;	/* MBO for 973 */

	/* get DMA address from dmamap */
	sc->sc_addr = lesc->sc_dmamap->dm_segs[0].ds_addr;
	sc->sc_memsize = PCNET_MEMSIZE;

	sc->sc_copytodesc = am7990_copytodesc_contig;
	sc->sc_copyfromdesc = am7990_copyfromdesc_contig;
	sc->sc_copytobuf = am7990_copytobuf_contig;
	sc->sc_copyfrombuf = am7990_copyfrombuf_contig;
	sc->sc_zerobuf = am7990_zerobuf_contig;

#ifdef DWIO
	/* make sure device is in DWIO mode by doing initial cycle */
	le_pci_wrcsr(sc, LE_CSR0, 0);
#endif
	sc->sc_rdcsr = le_pci_rdcsr;
	sc->sc_wrcsr = le_pci_wrcsr;

	sc->sc_hwreset = NULL;
	sc->sc_hwinit = le_pci_hwinit;
        
        /* Make sure the device is in 16bit initialisation mode */
        le_pci_wrcsr(sc, 58, 0);
        
	printf("%s", sc->sc_dev.dv_xname);
	am7990_config(sc);

	/* boost FIFO thresholds */
	(sc->sc_wrcsr) (sc, 80, 
			(2 << 12)	/* RCVFW=112 */
			| (3 << 10)	/* XMTSP=220 */
			| (0 << 8));	/* XMTFW=16 */

#ifdef PCI_FLAGS_MRL_OKAY
	if (pa->pa_flags & (PCI_FLAGS_MRL_OKAY|PCI_FLAGS_MRM_OKAY)) {
	    u_int16_t bctrl;
	    bus_space_write_r(iot, ioh, lesc->sc_rap, 18);
	    bctrl = bus_space_read_r(iot, ioh, lesc->sc_bdp);
	    bctrl |= (1 << 6) | (1 << 5);	/* enable burst accesses */
	    if (pa->pa_flags & PCI_FLAGS_MRM_OKAY)
		bctrl |= (1 << 9);		/* use memory read multiple */
	    else
		bctrl &= ~(1 << 9);		/* use memory read line */
	    bus_space_write_r(iot, ioh, lesc->sc_bdp, bctrl);
	}
#endif

	/* Enable the card. */
	csr = pci_conf_read(pc, pa->pa_tag,
	    PCI_COMMAND_STATUS_REG);
	pci_conf_write(pc, pa->pa_tag, PCI_COMMAND_STATUS_REG,
	    csr | PCI_COMMAND_MASTER_ENABLE);

	/* Map and establish the interrupt. */
	if (pci_intr_map(pc, pa->pa_intrtag, pa->pa_intrpin,
	    pa->pa_intrline, &ih)) {
		printf("%s: couldn't map interrupt\n", sc->sc_dev.dv_xname);
		return;
	}
	intrstr = pci_intr_string(pc, ih);
	lesc->sc_ih = pci_intr_establish(pc, ih, IPL_NET, am7990_intr, sc,
	    sc->sc_dev.dv_xname);
	if (lesc->sc_ih == NULL) {
		printf("%s: couldn't establish interrupt",
		    sc->sc_dev.dv_xname);
		if (intrstr != NULL)
			printf(" at %s", intrstr);
		printf("\n");
		return;
	}
	printf("%s: interrupting at %s\n", sc->sc_dev.dv_xname, intrstr);
}
