/* 
 * Sockets API functions for AlgNET.
 *
 * Copyright (c) 2001 Algorithmics Ltd
 *
 * This program is NOT free software, it is supplied under the terms
 * of the AlgNET License Agreement, a copy of which is available at:
 *
 *  http://www.algor.co.uk/algor/info/algnet-license.pdf
 *
 * Any company which has obtained and signed a valid AlgNET license
 * may use and modify this software internally and use (without
 * restrictions) any derived binary.  You may not, however,
 * redistribute this in whole or in part as source code, nor may you
 * modify or remove any part of this copyright message.  
 */

/*	$OpenBSD: uipc_syscalls.c,v 1.32 2000/10/12 09:58:05 itojun Exp $	*/
/*	$NetBSD: uipc_syscalls.c,v 1.19 1996/02/09 19:00:48 christos Exp $	*/

/*
 * Copyright (c) 1982, 1986, 1989, 1990, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	@(#)uipc_syscalls.c	8.4 (Berkeley) 2/21/94
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/proc.h>
#include <sys/file.h>
#include <sys/malloc.h>
#include <sys/mbuf.h>
#include <sys/protosw.h>
#include <sys/socket.h>
#include <sys/socketvar.h>
#include <sys/un.h>

extern int errno;

/*
 * System call interface to the socket abstraction.
 */
extern	struct fileops socketops;

static int getsock __P((int, struct file **));

int
socket (domain, type, protocol)
{
    struct socket *so;
    struct file *fp;
    int error;
    int fd;

    _syscall_enter ();

    error = _fileio_falloc(&fp, &fd);
    if (error)
	goto done;
    
    fp->f_flag = FREAD|FWRITE;
    fp->f_type = DTYPE_SOCKET;
    fp->f_ops = &socketops;
    
    error = socreate(domain, &so, type, protocol);
    if (error == 0)
	fp->f_data = (void *)so;

done:
    if (error)
	_fileio_fdremove (fd);
    _fileio_frelease (fp);

    return _syscall_leave (error, fd);
}


int
bind(sd, name, namelen)
    int sd;
    const struct sockaddr * name;
    socklen_t namelen;
{
    struct file *fp;
    struct mbuf *nam;
    int error;

    _syscall_enter ();
    error = getsock (sd, &fp);
    if (error)
	goto err;

    error = sockargs(&nam, (caddr_t)name, namelen, MT_SONAME);
    if (error)
	goto err;

    error = sobind((struct socket *)fp->f_data, nam);
    m_freem(nam);

 err:
    _fileio_frelease (fp);
    return _syscall_leave (error, 0);
}


int
listen(sd, backlog)
    int sd;
    int backlog;
{
    struct file *fp;
    int error;

    _syscall_enter ();

    error = getsock (sd, &fp);
    if (error == 0)
	error = solisten((struct socket *)fp->f_data, backlog);
    _fileio_frelease (fp);

    return _syscall_leave (error, 0);
}


int
accept(sd, name, anamelen)
    int sd;
    struct sockaddr * name;
    socklen_t * anamelen;
{
    struct file *fp;
    struct mbuf *nam = 0;
    socklen_t namelen;
    int s, tmpfd = -1;
    struct socket *so;
    int error;

    _syscall_enter ();

    if (name && (error = copyin((caddr_t)anamelen, (caddr_t)&namelen,
				sizeof (namelen))))
	goto err;

    error = getsock (sd, &fp);
    if (error) 
	goto err;

    s = splsoftnet();
    so = (struct socket *)fp->f_data;
    if ((so->so_options & SO_ACCEPTCONN) == 0) {
	error = EINVAL;
	goto errspl;
    }
    if ((so->so_state & SS_NBIO) && so->so_qlen == 0) {
	error = EINVAL;
	goto errspl;
    }
    while (so->so_qlen == 0 && so->so_error == 0) {
	if (so->so_state & SS_CANTRCVMORE) {
	    so->so_error = ECONNABORTED;
	    break;
	}
	error = tsleep((caddr_t)&so->so_timeo, PSOCK | PCATCH,
		       netcon, 0);
	if (error)
	    goto errspl;
    }
    if (so->so_error) {
	error = so->so_error;
	so->so_error = 0;
	goto errspl;
    }

    /* release original file handle */
    _fileio_frelease(fp);

    /* allocate new file handle and descriptor */
    error = _fileio_falloc(&fp, &tmpfd);
    if (error)
	goto errspl;

    { struct socket *aso = so->so_q;
    if (soqremque(aso, 1) == 0)
	panic("accept");
    so = aso;
    }

    fp->f_type = DTYPE_SOCKET;
    fp->f_flag = FREAD|FWRITE;
    fp->f_ops = &socketops;
    fp->f_data = (caddr_t)so;

    nam = m_get(M_WAIT, MT_SONAME);
    (void) soaccept(so, nam);
    if (name) {
	if (namelen > nam->m_len)
	    namelen = nam->m_len;
	/* SHOULD COPY OUT A CHAIN HERE */
	if ((error = copyout(mtod(nam, caddr_t),
			     (caddr_t)name, namelen)) == 0)
	    error = copyout((caddr_t)&namelen,
			    (caddr_t)anamelen,
			    sizeof (*anamelen));
    }
    m_freem(nam);

 errspl:
    splx(s);
 err:
    if (error)
	_fileio_fdremove (tmpfd);
    _fileio_frelease (fp);
    return _syscall_leave (error, tmpfd);
}


int
connect(sd, name, namelen)
    int sd;
    const struct sockaddr * name;
    socklen_t namelen;
{
    struct file *fp;
    struct socket *so;
    struct mbuf *nam;
    int error, s;

    _syscall_enter ();

    error = getsock (sd, &fp);
    if (error)
	goto bad3;

    so = (struct socket *)fp->f_data;
    if ((so->so_state & SS_NBIO) && (so->so_state & SS_ISCONNECTING)) {
	error = EALREADY;
	goto bad3;
    }

    error = sockargs(&nam, (caddr_t)name, namelen, MT_SONAME);
    if (error)
	goto bad2;

    error = soconnect(so, nam);
    if (error)
	goto bad;

    if ((so->so_state & SS_NBIO) && (so->so_state & SS_ISCONNECTING)) {
	error = EINPROGRESS;
	goto bad2;
    }

    s = splsoftnet();
    while ((so->so_state & SS_ISCONNECTING) && so->so_error == 0) {
	error = tsleep((caddr_t)&so->so_timeo, PSOCK | PCATCH,
		       netcon, 0);
	if (error)
	    break;
    }
    if (error == 0) {
	error = so->so_error;
	so->so_error = 0;
    }
    splx(s);

 bad:
    so->so_state &= ~SS_ISCONNECTING;
 bad2:
    m_freem(nam);
 bad3:
    _fileio_frelease (fp);
    return _syscall_leave (error, 0);
}


int
socketpair(domain, type, protocol, rsv)
    int domain;
    int type;
    int protocol;
    int * rsv;
{
    struct file *fp1, *fp2;
    struct socket *so1, *so2;
    int fd, error, sv[2];

    _syscall_enter ();

    error = socreate(domain, &so1, type, protocol);
    if (error)
	goto err;

    error = socreate(domain, &so2, type, protocol);
    if (error)
	goto free1;

    error = _fileio_falloc(&fp1, &fd);
    if (error)
	goto free2;

    sv[0] = fd;
    fp1->f_flag = FREAD|FWRITE;
    fp1->f_type = DTYPE_SOCKET;
    fp1->f_ops = &socketops;
    fp1->f_data = (caddr_t)so1;

    error = _fileio_falloc(&fp2, &fd);
    if (error)
	goto free3;

    sv[1] = fd;
    fp2->f_flag = FREAD|FWRITE;
    fp2->f_type = DTYPE_SOCKET;
    fp2->f_ops = &socketops;
    fp2->f_data = (caddr_t)so2;

    if ((error = soconnect2(so1, so2)) != 0)
	goto free4;

    if (type == SOCK_DGRAM) {
	/*
	 * Datagram socket connection is asymmetric.
	 */
	if ((error = soconnect2(so2, so1)) != 0)
	    goto free4;
    }

    error = copyout((caddr_t)sv, (caddr_t)rsv, 2 * sizeof (int));
    if (error != 0)
	goto free4;

    _fileio_frelease (fp2);
    _fileio_frelease (fp1);
    return _syscall_leave (error, 0);

 free4:
    _fileio_fdremove(sv[1]);
    _fileio_frelease (fp2);
 free3:
    _fileio_fdremove(sv[0]);
    _fileio_frelease(fp1);
 free2:
    (void)soclose(so2);
 free1:
    (void)soclose(so1);
 err:
    return _syscall_leave (error, 0);
}


ssize_t
sendto(sd, buf, len, flags, to, tolen)
    int sd;
    const void * buf;
    size_t len;
    int flags;
    const struct sockaddr * to;
    socklen_t tolen;
{
    struct msghdr msg;
    struct iovec aiov;
    struct file *fp;
    ssize_t retval;
    int error;

    _syscall_enter ();

    msg.msg_name = (caddr_t)to;
    msg.msg_namelen = tolen;
    msg.msg_iov = &aiov;
    msg.msg_iovlen = 1;
    msg.msg_control = 0;
#ifdef COMPAT_OLDSOCK
    msg.msg_flags = 0;
#endif
    aiov.iov_base = (char *)buf;
    aiov.iov_len = len;

    error = getsock (sd, &fp);
    if (error == 0)
	error = sendit((struct socket *)fp->f_data, &msg, flags, &retval);
    _fileio_frelease (fp);

    return _syscall_leave (error, retval);
}


ssize_t
sendmsg(sd, amsg, flags)
    int sd;
    const struct msghdr *amsg;
    int flags;
{
    struct msghdr msg;
    struct iovec aiov[UIO_SMALLIOV], *iov = aiov;
    struct file *fp;
    ssize_t retval;
    int error;
    
    _syscall_enter ();

    error = getsock (sd, &fp);
    if (error)
	goto done;

    error = copyin(amsg, (caddr_t)&msg, sizeof (msg));
    if (error)
	goto done;
    if (msg.msg_iovlen <= 0 || msg.msg_iovlen > IOV_MAX) {
	error = EMSGSIZE;
	goto done;
    }
    if (msg.msg_iovlen > UIO_SMALLIOV)
	iov = malloc(sizeof(struct iovec) * msg.msg_iovlen,
		     M_IOV, M_WAITOK);
    if (msg.msg_iovlen &&
	(error = copyin((caddr_t)msg.msg_iov, (caddr_t)iov,
			(unsigned)(msg.msg_iovlen * sizeof (struct iovec)))))
	goto done;

    msg.msg_iov = iov;
#ifdef COMPAT_OLDSOCK
    msg.msg_flags = 0;
#endif

    error = sendit((struct socket *)fp->f_data, &msg, flags, &retval);

 done:
    if (iov != aiov)
	free(iov, M_IOV);
    _fileio_frelease (fp);

    return _syscall_leave (error, retval);
}


int
sendit(so, mp, flags, retsize)
    struct socket *so;
    struct msghdr *mp;
    int flags;
    ssize_t *retsize;
{
    struct uio auio;
    register struct iovec *iov;
    register int i;
    struct mbuf *to, *control;
    int len, error;
	
    auio.uio_iov = mp->msg_iov;
    auio.uio_iovcnt = mp->msg_iovlen;
    auio.uio_segflg = UIO_USERSPACE;
    auio.uio_rw = UIO_WRITE;
    auio.uio_procp = curproc;
    auio.uio_offset = 0;			/* XXX */
    auio.uio_resid = 0;
    iov = mp->msg_iov;
    for (i = 0; i < mp->msg_iovlen; i++, iov++) {
	/* Don't allow sum > SSIZE_MAX */
	if (iov->iov_len > SSIZE_MAX ||
	    (auio.uio_resid += iov->iov_len) > SSIZE_MAX)
	    return (EINVAL);
    }
    if (mp->msg_name) {
	error = sockargs(&to, mp->msg_name, mp->msg_namelen,
			 MT_SONAME);
	if (error)
	    return (error);
    } else
	to = 0;
    if (mp->msg_control) {
	if (mp->msg_controllen < sizeof(struct cmsghdr)
#ifdef COMPAT_OLDSOCK
	    && mp->msg_flags != MSG_COMPAT
#endif
	    ) {
	    error = EINVAL;
	    goto bad;
	}
	error = sockargs(&control, mp->msg_control,
			 mp->msg_controllen, MT_CONTROL);
	if (error)
	    goto bad;
#ifdef COMPAT_OLDSOCK
	if (mp->msg_flags == MSG_COMPAT) {
	    register struct cmsghdr *cm;

	    M_PREPEND(control, sizeof(*cm), M_WAIT);
	    if (control == 0) {
		error = ENOBUFS;
		goto bad;
	    } else {
		cm = mtod(control, struct cmsghdr *);
		cm->cmsg_len = control->m_len;
		cm->cmsg_level = SOL_SOCKET;
		cm->cmsg_type = SCM_RIGHTS;
	    }
	}
#endif
    } else
	control = 0;
    len = auio.uio_resid;
    error = sosend(so, to, &auio, NULL, control, flags);
    if (error) {
	if (auio.uio_resid != len && (error == EINTR || error == EWOULDBLOCK))
	    error = 0;
	if (error == EPIPE)
	    psignal(curproc, SIGPIPE);
    }
    if (error == 0)
	*retsize = len - auio.uio_resid;
 bad:
    if (to)
	m_freem(to);
    return (error);
}


ssize_t
recvfrom(sd, buf, len, flags, from, fromlenaddr)
    int sd;
    void *buf;
    size_t len;
    int flags;
    struct sockaddr * from;
    socklen_t * fromlenaddr;
{
    struct msghdr msg;
    struct iovec aiov;
    struct file *fp;
    ssize_t retval;
    int error;

    _syscall_enter ();

    error = getsock (sd, &fp);
    if (error)
	goto done;

    if (fromlenaddr) {
	error = copyin((caddr_t)fromlenaddr, (caddr_t)&msg.msg_namelen,
		       sizeof (msg.msg_namelen));
	if (error)
	    goto done;
    } else
	msg.msg_namelen = 0;

    msg.msg_name = (caddr_t) from;
    msg.msg_iov = &aiov;
    msg.msg_iovlen = 1;
    aiov.iov_base = buf;
    aiov.iov_len = len;
    msg.msg_control = 0;
    msg.msg_flags = flags;

    error = recvit((struct socket *)fp->f_data, &msg, (caddr_t)fromlenaddr,
		   &retval);

 done:
    _fileio_frelease (fp);
    return _syscall_leave (error, retval);
}


ssize_t
recvmsg(sd, amsg, flags)
    int sd;
    struct msghdr * amsg;
    int flags;
{
    struct msghdr msg;
    struct iovec aiov[UIO_SMALLIOV], *uiov, *iov = aiov;
    struct file *fp;
    ssize_t retval;
    int error;

    _syscall_enter ();

    error = getsock (sd, &fp);
    if (error)
	goto done;

    error = copyin((caddr_t)amsg, (caddr_t)&msg, sizeof (msg));
    if (error)
	goto done;

    if (msg.msg_iovlen <= 0 || msg.msg_iovlen > IOV_MAX) {
	error = EMSGSIZE;
	goto done;
    }

    if (msg.msg_iovlen > UIO_SMALLIOV)
	iov = malloc(sizeof(struct iovec) * msg.msg_iovlen,
		     M_IOV, M_WAITOK);

#ifdef COMPAT_OLDSOCK
    msg.msg_flags = flags &~ MSG_COMPAT;
#else
    msg.msg_flags = flags;
#endif
    uiov = msg.msg_iov;
    msg.msg_iov = iov;

    error = copyin((caddr_t)uiov, (caddr_t)iov,
		   (unsigned)(msg.msg_iovlen * sizeof (struct iovec)));
    if (error)
	goto done;

    if ((error = recvit((struct socket *)fp->f_data, &msg,
			(caddr_t)0, &retval)) == 0) {
	msg.msg_iov = uiov;
	error = copyout((caddr_t)&msg, amsg, sizeof(msg));
    }

 done:
    if (iov != aiov)
	free(iov, M_IOV);
    _fileio_frelease (fp);
    return _syscall_leave (error, retval);
}


int
recvit(so, mp, namelenp, retsize)
    struct socket *so;
    register struct msghdr *mp;
    caddr_t namelenp;
    ssize_t *retsize;
{
    struct uio auio;
    register struct iovec *iov;
    register int i;
    size_t len;
    int error;
    struct mbuf *from = 0, *control = 0;
	
    auio.uio_iov = mp->msg_iov;
    auio.uio_iovcnt = mp->msg_iovlen;
    auio.uio_segflg = UIO_USERSPACE;
    auio.uio_rw = UIO_READ;
    auio.uio_procp = curproc;
    auio.uio_offset = 0;			/* XXX */
    auio.uio_resid = 0;
    iov = mp->msg_iov;
    for (i = 0; i < mp->msg_iovlen; i++, iov++) {
	/* Don't allow sum > SSIZE_MAX */
	if (iov->iov_len > SSIZE_MAX ||
	    (auio.uio_resid += iov->iov_len) > SSIZE_MAX)
	    return (EINVAL);
    }

    len = auio.uio_resid;
    error = soreceive(so, &from, &auio,
		      NULL, mp->msg_control ? &control : NULL,
		      &mp->msg_flags);
    if (error) {
	if (auio.uio_resid != len && (error == EINTR || error == EWOULDBLOCK))
	    error = 0;
    }
    if (error)
	goto out;
    *retsize = len - auio.uio_resid;

    if (mp->msg_name) {
	len = mp->msg_namelen;
	if (len <= 0 || from == 0)
	    len = 0;
	else {
	    /* save sa_len before it is destroyed by MSG_COMPAT */
	    if (len > from->m_len)
		len = from->m_len;
	    /* else if len < from->m_len ??? */
#ifdef COMPAT_OLDSOCK
	    if (mp->msg_flags & MSG_COMPAT)
		mtod(from, struct osockaddr *)->sa_family =
		    mtod(from, struct sockaddr *)->sa_family;
#endif
	    error = copyout(mtod(from, caddr_t),
			    (caddr_t)mp->msg_name, (unsigned)len);
	    if (error)
		goto out;
	}
	mp->msg_namelen = len;
	if (namelenp &&
	    (error = copyout((caddr_t)&len, namelenp, sizeof (int)))) {
#ifdef COMPAT_OLDSOCK
	    if (mp->msg_flags & MSG_COMPAT)
		error = 0;	/* old recvfrom didn't check */
	    else
#endif
		goto out;
	}
    }
    if (mp->msg_control) {
#ifdef COMPAT_OLDSOCK
	/*
	 * We assume that old recvmsg calls won't receive access
	 * rights and other control info, esp. as control info
	 * is always optional and those options didn't exist in 4.3.
	 * If we receive rights, trim the cmsghdr; anything else
	 * is tossed.
	 */
	if (control && mp->msg_flags & MSG_COMPAT) {
	    if (mtod(control, struct cmsghdr *)->cmsg_level !=
		SOL_SOCKET ||
		mtod(control, struct cmsghdr *)->cmsg_type !=
		SCM_RIGHTS) {
		mp->msg_controllen = 0;
		goto out;
	    }
	    control->m_len -= sizeof (struct cmsghdr);
	    control->m_data += sizeof (struct cmsghdr);
	}
#endif
	len = mp->msg_controllen;
	if (len <= 0 || control == 0)
	    len = 0;
	else {
	    struct mbuf *m = control;
	    caddr_t p = (caddr_t)mp->msg_control;
	    
	    do {
		i = m->m_len;
		if (len < i) {
		    mp->msg_flags |= MSG_CTRUNC;
		    i = len;
		}
		error = copyout(mtod(m, caddr_t), p,
				(unsigned)i);
		if (m->m_next)
		    i = ALIGN(i);
		p += i;
		len -= i;
		if (error != 0 || len <= 0)
		    break;
	    } while ((m = m->m_next) != NULL);
	    len = p - (caddr_t)mp->msg_control;
	}
	mp->msg_controllen = len;
    }
 out:
    if (from)
	m_freem(from);
    if (control)
	m_freem(control);
    return (error);
}


/* ARGSUSED */
int
shutdown(s, how)
    int s, how;
{
    struct file *fp;
    int error;
    
    _syscall_enter ();

    error = getsock (s, &fp);
    if (error == 0)
	error = soshutdown((struct socket *)fp->f_data, how);
    _fileio_frelease (fp);

    return _syscall_leave (error, 0);
}


int
setsockopt(sd, level, name, val, valsize)
    int sd;
    int level;
    int name;
    const void * val;
    socklen_t valsize;
{
    struct file *fp;
    struct mbuf *m = NULL;
    int error;
    
    _syscall_enter ();

    error = getsock (sd, &fp);
    if (error)
	goto done;

    if (valsize > MCLBYTES) {
	error = EINVAL;
	goto done;
    }

    if (val) {
	m = m_get(M_WAIT, MT_SOOPTS);
	if (valsize > MLEN) {
	    MCLGET(m, M_DONTWAIT);
	    if ((m->m_flags & M_EXT) == 0) {
		m_freem(m);
		error = ENOBUFS;
		goto done;
	    }
	}
	if (m == NULL) {
	    error = ENOBUFS;
	    goto done;
	}
	error = copyin(val, mtod(m, caddr_t), valsize);
	if (error) {
	    (void) m_free(m);
	    goto done;
	}
	m->m_len = valsize;
    }

    error = sosetopt((struct socket *)fp->f_data, level, name, m);

done:
    _fileio_frelease (fp);
    return _syscall_leave (error, 0);
}


int
getsockopt(sd, level, name, val, avalsize)
    int sd;
    int level;
    int name;
    void * val;
    socklen_t * avalsize;
{
    struct file *fp;
    struct mbuf *m = NULL;
    socklen_t valsize;
    int error;
    
    _syscall_enter ();

    error = getsock (sd, &fp);
    if (error)
	goto done;

    if (val) {
	error = copyin((caddr_t)avalsize, (caddr_t)&valsize, sizeof (valsize));
	if (error)
	    goto done;
    } else
	valsize = 0;

    error = sogetopt((struct socket *)fp->f_data, level, name, &m);
    if (error == 0 && val && valsize && m != NULL) {
	if (valsize > m->m_len)
	    valsize = m->m_len;
	error = copyout(mtod(m, caddr_t), val, valsize);
	if (error == 0)
	    error = copyout((caddr_t)&valsize, (caddr_t)avalsize, sizeof (valsize));
    }
    if (m != NULL)
	(void) m_free(m);

done:
    _fileio_frelease (fp);
    return _syscall_leave (error, 0);
}


int	pipe __P((int [2]));

int
pipe(retval)
    int retval[2];
{
    struct file *rf, *wf;
    struct socket *rso, *wso;
    int fd, error;
    int rv[2];

    _syscall_enter ();

    if ((error = socreate(AF_UNIX, &rso, SOCK_STREAM, 0)) != 0)
	return (error);
    if ((error = socreate(AF_UNIX, &wso, SOCK_STREAM, 0)) != 0)
	goto free1;

    error = _fileio_falloc (&rf, &fd);
    if (error)
	goto free2;

    rv[0] = fd;
    rf->f_flag = FREAD;
    rf->f_type = DTYPE_SOCKET;
    rf->f_ops = &socketops;
    rf->f_data = (caddr_t)rso;

    error = _fileio_falloc (&wf, &fd);
    if (error)
	goto free3;

    rv[1] = fd;
    wf->f_flag = FWRITE;
    wf->f_type = DTYPE_SOCKET;
    wf->f_ops = &socketops;
    wf->f_data = (caddr_t)wso;

    if ((error = unp_connect2(wso, rso)) != 0)
	goto free4;
    
    error = copyout((caddr_t)rv, (caddr_t)retval, 2 * sizeof (int));
    if (error) 
	goto free4;

    _fileio_frelease (wf);
    _fileio_frelease (rf);
    return _syscall_leave (error, 0);

 free4:
    _fileio_frelease (wf);
    _fileio_fdremove(rv[1]);
 free3:
    _fileio_frelease (rf);
    _fileio_fdremove(rv[0]);
 free2:
    (void)soclose(wso);
 free1:
    (void)soclose(rso);
    return _syscall_leave (error, 0);
}


/*
 * Get socket name.
 */
int
getsockname(sd, asa, alen)
    int sd;
    struct sockaddr * asa;
    socklen_t * alen;
{
    struct file *fp;
    register struct socket *so;
    struct mbuf *m = 0;
    socklen_t len;
    int error;

    _syscall_enter ();

    error = getsock (sd, &fp);
    if (error)
	goto done;

    error = copyin((caddr_t)alen, (caddr_t)&len, sizeof (len));
    if (error)
	goto done;

    m = m_getclr(M_WAIT, MT_SONAME);
    if (m == NULL) {
	error = ENOBUFS;
	goto done;
    }

    so = (struct socket *)fp->f_data;

    error = (*so->so_proto->pr_usrreq)(so, PRU_SOCKADDR, 0, m, 0);
    if (error)
	goto done;

    if (len > m->m_len)
	len = m->m_len;

    error = copyout(mtod(m, caddr_t), (caddr_t)asa, len);
    if (error == 0)
	error = copyout((caddr_t)&len, (caddr_t)alen, sizeof (len));

 done:
    if (m)
	m_freem(m);
    _fileio_frelease (fp);
    return _syscall_leave (error, 0);
}


/*
 * Get name of peer for connected socket.
 */
int
getpeername(sd, asa, alen)
    int sd;
    struct sockaddr * asa;
    socklen_t * alen;
{
    struct file *fp;
    register struct socket *so;
    socklen_t len;
    struct mbuf *m = 0;
    int error;

    _syscall_enter ();

    error = getsock (sd, &fp);
    if (error)
	goto done;

    so = (struct socket *)fp->f_data;
    if ((so->so_state & (SS_ISCONNECTED|SS_ISCONFIRMING)) == 0) {
	error = ENOTCONN;
	goto done;
    }

    error = copyin((caddr_t)alen, (caddr_t)&len, sizeof (len));
    if (error)
	goto done;

    m = m_getclr(M_WAIT, MT_SONAME);
    if (m == NULL) {
	error = ENOBUFS;
	goto done;
    }

    error = (*so->so_proto->pr_usrreq)(so, PRU_PEERADDR, 0, m, 0);
    if (error)
	goto done;

    if (len > m->m_len)
	len = m->m_len;
    error = copyout(mtod(m, caddr_t), (caddr_t)asa, len);
    if (error == 0)
	error = copyout((caddr_t)&len, (caddr_t)alen, sizeof (len));

 done:
    if (m)
	m_freem(m);
    _fileio_frelease (fp);
    return _syscall_leave (error, 0);
}


int
sockargs(mp, buf, buflen, type)
    struct mbuf **mp;
    caddr_t buf;
    socklen_t buflen;
    int type;
{
    register struct sockaddr *sa;
    register struct mbuf *m;
    int error;

    /*
     * We can't allow socket names > UCHAR_MAX in length, since that
     * will overflow sa_len.
     */
    if (type == MT_SONAME && (u_int)buflen > UCHAR_MAX)
	return (EINVAL);
    if ((u_int)buflen > MCLBYTES)
	return (EINVAL);

    /* Allocate an mbuf to hold the arguments. */
    m = m_get(M_WAIT, type);
    if ((u_int)buflen > MLEN) {
	MCLGET(m, M_WAITOK);
	if ((m->m_flags & M_EXT) == 0) {
	    m_free(m);
	    return ENOBUFS;
	}
    }
    m->m_len = buflen;
    error = copyin(buf, mtod(m, caddr_t), buflen);
    if (error) {
	(void) m_free(m);
	return (error);
    }
    *mp = m;
    if (type == MT_SONAME) {
	sa = mtod(m, struct sockaddr *);

#if defined(COMPAT_OLDSOCK) && BYTE_ORDER != BIG_ENDIAN
	if (sa->sa_family == 0 && sa->sa_len < AF_MAX)
	    sa->sa_family = sa->sa_len;
#endif
	sa->sa_len = buflen;
    }
    return (0);
}


static int
getsock(fd, fpp)
	int fd;
	struct file **fpp;
{
	struct file *fp;
	int error;

	error = _fileio_fget (fd, &fp);
	if (error == 0 && fp->f_type != DTYPE_SOCKET) {
	    _fileio_frelease (fp);
	    error = ENOTSOCK;
	}
	*fpp = error ? NULL : fp;
	return (error);
}



/* Other miscellaneou system call emulations */

int gethostid (void);
int gethostname (char *, size_t);
int sethostname (const char *, size_t);

static int hostid = 123;
static char hostname[MAXHOSTNAMELEN];
static int hostnamelen;


int
gethostid()
{
    return hostid;
}


int
gethostname (char *buf, size_t len)
{
    int error;
    if (len < hostnamelen)
	error = EINVAL;
    else
	error = copyout (hostname, buf, hostnamelen);
    if (error) {
	errno = error;
	return -1;
    }
    return 0;
}


int
sethostname (const char *buf, size_t len)
{
    int error;
    if (len > sizeof(hostname))
	error = EINVAL;
    else
	error = copyin (buf, hostname, len);
    if (error) {
	errno = error;
	return -1;
    }
    hostnamelen = len;
    return 0;
}
