/* 
 * Kernel memory management glue for AlgNET.
 *
 * Copyright (c) 2001 Algorithmics Ltd
 *
 * This program is NOT free software, it is supplied under the terms
 * of the AlgNET License Agreement, a copy of which is available at:
 *
 *  http://www.algor.co.uk/algor/info/algnet-license.pdf
 *
 * Any company which has obtained and signed a valid AlgNET license
 * may use and modify this software internally and use (without
 * restrictions) any derived binary.  You may not, however,
 * redistribute this in whole or in part as source code, nor may you
 * modify or remove any part of this copyright message.  
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/proc.h>
#include <sys/syslog.h>

#include <vm/vm.h>
#include <vm/vm_kern.h>

vm_map_t	kernel_map;
vm_map_t	kmem_map;
vm_map_t	mb_map;

#define NVMMAPS 3
static struct vm_map vm_maps[NVMMAPS];
static int nvmmaps;

void
kmem_init(start, end)
	vm_offset_t start, end;
{
    if (!kernel_map) {
	kernel_map = &vm_maps[nvmmaps++];
	kernel_map->next = kernel_map->base = start;
	kernel_map->lim = end;
    }
}


vm_offset_t
kmem_malloc (map, size, canwait)
    vm_map_t map;
    vm_size_t size;
{
    vm_offset_t p;

    size = round_page (size);
    if (map->next + size > map->lim) {
	log (LOG_DEBUG, "kmem_malloc(%d) fails\n", size);
	return 0;
    }
    p = map->next;
    map->next += size;
    return p;
}


vm_offset_t
kmem_alloc (map, size)
    vm_map_t map;
    vm_size_t size;
{
    return kmem_malloc (map, size, 0);
}


vm_map_t
kmem_suballoc (parent, min, max, size, pageable)
    vm_map_t parent;
    vm_offset_t *min, *max;
    vm_size_t size;
{
    vm_map_t res;
    vm_offset_t base;

    if (nvmmaps >= NVMMAPS)
	panic ("kmem_suballoc: too many maps");
    res = &vm_maps[nvmmaps++];

    base = kmem_malloc (parent, size, 0);
    if (!base)
	panic ("kmem_suballoc: no space");

    res->next = res->base = base;
    res->lim = base + size;
    *min = res->base;
    *max = res->lim;
    return res;
}


void
kmem_free (map, addr, size)
    vm_map_t map;
    vm_offset_t addr;
    vm_size_t size;
{
    /* We don't handle freeing memory back to the map yet */
    panic ("kmem_free");
}


int
kernacc(addr, len, rw)
    caddr_t addr;
    int len, rw;
{
    vm_offset_t saddr, eaddr;
    saddr = trunc_page(addr);
    eaddr = round_page(addr+len);
    return (saddr >= kernel_map->base && eaddr <= kernel_map->lim);
}
