/* @(#) $Header: /cvsroot/internal/sde/netxampl/netmon/bootp.h,v 1.1 2001/06/04 15:01:48 nigel Exp $ (LBL) */
/*
 * Bootstrap Protocol (BOOTP).  RFC951 and RFC1048.
 *
 * This file specifies the "implementation-independent" BOOTP protocol
 * information which is common to both client and server.
 *
 * Copyright 1988 by Carnegie Mellon.
 *
 * Permission to use, copy, modify, and distribute this program for any
 * purpose and without fee is hereby granted, provided that this copyright
 * and permission notice appear on all copies and supporting documentation,
 * the name of Carnegie Mellon not be used in advertising or publicity
 * pertaining to distribution of the program without specific prior
 * permission, and notice be given in supporting documentation that copying
 * and distribution is by permission of Carnegie Mellon and Stanford
 * University.  Carnegie Mellon makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 */

#define BOOTP_MAXVEND		64
#define DHCP_MAXVEND		312

struct bootp {
	unsigned char	bp_op;		/* packet opcode type */
	unsigned char	bp_htype;	/* hardware addr type */
	unsigned char	bp_hlen;	/* hardware addr length */
	unsigned char	bp_hops;	/* gateway hops */
	unsigned long	bp_xid;		/* transaction ID */
	unsigned short	bp_secs;	/* seconds since boot began */
	unsigned short	bp_unused;
	struct in_addr	bp_ciaddr;	/* client IP address */
	struct in_addr	bp_yiaddr;	/* 'your' IP address */
	struct in_addr	bp_siaddr;	/* server IP address */
	struct in_addr	bp_giaddr;	/* gateway IP address */
	unsigned char	bp_chaddr[16];	/* client hardware address */
	unsigned char	bp_sname[64];	/* server host name */
	unsigned char	bp_file[128];	/* boot file name */
	unsigned char	bp_vend[312];	/* vendor-specific area */
};

/*
 * UDP port numbers, server and client.
 */
#define	IPPORT_BOOTPS		67
#define	IPPORT_BOOTPC		68

/* bootp opcodes */
#define BOOTREQUEST		1
#define BOOTREPLY		2

/* dhcp message types */
#define DHCPDISCOVER		1
#define DHCPOFFER		2
#define DHCPREQUEST		3
#define DHCPDECLINE		4
#define DHCPACK			5
#define DHCPNAK			6
#define DHCPRELEASE		7
#define DHCPINFORM		8

/*
 * Hardware types from Assigned Numbers RFC.
 */
#define HTYPE_ETHERNET		  1
#define HTYPE_EXP_ETHERNET	  2
#define HTYPE_AX25		  3
#define HTYPE_PRONET		  4
#define HTYPE_CHAOS		  5
#define HTYPE_IEEE802		  6
#define HTYPE_ARCNET		  7


/*
 * Vendor magic cookie (v_magic) for CMU
 */
#define VM_CMU		"CMU"

/*
 * Vendor magic cookie (v_magic) for RFC1048
 */
#define VM_RFC1048	{ 99, 130, 83, 99 }



/*
 * Tag values used to specify what information is being supplied in
 * the vendor (options) data area of the packet.
 */

/* RFC 1497 extensions */
#define TAG_END			((unsigned char) 255)
#define TAG_PAD			((unsigned char)   0)
#define TAG_SUBNET_MASK		((unsigned char)   1)
#define TAG_TIME_OFFSET		((unsigned char)   2)
#define TAG_GATEWAY		((unsigned char)   3)
#define TAG_TIME_SERVER		((unsigned char)   4)
#define TAG_NAME_SERVER		((unsigned char)   5)
#define TAG_DOMAIN_SERVER	((unsigned char)   6)
#define TAG_LOG_SERVER		((unsigned char)   7)
#define TAG_COOKIE_SERVER	((unsigned char)   8)
#define TAG_LPR_SERVER		((unsigned char)   9)
#define TAG_IMPRESS_SERVER	((unsigned char)  10)
#define TAG_RLP_SERVER		((unsigned char)  11)
#define TAG_HOST_NAME		((unsigned char)  12)
#define TAG_BOOT_SIZE		((unsigned char)  13)
#define TAG_DUMP_FILE		((unsigned char)  14)
#define TAG_DOMAIN_NAME		((unsigned char)  15)
#define TAG_SWAP_SERVER		((unsigned char)  16)
#define TAG_ROOT_PATH		((unsigned char)  17)
#define TAG_EXTEN_FILE		((unsigned char)  18)
/* IP layer parameters per host */
#define TAG_IPHOST_FORWARD	((unsigned char)  19)
#define TAG_IPHOST_SRCROUTE	((unsigned char)  20)
#define TAG_IPHOST_FILTER	((unsigned char)  21)
#define TAG_IPHOST_MTU		((unsigned char)  22)
#define TAG_IPHOST_TTL		((unsigned char)  23)
#define TAG_IPHOST_MTU_TIMEOUT	((unsigned char)  24)
#define TAG_IPHOST_MTU_PLATEAU	((unsigned char)  25)
/* IP layer perameters per interface */
#define TAG_IP_MTU		((unsigned char)  26)
#define TAG_IP_SUBNETSLOCAL	((unsigned char)  27)
#define TAG_IP_BROADADDR	((unsigned char)  28)
#define TAG_IP_MASK_DISCOVER	((unsigned char)  29)
#define TAG_IP_MASK_SUPPLIER	((unsigned char)  30)
#define TAG_IP_ROUTE_DISCOVER	((unsigned char)  31)
#define TAG_IP_ROUTE_SOLICIT	((unsigned char)  32)
#define TAG_IP_ROUTE_STATIC	((unsigned char)  33)
/* Link-layer parameters per interface */
#define TAG_ETH_TRAILERS	((unsigned char)  34)
#define TAG_ETH_ARPTIMEOUT	((unsigned char)  35)
#define TAG_ETH_ENCAP		((unsigned char)  36)
/* TCP parameters per interface */
#define TAG_TCP_TTL		((unsigned char)  37)
#define TAG_TCP_KEEPALIVE_INTVL	((unsigned char)  38)
#define TAG_TCP_KEEPALIVE_GARB	((unsigned char)  39)
/* Application and Service parameters */
#define TAG_NIS_DOMAIN		((unsigned char)  40)
#define TAG_NIS_SERVER		((unsigned char)  41)
#define TAG_NTP_SERVER		((unsigned char)  42)
#define TAG_XFNT_SERVER		((unsigned char)  48)
#define TAG_XDM_SERVER		((unsigned char)  49)
/* DHCP extensions */
#define TAG_DHCP_REQADDR	((unsigned char)  50)
#define TAG_DHCP_LEASE		((unsigned char)  51)
#define TAG_DHCP_OVERLOAD	((unsigned char)  52)
#define TAG_DHCP_MSGTYPE	((unsigned char)  53)
#define TAG_DHCP_SERVER		((unsigned char)  54)
#define TAG_DHCP_OPTIONREQ	((unsigned char)  55)
#define TAG_DHCP_MESSAGE	((unsigned char)  56)
#define TAG_DHCP_MAXMSGSZ	((unsigned char)  57)
#define TAG_DHCP_T1		((unsigned char)  58)
#define TAG_DHCP_T2		((unsigned char)  59)
#define TAG_DHCP_CLASS		((unsigned char)  60)
#define TAG_DHCP_CLIENTID	((unsigned char)  61)
#define TAG_TFTP_SERVER		((unsigned char)  66)
#define TAG_BOOTFILE_NAME	((unsigned char)  67)
#define TAG_SMTP_SERVER		((unsigned char)  69)
#define TAG_POP3_SERVER		((unsigned char)  70)
#define TAG_NNTP_SERVER		((unsigned char)  71)
#define TAG_HTTP_SERVER		((unsigned char)  72)
#define TAG_FINGER_SERVER	((unsigned char)  73)
#define TAG_IRC_SERVER		((unsigned char)  74)
#define TAG_STREETTALK_SERVER	((unsigned char)  75)
#define TAG_STDA_SERVER		((unsigned char)  76)

/* XXX - Add new tags here */


/*
 * "vendor" data permitted for CMU bootp clients.
 */

struct cmu_vend {
	unsigned char	v_magic[4];	/* magic number */
	unsigned long	v_flags;	/* flags/opcodes, etc. */
	struct in_addr	v_smask;	/* Subnet mask */
	struct in_addr	v_dgate;	/* Default gateway */
	struct in_addr	v_dns1, v_dns2; /* Domain name servers */
	struct in_addr	v_ins1, v_ins2; /* IEN-116 name servers */
	struct in_addr	v_ts1, v_ts2;	/* Time servers */
	unsigned char	v_unused[25];	/* currently unused */
};


/* v_flags values */
#define VF_SMASK	1	/* Subnet mask field contains valid data */
