/* 
 * bootparams.c: Network bootstrap/kickstart data
 *
 * Copyright (c) 2001 Algorithmics Ltd - all rights reserved.
 * 
 * This program is NOT free software, it is supplied under the terms
 * of the AlgRTX License Agreement, a copy of which is available at:
 *
 *  http://www.algor.co.uk/algor/info/algrtx-license.pdf
 *
 * Any company which has obtained and signed a valid AlgRTX license
 * may use and modify this software internally and use (without
 * restrictions) any derived binary.  You may not, however,
 * redistribute this in whole or in part as source code, nor may you
 * modify or remove any part of this copyright message.  
 */

#define BOOT_MAXADDR	4

struct bootpaddrs {
    struct in_addr	addrs[BOOT_MAXADDR];
    int			naddr;
};

struct bootparams {
    SLIST_ENTRY(bootparams) list;
    char		*name;		/* interface name */
    unsigned short	iflags;		/* interface flags */
    unsigned short	need;		/* required parameters */
    unsigned short	have;		/* acquired parameters */
    int 		xid;		/* last xid */
    struct in_addr	addr;		/* interface address */
    struct in_addr	nmask;		/* natural net mask */
    struct in_addr	mask;		/* returned subnet mask */
    struct in_addr	broadaddr;	/* broadcast address */
    struct in_addr	bootip;		/* boot host address */
    char		*bootfile;	/* file to bootstrap from */
    char		*hostname;	/* our host name */
    char		*domain;	/* our DNS domain name */
    struct bootpaddrs	dnsip;		/* DNS servers */
    struct bootpaddrs	gateip;		/* default routes (gateway) */
    struct bootpaddrs	logip;		/* syslog server */
#if 0
    /* XXX not used yet */
    struct bootpaddrs	ntp;
#endif
};

#define	BOOT_ADDR		0x0001
#define	BOOT_MASK		0x0002
#define	BOOT_GATEIP		0x0004
#define	BOOT_DNSIP		0x0008
#define	BOOT_BOOTIP		0x0010
#define BOOT_HOSTNAME		0x0020
#define BOOT_DOMAINNAME		0x0040
#define BOOT_BOOTFILE		0x0080
#define BOOT_BROADADDR		0x0100
#define BOOT_LOGIP		0x0200
#define BOOT_NTPIP		0x0400
#define BOOT_IFCONFIG		0x7000	/* how to configure interface: */
# define BOOT_IFCONFIG_DHCP	 0x0000	  /* use DHCP, then... */
# define BOOT_IFCONFIG_BOOTP	 0x1000	  /* use BOOTP, then... */
# define BOOT_IFCONFIG_RARP	 0x2000	  /* use RARP, then... */
# define BOOT_IFCONFIG_ENV	 0x3000	  /* use non-volatile environment */
# define BOOT_IFCONFIG_SKIP	 0x4000	  /* skip this interface */
#define BOOT_ONLYIF		0x8000
