/* Stripped down teeny-tiny version of inetd, which only handles
   builtin services. Original copyrights read... */

/*	$OpenBSD: inetd.c,v 1.69 2000/08/22 14:47:54 millert Exp $	*/
/*	$NetBSD: inetd.c,v 1.11 1996/02/22 11:14:41 mycroft Exp $	*/
/*
 * Copyright (c) 1983,1991 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#if 0
#ifndef lint
char copyright[] =
"@(#) Copyright (c) 1983 Regents of the University of California.\n\
 All rights reserved.\n";
#endif /* not lint */

#ifndef lint
/*static char sccsid[] = "from: @(#)inetd.c	5.30 (Berkeley) 6/3/91";*/
static char rcsid[] = "$OpenBSD: inetd.c,v 1.69 2000/08/22 14:47:54 millert Exp $";
#endif /* not lint */
#endif

/*
 * Inetd - Internet super-server
 *
 * This program invokes all internet services as needed.
 * connection-oriented services are invoked each time a
 * connection is made, by creating a process.  This process
 * is passed the connection as file descriptor 0 and is
 * expected to do a getpeername to find out the source host
 * and port.
 *
 * Datagram oriented services are invoked when a datagram
 * arrives; a process is created and passed a pending message
 * on file descriptor 0.  Datagram servers may either connect
 * to their peer, freeing up the original socket for inetd
 * to receive further messages on, or ``take over the socket'',
 * processing all arriving datagrams and, eventually, timing
 * out.	 The first type of server is said to be ``multi-threaded'';
 * the second type of server ``single-threaded''. 
 *
 */

#include <sys/param.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/file.h>
#include <sys/wait.h>
#include <sys/time.h>

#include <netinet/in.h>
#include <arpa/inet.h>

#include <errno.h>
#include <signal.h>
#include <netdb.h>
#include <syslog.h>
#include <pwd.h>
#include <grp.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <getopt.h>
#include <pthread.h>

#define	TOOMANY		256		/* don't start more than TOOMANY */
#define	CNT_INTVL	60		/* servers in CNT_INTVL sec. */
#define	RETRYTIME	(60*10)		/* retry after bind or server fail */

#define	SIGBLOCK	(sigmask(SIGCHLD)|sigmask(SIGHUP)|sigmask(SIGALRM))

static void	reapchild __P((int));
static void	retry __P((int));
static void	goaway __P((int));

static void	config __P((void));

static int	debug = 0;
static int	nsock, maxsock;
static fd_set	allsock;
static int	toomany = TOOMANY;
static int	options;
static int	timingout;
static struct	servent *sp;

#ifndef OPEN_MAX
#define OPEN_MAX	64
#endif

static struct	servtab {
	char	*se_service;		/* name of service */
	int	se_socktype;		/* type of socket to use */
	int	se_family;		/* address family */
	char	*se_proto;		/* protocol used */
	pid_t	se_wait;		/* single threaded server */
	struct	biltin *se_bi;		/* if built-in, description */
	int	se_fd;			/* open descriptor */
	union {
		struct	sockaddr se_un_ctrladdr;
		struct	sockaddr_in se_un_ctrladdr_in;
		struct	sockaddr_in6 se_un_ctrladdr_in6;
		struct	sockaddr_un se_un_ctrladdr_un;
		struct	sockaddr_storage se_un_ctrladdr_storage;
	} se_un;			/* bound address */
#define se_ctrladdr	se_un.se_un_ctrladdr
#define se_ctrladdr_in	se_un.se_un_ctrladdr_in
#define se_ctrladdr_in6	se_un.se_un_ctrladdr_in6
#define se_ctrladdr_un	se_un.se_un_ctrladdr_un
#define se_ctrladdr_storage	se_un.se_un_ctrladdr_storage
	int	se_ctrladdr_size;
	int	se_max;			/* max # of instances of this service */
	int	se_count;		/* number started since se_time */
	struct	timeval se_time;	/* start of se_count */
    	char	se_title[32];		/* process title */
	struct	servtab *se_next;
} *servtab;

static struct servtab * dupconfig __P((struct servtab *));
static void freeconfig __P((struct servtab *));
static void print_service __P((char *, struct servtab *));

static int echo_stream __P((void *));
static int discard_stream __P((void *));
static int machtime_stream __P((void *));
static int daytime_stream __P((void *));
static int chargen_stream __P((void *));
static int echo_dg __P((void *));
static int discard_dg __P((void *));
static int machtime_dg __P((void *));
static int daytime_dg __P((void *));
static int chargen_dg __P((void *));

static struct biltin {
	char	*bi_service;		/* internally provided service name */
	int	bi_socktype;		/* type of socket supported */
	short	bi_fork;		/* 1 if should fork before call */
	short	bi_wait;		/* 1 if should wait for child */
    	int	(*bi_fn) __P((void *));
} biltins[] = {
	/* Echo received data */
	{ "echo",	SOCK_STREAM,	1, 0,	echo_stream },
	{ "echo",	SOCK_DGRAM,	0, 0,	echo_dg },

	/* Internet /dev/null */
	{ "discard",	SOCK_STREAM,	1, 0,	discard_stream },
	{ "discard",	SOCK_DGRAM,	0, 0,	discard_dg },

	/* Return 32 bit time since 1900 */
	{ "time",	SOCK_STREAM,	0, 0,	machtime_stream },
	{ "time",	SOCK_DGRAM,	0, 0,	machtime_dg },

	/* Return human-readable time */
	{ "daytime",	SOCK_STREAM,	0, 0,	daytime_stream },
	{ "daytime",	SOCK_DGRAM,	0, 0,	daytime_dg },

	/* Familiar character generator */
	{ "chargen",	SOCK_STREAM,	1, 0,	chargen_stream },
	{ "chargen",	SOCK_DGRAM,	0, 0,	chargen_dg },

	{ 0 }
};

static char	*progname = "inetd";

static int 	inetd (void *arg);

void
usg_inetd (FILE *fp)
{
    fprintf(fp, "usage: %s [-R rate] [-d]\n", progname);
}


int
com_inetd (argc, argv)
	int argc;
	char *argv[];
{
	int ch;

#ifndef unix
	/* reset variables */
	debug = 0;
	nsock = maxsock = 0;
	FD_ZERO(&allsock);
	toomany = TOOMANY;
	options = 0;
	timingout = 0;
	sp = NULL;
#endif

	while ((ch = getopt(argc, argv, "dR:")) != -1)
		switch(ch) {
		case 'd':
			debug = 1;
			options |= SO_DEBUG;
			break;
		case 'R': {	/* invocation rate */
			char *p;
			int val;

			val = strtoul(optarg, &p, 0);
			if (val >= 1 && *p == NULL) {
				toomany = val;
				break;
			}
			syslog(LOG_ERR,
			    "-R %s: bad value for service invocation rate",
			    optarg);
			break;
		}
		case '?':
		default:
			usg_inetd (stderr);
			exit(1);
		}
	argc -= optind;
	argv += optind;

	if (debug == 0) {
	        /* spawn a new process in the background */
		int err;
		if (err = pthread_proc_create_np (NULL, NULL, inetd, NULL))
		    perror ("fork");
	}
	else {
	    inetd (NULL);
	}

	exit (0);
}


static int
inetd (void *arg)
{
	register struct servtab *sep, *nsep;
	struct sigaction sa, sapipe;
	int dofork;
	int pid;
	char buf[50];
    
	if (debug == 0) {
#ifndef unix
	    /* set own process/thread name */
	    pthread_setname_np (pthread_self(), "inetd");
	    /* move to save directory */
	    chdir ("/");
	    /* detach from tty */
	    setpgrp ();
	    ioctl (0, TIOCNOTTY, 0);
#else
	    daemon (1, 1);
#endif
	}

	openlog(progname, LOG_PID | LOG_NOWAIT | LOG_CONS, LOG_DAEMON);

	memset((char *)&sa, 0, sizeof(sa));
	sigemptyset(&sa.sa_mask);	
	sigaddset(&sa.sa_mask, SIGALRM);
	sigaddset(&sa.sa_mask, SIGCHLD);
	sigaddset(&sa.sa_mask, SIGHUP);
	sa.sa_handler = retry;
	sigaction(SIGALRM, &sa, NULL);
	sa.sa_handler = reapchild;
	sigaction(SIGCHLD, &sa, NULL);
	sa.sa_handler = goaway;
	sigaction(SIGTERM, &sa, NULL);
	sa.sa_handler = goaway;
	sigaction(SIGINT, &sa, NULL);
	sa.sa_handler = goaway;
	sigaction(SIGHUP, &sa, NULL);
	sa.sa_handler = SIG_IGN;
	sigaction(SIGPIPE, &sa, &sapipe);

	config();

	for (;;) {
	    int n, ctrl = -1;
	    fd_set readable;

	    if (nsock == 0) {
		(void) sigblock(SIGBLOCK);
		while (nsock == 0)
		    sigpause(0L);
		(void) sigsetmask(0L);
	    }
	    readable = allsock;
	    if ((n = select(maxsock + 1, &readable, NULL, NULL, NULL)) <= 0) {
		    if (n < 0 && errno != EINTR) {
			syslog(LOG_WARNING, "select: %m");
			sleep(1);
		    }
		    continue;
	    }
	    for (sep = servtab; n && sep; sep = sep->se_next)
	    if (sep->se_fd != -1 && FD_ISSET(sep->se_fd, &readable)) {
		n--;
		if (debug)
			fprintf(stderr, "someone wants %s/%s\n", 
				sep->se_service, sep->se_proto);
		if (!sep->se_wait && sep->se_socktype == SOCK_STREAM) {
			ctrl = accept(sep->se_fd, NULL, NULL);
			if (debug)
				fprintf(stderr, "accept, ctrl %d\n", ctrl);
			if (ctrl < 0) {
				if (errno == EINTR)
					continue;
				syslog(LOG_WARNING, "accept (for %s): %m",
				    sep->se_service);
				continue;
			}
			if (sep->se_family == AF_INET &&
			    sep->se_socktype == SOCK_STREAM) {
				struct sockaddr_in peer;
				int plen = sizeof(peer);

				if (getpeername(ctrl, (struct sockaddr *)&peer,
				    &plen) < 0) {
					syslog(LOG_WARNING, "could not getpeername");
					close(ctrl);
					continue;
				}
				if (ntohs(peer.sin_port) == 20) {
					/* XXX ftp bounce */
					close(ctrl);
					continue;
				}
			}
		} else
			ctrl = sep->se_fd;
		(void) sigblock(SIGBLOCK);
		pid = 0;
		dofork = (sep->se_bi == 0 || sep->se_bi->bi_fork);
		if (dofork) {
			if (sep->se_count++ == 0)
			    (void)gettimeofday(&sep->se_time, NULL);
			else if (sep->se_count >= sep->se_max) {
				struct timeval now;

				(void)gettimeofday(&now, NULL);
				if (now.tv_sec - sep->se_time.tv_sec >
				    CNT_INTVL) {
					sep->se_time = now;
					sep->se_count = 1;
				} else {
					if (!sep->se_wait &&
					    sep->se_socktype == SOCK_STREAM)
						close(ctrl);
					if (sep->se_family == AF_INET &&
					    ntohs(sep->se_ctrladdr_in.sin_port) >=
					    IPPORT_RESERVED) {
						/*
						 * Cannot close it -- there are
						 * thieves on the system.
						 * Simply ignore the connection.
						 */
						--sep->se_count;
						continue;
					}
					syslog(LOG_ERR,
			"%s/%s server failing (looping), service terminated",
					    sep->se_service, sep->se_proto);
					if (!sep->se_wait &&
					    sep->se_socktype == SOCK_STREAM)
						close(ctrl);
					FD_CLR(sep->se_fd, &allsock);
					(void) close(sep->se_fd);
					sep->se_fd = -1;
					sep->se_count = 0;
					nsock--;
					sigsetmask(0L);
					if (!timingout) {
						timingout = 1;
						alarm(RETRYTIME);
					}
					continue;
				}
			}
			if (debug)
			    fprintf (stderr, "forking process\n");
			nsep = dupconfig (sep);
			nsep->se_fd = ctrl;
			pthread_proc_create_np (&pid, NULL, 
						nsep->se_bi->bi_fn, nsep);
		}
		if (pid < 0) {
			syslog(LOG_ERR, "fork: %m");
			if (!sep->se_wait && sep->se_socktype == SOCK_STREAM)
				close(ctrl);
			sigsetmask(0L);
			sleep(1);
			continue;
		}
		if (pid && sep->se_wait) {
			sep->se_wait = pid;
			FD_CLR(sep->se_fd, &allsock);
			nsock--;
		}
		sigsetmask(0L);
		if (pid == 0) {
			if (debug)
			    fprintf (stderr, "calling inline\n");
			nsep = dupconfig (sep);
			nsep->se_fd = ctrl;
		        (*sep->se_bi->bi_fn)(nsep);
		}

		if (!sep->se_wait && sep->se_socktype == SOCK_STREAM)
			close(ctrl);
	    }
	}
}

static int
dg_badinput(sa)
	struct sockaddr *sa;
{
	struct in_addr in;
#ifdef INET6
	struct in6_addr *in6;
#endif
	u_int16_t port;
	int i;

	switch (sa->sa_family) {
	case AF_INET:
		in.s_addr = ntohl(((struct sockaddr_in *)sa)->sin_addr.s_addr);
		port = ntohs(((struct sockaddr_in *)sa)->sin_port);
	v4chk:
		if (IN_MULTICAST(in.s_addr))
			goto bad;
		switch ((in.s_addr & 0xff000000) >> 24) {
		case 0: case 127: case 255:
			goto bad;
		}
		/* XXX check for subnet broadcast using getifaddrs(3) */
		break;
#ifdef INET6
	case AF_INET6:
		in6 = &((struct sockaddr_in6 *)sa)->sin6_addr;
		port = ntohs(((struct sockaddr_in6 *)sa)->sin6_port);
		if (IN6_IS_ADDR_MULTICAST(in6) || IN6_IS_ADDR_UNSPECIFIED(in6))
			goto bad;
		/*
		 * OpenBSD does not support IPv4 mapped adderss (RFC2553
		 * inbound behavior) at all.  We should drop it.
		 */
		if (IN6_IS_ADDR_V4MAPPED(in6))
			goto bad;
		if (IN6_IS_ADDR_V4COMPAT(in6)) {
			memcpy(&in, &in6->s6_addr[12], sizeof(in));
			in.s_addr = ntohl(in.s_addr);
			goto v4chk;
		}
		break;
#endif
	default:
		/* XXX unsupported af, is it safe to assume it to be safe? */
		return 0;
	}

	if (port < IPPORT_RESERVED)
		goto bad;

	return (0);

bad:
	return (1);
}

static void
reapchild(sig)
	int sig;
{
	pid_t pid;
	int save_errno = errno, status;
	register struct servtab *sep;

	if (debug)
	    fprintf (stderr, "reapchild entered\n");
	for (;;) {
		pid = wait3(&status, WNOHANG, NULL);
		if (pid <= 0)
			break;
		if (debug)
			fprintf(stderr, "%d reaped, status %x\n", pid, status);
		for (sep = servtab; sep; sep = sep->se_next)
			if (sep->se_wait == pid) {
				if (WIFEXITED(status) && WEXITSTATUS(status))
					syslog(LOG_WARNING,
					    "%s: exit status 0x%x",
					    sep->se_service, WEXITSTATUS(status));
				else if (WIFSIGNALED(status))
					syslog(LOG_WARNING,
					    "%s: exit signal 0x%x",
					    sep->se_service, WTERMSIG(status));
				sep->se_wait = 1;
				FD_SET(sep->se_fd, &allsock);
				nsock++;
				if (debug)
					fprintf(stderr, "restored %s, fd %d\n",
					    sep->se_service, sep->se_fd);
			}
	}
	if (debug)
	    fprintf (stderr, "reapchild done\n");
	errno = save_errno;
}

static void setup __P((struct servtab *));
static struct servtab *enter __P((struct servtab *));

static void
config()
{
	struct biltin *bi;
	struct servtab cp;
	register struct servtab *sep;
	int omask;
	int n;

	for (bi = biltins; bi->bi_service; bi++) {
	    memset(&cp, 0, sizeof cp);
	    
	    cp.se_bi = bi;
	    cp.se_service = bi->bi_service;
	    cp.se_socktype = bi->bi_socktype;
	    cp.se_wait = bi->bi_wait;
	    cp.se_family = AF_INET;
	    cp.se_max = toomany;
	    cp.se_ctrladdr_in.sin_family = AF_INET;
	    cp.se_ctrladdr_in.sin_addr.s_addr = INADDR_ANY;
	    cp.se_ctrladdr_in.sin_len = 
	      cp.se_ctrladdr_size = sizeof cp.se_ctrladdr_in;
	    if (cp.se_socktype == SOCK_STREAM)
	    	cp.se_proto = "tcp";
	    else
	    	cp.se_proto = "udp";
	    sp = getservbyname(cp.se_service, cp.se_proto);
	    if (sp == 0) {
		syslog(LOG_ERR,
		       "%s/%s: unknown service",
		       cp.se_service, cp.se_proto);
		continue;
	    }
	    cp.se_ctrladdr_in.sin_port = sp->s_port;
	    cp.se_fd = -1;
	    sep = enter(&cp);
	    setup(sep);
	    if (debug)
		print_service("ADD", sep);
	}
}

static void
retry(sig)
	int sig;
{
	register struct servtab *sep;

	timingout = 0;
	for (sep = servtab; sep; sep = sep->se_next) {
		if (sep->se_fd == -1) {
			switch (sep->se_family) {
			case AF_UNIX:
			case AF_INET:
			case AF_INET6:
				setup(sep);
				break;
			}
		}
	}
}

static void
goaway(sig)
	int sig;
{
	register struct servtab *sep;

	for (sep = servtab; sep; sep = sep->se_next) {
		if (sep->se_fd == -1)
			continue;
		(void)close(sep->se_fd);
	}
	exit(0);
}

static void
setup(sep)
	register struct servtab *sep;
{
	int on = 1;
	int r;

	if ((sep->se_fd = socket(sep->se_family, sep->se_socktype, 0)) < 0) {
		syslog(LOG_ERR, "%s/%s: socket: %m",
		    sep->se_service, sep->se_proto);
		return;
	}
#define	turnon(fd, opt) \
setsockopt(fd, SOL_SOCKET, opt, (char *)&on, sizeof (on))
	if (strncmp(sep->se_proto, "tcp", 3) == 0 && (options & SO_DEBUG) &&
	    turnon(sep->se_fd, SO_DEBUG) < 0)
		syslog(LOG_ERR, "setsockopt (SO_DEBUG): %m");
	if (turnon(sep->se_fd, SO_REUSEADDR) < 0)
		syslog(LOG_ERR, "setsockopt (SO_REUSEADDR): %m");
#undef turnon
	r = bind(sep->se_fd, &sep->se_ctrladdr, sep->se_ctrladdr_size);
	if (r < 0) {
		syslog(LOG_ERR, "%s/%s: bind: %m",
		    sep->se_service, sep->se_proto);
		(void) close(sep->se_fd);
		sep->se_fd = -1;
		if (!timingout) {
			timingout = 1;
			alarm(RETRYTIME);
		}
		return;
	}
	if (sep->se_socktype == SOCK_STREAM)
		listen(sep->se_fd, 10);

	FD_SET(sep->se_fd, &allsock);
	nsock++;
	if (sep->se_fd > maxsock) {
		maxsock = sep->se_fd;
	}
}



static struct servtab *
enter(cp)
	struct servtab *cp;
{
	register struct servtab *sep;
	int omask;

	sep = (struct servtab *)malloc(sizeof (*sep));
	if (sep == NULL) {
		syslog(LOG_ERR, "Out of memory.");
		exit(-1);
	}
	*sep = *cp;
	sep->se_fd = -1;
	omask = sigblock(SIGBLOCK);
	sep->se_next = servtab;
	servtab = sep;
	sigsetmask(omask);
	return (sep);
}


static void
freeconfig(cp)
	register struct servtab *cp;
{
	int i;

	if (cp->se_service)
		free(cp->se_service);
	if (cp->se_proto)
		free(cp->se_proto);
	free (cp);
}

static char *
newstr(cp)
	char *cp;
{
	if ((cp = strdup(cp ? cp : "")))
		return(cp);
	syslog(LOG_ERR, "strdup: %m");
	exit(-1);
}

static struct servtab *
dupconfig(sep)
	struct servtab *sep;
{
	struct servtab *newtab;
	int argc;

	newtab = (struct servtab *) malloc(sizeof(struct servtab));

	if (newtab == NULL) {
		syslog(LOG_ERR, "malloc: %m");
		exit (-1);
	}

	memset((char *)newtab, 0, sizeof(struct servtab));

	newtab->se_service = sep->se_service ? newstr(sep->se_service) : NULL;
	newtab->se_socktype = sep->se_socktype;
	newtab->se_family = sep->se_family;
	newtab->se_proto = sep->se_proto ? newstr(sep->se_proto) : NULL;
	newtab->se_bi = sep->se_bi;
	newtab->se_max = sep->se_max;

	return (newtab);
}

static void
inetd_setproctitle(sep)
	struct servtab *sep;
{
	int size;
	register char *cp;
	struct sockaddr_in sin;

	cp = sep->se_title;
	(void) snprintf(cp, sizeof sep->se_title, "[%d]%s", 
			getpid(), sep->se_service);
	size = sizeof(sin);
	if (getpeername(sep->se_fd, (struct sockaddr *)&sin, &size) == 0) {
		char *s = inet_ntoa(sin.sin_addr);
		strlcat(cp, " [", sizeof sep->se_title);
		strlcat(cp, s, sizeof sep->se_title);
		strlcat(cp, "]", sizeof sep->se_title);
	}
	pthread_setname_np (pthread_self(), cp);
}


/*
 * Internet services provided internally by inetd:
 */
#define	BUFSIZE	4096

/* ARGSUSED */
static int
echo_stream(arg)		/* Echo service -- echo data back */
	void *arg;
{
	struct servtab *sep = arg;
    	int s = sep->se_fd;
	char buffer[BUFSIZE];
	int i;

	inetd_setproctitle(sep);
	while ((i = read(s, buffer, sizeof(buffer))) > 0 &&
	    write(s, buffer, i) > 0)
		;
	freeconfig (sep);
	exit(0);
}

/* ARGSUSED */
static int
echo_dg(arg)			/* Echo service -- echo data back */
	void *arg;
{
	struct servtab *sep = arg;
    	int s = sep->se_fd;
	char buffer[BUFSIZE];
	int i, size;
	struct sockaddr_storage ss;

	freeconfig (sep);
	size = sizeof(ss);
	if ((i = recvfrom(s, buffer, sizeof(buffer), 0, (struct sockaddr *)&ss,
	    &size)) < 0)
		return 0;
	if (dg_badinput((struct sockaddr *)&ss))
		return 0;
	(void) sendto(s, buffer, i, 0, (struct sockaddr *)&ss, size);
	return 0;
}

/* ARGSUSED */
static int
discard_stream(arg)		/* Discard service -- ignore data */
	void *arg;
{
	struct servtab *sep = arg;
    	int s = sep->se_fd;
	char buffer[BUFSIZE];

	inetd_setproctitle(sep);
	while ((errno = 0, read(s, buffer, sizeof(buffer)) > 0) ||
	    errno == EINTR)
		;
	freeconfig (sep);
	exit(0);
}

/* ARGSUSED */
static int
discard_dg(arg)		/* Discard service -- ignore data */
	void *arg;
{
	struct servtab *sep = arg;
    	int s = sep->se_fd;
	char buffer[BUFSIZE];

	freeconfig (sep);
	(void) read(s, buffer, sizeof(buffer));
	return 0;
}

#include <ctype.h>
#define LINESIZ 72
static pthread_once_t ring_once = PTHREAD_ONCE_INIT;
static char ring[128];
static char *endring;

static void
initring()
{
	register int i;

	endring = ring;

	for (i = 0; i <= 128; ++i)
		if (isprint(i))
			*endring++ = i;
}

/* ARGSUSED */
static int
chargen_stream(arg)		/* Character generator */
	void *arg;
{
	struct servtab *sep = arg;
    	int s = sep->se_fd;
	register char *rs;
	int len;
	char text[LINESIZ+2];

	inetd_setproctitle(sep);

	pthread_once (&ring_once, initring);
	rs = ring;

	text[LINESIZ] = '\r';
	text[LINESIZ + 1] = '\n';
	for (rs = ring;;) {
		if ((len = endring - rs) >= LINESIZ)
			memmove(text, rs, LINESIZ);
		else {
			memmove(text, rs, len);
			memmove(text + len, ring, LINESIZ - len);
		}
		if (++rs == endring)
			rs = ring;
		if (write(s, text, sizeof(text)) != sizeof(text))
			break;
	}
	freeconfig (sep);
	exit(0);
}

/* ARGSUSED */
static int
chargen_dg(arg)		/* Character generator */
	void *arg;
{
	struct servtab *sep = arg;
    	int s = sep->se_fd;
	struct sockaddr_storage ss;
	static char *rs;
	int len, size;
	char text[LINESIZ+2];

	freeconfig (sep);

	pthread_once (&ring_once, initring);
	rs = ring;		/* XXX should be maintained somewhere */

	size = sizeof(ss);
	if (recvfrom(s, text, sizeof(text), 0, (struct sockaddr *)&ss,
	    &size) < 0)
		return;
	if (dg_badinput((struct sockaddr *)&ss))
		return;

	if ((len = endring - rs) >= LINESIZ)
		memmove(text, rs, LINESIZ);
	else {
		memmove(text, rs, len);
		memmove(text + len, ring, LINESIZ - len);
	}
	if (++rs == endring)
		rs = ring;
	text[LINESIZ] = '\r';
	text[LINESIZ + 1] = '\n';
	(void) sendto(s, text, sizeof(text), 0, (struct sockaddr *)&ss, size);
}

/*
 * Return a machine readable date and time, in the form of the
 * number of seconds since midnight, Jan 1, 1900.  Since gettimeofday
 * returns the number of seconds since midnight, Jan 1, 1970,
 * we must add 2208988800 seconds to this figure to make up for
 * some seventy years Bell Labs was asleep.
 */

static u_int
machtime()
{
	struct timeval tv;

	if (gettimeofday(&tv, NULL) < 0) {
		fprintf(stderr, "Unable to get time of day\n");
		return (0L);
	}
	return (htonl((u_int)tv.tv_sec + 2208988800UL));
}

/* ARGSUSED */
static int
machtime_stream(arg)
	void *arg;
{
	struct servtab *sep = arg;
    	int s = sep->se_fd;
	u_int result;

	result = machtime();
	(void) write(s, (char *) &result, sizeof(result));
	freeconfig (sep);
	return (0);
}

/* ARGSUSED */
static int
machtime_dg(arg)
	void *arg;
{
	struct servtab *sep = arg;
    	int s = sep->se_fd;
	u_int result;
	struct sockaddr_storage ss;
	int size;

	freeconfig (sep);
	size = sizeof(ss);
	if (recvfrom(s, (char *)&result, sizeof(result), 0,
	    (struct sockaddr *)&ss, &size) < 0)
		return 0;
	if (dg_badinput((struct sockaddr *)&ss))
		return 0;
	result = machtime();
	(void) sendto(s, (char *) &result, sizeof(result), 0,
	    (struct sockaddr *)&ss, size);
	return 0;
}

/* ARGSUSED */
static int
daytime_stream(arg)		/* Return human-readable time of day */
	void *arg;
{
	struct servtab *sep = arg;
    	int s = sep->se_fd;
	char buffer[32];
	time_t time(), clock;

	clock = time(NULL);

	(void) sprintf(buffer, "%.24s\r\n", ctime(&clock));
	(void) write(s, buffer, strlen(buffer));
	freeconfig (sep);
	return (0);
}

/* ARGSUSED */
static int
daytime_dg(arg)		/* Return human-readable time of day */
	void *arg;
{
	struct servtab *sep = arg;
    	int s = sep->se_fd;
	char buffer[32];
	time_t time(), clock;
	struct sockaddr_storage ss;
	int size;

	freeconfig (sep);

	clock = time((time_t *) 0);

	size = sizeof(ss);
	if (recvfrom(s, buffer, sizeof(buffer), 0, (struct sockaddr *)&ss,
	    &size) < 0)
		return 0;
	if (dg_badinput((struct sockaddr *)&ss))
		return 0;
	(void) sprintf(buffer, "%.24s\r\n", ctime(&clock));
	(void) sendto(s, buffer, strlen(buffer), 0, (struct sockaddr *)&ss,
	    size);
	return 0;
}

/*
 * print_service:
 *	Dump relevant information to stderr
 */
static void
print_service(action, sep)
	char *action;
	struct servtab *sep;
{
    	fprintf(stderr, "%s: %s ", action, sep->se_service);
        fprintf(stderr, "proto=%s,", sep->se_proto);
	if (sep->se_family == AF_INET)
	    fprintf(stderr, "port=%d", ntohs(sep->se_ctrladdr_in.sin_port));
	fprintf(stderr, " wait.max=%hd.%d\n",
		sep->se_wait, sep->se_max);
}
