#ident "sde/rtpx/rtpx_attr.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * rtpx_attr.c: handle Pthreads attributes 
 */


#define _POSIX_C_SOURCE 199506L

/* basic process stuff */
#include <stddef.h>
#include <errno.h>
#include "pthread_impl.h"

int
pthread_attr_init (pthread_attr_t *tha)
{
    tha->tha_detachstate = PTHREAD_CREATE_JOINABLE;
    tha->tha_scope = PTHREAD_SCOPE_PROCESS;
    tha->tha_inheritsched = PTHREAD_INHERIT_SCHED;

    /* XXX it's inherit sched so ... */
    tha->tha_schedpolicy = _RTPX_SCHED_DEFAULT;
    tha->tha_schedparam.sched_priority = _RTPX_PRIO_DEFAULT;

    tha->tha_stackaddr = NULL;
    tha->tha_stacksize = _RTPX_STACKSIZE_DEFAULT;
    tha->tha_guardsize = _RTPX_GUARDSIZE_DEFAULT;

    tha->tha_name = NULL;

    return 0;
}

int
pthread_attr_destroy (pthread_attr_t *tha)
{
    return 0;
}

#define GETSET(name, aname, type,cond) \
int pthread_attr_set##name(pthread_attr_t *tha, type v) \
{ \
    if(cond) { \
       tha->tha_##aname = v; \
       return 0; \
    } else { \
       return EINVAL; \
    } \
} \
\
int pthread_attr_get##name(const pthread_attr_t *tha, type *pv) \
{ \
  *pv = tha->tha_##aname; \
  return 0;\
}

GETSET (scope, scope, int, VALID_SCOPE (v))
GETSET (inheritsched, inheritsched, int, VALID_INHERITSCHED (v))
GETSET (stacksize, stacksize, size_t, VALID_STACKSIZE (v))
GETSET (guardsize_np, guardsize, size_t, 1)
GETSET (stackaddr, stackaddr, void *, 1)
GETSET (schedpolicy, schedpolicy, int, VALID_SCHEDPOLICY (v))
GETSET (detachstate, detachstate, int, VALID_DETACHSTATE (v))
#ifdef _POSIX_THREAD_ATTR_NAME_NP
GETSET (name_np, name, const char *, 1)
#endif

int
pthread_attr_setschedparam (pthread_attr_t *tha,
			    const struct sched_param *sp)
{
    if (!VALID_SCHEDPARAM (sp))
	return EINVAL;
    tha->tha_schedparam = *sp;
    return 0;
}

int
pthread_attr_getschedparam (const pthread_attr_t *tha,
			    struct sched_param *sp)
{
    *sp = tha->tha_schedparam;
    return 0;
}



int
pthread_mutexattr_init (pthread_mutexattr_t *mxa)
{
    mxa->mxa_pshared = PTHREAD_PROCESS_PRIVATE;
    mxa->mxa_protocol = PTHREAD_PRIO_NONE;
    mxa->mxa_prioceiling = _RTPX_PRIO_MIN;
    mxa->mxa_type = PTHREAD_MUTEX_DEFAULT_NP;
    mxa->mxa_name = NULL;
    return 0;
}

int
pthread_mutexattr_destroy (pthread_mutexattr_t *mxa)
{
    return 0;
}

#undef GETSET
#define GETSET(name,aname,type,cond) \
int pthread_mutexattr_set##name(pthread_mutexattr_t *mxa,type v) \
{ \
    if(cond) { \
       mxa->mxa_##aname = v; \
       return 0; \
    } else { \
       return EINVAL; \
    } \
} \
\
int pthread_mutexattr_get##name(const pthread_mutexattr_t *mxa,type *pv) \
{ \
  *pv = mxa->mxa_##aname; \
  return 0;\
}

GETSET (pshared, pshared, int, VALID_PSHARED (v))
#if defined(_POSIX_THREAD_PRIO_PROTECT) \
   || defined(_POSIX_THREAD_PRIO_PROTECTINHERIT)
GETSET (protocol, protocol, int, VALID_PROTOCOL (v))
#endif
#ifdef _POSIX_THREAD_PRIO_PROTECT
GETSET (prioceiling, prioceiling, int, VALID_PRIORITY (v))
#endif
#ifdef _POSIX_THREAD_MUTEX_TYPE_NP
GETSET (type_np, type, int, VALID_TYPE (v))
#endif
#ifdef _POSIX_THREAD_ATTR_NAME_NP
GETSET (name_np, name, const char *, 1)
#endif



int
pthread_condattr_init (pthread_condattr_t *cda)
{
    cda->cda_pshared = PTHREAD_PROCESS_PRIVATE;
    cda->cda_interruptible = 0;
    cda->cda_name = NULL;
    return 0;
}

int
pthread_condattr_destroy (pthread_condattr_t *cda)
{
    return 0;
}

int
pthread_condattr_setpshared (pthread_condattr_t *cda, int shared)
{
    if (! VALID_PSHARED (shared))
	return EINVAL;
    cda->cda_pshared = shared;
    return 0;
}

int
pthread_condattr_getpshared (const pthread_condattr_t *cda, int *pshared)
{
    *pshared = cda->cda_pshared;
    return 0;
}

int
pthread_condattr_setinterruptible_np (pthread_condattr_t *cda, int iu)
{
    cda->cda_interruptible = iu;
    return 0;
}

int
pthread_condattr_getinterruptible_np (const pthread_condattr_t *cda,
				      int *piu)
{
    *piu = cda->cda_interruptible;
    return 0;
}

#ifdef _POSIX_THREAD_ATTR_NAME_NP
int
pthread_condattr_setname_np (pthread_condattr_t *cda, const char *v)
{
    cda->cda_name = v;
    return 0;
}

int
pthread_condattr_getname_np (const pthread_condattr_t *cda,
			     const char **pv)
{
    *pv = cda->cda_name;
    return 0;
}
#endif



/* Doorbells */
int
pthread_doorbellattr_init (pthread_doorbellattr_t *dba)
{
    dba->dba_evn = 0;
    dba->dba_name = NULL;
    return 0;
}

int
pthread_doorbellattr_destroy (pthread_doorbellattr_t *dba)
{
    return 0;
}

int
pthread_doorbellattr_setevn (pthread_doorbellattr_t *dba, int evn)
{
    dba->dba_evn = evn;
    return 0;
}

int
pthread_doorbellattr_getevn (const pthread_doorbellattr_t *dba, int *pevn)
{
    *pevn = dba->dba_evn;
    return 0;
}

#ifdef _POSIX_THREAD_ATTR_NAME_NP
int
pthread_doorbellattr_setname (pthread_doorbellattr_t *dba, const char *v)
{
    dba->dba_name = v;
    return 0;
}

int
pthread_doorbellattr_getname (const pthread_doorbellattr_t *dba,
			      const char **pv)
{
    *pv = dba->dba_name;
    return 0;
}
#endif
