/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * rtpx_impl.h: RTPX private header file
 */


/* disable thread specific data support */
#define  _RTPX_TSD

/* enable flockfile support */
#define _RTPX_FLOCKFILE

/* enable support for fpa */
#define _RTPX_FLOATING_POINT

#define _RTPX_THREAD_MAX	128	/* maximum number of threads */
#define _RTPX_PRIO_MIN		0
#define _RTPX_PRIO_MAX		127
#define _RTPX_RR_PRIO_MAX	63
#define _RTPX_FIFO_PRIO_MAX	127

#define _RTPX_STACKSIZE_DEFAULT	(32*1024)
#define _RTPX_GUARDSIZE_DEFAULT	(16)

/* default attributes defined for pthread_attr_init */
#define _RTPX_SCHED_DEFAULT	SCHED_RR	/* was FIFO */
#define _RTPX_PRIO_DEFAULT	_RTPX_PRIO_MIN

/* mutex flags for fast path */
#define _RTPX_MXF_COMPLEX	0x0001	/* always lock slowly */
#define _RTPX_MXF_FASTLOCK	0x0002	/* mutex is fast-locked */
#define _RTPX_MXF_SLOWLOCK	0x0004	/* mutex is slow-locked */
#define _RTPX_MXF_LOCKED 	(_RTPX_MXF_FASTLOCK | _RTPX_MXF_SLOWLOCK)
#define _RTPX_MXF_WAITERS	0x0008	/* being waited for */
#define _RTPX_MXF_FIRSTTIME	0x0010	/* uninitialised */

/* for backwards compatibility, and namespace cleanliness */
#define rtpx_thread	_rtpx_thread
#define rtpx_thread_t	_rtpx_thread_t
#define rtpx_waitq	_rtpx_waitq
#define rtpx_waitq_t	_rtpx_waitq_t

/* forward declarations */
struct rtpx_proc;
struct rtpx_thread;
struct rtpx_waitq;
typedef struct rtpx_proc 	rtpx_proc_t;
typedef struct rtpx_thread 	rtpx_thread_t;
typedef struct rtpx_waitq 	rtpx_waitq_t;

#if defined(__mips)
#include "rtpx_mips.h"
#elif defined(__i386)
#include "rtpx_i386.h"
#else
#error unknown cpu
#endif

/* timer parameters */
extern long	_rtpx_tickspersec;	/* timer resolution */
extern int      _rtpx_rrclock;		/* round-robin scheduling in ticks */
extern int      _rtpx_rrfreq;		/* round-robin scheduling in Hz */

#ifndef _RTPX_RRFREQ
/* default round robin at 100 Hz (10ms) */
#define _RTPX_RRFREQ	100
#endif

#ifndef _RTPX_SIGNO_MAX
/* NSIG but a bit more specific */
#define _RTPX_SIGNO_MAX		65
#endif

/* Timer checking */
#define NSPERSEC	(1000 * 1000 * 1000)
#define timespecok(ts) \
((ts)->tv_sec >= 0 &&  (ts)->tv_nsec >= 0 && (ts)->tv_nsec < NSPERSEC)

/* Check for valid attributes */
#define VALID_INHERITSCHED(v) \
((v) == PTHREAD_INHERIT_SCHED \
 || (v) == PTHREAD_EXPLICIT_SCHED)

#define VALID_DETACHSTATE(v) \
((v) == PTHREAD_CREATE_DETACHED \
 || (v) == PTHREAD_CREATE_JOINABLE \
 || (v) == PTHREAD_CREATE_DAEMON_NP)

#define VALID_SCOPE(v) \
((v) == PTHREAD_SCOPE_PROCESS \
 || (v) == PTHREAD_SCOPE_SYSTEM)

#define VALID_STACKSIZE(v) \
((v) >= PTHREAD_STACK_MIN)

#define VALID_PRIORITY(v) \
((v) >= _RTPX_PRIO_MIN && (v) <= _RTPX_PRIO_MAX)

#define VALID_SCHEDPARAM(v) \
VALID_PRIORITY((v)->sched_priority)

#define VALID_SCHEDPOLICY(v) \
((v) == SCHED_FIFO \
 || (v) == SCHED_RR)

#define VALID_PROTOCOL(v) \
((v) == PTHREAD_PRIO_NONE \
 || (v) == PTHREAD_PRIO_INHERIT \
 || (v) == PTHREAD_PRIO_PROTECT)

#define VALID_PSHARED(v) \
((v) == PTHREAD_PROCESS_PRIVATE \
 || (v) == PTHREAD_PROCESS_SHARED)

#define VALID_TYPE(v) \
((v) == PTHREAD_MUTEX_FAST_NP \
 || (v) == PTHREAD_MUTEX_RECURSIVE_NP \
 || (v) == PTHREAD_MUTEX_ERRORCHECK_NP \
 || (v) == PTHREAD_MUTEX_DEFAULT_NP)

/* 
 * log control
 */
#define RTPXLOG_PRINTF	-1	/* explicit printf */
#define RTPXLOG_PANIC	0	/* fatal error */
#define RTPXLOG_EMERG	RTPXLOG_PANIC
#define	RTPXLOG_ALERT	1	/* action must be taken immediately */
#define	RTPXLOG_CRIT	2	/* critical conditions */
#define	RTPXLOG_ERR	3	/* error conditions */
#define RTPXLOG_ERROR	RTPXLOG_ERR
#define RTPXLOG_WARNING	4
#define RTPXLOG_NOTICE	5
#define RTPXLOG_INFO	6
#define RTPXLOG_DEBUG	7


/* Per-process data (a process is a thread with more state) */
struct rtpx_proc {
    pthread_mutex_t pr_mx;	/* protecting mutex */
    struct cpu_proc pr_cpu;	/* cpu specific process data */
    int             pr_state;	/* our state */
#define _RTPX_PROC_FREE		0
#define _RTPX_PROC_RUN		1
#define _RTPX_PROC_ZOMBIE	2
    int             pr_pid;	/* our process id */
    int             pr_ppid;	/* parent process id */
    int             pr_pgrp;	/* our process group */
    int             pr_nthread;	/* how many threads */
    int             pr_exitval;	/* exit argument */
    int		    pr_waiting;	/* waiting for child */
    pthread_cond_t  pr_waitcd;	/* wait(3) condition variable */
    sigset_t        pr_sigignore;
    sigset_t        pr_sigcatch;
    struct sigaction pr_sigaction[_RTPX_SIGNO_MAX];
    sigset_t        pr_sigpend;	/* pending simple signals */
    siginfo_t       pr_siginfo[_RTPX_SIGNO_MAX];
#ifdef _POSIX_REALTIME_SIGNALS
    int             pr_nsigqueue;	/* pending queued signals */
    siginfo_t       pr_sigqueue[SIGQUEUE_MAX];
#endif
    timer_t         pr_timer[NITIMER];
};

#ifndef _RTPX_PROC_MAX
/* maximum number of processes (not threads) */
#define _RTPX_PROC_MAX	20
#endif

/* The fixed size process table */
extern rtpx_proc_t _rtpx_proctab[_RTPX_PROC_MAX];

/* currently the timer resolution always fits 32 bits */
typedef unsigned long 		tick_t;

#ifndef _RTPX_CLOCK64_64BIT
/* default to 32-bit structure for clock64 type */
#define _RTPX_CLOCK64_64BIT	0
#endif

#if _RTPX_CLOCK64_64BIT

/* internal time is stored as a 64-bit count of ticks */
typedef unsigned long long	clock64_t;

#define	clock64clear(cp)		*(cp) = 0
#define	clock64isset(cp)		(*(cp) != 0)
#define	clock64subsec(cp)		(*(cp) < _rtpx_tickspersec)
#define	clock64ticks(cp)		((tick_t)*(cp))
#define	clock64cmp(ap, bp, cmp)					\
	(*(ap) cmp *(bp))
#define	clock64add(tp, up, vp)					\
	*(vp) = *(tp) + *(up)
#define	clock64sub(tp, up, vp)					\
	*(vp) = *(tp) - *(up)
#define	clock64inc(vp, inc)					\
	*(vp) += (inc)

#else

/* internal time stored in 32-bit chunks */
typedef struct {
    unsigned long   c_sec;
    long	    c_tick;
} clock64_t;

#define	clock64clear(cp)		(cp)->c_sec = (cp)->c_tick = 0
#define	clock64isset(cp)		((cp)->c_sec || (cp)->c_tick)
#define	clock64subsec(cp)		((cp)->c_sec == 0)
#define	clock64ticks(cp)		((tick_t)(cp)->c_tick)
#define	clock64cmp(ap, bp, cmp)					\
	(((ap)->c_sec == (bp)->c_sec) ?				\
	    ((ap)->c_tick cmp (bp)->c_tick) :			\
	    ((ap)->c_sec cmp (bp)->c_sec))
#define	clock64add(tp, up, vp)					\
	do {							\
		(vp)->c_sec = (tp)->c_sec + (up)->c_sec;	\
		(vp)->c_tick = (tp)->c_tick + (up)->c_tick;	\
		if ((vp)->c_tick >= _rtpx_tickspersec) {	\
			(vp)->c_sec++;				\
			(vp)->c_tick -= _rtpx_tickspersec;	\
		}						\
	} while (0)
#define	clock64sub(tp, up, vp)					\
	do {							\
		(vp)->c_sec = (tp)->c_sec - (up)->c_sec;	\
		(vp)->c_tick = (tp)->c_tick - (up)->c_tick;	\
		if ((vp)->c_tick < 0) {				\
			(vp)->c_sec--;				\
			(vp)->c_tick += _rtpx_tickspersec;	\
		}						\
	} while (0)
#define	clock64inc(vp, inc)					\
	do {							\
		(vp)->c_tick += (inc);				\
		if ((vp)->c_tick >= _rtpx_tickspersec) {	\
			(vp)->c_sec++;				\
			(vp)->c_tick -= _rtpx_tickspersec;	\
		}						\
	} while (0)
#endif


/* a clock accumulates absolute time and stores the
   number of ticks until the next timer event */
struct rtpx_clock {
    int             clk_status;
#define CLK_STOPPED	0
#define CLK_RUNNING	1
    clock64_t       clk_t64;
    tick_t	    clk_when;
};


/* interval timer */
struct rtpx_itimer {
    struct rtpx_itimer *it_next;
    struct rtpx_itimer *it_prev;

    int             it_type;		/* timer state */
#define IT_TYPE_DEAD		0	  /* expired */
#define IT_TYPE_BASE		1	  /* internal */
#define IT_TYPE_SLEEP		2	  /* sleep timer */
#define IT_TYPE_TIMEOUT		3         /* wait timeout */
#define IT_TYPE_TIMER		4	  /* interval timer (signal) */

    rtpx_thread_t  *it_thread;		/* owning thread */

    clock64_t       it_value;		/* next alarm time */
    clock64_t       it_interval;	/* repeat time */
};
typedef struct rtpx_itimer rtpx_itimer_t;

/* POSIX.4 process-level timer emulation */
struct rtpx_timer {
    rtpx_itimer_t   ti_it;		/* internal timer */
    rtpx_proc_t    *ti_pr;		/* owning process */
    struct sigevent ti_sigev;		/* signal to generate */
    int             ti_overrun;		/* should be set when sig delivered */
};
typedef struct rtpx_timer rtpx_timer_t;


#if defined(_RTPX_TSD)
/* thread-specific data aka pthread_key */
struct rtpx_tsd {
    pthread_mutex_t tsd_mx;		/* protecting mutex */	
    long            tsd_count;		/* reference count */
    void            (*tsd_destructor) (void *);	/* destructor */
};
typedef struct rtpx_tsd rtpx_tsd_t;
#endif


/* this is it, the per-thread data structure */
struct rtpx_thread {
    struct cpu_th   th_cpu;			/* cpu-specific state */

    struct rtpx_thread *th_next;
    struct rtpx_thread *th_prev;

    unsigned char   th_prio;			/* effective prio */

    unsigned short  th_id;			/* thread identifier */

#define RTPX_THF_CANCEL_DISABLE		0x001	/* cancellation disabled */
#define RTPX_THF_CANCEL_DEFERRED	0x002	/* cancellation deferred */
#define RTPX_THF_CANCEL_PENDING		0x004	/* cancellation pending */
#define RTPX_THF_CANCELLED		0x008	/* cancellation posted */
#define RTPX_THF_CANCELLING		0x010	/* cancellation in progress */
#define RTPX_THF_CONDINTRUPT		0x020	/* in interruptible condwait */
#define RTPX_THF_MAIN			0x040	/* main() thread */
#define RTPX_THF_EXITING		0x080	/* main() thread */
    unsigned int    th_flags;

    char            th_name[16];		/* default name */

    enum {					/* thread state */
	THSTATE_CPUWAIT,
	THSTATE_TIMEWAIT,
	THSTATE_SEMWAIT,
	THSTATE_MUTEXWAIT,
	THSTATE_CONDWAIT,
	THSTATE_SIGWAIT,
	THSTATE_DOORBELLWAIT,
	THSTATE_ZOMBIE,
	THSTATE_DEAD,
    } th_state;

    pthread_mutex_t *th_mxlist;			/* owned priority muticies */

    int             th_wval;			/* wait return value */
    int             th_errno;			/* per-thread errno */

    unsigned int    th_ticksToRr;		/* ticks until resched */

    struct rtpx_waitq *th_waitq;		/* object waited for */

    pthread_mutex_t *th_waitMx;			/* condwait mutex */

    /* thread signal state */
    sigset_t        th_sigmask;
    sigset_t        th_sigpend;
    sigset_t        th_sigwait;
    sigset_t        th_sigdefermask;
    siginfo_t       th_siginfo[_RTPX_SIGNO_MAX];

    pthread_attr_t  th_tha;			/* thread attributes */

    rtpx_proc_t    *th_proc;			/* owning process */

    unsigned char  *th_stack;			/* thread stack */

    rtpx_itimer_t   th_itimer;			/* default timer for waits */

    /* for join/exit processing */
    void           *th_zval;			/* when zombie */
    pthread_cond_t  th_joinCd;			/* for join wait */

    struct _pthread_handler_rec *th_cleanups;	/* pthread_cleanup */

#if defined(_RTPX_TSD)
    const void    **th_tsd;			/* thread-specific data */
    unsigned int    th_tsdcount;
#endif
};

/* Initialisation */
void            _pthread_init (void);
void            pthread_init (void);

/* Enter/Leave kernel mode */
rtpx_thread_t * _rtpx_enter (void);
int		_rtpx_leaveWithValue (rtpx_thread_t *, int);
int		_rtpx_leaveWithError (rtpx_thread_t *, int);
int		_rtpx_leaveWithValueLong (rtpx_thread_t *, int);
#define rtpx_enter 		_rtpx_enter
#define rtpx_leaveWithValue	_rtpx_leaveWithValue
#define rtpx_leaveWithError 	_rtpx_leaveWithError
#define rtpx_leaveWithValueLong _rtpx_leaveWithValueLong

/* Thread suspend and resume */
rtpx_thread_t *	_rtpx_popq (rtpx_waitq_t *);
int		_rtpx_leaveWhenResumed (rtpx_thread_t *, int,
					rtpx_waitq_t *, int);
int		_rtpx_leaveWithValueAndDispatch (rtpx_thread_t *, int);
int		_rtpx_resumeWithValue (rtpx_thread_t *, int);
int		_rtpx_resumeWithError (rtpx_thread_t *, int);
#define rtpx_leaveWhenResumed	_rtpx_leaveWhenResumed 
#define rtpx_leaveWithValueAndDispatch _rtpx_leaveWithValueAndDispatch
#define rtpx_resumeWithValue	_rtpx_resumeWithValue
#define rtpx_resumeWithError	_rtpx_resumeWithError

/* End of interrupt processing */
int             _rtpx_eoi (rtpx_thread_t *);

/* Thread table iteration */
rtpx_thread_t  *_rtpx_findthread (int);
rtpx_thread_t  *_rtpx_firstthread (void);
rtpx_thread_t  *_rtpx_nextthread (rtpx_thread_t *);

/* Signal handling */
int             _rtpx_sigraise (pid_t, int, const struct sigevent *);
int             _rtpx_sigqueue (rtpx_proc_t *, int, const struct sigevent *);
int             _rtpx_resumeWithSig (rtpx_thread_t *, int,
				     const struct siginfo *);
int		_rtpx_sigfatal (int);
int		_rtpx_sigabort (int);
extern int      _rtpx_sigmask (rtpx_thread_t *, int,
			       const sigset_t *, sigset_t *);
extern int      _rtpx_kill (pid_t, int, union sigval, int);
extern void     _rtpx_sigcheck (rtpx_thread_t *);
extern int      _rtpx_sigfirst (sigset_t);
extern void     _rtpx_sigcopy (rtpx_proc_t *, const rtpx_proc_t *);
extern void     _rtpx_siginit (rtpx_proc_t *);
extern void     _rtpx_sigexit (rtpx_proc_t *);

/* Cancellation */
int		_rtpx_sendcancel (rtpx_thread_t *, rtpx_thread_t *);

/* Clock handling */
clock64_t       _rtpx_clock (void);	/* wallclock time at last tick */
clock64_t	_rtpx_clockNow (void);	/* wallclock time now */
clock64_t       _rtpx_relclock (void);	/* elapsed time since boot */
void            _rtpx_tsFromClock64 (struct timespec *, clock64_t);
clock64_t       _rtpx_tsToClock64 (const struct timespec *ts);

/* Low-level interval timer */
void		_rtpx_itimerstartabs (rtpx_itimer_t *, int, clock64_t);
void		_rtpx_itimerstart (rtpx_itimer_t *, int, clock64_t);
void		_rtpx_itimerstop (rtpx_itimer_t *);

/* Return stack usage */
int             _rtpx_stackused (rtpx_thread_t *);

/* Set round-robin interval */
int             _rtpx_rr_set_interval (pid_t, const struct timespec *);

/* Logging and messages */
void            _rtpx_vlog (int, const char *, va_list);
void            _rtpx_log (int, const char *, ...)
		/* __attribute__ ((format (printf, 2, 3))) */;

#ifdef _RTPX_TSD
/* Thread-specific data aka pthread_key setup and tear-down */
extern void     _rtpx_tsdinit (void);
extern void     _rtpx_tsdcleanup (void);
#endif

#ifdef _RTPX_FLOCKFILE
/* Per-thread stdio file locking setup and tear-down */
extern void     _rtpx_flockfileinit (void);
extern void     _rtpx_flockfilecleanup (void);
#endif

/* Process control */
#define _rtpx_self_proc() (rtpx_self()->th_proc)
extern rtpx_proc_t *_rtpx_findproc (int);
extern int	_rtpx_newproc (int *);
extern void     _rtpx_joinproc (rtpx_thread_t *);
extern int      _rtpx_leaveproc (void);
