/*
 * Copyright (c) 1997-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * rtpx_mips.h: MIPS SDE machine-specific definitions
 */


#include <mips/cpu.h>
#include <mips/xcpt.h>
#include <mips/fpa.h>

/* include board-specific definitions */
#include "sbdrtpx.h"
#include "kit_impl.h"

/* use 64-bit clock64 type only when compiled with 64-bit registers */
#ifndef _RTPX_CLOCK64_64BIT
#if __mips64
#define _RTPX_CLOCK64_64BIT	1
#else
#define _RTPX_CLOCK64_64BIT	0
#endif
#endif

struct cpu {
    rtpx_thread_t  *runThread;
    unsigned long   startTicks;		/* ticks when thread started */
    unsigned long   timerTicks;		/* ticks when timer should fire */
    unsigned int    timerReload;	/* force reload of timer */
};

extern struct cpu _cpuTab[_SBD_CPU_MAX];

#define rtpx_self()	(_cpuTab[0].runThread)


/* cpu-specific structure added to thread info */
struct cpu_th {
    unsigned        cpu_sr;	/* sr value for this thread */
    struct xcptcontext *cpu_xcp;
    xcptjmp_buf     cpu_gprCtx;	/* "s" reg sp/ra/sr context */
    int             cpu_dbgnotified;
#ifdef _RTPX_FLOATING_POINT
    struct fpactx   cpu_fpaCtx;
#endif
#ifdef _RTPX_COREXTEND
    struct corextend_ctx cpu_udiCtx;
#endif
};


/* cpu-specific structure added to proess info */
struct cpu_proc {
    struct _libc_data *	cpu_libc;	/* libc data */
    struct filedesc *	cpu_fds;	/* file descriptors */
    int			cpu_first;
};


/* cpu-specific initialisation */
void            _cpu_rtpxinit (void);

/* return time in seconds from real-time clock */
time_t          _cpu_time (time_t * tp);

/* called to enter/exit scheduler */
rtpx_thread_t  *_cpu_enter (void);
int             _cpu_leave (rtpx_thread_t *, int);

/* initialise thread */
void            _cpu_thinit (rtpx_thread_t *, void *(*)(void *), void *);

/* destroy thread hook */
void            _cpu_thdestroy (rtpx_thread_t *);

/* initialise first process */
void		_cpu_procfirst (struct rtpx_proc *);

/* initialise subsequent new process */
int		_cpu_procnew (struct rtpx_proc *, struct rtpx_proc *);

/* process destruction */
void		_cpu_procdestroy (struct rtpx_proc *);

/* switch threads */
rtpx_thread_t  *_cpu_swtch (_rtpx_thread_t *, _rtpx_thread_t *);

/* note when a thread is scheduled and about to return to "user" */
#define _cpu_schedhook(th)  (th)->th_cpu.cpu_xcp = NULL

/* install default exception handler for signal */
void            _cpu_sigcatch (int, int);

/* pass signal to debugger and return signal number or 0 */
int             _cpu_sighook (rtpx_thread_t *, int, int);

/* fatal signal */
void            _cpu_sigabort (rtpx_thread_t *, int, struct xcptcontext *)
    				__attribute__ ((noreturn));

/* normal termination */
void		_cpu_exit (int)  __attribute__ ((noreturn));

/* return stack limits of initial main thread */
void            _cpu_stacklimits (void **, unsigned int *);

/* cpu-specific atomic operations */
#define _cpu_atomic_bis		mips_atomic_bis
#define _cpu_atomic_bic		mips_atomic_bic
#define _cpu_atomic_bcs		mips_atomic_bcs
#define _cpu_atomic_set		mips_atomic_swap
#define _cpu_atomic_add		mips_atomic_add

#define _cpu_intdisable		_mips_intdisable
#define _cpu_intrestore		_mips_intrestore

#define _cpu_badaddr(ptr)					\
	(! IS_KVA01(ptr) 					\
	 || ((vaddr_t)(ptr) & (__alignof__(*(ptr)) - 1)) != 0)

/* board-specific interrupt handling */
void            _sbd_intrinit (void);
int             _sbd_intrpoll (int evn);
void            _sbd_intrenable (int evn);
int             _sbd_intrnext (int idle);
void            (_sbd_intrenter) (void);
void            (_sbd_intrleave) (void);
int             _sbd_intrhandler (unsigned int, struct xcptcontext *);
int		_sbd_intrtype (int, const intrtype_t *, intrtype_t *);
int		_sbd_probable_ipl (unsigned int);

/* returns current 32bit ticker value */
unsigned int    _sbd_tickspersec (void);
unsigned int    _sbd_gettickval (void);
unsigned int    _sbd_gettickcmp (void);
void            _sbd_settickcmp (unsigned int);

#ifdef _RTPX_COREXTEND
/* The RTPX porter must supply versions of these functions
   specific to their core & UDI block. */
int		_corextend_enable (void);
void		_corextend_save (struct corextend_ctx *);
void		_corextend_restore (const struct corextend_ctx *);
#endif
