#ident "sde/rtpx/rtpx_ptimer.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2000-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * rtpx_ptimer.c: old POSIX process-level interval timer emulation
 */


#include "pthread_impl.h"

int
setitimer (unsigned which, struct itimerval *value,
	   struct itimerval *ovalue)
{
    struct rtpx_proc *pr = _rtpx_self_proc ();
    struct itimerspec ts,
                    ots;
    timer_t        *tp;
    int             rv;

    if (which >= NITIMER) {
	errno = EINVAL;
	return -1;
    }

    pthread_mutex_lock (&pr->pr_mx);

    tp = &pr->pr_timer[which];
    if (!*tp) {
	struct sigevent sigev;
	static const int itimersig[NITIMER] = {
	    SIGALRM, SIGVTALRM, SIGPROF, SIGUSR1
	};

	clockid_t       type = CLOCK_REALTIME;	/* XXX */
	sigev.sigev_notify = SIGEV_SIGNAL;
	sigev.sigev_signo = itimersig[which];
	sigev.sigev_value.sival_int = 0;
	if (timer_create (type, &sigev, tp)) {
	    pthread_mutex_unlock (&pr->pr_mx);
	    return -1;
	}
    }

    if (value) {
	ts.it_value.tv_sec = value->it_value.tv_sec;
	ts.it_value.tv_nsec = value->it_value.tv_usec * 1000;
	ts.it_interval.tv_sec = value->it_interval.tv_sec;
	ts.it_interval.tv_nsec = value->it_interval.tv_usec * 1000;
	rv = timer_settime (*tp, 0, &ts, &ots);
    }
    else
	rv = timer_gettime (*tp, &ots);
    pthread_mutex_unlock (&pr->pr_mx);

    if (ovalue && rv == 0) {
	ovalue->it_value.tv_sec = ots.it_value.tv_sec;
	ovalue->it_value.tv_usec = ots.it_value.tv_nsec / 1000;
	ovalue->it_interval.tv_sec = ots.it_interval.tv_sec;
	ovalue->it_interval.tv_usec = ots.it_interval.tv_nsec / 1000;
    }

    return rv ? -1 : 0;
}


int
getitimer (unsigned which, struct itimerval *value)
{
    struct rtpx_proc *pr = _rtpx_self_proc ();
    struct itimerspec ts;
    timer_t        *tp;
    int             rv;

    if (which >= NITIMER) {
	errno = EINVAL;
	return -1;
    }

    pthread_mutex_lock (&pr->pr_mx);
    tp = &pr->pr_timer[which];
    if (!*tp) {
	timerclear (&value->it_value);
	timerclear (&value->it_interval);
	rv = 0;
    }
    else if ((rv = timer_gettime (*tp, &ts)) == 0) {
	value->it_value.tv_sec = ts.it_value.tv_sec;
	value->it_value.tv_usec = ts.it_value.tv_nsec / 1000;
	value->it_interval.tv_sec = ts.it_interval.tv_sec;
	value->it_interval.tv_usec = ts.it_interval.tv_nsec / 1000;
    }
    else {
	rv = -1;
    }
    pthread_mutex_unlock (&pr->pr_mx);
    return rv;
}


unsigned int
alarm (unsigned int secs)
{
    struct itimerval it, oitv;
    register struct itimerval *itp = &it;
    
    timerclear(&itp->it_interval);
    itp->it_value.tv_sec = secs;
    itp->it_value.tv_usec = 0;
    if (setitimer(ITIMER_REAL, itp, &oitv) < 0)
	return (-1);
    if (oitv.it_value.tv_usec)
	oitv.it_value.tv_sec++;
    return (oitv.it_value.tv_sec);
}
