/* 
 * rtpx_sdetty.c: select(2) system call for AlgRTX/MIPS SDE
 *
 * Copyright (c) 2001 Algorithmics Ltd - all rights reserved.
 * 
 * This program is NOT free software, it is supplied under the terms
 * of the AlgRTX License Agreement, a copy of which is available at:
 *
 *  http://www.algor.co.uk/algor/info/algrtx-license.pdf
 *
 * Any company which has obtained and signed a valid AlgRTX license
 * may use and modify this software internally and use (without
 * restrictions) any derived binary.  You may not, however,
 * redistribute this in whole or in part as source code, nor may you
 * modify or remove any part of this copyright message.  
 */

/*
 * Copyright (c) 1996 Theo de Raadt
 * Copyright (c) 1982, 1986, 1989, 1993
 *	The Regents of the University of California.  All rights reserved.
 * (c) UNIX System Laboratories, Inc.
 * All or some portions of this file are derived from material licensed
 * to the University of California by American Telephone and Telegraph
 * Co. or Unix System Laboratories, Inc. and are reproduced herein with
 * the permission of UNIX System Laboratories, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	@(#)sys_generic.c	8.5 (Berkeley) 1/21/94
 */

#define _KERNEL
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/select.h>
#include <pthread.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "file.h"

/*static void pollscan __P((struct pollfd *, int, int *));*/
static int selscan (fd_set *, fd_set *, int, int *);
int _seltrue (struct file *, int);

static pthread_once_t	select_once = PTHREAD_ONCE_INIT;
static pthread_mutex_t	select_mx = PTHREAD_MUTEX_INITIALIZER;
static pthread_cond_t	select_cd;
static int		nselwakeups;

static void
selinit ()
{
    /* make select condition variable interruptible */
    pthread_condattr_t cda;
    pthread_condattr_init (&cda);
    pthread_condattr_setinterruptible_np (&cda, 1);
    pthread_condattr_setname_np (&cda, "select");
    pthread_cond_init (&select_cd, &cda);
    pthread_condattr_destroy (&cda);

    pthread_mutex_setname_np (&select_mx, "select_mx");
}


int	select (int, fd_set *, fd_set *, fd_set *, struct timeval *);

/*
 * Select system call.
 */
int
select(nd, in, ou, ex, tv)
    int nd;
    fd_set *in, *ou, *ex;
    struct timeval *tv;
{
    fd_set ibits[3], obits[3];
    struct timeval atv;
    struct timespec ts, *tsp;
    int error = 0;
    int retval = 0;
    int nwakeups;
    u_int ni;

    _syscall_enter ();
    
    pthread_once (&select_once, selinit);

    /* Don't allow more than FD_SETSIZE bits */
    /* XXX real *BSD allows this to be increased */
    if (nd > FD_SETSIZE)
	/* forgiving; slightly wrong */
	nd = FD_SETSIZE;

    ni = howmany(nd, NFDBITS) * sizeof(fd_mask);
    memset(ibits, 0, sizeof(ibits));
    memset(obits, 0, sizeof(obits));

#define	getbits(name, x) \
    if (name) \
        memcpy(&ibits[x], name, ni)
    getbits(in, 0);
    getbits(ou, 1);
    getbits(ex, 2);
#undef	getbits

    if (tv) {
	memcpy(&atv, tv, sizeof (atv));
	if (error)
	    goto done;
	if (tv->tv_sec < 0 || tv->tv_sec > 100000000 ||
	    tv->tv_usec < 0 || tv->tv_usec >= 1000000) {
	    error = EINVAL;
	    goto done;
	}
	clock_gettime (CLOCK_REALTIME, tsp = &ts);
	ts.tv_sec += atv.tv_sec;
	ts.tv_nsec += atv.tv_usec * 1000;
	if (ts.tv_nsec >= 1000000000L) {
	    ts.tv_nsec -= 1000000000L;
	    ts.tv_sec++;
	}
    } else
	tsp = NULL;

    do {
	nwakeups = nselwakeups;
	error = selscan(&ibits[0], &obits[0], nd, &retval);
	pthread_mutex_lock (&select_mx);
	/* If nothing was ready, and no-one called selwakeup() while
	   we were scanning, then block until selwakeup() is called. */
	if (error == 0 && retval == 0 && nwakeups == nselwakeups) 
	    error = pthread_cond_timedwait (&select_cd, &select_mx, tsp);
	pthread_mutex_unlock (&select_mx);
    } while (error == 0 && retval == 0);

    if (error == EWOULDBLOCK || error == ETIME)
	error = 0;

done:
#define	putbits(name, x) \
    if (name) \
      memcpy(name,  &obits[x], ni)
    if (error == 0) {
	putbits(in, 0);
	putbits(ou, 1);
	putbits(ex, 2);
#undef putbits
    }
	
    return _syscall_leave (error, retval);
}


static int
selscan(ibits, obits, nfd, retval)
    fd_set *ibits, *obits;
    int nfd;
    int *retval;
{
    register int msk, i, j, fd;
    register fd_mask bits;
    int n = 0;
    static const int flag[3] = { FREAD, FWRITE, 0 };
    
    for (msk = 0; msk < 3; msk++) {
	fd_set *pibits = &ibits[msk];
	fd_set *pobits = &obits[msk];
	
	for (i = 0; i < nfd; i += NFDBITS) {
	    bits = pibits->fds_bits[i/NFDBITS];
	    while ((j = ffs(bits)) && (fd = i + --j) < nfd) {
		struct file *fp;
		int error = _fileio_fget (fd, &fp);
		if (error)
		    return (error);
		if (! fp->f_ops->fo_select ||
		    (*fp->f_ops->fo_select)(fp, flag[msk])) {
		    FD_SET(fd, pobits);
		    n++;
		}
		_fileio_frelease (fp);
		bits &= ~(1 << j);
	    }
	}
    }
    *retval = n;
    return (0);
}


/*
 * Record a select request.
 */
void
_selrecord(sip)
    struct selinfo *sip;
{
    pthread_mutex_lock (&select_mx);
    if (sip->si_flags & SI_SEL)
	/* more than one thread waiting on selector */
	sip->si_flags |= SI_COLL;
    else
	/* XXX record the current thread */
	sip->si_flags |= SI_SEL;
    pthread_mutex_unlock (&select_mx);
}

/*
 * Do a wakeup when a selectable event occurs.
 */
void
_selwakeup(sip)
    register struct selinfo *sip;
{
    pthread_mutex_lock (&select_mx);
    if (sip->si_flags & SI_SEL) {
	/* XXX Note that the flags for other descriptors being waited
	   for don't get reset, so they will do a cond_broadcast when
	   an event happens to them, even if no-one is actually
	   waiting for them any more. This isn't too expensive
	   though. */
	++nselwakeups;
	sip->si_flags = 0;
	/* XXX At the moment this will wake up all threads blocked in
	   select - we should try to wakeup only the thread which is
	   waiting for this descriptor, if there is only one. */
	pthread_cond_broadcast (&select_cd);
    }
    pthread_mutex_unlock (&select_mx);
}
