/*
 * Copyright (c) 1996-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * rtpx_stats.h: real-time statistics and logging
 */

#include <pthread/rtpxint.h>

#ifndef _RTPX_EVLOGSIZE
#define _RTPX_EVLOGSIZE	256
#endif

/* each entry is 32 bytes */
struct rtpx_evrec {
    clock64_t       ev_time;
    rtpx_thread_t  *ev_th;
    unsigned short  ev_what;
#define ELOG_SEM	0x2000	/* | wait,arg=sem */
#define ELOG_CANCEL	0x3000	/* | flg,arg=oth */
#define ELOG_SIGDELIVER	0x4000	/* | signo */
#define ELOG_SIGQUEUE	0x5000	/* | signo, arg=oth */
#define ELOG_TIMER	0x6000	/* | type */
#define ELOG_MUTEX	0x7000	/* | lock, arg = waitq */
#define ELOG_STATE	0x8000	/* | newstate, arg = waitq */
#define ELOG_PRIO	0x9000	/* | newpri */
#define ELOG_EVENT	0xA000	/* | evn */
#define ELOG_INTERUPT	0xB000	/* | crbits, arg = sr */
#define ELOG_SWTCH	0xC000	/* | newpri, arg = newth */
#define ELOG_CREATE	0xD000	/* | newpri, arg = newth */
#define ELOG_SCHED	0xE000	/* | 0, arg = rrval */
#define ELOG_USER	0xF000	/* | nargs, arg0=fmt, ... */
    unsigned char   ev_pri;
    unsigned char   ev_state;
#define ELOG_NARGS 4
    void           *ev_arglist[ELOG_NARGS];
#define ev_arg	ev_arglist[0]
};

extern unsigned int _rtpx_evlog_mask;

void            _rtpx_elog_add (rtpx_thread_t *th, unsigned short what,
				void *arg);

#ifndef _RTPX_NO_STATS
#define EVLOG(th,type,what,arg)	\
	((_rtpx_evlog_mask & (1 << ((type) >> 12))) \
	 ? _rtpx_elog_add (th, (type)|(what), arg) \
	 : (void)0)
#else
#define EVLOG(th,type,what,arg)	(void)0
#endif

#define EVLOGMUTEX(th,lock,mx) 		EVLOG(th, ELOG_MUTEX, lock, mx)
#define EVLOGSTATE(th,nstate,arg) 	EVLOG(th, ELOG_STATE, nstate, arg)
#define EVLOGPRIO(th,npri) 		EVLOG(th, ELOG_PRIO, npri, NULL)
#define EVLOGEVN(th,evn,db)		EVLOG(th, ELOG_EVENT, evn, db)
#define EVLOGSWTCH(th,nth,npri)		EVLOG(th, ELOG_SWTCH, npri, nth)
#define EVLOGCREATE(th,nth,npri)	EVLOG(th, ELOG_CREATE, npri, nth)
#define EVLOGSCHED(th,oth)		EVLOG(th, ELOG_SCHED, 0, oth)
#define EVLOGTIMER(th,type)		EVLOG(th, ELOG_TIMER, type, NULL)
#define EVLOGSIGQUEUE(th,signo,oth)	EVLOG(th, ELOG_SIGQUEUE, signo, oth)
#define EVLOGSIGDELIVER(th,signo)	EVLOG(th, ELOG_SIGDELIVER, signo, NULL)
#define EVLOGSIGRETURNED(th,signo)	EVLOG(th, ELOG_SIGDELIVER, signo, (void*)1)
#define EVLOGCANCEL(th,oth,flg)		EVLOG(th, ELOG_CANCEL, flg, oth)
#define EVLOGSEM(th,sem,post)		EVLOG(th, ELOG_SEM, wait, sem)

/* counters (export to user somehow) */
struct rtpx_stats {
    unsigned long   ncreate;	/* # threads created */
    unsigned long   ncswitch;	/* # context switches */
    unsigned long   nintr;	/* # interrupts taken */
    unsigned long   nmutexwait;	/* # mutex locks which wait */
    unsigned long   nmutexdead;	/* # mutex locks which deadlock */
    unsigned long   nmutexwake;	/* # mutex unlock which wake */
    /* XXX what else would be useful? */
};

extern struct rtpx_stats _rtpx_stats;

#ifndef _RTPX_NO_STATS
#define RTPX_INCSTAT(x)	(_rtpx_stats.x++)
#else
#define RTPX_INCSTAT(x)	(void)0
#endif

void            _rtpx_stats_init (void);
