#ident "sde/rtpx/rtpx_sysconf.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2000-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * rtpx_sysconf.c: implement Pthreads specific sysconf() function
 */


#include "pthread_impl.h"

long
pthread_sysconf_np (int name)
{
    switch (name) {

	/* Pthreads */
    case _SC_THREADS:
    case _SC_THREAD_SAFE_FUNCTIONS:
    case _SC_THREAD_ATTR_STACKADDR:
    case _SC_THREAD_ATTR_STACKSIZE:
    case _SC_THREAD_PROCESS_SHARED:
    case _SC_THREAD_PRIORITY_SCHEDULING:
    case _SC_THREAD_PRIO_INHERIT:
    case _SC_THREAD_PRIO_PROTECT:
	return 1;
    case _SC_THREAD_THREADS_MAX:
	return _RTPX_THREAD_MAX;
    case _SC_THREAD_DESTRUCTOR_ITERATIONS:
	return PTHREAD_DESTRUCTOR_ITERATIONS;
    case _SC_THREAD_STACK_MIN:
	return PTHREAD_STACK_MIN;
    case _SC_THREAD_KEYS_MAX:
	return PTHREAD_KEYS_MAX;

	/* POSIX.2 */
    case _SC_CLK_TCK:
	return (int) _rtpx_tickspersec;
    case _SC_NPROCESSORS_ONLN:
    case _SC_NPROCESSORS_CONF:
	return 1;

	/* POSIX.4 */
    case _SC_AIO_LISTIO_MAX:
    case _SC_AIO_MAX:
    case _SC_AIO_PRIO_DELTA_MAX:
    case _SC_ASYNCHRONOUS_IO:
	return -1;
    case _SC_DELAYTIMER_MAX:
	return DELAYTIMER_MAX;
    case _SC_MAPPED_FILES:
    case _SC_MEMLOCK:
    case _SC_MEMLOCK_RANGE:
    case _SC_MEMLOCK_PROTECTION:
    case _SC_MEMORY_PROTECTION:
	return -1;
    case _SC_MESSAGE_PASSING:
	return 1;
    case _SC_MQ_OPEN_MAX:
	return MQ_OPEN_MAX;
    case _SC_MQ_PRIO_MAX:
	return MQ_PRIO_MAX;
    case _SC_PRIORITIZED_IO:
	return -1;
    case _SC_PRIORITY_SCHEDULING:
	return 1;
    case _SC_REALTIME_SIGNALS:
	return 1;
    case _SC_RTSIG_MAX:
	return RTSIG_MAX;
    case _SC_SEMAPHORES:
	return 1;
    case _SC_SEM_NSEMS_MAX:
	return SEM_NSEMS_MAX;
    case _SC_SEM_VALUE_MAX:
	return SEM_VALUE_MAX;
    case _SC_SHARED_MEMORY_OBJECTS:
	return -1;
    case _SC_SYNCHRONIZED_IO:
	return -1;
    case _SC_SIGQUEUE_MAX:
	return SIGQUEUE_MAX;
    case _SC_TIMER_MAX:
	return TIMER_MAX;
    case _SC_TIMERS:
	return 1;
    }
    return -1;
}
