#ident "sde/rtpx/rtpx_time.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2000-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * rtpx_time.c: older POSIX interfaces to real-time clock
 */


#include <stddef.h>
#include <time.h>
#include <pthread.h>

static pthread_once_t clockonce = PTHREAD_ONCE_INIT;
static struct timespec tsfirst;

static void
clockfirst ()
{
    clock_gettime (CLOCK_MONOTONIC, &tsfirst);
}


/* return elapsed time in usecs */
clock_t
clock ()
{
    struct timespec ts;

    pthread_once (&clockonce, clockfirst);
    clock_gettime (CLOCK_MONOTONIC, &ts);
    ts.tv_sec -= tsfirst.tv_sec;
    ts.tv_nsec -= tsfirst.tv_nsec;
    return (ts.tv_sec * 1000000) + ((ts.tv_nsec + 500) / 1000);
}


int
gettimeofday (struct timeval *tvp, struct timezone *tzp)
{
    struct timespec ts;
    clock_gettime (CLOCK_REALTIME, &ts);
    tvp->tv_sec = ts.tv_sec;
    tvp->tv_usec = (ts.tv_nsec + 500) / 1000;
    if (tzp)
	tzp->tz_minuteswest = tzp->tz_dsttime = 0;
    return 0;
}

int
settimeofday (const struct timeval *tvp, const struct timezone *tzp)
{
    struct timespec ts;
    ts.tv_sec = tvp->tv_sec;
    ts.tv_nsec = tvp->tv_usec * 1000;
    return clock_settime (CLOCK_REALTIME, &ts);
}

/* return wall clock time in secs */
time_t
time (time_t * tp)
{
    struct timespec ts;
    clock_gettime (CLOCK_REALTIME, &ts);
    if (tp)
	*tp = ts.tv_sec;
    return ts.tv_sec;
}
