#ident "sde/rtpx/rtpx_z8530.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 2001-2004 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * rtpx_z8530.c: interrupt-driven serial i/o for AMD 85c30 / Z8530 DUART
 *
 * XXX Not finished yet XXX
 */


#define _KERNEL
#include <stddef.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <pthread.h>
#include <errno.h>
#include <assert.h>
#include <sys/fcntl.h>
#include <sys/termios.h>
#include <sys/signal.h>
#include <sys/time.h>

#include <mips/cpu.h>
#include <mips/xcpt.h>

#include "tty.h"
#include "sbd.h"
#include "sbdrtpx.h"
#include "sbdser.h"
#include "z8530.h"

#define Z8530_DEBUG 2
#ifndef Z8530_DEBUG
#define Z8530_DEBUG 0
#endif

/* private data */
static struct serial {
    volatile z8530dev *	dev;
    struct serial     * realcha;
    struct serial     * realchb;
    unsigned char	unit;
    unsigned char 	evn;		/* doorbell event number */
    unsigned char	fifolen;	/* length of tx/rx fifo */
    unsigned char	mcr;		/* soft copy of mcr register */
    /* the tty structure */
    struct tty		tty;
    /* thread control */
    pthread_cond_t	cd;		/* wait point */
    char		cdname[16];
    pthread_doorbell_t	db;		/* interrupt doorbell */
    char		dbname[16];
    pthread_t		tid;		/* interrupt task id */
    char		thname[16];
    char		mxname[16];
    /* statistics */
    unsigned int	perrs;		/* parity errors */
    unsigned int	ferrs;		/* framing errors */
    unsigned int	oerrs;		/* overflow errors */
} serial[] = {
    {(z8530dev *) PA_TO_KVA1(DUART_BASE + Z8530_CHA), 0, INTR_DUART},
    {(z8530dev *) PA_TO_KVA1(DUART_BASE + Z8530_CHB), 1, -1},
#ifdef DUART2_BASE
    {(z8530dev *) PA_TO_KVA1(DUART2_BASE + Z8530_CHA), 2, INTR_DUART2},
    {(z8530dev *) PA_TO_KVA1(DUART2_BASE + Z8530_CHB), 3, -1},
#endif
};

#define dimof(arr) (sizeof(arr) / sizeof(arr[0]))


/*
 * Public entrpoint table 
 */
static int	z8init (int ch, int dev);
static int	z8open (int ch, struct tty **);

struct sbdserint z8530int = {
    "ttyz", dimof(serial),
    z8init,
    z8open,
};

/* global debug flags */
int 		_rtpx_break_debug;




#ifndef Z8530DELAY
/* 2us default turnaround time between register accesses */
#define Z8530DELAY	2
#endif

#if Z8530DELAY > 0
#define zdelay()     sbd_usdelay (Z8530DELAY)
#else
#define zdelay()
#endif

static __inline__ unsigned char
zgetcmd (volatile z8530dev *dp)
{
    unsigned char v = dp->cmd;
    zdelay ();
    return v;
}

static __inline__ void
zputcmd (volatile z8530dev *dp, unsigned char v)
{
    dp->cmd = v;
    mips_wbflush(); 
    zdelay ();
}

static __inline__ unsigned char
zgetdata (volatile z8530dev *dp)
{
    unsigned char v = dp->data;
    zdelay ();
    return v;
}

static __inline__ void
zputdata (volatile z8530dev *dp, unsigned char v)
{
    dp->data = v;
    mips_wbflush(); 
    zdelay ();
}

static void
zcmd (volatile z8530dev *dp, unsigned reg, unsigned val)
{
    zputcmd (dp, reg);
    zputcmd (dp, val);
}

static unsigned char
zstat (volatile z8530dev *dp, unsigned reg)
{
    zputcmd (dp, reg);
    return zgetcmd (dp);
}



static void
z8cleanup (void *arg)
{
    struct tty *tp = arg;
    pthread_mutex_unlock (&tp->t_mx);
}


/* wait for a tx or rx event to occur */
/* XXX we should use different condition variables for each event */

static int
z8wait (struct tty *tp, int waitflags, int tenths)
{
    struct serial *ser = tp->t_addr;
    struct timespec ts;
    int res;

    pthread_cleanup_push (z8cleanup, tp);
    tp->t_state |= waitflags;
    if (tenths >= 0) {
	clock_gettime (CLOCK_REALTIME, &ts);
	ts.tv_sec += tenths / 10;
	ts.tv_nsec += (tenths % 10) * 100*1000*1000;
	if (ts.tv_nsec >= 1000000000)
	    ts.tv_sec++, ts.tv_nsec -= 1000000000;
	res = pthread_cond_timedwait (&ser->cd, &tp->t_mx, &ts);
    }
    else {
	res = pthread_cond_wait (&ser->cd, &tp->t_mx);
    }
    pthread_cleanup_pop (0);

    if (res == ETIME)
	_RTPX_ELOG ("z8uart%d: %d.%ds wait timeout", ser->unit, 
		    tenths / 10, tenths % 10); 
    else if (res != 0)
	_RTPX_ELOG ("z8uart%d: wait error: %s", ser->unit, strerror(res)); 

    return res;
}


static void
z8wakeup (struct tty *tp, int flags)
{
    int wakeup = 0;
    if (flags & FWRITE) {
	if (tp->t_state & TS_TXWAIT && tp->t_outq.c_cc < tp->t_hiwat) {
	    tp->t_state &= ~TS_TXWAIT;
	    wakeup = 1;
	}
	if (tp->t_state & TS_TXDRAIN && tp->t_outq.c_cc == 0) {
	    tp->t_state &= ~TS_TXDRAIN;
	    wakeup = 1;
	}
    }
    if (flags & FREAD) {
	if (tp->t_state & TS_RXWAIT) {
	    tp->t_state &= ~TS_RXWAIT;
	    wakeup = 1;
	}
    }
    if (wakeup) {
	struct serial *ser = tp->t_addr;
	pthread_cond_broadcast (&ser->cd);
    }
}

/* 
 * Start transmitting to uart
 */

static void
z8start (struct tty *tp)
{
    struct serial *ser = tp->t_addr;
    volatile z8530dev *dp = ser->dev;
    int c;

    if (tp->t_state & TS_BUSY)
	/* already transmitting */
	return;

    if (tp->t_state & TS_XON_PEND) {
	tp->t_state &= ~TS_XON_PEND;
	c = tp->t_cc[VSTART];
#if Z8530_DEBUG
	_RTPX_ELOG ("xon>z8uart%d", ser->unit); 
#endif
    }
    else if (tp->t_state & TS_XOFF_PEND) {
	tp->t_state &= ~TS_XOFF_PEND;
	c = tp->t_cc[VSTART];
#if Z8530_DEBUG
	_RTPX_ELOG ("xoff>z8uart%d", ser->unit); 
#endif
    }
    else if (!(tp->t_state & TS_TTSTOP)) {
	c = _ttygetc (&tp->t_outq);
	if (c < 0)
	    return;
#if Z8530_DEBUG > 1
	_RTPX_ELOG ("%02x(%c)>z8uart%d", 
		    c, (c < 0x20 || c >= 0x7f) ? '?' : c, 
		    ser->unit);
#endif
	z8wakeup (tp, FWRITE);
    }

    zputdat (dp, c);
    tp->t_state |= TS_BUSY;
}



static void
z8rxintr (struct serial *ser)
{
    volatile z8530dev *dp = ser->dev;
    struct tty *tp = &ser->tty;
    int stat, err, c;

    while ((stat = zstat (dp, RR_STATUS)) & US_RXRDY) {

	err = zstat (dp, RR_ERROR);
	c = z8getdat (dp);
	if (err & (US_FRAMING | US_PARITY | US_OVERRUN))
	    zcmd (dp, WR_COMMAND, UC_RESIERR);

	if (stat & US_BREAK) {
	    _RTPX_ELOG ("[brk]");
	    if (_rtpx_break_debug) {
		/* immediate debug interrupt */
		extern void _debug_breakpoint (void);
		_debug_breakpoint ();
		continue;
	    }
	    c = TTY_FE | 0;
	}
	else if (err & US_FRAMING) {
	    _RTPX_ELOG ("[fram]");
	    ser->ferrs++;
	    c |= TTY_FE;
	}
	else if (err & US_PARITY) {
	    _RTPX_ELOG ("[ptye]");
	    ser->perrs++;
	    c |= TTY_PE;
	}
	if (err & US_OVERRUN) {
	    _RTPX_ELOG ("[oflo]");
	    ser->oerrs++;
	}
#if Z8530_DEBUG
	{
	    char cm = c & 0x7f;
	    _RTPX_ELOG ("%02x(%c)<z8uart%d", 
			cm, (cm < 0x20 || cm >= 0x7f) ? '?' : cm, 
			ser->unit); 
	}
#endif
	if (tp->t_state & TS_ISOPEN)
	    _ttyinput (c, tp);
    }    
}


static void
z8txintr (struct serial *ser)
{
    volatile z8530dev *dp = ser->dev;
    struct tty *tp = &ser->tty;

#if Z8530_DEBUG > 1
    _RTPX_ELOG ("[txrdy]");
#endif
    tp->t_state &= ~TS_BUSY;
    zcmd (dp, WR_COMMAND, UC_RESITX);
    if (tp->t_state & TS_ISOPEN)
	z8start (tp);
}


static void
z8statintr (struct serial *ser)
{
    volatile z8530dev *dp = ser->dev;
    struct tty *tp = &ser->tty;
    
    msr = dp->msr;
    _RTPX_ELOG ("[stat %x]");
    zcmd (dp, WR_COMMAND, UC_RESIST);
#if 0
    /* pass carrier-detect change to tty.c */
    if (msr & MSR_DDCD)
	_ttymodem (tp, msr & MSR_DCD);
#endif
    break;
}

		
/* 
 * Main "interrupt" task
 */

static void *
z8task (void *arg)
{
    struct serial *ser = arg;
    volatile z8530dev *dp = ser->dev;
    struct tty *tp = &ser->tty;
    unsigned char iir, lsr, msr;
    int c;
    sigset_t sm;

    /* disable all signals in this task */
    sigfillset (&sm);
    pthread_sigmask (SIG_SETMASK, &sm, NULL);

    /* get the lock */
    pthread_mutex_lock (&tp->t_mx);

    /* enable uart interrupts */
    zcmd (dp, WR_STATCNTRL, UM_BREAKIE /* | UM_CTSIE | UM_DCDIE*/);
    zcmd (dp, WR_TXRXINT, UM_RXINT | UM_EXTINT | UM_TXINT);

    while (1) {

	/* wait for an interrupt */
	pthread_doorbell_wait (&ser->db, &tp->t_mx);

	while ((ipend = zstat (dp, RR_IPEND)) != 0) {
	    if (ipend & US_RXAIP)
		z8rxintr (ser);
	    if (ipend & US_RXBIP)
		z8rxintr (ser);
	    if (ipend & US_EXTSTAIP)
		z8statintr (ser);
	    if (ipend & US_EXTSTBIP)
		z8statintr (ser);
	    if (ipend & US_TXAIP)
		z8txintr (ser);
	    if (ipend & US_TXBIP)
		z8txintr (ser);
	}
		break;
		
	    default:
		_RTPX_ELOG ("[baduint]");
		/* unexpected interrupt, ignore */
		break;
	    }
	}
    }
}


static void
z8taskstart (struct serial *ser)
{
    /* create interrupt thread on first open */
    pthread_attr_t tha;
    struct sched_param sp;
    
    pthread_attr_init(&tha);
    
    /* set FIFO scheduling */
    pthread_attr_setdetachstate (&tha, PTHREAD_CREATE_DAEMON_NP);
    pthread_attr_setinheritsched (&tha, PTHREAD_EXPLICIT_SCHED);
    pthread_attr_setschedpolicy (&tha, SCHED_FIFO);
    
    pthread_attr_getschedparam (&tha,&sp);
#ifdef Z8530_PRIO
    sp.sched_priority = Z8530_PRIO;
#else
    /* set maximum priority to catch break debug interrupts */
    sp.sched_priority = sched_get_priority_max (SCHED_FIFO);
#endif
    pthread_attr_setschedparam (&tha, &sp);
    
    pthread_attr_setname_np (&tha, ser->thname);
    
    if (pthread_create(&ser->tid, &tha, z8task, ser) != 0)
	abort ();
    
    pthread_attr_destroy (&tha);
}



static void
z8rxflow (struct tty *tp, int enb)
{
    struct serial *ser = tp->t_addr;
    volatile z8530dev *dp = ser->dev;

    if (enb)
	ser->mcr |= MCR_RTS;
    else
	ser->mcr &= ~MCR_RTS;
    dp->mcr = ser->mcr;
}


static void
z8txstop (struct tty *tp, int flush)
{
    /* nothing can be done with already output characters */
}


static void
z8sendbrk (struct tty *tp, int duration)
{
    struct serial *ser = tp->t_addr;
    volatile z8530dev *dp = ser->dev;
    struct timespec ts;

    if (duration <= 0)
	duration = 1;
    else if (duration > 40)
	duration = 40;
    duration *= 250;
    ts.tv_sec = duration / 1000;
    ts.tv_nsec = (duration % 1000) * 1000000;

    dp->cfcr |= CFCR_SBREAK;
    while (nanosleep (&ts, &ts) == -1 && errno == EINTR)
	continue;
    dp->cfcr &= ~CFCR_SBREAK;
}



/*
 * z8config
 *
 * Abstract:
 *  Set up the serial port according to specified parameters.  
 *
 * Arguments:
 *   ch:        channel number
 *   tio:	pointer to termios structure
 *
 * Returns:
 *   true on success
 */

static int
z8params (struct tty *tp, int set, struct termios *tio)
{
    struct serial *ser = tp->t_addr;
    volatile z8530dev *dp = ser->dev;
    long brtc;
    unsigned char cfcr;

#define divrnd(n, q)	(((n) * 2 / (q) + 1) / 2) /* divide and round off */
    if (set) {
	/* set h/w settings from termios */
	long brtc, err;

	if (tio->c_ospeed != tio->c_ispeed)
	    return EINVAL;

	if (tio->c_ospeed == 0)
	    return 0;

	brtc = divrnd(Z8530_TICK, tio->c_ospeed);
	if (brtc <= 0)
	    return EINVAL;

#if Z8530_TICK < INT_MAX / 500
	err = divrnd(1000 * Z8530_TICK, brtc * tio->c_ospeed) - 1000;
#else
	err = 1000 * divrnd(Z8530_TICK, brtc * tio->c_ospeed) - 1000;
#endif	
	if (err < 0)
	    err = -err;
	if (err > SPEED_TOLERANCE)
	    return EINVAL;
	
	cfcr = 0;
	if (tio->c_cflag & PARENB) {
	    if (tio->c_cflag & PARODD)
		cfcr |= CFCR_PENAB | CFCR_PODD;
	    else
		cfcr |= CFCR_PENAB | CFCR_PEVEN;
	}

	switch (tio->c_cflag & CSIZE) {
	case CS5: 
	    cfcr |= CFCR_5BITS;
	    break;
	case CS6: 
	    cfcr |= CFCR_6BITS;
	    break;
	case CS7: 
	    cfcr |= CFCR_7BITS;
	    break;
	case CS8:
	    cfcr |= CFCR_8BITS;
	    break;
	}
	
	if (tio->c_cflag & CSTOPB)
	    cfcr |= CFCR_STOPB;
	
	/* wait for fully empty fifo */
	while (!(dp->lsr & LSR_TSRE))
	    continue;

	dp->cfcr = CFCR_DLAB | cfcr;
	dp->data = brtc & 0xff;
	dp->ier = brtc >> 8;
	dp->cfcr = cfcr;

	/* restart transmitter */
	z8start (tp);
    }
    else {
	/* convert from h/w settings to termios */

	cfcr = dp->cfcr;
	dp->cfcr = CFCR_DLAB | cfcr;
	brtc = ((dp->ier & 0xff) << 8) | (dp->data & 0xff);
	dp->cfcr = cfcr;

	if (brtc == 0)
	    tio->c_ispeed = 9600;	/* default */
	else
	    tio->c_ispeed = divrnd (Z8530_TICK, brtc);
	tio->c_ospeed = tio->c_ispeed;

	tio->c_cflag = CREAD | CLOCAL | HUPCL;

	if (cfcr & CFCR_PENAB) {
	    tio->c_cflag |= PARENB;
	    if (!(cfcr & CFCR_PEVEN))
		tio->c_cflag |= PARODD;
	}

	switch (cfcr & CFCR_CSIZE) {
	case CFCR_5BITS: 
	    tio->c_cflag |= CS5;
	    break;
	case CFCR_6BITS: 
	    tio->c_cflag |= CS6;
	    break;
	case CFCR_7BITS: 
	    tio->c_cflag |= CS7;
	    break;
	case CFCR_8BITS: 
	    tio->c_cflag |= CS8;
	    break;
	}

	if (cfcr & CFCR_STOPB)
	    tio->c_cflag |= CSTOPB;

    }
#undef divrnd

    return 0;
}



/*
 * z8close
 *
 * Abstract:
 *   Epilogue for the serial port BIOS code
 *
 * Arguments:
 *   ch:  channel number to close
 *
 * Returns:
 *   nothing
 */

static void 
z8close (struct tty *tp)
{
    struct serial *ser = tp->t_addr;
    volatile z8530dev *dp = ser->dev;
#if 0
    /* leave thread running to catch debug interrupts */
    if (ser->tid) {
	pthread_cancel (ser->tid);
	ser->tid = 0;
	dp->ier = 0;
    }
#endif
    /* disable DTR */
    dp->mcr = (ser->mcr &= ~MCR_DTR);
}


/*
 * z8open
 *
 * Abstract:
 *   Initialise a serial port and its low-level buffers.
 *
 * Arguments:
 *   ch:      Channel to open
 *
 * Returns:
 *   0 if hunky-dory, error code if the channel is inaccessible
 */

static int
z8open (int ch, struct tty **tpp)
{
    struct serial *ser;
    volatile z8530dev *dp;
    int ret;

    if (ch < 0 || ch >= dimof(serial))   
	return ENXIO;
    ser = &serial[ch];

    /* have we initialised OK */
    if (ser->fifolen <= 0)
	return ENODEV;

    /* start interrupt task if not running */
    if (!ser->tid)
	z8taskstart (ser);

    /* return tty pointer and allocate buffers */
    *tpp = &ser->tty;
    if (ret = _ttyclalloc (&ser->tty))
	return ret;

    /* enable DTR */
    dp = ser->dev;
    dp->mcr = (ser->mcr |= MCR_DTR);
    return 0;
}



/* guaranteed called at init time and safe from multiple threads */
static int
z8init (int ch, int dev)
{
    struct serial *ser;
    struct tty *tp;
    pthread_mutexattr_t mxa;
    pthread_condattr_t cda;
    pthread_doorbellattr_t dba;
    struct timespec ts;
    volatile z8530dev *dp;

    if (ch < 0 || ch >= dimof(serial))   
	return ENODEV;

    ser = &serial[ch];
    dp = ser->dev;
    tp = &ser->tty;
    tp->t_addr = ser;	/* back link */

    /* force access to id reg */
    dp->cfcr &= ~CFCR_DLAB;
    dp->iir = 0;
    if ((dp->iir & 0x38) != 0) {
	/* not a 16550 */
	ser->fifolen = 0;
	return ENODEV;
    }

    /* in case it really is a 16550, enable the fifos */
    dp->iir = FIFO_ENABLE | FIFO_RCV_RST | FIFO_XMT_RST | FIFO_TRIGGER_4;

    /* wait 100us */
    ts.tv_sec = 0; ts.tv_nsec = 100*1000;
    nanosleep (&ts, NULL);

    /* determine fifo size */
    ser->fifolen = ((dp->iir & IIR_FIFO_MASK) == IIR_FIFO_MASK) ? 16 : 1;

    /* clear pending status reports. */
    (void) dp->lsr;
    (void) dp->msr;

    /* 
     * Enable external interrupt gate (to avoid floating IRQ), enable RTS,
     * but don't enable DTR yet. 
     */
    dp->mcr = ser->mcr = MCR_IENABLE | MCR_RTS;

    /* initialise doorbell */
    pthread_doorbellattr_init (&dba);
    pthread_doorbellattr_setevn (&dba, ser->evn);
    strcpy (ser->dbname, "ttyxxdb");
    ser->dbname[3] = (ser->unit / 10) + '0';
    ser->dbname[4] = (ser->unit % 10) + '0';
    pthread_doorbellattr_setname (&dba, ser->dbname);
    pthread_doorbell_init (&ser->db, &dba);
    pthread_doorbellattr_destroy (&dba);

    /* initialise condition */
    pthread_condattr_init (&cda);
    strcpy (ser->cdname, ser->dbname);
    ser->cdname[5] = 'c';
    ser->cdname[6] = 'd';
    pthread_condattr_setname_np (&cda, ser->cdname);
    pthread_condattr_setinterruptible_np (&cda, 1);
    pthread_cond_init (&ser->cd, &cda);
    pthread_condattr_destroy (&cda);

    /* initialise shared mutex */
    pthread_mutexattr_init (&mxa);
    strcpy (ser->mxname, ser->dbname);
    ser->mxname[5] = 'm';
    ser->mxname[6] = 'x';
    pthread_mutexattr_setname_np (&mxa, ser->mxname);
#ifdef _POSIX_THREAD_PRIO_INHERIT
    /* use priority inheritance so serial port interrupts are
       not locked out by a low-priority thread in tty.c */
    pthread_mutexattr_setprotocol (&mxa, PTHREAD_PRIO_INHERIT);
#endif
    /* XXX do we need to share one mutex between both channels! */
    pthread_mutex_init (&tp->t_mx, &mxa);
    pthread_mutexattr_destroy (&mxa);

    /* create links for generic tty driver */
    tp->t_state         |= TS_INTRUPT;
    tp->t_dev		= dev;
    tp->t_params	= z8params;
    tp->t_start		= z8start;
    tp->t_wait		= z8wait;
    tp->t_wakeup	= z8wakeup;
    tp->t_rxflow	= z8rxflow;
    tp->t_txstop	= z8txstop;
    tp->t_sendbrk	= z8sendbrk;
    tp->t_close		= z8close;

    /* set interrupt thread name */
    strcpy (ser->thname, ser->dbname);
    ser->thname[5] = 0;
    ser->thname[6] = 0;

    /* start the interrupt thread now (to catch debug interrupts) */
    z8taskstart (ser);

    return 0;
}
