#define _POSIX_C_SOURCE 199309L
#define ALLTESTS

#include <stdio.h>
#include <stdlib.h>
#include "conc.h"

static void randomCompute(void)
{
#ifdef R3KEMU
  int n = rand() % 10000;
#else
  int n = rand() % 10000 * 200;
#endif
  while(n--)
    rand();
}

static void hellon(Process *p, int x)
{
  randomCompute();  
  printf("hello %d ",x);
}

static void worldn(Process *p, int x)
{
  randomCompute();  
  printf("world %d ",x);
}


static void floatn (Process *p, int x)
{
#ifdef R3KEMU
  int n = rand() % 10000;
#else
  int n = rand() % 10000 * 200;
#endif
  while (n-- != 0) {
      extern double sqrt (double);
      double v = rand();
      if (sqrt(v * v) != v) {
	  printf ("floatn(%d) failed, v=%f\n", x, v);
	  return;
      }
  }
  printf ("floatn(%d) done\n", x);
}


static Semaphore incSem = SEMAPHOREINIT;

static int incVal = 0;

#ifdef R3KEMU
#define INCERLOOP	1000
#else
#define INCERLOOP	100000
#endif

static void incer()
{
  int i;
  
  for(i=0;i < INCERLOOP; i++) {
    SemP(incSem);
    {
      int x = incVal;
      rand();
      incVal = x + 1;
    }
    SemV(incSem);
  }
}

int main()
{
  int i;

  /* force initial priority what is it ? */
  ProcToLow();

#ifdef ALLTESTS
  printf ("10 sequential hellos\n");
  for(i=0; i<10;i++) {
    Process *p = ProcAlloc(hellon,0,1,i);
    
    ProcPar(p,NULL);
    
    printf("\n");

    ProcFree(p);
  }
  
  /* 10 random hellos all in a row
   */
  printf ("10 parallel hellos\n");
  {
    Process *ptab[11];
    
    for(i=0; i<10;i++) 
      ptab[i] = ProcAlloc(hellon,0,1,i);
    ptab[10] = NULL;
    
    ProcParList(ptab);
    
    for(i=0; i<10;i++)
      ProcFree(ptab[i]);
  }
  
  /* 10 ordered hello worlds (random order of hello and world)
   */
  printf ("10 sequential Par(hello,world)\n");
  for(i=0; i<10;i++) {
    Process *p1 = ProcAlloc(hellon,0,1,i);
    Process *p2 = ProcAlloc(worldn,0,1,i);

    /* indeterminate which runs first !! */
    ProcPar(p1,p2,NULL);
    printf("\n");
    
    ProcFree(p1);
    ProcFree(p2);
  }
  
  /* 10 ordered hello worlds (fixed order of hello and world)
   */
  printf ("10 sequential PriPar(hello,world)\n");
  for(i=0; i<10;i++) {
    Process *p1 = ProcAlloc(hellon,0,1,i);
    Process *p2 = ProcAlloc(worldn,0,1,i);

    /* p1 is higher priority */
    ProcPriPar(p1,p2);
    printf("\n");
    
    ProcFree(p1);
    ProcFree(p2);
  }

  /* 10 floating point tests in parallel
   */
  printf ("10 parallel floaters\n");
  {
    Process *ptab[11];
    
    for(i=0; i<10;i++) 
      ptab[i] = ProcAlloc(floatn,0,1,i);
    ptab[10] = NULL;
    
    ProcParList(ptab);
    
    for(i=0; i<10;i++)
      ProcFree(ptab[i]);
  }


  /* 10 incers all with access to incVal
   */
  printf ("10 parallel counts to %d, no semaphore\n", INCERLOOP);
  {
    Process *ptab[11];
    
    for(i=0; i<10;i++) 
      ptab[i] = ProcAlloc(incer,0,0);
    ptab[10] = NULL;
    
    incVal = 0;
    
    /* enable 10 concurrent accesses to incVal */
    for(i=0;i<10;i++)
      SemV(incSem);
    
    ProcParList(ptab);
    
    /* drain semaphore */
    for(i=0;i<10;i++)
      SemP(incSem);
    
    for(i=0; i<10;i++)
      ProcFree(ptab[i]);

    printf("result = %d\n",incVal);
  }

  /* 10 incers 1 conncurrent access
   */
  printf ("10 parallel counts to %d, with semaphore\n", INCERLOOP);
  {
    Process *ptab[11];
    
    for(i=0; i<10;i++) 
      ptab[i] = ProcAlloc(incer,0,0);
    ptab[10] = NULL;
    
    incVal = 0;
    /* enable 1 concurrent accesses to incVal */
    SemV(incSem);
    
    ProcParList(ptab);
    
    /* drain semaphore */
    SemP(incSem);

    for(i=0; i<10;i++)
      ProcFree(ptab[i]);
    
    printf("result = %d\n",incVal);
  }

  /* run 10 processes and stop them */
  ProcToHigh();
  printf("Sleeping for 10 seconds at high priority\n");
  ProcWait(10 * 1000 * 1000);
  printf("done\n");
  
  ProcToLow();
  printf("Sleeping for 10 seconds at low priority\n");
  ProcWait( (10 * 1000 * 1000)/64) ;
  printf("done\n");
#endif

#if JUNK
  {
    Process *ptab[11];
    
    ProcToHigh();
    
    printf ("start 10 parallel low prio hellos\n");
    for(i=0; i<10;i++) {
      ptab[i] = ProcAlloc(hellon,0,1,i);
      ProcRunLow(ptab[i]);
    }
    
    /* wait at high priority => us ticks */
    printf ("sleep 2 seconds at high prio\n");
    ProcWait(2 * 1000 * 1000);
    
    printf ("stop hellos\n");
    for(i=0; i<10;i++) {
      ProcStop(ptab[i]);
      ProcFree(ptab[i]);
    }
  }
#endif
  
  fprintf(stderr,"All Done\n");
  
  pthread_exit((void *)1);
  
  return 0;
}
