#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <assert.h>
#include <time.h>
#include "conc.h"

#include <mips/cpu.h>
#if SB3000
#include <algvme/cpu_board.h>
#else
#include <p4000/cpu_board.h>
#endif


static void pokeDisplay(size_t idx,char c)
{ 
  volatile unsigned long *display;
#ifdef SB3000
  *(volatile unsigned int *)PA_TO_KVA1(BCRR) 
    = BCRR_AUBLK|BCRR_AUCLR; /* |BCRR_SCSI|BCRR_ETH; */
  display = (volatile unsigned long  *)PA_TO_KVA1(ALPHA_DISP(idx));
#else
  *(volatile unsigned long *) PA_TO_KVA1 (ALPHN_BLNK_) = RESET_ONE;
  display = (unsigned long *) PA_TO_KVA1 (ALPHN_CHAR (idx));
#endif
  *display = c << 24;
}

#define TRUE	1
#define FALSE	0

void hello (Process *p, int idx );
void checker (Process *p, int idx );
void sonic(Process *p, Channel *cp);

int main(int argc,char **argv)
{
  Process *p;
  int i;
  int process_count = 1;

  printf("Channel test program\n");
  printf("Find crash when wait time is close to resched time\n");

  /* force initial priority low */
  ProcToLow();
  
  /* run background processes */
  for (i = 1; i <= 2; i++) {
    p = ProcAlloc(hello,0,1,process_count++);
    ProcRunLow(p);
  }

  printf ("Processes started...\n");

#if 0
  /* run a sonic interrupt monitor */
  p = ProcAlloc(sonic,0,1,LINK5IN);
  ProcRunHigh(p);
#endif  

  /* final process is us */
  ProcToHigh();
  checker(NULL,process_count++);

  exit(0);
}


unsigned int prevlcount[4];
unsigned int lcount[4];

void hello (Process *p, int idx )
{
  unsigned long dcount = 0;
  int rcount;

  rcount = 0;
  while (1) {
#if 0
      if (--rcount <= 0) {
	  pokeDisplay (idx, 'Z');
	  ProcWaitMs(rand() % 1000);
	  rcount = (rand() % 1000) * 1000;
      }
#else
      /* 5-10ms delay */
      ProcWaitUs(5000 + (rand() % 5000));
#endif
      pokeDisplay (idx, "0123456789abcdef"[++dcount&0xf]);
      ++lcount[idx];
  }
}


static char *
myctime (const time_t *tp)
{
    static char stime[32];
    strftime (stime, sizeof(stime), "%a %d/%m/%y %H:%M:%S", localtime(tp));
    return stime;
}


void checker (Process *p, int idx)
{
    unsigned int dcount = 0;
    time_t then, now;
    int diff, elapsed;
    int i;

    then = _sbd_gettod ();
    printf ("%s: checker started\n", myctime (&then));

#define TICK	2			/* 2 secs */
#define LOG	(10 * 60 / TICK)	/* 10 mins */

    elapsed = LOG;

    while (1) {

	ProcWaitMs(2000);	/* 2 secs */

	now = _sbd_gettod ();
	diff = now - then;
	if (diff > 4 || diff < 1) {
	    printf ("%s: ERROR: checker delayed for %d secs\n", 
		    myctime (&now), diff);
	}
	then = now;

	for (i = 1; i < idx; i++) {
	    if (lcount[i] == prevlcount[i]) {
		printf ("%s: ERROR: process %d stuck at count %d\n", 
			myctime (&now), i, prevlcount[i]);
	    }
	    prevlcount[i] = lcount[i];
	}

	if (--elapsed == 0) {
	    printf ("%s: checker running\n", myctime (&now));
	    elapsed = LOG;
	}

	pokeDisplay (idx, "0123456789abcdef"[++dcount&0xf]);
    }
}

#include <algvme/sonic.h>
  
#define SONIC_FREQ	5000000		/* clock is 5 MHz (half ethernet) */

/*
 * Load Sonic timer with timeout in uSecs 
 */ 
void sonic_load(unsigned long us)
{
  volatile struct sonic_reg *snr = PA_TO_KVA1 (SONIC_BASE);
  unsigned long cnt = us * (SONIC_FREQ / 1000000);
  
  /* ack any existing timer */
  snr->s_isr = ISR_TC; mips_wbflush();
  
  /* load timeout */
  snr->s_wt1 = cnt>>16; snr->s_wt0 = cnt; mips_wbflush();
  
  /* enable timer */
  snr->s_imr = IMR_TCEN;
  
  /* start timer comand */
  snr->s_cr = CR_ST; mips_wbflush();
}


/*
 * Acknowledge and clear Sonic timer interrupt.
 */ 
void sonic_ack()
{
  volatile struct sonic_reg *snr = PA_TO_KVA1 (SONIC_BASE);

  /* stop timer */
  snr->s_cr = CR_STP; mips_wbflush();

  /* acknowledge the timer interupt (if any) */
  snr->s_isr = ISR_TC; mips_wbflush();

  /* acknowledge all interupts (if any) */
  snr->s_isr = snr->s_isr; mips_wbflush();
  snr->s_imr = 0;		/* and mask */
}


void sonic(Process *p,Channel *cp)
{
    int i;

    while(1) {
	/* wait 1 sec */
	pokeDisplay(0,'W');
	
	/* 1 sec timeout*/ 
	pokeDisplay(0,'L');
	sonic_load(5*1000);	/* 5 msec (ish) */

	/* wait for sonic timer interrupt to fire */
	pokeDisplay(0,'C');

	(void) ChanInInt(cp);

	pokeDisplay(0,'A');
	sonic_ack();
	
        ++lcount[0];
#if 0
	/* freeze the background processes by spinning in a tight loop */
	for (i = 1*1000; i != 0; i--)
	    pokeDisplay(0,"/-\\|"[i&3]);
#endif
    }
}
  

