#define _POSIX_C_SOURCE 199309L

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <assert.h>
#include "conc.h"

#include <mips/cpu.h>
#if SB3000
#include <algvme/cpu_board.h>
#else
#include <p4000/cpu_board.h>
#endif


static void pokeDisplay(size_t idx,char c)
{ 
  volatile unsigned long *display;
#ifdef SB3000
  *(volatile unsigned int *)PA_TO_KVA1(BCRR) 
    = BCRR_AUBLK|BCRR_AUCLR; /* |BCRR_SCSI|BCRR_ETH; */
  display = (volatile unsigned long  *)PA_TO_KVA1(ALPHA_DISP(idx));
#else
  *(volatile unsigned long *) PA_TO_KVA1 (ALPHN_BLNK_) = RESET_ONE;
  display = (unsigned long *) PA_TO_KVA1 (ALPHN_CHAR (idx));
#endif
  *display = c << 24;
}


void hello (Process *p, int idx );
void sonic(Process *p,Channel *cp);

int main(int argc,char **argv)
{
  Process *p;
  int i;

  printf("Channel test program\n");
  printf("Shows up assertion error when used with ProcToHigh()\n");

  /* force initial priority low */
  ProcToLow();
  
  /* run a sonic interrupt monitor */
  p = ProcAlloc(sonic,0,1,LINK5IN);
  ProcRunHigh(p);
  
  /* run background processes */
  for (i = 1; i <= 2; i++) {
    p = ProcAlloc(hello,0,1,i);
    ProcRunLow(p);
  }

  printf ("Processes started...\n");

  /* final background process is us! */
  hello(NULL,3);

  exit(0);
}


void hello (Process *p, int idx )
{
  long lcount = 0;
  long dcount = 0;
  int n;
  
  while (1) {
      /* use ProcToHigh to protect code below if idx == 2 */
      if (idx == 2)
	  ProcToHigh();
      for (n = 1000; n != 0; n--)
	  pokeDisplay(idx,"0123456789abcdef"[++dcount & 0xf]);
      if (idx == 2) {
	  ProcToLow();
	  ProcReschedule ();
      }
  }
}


#include <algvme/sonic.h>
  
#define SONIC_FREQ	5000000		/* clock is 5 MHz (half ethernet) */

/*
 * Load Sonic timer with timeout in uSecs 
 */ 
void sonic_load(unsigned long us)
{
  volatile struct sonic_reg *snr = PA_TO_KVA1 (SONIC_BASE);
  unsigned long cnt = us * (SONIC_FREQ / 1000000);
  
  /* ack any existing timer */
  snr->s_isr = ISR_TC; mips_wbflush();
  
  /* load timeout */
  snr->s_wt1 = cnt>>16; snr->s_wt0 = cnt; mips_wbflush();
  
  /* enable timer */
  snr->s_imr = IMR_TCEN;
  
  /* start timer comand */
  snr->s_cr = CR_ST; mips_wbflush();
}


/*
 * Acknowledge and clear Sonic timer interrupt.
 */ 
void sonic_ack()
{
  volatile struct sonic_reg *snr = PA_TO_KVA1 (SONIC_BASE);

  /* stop timer */
  snr->s_cr = CR_STP; mips_wbflush();

  /* acknowledge the timer interupt (if any) */
  snr->s_isr = ISR_TC; mips_wbflush();

  /* acknowledge all interupts (if any) */
  snr->s_isr = snr->s_isr; mips_wbflush();
  snr->s_imr = 0;		/* and mask */
}


void sonic(Process *p,Channel *cp)
{
    int i;

    while(1) {
#if 0
	/* wait 100 ms */
	pokeDisplay(0,'W');
	ProcWaitMs(100);
#endif
	
	/* 1sec timer*/ 
	pokeDisplay(0,'L');
	sonic_load(1000*1000);

	/* wait for sonic timer interrupt to fire */
	pokeDisplay(0,'C');
	(void) ChanInInt(cp);

	pokeDisplay(0,'A');
	sonic_ack();
	
	/* freeze the background processes by spinning in a tight loop */
	for (i = 100*1000; i != 0; i--)
	    pokeDisplay(0,"/-\\|"[i&3]);
    }
}
