/*
 * Copyright (c) 2008 MIPS Technologies, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * __sdethread stubs for pre-MIPSr2 implementations
 */

#if __mips != 32 || __mips_isa_rev < 2

#include <sys/types.h>
#include <stddef.h>
#include <stdlib.h>
#include <inttypes.h>

#include <errno.h>
#undef errno

#include <libc_thread.h>
#include <sdethread.h>

#pragma weak __isthreaded
int __isthreaded = 0;

#pragma weak _pthread_init
void _pthread_init (void);
void _pthread_init()
{}

int
__sdethread_mutex_init (__sdethread_mutex_t *mx, const char *name)
{
  return 0;
}

int
__sdethread_mutex_destroy (__sdethread_mutex_t *mx)
{
  return 0;
}

int
__sdethread_mutex_lock (__sdethread_mutex_t *mx)
{
  return 0;
}

int
__sdethread_mutex_trylock (__sdethread_mutex_t *mx)
{
  return 0;
}

int
__sdethread_mutex_unlock (__sdethread_mutex_t *mx)
{
  return 0;
}

void
__sdethread_sigdisable (__sdethread_sigstate_t *state)
{/* no signals */}

void
__sdethread_sigrestore (const __sdethread_sigstate_t *state)
{/* no signals */}

int
___sdethread_once (__sdethread_once_t *once, void (*fn)(void))
{
  if (once->so_state == __SDETHREAD_ONCE_NEVER) {
    once->so_state = __SDETHREAD_ONCE_DONE;
    (*fn) ();
  }
  return 0;
}

#undef __sdethread_once
int __sdethread_once (__sdethread_once_t *oncep, void (*funcp)(void))
{
  return ___sdethread_once (oncep, funcp);
}

int *
__sdethread_errno_pointer (void)
{
  return &errno;
}

__attribute__((__nomips16__))
int
__sdethread_set_errno (int err, int rv)
{
  errno = err;
  return rv;
}

/* A function which returns the unique thread ID.  */
__sdethread_t __sdethread_self (void)
{
  return (__sdethread_t)0;
}

void *
_libc_private_storage (struct _thread_private_key_struct *key,
		       void *storage, size_t size, void *err)
{
  /* return static storage only */
  return storage;
}

#include <unistd.h>

pid_t
getpid (void)
{
  return 1;
}


pid_t fork(void)
{
  errno = ENOSYS;
  return -1;
}

int execl (const char *path, const char *arg, ...)
{
  errno = ENOSYS;
  return -1;
}

int execv (const char *path, const char * const *argv)
{
  errno = ENOSYS;
  return -1;
}

#include <sys/wait.h>

pid_t	waitpid (pid_t pid, int *status, int options)
{
  errno = ENOSYS;
  return -1;
}

#endif /* __mips != 32 || __mips_isa_rev < 2 */
